/**
 *   Copyright &copy; 2008-2012 NetAllied Systems GmbH, Ravensburg, Germany. 
 *       
 *   Licensed under the MIT Open Source License, 
 *   for details please see LICENSE file or the website
 *   http://www.opensource.org/licenses/mit-license.php
*/
package de.netallied.xsd2cppsax;

/**
 * A primitive pair of strings.
 * 
 */
public class StringPair {

    public String str1;

    public String str2;

    public StringPair(String str1, String str2) {
        super();
        this.str1 = str1;
        this.str2 = str2;
    }
}
