/*
    Copyright (c) 2008-2009 NetAllied Systems GmbH

    This file is part of GeneratedSaxParser.

    Licensed under the MIT Open Source License, 
    for details please see LICENSE file or the website
    http://www.opensource.org/licenses/mit-license.php
*/

#include "GeneratedSaxParserNamespaceStack.h"

namespace GeneratedSaxParser
{
    //---------------------------------------------------------------------
    const char* XML_ATTRIBUTE_XMLNS = "xmlns";
}
