/*
    Copyright (c) 2009 NetAllied Systems GmbH

    This file is part of Common libftoa.
	
    Licensed under the MIT Open Source License, 
    for details please see LICENSE file or the website
    http://www.opensource.org/licenses/mit-license.php
*/

#ifndef ___ITOAUNITTEST_H__
#define ___ITOAUNITTEST_H__


bool itoaUnitTest();


#endif // ___ITOAUNITTEST_H__
