body {  font-family: Verdana, Arial, Helvetica, sans-serif; font-size: 10pt; color: black; background-color: white}
a:link {  color: #f0ac1c; text-decoration: underline}
a:hover {  color: #F5BC52; text-decoration: underline}
a:visited {  color: #f0ac1c; text-decoration: underline}
li {  color: #f0ac1c; list-style-type: diamond; position: relative; left: -15px; clip:    rect(   )}
td {  font-size: 10pt}
th {  font-size: 10pt}

.MainHeader {  font-weight: bold; color: #F5BC52; background-color: #f0ac1c; font-size: 10pt}
.BorderHeader {  background-color: #f0ac1c; font-size: 8pt; font-weight: bold; color: #f0ac1c; text-align: center} 	  
.MainContent { font-family: Verdana, Arial, Helvetica, sans-serif; font-size: 10pt; color: #f0ac1c } 	 
.BorderContent {  font-size: 8pt; color: #f0ac1c; border-color: black #f0ac1c #f0ac1c; 
                  padding-top: 2px; padding-right: 2px; padding-bottom: 10px; padding-left: 2px;
                  margin-bottom: 2px; border-style: solid; 
                  border-top-width: 0px; border-right-width: 1px; border-bottom-width: 1px; border-left-width: 1px}
.NewsDate {  color: #f0ac1c; font-weight: bold}
.Annotation {  font-size: 10px}

H1, H2, H3 {
        background-color: #f0ac1c;
	color: white;
	padding: 2px;
	padding-left: 5px;
	border-style: solid;
	border-width: 1px;
	border-color: black;
}

H1 {
	text-align: center; 
}
H2 {
}
H3 {
    FONT-FAMILY: trebuchet ms,helvetica,arial;
}
pre {
	background-color: #eeffee;
	color: black;
	padding: 5px;
}
td.example {
	border-style: solid;
	border-width: 1px;
	border-color: black;
}
dt { color: #3D653D; font-weight: bold;}

A.qindex {} 	 
A.qindexRef {} 	 
A.el { text-decoration: none; font-weight: bold } 	 
A.elRef { font-weight: bold } 	 
A.code { text-decoration: none; font-weight: normal; color: Blue } 	 
A:visited.code { text-decoration: none; font-weight: normal; color: Navy } 	 
A:hover.code { text-decoration: underline; font-weight: normal; color: Blue } 	 
A.codeRef { font-weight: normal; color: #4444ee } 	 
DL.el { margin-left: -1cm } 	 
DIV.fragment { width: 100%; border: none; background-color: #003300 } 	 
DIV.ah { color: white; background-color: black; margin-bottom: 3; margin-top: 3 } 	 
DIV.groupHeader { margin-left: 16; margin-top: 12; margin-bottom: 6; font-weight: bold } 	 
DIV.groupText { margin-left: 16; font-style: italic; font-size: smaller } 	 
FONT.keyword       { color: #008000 } 	 
FONT.keywordtype   { color: #999920 } 	 
FONT.keywordflow   { color: #e0FF00 } 	 
FONT.comment       { color: #009900 } 	 
FONT.preprocessor  { color: #809020 } 	 
FONT.stringliteral { color: #002080 } 	 
FONT.charliteral   { color: #008080 }
.ex { 	 
     FONT-SIZE: 8pt; FONT-FAMILY: tahoma
}
CODE.keywordtype { 	 
     COLOR: #00ccff 	 
} 	 
CODE.keyword { 	 
     FONT-WEIGHT: bold; COLOR: #00ccff 	 
} 	 
CODE.fn { 	 
     COLOR: white; 	 
     FONT-WEIGHT: bold; 	 
} 	 
CODE.var { 	 
     COLOR: #dcdcdc; 	 
} 	 
CODE.macro { 	 
     COLOR: #809020; 	 
     FONT-WEIGHT: bold; 	 
 } 	 
CODE.comm { 	 
     COLOR: lime 	 
 }

CODE.num {
	COLOR: purple
	}
CODE.str {
	COLOR: #ffcc33;
	font-style: italic
}
/* Used for rendering members in API specification */

.memproto
{
	background-color: #eeffee;
	color: #ED9C08;
    width: 100%;
	border-top-style: solid;
	border-top-color: black;
	border-top-width: 1px;	
}

.memname
{
   font-size: 12pt;
   font-weight: bold;
   white-space:nowrap;
}

.paramtype
{
	font-size: 12pt;
	font-weight: bold;
	white-space:nowrap;
}
.paramtype A.el {
	color: #F5BC52;
}
paramtype A:link.el {
	color: #007700;
}
.paramtype A:hover.el {
	color: #F5BC52;
}
.paramtype A:visited.el {
	color: #229922;
}
.paramname
{
   font-size: 12pt;
   font-weight: normal;
   font-style: italic;
   white-space:nowrap;
}

/* There's no class on the graph images in the API docs, but they're
all centered.  TODO: get doxygen to give them a class.  (This won't 
look terrible on any image, though, really.) */
center img {
	border-style: solid;
	border-width: 1px;
	border-color: #AACCAA;
}

.memItemLeft
{
	background-color: #eeffee;
}
.memItemRight
{
	background-color: #eeffee;
}


HR { height: 1px; }

// TABS
DIV.tabs
{
   float            : left;
   width            : 100%;   
}

DIV.tabs UL
{
   margin           : 0px 0px 10px 0px;
   padding          : 0px 0px 0px 10px;
   list-style       : none;
   border-bottom    : 1px solid black;   
}

DIV.tabs LI, DIV.tabs FORM
{
   display          : inline;
   margin           : 0px;
   padding-left     : 10px;
   padding-right    : 10px;   
   //border-left      : 1px solid black;
   //border-right     : 1px solid black;
}

DIV.tabs A
{
   font-weight      : bold;
   text-decoration  : none;
   
}
DIV.tabs A:before
{
    content: "[";
}
DIV.tabs A:after
{
    content: "]";
}
DIV.tabs A:hover
{
}

DIV.tabs A:link, DIV.tabs A:visited,
DIV.tabs A:active, DIV.tabs A:hover
{
    color: #003300;
}

DIV.nav
{
    padding          : 0px 0px 0px 10px;
    background       : none;
    border           : none;
    border-bottom    : 1px solid black;
}
