/*
    Copyright (c) 2008-2009 NetAllied Systems GmbH

    This file is part of COLLADABaseUtils.

    Licensed under the MIT Open Source License, 
    for details please see LICENSE file or the website
    http://www.opensource.org/licenses/mit-license.php
*/

#include "[!output NAMESPACE]StableHeaders.h"
#include "[!output NAMESPACE][!output CLASS_NAME].h"


namespace [!output NAMESPACE]
{

    //------------------------------
	[!output CLASS_NAME]::[!output CLASS_NAME]()
	{
	}
	
    //------------------------------
	[!output CLASS_NAME]::~[!output CLASS_NAME]()
	{
	}

} // namespace [!output NAMESPACE]
