/*
    Copyright (c) 2008-2009 NetAllied Systems GmbH

    This file is part of DAE2MA.

    Licensed under the MIT Open Source License, 
    for details please see LICENSE file or the website
    http://www.opensource.org/licenses/mit-license.php
*/

#include "DAE2MAStableHeaders.h"
#include "DAE2MAMorphAnimation.h"


namespace DAE2MA
{

    //------------------------------
	MorphAnimation::MorphAnimation() : BaseAnimation ()
	{
	}
	
    //------------------------------
	MorphAnimation::~MorphAnimation()
	{
	}

} // namespace DAE2MA
