/*
    Copyright (c) 2008-2009 NetAllied Systems GmbH

    This file is part of COLLADAFramework.

    Licensed under the MIT Open Source License, 
    for details please see LICENSE file or the website
    http://www.opensource.org/licenses/mit-license.php
*/

#include "COLLADAFWStableHeaders.h"
#include "COLLADAFWMaterial.h"


namespace COLLADAFW
{

    //------------------------------
    Material::Material ( const UniqueId& uniqueId )
		: ObjectTemplate<COLLADA_TYPE::MATERIAL> ( uniqueId )
	{
	}
	
    //------------------------------
	Material::~Material()
	{
	}

} // namespace COLLADAFW
