#ifndef __COLLADABU_VERSIONINFO_H__                                   
#define __COLLADABU_VERSIONINFO_H__                                          

namespace COLLADABU   

{   
	// CURRENT_REVISION = MajorVersion.MinorVersion.Revision
	//

	const String CURRENT_REVISION = "1.6.developer_version";
}  
#endif // __COLLADABU_VERSIONINFO_H__          
