/*
Copyright (c) 2009 NetAllied Systems GmbH

This file is part of dae2ogre.

Licensed under the MIT Open Source License, 
for details please see LICENSE file or the website
http://www.opensource.org/licenses/mit-license.php
*/

#ifndef __DAE2OGRE_STABLE_HEADERS_H__
#define __DAE2OGRE_STABLE_HEADERS_H__

#include <string>
#include <map>
#include <list>
#include <stack>
#include <vector>
#include <iostream>

#endif //__DAE2OGRE_STABLE_HEADERS_H__