/*
    Copyright (c) 2008-2009 NetAllied Systems GmbH

    This file is part of COLLADASaxFrameworkLoader.

    Licensed under the MIT Open Source License,
    for details please see LICENSE file or the website
    http://www.opensource.org/licenses/mit-license.php
*/

#include "COLLADASaxFWLStableHeaders.h"
#include "COLLADASaxFWLColladaParserAutoGen15Private.h"


namespace COLLADASaxFWL15
{


//---------------------------------------------------------------------
void ColladaParserAutoGen15Private::initFunctionMap()
{
mElementFunctionMap[HASH_ELEMENT_COLLADA] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__COLLADA, &ColladaParserAutoGen15Private::_data__COLLADA, &ColladaParserAutoGen15Private::_end__COLLADA, &ColladaParserAutoGen15Private::_preBegin__COLLADA, &ColladaParserAutoGen15Private::_preEnd__COLLADA, &ColladaParserAutoGen15Private::_freeAttributes__COLLADA);
mElementToNamespaceMap[HASH_ELEMENT_COLLADA] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ASSET] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
mElementToNamespaceMap[HASH_ELEMENT_ASSET] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CONTRIBUTOR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__contributor, &ColladaParserAutoGen15Private::_data__contributor, &ColladaParserAutoGen15Private::_end__contributor, &ColladaParserAutoGen15Private::_preBegin__contributor, &ColladaParserAutoGen15Private::_preEnd__contributor, &ColladaParserAutoGen15Private::_freeAttributes__contributor);
mElementToNamespaceMap[HASH_ELEMENT_CONTRIBUTOR] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_AUTHOR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__author, &ColladaParserAutoGen15Private::_data__author, &ColladaParserAutoGen15Private::_end__author, &ColladaParserAutoGen15Private::_preBegin__author, &ColladaParserAutoGen15Private::_preEnd__author, &ColladaParserAutoGen15Private::_freeAttributes__author);
mElementToNamespaceMap[HASH_ELEMENT_AUTHOR] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_AUTHOR_EMAIL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_email, &ColladaParserAutoGen15Private::_data__author_email, &ColladaParserAutoGen15Private::_end__author_email, &ColladaParserAutoGen15Private::_preBegin__author_email, &ColladaParserAutoGen15Private::_preEnd__author_email, &ColladaParserAutoGen15Private::_freeAttributes__author_email);
mElementToNamespaceMap[HASH_ELEMENT_AUTHOR_EMAIL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_AUTHOR_WEBSITE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_website, &ColladaParserAutoGen15Private::_data__author_website, &ColladaParserAutoGen15Private::_end__author_website, &ColladaParserAutoGen15Private::_preBegin__author_website, &ColladaParserAutoGen15Private::_preEnd__author_website, &ColladaParserAutoGen15Private::_freeAttributes__author_website);
mElementToNamespaceMap[HASH_ELEMENT_AUTHOR_WEBSITE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_AUTHORING_TOOL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__authoring_tool, &ColladaParserAutoGen15Private::_data__authoring_tool, &ColladaParserAutoGen15Private::_end__authoring_tool, &ColladaParserAutoGen15Private::_preBegin__authoring_tool, &ColladaParserAutoGen15Private::_preEnd__authoring_tool, &ColladaParserAutoGen15Private::_freeAttributes__authoring_tool);
mElementToNamespaceMap[HASH_ELEMENT_AUTHORING_TOOL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_COMMENTS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__comments, &ColladaParserAutoGen15Private::_data__comments, &ColladaParserAutoGen15Private::_end__comments, &ColladaParserAutoGen15Private::_preBegin__comments, &ColladaParserAutoGen15Private::_preEnd__comments, &ColladaParserAutoGen15Private::_freeAttributes__comments);
mElementToNamespaceMap[HASH_ELEMENT_COMMENTS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_COPYRIGHT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__copyright, &ColladaParserAutoGen15Private::_data__copyright, &ColladaParserAutoGen15Private::_end__copyright, &ColladaParserAutoGen15Private::_preBegin__copyright, &ColladaParserAutoGen15Private::_preEnd__copyright, &ColladaParserAutoGen15Private::_freeAttributes__copyright);
mElementToNamespaceMap[HASH_ELEMENT_COPYRIGHT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SOURCE_DATA] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__source_data, &ColladaParserAutoGen15Private::_data__source_data, &ColladaParserAutoGen15Private::_end__source_data, &ColladaParserAutoGen15Private::_preBegin__source_data, &ColladaParserAutoGen15Private::_preEnd__source_data, &ColladaParserAutoGen15Private::_freeAttributes__source_data);
mElementToNamespaceMap[HASH_ELEMENT_SOURCE_DATA] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_COVERAGE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__coverage, &ColladaParserAutoGen15Private::_data__coverage, &ColladaParserAutoGen15Private::_end__coverage, &ColladaParserAutoGen15Private::_preBegin__coverage, &ColladaParserAutoGen15Private::_preEnd__coverage, &ColladaParserAutoGen15Private::_freeAttributes__coverage);
mElementToNamespaceMap[HASH_ELEMENT_COVERAGE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_GEOGRAPHIC_LOCATION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__geographic_location, &ColladaParserAutoGen15Private::_data__geographic_location, &ColladaParserAutoGen15Private::_end__geographic_location, &ColladaParserAutoGen15Private::_preBegin__geographic_location, &ColladaParserAutoGen15Private::_preEnd__geographic_location, &ColladaParserAutoGen15Private::_freeAttributes__geographic_location);
mElementToNamespaceMap[HASH_ELEMENT_GEOGRAPHIC_LOCATION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LONGITUDE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__longitude, &ColladaParserAutoGen15Private::_data__longitude, &ColladaParserAutoGen15Private::_end__longitude, &ColladaParserAutoGen15Private::_preBegin__longitude, &ColladaParserAutoGen15Private::_preEnd__longitude, &ColladaParserAutoGen15Private::_freeAttributes__longitude);
mElementToNamespaceMap[HASH_ELEMENT_LONGITUDE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LATITUDE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__latitude, &ColladaParserAutoGen15Private::_data__latitude, &ColladaParserAutoGen15Private::_end__latitude, &ColladaParserAutoGen15Private::_preBegin__latitude, &ColladaParserAutoGen15Private::_preEnd__latitude, &ColladaParserAutoGen15Private::_freeAttributes__latitude);
mElementToNamespaceMap[HASH_ELEMENT_LATITUDE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ALTITUDE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__altitude, &ColladaParserAutoGen15Private::_data__altitude, &ColladaParserAutoGen15Private::_end__altitude, &ColladaParserAutoGen15Private::_preBegin__altitude, &ColladaParserAutoGen15Private::_preEnd__altitude, &ColladaParserAutoGen15Private::_freeAttributes__altitude);
mElementToNamespaceMap[HASH_ELEMENT_ALTITUDE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CREATED] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__created, &ColladaParserAutoGen15Private::_data__created, &ColladaParserAutoGen15Private::_end__created, &ColladaParserAutoGen15Private::_preBegin__created, &ColladaParserAutoGen15Private::_preEnd__created, &ColladaParserAutoGen15Private::_freeAttributes__created);
mElementToNamespaceMap[HASH_ELEMENT_CREATED] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_KEYWORDS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
mElementToNamespaceMap[HASH_ELEMENT_KEYWORDS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_MODIFIED] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
mElementToNamespaceMap[HASH_ELEMENT_MODIFIED] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_REVISION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
mElementToNamespaceMap[HASH_ELEMENT_REVISION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SUBJECT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
mElementToNamespaceMap[HASH_ELEMENT_SUBJECT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TITLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
mElementToNamespaceMap[HASH_ELEMENT_TITLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_UNIT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
mElementToNamespaceMap[HASH_ELEMENT_UNIT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_UP_AXIS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
mElementToNamespaceMap[HASH_ELEMENT_UP_AXIS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_EXTRA] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
mElementToNamespaceMap[HASH_ELEMENT_EXTRA] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
mElementToNamespaceMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LIBRARY_ANIMATIONS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_animations, &ColladaParserAutoGen15Private::_data__library_animations, &ColladaParserAutoGen15Private::_end__library_animations, &ColladaParserAutoGen15Private::_preBegin__library_animations, &ColladaParserAutoGen15Private::_preEnd__library_animations, &ColladaParserAutoGen15Private::_freeAttributes__library_animations);
mElementToNamespaceMap[HASH_ELEMENT_LIBRARY_ANIMATIONS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ANIMATION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__animation, &ColladaParserAutoGen15Private::_data__animation, &ColladaParserAutoGen15Private::_end__animation, &ColladaParserAutoGen15Private::_preBegin__animation, &ColladaParserAutoGen15Private::_preEnd__animation, &ColladaParserAutoGen15Private::_freeAttributes__animation);
mElementToNamespaceMap[HASH_ELEMENT_ANIMATION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SOURCE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__source, &ColladaParserAutoGen15Private::_data__source, &ColladaParserAutoGen15Private::_end__source, &ColladaParserAutoGen15Private::_preBegin__source, &ColladaParserAutoGen15Private::_preEnd__source, &ColladaParserAutoGen15Private::_freeAttributes__source);
mElementToNamespaceMap[HASH_ELEMENT_SOURCE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TOKEN_ARRAY] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__token_array, &ColladaParserAutoGen15Private::_data__token_array, &ColladaParserAutoGen15Private::_end__token_array, &ColladaParserAutoGen15Private::_preBegin__token_array, &ColladaParserAutoGen15Private::_preEnd__token_array, &ColladaParserAutoGen15Private::_freeAttributes__token_array);
mElementToNamespaceMap[HASH_ELEMENT_TOKEN_ARRAY] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_IDREF_ARRAY] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__IDREF_array, &ColladaParserAutoGen15Private::_data__IDREF_array, &ColladaParserAutoGen15Private::_end__IDREF_array, &ColladaParserAutoGen15Private::_preBegin__IDREF_array, &ColladaParserAutoGen15Private::_preEnd__IDREF_array, &ColladaParserAutoGen15Private::_freeAttributes__IDREF_array);
mElementToNamespaceMap[HASH_ELEMENT_IDREF_ARRAY] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_NAME_ARRAY] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__Name_array, &ColladaParserAutoGen15Private::_data__Name_array, &ColladaParserAutoGen15Private::_end__Name_array, &ColladaParserAutoGen15Private::_preBegin__Name_array, &ColladaParserAutoGen15Private::_preEnd__Name_array, &ColladaParserAutoGen15Private::_freeAttributes__Name_array);
mElementToNamespaceMap[HASH_ELEMENT_NAME_ARRAY] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BOOL_ARRAY] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool_array, &ColladaParserAutoGen15Private::_data__bool_array, &ColladaParserAutoGen15Private::_end__bool_array, &ColladaParserAutoGen15Private::_preBegin__bool_array, &ColladaParserAutoGen15Private::_preEnd__bool_array, &ColladaParserAutoGen15Private::_freeAttributes__bool_array);
mElementToNamespaceMap[HASH_ELEMENT_BOOL_ARRAY] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FLOAT_ARRAY] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__float_array, &ColladaParserAutoGen15Private::_data__float_array, &ColladaParserAutoGen15Private::_end__float_array, &ColladaParserAutoGen15Private::_preBegin__float_array, &ColladaParserAutoGen15Private::_preEnd__float_array, &ColladaParserAutoGen15Private::_freeAttributes__float_array);
mElementToNamespaceMap[HASH_ELEMENT_FLOAT_ARRAY] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INT_ARRAY] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__int_array, &ColladaParserAutoGen15Private::_data__int_array, &ColladaParserAutoGen15Private::_end__int_array, &ColladaParserAutoGen15Private::_preBegin__int_array, &ColladaParserAutoGen15Private::_preEnd__int_array, &ColladaParserAutoGen15Private::_freeAttributes__int_array);
mElementToNamespaceMap[HASH_ELEMENT_INT_ARRAY] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SIDREF_ARRAY] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__SIDREF_array, &ColladaParserAutoGen15Private::_data__SIDREF_array, &ColladaParserAutoGen15Private::_end__SIDREF_array, &ColladaParserAutoGen15Private::_preBegin__SIDREF_array, &ColladaParserAutoGen15Private::_preEnd__SIDREF_array, &ColladaParserAutoGen15Private::_freeAttributes__SIDREF_array);
mElementToNamespaceMap[HASH_ELEMENT_SIDREF_ARRAY] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SOURCE_TYPE____TECHNIQUE_COMMON] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__source_type____technique_common, &ColladaParserAutoGen15Private::_data__source_type____technique_common, &ColladaParserAutoGen15Private::_end__source_type____technique_common, &ColladaParserAutoGen15Private::_preBegin__source_type____technique_common, &ColladaParserAutoGen15Private::_preEnd__source_type____technique_common, &ColladaParserAutoGen15Private::_freeAttributes__source_type____technique_common);
mElementToNamespaceMap[HASH_ELEMENT_SOURCE_TYPE____TECHNIQUE_COMMON] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ACCESSOR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__accessor, &ColladaParserAutoGen15Private::_data__accessor, &ColladaParserAutoGen15Private::_end__accessor, &ColladaParserAutoGen15Private::_preBegin__accessor, &ColladaParserAutoGen15Private::_preEnd__accessor, &ColladaParserAutoGen15Private::_freeAttributes__accessor);
mElementToNamespaceMap[HASH_ELEMENT_ACCESSOR] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PARAM____PARAM_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__param____param_type, &ColladaParserAutoGen15Private::_data__param____param_type, &ColladaParserAutoGen15Private::_end__param____param_type, &ColladaParserAutoGen15Private::_preBegin__param____param_type, &ColladaParserAutoGen15Private::_preEnd__param____param_type, &ColladaParserAutoGen15Private::_freeAttributes__param____param_type);
mElementToNamespaceMap[HASH_ELEMENT_PARAM____PARAM_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SAMPLER] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler, &ColladaParserAutoGen15Private::_data__sampler, &ColladaParserAutoGen15Private::_end__sampler, &ColladaParserAutoGen15Private::_preBegin__sampler, &ColladaParserAutoGen15Private::_preEnd__sampler, &ColladaParserAutoGen15Private::_freeAttributes__sampler);
mElementToNamespaceMap[HASH_ELEMENT_SAMPLER] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INPUT____INPUT_LOCAL_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__input____input_local_type, &ColladaParserAutoGen15Private::_data__input____input_local_type, &ColladaParserAutoGen15Private::_end__input____input_local_type, &ColladaParserAutoGen15Private::_preBegin__input____input_local_type, &ColladaParserAutoGen15Private::_preEnd__input____input_local_type, &ColladaParserAutoGen15Private::_freeAttributes__input____input_local_type);
mElementToNamespaceMap[HASH_ELEMENT_INPUT____INPUT_LOCAL_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CHANNEL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__channel, &ColladaParserAutoGen15Private::_data__channel, &ColladaParserAutoGen15Private::_end__channel, &ColladaParserAutoGen15Private::_preBegin__channel, &ColladaParserAutoGen15Private::_preEnd__channel, &ColladaParserAutoGen15Private::_freeAttributes__channel);
mElementToNamespaceMap[HASH_ELEMENT_CHANNEL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_animation_clips, &ColladaParserAutoGen15Private::_data__library_animation_clips, &ColladaParserAutoGen15Private::_end__library_animation_clips, &ColladaParserAutoGen15Private::_preBegin__library_animation_clips, &ColladaParserAutoGen15Private::_preEnd__library_animation_clips, &ColladaParserAutoGen15Private::_freeAttributes__library_animation_clips);
mElementToNamespaceMap[HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ANIMATION_CLIP] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__animation_clip, &ColladaParserAutoGen15Private::_data__animation_clip, &ColladaParserAutoGen15Private::_end__animation_clip, &ColladaParserAutoGen15Private::_preBegin__animation_clip, &ColladaParserAutoGen15Private::_preEnd__animation_clip, &ColladaParserAutoGen15Private::_freeAttributes__animation_clip);
mElementToNamespaceMap[HASH_ELEMENT_ANIMATION_CLIP] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INSTANCE_ANIMATION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_animation, &ColladaParserAutoGen15Private::_data__instance_animation, &ColladaParserAutoGen15Private::_end__instance_animation, &ColladaParserAutoGen15Private::_preBegin__instance_animation, &ColladaParserAutoGen15Private::_preEnd__instance_animation, &ColladaParserAutoGen15Private::_freeAttributes__instance_animation);
mElementToNamespaceMap[HASH_ELEMENT_INSTANCE_ANIMATION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INSTANCE_FORMULA] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_formula, &ColladaParserAutoGen15Private::_data__instance_formula, &ColladaParserAutoGen15Private::_end__instance_formula, &ColladaParserAutoGen15Private::_preBegin__instance_formula, &ColladaParserAutoGen15Private::_preEnd__instance_formula, &ColladaParserAutoGen15Private::_freeAttributes__instance_formula);
mElementToNamespaceMap[HASH_ELEMENT_INSTANCE_FORMULA] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SETPARAM____FORMULA_SETPARAM_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__setparam____formula_setparam_type, &ColladaParserAutoGen15Private::_data__setparam____formula_setparam_type, &ColladaParserAutoGen15Private::_end__setparam____formula_setparam_type, &ColladaParserAutoGen15Private::_preBegin__setparam____formula_setparam_type, &ColladaParserAutoGen15Private::_preEnd__setparam____formula_setparam_type, &ColladaParserAutoGen15Private::_freeAttributes__setparam____formula_setparam_type);
mElementToNamespaceMap[HASH_ELEMENT_SETPARAM____FORMULA_SETPARAM_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FLOAT____FLOAT_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__float____float_type, &ColladaParserAutoGen15Private::_data__float____float_type, &ColladaParserAutoGen15Private::_end__float____float_type, &ColladaParserAutoGen15Private::_preBegin__float____float_type, &ColladaParserAutoGen15Private::_preEnd__float____float_type, &ColladaParserAutoGen15Private::_freeAttributes__float____float_type);
mElementToNamespaceMap[HASH_ELEMENT_FLOAT____FLOAT_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INT____INT_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__int____int_type, &ColladaParserAutoGen15Private::_data__int____int_type, &ColladaParserAutoGen15Private::_end__int____int_type, &ColladaParserAutoGen15Private::_preBegin__int____int_type, &ColladaParserAutoGen15Private::_preEnd__int____int_type, &ColladaParserAutoGen15Private::_freeAttributes__int____int_type);
mElementToNamespaceMap[HASH_ELEMENT_INT____INT_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SIDREF] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__SIDREF, &ColladaParserAutoGen15Private::_data__SIDREF, &ColladaParserAutoGen15Private::_end__SIDREF, &ColladaParserAutoGen15Private::_preBegin__SIDREF, &ColladaParserAutoGen15Private::_preEnd__SIDREF, &ColladaParserAutoGen15Private::_freeAttributes__SIDREF);
mElementToNamespaceMap[HASH_ELEMENT_SIDREF] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BOOL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool, &ColladaParserAutoGen15Private::_data__bool, &ColladaParserAutoGen15Private::_end__bool, &ColladaParserAutoGen15Private::_preBegin__bool, &ColladaParserAutoGen15Private::_preEnd__bool, &ColladaParserAutoGen15Private::_freeAttributes__bool);
mElementToNamespaceMap[HASH_ELEMENT_BOOL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CONNECT_PARAM] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__connect_param, &ColladaParserAutoGen15Private::_data__connect_param, &ColladaParserAutoGen15Private::_end__connect_param, &ColladaParserAutoGen15Private::_preBegin__connect_param, &ColladaParserAutoGen15Private::_preEnd__connect_param, &ColladaParserAutoGen15Private::_freeAttributes__connect_param);
mElementToNamespaceMap[HASH_ELEMENT_CONNECT_PARAM] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LIBRARY_CAMERAS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_cameras, &ColladaParserAutoGen15Private::_data__library_cameras, &ColladaParserAutoGen15Private::_end__library_cameras, &ColladaParserAutoGen15Private::_preBegin__library_cameras, &ColladaParserAutoGen15Private::_preEnd__library_cameras, &ColladaParserAutoGen15Private::_freeAttributes__library_cameras);
mElementToNamespaceMap[HASH_ELEMENT_LIBRARY_CAMERAS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CAMERA] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__camera, &ColladaParserAutoGen15Private::_data__camera, &ColladaParserAutoGen15Private::_end__camera, &ColladaParserAutoGen15Private::_preBegin__camera, &ColladaParserAutoGen15Private::_preEnd__camera, &ColladaParserAutoGen15Private::_freeAttributes__camera);
mElementToNamespaceMap[HASH_ELEMENT_CAMERA] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_OPTICS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__optics, &ColladaParserAutoGen15Private::_data__optics, &ColladaParserAutoGen15Private::_end__optics, &ColladaParserAutoGen15Private::_preBegin__optics, &ColladaParserAutoGen15Private::_preEnd__optics, &ColladaParserAutoGen15Private::_freeAttributes__optics);
mElementToNamespaceMap[HASH_ELEMENT_OPTICS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_OPTICS__TECHNIQUE_COMMON] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__optics__technique_common, &ColladaParserAutoGen15Private::_data__optics__technique_common, &ColladaParserAutoGen15Private::_end__optics__technique_common, &ColladaParserAutoGen15Private::_preBegin__optics__technique_common, &ColladaParserAutoGen15Private::_preEnd__optics__technique_common, &ColladaParserAutoGen15Private::_freeAttributes__optics__technique_common);
mElementToNamespaceMap[HASH_ELEMENT_OPTICS__TECHNIQUE_COMMON] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ORTHOGRAPHIC] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__orthographic, &ColladaParserAutoGen15Private::_data__orthographic, &ColladaParserAutoGen15Private::_end__orthographic, &ColladaParserAutoGen15Private::_preBegin__orthographic, &ColladaParserAutoGen15Private::_preEnd__orthographic, &ColladaParserAutoGen15Private::_freeAttributes__orthographic);
mElementToNamespaceMap[HASH_ELEMENT_ORTHOGRAPHIC] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_XMAG] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__xmag, &ColladaParserAutoGen15Private::_data__xmag, &ColladaParserAutoGen15Private::_end__xmag, &ColladaParserAutoGen15Private::_preBegin__xmag, &ColladaParserAutoGen15Private::_preEnd__xmag, &ColladaParserAutoGen15Private::_freeAttributes__xmag);
mElementToNamespaceMap[HASH_ELEMENT_XMAG] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_YMAG] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__ymag, &ColladaParserAutoGen15Private::_data__ymag, &ColladaParserAutoGen15Private::_end__ymag, &ColladaParserAutoGen15Private::_preBegin__ymag, &ColladaParserAutoGen15Private::_preEnd__ymag, &ColladaParserAutoGen15Private::_freeAttributes__ymag);
mElementToNamespaceMap[HASH_ELEMENT_YMAG] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ASPECT_RATIO] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__aspect_ratio, &ColladaParserAutoGen15Private::_data__aspect_ratio, &ColladaParserAutoGen15Private::_end__aspect_ratio, &ColladaParserAutoGen15Private::_preBegin__aspect_ratio, &ColladaParserAutoGen15Private::_preEnd__aspect_ratio, &ColladaParserAutoGen15Private::_freeAttributes__aspect_ratio);
mElementToNamespaceMap[HASH_ELEMENT_ASPECT_RATIO] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ZNEAR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__znear, &ColladaParserAutoGen15Private::_data__znear, &ColladaParserAutoGen15Private::_end__znear, &ColladaParserAutoGen15Private::_preBegin__znear, &ColladaParserAutoGen15Private::_preEnd__znear, &ColladaParserAutoGen15Private::_freeAttributes__znear);
mElementToNamespaceMap[HASH_ELEMENT_ZNEAR] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ZFAR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__zfar, &ColladaParserAutoGen15Private::_data__zfar, &ColladaParserAutoGen15Private::_end__zfar, &ColladaParserAutoGen15Private::_preBegin__zfar, &ColladaParserAutoGen15Private::_preEnd__zfar, &ColladaParserAutoGen15Private::_freeAttributes__zfar);
mElementToNamespaceMap[HASH_ELEMENT_ZFAR] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PERSPECTIVE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__perspective, &ColladaParserAutoGen15Private::_data__perspective, &ColladaParserAutoGen15Private::_end__perspective, &ColladaParserAutoGen15Private::_preBegin__perspective, &ColladaParserAutoGen15Private::_preEnd__perspective, &ColladaParserAutoGen15Private::_freeAttributes__perspective);
mElementToNamespaceMap[HASH_ELEMENT_PERSPECTIVE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_XFOV] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__xfov, &ColladaParserAutoGen15Private::_data__xfov, &ColladaParserAutoGen15Private::_end__xfov, &ColladaParserAutoGen15Private::_preBegin__xfov, &ColladaParserAutoGen15Private::_preEnd__xfov, &ColladaParserAutoGen15Private::_freeAttributes__xfov);
mElementToNamespaceMap[HASH_ELEMENT_XFOV] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_YFOV] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__yfov, &ColladaParserAutoGen15Private::_data__yfov, &ColladaParserAutoGen15Private::_end__yfov, &ColladaParserAutoGen15Private::_preBegin__yfov, &ColladaParserAutoGen15Private::_preEnd__yfov, &ColladaParserAutoGen15Private::_freeAttributes__yfov);
mElementToNamespaceMap[HASH_ELEMENT_YFOV] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_IMAGER] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__imager, &ColladaParserAutoGen15Private::_data__imager, &ColladaParserAutoGen15Private::_end__imager, &ColladaParserAutoGen15Private::_preBegin__imager, &ColladaParserAutoGen15Private::_preEnd__imager, &ColladaParserAutoGen15Private::_freeAttributes__imager);
mElementToNamespaceMap[HASH_ELEMENT_IMAGER] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LIBRARY_CONTROLLERS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_controllers, &ColladaParserAutoGen15Private::_data__library_controllers, &ColladaParserAutoGen15Private::_end__library_controllers, &ColladaParserAutoGen15Private::_preBegin__library_controllers, &ColladaParserAutoGen15Private::_preEnd__library_controllers, &ColladaParserAutoGen15Private::_freeAttributes__library_controllers);
mElementToNamespaceMap[HASH_ELEMENT_LIBRARY_CONTROLLERS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CONTROLLER] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__controller, &ColladaParserAutoGen15Private::_data__controller, &ColladaParserAutoGen15Private::_end__controller, &ColladaParserAutoGen15Private::_preBegin__controller, &ColladaParserAutoGen15Private::_preEnd__controller, &ColladaParserAutoGen15Private::_freeAttributes__controller);
mElementToNamespaceMap[HASH_ELEMENT_CONTROLLER] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SKIN] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__skin, &ColladaParserAutoGen15Private::_data__skin, &ColladaParserAutoGen15Private::_end__skin, &ColladaParserAutoGen15Private::_preBegin__skin, &ColladaParserAutoGen15Private::_preEnd__skin, &ColladaParserAutoGen15Private::_freeAttributes__skin);
mElementToNamespaceMap[HASH_ELEMENT_SKIN] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BIND_SHAPE_MATRIX] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__bind_shape_matrix, &ColladaParserAutoGen15Private::_data__bind_shape_matrix, &ColladaParserAutoGen15Private::_end__bind_shape_matrix, &ColladaParserAutoGen15Private::_preBegin__bind_shape_matrix, &ColladaParserAutoGen15Private::_preEnd__bind_shape_matrix, &ColladaParserAutoGen15Private::_freeAttributes__bind_shape_matrix);
mElementToNamespaceMap[HASH_ELEMENT_BIND_SHAPE_MATRIX] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_JOINTS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__joints, &ColladaParserAutoGen15Private::_data__joints, &ColladaParserAutoGen15Private::_end__joints, &ColladaParserAutoGen15Private::_preBegin__joints, &ColladaParserAutoGen15Private::_preEnd__joints, &ColladaParserAutoGen15Private::_freeAttributes__joints);
mElementToNamespaceMap[HASH_ELEMENT_JOINTS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_VERTEX_WEIGHTS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__vertex_weights, &ColladaParserAutoGen15Private::_data__vertex_weights, &ColladaParserAutoGen15Private::_end__vertex_weights, &ColladaParserAutoGen15Private::_preBegin__vertex_weights, &ColladaParserAutoGen15Private::_preEnd__vertex_weights, &ColladaParserAutoGen15Private::_freeAttributes__vertex_weights);
mElementToNamespaceMap[HASH_ELEMENT_VERTEX_WEIGHTS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INPUT____INPUT_LOCAL_OFFSET_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__input____input_local_offset_type, &ColladaParserAutoGen15Private::_data__input____input_local_offset_type, &ColladaParserAutoGen15Private::_end__input____input_local_offset_type, &ColladaParserAutoGen15Private::_preBegin__input____input_local_offset_type, &ColladaParserAutoGen15Private::_preEnd__input____input_local_offset_type, &ColladaParserAutoGen15Private::_freeAttributes__input____input_local_offset_type);
mElementToNamespaceMap[HASH_ELEMENT_INPUT____INPUT_LOCAL_OFFSET_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_VCOUNT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__vcount, &ColladaParserAutoGen15Private::_data__vcount, &ColladaParserAutoGen15Private::_end__vcount, &ColladaParserAutoGen15Private::_preBegin__vcount, &ColladaParserAutoGen15Private::_preEnd__vcount, &ColladaParserAutoGen15Private::_freeAttributes__vcount);
mElementToNamespaceMap[HASH_ELEMENT_VCOUNT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_V] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__v, &ColladaParserAutoGen15Private::_data__v, &ColladaParserAutoGen15Private::_end__v, &ColladaParserAutoGen15Private::_preBegin__v, &ColladaParserAutoGen15Private::_preEnd__v, &ColladaParserAutoGen15Private::_freeAttributes__v);
mElementToNamespaceMap[HASH_ELEMENT_V] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_MORPH] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__morph, &ColladaParserAutoGen15Private::_data__morph, &ColladaParserAutoGen15Private::_end__morph, &ColladaParserAutoGen15Private::_preBegin__morph, &ColladaParserAutoGen15Private::_preEnd__morph, &ColladaParserAutoGen15Private::_freeAttributes__morph);
mElementToNamespaceMap[HASH_ELEMENT_MORPH] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TARGETS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__targets, &ColladaParserAutoGen15Private::_data__targets, &ColladaParserAutoGen15Private::_end__targets, &ColladaParserAutoGen15Private::_preBegin__targets, &ColladaParserAutoGen15Private::_preEnd__targets, &ColladaParserAutoGen15Private::_freeAttributes__targets);
mElementToNamespaceMap[HASH_ELEMENT_TARGETS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LIBRARY_GEOMETRIES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_geometries, &ColladaParserAutoGen15Private::_data__library_geometries, &ColladaParserAutoGen15Private::_end__library_geometries, &ColladaParserAutoGen15Private::_preBegin__library_geometries, &ColladaParserAutoGen15Private::_preEnd__library_geometries, &ColladaParserAutoGen15Private::_freeAttributes__library_geometries);
mElementToNamespaceMap[HASH_ELEMENT_LIBRARY_GEOMETRIES] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_GEOMETRY] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__geometry, &ColladaParserAutoGen15Private::_data__geometry, &ColladaParserAutoGen15Private::_end__geometry, &ColladaParserAutoGen15Private::_preBegin__geometry, &ColladaParserAutoGen15Private::_preEnd__geometry, &ColladaParserAutoGen15Private::_freeAttributes__geometry);
mElementToNamespaceMap[HASH_ELEMENT_GEOMETRY] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CONVEX_MESH] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__convex_mesh, &ColladaParserAutoGen15Private::_data__convex_mesh, &ColladaParserAutoGen15Private::_end__convex_mesh, &ColladaParserAutoGen15Private::_preBegin__convex_mesh, &ColladaParserAutoGen15Private::_preEnd__convex_mesh, &ColladaParserAutoGen15Private::_freeAttributes__convex_mesh);
mElementToNamespaceMap[HASH_ELEMENT_CONVEX_MESH] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_VERTICES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__vertices, &ColladaParserAutoGen15Private::_data__vertices, &ColladaParserAutoGen15Private::_end__vertices, &ColladaParserAutoGen15Private::_preBegin__vertices, &ColladaParserAutoGen15Private::_preEnd__vertices, &ColladaParserAutoGen15Private::_freeAttributes__vertices);
mElementToNamespaceMap[HASH_ELEMENT_VERTICES] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LINES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__lines, &ColladaParserAutoGen15Private::_data__lines, &ColladaParserAutoGen15Private::_end__lines, &ColladaParserAutoGen15Private::_preBegin__lines, &ColladaParserAutoGen15Private::_preEnd__lines, &ColladaParserAutoGen15Private::_freeAttributes__lines);
mElementToNamespaceMap[HASH_ELEMENT_LINES] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_P] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__p, &ColladaParserAutoGen15Private::_data__p, &ColladaParserAutoGen15Private::_end__p, &ColladaParserAutoGen15Private::_preBegin__p, &ColladaParserAutoGen15Private::_preEnd__p, &ColladaParserAutoGen15Private::_freeAttributes__p);
mElementToNamespaceMap[HASH_ELEMENT_P] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LINESTRIPS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__linestrips, &ColladaParserAutoGen15Private::_data__linestrips, &ColladaParserAutoGen15Private::_end__linestrips, &ColladaParserAutoGen15Private::_preBegin__linestrips, &ColladaParserAutoGen15Private::_preEnd__linestrips, &ColladaParserAutoGen15Private::_freeAttributes__linestrips);
mElementToNamespaceMap[HASH_ELEMENT_LINESTRIPS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_POLYGONS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__polygons, &ColladaParserAutoGen15Private::_data__polygons, &ColladaParserAutoGen15Private::_end__polygons, &ColladaParserAutoGen15Private::_preBegin__polygons, &ColladaParserAutoGen15Private::_preEnd__polygons, &ColladaParserAutoGen15Private::_freeAttributes__polygons);
mElementToNamespaceMap[HASH_ELEMENT_POLYGONS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PH] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__ph, &ColladaParserAutoGen15Private::_data__ph, &ColladaParserAutoGen15Private::_end__ph, &ColladaParserAutoGen15Private::_preBegin__ph, &ColladaParserAutoGen15Private::_preEnd__ph, &ColladaParserAutoGen15Private::_freeAttributes__ph);
mElementToNamespaceMap[HASH_ELEMENT_PH] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_H] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__h, &ColladaParserAutoGen15Private::_data__h, &ColladaParserAutoGen15Private::_end__h, &ColladaParserAutoGen15Private::_preBegin__h, &ColladaParserAutoGen15Private::_preEnd__h, &ColladaParserAutoGen15Private::_freeAttributes__h);
mElementToNamespaceMap[HASH_ELEMENT_H] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_POLYLIST] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__polylist, &ColladaParserAutoGen15Private::_data__polylist, &ColladaParserAutoGen15Private::_end__polylist, &ColladaParserAutoGen15Private::_preBegin__polylist, &ColladaParserAutoGen15Private::_preEnd__polylist, &ColladaParserAutoGen15Private::_freeAttributes__polylist);
mElementToNamespaceMap[HASH_ELEMENT_POLYLIST] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TRIANGLES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__triangles, &ColladaParserAutoGen15Private::_data__triangles, &ColladaParserAutoGen15Private::_end__triangles, &ColladaParserAutoGen15Private::_preBegin__triangles, &ColladaParserAutoGen15Private::_preEnd__triangles, &ColladaParserAutoGen15Private::_freeAttributes__triangles);
mElementToNamespaceMap[HASH_ELEMENT_TRIANGLES] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TRIFANS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__trifans, &ColladaParserAutoGen15Private::_data__trifans, &ColladaParserAutoGen15Private::_end__trifans, &ColladaParserAutoGen15Private::_preBegin__trifans, &ColladaParserAutoGen15Private::_preEnd__trifans, &ColladaParserAutoGen15Private::_freeAttributes__trifans);
mElementToNamespaceMap[HASH_ELEMENT_TRIFANS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TRISTRIPS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__tristrips, &ColladaParserAutoGen15Private::_data__tristrips, &ColladaParserAutoGen15Private::_end__tristrips, &ColladaParserAutoGen15Private::_preBegin__tristrips, &ColladaParserAutoGen15Private::_preEnd__tristrips, &ColladaParserAutoGen15Private::_freeAttributes__tristrips);
mElementToNamespaceMap[HASH_ELEMENT_TRISTRIPS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_MESH] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__mesh, &ColladaParserAutoGen15Private::_data__mesh, &ColladaParserAutoGen15Private::_end__mesh, &ColladaParserAutoGen15Private::_preBegin__mesh, &ColladaParserAutoGen15Private::_preEnd__mesh, &ColladaParserAutoGen15Private::_freeAttributes__mesh);
mElementToNamespaceMap[HASH_ELEMENT_MESH] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SPLINE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__spline, &ColladaParserAutoGen15Private::_data__spline, &ColladaParserAutoGen15Private::_end__spline, &ColladaParserAutoGen15Private::_preBegin__spline, &ColladaParserAutoGen15Private::_preEnd__spline, &ColladaParserAutoGen15Private::_freeAttributes__spline);
mElementToNamespaceMap[HASH_ELEMENT_SPLINE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SPLINE_TYPE____CONTROL_VERTICES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__spline_type____control_vertices, &ColladaParserAutoGen15Private::_data__spline_type____control_vertices, &ColladaParserAutoGen15Private::_end__spline_type____control_vertices, &ColladaParserAutoGen15Private::_preBegin__spline_type____control_vertices, &ColladaParserAutoGen15Private::_preEnd__spline_type____control_vertices, &ColladaParserAutoGen15Private::_freeAttributes__spline_type____control_vertices);
mElementToNamespaceMap[HASH_ELEMENT_SPLINE_TYPE____CONTROL_VERTICES] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BREP] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__brep, &ColladaParserAutoGen15Private::_data__brep, &ColladaParserAutoGen15Private::_end__brep, &ColladaParserAutoGen15Private::_preBegin__brep, &ColladaParserAutoGen15Private::_preEnd__brep, &ColladaParserAutoGen15Private::_freeAttributes__brep);
mElementToNamespaceMap[HASH_ELEMENT_BREP] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CURVES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__curves, &ColladaParserAutoGen15Private::_data__curves, &ColladaParserAutoGen15Private::_end__curves, &ColladaParserAutoGen15Private::_preBegin__curves, &ColladaParserAutoGen15Private::_preEnd__curves, &ColladaParserAutoGen15Private::_freeAttributes__curves);
mElementToNamespaceMap[HASH_ELEMENT_CURVES] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CURVE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__curve, &ColladaParserAutoGen15Private::_data__curve, &ColladaParserAutoGen15Private::_end__curve, &ColladaParserAutoGen15Private::_preBegin__curve, &ColladaParserAutoGen15Private::_preEnd__curve, &ColladaParserAutoGen15Private::_freeAttributes__curve);
mElementToNamespaceMap[HASH_ELEMENT_CURVE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LINE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__line, &ColladaParserAutoGen15Private::_data__line, &ColladaParserAutoGen15Private::_end__line, &ColladaParserAutoGen15Private::_preBegin__line, &ColladaParserAutoGen15Private::_preEnd__line, &ColladaParserAutoGen15Private::_freeAttributes__line);
mElementToNamespaceMap[HASH_ELEMENT_LINE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ORIGIN____FLOAT3_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__origin____float3_type, &ColladaParserAutoGen15Private::_data__origin____float3_type, &ColladaParserAutoGen15Private::_end__origin____float3_type, &ColladaParserAutoGen15Private::_preBegin__origin____float3_type, &ColladaParserAutoGen15Private::_preEnd__origin____float3_type, &ColladaParserAutoGen15Private::_freeAttributes__origin____float3_type);
mElementToNamespaceMap[HASH_ELEMENT_ORIGIN____FLOAT3_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_DIRECTION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__direction, &ColladaParserAutoGen15Private::_data__direction, &ColladaParserAutoGen15Private::_end__direction, &ColladaParserAutoGen15Private::_preBegin__direction, &ColladaParserAutoGen15Private::_preEnd__direction, &ColladaParserAutoGen15Private::_freeAttributes__direction);
mElementToNamespaceMap[HASH_ELEMENT_DIRECTION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CIRCLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__circle, &ColladaParserAutoGen15Private::_data__circle, &ColladaParserAutoGen15Private::_end__circle, &ColladaParserAutoGen15Private::_preBegin__circle, &ColladaParserAutoGen15Private::_preEnd__circle, &ColladaParserAutoGen15Private::_freeAttributes__circle);
mElementToNamespaceMap[HASH_ELEMENT_CIRCLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_RADIUS____FLOAT_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__radius____float_type, &ColladaParserAutoGen15Private::_data__radius____float_type, &ColladaParserAutoGen15Private::_end__radius____float_type, &ColladaParserAutoGen15Private::_preBegin__radius____float_type, &ColladaParserAutoGen15Private::_preEnd__radius____float_type, &ColladaParserAutoGen15Private::_freeAttributes__radius____float_type);
mElementToNamespaceMap[HASH_ELEMENT_RADIUS____FLOAT_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ELLIPSE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__ellipse, &ColladaParserAutoGen15Private::_data__ellipse, &ColladaParserAutoGen15Private::_end__ellipse, &ColladaParserAutoGen15Private::_preBegin__ellipse, &ColladaParserAutoGen15Private::_preEnd__ellipse, &ColladaParserAutoGen15Private::_freeAttributes__ellipse);
mElementToNamespaceMap[HASH_ELEMENT_ELLIPSE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_RADIUS____FLOAT2_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__radius____float2_type, &ColladaParserAutoGen15Private::_data__radius____float2_type, &ColladaParserAutoGen15Private::_end__radius____float2_type, &ColladaParserAutoGen15Private::_preBegin__radius____float2_type, &ColladaParserAutoGen15Private::_preEnd__radius____float2_type, &ColladaParserAutoGen15Private::_freeAttributes__radius____float2_type);
mElementToNamespaceMap[HASH_ELEMENT_RADIUS____FLOAT2_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PARABOLA] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__parabola, &ColladaParserAutoGen15Private::_data__parabola, &ColladaParserAutoGen15Private::_end__parabola, &ColladaParserAutoGen15Private::_preBegin__parabola, &ColladaParserAutoGen15Private::_preEnd__parabola, &ColladaParserAutoGen15Private::_freeAttributes__parabola);
mElementToNamespaceMap[HASH_ELEMENT_PARABOLA] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FOCAL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__focal, &ColladaParserAutoGen15Private::_data__focal, &ColladaParserAutoGen15Private::_end__focal, &ColladaParserAutoGen15Private::_preBegin__focal, &ColladaParserAutoGen15Private::_preEnd__focal, &ColladaParserAutoGen15Private::_freeAttributes__focal);
mElementToNamespaceMap[HASH_ELEMENT_FOCAL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_HYPERBOLA] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__hyperbola, &ColladaParserAutoGen15Private::_data__hyperbola, &ColladaParserAutoGen15Private::_end__hyperbola, &ColladaParserAutoGen15Private::_preBegin__hyperbola, &ColladaParserAutoGen15Private::_preEnd__hyperbola, &ColladaParserAutoGen15Private::_freeAttributes__hyperbola);
mElementToNamespaceMap[HASH_ELEMENT_HYPERBOLA] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_NURBS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__nurbs, &ColladaParserAutoGen15Private::_data__nurbs, &ColladaParserAutoGen15Private::_end__nurbs, &ColladaParserAutoGen15Private::_preBegin__nurbs, &ColladaParserAutoGen15Private::_preEnd__nurbs, &ColladaParserAutoGen15Private::_freeAttributes__nurbs);
mElementToNamespaceMap[HASH_ELEMENT_NURBS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_NURBS_TYPE____CONTROL_VERTICES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__nurbs_type____control_vertices, &ColladaParserAutoGen15Private::_data__nurbs_type____control_vertices, &ColladaParserAutoGen15Private::_end__nurbs_type____control_vertices, &ColladaParserAutoGen15Private::_preBegin__nurbs_type____control_vertices, &ColladaParserAutoGen15Private::_preEnd__nurbs_type____control_vertices, &ColladaParserAutoGen15Private::_freeAttributes__nurbs_type____control_vertices);
mElementToNamespaceMap[HASH_ELEMENT_NURBS_TYPE____CONTROL_VERTICES] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ORIENT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__orient, &ColladaParserAutoGen15Private::_data__orient, &ColladaParserAutoGen15Private::_end__orient, &ColladaParserAutoGen15Private::_preBegin__orient, &ColladaParserAutoGen15Private::_preEnd__orient, &ColladaParserAutoGen15Private::_freeAttributes__orient);
mElementToNamespaceMap[HASH_ELEMENT_ORIENT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ORIGIN____ORIGIN_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__origin____origin_type, &ColladaParserAutoGen15Private::_data__origin____origin_type, &ColladaParserAutoGen15Private::_end__origin____origin_type, &ColladaParserAutoGen15Private::_preBegin__origin____origin_type, &ColladaParserAutoGen15Private::_preEnd__origin____origin_type, &ColladaParserAutoGen15Private::_freeAttributes__origin____origin_type);
mElementToNamespaceMap[HASH_ELEMENT_ORIGIN____ORIGIN_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SURFACE_CURVES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__surface_curves, &ColladaParserAutoGen15Private::_data__surface_curves, &ColladaParserAutoGen15Private::_end__surface_curves, &ColladaParserAutoGen15Private::_preBegin__surface_curves, &ColladaParserAutoGen15Private::_preEnd__surface_curves, &ColladaParserAutoGen15Private::_freeAttributes__surface_curves);
mElementToNamespaceMap[HASH_ELEMENT_SURFACE_CURVES] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SURFACES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__surfaces, &ColladaParserAutoGen15Private::_data__surfaces, &ColladaParserAutoGen15Private::_end__surfaces, &ColladaParserAutoGen15Private::_preBegin__surfaces, &ColladaParserAutoGen15Private::_preEnd__surfaces, &ColladaParserAutoGen15Private::_freeAttributes__surfaces);
mElementToNamespaceMap[HASH_ELEMENT_SURFACES] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SURFACE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__surface, &ColladaParserAutoGen15Private::_data__surface, &ColladaParserAutoGen15Private::_end__surface, &ColladaParserAutoGen15Private::_preBegin__surface, &ColladaParserAutoGen15Private::_preEnd__surface, &ColladaParserAutoGen15Private::_freeAttributes__surface);
mElementToNamespaceMap[HASH_ELEMENT_SURFACE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PLANE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__plane, &ColladaParserAutoGen15Private::_data__plane, &ColladaParserAutoGen15Private::_end__plane, &ColladaParserAutoGen15Private::_preBegin__plane, &ColladaParserAutoGen15Private::_preEnd__plane, &ColladaParserAutoGen15Private::_freeAttributes__plane);
mElementToNamespaceMap[HASH_ELEMENT_PLANE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_EQUATION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__equation, &ColladaParserAutoGen15Private::_data__equation, &ColladaParserAutoGen15Private::_end__equation, &ColladaParserAutoGen15Private::_preBegin__equation, &ColladaParserAutoGen15Private::_preEnd__equation, &ColladaParserAutoGen15Private::_freeAttributes__equation);
mElementToNamespaceMap[HASH_ELEMENT_EQUATION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SPHERE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__sphere, &ColladaParserAutoGen15Private::_data__sphere, &ColladaParserAutoGen15Private::_end__sphere, &ColladaParserAutoGen15Private::_preBegin__sphere, &ColladaParserAutoGen15Private::_preEnd__sphere, &ColladaParserAutoGen15Private::_freeAttributes__sphere);
mElementToNamespaceMap[HASH_ELEMENT_SPHERE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TORUS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__torus, &ColladaParserAutoGen15Private::_data__torus, &ColladaParserAutoGen15Private::_end__torus, &ColladaParserAutoGen15Private::_preBegin__torus, &ColladaParserAutoGen15Private::_preEnd__torus, &ColladaParserAutoGen15Private::_freeAttributes__torus);
mElementToNamespaceMap[HASH_ELEMENT_TORUS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SWEPT_SURFACE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__swept_surface, &ColladaParserAutoGen15Private::_data__swept_surface, &ColladaParserAutoGen15Private::_end__swept_surface, &ColladaParserAutoGen15Private::_preBegin__swept_surface, &ColladaParserAutoGen15Private::_preEnd__swept_surface, &ColladaParserAutoGen15Private::_freeAttributes__swept_surface);
mElementToNamespaceMap[HASH_ELEMENT_SWEPT_SURFACE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_AXIS____FLOAT3_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__axis____float3_type, &ColladaParserAutoGen15Private::_data__axis____float3_type, &ColladaParserAutoGen15Private::_end__axis____float3_type, &ColladaParserAutoGen15Private::_preBegin__axis____float3_type, &ColladaParserAutoGen15Private::_preEnd__axis____float3_type, &ColladaParserAutoGen15Private::_freeAttributes__axis____float3_type);
mElementToNamespaceMap[HASH_ELEMENT_AXIS____FLOAT3_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_NURBS_SURFACE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__nurbs_surface, &ColladaParserAutoGen15Private::_data__nurbs_surface, &ColladaParserAutoGen15Private::_end__nurbs_surface, &ColladaParserAutoGen15Private::_preBegin__nurbs_surface, &ColladaParserAutoGen15Private::_preEnd__nurbs_surface, &ColladaParserAutoGen15Private::_freeAttributes__nurbs_surface);
mElementToNamespaceMap[HASH_ELEMENT_NURBS_SURFACE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_NURBS_SURFACE_TYPE____CONTROL_VERTICES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__nurbs_surface_type____control_vertices, &ColladaParserAutoGen15Private::_data__nurbs_surface_type____control_vertices, &ColladaParserAutoGen15Private::_end__nurbs_surface_type____control_vertices, &ColladaParserAutoGen15Private::_preBegin__nurbs_surface_type____control_vertices, &ColladaParserAutoGen15Private::_preEnd__nurbs_surface_type____control_vertices, &ColladaParserAutoGen15Private::_freeAttributes__nurbs_surface_type____control_vertices);
mElementToNamespaceMap[HASH_ELEMENT_NURBS_SURFACE_TYPE____CONTROL_VERTICES] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CONE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__cone, &ColladaParserAutoGen15Private::_data__cone, &ColladaParserAutoGen15Private::_end__cone, &ColladaParserAutoGen15Private::_preBegin__cone, &ColladaParserAutoGen15Private::_preEnd__cone, &ColladaParserAutoGen15Private::_freeAttributes__cone);
mElementToNamespaceMap[HASH_ELEMENT_CONE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ANGLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__angle, &ColladaParserAutoGen15Private::_data__angle, &ColladaParserAutoGen15Private::_end__angle, &ColladaParserAutoGen15Private::_preBegin__angle, &ColladaParserAutoGen15Private::_preEnd__angle, &ColladaParserAutoGen15Private::_freeAttributes__angle);
mElementToNamespaceMap[HASH_ELEMENT_ANGLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SURFACE_TYPE____CYLINDER] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__surface_type____cylinder, &ColladaParserAutoGen15Private::_data__surface_type____cylinder, &ColladaParserAutoGen15Private::_end__surface_type____cylinder, &ColladaParserAutoGen15Private::_preBegin__surface_type____cylinder, &ColladaParserAutoGen15Private::_preEnd__surface_type____cylinder, &ColladaParserAutoGen15Private::_freeAttributes__surface_type____cylinder);
mElementToNamespaceMap[HASH_ELEMENT_SURFACE_TYPE____CYLINDER] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_EDGES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__edges, &ColladaParserAutoGen15Private::_data__edges, &ColladaParserAutoGen15Private::_end__edges, &ColladaParserAutoGen15Private::_preBegin__edges, &ColladaParserAutoGen15Private::_preEnd__edges, &ColladaParserAutoGen15Private::_freeAttributes__edges);
mElementToNamespaceMap[HASH_ELEMENT_EDGES] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_WIRES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__wires, &ColladaParserAutoGen15Private::_data__wires, &ColladaParserAutoGen15Private::_end__wires, &ColladaParserAutoGen15Private::_preBegin__wires, &ColladaParserAutoGen15Private::_preEnd__wires, &ColladaParserAutoGen15Private::_freeAttributes__wires);
mElementToNamespaceMap[HASH_ELEMENT_WIRES] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FACES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__faces, &ColladaParserAutoGen15Private::_data__faces, &ColladaParserAutoGen15Private::_end__faces, &ColladaParserAutoGen15Private::_preBegin__faces, &ColladaParserAutoGen15Private::_preEnd__faces, &ColladaParserAutoGen15Private::_freeAttributes__faces);
mElementToNamespaceMap[HASH_ELEMENT_FACES] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PCURVES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__pcurves, &ColladaParserAutoGen15Private::_data__pcurves, &ColladaParserAutoGen15Private::_end__pcurves, &ColladaParserAutoGen15Private::_preBegin__pcurves, &ColladaParserAutoGen15Private::_preEnd__pcurves, &ColladaParserAutoGen15Private::_freeAttributes__pcurves);
mElementToNamespaceMap[HASH_ELEMENT_PCURVES] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SHELLS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__shells, &ColladaParserAutoGen15Private::_data__shells, &ColladaParserAutoGen15Private::_end__shells, &ColladaParserAutoGen15Private::_preBegin__shells, &ColladaParserAutoGen15Private::_preEnd__shells, &ColladaParserAutoGen15Private::_freeAttributes__shells);
mElementToNamespaceMap[HASH_ELEMENT_SHELLS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SOLIDS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__solids, &ColladaParserAutoGen15Private::_data__solids, &ColladaParserAutoGen15Private::_end__solids, &ColladaParserAutoGen15Private::_preBegin__solids, &ColladaParserAutoGen15Private::_preEnd__solids, &ColladaParserAutoGen15Private::_freeAttributes__solids);
mElementToNamespaceMap[HASH_ELEMENT_SOLIDS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LIBRARY_EFFECTS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_effects, &ColladaParserAutoGen15Private::_data__library_effects, &ColladaParserAutoGen15Private::_end__library_effects, &ColladaParserAutoGen15Private::_preBegin__library_effects, &ColladaParserAutoGen15Private::_preEnd__library_effects, &ColladaParserAutoGen15Private::_freeAttributes__library_effects);
mElementToNamespaceMap[HASH_ELEMENT_LIBRARY_EFFECTS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_EFFECT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__effect, &ColladaParserAutoGen15Private::_data__effect, &ColladaParserAutoGen15Private::_end__effect, &ColladaParserAutoGen15Private::_preBegin__effect, &ColladaParserAutoGen15Private::_preEnd__effect, &ColladaParserAutoGen15Private::_freeAttributes__effect);
mElementToNamespaceMap[HASH_ELEMENT_EFFECT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ANNOTATE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__annotate, &ColladaParserAutoGen15Private::_data__annotate, &ColladaParserAutoGen15Private::_end__annotate, &ColladaParserAutoGen15Private::_preBegin__annotate, &ColladaParserAutoGen15Private::_preEnd__annotate, &ColladaParserAutoGen15Private::_freeAttributes__annotate);
mElementToNamespaceMap[HASH_ELEMENT_ANNOTATE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BOOL2] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool2, &ColladaParserAutoGen15Private::_data__bool2, &ColladaParserAutoGen15Private::_end__bool2, &ColladaParserAutoGen15Private::_preBegin__bool2, &ColladaParserAutoGen15Private::_preEnd__bool2, &ColladaParserAutoGen15Private::_freeAttributes__bool2);
mElementToNamespaceMap[HASH_ELEMENT_BOOL2] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BOOL3] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool3, &ColladaParserAutoGen15Private::_data__bool3, &ColladaParserAutoGen15Private::_end__bool3, &ColladaParserAutoGen15Private::_preBegin__bool3, &ColladaParserAutoGen15Private::_preEnd__bool3, &ColladaParserAutoGen15Private::_freeAttributes__bool3);
mElementToNamespaceMap[HASH_ELEMENT_BOOL3] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BOOL4] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool4, &ColladaParserAutoGen15Private::_data__bool4, &ColladaParserAutoGen15Private::_end__bool4, &ColladaParserAutoGen15Private::_preBegin__bool4, &ColladaParserAutoGen15Private::_preEnd__bool4, &ColladaParserAutoGen15Private::_freeAttributes__bool4);
mElementToNamespaceMap[HASH_ELEMENT_BOOL4] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INT2] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__int2, &ColladaParserAutoGen15Private::_data__int2, &ColladaParserAutoGen15Private::_end__int2, &ColladaParserAutoGen15Private::_preBegin__int2, &ColladaParserAutoGen15Private::_preEnd__int2, &ColladaParserAutoGen15Private::_freeAttributes__int2);
mElementToNamespaceMap[HASH_ELEMENT_INT2] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INT3] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__int3, &ColladaParserAutoGen15Private::_data__int3, &ColladaParserAutoGen15Private::_end__int3, &ColladaParserAutoGen15Private::_preBegin__int3, &ColladaParserAutoGen15Private::_preEnd__int3, &ColladaParserAutoGen15Private::_freeAttributes__int3);
mElementToNamespaceMap[HASH_ELEMENT_INT3] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INT4] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__int4, &ColladaParserAutoGen15Private::_data__int4, &ColladaParserAutoGen15Private::_end__int4, &ColladaParserAutoGen15Private::_preBegin__int4, &ColladaParserAutoGen15Private::_preEnd__int4, &ColladaParserAutoGen15Private::_freeAttributes__int4);
mElementToNamespaceMap[HASH_ELEMENT_INT4] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FLOAT2] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2, &ColladaParserAutoGen15Private::_data__float2, &ColladaParserAutoGen15Private::_end__float2, &ColladaParserAutoGen15Private::_preBegin__float2, &ColladaParserAutoGen15Private::_preEnd__float2, &ColladaParserAutoGen15Private::_freeAttributes__float2);
mElementToNamespaceMap[HASH_ELEMENT_FLOAT2] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FLOAT3] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3, &ColladaParserAutoGen15Private::_data__float3, &ColladaParserAutoGen15Private::_end__float3, &ColladaParserAutoGen15Private::_preBegin__float3, &ColladaParserAutoGen15Private::_preEnd__float3, &ColladaParserAutoGen15Private::_freeAttributes__float3);
mElementToNamespaceMap[HASH_ELEMENT_FLOAT3] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FLOAT4] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4, &ColladaParserAutoGen15Private::_data__float4, &ColladaParserAutoGen15Private::_end__float4, &ColladaParserAutoGen15Private::_preBegin__float4, &ColladaParserAutoGen15Private::_preEnd__float4, &ColladaParserAutoGen15Private::_freeAttributes__float4);
mElementToNamespaceMap[HASH_ELEMENT_FLOAT4] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FLOAT2X2] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2x2, &ColladaParserAutoGen15Private::_data__float2x2, &ColladaParserAutoGen15Private::_end__float2x2, &ColladaParserAutoGen15Private::_preBegin__float2x2, &ColladaParserAutoGen15Private::_preEnd__float2x2, &ColladaParserAutoGen15Private::_freeAttributes__float2x2);
mElementToNamespaceMap[HASH_ELEMENT_FLOAT2X2] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FLOAT3X3] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3x3, &ColladaParserAutoGen15Private::_data__float3x3, &ColladaParserAutoGen15Private::_end__float3x3, &ColladaParserAutoGen15Private::_preBegin__float3x3, &ColladaParserAutoGen15Private::_preEnd__float3x3, &ColladaParserAutoGen15Private::_freeAttributes__float3x3);
mElementToNamespaceMap[HASH_ELEMENT_FLOAT3X3] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FLOAT4X4] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4x4, &ColladaParserAutoGen15Private::_data__float4x4, &ColladaParserAutoGen15Private::_end__float4x4, &ColladaParserAutoGen15Private::_preBegin__float4x4, &ColladaParserAutoGen15Private::_preEnd__float4x4, &ColladaParserAutoGen15Private::_freeAttributes__float4x4);
mElementToNamespaceMap[HASH_ELEMENT_FLOAT4X4] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_STRING] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__string, &ColladaParserAutoGen15Private::_data__string, &ColladaParserAutoGen15Private::_end__string, &ColladaParserAutoGen15Private::_preBegin__string, &ColladaParserAutoGen15Private::_preEnd__string, &ColladaParserAutoGen15Private::_freeAttributes__string);
mElementToNamespaceMap[HASH_ELEMENT_STRING] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_NEWPARAM____FX_NEWPARAM_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__newparam____fx_newparam_type, &ColladaParserAutoGen15Private::_data__newparam____fx_newparam_type, &ColladaParserAutoGen15Private::_end__newparam____fx_newparam_type, &ColladaParserAutoGen15Private::_preBegin__newparam____fx_newparam_type, &ColladaParserAutoGen15Private::_preEnd__newparam____fx_newparam_type, &ColladaParserAutoGen15Private::_freeAttributes__newparam____fx_newparam_type);
mElementToNamespaceMap[HASH_ELEMENT_NEWPARAM____FX_NEWPARAM_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SEMANTIC____NCNAME] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__semantic____NCName, &ColladaParserAutoGen15Private::_data__semantic____NCName, &ColladaParserAutoGen15Private::_end__semantic____NCName, &ColladaParserAutoGen15Private::_preBegin__semantic____NCName, &ColladaParserAutoGen15Private::_preEnd__semantic____NCName, &ColladaParserAutoGen15Private::_freeAttributes__semantic____NCName);
mElementToNamespaceMap[HASH_ELEMENT_SEMANTIC____NCNAME] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_MODIFIER] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__modifier, &ColladaParserAutoGen15Private::_data__modifier, &ColladaParserAutoGen15Private::_end__modifier, &ColladaParserAutoGen15Private::_preBegin__modifier, &ColladaParserAutoGen15Private::_preEnd__modifier, &ColladaParserAutoGen15Private::_freeAttributes__modifier);
mElementToNamespaceMap[HASH_ELEMENT_MODIFIER] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FLOAT2X1] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2x1, &ColladaParserAutoGen15Private::_data__float2x1, &ColladaParserAutoGen15Private::_end__float2x1, &ColladaParserAutoGen15Private::_preBegin__float2x1, &ColladaParserAutoGen15Private::_preEnd__float2x1, &ColladaParserAutoGen15Private::_freeAttributes__float2x1);
mElementToNamespaceMap[HASH_ELEMENT_FLOAT2X1] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FLOAT2X3] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2x3, &ColladaParserAutoGen15Private::_data__float2x3, &ColladaParserAutoGen15Private::_end__float2x3, &ColladaParserAutoGen15Private::_preBegin__float2x3, &ColladaParserAutoGen15Private::_preEnd__float2x3, &ColladaParserAutoGen15Private::_freeAttributes__float2x3);
mElementToNamespaceMap[HASH_ELEMENT_FLOAT2X3] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FLOAT2X4] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2x4, &ColladaParserAutoGen15Private::_data__float2x4, &ColladaParserAutoGen15Private::_end__float2x4, &ColladaParserAutoGen15Private::_preBegin__float2x4, &ColladaParserAutoGen15Private::_preEnd__float2x4, &ColladaParserAutoGen15Private::_freeAttributes__float2x4);
mElementToNamespaceMap[HASH_ELEMENT_FLOAT2X4] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FLOAT3X1] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3x1, &ColladaParserAutoGen15Private::_data__float3x1, &ColladaParserAutoGen15Private::_end__float3x1, &ColladaParserAutoGen15Private::_preBegin__float3x1, &ColladaParserAutoGen15Private::_preEnd__float3x1, &ColladaParserAutoGen15Private::_freeAttributes__float3x1);
mElementToNamespaceMap[HASH_ELEMENT_FLOAT3X1] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FLOAT3X2] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3x2, &ColladaParserAutoGen15Private::_data__float3x2, &ColladaParserAutoGen15Private::_end__float3x2, &ColladaParserAutoGen15Private::_preBegin__float3x2, &ColladaParserAutoGen15Private::_preEnd__float3x2, &ColladaParserAutoGen15Private::_freeAttributes__float3x2);
mElementToNamespaceMap[HASH_ELEMENT_FLOAT3X2] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FLOAT3X4] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3x4, &ColladaParserAutoGen15Private::_data__float3x4, &ColladaParserAutoGen15Private::_end__float3x4, &ColladaParserAutoGen15Private::_preBegin__float3x4, &ColladaParserAutoGen15Private::_preEnd__float3x4, &ColladaParserAutoGen15Private::_freeAttributes__float3x4);
mElementToNamespaceMap[HASH_ELEMENT_FLOAT3X4] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FLOAT4X1] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4x1, &ColladaParserAutoGen15Private::_data__float4x1, &ColladaParserAutoGen15Private::_end__float4x1, &ColladaParserAutoGen15Private::_preBegin__float4x1, &ColladaParserAutoGen15Private::_preEnd__float4x1, &ColladaParserAutoGen15Private::_freeAttributes__float4x1);
mElementToNamespaceMap[HASH_ELEMENT_FLOAT4X1] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FLOAT4X2] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4x2, &ColladaParserAutoGen15Private::_data__float4x2, &ColladaParserAutoGen15Private::_end__float4x2, &ColladaParserAutoGen15Private::_preBegin__float4x2, &ColladaParserAutoGen15Private::_preEnd__float4x2, &ColladaParserAutoGen15Private::_freeAttributes__float4x2);
mElementToNamespaceMap[HASH_ELEMENT_FLOAT4X2] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FLOAT4X3] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4x3, &ColladaParserAutoGen15Private::_data__float4x3, &ColladaParserAutoGen15Private::_end__float4x3, &ColladaParserAutoGen15Private::_preBegin__float4x3, &ColladaParserAutoGen15Private::_preEnd__float4x3, &ColladaParserAutoGen15Private::_freeAttributes__float4x3);
mElementToNamespaceMap[HASH_ELEMENT_FLOAT4X3] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SAMPLER1D] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler1D, &ColladaParserAutoGen15Private::_data__sampler1D, &ColladaParserAutoGen15Private::_end__sampler1D, &ColladaParserAutoGen15Private::_preBegin__sampler1D, &ColladaParserAutoGen15Private::_preEnd__sampler1D, &ColladaParserAutoGen15Private::_freeAttributes__sampler1D);
mElementToNamespaceMap[HASH_ELEMENT_SAMPLER1D] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INSTANCE_IMAGE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_image, &ColladaParserAutoGen15Private::_data__instance_image, &ColladaParserAutoGen15Private::_end__instance_image, &ColladaParserAutoGen15Private::_preBegin__instance_image, &ColladaParserAutoGen15Private::_preEnd__instance_image, &ColladaParserAutoGen15Private::_freeAttributes__instance_image);
mElementToNamespaceMap[HASH_ELEMENT_INSTANCE_IMAGE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_WRAP_S____FX_SAMPLER_WRAP_ENUM] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__wrap_s____fx_sampler_wrap_enum, &ColladaParserAutoGen15Private::_data__wrap_s____fx_sampler_wrap_enum, &ColladaParserAutoGen15Private::_end__wrap_s____fx_sampler_wrap_enum, &ColladaParserAutoGen15Private::_preBegin__wrap_s____fx_sampler_wrap_enum, &ColladaParserAutoGen15Private::_preEnd__wrap_s____fx_sampler_wrap_enum, &ColladaParserAutoGen15Private::_freeAttributes__wrap_s____fx_sampler_wrap_enum);
mElementToNamespaceMap[HASH_ELEMENT_WRAP_S____FX_SAMPLER_WRAP_ENUM] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_WRAP_T____FX_SAMPLER_WRAP_ENUM] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__wrap_t____fx_sampler_wrap_enum, &ColladaParserAutoGen15Private::_data__wrap_t____fx_sampler_wrap_enum, &ColladaParserAutoGen15Private::_end__wrap_t____fx_sampler_wrap_enum, &ColladaParserAutoGen15Private::_preBegin__wrap_t____fx_sampler_wrap_enum, &ColladaParserAutoGen15Private::_preEnd__wrap_t____fx_sampler_wrap_enum, &ColladaParserAutoGen15Private::_freeAttributes__wrap_t____fx_sampler_wrap_enum);
mElementToNamespaceMap[HASH_ELEMENT_WRAP_T____FX_SAMPLER_WRAP_ENUM] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_WRAP_P] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__wrap_p, &ColladaParserAutoGen15Private::_data__wrap_p, &ColladaParserAutoGen15Private::_end__wrap_p, &ColladaParserAutoGen15Private::_preBegin__wrap_p, &ColladaParserAutoGen15Private::_preEnd__wrap_p, &ColladaParserAutoGen15Private::_freeAttributes__wrap_p);
mElementToNamespaceMap[HASH_ELEMENT_WRAP_P] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_MINFILTER] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__minfilter, &ColladaParserAutoGen15Private::_data__minfilter, &ColladaParserAutoGen15Private::_end__minfilter, &ColladaParserAutoGen15Private::_preBegin__minfilter, &ColladaParserAutoGen15Private::_preEnd__minfilter, &ColladaParserAutoGen15Private::_freeAttributes__minfilter);
mElementToNamespaceMap[HASH_ELEMENT_MINFILTER] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_MAGFILTER] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__magfilter, &ColladaParserAutoGen15Private::_data__magfilter, &ColladaParserAutoGen15Private::_end__magfilter, &ColladaParserAutoGen15Private::_preBegin__magfilter, &ColladaParserAutoGen15Private::_preEnd__magfilter, &ColladaParserAutoGen15Private::_freeAttributes__magfilter);
mElementToNamespaceMap[HASH_ELEMENT_MAGFILTER] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_MIPFILTER] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__mipfilter, &ColladaParserAutoGen15Private::_data__mipfilter, &ColladaParserAutoGen15Private::_end__mipfilter, &ColladaParserAutoGen15Private::_preBegin__mipfilter, &ColladaParserAutoGen15Private::_preEnd__mipfilter, &ColladaParserAutoGen15Private::_freeAttributes__mipfilter);
mElementToNamespaceMap[HASH_ELEMENT_MIPFILTER] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BORDER_COLOR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__border_color, &ColladaParserAutoGen15Private::_data__border_color, &ColladaParserAutoGen15Private::_end__border_color, &ColladaParserAutoGen15Private::_preBegin__border_color, &ColladaParserAutoGen15Private::_preEnd__border_color, &ColladaParserAutoGen15Private::_freeAttributes__border_color);
mElementToNamespaceMap[HASH_ELEMENT_BORDER_COLOR] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_MIP_MAX_LEVEL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__mip_max_level, &ColladaParserAutoGen15Private::_data__mip_max_level, &ColladaParserAutoGen15Private::_end__mip_max_level, &ColladaParserAutoGen15Private::_preBegin__mip_max_level, &ColladaParserAutoGen15Private::_preEnd__mip_max_level, &ColladaParserAutoGen15Private::_freeAttributes__mip_max_level);
mElementToNamespaceMap[HASH_ELEMENT_MIP_MAX_LEVEL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_MIP_MIN_LEVEL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__mip_min_level, &ColladaParserAutoGen15Private::_data__mip_min_level, &ColladaParserAutoGen15Private::_end__mip_min_level, &ColladaParserAutoGen15Private::_preBegin__mip_min_level, &ColladaParserAutoGen15Private::_preEnd__mip_min_level, &ColladaParserAutoGen15Private::_freeAttributes__mip_min_level);
mElementToNamespaceMap[HASH_ELEMENT_MIP_MIN_LEVEL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_MIP_BIAS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__mip_bias, &ColladaParserAutoGen15Private::_data__mip_bias, &ColladaParserAutoGen15Private::_end__mip_bias, &ColladaParserAutoGen15Private::_preBegin__mip_bias, &ColladaParserAutoGen15Private::_preEnd__mip_bias, &ColladaParserAutoGen15Private::_freeAttributes__mip_bias);
mElementToNamespaceMap[HASH_ELEMENT_MIP_BIAS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_MAX_ANISOTROPY] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__max_anisotropy, &ColladaParserAutoGen15Private::_data__max_anisotropy, &ColladaParserAutoGen15Private::_end__max_anisotropy, &ColladaParserAutoGen15Private::_preBegin__max_anisotropy, &ColladaParserAutoGen15Private::_preEnd__max_anisotropy, &ColladaParserAutoGen15Private::_freeAttributes__max_anisotropy);
mElementToNamespaceMap[HASH_ELEMENT_MAX_ANISOTROPY] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SAMPLER2D____FX_SAMPLER2D_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_data__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_end__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_preBegin__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_preEnd__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_freeAttributes__sampler2D____fx_sampler2D_type);
mElementToNamespaceMap[HASH_ELEMENT_SAMPLER2D____FX_SAMPLER2D_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SAMPLER3D] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler3D, &ColladaParserAutoGen15Private::_data__sampler3D, &ColladaParserAutoGen15Private::_end__sampler3D, &ColladaParserAutoGen15Private::_preBegin__sampler3D, &ColladaParserAutoGen15Private::_preEnd__sampler3D, &ColladaParserAutoGen15Private::_freeAttributes__sampler3D);
mElementToNamespaceMap[HASH_ELEMENT_SAMPLER3D] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SAMPLERCUBE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__samplerCUBE, &ColladaParserAutoGen15Private::_data__samplerCUBE, &ColladaParserAutoGen15Private::_end__samplerCUBE, &ColladaParserAutoGen15Private::_preBegin__samplerCUBE, &ColladaParserAutoGen15Private::_preEnd__samplerCUBE, &ColladaParserAutoGen15Private::_freeAttributes__samplerCUBE);
mElementToNamespaceMap[HASH_ELEMENT_SAMPLERCUBE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SAMPLERRECT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__samplerRECT, &ColladaParserAutoGen15Private::_data__samplerRECT, &ColladaParserAutoGen15Private::_end__samplerRECT, &ColladaParserAutoGen15Private::_preBegin__samplerRECT, &ColladaParserAutoGen15Private::_preEnd__samplerRECT, &ColladaParserAutoGen15Private::_freeAttributes__samplerRECT);
mElementToNamespaceMap[HASH_ELEMENT_SAMPLERRECT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SAMPLERDEPTH] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__samplerDEPTH, &ColladaParserAutoGen15Private::_data__samplerDEPTH, &ColladaParserAutoGen15Private::_end__samplerDEPTH, &ColladaParserAutoGen15Private::_preBegin__samplerDEPTH, &ColladaParserAutoGen15Private::_preEnd__samplerDEPTH, &ColladaParserAutoGen15Private::_freeAttributes__samplerDEPTH);
mElementToNamespaceMap[HASH_ELEMENT_SAMPLERDEPTH] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ENUM____STRING] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__enum____string, &ColladaParserAutoGen15Private::_data__enum____string, &ColladaParserAutoGen15Private::_end__enum____string, &ColladaParserAutoGen15Private::_preBegin__enum____string, &ColladaParserAutoGen15Private::_preEnd__enum____string, &ColladaParserAutoGen15Private::_freeAttributes__enum____string);
mElementToNamespaceMap[HASH_ELEMENT_ENUM____STRING] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_COMMON] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_COMMON, &ColladaParserAutoGen15Private::_data__profile_COMMON, &ColladaParserAutoGen15Private::_end__profile_COMMON, &ColladaParserAutoGen15Private::_preBegin__profile_COMMON, &ColladaParserAutoGen15Private::_preEnd__profile_COMMON, &ColladaParserAutoGen15Private::_freeAttributes__profile_COMMON);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_COMMON] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_NEWPARAM____FX_COMMON_NEWPARAM_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__newparam____fx_common_newparam_type, &ColladaParserAutoGen15Private::_data__newparam____fx_common_newparam_type, &ColladaParserAutoGen15Private::_end__newparam____fx_common_newparam_type, &ColladaParserAutoGen15Private::_preBegin__newparam____fx_common_newparam_type, &ColladaParserAutoGen15Private::_preEnd__newparam____fx_common_newparam_type, &ColladaParserAutoGen15Private::_freeAttributes__newparam____fx_common_newparam_type);
mElementToNamespaceMap[HASH_ELEMENT_NEWPARAM____FX_COMMON_NEWPARAM_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_COMMON_TYPE____TECHNIQUE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_common_type____technique, &ColladaParserAutoGen15Private::_data__profile_common_type____technique, &ColladaParserAutoGen15Private::_end__profile_common_type____technique, &ColladaParserAutoGen15Private::_preBegin__profile_common_type____technique, &ColladaParserAutoGen15Private::_preEnd__profile_common_type____technique, &ColladaParserAutoGen15Private::_freeAttributes__profile_common_type____technique);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_COMMON_TYPE____TECHNIQUE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_COMMON__TECHNIQUE__CONSTANT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_COMMON__technique__constant, &ColladaParserAutoGen15Private::_data__profile_COMMON__technique__constant, &ColladaParserAutoGen15Private::_end__profile_COMMON__technique__constant, &ColladaParserAutoGen15Private::_preBegin__profile_COMMON__technique__constant, &ColladaParserAutoGen15Private::_preEnd__profile_COMMON__technique__constant, &ColladaParserAutoGen15Private::_freeAttributes__profile_COMMON__technique__constant);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_COMMON__TECHNIQUE__CONSTANT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_EMISSION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__emission, &ColladaParserAutoGen15Private::_data__emission, &ColladaParserAutoGen15Private::_end__emission, &ColladaParserAutoGen15Private::_preBegin__emission, &ColladaParserAutoGen15Private::_preEnd__emission, &ColladaParserAutoGen15Private::_freeAttributes__emission);
mElementToNamespaceMap[HASH_ELEMENT_EMISSION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FX_COMMON_COLOR_OR_TEXTURE_TYPE____COLOR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__fx_common_color_or_texture_type____color, &ColladaParserAutoGen15Private::_data__fx_common_color_or_texture_type____color, &ColladaParserAutoGen15Private::_end__fx_common_color_or_texture_type____color, &ColladaParserAutoGen15Private::_preBegin__fx_common_color_or_texture_type____color, &ColladaParserAutoGen15Private::_preEnd__fx_common_color_or_texture_type____color, &ColladaParserAutoGen15Private::_freeAttributes__fx_common_color_or_texture_type____color);
mElementToNamespaceMap[HASH_ELEMENT_FX_COMMON_COLOR_OR_TEXTURE_TYPE____COLOR] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FX_COMMON_COLOR_OR_TEXTURE_TYPE____PARAM] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__fx_common_color_or_texture_type____param, &ColladaParserAutoGen15Private::_data__fx_common_color_or_texture_type____param, &ColladaParserAutoGen15Private::_end__fx_common_color_or_texture_type____param, &ColladaParserAutoGen15Private::_preBegin__fx_common_color_or_texture_type____param, &ColladaParserAutoGen15Private::_preEnd__fx_common_color_or_texture_type____param, &ColladaParserAutoGen15Private::_freeAttributes__fx_common_color_or_texture_type____param);
mElementToNamespaceMap[HASH_ELEMENT_FX_COMMON_COLOR_OR_TEXTURE_TYPE____PARAM] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TEXTURE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__texture, &ColladaParserAutoGen15Private::_data__texture, &ColladaParserAutoGen15Private::_end__texture, &ColladaParserAutoGen15Private::_preBegin__texture, &ColladaParserAutoGen15Private::_preEnd__texture, &ColladaParserAutoGen15Private::_freeAttributes__texture);
mElementToNamespaceMap[HASH_ELEMENT_TEXTURE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_REFLECTIVE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__reflective, &ColladaParserAutoGen15Private::_data__reflective, &ColladaParserAutoGen15Private::_end__reflective, &ColladaParserAutoGen15Private::_preBegin__reflective, &ColladaParserAutoGen15Private::_preEnd__reflective, &ColladaParserAutoGen15Private::_freeAttributes__reflective);
mElementToNamespaceMap[HASH_ELEMENT_REFLECTIVE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_REFLECTIVITY] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__reflectivity, &ColladaParserAutoGen15Private::_data__reflectivity, &ColladaParserAutoGen15Private::_end__reflectivity, &ColladaParserAutoGen15Private::_preBegin__reflectivity, &ColladaParserAutoGen15Private::_preEnd__reflectivity, &ColladaParserAutoGen15Private::_freeAttributes__reflectivity);
mElementToNamespaceMap[HASH_ELEMENT_REFLECTIVITY] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FX_COMMON_FLOAT_OR_PARAM_TYPE____FLOAT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__fx_common_float_or_param_type____float, &ColladaParserAutoGen15Private::_data__fx_common_float_or_param_type____float, &ColladaParserAutoGen15Private::_end__fx_common_float_or_param_type____float, &ColladaParserAutoGen15Private::_preBegin__fx_common_float_or_param_type____float, &ColladaParserAutoGen15Private::_preEnd__fx_common_float_or_param_type____float, &ColladaParserAutoGen15Private::_freeAttributes__fx_common_float_or_param_type____float);
mElementToNamespaceMap[HASH_ELEMENT_FX_COMMON_FLOAT_OR_PARAM_TYPE____FLOAT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FX_COMMON_FLOAT_OR_PARAM_TYPE____PARAM] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__fx_common_float_or_param_type____param, &ColladaParserAutoGen15Private::_data__fx_common_float_or_param_type____param, &ColladaParserAutoGen15Private::_end__fx_common_float_or_param_type____param, &ColladaParserAutoGen15Private::_preBegin__fx_common_float_or_param_type____param, &ColladaParserAutoGen15Private::_preEnd__fx_common_float_or_param_type____param, &ColladaParserAutoGen15Private::_freeAttributes__fx_common_float_or_param_type____param);
mElementToNamespaceMap[HASH_ELEMENT_FX_COMMON_FLOAT_OR_PARAM_TYPE____PARAM] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TRANSPARENT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__transparent, &ColladaParserAutoGen15Private::_data__transparent, &ColladaParserAutoGen15Private::_end__transparent, &ColladaParserAutoGen15Private::_preBegin__transparent, &ColladaParserAutoGen15Private::_preEnd__transparent, &ColladaParserAutoGen15Private::_freeAttributes__transparent);
mElementToNamespaceMap[HASH_ELEMENT_TRANSPARENT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TRANSPARENCY] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__transparency, &ColladaParserAutoGen15Private::_data__transparency, &ColladaParserAutoGen15Private::_end__transparency, &ColladaParserAutoGen15Private::_preBegin__transparency, &ColladaParserAutoGen15Private::_preEnd__transparency, &ColladaParserAutoGen15Private::_freeAttributes__transparency);
mElementToNamespaceMap[HASH_ELEMENT_TRANSPARENCY] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INDEX_OF_REFRACTION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__index_of_refraction, &ColladaParserAutoGen15Private::_data__index_of_refraction, &ColladaParserAutoGen15Private::_end__index_of_refraction, &ColladaParserAutoGen15Private::_preBegin__index_of_refraction, &ColladaParserAutoGen15Private::_preEnd__index_of_refraction, &ColladaParserAutoGen15Private::_freeAttributes__index_of_refraction);
mElementToNamespaceMap[HASH_ELEMENT_INDEX_OF_REFRACTION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LAMBERT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__lambert, &ColladaParserAutoGen15Private::_data__lambert, &ColladaParserAutoGen15Private::_end__lambert, &ColladaParserAutoGen15Private::_preBegin__lambert, &ColladaParserAutoGen15Private::_preEnd__lambert, &ColladaParserAutoGen15Private::_freeAttributes__lambert);
mElementToNamespaceMap[HASH_ELEMENT_LAMBERT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_AMBIENT____FX_COMMON_COLOR_OR_TEXTURE_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__ambient____fx_common_color_or_texture_type, &ColladaParserAutoGen15Private::_data__ambient____fx_common_color_or_texture_type, &ColladaParserAutoGen15Private::_end__ambient____fx_common_color_or_texture_type, &ColladaParserAutoGen15Private::_preBegin__ambient____fx_common_color_or_texture_type, &ColladaParserAutoGen15Private::_preEnd__ambient____fx_common_color_or_texture_type, &ColladaParserAutoGen15Private::_freeAttributes__ambient____fx_common_color_or_texture_type);
mElementToNamespaceMap[HASH_ELEMENT_AMBIENT____FX_COMMON_COLOR_OR_TEXTURE_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_DIFFUSE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__diffuse, &ColladaParserAutoGen15Private::_data__diffuse, &ColladaParserAutoGen15Private::_end__diffuse, &ColladaParserAutoGen15Private::_preBegin__diffuse, &ColladaParserAutoGen15Private::_preEnd__diffuse, &ColladaParserAutoGen15Private::_freeAttributes__diffuse);
mElementToNamespaceMap[HASH_ELEMENT_DIFFUSE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PHONG] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__phong, &ColladaParserAutoGen15Private::_data__phong, &ColladaParserAutoGen15Private::_end__phong, &ColladaParserAutoGen15Private::_preBegin__phong, &ColladaParserAutoGen15Private::_preEnd__phong, &ColladaParserAutoGen15Private::_freeAttributes__phong);
mElementToNamespaceMap[HASH_ELEMENT_PHONG] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SPECULAR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__specular, &ColladaParserAutoGen15Private::_data__specular, &ColladaParserAutoGen15Private::_end__specular, &ColladaParserAutoGen15Private::_preBegin__specular, &ColladaParserAutoGen15Private::_preEnd__specular, &ColladaParserAutoGen15Private::_freeAttributes__specular);
mElementToNamespaceMap[HASH_ELEMENT_SPECULAR] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SHININESS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__shininess, &ColladaParserAutoGen15Private::_data__shininess, &ColladaParserAutoGen15Private::_end__shininess, &ColladaParserAutoGen15Private::_preBegin__shininess, &ColladaParserAutoGen15Private::_preEnd__shininess, &ColladaParserAutoGen15Private::_freeAttributes__shininess);
mElementToNamespaceMap[HASH_ELEMENT_SHININESS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BLINN] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__blinn, &ColladaParserAutoGen15Private::_data__blinn, &ColladaParserAutoGen15Private::_end__blinn, &ColladaParserAutoGen15Private::_preBegin__blinn, &ColladaParserAutoGen15Private::_preEnd__blinn, &ColladaParserAutoGen15Private::_freeAttributes__blinn);
mElementToNamespaceMap[HASH_ELEMENT_BLINN] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_BRIDGE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_BRIDGE, &ColladaParserAutoGen15Private::_data__profile_BRIDGE, &ColladaParserAutoGen15Private::_end__profile_BRIDGE, &ColladaParserAutoGen15Private::_preBegin__profile_BRIDGE, &ColladaParserAutoGen15Private::_preEnd__profile_BRIDGE, &ColladaParserAutoGen15Private::_freeAttributes__profile_BRIDGE);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_BRIDGE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2, &ColladaParserAutoGen15Private::_data__profile_GLES2, &ColladaParserAutoGen15Private::_end__profile_GLES2, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INCLUDE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__include, &ColladaParserAutoGen15Private::_data__include, &ColladaParserAutoGen15Private::_end__include, &ColladaParserAutoGen15Private::_preBegin__include, &ColladaParserAutoGen15Private::_preEnd__include, &ColladaParserAutoGen15Private::_freeAttributes__include);
mElementToNamespaceMap[HASH_ELEMENT_INCLUDE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CODE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__code, &ColladaParserAutoGen15Private::_data__code, &ColladaParserAutoGen15Private::_end__code, &ColladaParserAutoGen15Private::_preBegin__code, &ColladaParserAutoGen15Private::_preEnd__code, &ColladaParserAutoGen15Private::_freeAttributes__code);
mElementToNamespaceMap[HASH_ELEMENT_CODE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2_TYPE____NEWPARAM] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_gles2_type____newparam, &ColladaParserAutoGen15Private::_data__profile_gles2_type____newparam, &ColladaParserAutoGen15Private::_end__profile_gles2_type____newparam, &ColladaParserAutoGen15Private::_preBegin__profile_gles2_type____newparam, &ColladaParserAutoGen15Private::_preEnd__profile_gles2_type____newparam, &ColladaParserAutoGen15Private::_freeAttributes__profile_gles2_type____newparam);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2_TYPE____NEWPARAM] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BVEC2] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__bvec2, &ColladaParserAutoGen15Private::_data__bvec2, &ColladaParserAutoGen15Private::_end__bvec2, &ColladaParserAutoGen15Private::_preBegin__bvec2, &ColladaParserAutoGen15Private::_preEnd__bvec2, &ColladaParserAutoGen15Private::_freeAttributes__bvec2);
mElementToNamespaceMap[HASH_ELEMENT_BVEC2] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BVEC3] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__bvec3, &ColladaParserAutoGen15Private::_data__bvec3, &ColladaParserAutoGen15Private::_end__bvec3, &ColladaParserAutoGen15Private::_preBegin__bvec3, &ColladaParserAutoGen15Private::_preEnd__bvec3, &ColladaParserAutoGen15Private::_freeAttributes__bvec3);
mElementToNamespaceMap[HASH_ELEMENT_BVEC3] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BVEC4] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__bvec4, &ColladaParserAutoGen15Private::_data__bvec4, &ColladaParserAutoGen15Private::_end__bvec4, &ColladaParserAutoGen15Private::_preBegin__bvec4, &ColladaParserAutoGen15Private::_preEnd__bvec4, &ColladaParserAutoGen15Private::_freeAttributes__bvec4);
mElementToNamespaceMap[HASH_ELEMENT_BVEC4] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_VEC2] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__vec2, &ColladaParserAutoGen15Private::_data__vec2, &ColladaParserAutoGen15Private::_end__vec2, &ColladaParserAutoGen15Private::_preBegin__vec2, &ColladaParserAutoGen15Private::_preEnd__vec2, &ColladaParserAutoGen15Private::_freeAttributes__vec2);
mElementToNamespaceMap[HASH_ELEMENT_VEC2] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_VEC3] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__vec3, &ColladaParserAutoGen15Private::_data__vec3, &ColladaParserAutoGen15Private::_end__vec3, &ColladaParserAutoGen15Private::_preBegin__vec3, &ColladaParserAutoGen15Private::_preEnd__vec3, &ColladaParserAutoGen15Private::_freeAttributes__vec3);
mElementToNamespaceMap[HASH_ELEMENT_VEC3] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_VEC4] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__vec4, &ColladaParserAutoGen15Private::_data__vec4, &ColladaParserAutoGen15Private::_end__vec4, &ColladaParserAutoGen15Private::_preBegin__vec4, &ColladaParserAutoGen15Private::_preEnd__vec4, &ColladaParserAutoGen15Private::_freeAttributes__vec4);
mElementToNamespaceMap[HASH_ELEMENT_VEC4] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_MAT2] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__mat2, &ColladaParserAutoGen15Private::_data__mat2, &ColladaParserAutoGen15Private::_end__mat2, &ColladaParserAutoGen15Private::_preBegin__mat2, &ColladaParserAutoGen15Private::_preEnd__mat2, &ColladaParserAutoGen15Private::_freeAttributes__mat2);
mElementToNamespaceMap[HASH_ELEMENT_MAT2] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_MAT3] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__mat3, &ColladaParserAutoGen15Private::_data__mat3, &ColladaParserAutoGen15Private::_end__mat3, &ColladaParserAutoGen15Private::_preBegin__mat3, &ColladaParserAutoGen15Private::_preEnd__mat3, &ColladaParserAutoGen15Private::_freeAttributes__mat3);
mElementToNamespaceMap[HASH_ELEMENT_MAT3] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_MAT4] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__mat4, &ColladaParserAutoGen15Private::_data__mat4, &ColladaParserAutoGen15Private::_end__mat4, &ColladaParserAutoGen15Private::_preBegin__mat4, &ColladaParserAutoGen15Private::_preEnd__mat4, &ColladaParserAutoGen15Private::_freeAttributes__mat4);
mElementToNamespaceMap[HASH_ELEMENT_MAT4] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_IVEC2] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__ivec2, &ColladaParserAutoGen15Private::_data__ivec2, &ColladaParserAutoGen15Private::_end__ivec2, &ColladaParserAutoGen15Private::_preBegin__ivec2, &ColladaParserAutoGen15Private::_preEnd__ivec2, &ColladaParserAutoGen15Private::_freeAttributes__ivec2);
mElementToNamespaceMap[HASH_ELEMENT_IVEC2] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_IVEC3] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__ivec3, &ColladaParserAutoGen15Private::_data__ivec3, &ColladaParserAutoGen15Private::_end__ivec3, &ColladaParserAutoGen15Private::_preBegin__ivec3, &ColladaParserAutoGen15Private::_preEnd__ivec3, &ColladaParserAutoGen15Private::_freeAttributes__ivec3);
mElementToNamespaceMap[HASH_ELEMENT_IVEC3] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_IVEC4] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__ivec4, &ColladaParserAutoGen15Private::_data__ivec4, &ColladaParserAutoGen15Private::_end__ivec4, &ColladaParserAutoGen15Private::_preBegin__ivec4, &ColladaParserAutoGen15Private::_preEnd__ivec4, &ColladaParserAutoGen15Private::_freeAttributes__ivec4);
mElementToNamespaceMap[HASH_ELEMENT_IVEC4] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ENUM____GL_ENUMERATION_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_data__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_end__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_preBegin__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_preEnd__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_freeAttributes__enum____gl_enumeration_type);
mElementToNamespaceMap[HASH_ELEMENT_ENUM____GL_ENUMERATION_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__NEWPARAM__USERTYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__newparam__usertype, &ColladaParserAutoGen15Private::_data__profile_GLES2__newparam__usertype, &ColladaParserAutoGen15Private::_end__profile_GLES2__newparam__usertype, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__newparam__usertype, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__newparam__usertype, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__newparam__usertype);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__NEWPARAM__USERTYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__NEWPARAM__USERTYPE__SETPARAM] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__newparam__usertype__setparam, &ColladaParserAutoGen15Private::_data__profile_GLES2__newparam__usertype__setparam, &ColladaParserAutoGen15Private::_end__profile_GLES2__newparam__usertype__setparam, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__newparam__usertype__setparam, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__newparam__usertype__setparam, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__newparam__usertype__setparam);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__NEWPARAM__USERTYPE__SETPARAM] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__NEWPARAM__USERTYPE__SETPARAM__ARRAY] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__newparam__usertype__setparam__array, &ColladaParserAutoGen15Private::_data__profile_GLES2__newparam__usertype__setparam__array, &ColladaParserAutoGen15Private::_end__profile_GLES2__newparam__usertype__setparam__array, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__newparam__usertype__setparam__array, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__newparam__usertype__setparam__array, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__newparam__usertype__setparam__array);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__NEWPARAM__USERTYPE__SETPARAM__ARRAY] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2_TYPE____TECHNIQUE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_gles2_type____technique, &ColladaParserAutoGen15Private::_data__profile_gles2_type____technique, &ColladaParserAutoGen15Private::_end__profile_gles2_type____technique, &ColladaParserAutoGen15Private::_preBegin__profile_gles2_type____technique, &ColladaParserAutoGen15Private::_preEnd__profile_gles2_type____technique, &ColladaParserAutoGen15Private::_freeAttributes__profile_gles2_type____technique);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2_TYPE____TECHNIQUE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PASS____GLES2_PASS_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__pass____gles2_pass_type, &ColladaParserAutoGen15Private::_data__pass____gles2_pass_type, &ColladaParserAutoGen15Private::_end__pass____gles2_pass_type, &ColladaParserAutoGen15Private::_preBegin__pass____gles2_pass_type, &ColladaParserAutoGen15Private::_preEnd__pass____gles2_pass_type, &ColladaParserAutoGen15Private::_freeAttributes__pass____gles2_pass_type);
mElementToNamespaceMap[HASH_ELEMENT_PASS____GLES2_PASS_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_GLES2_PASS_TYPE____STATES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__gles2_pass_type____states, &ColladaParserAutoGen15Private::_data__gles2_pass_type____states, &ColladaParserAutoGen15Private::_end__gles2_pass_type____states, &ColladaParserAutoGen15Private::_preBegin__gles2_pass_type____states, &ColladaParserAutoGen15Private::_preEnd__gles2_pass_type____states, &ColladaParserAutoGen15Private::_freeAttributes__gles2_pass_type____states);
mElementToNamespaceMap[HASH_ELEMENT_GLES2_PASS_TYPE____STATES] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_COLOR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__blend_color, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__blend_color, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__blend_color, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__blend_color, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__blend_color, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__blend_color);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_COLOR] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_EQUATION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__blend_equation, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__blend_equation, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__blend_equation, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__blend_equation, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__blend_equation, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__blend_equation);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_EQUATION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_EQUATION_SEPARATE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__blend_equation_separate, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__blend_equation_separate, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__blend_equation_separate, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__blend_equation_separate, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__blend_equation_separate, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__blend_equation_separate);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_EQUATION_SEPARATE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_EQUATION_SEPARATE__RGB] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__blend_equation_separate__rgb, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__blend_equation_separate__rgb, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__blend_equation_separate__rgb, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__blend_equation_separate__rgb, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__blend_equation_separate__rgb, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__blend_equation_separate__rgb);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_EQUATION_SEPARATE__RGB] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_EQUATION_SEPARATE__ALPHA] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__blend_equation_separate__alpha, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__blend_equation_separate__alpha, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__blend_equation_separate__alpha, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__blend_equation_separate__alpha, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__blend_equation_separate__alpha, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__blend_equation_separate__alpha);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_EQUATION_SEPARATE__ALPHA] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_FUNC] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__blend_func, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__blend_func, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__blend_func, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__blend_func, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__blend_func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__blend_func);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_FUNC] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_FUNC__SRC] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__blend_func__src, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__blend_func__src, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__blend_func__src, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__blend_func__src, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__blend_func__src, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__blend_func__src);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_FUNC__SRC] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_FUNC__DEST] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__blend_func__dest, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__blend_func__dest, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__blend_func__dest, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__blend_func__dest, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__blend_func__dest, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__blend_func__dest);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_FUNC__DEST] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_FUNC_SEPARATE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__blend_func_separate, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__blend_func_separate, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__blend_func_separate, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__blend_func_separate, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__blend_func_separate, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__blend_func_separate);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_FUNC_SEPARATE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_FUNC_SEPARATE__SRC_RGB] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__blend_func_separate__src_rgb, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__blend_func_separate__src_rgb, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__blend_func_separate__src_rgb, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__blend_func_separate__src_rgb, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__blend_func_separate__src_rgb, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__blend_func_separate__src_rgb);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_FUNC_SEPARATE__SRC_RGB] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_FUNC_SEPARATE__DEST_RGB] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__blend_func_separate__dest_rgb, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__blend_func_separate__dest_rgb, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__blend_func_separate__dest_rgb, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__blend_func_separate__dest_rgb, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__blend_func_separate__dest_rgb, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__blend_func_separate__dest_rgb);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_FUNC_SEPARATE__DEST_RGB] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_FUNC_SEPARATE__SRC_ALPHA] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__blend_func_separate__src_alpha, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__blend_func_separate__src_alpha, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__blend_func_separate__src_alpha, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__blend_func_separate__src_alpha, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__blend_func_separate__src_alpha, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__blend_func_separate__src_alpha);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_FUNC_SEPARATE__SRC_ALPHA] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_FUNC_SEPARATE__DEST_ALPHA] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__blend_func_separate__dest_alpha, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__blend_func_separate__dest_alpha, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__blend_func_separate__dest_alpha, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__blend_func_separate__dest_alpha, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__blend_func_separate__dest_alpha, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__blend_func_separate__dest_alpha);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_FUNC_SEPARATE__DEST_ALPHA] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__COLOR_MASK] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__color_mask, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__color_mask, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__color_mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__color_mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__color_mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__color_mask);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__COLOR_MASK] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__CULL_FACE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__cull_face, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__cull_face, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__cull_face, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__cull_face, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__cull_face, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__cull_face);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__CULL_FACE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__DEPTH_FUNC] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__depth_func, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__depth_func, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__depth_func, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__depth_func, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__depth_func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__depth_func);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__DEPTH_FUNC] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__DEPTH_MASK] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__depth_mask, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__depth_mask, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__depth_mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__depth_mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__depth_mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__depth_mask);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__DEPTH_MASK] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__DEPTH_RANGE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__depth_range, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__depth_range, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__depth_range, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__depth_range, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__depth_range, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__depth_range);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__DEPTH_RANGE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__FRONT_FACE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__front_face, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__front_face, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__front_face, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__front_face, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__front_face, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__front_face);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__FRONT_FACE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__LINE_WIDTH] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__line_width, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__line_width, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__line_width, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__line_width, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__line_width, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__line_width);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__LINE_WIDTH] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__POLYGON_OFFSET] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__polygon_offset, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__polygon_offset, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__polygon_offset, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__polygon_offset, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__polygon_offset, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__polygon_offset);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__POLYGON_OFFSET] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__POINT_SIZE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__point_size, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__point_size, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__point_size, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__point_size, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__point_size, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__point_size);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__POINT_SIZE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SAMPLE_COVERAGE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__sample_coverage, &ColladaParserAutoGen15Private::_data__sample_coverage, &ColladaParserAutoGen15Private::_end__sample_coverage, &ColladaParserAutoGen15Private::_preBegin__sample_coverage, &ColladaParserAutoGen15Private::_preEnd__sample_coverage, &ColladaParserAutoGen15Private::_freeAttributes__sample_coverage);
mElementToNamespaceMap[HASH_ELEMENT_SAMPLE_COVERAGE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SAMPLE_COVERAGE__VALUE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__sample_coverage__value, &ColladaParserAutoGen15Private::_data__sample_coverage__value, &ColladaParserAutoGen15Private::_end__sample_coverage__value, &ColladaParserAutoGen15Private::_preBegin__sample_coverage__value, &ColladaParserAutoGen15Private::_preEnd__sample_coverage__value, &ColladaParserAutoGen15Private::_freeAttributes__sample_coverage__value);
mElementToNamespaceMap[HASH_ELEMENT_SAMPLE_COVERAGE__VALUE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INVERT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__invert, &ColladaParserAutoGen15Private::_data__invert, &ColladaParserAutoGen15Private::_end__invert, &ColladaParserAutoGen15Private::_preBegin__invert, &ColladaParserAutoGen15Private::_preEnd__invert, &ColladaParserAutoGen15Private::_freeAttributes__invert);
mElementToNamespaceMap[HASH_ELEMENT_INVERT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__SCISSOR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__scissor, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__scissor, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__scissor, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__scissor, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__scissor, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__scissor);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__SCISSOR] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_FUNC] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_func, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_func, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_func, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_func, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_func);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_FUNC] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_FUNC__FUNC] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_func__func, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_func__func, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_func__func, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_func__func, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_func__func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_func__func);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_FUNC__FUNC] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_FUNC__REF] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_func__ref, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_func__ref, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_func__ref, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_func__ref, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_func__ref, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_func__ref);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_FUNC__REF] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_FUNC__MASK] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_func__mask, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_func__mask, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_func__mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_func__mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_func__mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_func__mask);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_FUNC__MASK] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_FUNC_SEPARATE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_func_separate, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_func_separate, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_func_separate, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_func_separate, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_func_separate, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_func_separate);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_FUNC_SEPARATE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_FUNC_SEPARATE__FRONT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_func_separate__front, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_func_separate__front, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_func_separate__front, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_func_separate__front, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_func_separate__front, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_func_separate__front);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_FUNC_SEPARATE__FRONT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_FUNC_SEPARATE__BACK] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_func_separate__back, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_func_separate__back, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_func_separate__back, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_func_separate__back, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_func_separate__back, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_func_separate__back);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_FUNC_SEPARATE__BACK] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_FUNC_SEPARATE__REF] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_func_separate__ref, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_func_separate__ref, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_func_separate__ref, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_func_separate__ref, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_func_separate__ref, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_func_separate__ref);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_FUNC_SEPARATE__REF] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_FUNC_SEPARATE__MASK] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_func_separate__mask, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_func_separate__mask, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_func_separate__mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_func_separate__mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_func_separate__mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_func_separate__mask);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_FUNC_SEPARATE__MASK] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_MASK] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_mask, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_mask, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_mask);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_MASK] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_MASK_SEPARATE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_mask_separate, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_mask_separate, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_mask_separate, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_mask_separate, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_mask_separate, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_mask_separate);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_MASK_SEPARATE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_MASK_SEPARATE__FACE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_mask_separate__face, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_mask_separate__face, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_mask_separate__face, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_mask_separate__face, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_mask_separate__face, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_mask_separate__face);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_MASK_SEPARATE__FACE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_MASK_SEPARATE__MASK] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_mask_separate__mask, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_mask_separate__mask, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_mask_separate__mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_mask_separate__mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_mask_separate__mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_mask_separate__mask);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_MASK_SEPARATE__MASK] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_OP] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_op, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_op, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_op, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_op, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_op, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_op);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_OP] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_OP__FAIL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_op__fail, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_op__fail, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_op__fail, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_op__fail, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_op__fail, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_op__fail);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_OP__FAIL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_OP__ZFAIL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_op__zfail, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_op__zfail, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_op__zfail, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_op__zfail, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_op__zfail, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_op__zfail);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_OP__ZFAIL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_OP__ZPASS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_op__zpass, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_op__zpass, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_op__zpass, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_op__zpass, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_op__zpass, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_op__zpass);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_OP__ZPASS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_OP_SEPARATE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_op_separate, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_op_separate, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_op_separate, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_op_separate, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_op_separate, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_op_separate);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_OP_SEPARATE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_OP_SEPARATE__FACE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_op_separate__face, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_op_separate__face, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_op_separate__face, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_op_separate__face, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_op_separate__face, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_op_separate__face);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_OP_SEPARATE__FACE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_OP_SEPARATE__FAIL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_op_separate__fail, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_op_separate__fail, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_op_separate__fail, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_op_separate__fail, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_op_separate__fail, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_op_separate__fail);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_OP_SEPARATE__FAIL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_OP_SEPARATE__ZFAIL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_op_separate__zfail, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_op_separate__zfail, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_op_separate__zfail, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_op_separate__zfail, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_op_separate__zfail, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_op_separate__zfail);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_OP_SEPARATE__ZFAIL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_OP_SEPARATE__ZPASS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_op_separate__zpass, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_op_separate__zpass, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_op_separate__zpass, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_op_separate__zpass, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_op_separate__zpass, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_op_separate__zpass);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_OP_SEPARATE__ZPASS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__blend_enable, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__blend_enable, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__blend_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__blend_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__blend_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__blend_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__CULL_FACE_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__cull_face_enable, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__cull_face_enable, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__cull_face_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__cull_face_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__cull_face_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__cull_face_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__CULL_FACE_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__DEPTH_TEST_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__depth_test_enable, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__depth_test_enable, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__depth_test_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__depth_test_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__depth_test_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__depth_test_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__DEPTH_TEST_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__DITHER_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__dither_enable, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__dither_enable, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__dither_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__dither_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__dither_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__dither_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__DITHER_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__POLYGON_OFFSET_FILL_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__polygon_offset_fill_enable, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__polygon_offset_fill_enable, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__polygon_offset_fill_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__polygon_offset_fill_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__polygon_offset_fill_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__polygon_offset_fill_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__POLYGON_OFFSET_FILL_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_POINT_SIZE_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__point_size_enable, &ColladaParserAutoGen15Private::_data__point_size_enable, &ColladaParserAutoGen15Private::_end__point_size_enable, &ColladaParserAutoGen15Private::_preBegin__point_size_enable, &ColladaParserAutoGen15Private::_preEnd__point_size_enable, &ColladaParserAutoGen15Private::_freeAttributes__point_size_enable);
mElementToNamespaceMap[HASH_ELEMENT_POINT_SIZE_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__SAMPLE_ALPHA_TO_COVERAGE_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__sample_alpha_to_coverage_enable, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__sample_alpha_to_coverage_enable, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__sample_alpha_to_coverage_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__sample_alpha_to_coverage_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__sample_alpha_to_coverage_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__sample_alpha_to_coverage_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__SAMPLE_ALPHA_TO_COVERAGE_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__SAMPLE_COVERAGE_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__sample_coverage_enable, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__sample_coverage_enable, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__sample_coverage_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__sample_coverage_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__sample_coverage_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__sample_coverage_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__SAMPLE_COVERAGE_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__SCISSOR_TEST_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__scissor_test_enable, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__scissor_test_enable, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__scissor_test_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__scissor_test_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__scissor_test_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__scissor_test_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__SCISSOR_TEST_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_TEST_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_test_enable, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_test_enable, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_test_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_test_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_test_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_test_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_TEST_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROGRAM____GLES2_PROGRAM_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__program____gles2_program_type, &ColladaParserAutoGen15Private::_data__program____gles2_program_type, &ColladaParserAutoGen15Private::_end__program____gles2_program_type, &ColladaParserAutoGen15Private::_preBegin__program____gles2_program_type, &ColladaParserAutoGen15Private::_preEnd__program____gles2_program_type, &ColladaParserAutoGen15Private::_freeAttributes__program____gles2_program_type);
mElementToNamespaceMap[HASH_ELEMENT_PROGRAM____GLES2_PROGRAM_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SHADER____GLES2_SHADER_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__shader____gles2_shader_type, &ColladaParserAutoGen15Private::_data__shader____gles2_shader_type, &ColladaParserAutoGen15Private::_end__shader____gles2_shader_type, &ColladaParserAutoGen15Private::_preBegin__shader____gles2_shader_type, &ColladaParserAutoGen15Private::_preEnd__shader____gles2_shader_type, &ColladaParserAutoGen15Private::_freeAttributes__shader____gles2_shader_type);
mElementToNamespaceMap[HASH_ELEMENT_SHADER____GLES2_SHADER_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_GLES2_SHADER_TYPE____SOURCES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__gles2_shader_type____sources, &ColladaParserAutoGen15Private::_data__gles2_shader_type____sources, &ColladaParserAutoGen15Private::_end__gles2_shader_type____sources, &ColladaParserAutoGen15Private::_preBegin__gles2_shader_type____sources, &ColladaParserAutoGen15Private::_preEnd__gles2_shader_type____sources, &ColladaParserAutoGen15Private::_freeAttributes__gles2_shader_type____sources);
mElementToNamespaceMap[HASH_ELEMENT_GLES2_SHADER_TYPE____SOURCES] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INLINE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__inline, &ColladaParserAutoGen15Private::_data__inline, &ColladaParserAutoGen15Private::_end__inline, &ColladaParserAutoGen15Private::_preBegin__inline, &ColladaParserAutoGen15Private::_preEnd__inline, &ColladaParserAutoGen15Private::_freeAttributes__inline);
mElementToNamespaceMap[HASH_ELEMENT_INLINE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_IMPORT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__import, &ColladaParserAutoGen15Private::_data__import, &ColladaParserAutoGen15Private::_end__import, &ColladaParserAutoGen15Private::_preBegin__import, &ColladaParserAutoGen15Private::_preEnd__import, &ColladaParserAutoGen15Private::_freeAttributes__import);
mElementToNamespaceMap[HASH_ELEMENT_IMPORT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_COMPILER] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__compiler, &ColladaParserAutoGen15Private::_data__compiler, &ColladaParserAutoGen15Private::_end__compiler, &ColladaParserAutoGen15Private::_preBegin__compiler, &ColladaParserAutoGen15Private::_preEnd__compiler, &ColladaParserAutoGen15Private::_freeAttributes__compiler);
mElementToNamespaceMap[HASH_ELEMENT_COMPILER] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BINARY] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__binary, &ColladaParserAutoGen15Private::_data__binary, &ColladaParserAutoGen15Private::_end__binary, &ColladaParserAutoGen15Private::_preBegin__binary, &ColladaParserAutoGen15Private::_preEnd__binary, &ColladaParserAutoGen15Private::_freeAttributes__binary);
mElementToNamespaceMap[HASH_ELEMENT_BINARY] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_REF____ANYURI] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__ref____anyURI, &ColladaParserAutoGen15Private::_data__ref____anyURI, &ColladaParserAutoGen15Private::_end__ref____anyURI, &ColladaParserAutoGen15Private::_preBegin__ref____anyURI, &ColladaParserAutoGen15Private::_preEnd__ref____anyURI, &ColladaParserAutoGen15Private::_freeAttributes__ref____anyURI);
mElementToNamespaceMap[HASH_ELEMENT_REF____ANYURI] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BINARY__HEX] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__binary__hex, &ColladaParserAutoGen15Private::_data__binary__hex, &ColladaParserAutoGen15Private::_end__binary__hex, &ColladaParserAutoGen15Private::_preBegin__binary__hex, &ColladaParserAutoGen15Private::_preEnd__binary__hex, &ColladaParserAutoGen15Private::_freeAttributes__binary__hex);
mElementToNamespaceMap[HASH_ELEMENT_BINARY__HEX] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LINKER] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__linker, &ColladaParserAutoGen15Private::_data__linker, &ColladaParserAutoGen15Private::_end__linker, &ColladaParserAutoGen15Private::_preBegin__linker, &ColladaParserAutoGen15Private::_preEnd__linker, &ColladaParserAutoGen15Private::_freeAttributes__linker);
mElementToNamespaceMap[HASH_ELEMENT_LINKER] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_GLES2_PROGRAM_TYPE____BIND_ATTRIBUTE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__gles2_program_type____bind_attribute, &ColladaParserAutoGen15Private::_data__gles2_program_type____bind_attribute, &ColladaParserAutoGen15Private::_end__gles2_program_type____bind_attribute, &ColladaParserAutoGen15Private::_preBegin__gles2_program_type____bind_attribute, &ColladaParserAutoGen15Private::_preEnd__gles2_program_type____bind_attribute, &ColladaParserAutoGen15Private::_freeAttributes__gles2_program_type____bind_attribute);
mElementToNamespaceMap[HASH_ELEMENT_GLES2_PROGRAM_TYPE____BIND_ATTRIBUTE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SEMANTIC____TOKEN] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__semantic____token, &ColladaParserAutoGen15Private::_data__semantic____token, &ColladaParserAutoGen15Private::_end__semantic____token, &ColladaParserAutoGen15Private::_preBegin__semantic____token, &ColladaParserAutoGen15Private::_preEnd__semantic____token, &ColladaParserAutoGen15Private::_freeAttributes__semantic____token);
mElementToNamespaceMap[HASH_ELEMENT_SEMANTIC____TOKEN] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_GLES2_PROGRAM_TYPE____BIND_UNIFORM] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__gles2_program_type____bind_uniform, &ColladaParserAutoGen15Private::_data__gles2_program_type____bind_uniform, &ColladaParserAutoGen15Private::_end__gles2_program_type____bind_uniform, &ColladaParserAutoGen15Private::_preBegin__gles2_program_type____bind_uniform, &ColladaParserAutoGen15Private::_preEnd__gles2_program_type____bind_uniform, &ColladaParserAutoGen15Private::_freeAttributes__gles2_program_type____bind_uniform);
mElementToNamespaceMap[HASH_ELEMENT_GLES2_PROGRAM_TYPE____BIND_UNIFORM] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__PROGRAM__BIND_UNIFORM__PARAM] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__program__bind_uniform__param, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__program__bind_uniform__param, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__program__bind_uniform__param, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__program__bind_uniform__param, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__program__bind_uniform__param, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__program__bind_uniform__param);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__PROGRAM__BIND_UNIFORM__PARAM] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_GLES2_PASS_TYPE____EVALUATE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__gles2_pass_type____evaluate, &ColladaParserAutoGen15Private::_data__gles2_pass_type____evaluate, &ColladaParserAutoGen15Private::_end__gles2_pass_type____evaluate, &ColladaParserAutoGen15Private::_preBegin__gles2_pass_type____evaluate, &ColladaParserAutoGen15Private::_preEnd__gles2_pass_type____evaluate, &ColladaParserAutoGen15Private::_freeAttributes__gles2_pass_type____evaluate);
mElementToNamespaceMap[HASH_ELEMENT_GLES2_PASS_TYPE____EVALUATE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_COLOR_TARGET] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__color_target, &ColladaParserAutoGen15Private::_data__color_target, &ColladaParserAutoGen15Private::_end__color_target, &ColladaParserAutoGen15Private::_preBegin__color_target, &ColladaParserAutoGen15Private::_preEnd__color_target, &ColladaParserAutoGen15Private::_freeAttributes__color_target);
mElementToNamespaceMap[HASH_ELEMENT_COLOR_TARGET] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FX_COLORTARGET_TYPE____PARAM] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__fx_colortarget_type____param, &ColladaParserAutoGen15Private::_data__fx_colortarget_type____param, &ColladaParserAutoGen15Private::_end__fx_colortarget_type____param, &ColladaParserAutoGen15Private::_preBegin__fx_colortarget_type____param, &ColladaParserAutoGen15Private::_preEnd__fx_colortarget_type____param, &ColladaParserAutoGen15Private::_freeAttributes__fx_colortarget_type____param);
mElementToNamespaceMap[HASH_ELEMENT_FX_COLORTARGET_TYPE____PARAM] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_DEPTH_TARGET] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__depth_target, &ColladaParserAutoGen15Private::_data__depth_target, &ColladaParserAutoGen15Private::_end__depth_target, &ColladaParserAutoGen15Private::_preBegin__depth_target, &ColladaParserAutoGen15Private::_preEnd__depth_target, &ColladaParserAutoGen15Private::_freeAttributes__depth_target);
mElementToNamespaceMap[HASH_ELEMENT_DEPTH_TARGET] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_STENCIL_TARGET] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__stencil_target, &ColladaParserAutoGen15Private::_data__stencil_target, &ColladaParserAutoGen15Private::_end__stencil_target, &ColladaParserAutoGen15Private::_preBegin__stencil_target, &ColladaParserAutoGen15Private::_preEnd__stencil_target, &ColladaParserAutoGen15Private::_freeAttributes__stencil_target);
mElementToNamespaceMap[HASH_ELEMENT_STENCIL_TARGET] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_COLOR_CLEAR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__color_clear, &ColladaParserAutoGen15Private::_data__color_clear, &ColladaParserAutoGen15Private::_end__color_clear, &ColladaParserAutoGen15Private::_preBegin__color_clear, &ColladaParserAutoGen15Private::_preEnd__color_clear, &ColladaParserAutoGen15Private::_freeAttributes__color_clear);
mElementToNamespaceMap[HASH_ELEMENT_COLOR_CLEAR] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_STENCIL_CLEAR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__stencil_clear, &ColladaParserAutoGen15Private::_data__stencil_clear, &ColladaParserAutoGen15Private::_end__stencil_clear, &ColladaParserAutoGen15Private::_preBegin__stencil_clear, &ColladaParserAutoGen15Private::_preEnd__stencil_clear, &ColladaParserAutoGen15Private::_freeAttributes__stencil_clear);
mElementToNamespaceMap[HASH_ELEMENT_STENCIL_CLEAR] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_DEPTH_CLEAR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__depth_clear, &ColladaParserAutoGen15Private::_data__depth_clear, &ColladaParserAutoGen15Private::_end__depth_clear, &ColladaParserAutoGen15Private::_preBegin__depth_clear, &ColladaParserAutoGen15Private::_preEnd__depth_clear, &ColladaParserAutoGen15Private::_freeAttributes__depth_clear);
mElementToNamespaceMap[HASH_ELEMENT_DEPTH_CLEAR] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_DRAW] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__draw, &ColladaParserAutoGen15Private::_data__draw, &ColladaParserAutoGen15Private::_end__draw, &ColladaParserAutoGen15Private::_preBegin__draw, &ColladaParserAutoGen15Private::_preEnd__draw, &ColladaParserAutoGen15Private::_freeAttributes__draw);
mElementToNamespaceMap[HASH_ELEMENT_DRAW] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL, &ColladaParserAutoGen15Private::_data__profile_GLSL, &ColladaParserAutoGen15Private::_end__profile_GLSL, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_NEWPARAM____GLSL_NEWPARAM_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__newparam____glsl_newparam_type, &ColladaParserAutoGen15Private::_data__newparam____glsl_newparam_type, &ColladaParserAutoGen15Private::_end__newparam____glsl_newparam_type, &ColladaParserAutoGen15Private::_preBegin__newparam____glsl_newparam_type, &ColladaParserAutoGen15Private::_preEnd__newparam____glsl_newparam_type, &ColladaParserAutoGen15Private::_freeAttributes__newparam____glsl_newparam_type);
mElementToNamespaceMap[HASH_ELEMENT_NEWPARAM____GLSL_NEWPARAM_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ARRAY____GLSL_ARRAY_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__array____glsl_array_type, &ColladaParserAutoGen15Private::_data__array____glsl_array_type, &ColladaParserAutoGen15Private::_end__array____glsl_array_type, &ColladaParserAutoGen15Private::_preBegin__array____glsl_array_type, &ColladaParserAutoGen15Private::_preEnd__array____glsl_array_type, &ColladaParserAutoGen15Private::_freeAttributes__array____glsl_array_type);
mElementToNamespaceMap[HASH_ELEMENT_ARRAY____GLSL_ARRAY_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL_TYPE____TECHNIQUE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_glsl_type____technique, &ColladaParserAutoGen15Private::_data__profile_glsl_type____technique, &ColladaParserAutoGen15Private::_end__profile_glsl_type____technique, &ColladaParserAutoGen15Private::_preBegin__profile_glsl_type____technique, &ColladaParserAutoGen15Private::_preEnd__profile_glsl_type____technique, &ColladaParserAutoGen15Private::_freeAttributes__profile_glsl_type____technique);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL_TYPE____TECHNIQUE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__ALPHA_FUNC] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__alpha_func, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__alpha_func, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__alpha_func, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__alpha_func, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__alpha_func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__alpha_func);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__ALPHA_FUNC] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__ALPHA_FUNC__FUNC] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__alpha_func__func, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__alpha_func__func, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__alpha_func__func, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__alpha_func__func, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__alpha_func__func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__alpha_func__func);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__ALPHA_FUNC__FUNC] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__ALPHA_FUNC__VALUE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__alpha_func__value, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__alpha_func__value, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__alpha_func__value, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__alpha_func__value, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__alpha_func__value, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__alpha_func__value);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__ALPHA_FUNC__VALUE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_FUNC] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__blend_func, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__blend_func, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__blend_func, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__blend_func, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__blend_func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__blend_func);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_FUNC] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_FUNC__SRC] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__blend_func__src, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__blend_func__src, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__blend_func__src, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__blend_func__src, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__blend_func__src, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__blend_func__src);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_FUNC__SRC] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_FUNC__DEST] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__blend_func__dest, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__blend_func__dest, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__blend_func__dest, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__blend_func__dest, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__blend_func__dest, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__blend_func__dest);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_FUNC__DEST] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_FUNC_SEPARATE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__blend_func_separate, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__blend_func_separate, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__blend_func_separate, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__blend_func_separate, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__blend_func_separate, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__blend_func_separate);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_FUNC_SEPARATE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_FUNC_SEPARATE__SRC_RGB] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__blend_func_separate__src_rgb, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__blend_func_separate__src_rgb, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__blend_func_separate__src_rgb, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__blend_func_separate__src_rgb, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__blend_func_separate__src_rgb, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__blend_func_separate__src_rgb);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_FUNC_SEPARATE__SRC_RGB] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_FUNC_SEPARATE__DEST_RGB] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__blend_func_separate__dest_rgb, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__blend_func_separate__dest_rgb, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__blend_func_separate__dest_rgb, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__blend_func_separate__dest_rgb, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__blend_func_separate__dest_rgb, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__blend_func_separate__dest_rgb);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_FUNC_SEPARATE__DEST_RGB] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_FUNC_SEPARATE__SRC_ALPHA] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__blend_func_separate__src_alpha, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__blend_func_separate__src_alpha, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__blend_func_separate__src_alpha, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__blend_func_separate__src_alpha, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__blend_func_separate__src_alpha, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__blend_func_separate__src_alpha);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_FUNC_SEPARATE__SRC_ALPHA] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_FUNC_SEPARATE__DEST_ALPHA] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__blend_func_separate__dest_alpha, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__blend_func_separate__dest_alpha, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__blend_func_separate__dest_alpha, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__blend_func_separate__dest_alpha, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__blend_func_separate__dest_alpha, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__blend_func_separate__dest_alpha);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_FUNC_SEPARATE__DEST_ALPHA] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_EQUATION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__blend_equation, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__blend_equation, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__blend_equation, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__blend_equation, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__blend_equation, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__blend_equation);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_EQUATION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_EQUATION_SEPARATE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__blend_equation_separate, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__blend_equation_separate, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__blend_equation_separate, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__blend_equation_separate, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__blend_equation_separate, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__blend_equation_separate);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_EQUATION_SEPARATE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_EQUATION_SEPARATE__RGB] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__blend_equation_separate__rgb, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__blend_equation_separate__rgb, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__blend_equation_separate__rgb, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__blend_equation_separate__rgb, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__blend_equation_separate__rgb, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__blend_equation_separate__rgb);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_EQUATION_SEPARATE__RGB] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_EQUATION_SEPARATE__ALPHA] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__blend_equation_separate__alpha, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__blend_equation_separate__alpha, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__blend_equation_separate__alpha, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__blend_equation_separate__alpha, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__blend_equation_separate__alpha, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__blend_equation_separate__alpha);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_EQUATION_SEPARATE__ALPHA] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_COLOR_MATERIAL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__color_material, &ColladaParserAutoGen15Private::_data__color_material, &ColladaParserAutoGen15Private::_end__color_material, &ColladaParserAutoGen15Private::_preBegin__color_material, &ColladaParserAutoGen15Private::_preEnd__color_material, &ColladaParserAutoGen15Private::_freeAttributes__color_material);
mElementToNamespaceMap[HASH_ELEMENT_COLOR_MATERIAL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_COLOR_MATERIAL__FACE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__color_material__face, &ColladaParserAutoGen15Private::_data__color_material__face, &ColladaParserAutoGen15Private::_end__color_material__face, &ColladaParserAutoGen15Private::_preBegin__color_material__face, &ColladaParserAutoGen15Private::_preEnd__color_material__face, &ColladaParserAutoGen15Private::_freeAttributes__color_material__face);
mElementToNamespaceMap[HASH_ELEMENT_COLOR_MATERIAL__FACE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_COLOR_MATERIAL__MODE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__color_material__mode, &ColladaParserAutoGen15Private::_data__color_material__mode, &ColladaParserAutoGen15Private::_end__color_material__mode, &ColladaParserAutoGen15Private::_preBegin__color_material__mode, &ColladaParserAutoGen15Private::_preEnd__color_material__mode, &ColladaParserAutoGen15Private::_freeAttributes__color_material__mode);
mElementToNamespaceMap[HASH_ELEMENT_COLOR_MATERIAL__MODE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__CULL_FACE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__cull_face, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__cull_face, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__cull_face, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__cull_face, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__cull_face, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__cull_face);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__CULL_FACE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__DEPTH_FUNC] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__depth_func, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__depth_func, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__depth_func, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__depth_func, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__depth_func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__depth_func);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__DEPTH_FUNC] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__FOG_MODE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__fog_mode, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__fog_mode, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__fog_mode, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__fog_mode, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__fog_mode, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__fog_mode);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__FOG_MODE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FOG_COORD_SRC] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__fog_coord_src, &ColladaParserAutoGen15Private::_data__fog_coord_src, &ColladaParserAutoGen15Private::_end__fog_coord_src, &ColladaParserAutoGen15Private::_preBegin__fog_coord_src, &ColladaParserAutoGen15Private::_preEnd__fog_coord_src, &ColladaParserAutoGen15Private::_freeAttributes__fog_coord_src);
mElementToNamespaceMap[HASH_ELEMENT_FOG_COORD_SRC] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__FRONT_FACE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__front_face, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__front_face, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__front_face, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__front_face, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__front_face, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__front_face);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__FRONT_FACE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__light_model_color_control, &ColladaParserAutoGen15Private::_data__light_model_color_control, &ColladaParserAutoGen15Private::_end__light_model_color_control, &ColladaParserAutoGen15Private::_preBegin__light_model_color_control, &ColladaParserAutoGen15Private::_preEnd__light_model_color_control, &ColladaParserAutoGen15Private::_freeAttributes__light_model_color_control);
mElementToNamespaceMap[HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LOGIC_OP] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__logic_op, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__logic_op, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__logic_op, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__logic_op, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__logic_op, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__logic_op);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LOGIC_OP] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_POLYGON_MODE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__polygon_mode, &ColladaParserAutoGen15Private::_data__polygon_mode, &ColladaParserAutoGen15Private::_end__polygon_mode, &ColladaParserAutoGen15Private::_preBegin__polygon_mode, &ColladaParserAutoGen15Private::_preEnd__polygon_mode, &ColladaParserAutoGen15Private::_freeAttributes__polygon_mode);
mElementToNamespaceMap[HASH_ELEMENT_POLYGON_MODE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_POLYGON_MODE__FACE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__polygon_mode__face, &ColladaParserAutoGen15Private::_data__polygon_mode__face, &ColladaParserAutoGen15Private::_end__polygon_mode__face, &ColladaParserAutoGen15Private::_preBegin__polygon_mode__face, &ColladaParserAutoGen15Private::_preEnd__polygon_mode__face, &ColladaParserAutoGen15Private::_freeAttributes__polygon_mode__face);
mElementToNamespaceMap[HASH_ELEMENT_POLYGON_MODE__FACE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_POLYGON_MODE__MODE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__polygon_mode__mode, &ColladaParserAutoGen15Private::_data__polygon_mode__mode, &ColladaParserAutoGen15Private::_end__polygon_mode__mode, &ColladaParserAutoGen15Private::_preBegin__polygon_mode__mode, &ColladaParserAutoGen15Private::_preEnd__polygon_mode__mode, &ColladaParserAutoGen15Private::_freeAttributes__polygon_mode__mode);
mElementToNamespaceMap[HASH_ELEMENT_POLYGON_MODE__MODE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__SHADE_MODEL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__shade_model, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__shade_model, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__shade_model, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__shade_model, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__shade_model, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__shade_model);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__SHADE_MODEL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_FUNC] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_func, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_func, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_func, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_func, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_func);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_FUNC] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_FUNC__FUNC] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_func__func, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_func__func, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_func__func, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_func__func, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_func__func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_func__func);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_FUNC__FUNC] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_FUNC__REF] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_func__ref, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_func__ref, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_func__ref, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_func__ref, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_func__ref, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_func__ref);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_FUNC__REF] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_FUNC__MASK] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_func__mask, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_func__mask, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_func__mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_func__mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_func__mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_func__mask);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_FUNC__MASK] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_OP] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_op, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_op, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_op, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_op, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_op, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_op);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_OP] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_OP__FAIL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_op__fail, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_op__fail, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_op__fail, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_op__fail, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_op__fail, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_op__fail);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_OP__FAIL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_OP__ZFAIL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_op__zfail, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_op__zfail, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_op__zfail, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_op__zfail, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_op__zfail, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_op__zfail);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_OP__ZFAIL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_OP__ZPASS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_op__zpass, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_op__zpass, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_op__zpass, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_op__zpass, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_op__zpass, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_op__zpass);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_OP__ZPASS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_FUNC_SEPARATE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_func_separate, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_func_separate, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_func_separate, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_func_separate, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_func_separate, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_func_separate);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_FUNC_SEPARATE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_FUNC_SEPARATE__FRONT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_func_separate__front, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_func_separate__front, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_func_separate__front, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_func_separate__front, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_func_separate__front, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_func_separate__front);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_FUNC_SEPARATE__FRONT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_FUNC_SEPARATE__BACK] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_func_separate__back, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_func_separate__back, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_func_separate__back, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_func_separate__back, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_func_separate__back, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_func_separate__back);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_FUNC_SEPARATE__BACK] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_FUNC_SEPARATE__REF] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_func_separate__ref, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_func_separate__ref, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_func_separate__ref, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_func_separate__ref, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_func_separate__ref, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_func_separate__ref);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_FUNC_SEPARATE__REF] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_FUNC_SEPARATE__MASK] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_func_separate__mask, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_func_separate__mask, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_func_separate__mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_func_separate__mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_func_separate__mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_func_separate__mask);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_FUNC_SEPARATE__MASK] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_OP_SEPARATE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_op_separate, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_op_separate, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_op_separate, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_op_separate, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_op_separate, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_op_separate);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_OP_SEPARATE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_OP_SEPARATE__FACE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_op_separate__face, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_op_separate__face, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_op_separate__face, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_op_separate__face, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_op_separate__face, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_op_separate__face);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_OP_SEPARATE__FACE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_OP_SEPARATE__FAIL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_op_separate__fail, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_op_separate__fail, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_op_separate__fail, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_op_separate__fail, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_op_separate__fail, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_op_separate__fail);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_OP_SEPARATE__FAIL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_OP_SEPARATE__ZFAIL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_op_separate__zfail, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_op_separate__zfail, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_op_separate__zfail, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_op_separate__zfail, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_op_separate__zfail, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_op_separate__zfail);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_OP_SEPARATE__ZFAIL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_OP_SEPARATE__ZPASS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_op_separate__zpass, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_op_separate__zpass, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_op_separate__zpass, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_op_separate__zpass, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_op_separate__zpass, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_op_separate__zpass);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_OP_SEPARATE__ZPASS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_MASK_SEPARATE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_mask_separate, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_mask_separate, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_mask_separate, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_mask_separate, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_mask_separate, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_mask_separate);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_MASK_SEPARATE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_MASK_SEPARATE__FACE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_mask_separate__face, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_mask_separate__face, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_mask_separate__face, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_mask_separate__face, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_mask_separate__face, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_mask_separate__face);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_MASK_SEPARATE__FACE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_MASK_SEPARATE__MASK] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_mask_separate__mask, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_mask_separate__mask, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_mask_separate__mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_mask_separate__mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_mask_separate__mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_mask_separate__mask);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_MASK_SEPARATE__MASK] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__light_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__light_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__light_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__light_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__light_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__light_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_AMBIENT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__light_ambient, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__light_ambient, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__light_ambient, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__light_ambient, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__light_ambient, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__light_ambient);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_AMBIENT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_DIFFUSE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__light_diffuse, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__light_diffuse, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__light_diffuse, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__light_diffuse, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__light_diffuse, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__light_diffuse);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_DIFFUSE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_SPECULAR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__light_specular, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__light_specular, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__light_specular, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__light_specular, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__light_specular, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__light_specular);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_SPECULAR] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_POSITION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__light_position, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__light_position, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__light_position, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__light_position, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__light_position, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__light_position);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_POSITION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_CONSTANT_ATTENUATION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__light_constant_attenuation, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__light_constant_attenuation, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__light_constant_attenuation, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__light_constant_attenuation, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__light_constant_attenuation, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__light_constant_attenuation);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_CONSTANT_ATTENUATION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_LINEAR_ATTENUATION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__light_linear_attenuation, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__light_linear_attenuation, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__light_linear_attenuation, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__light_linear_attenuation, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__light_linear_attenuation, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__light_linear_attenuation);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_LINEAR_ATTENUATION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_QUADRATIC_ATTENUATION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__light_quadratic_attenuation, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__light_quadratic_attenuation, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__light_quadratic_attenuation, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__light_quadratic_attenuation, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__light_quadratic_attenuation, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__light_quadratic_attenuation);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_QUADRATIC_ATTENUATION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_SPOT_CUTOFF] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__light_spot_cutoff, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__light_spot_cutoff, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__light_spot_cutoff, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__light_spot_cutoff, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__light_spot_cutoff, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__light_spot_cutoff);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_SPOT_CUTOFF] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_SPOT_DIRECTION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__light_spot_direction, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__light_spot_direction, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__light_spot_direction, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__light_spot_direction, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__light_spot_direction, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__light_spot_direction);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_SPOT_DIRECTION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_SPOT_EXPONENT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__light_spot_exponent, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__light_spot_exponent, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__light_spot_exponent, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__light_spot_exponent, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__light_spot_exponent, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__light_spot_exponent);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_SPOT_EXPONENT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TEXTURE1D] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__texture1D, &ColladaParserAutoGen15Private::_data__texture1D, &ColladaParserAutoGen15Private::_end__texture1D, &ColladaParserAutoGen15Private::_preBegin__texture1D, &ColladaParserAutoGen15Private::_preEnd__texture1D, &ColladaParserAutoGen15Private::_freeAttributes__texture1D);
mElementToNamespaceMap[HASH_ELEMENT_TEXTURE1D] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_VALUE____FX_SAMPLER1D_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__value____fx_sampler1D_type, &ColladaParserAutoGen15Private::_data__value____fx_sampler1D_type, &ColladaParserAutoGen15Private::_end__value____fx_sampler1D_type, &ColladaParserAutoGen15Private::_preBegin__value____fx_sampler1D_type, &ColladaParserAutoGen15Private::_preEnd__value____fx_sampler1D_type, &ColladaParserAutoGen15Private::_freeAttributes__value____fx_sampler1D_type);
mElementToNamespaceMap[HASH_ELEMENT_VALUE____FX_SAMPLER1D_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PARAM____NCNAME] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__param____NCName, &ColladaParserAutoGen15Private::_data__param____NCName, &ColladaParserAutoGen15Private::_end__param____NCName, &ColladaParserAutoGen15Private::_preBegin__param____NCName, &ColladaParserAutoGen15Private::_preEnd__param____NCName, &ColladaParserAutoGen15Private::_freeAttributes__param____NCName);
mElementToNamespaceMap[HASH_ELEMENT_PARAM____NCNAME] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TEXTURE2D] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__texture2D, &ColladaParserAutoGen15Private::_data__texture2D, &ColladaParserAutoGen15Private::_end__texture2D, &ColladaParserAutoGen15Private::_preBegin__texture2D, &ColladaParserAutoGen15Private::_preEnd__texture2D, &ColladaParserAutoGen15Private::_freeAttributes__texture2D);
mElementToNamespaceMap[HASH_ELEMENT_TEXTURE2D] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_VALUE____FX_SAMPLER2D_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__value____fx_sampler2D_type, &ColladaParserAutoGen15Private::_data__value____fx_sampler2D_type, &ColladaParserAutoGen15Private::_end__value____fx_sampler2D_type, &ColladaParserAutoGen15Private::_preBegin__value____fx_sampler2D_type, &ColladaParserAutoGen15Private::_preEnd__value____fx_sampler2D_type, &ColladaParserAutoGen15Private::_freeAttributes__value____fx_sampler2D_type);
mElementToNamespaceMap[HASH_ELEMENT_VALUE____FX_SAMPLER2D_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TEXTURE3D] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__texture3D, &ColladaParserAutoGen15Private::_data__texture3D, &ColladaParserAutoGen15Private::_end__texture3D, &ColladaParserAutoGen15Private::_preBegin__texture3D, &ColladaParserAutoGen15Private::_preEnd__texture3D, &ColladaParserAutoGen15Private::_freeAttributes__texture3D);
mElementToNamespaceMap[HASH_ELEMENT_TEXTURE3D] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_VALUE____FX_SAMPLER3D_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__value____fx_sampler3D_type, &ColladaParserAutoGen15Private::_data__value____fx_sampler3D_type, &ColladaParserAutoGen15Private::_end__value____fx_sampler3D_type, &ColladaParserAutoGen15Private::_preBegin__value____fx_sampler3D_type, &ColladaParserAutoGen15Private::_preEnd__value____fx_sampler3D_type, &ColladaParserAutoGen15Private::_freeAttributes__value____fx_sampler3D_type);
mElementToNamespaceMap[HASH_ELEMENT_VALUE____FX_SAMPLER3D_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TEXTURECUBE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__textureCUBE, &ColladaParserAutoGen15Private::_data__textureCUBE, &ColladaParserAutoGen15Private::_end__textureCUBE, &ColladaParserAutoGen15Private::_preBegin__textureCUBE, &ColladaParserAutoGen15Private::_preEnd__textureCUBE, &ColladaParserAutoGen15Private::_freeAttributes__textureCUBE);
mElementToNamespaceMap[HASH_ELEMENT_TEXTURECUBE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_VALUE____FX_SAMPLERCUBE_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__value____fx_samplerCUBE_type, &ColladaParserAutoGen15Private::_data__value____fx_samplerCUBE_type, &ColladaParserAutoGen15Private::_end__value____fx_samplerCUBE_type, &ColladaParserAutoGen15Private::_preBegin__value____fx_samplerCUBE_type, &ColladaParserAutoGen15Private::_preEnd__value____fx_samplerCUBE_type, &ColladaParserAutoGen15Private::_freeAttributes__value____fx_samplerCUBE_type);
mElementToNamespaceMap[HASH_ELEMENT_VALUE____FX_SAMPLERCUBE_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TEXTURERECT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__textureRECT, &ColladaParserAutoGen15Private::_data__textureRECT, &ColladaParserAutoGen15Private::_end__textureRECT, &ColladaParserAutoGen15Private::_preBegin__textureRECT, &ColladaParserAutoGen15Private::_preEnd__textureRECT, &ColladaParserAutoGen15Private::_freeAttributes__textureRECT);
mElementToNamespaceMap[HASH_ELEMENT_TEXTURERECT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_VALUE____FX_SAMPLERRECT_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__value____fx_samplerRECT_type, &ColladaParserAutoGen15Private::_data__value____fx_samplerRECT_type, &ColladaParserAutoGen15Private::_end__value____fx_samplerRECT_type, &ColladaParserAutoGen15Private::_preBegin__value____fx_samplerRECT_type, &ColladaParserAutoGen15Private::_preEnd__value____fx_samplerRECT_type, &ColladaParserAutoGen15Private::_freeAttributes__value____fx_samplerRECT_type);
mElementToNamespaceMap[HASH_ELEMENT_VALUE____FX_SAMPLERRECT_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TEXTUREDEPTH] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__textureDEPTH, &ColladaParserAutoGen15Private::_data__textureDEPTH, &ColladaParserAutoGen15Private::_end__textureDEPTH, &ColladaParserAutoGen15Private::_preBegin__textureDEPTH, &ColladaParserAutoGen15Private::_preEnd__textureDEPTH, &ColladaParserAutoGen15Private::_freeAttributes__textureDEPTH);
mElementToNamespaceMap[HASH_ELEMENT_TEXTUREDEPTH] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_VALUE____FX_SAMPLERDEPTH_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__value____fx_samplerDEPTH_type, &ColladaParserAutoGen15Private::_data__value____fx_samplerDEPTH_type, &ColladaParserAutoGen15Private::_end__value____fx_samplerDEPTH_type, &ColladaParserAutoGen15Private::_preBegin__value____fx_samplerDEPTH_type, &ColladaParserAutoGen15Private::_preEnd__value____fx_samplerDEPTH_type, &ColladaParserAutoGen15Private::_freeAttributes__value____fx_samplerDEPTH_type);
mElementToNamespaceMap[HASH_ELEMENT_VALUE____FX_SAMPLERDEPTH_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TEXTURE1D_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__texture1D_enable, &ColladaParserAutoGen15Private::_data__texture1D_enable, &ColladaParserAutoGen15Private::_end__texture1D_enable, &ColladaParserAutoGen15Private::_preBegin__texture1D_enable, &ColladaParserAutoGen15Private::_preEnd__texture1D_enable, &ColladaParserAutoGen15Private::_freeAttributes__texture1D_enable);
mElementToNamespaceMap[HASH_ELEMENT_TEXTURE1D_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TEXTURE2D_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__texture2D_enable, &ColladaParserAutoGen15Private::_data__texture2D_enable, &ColladaParserAutoGen15Private::_end__texture2D_enable, &ColladaParserAutoGen15Private::_preBegin__texture2D_enable, &ColladaParserAutoGen15Private::_preEnd__texture2D_enable, &ColladaParserAutoGen15Private::_freeAttributes__texture2D_enable);
mElementToNamespaceMap[HASH_ELEMENT_TEXTURE2D_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TEXTURE3D_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__texture3D_enable, &ColladaParserAutoGen15Private::_data__texture3D_enable, &ColladaParserAutoGen15Private::_end__texture3D_enable, &ColladaParserAutoGen15Private::_preBegin__texture3D_enable, &ColladaParserAutoGen15Private::_preEnd__texture3D_enable, &ColladaParserAutoGen15Private::_freeAttributes__texture3D_enable);
mElementToNamespaceMap[HASH_ELEMENT_TEXTURE3D_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TEXTURECUBE_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__textureCUBE_enable, &ColladaParserAutoGen15Private::_data__textureCUBE_enable, &ColladaParserAutoGen15Private::_end__textureCUBE_enable, &ColladaParserAutoGen15Private::_preBegin__textureCUBE_enable, &ColladaParserAutoGen15Private::_preEnd__textureCUBE_enable, &ColladaParserAutoGen15Private::_freeAttributes__textureCUBE_enable);
mElementToNamespaceMap[HASH_ELEMENT_TEXTURECUBE_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TEXTURERECT_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__textureRECT_enable, &ColladaParserAutoGen15Private::_data__textureRECT_enable, &ColladaParserAutoGen15Private::_end__textureRECT_enable, &ColladaParserAutoGen15Private::_preBegin__textureRECT_enable, &ColladaParserAutoGen15Private::_preEnd__textureRECT_enable, &ColladaParserAutoGen15Private::_freeAttributes__textureRECT_enable);
mElementToNamespaceMap[HASH_ELEMENT_TEXTURERECT_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TEXTUREDEPTH_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__textureDEPTH_enable, &ColladaParserAutoGen15Private::_data__textureDEPTH_enable, &ColladaParserAutoGen15Private::_end__textureDEPTH_enable, &ColladaParserAutoGen15Private::_preBegin__textureDEPTH_enable, &ColladaParserAutoGen15Private::_preEnd__textureDEPTH_enable, &ColladaParserAutoGen15Private::_freeAttributes__textureDEPTH_enable);
mElementToNamespaceMap[HASH_ELEMENT_TEXTUREDEPTH_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TEXTURE_ENV_COLOR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__texture_env_color, &ColladaParserAutoGen15Private::_data__texture_env_color, &ColladaParserAutoGen15Private::_end__texture_env_color, &ColladaParserAutoGen15Private::_preBegin__texture_env_color, &ColladaParserAutoGen15Private::_preEnd__texture_env_color, &ColladaParserAutoGen15Private::_freeAttributes__texture_env_color);
mElementToNamespaceMap[HASH_ELEMENT_TEXTURE_ENV_COLOR] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TEXTURE_ENV_MODE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__texture_env_mode, &ColladaParserAutoGen15Private::_data__texture_env_mode, &ColladaParserAutoGen15Private::_end__texture_env_mode, &ColladaParserAutoGen15Private::_preBegin__texture_env_mode, &ColladaParserAutoGen15Private::_preEnd__texture_env_mode, &ColladaParserAutoGen15Private::_freeAttributes__texture_env_mode);
mElementToNamespaceMap[HASH_ELEMENT_TEXTURE_ENV_MODE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__CLIP_PLANE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__clip_plane, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__clip_plane, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__clip_plane, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__clip_plane, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__clip_plane, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__clip_plane);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__CLIP_PLANE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__CLIP_PLANE_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__clip_plane_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__clip_plane_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__clip_plane_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__clip_plane_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__clip_plane_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__clip_plane_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__CLIP_PLANE_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_COLOR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__blend_color, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__blend_color, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__blend_color, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__blend_color, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__blend_color, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__blend_color);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_COLOR] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__COLOR_MASK] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__color_mask, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__color_mask, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__color_mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__color_mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__color_mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__color_mask);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__COLOR_MASK] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_DEPTH_BOUNDS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__depth_bounds, &ColladaParserAutoGen15Private::_data__depth_bounds, &ColladaParserAutoGen15Private::_end__depth_bounds, &ColladaParserAutoGen15Private::_preBegin__depth_bounds, &ColladaParserAutoGen15Private::_preEnd__depth_bounds, &ColladaParserAutoGen15Private::_freeAttributes__depth_bounds);
mElementToNamespaceMap[HASH_ELEMENT_DEPTH_BOUNDS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__DEPTH_MASK] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__depth_mask, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__depth_mask, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__depth_mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__depth_mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__depth_mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__depth_mask);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__DEPTH_MASK] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__DEPTH_RANGE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__depth_range, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__depth_range, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__depth_range, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__depth_range, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__depth_range, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__depth_range);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__DEPTH_RANGE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__FOG_DENSITY] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__fog_density, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__fog_density, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__fog_density, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__fog_density, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__fog_density, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__fog_density);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__FOG_DENSITY] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__FOG_START] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__fog_start, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__fog_start, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__fog_start, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__fog_start, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__fog_start, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__fog_start);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__FOG_START] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__FOG_END] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__fog_end, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__fog_end, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__fog_end, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__fog_end, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__fog_end, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__fog_end);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__FOG_END] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__FOG_COLOR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__fog_color, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__fog_color, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__fog_color, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__fog_color, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__fog_color, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__fog_color);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__FOG_COLOR] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_MODEL_AMBIENT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__light_model_ambient, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__light_model_ambient, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__light_model_ambient, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__light_model_ambient, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__light_model_ambient, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__light_model_ambient);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_MODEL_AMBIENT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHTING_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__lighting_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__lighting_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__lighting_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__lighting_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__lighting_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__lighting_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHTING_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LINE_STIPPLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__line_stipple, &ColladaParserAutoGen15Private::_data__line_stipple, &ColladaParserAutoGen15Private::_end__line_stipple, &ColladaParserAutoGen15Private::_preBegin__line_stipple, &ColladaParserAutoGen15Private::_preEnd__line_stipple, &ColladaParserAutoGen15Private::_freeAttributes__line_stipple);
mElementToNamespaceMap[HASH_ELEMENT_LINE_STIPPLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LINE_WIDTH] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__line_width, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__line_width, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__line_width, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__line_width, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__line_width, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__line_width);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LINE_WIDTH] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__MATERIAL_AMBIENT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__material_ambient, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__material_ambient, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__material_ambient, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__material_ambient, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__material_ambient, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__material_ambient);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__MATERIAL_AMBIENT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__MATERIAL_DIFFUSE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__material_diffuse, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__material_diffuse, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__material_diffuse, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__material_diffuse, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__material_diffuse, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__material_diffuse);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__MATERIAL_DIFFUSE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__MATERIAL_EMISSION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__material_emission, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__material_emission, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__material_emission, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__material_emission, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__material_emission, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__material_emission);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__MATERIAL_EMISSION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__MATERIAL_SHININESS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__material_shininess, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__material_shininess, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__material_shininess, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__material_shininess, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__material_shininess, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__material_shininess);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__MATERIAL_SHININESS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__MATERIAL_SPECULAR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__material_specular, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__material_specular, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__material_specular, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__material_specular, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__material_specular, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__material_specular);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__MATERIAL_SPECULAR] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__MODEL_VIEW_MATRIX] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__model_view_matrix, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__model_view_matrix, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__model_view_matrix, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__model_view_matrix, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__model_view_matrix, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__model_view_matrix);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__MODEL_VIEW_MATRIX] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__POINT_DISTANCE_ATTENUATION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__point_distance_attenuation, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__point_distance_attenuation, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__point_distance_attenuation, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__point_distance_attenuation, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__point_distance_attenuation, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__point_distance_attenuation);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__POINT_DISTANCE_ATTENUATION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__POINT_FADE_THRESHOLD_SIZE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__point_fade_threshold_size, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__point_fade_threshold_size, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__point_fade_threshold_size, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__point_fade_threshold_size, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__point_fade_threshold_size, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__point_fade_threshold_size);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__POINT_FADE_THRESHOLD_SIZE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__POINT_SIZE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__point_size, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__point_size, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__point_size, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__point_size, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__point_size, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__point_size);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__POINT_SIZE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__POINT_SIZE_MIN] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__point_size_min, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__point_size_min, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__point_size_min, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__point_size_min, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__point_size_min, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__point_size_min);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__POINT_SIZE_MIN] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__POINT_SIZE_MAX] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__point_size_max, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__point_size_max, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__point_size_max, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__point_size_max, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__point_size_max, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__point_size_max);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__POINT_SIZE_MAX] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__POLYGON_OFFSET] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__polygon_offset, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__polygon_offset, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__polygon_offset, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__polygon_offset, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__polygon_offset, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__polygon_offset);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__POLYGON_OFFSET] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__PROJECTION_MATRIX] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__projection_matrix, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__projection_matrix, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__projection_matrix, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__projection_matrix, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__projection_matrix, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__projection_matrix);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__PROJECTION_MATRIX] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__SCISSOR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__scissor, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__scissor, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__scissor, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__scissor, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__scissor, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__scissor);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__SCISSOR] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_MASK] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_mask, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_mask, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_mask);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_MASK] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__ALPHA_TEST_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__alpha_test_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__alpha_test_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__alpha_test_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__alpha_test_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__alpha_test_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__alpha_test_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__ALPHA_TEST_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__blend_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__blend_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__blend_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__blend_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__blend_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__blend_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__COLOR_LOGIC_OP_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__color_logic_op_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__color_logic_op_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__color_logic_op_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__color_logic_op_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__color_logic_op_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__color_logic_op_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__COLOR_LOGIC_OP_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__COLOR_MATERIAL_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__color_material_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__color_material_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__color_material_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__color_material_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__color_material_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__color_material_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__COLOR_MATERIAL_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__CULL_FACE_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__cull_face_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__cull_face_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__cull_face_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__cull_face_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__cull_face_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__cull_face_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__CULL_FACE_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_DEPTH_BOUNDS_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__depth_bounds_enable, &ColladaParserAutoGen15Private::_data__depth_bounds_enable, &ColladaParserAutoGen15Private::_end__depth_bounds_enable, &ColladaParserAutoGen15Private::_preBegin__depth_bounds_enable, &ColladaParserAutoGen15Private::_preEnd__depth_bounds_enable, &ColladaParserAutoGen15Private::_freeAttributes__depth_bounds_enable);
mElementToNamespaceMap[HASH_ELEMENT_DEPTH_BOUNDS_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_DEPTH_CLAMP_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__depth_clamp_enable, &ColladaParserAutoGen15Private::_data__depth_clamp_enable, &ColladaParserAutoGen15Private::_end__depth_clamp_enable, &ColladaParserAutoGen15Private::_preBegin__depth_clamp_enable, &ColladaParserAutoGen15Private::_preEnd__depth_clamp_enable, &ColladaParserAutoGen15Private::_freeAttributes__depth_clamp_enable);
mElementToNamespaceMap[HASH_ELEMENT_DEPTH_CLAMP_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__DEPTH_TEST_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__depth_test_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__depth_test_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__depth_test_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__depth_test_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__depth_test_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__depth_test_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__DEPTH_TEST_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__DITHER_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__dither_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__dither_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__dither_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__dither_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__dither_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__dither_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__DITHER_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__FOG_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__fog_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__fog_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__fog_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__fog_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__fog_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__fog_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__FOG_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__light_model_local_viewer_enable, &ColladaParserAutoGen15Private::_data__light_model_local_viewer_enable, &ColladaParserAutoGen15Private::_end__light_model_local_viewer_enable, &ColladaParserAutoGen15Private::_preBegin__light_model_local_viewer_enable, &ColladaParserAutoGen15Private::_preEnd__light_model_local_viewer_enable, &ColladaParserAutoGen15Private::_freeAttributes__light_model_local_viewer_enable);
mElementToNamespaceMap[HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_MODEL_TWO_SIDE_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__light_model_two_side_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__light_model_two_side_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__light_model_two_side_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__light_model_two_side_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__light_model_two_side_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__light_model_two_side_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_MODEL_TWO_SIDE_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LINE_SMOOTH_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__line_smooth_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__line_smooth_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__line_smooth_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__line_smooth_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__line_smooth_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__line_smooth_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LINE_SMOOTH_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LINE_STIPPLE_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__line_stipple_enable, &ColladaParserAutoGen15Private::_data__line_stipple_enable, &ColladaParserAutoGen15Private::_end__line_stipple_enable, &ColladaParserAutoGen15Private::_preBegin__line_stipple_enable, &ColladaParserAutoGen15Private::_preEnd__line_stipple_enable, &ColladaParserAutoGen15Private::_freeAttributes__line_stipple_enable);
mElementToNamespaceMap[HASH_ELEMENT_LINE_STIPPLE_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LOGIC_OP_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__logic_op_enable, &ColladaParserAutoGen15Private::_data__logic_op_enable, &ColladaParserAutoGen15Private::_end__logic_op_enable, &ColladaParserAutoGen15Private::_preBegin__logic_op_enable, &ColladaParserAutoGen15Private::_preEnd__logic_op_enable, &ColladaParserAutoGen15Private::_freeAttributes__logic_op_enable);
mElementToNamespaceMap[HASH_ELEMENT_LOGIC_OP_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__MULTISAMPLE_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__multisample_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__multisample_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__multisample_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__multisample_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__multisample_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__multisample_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__MULTISAMPLE_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__NORMALIZE_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__normalize_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__normalize_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__normalize_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__normalize_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__normalize_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__normalize_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__NORMALIZE_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__POINT_SMOOTH_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__point_smooth_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__point_smooth_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__point_smooth_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__point_smooth_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__point_smooth_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__point_smooth_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__POINT_SMOOTH_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__POLYGON_OFFSET_FILL_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__polygon_offset_fill_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__polygon_offset_fill_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__polygon_offset_fill_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__polygon_offset_fill_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__polygon_offset_fill_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__polygon_offset_fill_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__POLYGON_OFFSET_FILL_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__polygon_offset_line_enable, &ColladaParserAutoGen15Private::_data__polygon_offset_line_enable, &ColladaParserAutoGen15Private::_end__polygon_offset_line_enable, &ColladaParserAutoGen15Private::_preBegin__polygon_offset_line_enable, &ColladaParserAutoGen15Private::_preEnd__polygon_offset_line_enable, &ColladaParserAutoGen15Private::_freeAttributes__polygon_offset_line_enable);
mElementToNamespaceMap[HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__polygon_offset_point_enable, &ColladaParserAutoGen15Private::_data__polygon_offset_point_enable, &ColladaParserAutoGen15Private::_end__polygon_offset_point_enable, &ColladaParserAutoGen15Private::_preBegin__polygon_offset_point_enable, &ColladaParserAutoGen15Private::_preEnd__polygon_offset_point_enable, &ColladaParserAutoGen15Private::_freeAttributes__polygon_offset_point_enable);
mElementToNamespaceMap[HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_POLYGON_SMOOTH_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__polygon_smooth_enable, &ColladaParserAutoGen15Private::_data__polygon_smooth_enable, &ColladaParserAutoGen15Private::_end__polygon_smooth_enable, &ColladaParserAutoGen15Private::_preBegin__polygon_smooth_enable, &ColladaParserAutoGen15Private::_preEnd__polygon_smooth_enable, &ColladaParserAutoGen15Private::_freeAttributes__polygon_smooth_enable);
mElementToNamespaceMap[HASH_ELEMENT_POLYGON_SMOOTH_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_POLYGON_STIPPLE_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__polygon_stipple_enable, &ColladaParserAutoGen15Private::_data__polygon_stipple_enable, &ColladaParserAutoGen15Private::_end__polygon_stipple_enable, &ColladaParserAutoGen15Private::_preBegin__polygon_stipple_enable, &ColladaParserAutoGen15Private::_preEnd__polygon_stipple_enable, &ColladaParserAutoGen15Private::_freeAttributes__polygon_stipple_enable);
mElementToNamespaceMap[HASH_ELEMENT_POLYGON_STIPPLE_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__RESCALE_NORMAL_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__rescale_normal_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__rescale_normal_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__rescale_normal_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__rescale_normal_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__rescale_normal_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__rescale_normal_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__RESCALE_NORMAL_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__SAMPLE_ALPHA_TO_COVERAGE_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__sample_alpha_to_coverage_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__sample_alpha_to_coverage_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__sample_alpha_to_coverage_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__sample_alpha_to_coverage_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__sample_alpha_to_coverage_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__sample_alpha_to_coverage_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__SAMPLE_ALPHA_TO_COVERAGE_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__SAMPLE_ALPHA_TO_ONE_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__sample_alpha_to_one_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__sample_alpha_to_one_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__sample_alpha_to_one_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__sample_alpha_to_one_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__sample_alpha_to_one_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__sample_alpha_to_one_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__SAMPLE_ALPHA_TO_ONE_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__SAMPLE_COVERAGE_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__sample_coverage_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__sample_coverage_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__sample_coverage_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__sample_coverage_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__sample_coverage_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__sample_coverage_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__SAMPLE_COVERAGE_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__SCISSOR_TEST_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__scissor_test_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__scissor_test_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__scissor_test_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__scissor_test_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__scissor_test_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__scissor_test_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__SCISSOR_TEST_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_TEST_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_test_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_test_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_test_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_test_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_test_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_test_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_TEST_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROGRAM____GLSL_PROGRAM_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__program____glsl_program_type, &ColladaParserAutoGen15Private::_data__program____glsl_program_type, &ColladaParserAutoGen15Private::_end__program____glsl_program_type, &ColladaParserAutoGen15Private::_preBegin__program____glsl_program_type, &ColladaParserAutoGen15Private::_preEnd__program____glsl_program_type, &ColladaParserAutoGen15Private::_freeAttributes__program____glsl_program_type);
mElementToNamespaceMap[HASH_ELEMENT_PROGRAM____GLSL_PROGRAM_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SHADER____GLSL_SHADER_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__shader____glsl_shader_type, &ColladaParserAutoGen15Private::_data__shader____glsl_shader_type, &ColladaParserAutoGen15Private::_end__shader____glsl_shader_type, &ColladaParserAutoGen15Private::_preBegin__shader____glsl_shader_type, &ColladaParserAutoGen15Private::_preEnd__shader____glsl_shader_type, &ColladaParserAutoGen15Private::_freeAttributes__shader____glsl_shader_type);
mElementToNamespaceMap[HASH_ELEMENT_SHADER____GLSL_SHADER_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SOURCES____FX_SOURCES_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__sources____fx_sources_type, &ColladaParserAutoGen15Private::_data__sources____fx_sources_type, &ColladaParserAutoGen15Private::_end__sources____fx_sources_type, &ColladaParserAutoGen15Private::_preBegin__sources____fx_sources_type, &ColladaParserAutoGen15Private::_preEnd__sources____fx_sources_type, &ColladaParserAutoGen15Private::_freeAttributes__sources____fx_sources_type);
mElementToNamespaceMap[HASH_ELEMENT_SOURCES____FX_SOURCES_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_GLSL_PROGRAM_TYPE____BIND_ATTRIBUTE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__glsl_program_type____bind_attribute, &ColladaParserAutoGen15Private::_data__glsl_program_type____bind_attribute, &ColladaParserAutoGen15Private::_end__glsl_program_type____bind_attribute, &ColladaParserAutoGen15Private::_preBegin__glsl_program_type____bind_attribute, &ColladaParserAutoGen15Private::_preEnd__glsl_program_type____bind_attribute, &ColladaParserAutoGen15Private::_freeAttributes__glsl_program_type____bind_attribute);
mElementToNamespaceMap[HASH_ELEMENT_GLSL_PROGRAM_TYPE____BIND_ATTRIBUTE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_GLSL_PROGRAM_TYPE____BIND_UNIFORM] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__glsl_program_type____bind_uniform, &ColladaParserAutoGen15Private::_data__glsl_program_type____bind_uniform, &ColladaParserAutoGen15Private::_end__glsl_program_type____bind_uniform, &ColladaParserAutoGen15Private::_preBegin__glsl_program_type____bind_uniform, &ColladaParserAutoGen15Private::_preEnd__glsl_program_type____bind_uniform, &ColladaParserAutoGen15Private::_freeAttributes__glsl_program_type____bind_uniform);
mElementToNamespaceMap[HASH_ELEMENT_GLSL_PROGRAM_TYPE____BIND_UNIFORM] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__PROGRAM__BIND_UNIFORM__PARAM] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__program__bind_uniform__param, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__program__bind_uniform__param, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__program__bind_uniform__param, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__program__bind_uniform__param, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__program__bind_uniform__param, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__program__bind_uniform__param);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__PROGRAM__BIND_UNIFORM__PARAM] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__EVALUATE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__evaluate, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__evaluate, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__evaluate, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__evaluate, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__evaluate, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__evaluate);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__EVALUATE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_CG] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_CG, &ColladaParserAutoGen15Private::_data__profile_CG, &ColladaParserAutoGen15Private::_end__profile_CG, &ColladaParserAutoGen15Private::_preBegin__profile_CG, &ColladaParserAutoGen15Private::_preEnd__profile_CG, &ColladaParserAutoGen15Private::_freeAttributes__profile_CG);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_CG] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_NEWPARAM____CG_NEWPARAM_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__newparam____cg_newparam_type, &ColladaParserAutoGen15Private::_data__newparam____cg_newparam_type, &ColladaParserAutoGen15Private::_end__newparam____cg_newparam_type, &ColladaParserAutoGen15Private::_preBegin__newparam____cg_newparam_type, &ColladaParserAutoGen15Private::_preEnd__newparam____cg_newparam_type, &ColladaParserAutoGen15Private::_freeAttributes__newparam____cg_newparam_type);
mElementToNamespaceMap[HASH_ELEMENT_NEWPARAM____CG_NEWPARAM_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BOOL2X1] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool2x1, &ColladaParserAutoGen15Private::_data__bool2x1, &ColladaParserAutoGen15Private::_end__bool2x1, &ColladaParserAutoGen15Private::_preBegin__bool2x1, &ColladaParserAutoGen15Private::_preEnd__bool2x1, &ColladaParserAutoGen15Private::_freeAttributes__bool2x1);
mElementToNamespaceMap[HASH_ELEMENT_BOOL2X1] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BOOL2X2] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool2x2, &ColladaParserAutoGen15Private::_data__bool2x2, &ColladaParserAutoGen15Private::_end__bool2x2, &ColladaParserAutoGen15Private::_preBegin__bool2x2, &ColladaParserAutoGen15Private::_preEnd__bool2x2, &ColladaParserAutoGen15Private::_freeAttributes__bool2x2);
mElementToNamespaceMap[HASH_ELEMENT_BOOL2X2] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BOOL2X3] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool2x3, &ColladaParserAutoGen15Private::_data__bool2x3, &ColladaParserAutoGen15Private::_end__bool2x3, &ColladaParserAutoGen15Private::_preBegin__bool2x3, &ColladaParserAutoGen15Private::_preEnd__bool2x3, &ColladaParserAutoGen15Private::_freeAttributes__bool2x3);
mElementToNamespaceMap[HASH_ELEMENT_BOOL2X3] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BOOL2X4] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool2x4, &ColladaParserAutoGen15Private::_data__bool2x4, &ColladaParserAutoGen15Private::_end__bool2x4, &ColladaParserAutoGen15Private::_preBegin__bool2x4, &ColladaParserAutoGen15Private::_preEnd__bool2x4, &ColladaParserAutoGen15Private::_freeAttributes__bool2x4);
mElementToNamespaceMap[HASH_ELEMENT_BOOL2X4] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BOOL3X1] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool3x1, &ColladaParserAutoGen15Private::_data__bool3x1, &ColladaParserAutoGen15Private::_end__bool3x1, &ColladaParserAutoGen15Private::_preBegin__bool3x1, &ColladaParserAutoGen15Private::_preEnd__bool3x1, &ColladaParserAutoGen15Private::_freeAttributes__bool3x1);
mElementToNamespaceMap[HASH_ELEMENT_BOOL3X1] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BOOL3X2] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool3x2, &ColladaParserAutoGen15Private::_data__bool3x2, &ColladaParserAutoGen15Private::_end__bool3x2, &ColladaParserAutoGen15Private::_preBegin__bool3x2, &ColladaParserAutoGen15Private::_preEnd__bool3x2, &ColladaParserAutoGen15Private::_freeAttributes__bool3x2);
mElementToNamespaceMap[HASH_ELEMENT_BOOL3X2] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BOOL3X3] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool3x3, &ColladaParserAutoGen15Private::_data__bool3x3, &ColladaParserAutoGen15Private::_end__bool3x3, &ColladaParserAutoGen15Private::_preBegin__bool3x3, &ColladaParserAutoGen15Private::_preEnd__bool3x3, &ColladaParserAutoGen15Private::_freeAttributes__bool3x3);
mElementToNamespaceMap[HASH_ELEMENT_BOOL3X3] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BOOL3X4] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool3x4, &ColladaParserAutoGen15Private::_data__bool3x4, &ColladaParserAutoGen15Private::_end__bool3x4, &ColladaParserAutoGen15Private::_preBegin__bool3x4, &ColladaParserAutoGen15Private::_preEnd__bool3x4, &ColladaParserAutoGen15Private::_freeAttributes__bool3x4);
mElementToNamespaceMap[HASH_ELEMENT_BOOL3X4] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BOOL4X1] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool4x1, &ColladaParserAutoGen15Private::_data__bool4x1, &ColladaParserAutoGen15Private::_end__bool4x1, &ColladaParserAutoGen15Private::_preBegin__bool4x1, &ColladaParserAutoGen15Private::_preEnd__bool4x1, &ColladaParserAutoGen15Private::_freeAttributes__bool4x1);
mElementToNamespaceMap[HASH_ELEMENT_BOOL4X1] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BOOL4X2] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool4x2, &ColladaParserAutoGen15Private::_data__bool4x2, &ColladaParserAutoGen15Private::_end__bool4x2, &ColladaParserAutoGen15Private::_preBegin__bool4x2, &ColladaParserAutoGen15Private::_preEnd__bool4x2, &ColladaParserAutoGen15Private::_freeAttributes__bool4x2);
mElementToNamespaceMap[HASH_ELEMENT_BOOL4X2] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BOOL4X3] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool4x3, &ColladaParserAutoGen15Private::_data__bool4x3, &ColladaParserAutoGen15Private::_end__bool4x3, &ColladaParserAutoGen15Private::_preBegin__bool4x3, &ColladaParserAutoGen15Private::_preEnd__bool4x3, &ColladaParserAutoGen15Private::_freeAttributes__bool4x3);
mElementToNamespaceMap[HASH_ELEMENT_BOOL4X3] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BOOL4X4] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool4x4, &ColladaParserAutoGen15Private::_data__bool4x4, &ColladaParserAutoGen15Private::_end__bool4x4, &ColladaParserAutoGen15Private::_preBegin__bool4x4, &ColladaParserAutoGen15Private::_preEnd__bool4x4, &ColladaParserAutoGen15Private::_freeAttributes__bool4x4);
mElementToNamespaceMap[HASH_ELEMENT_BOOL4X4] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INT2X1] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__int2x1, &ColladaParserAutoGen15Private::_data__int2x1, &ColladaParserAutoGen15Private::_end__int2x1, &ColladaParserAutoGen15Private::_preBegin__int2x1, &ColladaParserAutoGen15Private::_preEnd__int2x1, &ColladaParserAutoGen15Private::_freeAttributes__int2x1);
mElementToNamespaceMap[HASH_ELEMENT_INT2X1] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INT2X2] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__int2x2, &ColladaParserAutoGen15Private::_data__int2x2, &ColladaParserAutoGen15Private::_end__int2x2, &ColladaParserAutoGen15Private::_preBegin__int2x2, &ColladaParserAutoGen15Private::_preEnd__int2x2, &ColladaParserAutoGen15Private::_freeAttributes__int2x2);
mElementToNamespaceMap[HASH_ELEMENT_INT2X2] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INT2X3] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__int2x3, &ColladaParserAutoGen15Private::_data__int2x3, &ColladaParserAutoGen15Private::_end__int2x3, &ColladaParserAutoGen15Private::_preBegin__int2x3, &ColladaParserAutoGen15Private::_preEnd__int2x3, &ColladaParserAutoGen15Private::_freeAttributes__int2x3);
mElementToNamespaceMap[HASH_ELEMENT_INT2X3] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INT2X4] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__int2x4, &ColladaParserAutoGen15Private::_data__int2x4, &ColladaParserAutoGen15Private::_end__int2x4, &ColladaParserAutoGen15Private::_preBegin__int2x4, &ColladaParserAutoGen15Private::_preEnd__int2x4, &ColladaParserAutoGen15Private::_freeAttributes__int2x4);
mElementToNamespaceMap[HASH_ELEMENT_INT2X4] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INT3X1] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__int3x1, &ColladaParserAutoGen15Private::_data__int3x1, &ColladaParserAutoGen15Private::_end__int3x1, &ColladaParserAutoGen15Private::_preBegin__int3x1, &ColladaParserAutoGen15Private::_preEnd__int3x1, &ColladaParserAutoGen15Private::_freeAttributes__int3x1);
mElementToNamespaceMap[HASH_ELEMENT_INT3X1] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INT3X2] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__int3x2, &ColladaParserAutoGen15Private::_data__int3x2, &ColladaParserAutoGen15Private::_end__int3x2, &ColladaParserAutoGen15Private::_preBegin__int3x2, &ColladaParserAutoGen15Private::_preEnd__int3x2, &ColladaParserAutoGen15Private::_freeAttributes__int3x2);
mElementToNamespaceMap[HASH_ELEMENT_INT3X2] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INT3X3] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__int3x3, &ColladaParserAutoGen15Private::_data__int3x3, &ColladaParserAutoGen15Private::_end__int3x3, &ColladaParserAutoGen15Private::_preBegin__int3x3, &ColladaParserAutoGen15Private::_preEnd__int3x3, &ColladaParserAutoGen15Private::_freeAttributes__int3x3);
mElementToNamespaceMap[HASH_ELEMENT_INT3X3] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INT3X4] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__int3x4, &ColladaParserAutoGen15Private::_data__int3x4, &ColladaParserAutoGen15Private::_end__int3x4, &ColladaParserAutoGen15Private::_preBegin__int3x4, &ColladaParserAutoGen15Private::_preEnd__int3x4, &ColladaParserAutoGen15Private::_freeAttributes__int3x4);
mElementToNamespaceMap[HASH_ELEMENT_INT3X4] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INT4X1] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__int4x1, &ColladaParserAutoGen15Private::_data__int4x1, &ColladaParserAutoGen15Private::_end__int4x1, &ColladaParserAutoGen15Private::_preBegin__int4x1, &ColladaParserAutoGen15Private::_preEnd__int4x1, &ColladaParserAutoGen15Private::_freeAttributes__int4x1);
mElementToNamespaceMap[HASH_ELEMENT_INT4X1] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INT4X2] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__int4x2, &ColladaParserAutoGen15Private::_data__int4x2, &ColladaParserAutoGen15Private::_end__int4x2, &ColladaParserAutoGen15Private::_preBegin__int4x2, &ColladaParserAutoGen15Private::_preEnd__int4x2, &ColladaParserAutoGen15Private::_freeAttributes__int4x2);
mElementToNamespaceMap[HASH_ELEMENT_INT4X2] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INT4X3] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__int4x3, &ColladaParserAutoGen15Private::_data__int4x3, &ColladaParserAutoGen15Private::_end__int4x3, &ColladaParserAutoGen15Private::_preBegin__int4x3, &ColladaParserAutoGen15Private::_preEnd__int4x3, &ColladaParserAutoGen15Private::_freeAttributes__int4x3);
mElementToNamespaceMap[HASH_ELEMENT_INT4X3] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INT4X4] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__int4x4, &ColladaParserAutoGen15Private::_data__int4x4, &ColladaParserAutoGen15Private::_end__int4x4, &ColladaParserAutoGen15Private::_preBegin__int4x4, &ColladaParserAutoGen15Private::_preEnd__int4x4, &ColladaParserAutoGen15Private::_freeAttributes__int4x4);
mElementToNamespaceMap[HASH_ELEMENT_INT4X4] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_HALF] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__half, &ColladaParserAutoGen15Private::_data__half, &ColladaParserAutoGen15Private::_end__half, &ColladaParserAutoGen15Private::_preBegin__half, &ColladaParserAutoGen15Private::_preEnd__half, &ColladaParserAutoGen15Private::_freeAttributes__half);
mElementToNamespaceMap[HASH_ELEMENT_HALF] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_HALF2] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__half2, &ColladaParserAutoGen15Private::_data__half2, &ColladaParserAutoGen15Private::_end__half2, &ColladaParserAutoGen15Private::_preBegin__half2, &ColladaParserAutoGen15Private::_preEnd__half2, &ColladaParserAutoGen15Private::_freeAttributes__half2);
mElementToNamespaceMap[HASH_ELEMENT_HALF2] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_HALF3] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__half3, &ColladaParserAutoGen15Private::_data__half3, &ColladaParserAutoGen15Private::_end__half3, &ColladaParserAutoGen15Private::_preBegin__half3, &ColladaParserAutoGen15Private::_preEnd__half3, &ColladaParserAutoGen15Private::_freeAttributes__half3);
mElementToNamespaceMap[HASH_ELEMENT_HALF3] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_HALF4] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__half4, &ColladaParserAutoGen15Private::_data__half4, &ColladaParserAutoGen15Private::_end__half4, &ColladaParserAutoGen15Private::_preBegin__half4, &ColladaParserAutoGen15Private::_preEnd__half4, &ColladaParserAutoGen15Private::_freeAttributes__half4);
mElementToNamespaceMap[HASH_ELEMENT_HALF4] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_HALF2X1] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__half2x1, &ColladaParserAutoGen15Private::_data__half2x1, &ColladaParserAutoGen15Private::_end__half2x1, &ColladaParserAutoGen15Private::_preBegin__half2x1, &ColladaParserAutoGen15Private::_preEnd__half2x1, &ColladaParserAutoGen15Private::_freeAttributes__half2x1);
mElementToNamespaceMap[HASH_ELEMENT_HALF2X1] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_HALF2X2] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__half2x2, &ColladaParserAutoGen15Private::_data__half2x2, &ColladaParserAutoGen15Private::_end__half2x2, &ColladaParserAutoGen15Private::_preBegin__half2x2, &ColladaParserAutoGen15Private::_preEnd__half2x2, &ColladaParserAutoGen15Private::_freeAttributes__half2x2);
mElementToNamespaceMap[HASH_ELEMENT_HALF2X2] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_HALF2X3] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__half2x3, &ColladaParserAutoGen15Private::_data__half2x3, &ColladaParserAutoGen15Private::_end__half2x3, &ColladaParserAutoGen15Private::_preBegin__half2x3, &ColladaParserAutoGen15Private::_preEnd__half2x3, &ColladaParserAutoGen15Private::_freeAttributes__half2x3);
mElementToNamespaceMap[HASH_ELEMENT_HALF2X3] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_HALF2X4] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__half2x4, &ColladaParserAutoGen15Private::_data__half2x4, &ColladaParserAutoGen15Private::_end__half2x4, &ColladaParserAutoGen15Private::_preBegin__half2x4, &ColladaParserAutoGen15Private::_preEnd__half2x4, &ColladaParserAutoGen15Private::_freeAttributes__half2x4);
mElementToNamespaceMap[HASH_ELEMENT_HALF2X4] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_HALF3X1] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__half3x1, &ColladaParserAutoGen15Private::_data__half3x1, &ColladaParserAutoGen15Private::_end__half3x1, &ColladaParserAutoGen15Private::_preBegin__half3x1, &ColladaParserAutoGen15Private::_preEnd__half3x1, &ColladaParserAutoGen15Private::_freeAttributes__half3x1);
mElementToNamespaceMap[HASH_ELEMENT_HALF3X1] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_HALF3X2] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__half3x2, &ColladaParserAutoGen15Private::_data__half3x2, &ColladaParserAutoGen15Private::_end__half3x2, &ColladaParserAutoGen15Private::_preBegin__half3x2, &ColladaParserAutoGen15Private::_preEnd__half3x2, &ColladaParserAutoGen15Private::_freeAttributes__half3x2);
mElementToNamespaceMap[HASH_ELEMENT_HALF3X2] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_HALF3X3] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__half3x3, &ColladaParserAutoGen15Private::_data__half3x3, &ColladaParserAutoGen15Private::_end__half3x3, &ColladaParserAutoGen15Private::_preBegin__half3x3, &ColladaParserAutoGen15Private::_preEnd__half3x3, &ColladaParserAutoGen15Private::_freeAttributes__half3x3);
mElementToNamespaceMap[HASH_ELEMENT_HALF3X3] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_HALF3X4] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__half3x4, &ColladaParserAutoGen15Private::_data__half3x4, &ColladaParserAutoGen15Private::_end__half3x4, &ColladaParserAutoGen15Private::_preBegin__half3x4, &ColladaParserAutoGen15Private::_preEnd__half3x4, &ColladaParserAutoGen15Private::_freeAttributes__half3x4);
mElementToNamespaceMap[HASH_ELEMENT_HALF3X4] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_HALF4X1] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__half4x1, &ColladaParserAutoGen15Private::_data__half4x1, &ColladaParserAutoGen15Private::_end__half4x1, &ColladaParserAutoGen15Private::_preBegin__half4x1, &ColladaParserAutoGen15Private::_preEnd__half4x1, &ColladaParserAutoGen15Private::_freeAttributes__half4x1);
mElementToNamespaceMap[HASH_ELEMENT_HALF4X1] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_HALF4X2] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__half4x2, &ColladaParserAutoGen15Private::_data__half4x2, &ColladaParserAutoGen15Private::_end__half4x2, &ColladaParserAutoGen15Private::_preBegin__half4x2, &ColladaParserAutoGen15Private::_preEnd__half4x2, &ColladaParserAutoGen15Private::_freeAttributes__half4x2);
mElementToNamespaceMap[HASH_ELEMENT_HALF4X2] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_HALF4X3] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__half4x3, &ColladaParserAutoGen15Private::_data__half4x3, &ColladaParserAutoGen15Private::_end__half4x3, &ColladaParserAutoGen15Private::_preBegin__half4x3, &ColladaParserAutoGen15Private::_preEnd__half4x3, &ColladaParserAutoGen15Private::_freeAttributes__half4x3);
mElementToNamespaceMap[HASH_ELEMENT_HALF4X3] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_HALF4X4] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__half4x4, &ColladaParserAutoGen15Private::_data__half4x4, &ColladaParserAutoGen15Private::_end__half4x4, &ColladaParserAutoGen15Private::_preBegin__half4x4, &ColladaParserAutoGen15Private::_preEnd__half4x4, &ColladaParserAutoGen15Private::_freeAttributes__half4x4);
mElementToNamespaceMap[HASH_ELEMENT_HALF4X4] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FIXED] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed, &ColladaParserAutoGen15Private::_data__fixed, &ColladaParserAutoGen15Private::_end__fixed, &ColladaParserAutoGen15Private::_preBegin__fixed, &ColladaParserAutoGen15Private::_preEnd__fixed, &ColladaParserAutoGen15Private::_freeAttributes__fixed);
mElementToNamespaceMap[HASH_ELEMENT_FIXED] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FIXED2] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed2, &ColladaParserAutoGen15Private::_data__fixed2, &ColladaParserAutoGen15Private::_end__fixed2, &ColladaParserAutoGen15Private::_preBegin__fixed2, &ColladaParserAutoGen15Private::_preEnd__fixed2, &ColladaParserAutoGen15Private::_freeAttributes__fixed2);
mElementToNamespaceMap[HASH_ELEMENT_FIXED2] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FIXED3] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed3, &ColladaParserAutoGen15Private::_data__fixed3, &ColladaParserAutoGen15Private::_end__fixed3, &ColladaParserAutoGen15Private::_preBegin__fixed3, &ColladaParserAutoGen15Private::_preEnd__fixed3, &ColladaParserAutoGen15Private::_freeAttributes__fixed3);
mElementToNamespaceMap[HASH_ELEMENT_FIXED3] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FIXED4] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed4, &ColladaParserAutoGen15Private::_data__fixed4, &ColladaParserAutoGen15Private::_end__fixed4, &ColladaParserAutoGen15Private::_preBegin__fixed4, &ColladaParserAutoGen15Private::_preEnd__fixed4, &ColladaParserAutoGen15Private::_freeAttributes__fixed4);
mElementToNamespaceMap[HASH_ELEMENT_FIXED4] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FIXED2X1] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed2x1, &ColladaParserAutoGen15Private::_data__fixed2x1, &ColladaParserAutoGen15Private::_end__fixed2x1, &ColladaParserAutoGen15Private::_preBegin__fixed2x1, &ColladaParserAutoGen15Private::_preEnd__fixed2x1, &ColladaParserAutoGen15Private::_freeAttributes__fixed2x1);
mElementToNamespaceMap[HASH_ELEMENT_FIXED2X1] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FIXED2X2] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed2x2, &ColladaParserAutoGen15Private::_data__fixed2x2, &ColladaParserAutoGen15Private::_end__fixed2x2, &ColladaParserAutoGen15Private::_preBegin__fixed2x2, &ColladaParserAutoGen15Private::_preEnd__fixed2x2, &ColladaParserAutoGen15Private::_freeAttributes__fixed2x2);
mElementToNamespaceMap[HASH_ELEMENT_FIXED2X2] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FIXED2X3] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed2x3, &ColladaParserAutoGen15Private::_data__fixed2x3, &ColladaParserAutoGen15Private::_end__fixed2x3, &ColladaParserAutoGen15Private::_preBegin__fixed2x3, &ColladaParserAutoGen15Private::_preEnd__fixed2x3, &ColladaParserAutoGen15Private::_freeAttributes__fixed2x3);
mElementToNamespaceMap[HASH_ELEMENT_FIXED2X3] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FIXED2X4] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed2x4, &ColladaParserAutoGen15Private::_data__fixed2x4, &ColladaParserAutoGen15Private::_end__fixed2x4, &ColladaParserAutoGen15Private::_preBegin__fixed2x4, &ColladaParserAutoGen15Private::_preEnd__fixed2x4, &ColladaParserAutoGen15Private::_freeAttributes__fixed2x4);
mElementToNamespaceMap[HASH_ELEMENT_FIXED2X4] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FIXED3X1] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed3x1, &ColladaParserAutoGen15Private::_data__fixed3x1, &ColladaParserAutoGen15Private::_end__fixed3x1, &ColladaParserAutoGen15Private::_preBegin__fixed3x1, &ColladaParserAutoGen15Private::_preEnd__fixed3x1, &ColladaParserAutoGen15Private::_freeAttributes__fixed3x1);
mElementToNamespaceMap[HASH_ELEMENT_FIXED3X1] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FIXED3X2] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed3x2, &ColladaParserAutoGen15Private::_data__fixed3x2, &ColladaParserAutoGen15Private::_end__fixed3x2, &ColladaParserAutoGen15Private::_preBegin__fixed3x2, &ColladaParserAutoGen15Private::_preEnd__fixed3x2, &ColladaParserAutoGen15Private::_freeAttributes__fixed3x2);
mElementToNamespaceMap[HASH_ELEMENT_FIXED3X2] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FIXED3X3] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed3x3, &ColladaParserAutoGen15Private::_data__fixed3x3, &ColladaParserAutoGen15Private::_end__fixed3x3, &ColladaParserAutoGen15Private::_preBegin__fixed3x3, &ColladaParserAutoGen15Private::_preEnd__fixed3x3, &ColladaParserAutoGen15Private::_freeAttributes__fixed3x3);
mElementToNamespaceMap[HASH_ELEMENT_FIXED3X3] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FIXED3X4] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed3x4, &ColladaParserAutoGen15Private::_data__fixed3x4, &ColladaParserAutoGen15Private::_end__fixed3x4, &ColladaParserAutoGen15Private::_preBegin__fixed3x4, &ColladaParserAutoGen15Private::_preEnd__fixed3x4, &ColladaParserAutoGen15Private::_freeAttributes__fixed3x4);
mElementToNamespaceMap[HASH_ELEMENT_FIXED3X4] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FIXED4X1] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed4x1, &ColladaParserAutoGen15Private::_data__fixed4x1, &ColladaParserAutoGen15Private::_end__fixed4x1, &ColladaParserAutoGen15Private::_preBegin__fixed4x1, &ColladaParserAutoGen15Private::_preEnd__fixed4x1, &ColladaParserAutoGen15Private::_freeAttributes__fixed4x1);
mElementToNamespaceMap[HASH_ELEMENT_FIXED4X1] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FIXED4X2] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed4x2, &ColladaParserAutoGen15Private::_data__fixed4x2, &ColladaParserAutoGen15Private::_end__fixed4x2, &ColladaParserAutoGen15Private::_preBegin__fixed4x2, &ColladaParserAutoGen15Private::_preEnd__fixed4x2, &ColladaParserAutoGen15Private::_freeAttributes__fixed4x2);
mElementToNamespaceMap[HASH_ELEMENT_FIXED4X2] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FIXED4X3] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed4x3, &ColladaParserAutoGen15Private::_data__fixed4x3, &ColladaParserAutoGen15Private::_end__fixed4x3, &ColladaParserAutoGen15Private::_preBegin__fixed4x3, &ColladaParserAutoGen15Private::_preEnd__fixed4x3, &ColladaParserAutoGen15Private::_freeAttributes__fixed4x3);
mElementToNamespaceMap[HASH_ELEMENT_FIXED4X3] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FIXED4X4] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed4x4, &ColladaParserAutoGen15Private::_data__fixed4x4, &ColladaParserAutoGen15Private::_end__fixed4x4, &ColladaParserAutoGen15Private::_preBegin__fixed4x4, &ColladaParserAutoGen15Private::_preEnd__fixed4x4, &ColladaParserAutoGen15Private::_freeAttributes__fixed4x4);
mElementToNamespaceMap[HASH_ELEMENT_FIXED4X4] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ARRAY____CG_ARRAY_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__array____cg_array_type, &ColladaParserAutoGen15Private::_data__array____cg_array_type, &ColladaParserAutoGen15Private::_end__array____cg_array_type, &ColladaParserAutoGen15Private::_preBegin__array____cg_array_type, &ColladaParserAutoGen15Private::_preEnd__array____cg_array_type, &ColladaParserAutoGen15Private::_freeAttributes__array____cg_array_type);
mElementToNamespaceMap[HASH_ELEMENT_ARRAY____CG_ARRAY_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_USERTYPE____CG_USER_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__usertype____cg_user_type, &ColladaParserAutoGen15Private::_data__usertype____cg_user_type, &ColladaParserAutoGen15Private::_end__usertype____cg_user_type, &ColladaParserAutoGen15Private::_preBegin__usertype____cg_user_type, &ColladaParserAutoGen15Private::_preEnd__usertype____cg_user_type, &ColladaParserAutoGen15Private::_freeAttributes__usertype____cg_user_type);
mElementToNamespaceMap[HASH_ELEMENT_USERTYPE____CG_USER_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SETPARAM____CG_SETPARAM_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__setparam____cg_setparam_type, &ColladaParserAutoGen15Private::_data__setparam____cg_setparam_type, &ColladaParserAutoGen15Private::_end__setparam____cg_setparam_type, &ColladaParserAutoGen15Private::_preBegin__setparam____cg_setparam_type, &ColladaParserAutoGen15Private::_preEnd__setparam____cg_setparam_type, &ColladaParserAutoGen15Private::_freeAttributes__setparam____cg_setparam_type);
mElementToNamespaceMap[HASH_ELEMENT_SETPARAM____CG_SETPARAM_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_CG_TYPE____TECHNIQUE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_cg_type____technique, &ColladaParserAutoGen15Private::_data__profile_cg_type____technique, &ColladaParserAutoGen15Private::_end__profile_cg_type____technique, &ColladaParserAutoGen15Private::_preBegin__profile_cg_type____technique, &ColladaParserAutoGen15Private::_preEnd__profile_cg_type____technique, &ColladaParserAutoGen15Private::_freeAttributes__profile_cg_type____technique);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_CG_TYPE____TECHNIQUE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PASS____CG_PASS_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__pass____cg_pass_type, &ColladaParserAutoGen15Private::_data__pass____cg_pass_type, &ColladaParserAutoGen15Private::_end__pass____cg_pass_type, &ColladaParserAutoGen15Private::_preBegin__pass____cg_pass_type, &ColladaParserAutoGen15Private::_preEnd__pass____cg_pass_type, &ColladaParserAutoGen15Private::_freeAttributes__pass____cg_pass_type);
mElementToNamespaceMap[HASH_ELEMENT_PASS____CG_PASS_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CG_PASS_TYPE____STATES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__cg_pass_type____states, &ColladaParserAutoGen15Private::_data__cg_pass_type____states, &ColladaParserAutoGen15Private::_end__cg_pass_type____states, &ColladaParserAutoGen15Private::_preBegin__cg_pass_type____states, &ColladaParserAutoGen15Private::_preEnd__cg_pass_type____states, &ColladaParserAutoGen15Private::_freeAttributes__cg_pass_type____states);
mElementToNamespaceMap[HASH_ELEMENT_CG_PASS_TYPE____STATES] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CG_PASS_TYPE____PROGRAM] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__cg_pass_type____program, &ColladaParserAutoGen15Private::_data__cg_pass_type____program, &ColladaParserAutoGen15Private::_end__cg_pass_type____program, &ColladaParserAutoGen15Private::_preBegin__cg_pass_type____program, &ColladaParserAutoGen15Private::_preEnd__cg_pass_type____program, &ColladaParserAutoGen15Private::_freeAttributes__cg_pass_type____program);
mElementToNamespaceMap[HASH_ELEMENT_CG_PASS_TYPE____PROGRAM] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_CG__TECHNIQUE__PASS__PROGRAM__SHADER] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_CG__technique__pass__program__shader, &ColladaParserAutoGen15Private::_data__profile_CG__technique__pass__program__shader, &ColladaParserAutoGen15Private::_end__profile_CG__technique__pass__program__shader, &ColladaParserAutoGen15Private::_preBegin__profile_CG__technique__pass__program__shader, &ColladaParserAutoGen15Private::_preEnd__profile_CG__technique__pass__program__shader, &ColladaParserAutoGen15Private::_freeAttributes__profile_CG__technique__pass__program__shader);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_CG__TECHNIQUE__PASS__PROGRAM__SHADER] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_CG__TECHNIQUE__PASS__PROGRAM__SHADER__SOURCES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_CG__technique__pass__program__shader__sources, &ColladaParserAutoGen15Private::_data__profile_CG__technique__pass__program__shader__sources, &ColladaParserAutoGen15Private::_end__profile_CG__technique__pass__program__shader__sources, &ColladaParserAutoGen15Private::_preBegin__profile_CG__technique__pass__program__shader__sources, &ColladaParserAutoGen15Private::_preEnd__profile_CG__technique__pass__program__shader__sources, &ColladaParserAutoGen15Private::_freeAttributes__profile_CG__technique__pass__program__shader__sources);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_CG__TECHNIQUE__PASS__PROGRAM__SHADER__SOURCES] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_CG__TECHNIQUE__PASS__PROGRAM__SHADER__BIND_UNIFORM] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_CG__technique__pass__program__shader__bind_uniform, &ColladaParserAutoGen15Private::_data__profile_CG__technique__pass__program__shader__bind_uniform, &ColladaParserAutoGen15Private::_end__profile_CG__technique__pass__program__shader__bind_uniform, &ColladaParserAutoGen15Private::_preBegin__profile_CG__technique__pass__program__shader__bind_uniform, &ColladaParserAutoGen15Private::_preEnd__profile_CG__technique__pass__program__shader__bind_uniform, &ColladaParserAutoGen15Private::_freeAttributes__profile_CG__technique__pass__program__shader__bind_uniform);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_CG__TECHNIQUE__PASS__PROGRAM__SHADER__BIND_UNIFORM] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_CG__TECHNIQUE__PASS__PROGRAM__SHADER__BIND_UNIFORM__PARAM] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_CG__technique__pass__program__shader__bind_uniform__param, &ColladaParserAutoGen15Private::_data__profile_CG__technique__pass__program__shader__bind_uniform__param, &ColladaParserAutoGen15Private::_end__profile_CG__technique__pass__program__shader__bind_uniform__param, &ColladaParserAutoGen15Private::_preBegin__profile_CG__technique__pass__program__shader__bind_uniform__param, &ColladaParserAutoGen15Private::_preEnd__profile_CG__technique__pass__program__shader__bind_uniform__param, &ColladaParserAutoGen15Private::_freeAttributes__profile_CG__technique__pass__program__shader__bind_uniform__param);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_CG__TECHNIQUE__PASS__PROGRAM__SHADER__BIND_UNIFORM__PARAM] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CG_PASS_TYPE____EVALUATE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__cg_pass_type____evaluate, &ColladaParserAutoGen15Private::_data__cg_pass_type____evaluate, &ColladaParserAutoGen15Private::_end__cg_pass_type____evaluate, &ColladaParserAutoGen15Private::_preBegin__cg_pass_type____evaluate, &ColladaParserAutoGen15Private::_preEnd__cg_pass_type____evaluate, &ColladaParserAutoGen15Private::_freeAttributes__cg_pass_type____evaluate);
mElementToNamespaceMap[HASH_ELEMENT_CG_PASS_TYPE____EVALUATE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES, &ColladaParserAutoGen15Private::_data__profile_GLES, &ColladaParserAutoGen15Private::_end__profile_GLES, &ColladaParserAutoGen15Private::_preBegin__profile_GLES, &ColladaParserAutoGen15Private::_preEnd__profile_GLES, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_NEWPARAM____GLES_NEWPARAM_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__newparam____gles_newparam_type, &ColladaParserAutoGen15Private::_data__newparam____gles_newparam_type, &ColladaParserAutoGen15Private::_end__newparam____gles_newparam_type, &ColladaParserAutoGen15Private::_preBegin__newparam____gles_newparam_type, &ColladaParserAutoGen15Private::_preEnd__newparam____gles_newparam_type, &ColladaParserAutoGen15Private::_freeAttributes__newparam____gles_newparam_type);
mElementToNamespaceMap[HASH_ELEMENT_NEWPARAM____GLES_NEWPARAM_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FLOAT1X1] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__float1x1, &ColladaParserAutoGen15Private::_data__float1x1, &ColladaParserAutoGen15Private::_end__float1x1, &ColladaParserAutoGen15Private::_preBegin__float1x1, &ColladaParserAutoGen15Private::_preEnd__float1x1, &ColladaParserAutoGen15Private::_freeAttributes__float1x1);
mElementToNamespaceMap[HASH_ELEMENT_FLOAT1X1] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FLOAT1X2] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__float1x2, &ColladaParserAutoGen15Private::_data__float1x2, &ColladaParserAutoGen15Private::_end__float1x2, &ColladaParserAutoGen15Private::_preBegin__float1x2, &ColladaParserAutoGen15Private::_preEnd__float1x2, &ColladaParserAutoGen15Private::_freeAttributes__float1x2);
mElementToNamespaceMap[HASH_ELEMENT_FLOAT1X2] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FLOAT1X3] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__float1x3, &ColladaParserAutoGen15Private::_data__float1x3, &ColladaParserAutoGen15Private::_end__float1x3, &ColladaParserAutoGen15Private::_preBegin__float1x3, &ColladaParserAutoGen15Private::_preEnd__float1x3, &ColladaParserAutoGen15Private::_freeAttributes__float1x3);
mElementToNamespaceMap[HASH_ELEMENT_FLOAT1X3] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FLOAT1X4] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__float1x4, &ColladaParserAutoGen15Private::_data__float1x4, &ColladaParserAutoGen15Private::_end__float1x4, &ColladaParserAutoGen15Private::_preBegin__float1x4, &ColladaParserAutoGen15Private::_preEnd__float1x4, &ColladaParserAutoGen15Private::_freeAttributes__float1x4);
mElementToNamespaceMap[HASH_ELEMENT_FLOAT1X4] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SAMPLER2D____GLES_SAMPLER_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler2D____gles_sampler_type, &ColladaParserAutoGen15Private::_data__sampler2D____gles_sampler_type, &ColladaParserAutoGen15Private::_end__sampler2D____gles_sampler_type, &ColladaParserAutoGen15Private::_preBegin__sampler2D____gles_sampler_type, &ColladaParserAutoGen15Private::_preEnd__sampler2D____gles_sampler_type, &ColladaParserAutoGen15Private::_freeAttributes__sampler2D____gles_sampler_type);
mElementToNamespaceMap[HASH_ELEMENT_SAMPLER2D____GLES_SAMPLER_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TEXCOORD] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__texcoord, &ColladaParserAutoGen15Private::_data__texcoord, &ColladaParserAutoGen15Private::_end__texcoord, &ColladaParserAutoGen15Private::_preBegin__texcoord, &ColladaParserAutoGen15Private::_preEnd__texcoord, &ColladaParserAutoGen15Private::_freeAttributes__texcoord);
mElementToNamespaceMap[HASH_ELEMENT_TEXCOORD] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_WRAP_S____GLES_SAMPLER_WRAP_ENUM] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__wrap_s____gles_sampler_wrap_enum, &ColladaParserAutoGen15Private::_data__wrap_s____gles_sampler_wrap_enum, &ColladaParserAutoGen15Private::_end__wrap_s____gles_sampler_wrap_enum, &ColladaParserAutoGen15Private::_preBegin__wrap_s____gles_sampler_wrap_enum, &ColladaParserAutoGen15Private::_preEnd__wrap_s____gles_sampler_wrap_enum, &ColladaParserAutoGen15Private::_freeAttributes__wrap_s____gles_sampler_wrap_enum);
mElementToNamespaceMap[HASH_ELEMENT_WRAP_S____GLES_SAMPLER_WRAP_ENUM] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_WRAP_T____GLES_SAMPLER_WRAP_ENUM] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__wrap_t____gles_sampler_wrap_enum, &ColladaParserAutoGen15Private::_data__wrap_t____gles_sampler_wrap_enum, &ColladaParserAutoGen15Private::_end__wrap_t____gles_sampler_wrap_enum, &ColladaParserAutoGen15Private::_preBegin__wrap_t____gles_sampler_wrap_enum, &ColladaParserAutoGen15Private::_preEnd__wrap_t____gles_sampler_wrap_enum, &ColladaParserAutoGen15Private::_freeAttributes__wrap_t____gles_sampler_wrap_enum);
mElementToNamespaceMap[HASH_ELEMENT_WRAP_T____GLES_SAMPLER_WRAP_ENUM] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ENUM____GLES_ENUMERATION_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__enum____gles_enumeration_type, &ColladaParserAutoGen15Private::_data__enum____gles_enumeration_type, &ColladaParserAutoGen15Private::_end__enum____gles_enumeration_type, &ColladaParserAutoGen15Private::_preBegin__enum____gles_enumeration_type, &ColladaParserAutoGen15Private::_preEnd__enum____gles_enumeration_type, &ColladaParserAutoGen15Private::_freeAttributes__enum____gles_enumeration_type);
mElementToNamespaceMap[HASH_ELEMENT_ENUM____GLES_ENUMERATION_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES_TYPE____TECHNIQUE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_gles_type____technique, &ColladaParserAutoGen15Private::_data__profile_gles_type____technique, &ColladaParserAutoGen15Private::_end__profile_gles_type____technique, &ColladaParserAutoGen15Private::_preBegin__profile_gles_type____technique, &ColladaParserAutoGen15Private::_preEnd__profile_gles_type____technique, &ColladaParserAutoGen15Private::_freeAttributes__profile_gles_type____technique);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES_TYPE____TECHNIQUE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__ALPHA_FUNC] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__alpha_func, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__alpha_func, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__alpha_func, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__alpha_func, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__alpha_func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__alpha_func);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__ALPHA_FUNC] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__ALPHA_FUNC__FUNC] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__alpha_func__func, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__alpha_func__func, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__alpha_func__func, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__alpha_func__func, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__alpha_func__func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__alpha_func__func);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__ALPHA_FUNC__FUNC] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__ALPHA_FUNC__VALUE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__alpha_func__value, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__alpha_func__value, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__alpha_func__value, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__alpha_func__value, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__alpha_func__value, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__alpha_func__value);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__ALPHA_FUNC__VALUE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__BLEND_FUNC] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__blend_func, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__blend_func, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__blend_func, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__blend_func, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__blend_func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__blend_func);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__BLEND_FUNC] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__BLEND_FUNC__SRC] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__blend_func__src, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__blend_func__src, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__blend_func__src, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__blend_func__src, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__blend_func__src, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__blend_func__src);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__BLEND_FUNC__SRC] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__BLEND_FUNC__DEST] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__blend_func__dest, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__blend_func__dest, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__blend_func__dest, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__blend_func__dest, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__blend_func__dest, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__blend_func__dest);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__BLEND_FUNC__DEST] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__CLIP_PLANE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__clip_plane, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__clip_plane, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__clip_plane, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__clip_plane, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__clip_plane, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__clip_plane);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__CLIP_PLANE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__COLOR_MASK] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__color_mask, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__color_mask, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__color_mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__color_mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__color_mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__color_mask);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__COLOR_MASK] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__CULL_FACE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__cull_face, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__cull_face, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__cull_face, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__cull_face, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__cull_face, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__cull_face);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__CULL_FACE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__DEPTH_FUNC] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__depth_func, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__depth_func, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__depth_func, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__depth_func, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__depth_func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__depth_func);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__DEPTH_FUNC] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__DEPTH_MASK] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__depth_mask, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__depth_mask, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__depth_mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__depth_mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__depth_mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__depth_mask);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__DEPTH_MASK] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__DEPTH_RANGE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__depth_range, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__depth_range, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__depth_range, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__depth_range, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__depth_range, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__depth_range);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__DEPTH_RANGE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__FOG_COLOR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__fog_color, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__fog_color, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__fog_color, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__fog_color, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__fog_color, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__fog_color);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__FOG_COLOR] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__FOG_DENSITY] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__fog_density, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__fog_density, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__fog_density, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__fog_density, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__fog_density, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__fog_density);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__FOG_DENSITY] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__FOG_MODE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__fog_mode, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__fog_mode, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__fog_mode, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__fog_mode, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__fog_mode, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__fog_mode);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__FOG_MODE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__FOG_START] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__fog_start, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__fog_start, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__fog_start, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__fog_start, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__fog_start, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__fog_start);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__FOG_START] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__FOG_END] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__fog_end, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__fog_end, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__fog_end, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__fog_end, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__fog_end, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__fog_end);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__FOG_END] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__FRONT_FACE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__front_face, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__front_face, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__front_face, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__front_face, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__front_face, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__front_face);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__FRONT_FACE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LOGIC_OP] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__logic_op, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__logic_op, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__logic_op, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__logic_op, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__logic_op, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__logic_op);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LOGIC_OP] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_AMBIENT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__light_ambient, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__light_ambient, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__light_ambient, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__light_ambient, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__light_ambient, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__light_ambient);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_AMBIENT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_DIFFUSE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__light_diffuse, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__light_diffuse, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__light_diffuse, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__light_diffuse, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__light_diffuse, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__light_diffuse);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_DIFFUSE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_SPECULAR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__light_specular, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__light_specular, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__light_specular, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__light_specular, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__light_specular, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__light_specular);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_SPECULAR] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_POSITION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__light_position, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__light_position, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__light_position, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__light_position, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__light_position, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__light_position);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_POSITION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_CONSTANT_ATTENUATION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__light_constant_attenuation, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__light_constant_attenuation, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__light_constant_attenuation, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__light_constant_attenuation, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__light_constant_attenuation, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__light_constant_attenuation);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_CONSTANT_ATTENUATION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_LINEAR_ATTENUATION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__light_linear_attenuation, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__light_linear_attenuation, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__light_linear_attenuation, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__light_linear_attenuation, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__light_linear_attenuation, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__light_linear_attenuation);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_LINEAR_ATTENUATION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_QUADRATIC_ATTENUATION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__light_quadratic_attenuation, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__light_quadratic_attenuation, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__light_quadratic_attenuation, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__light_quadratic_attenuation, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__light_quadratic_attenuation, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__light_quadratic_attenuation);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_QUADRATIC_ATTENUATION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_SPOT_CUTOFF] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__light_spot_cutoff, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__light_spot_cutoff, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__light_spot_cutoff, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__light_spot_cutoff, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__light_spot_cutoff, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__light_spot_cutoff);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_SPOT_CUTOFF] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_SPOT_DIRECTION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__light_spot_direction, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__light_spot_direction, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__light_spot_direction, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__light_spot_direction, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__light_spot_direction, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__light_spot_direction);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_SPOT_DIRECTION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_SPOT_EXPONENT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__light_spot_exponent, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__light_spot_exponent, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__light_spot_exponent, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__light_spot_exponent, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__light_spot_exponent, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__light_spot_exponent);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_SPOT_EXPONENT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_MODEL_AMBIENT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__light_model_ambient, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__light_model_ambient, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__light_model_ambient, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__light_model_ambient, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__light_model_ambient, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__light_model_ambient);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_MODEL_AMBIENT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LINE_WIDTH] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__line_width, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__line_width, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__line_width, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__line_width, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__line_width, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__line_width);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LINE_WIDTH] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__MATERIAL_AMBIENT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__material_ambient, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__material_ambient, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__material_ambient, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__material_ambient, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__material_ambient, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__material_ambient);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__MATERIAL_AMBIENT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__MATERIAL_DIFFUSE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__material_diffuse, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__material_diffuse, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__material_diffuse, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__material_diffuse, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__material_diffuse, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__material_diffuse);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__MATERIAL_DIFFUSE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__MATERIAL_EMISSION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__material_emission, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__material_emission, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__material_emission, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__material_emission, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__material_emission, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__material_emission);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__MATERIAL_EMISSION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__MATERIAL_SHININESS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__material_shininess, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__material_shininess, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__material_shininess, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__material_shininess, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__material_shininess, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__material_shininess);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__MATERIAL_SHININESS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__MATERIAL_SPECULAR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__material_specular, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__material_specular, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__material_specular, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__material_specular, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__material_specular, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__material_specular);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__MATERIAL_SPECULAR] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__MODEL_VIEW_MATRIX] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__model_view_matrix, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__model_view_matrix, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__model_view_matrix, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__model_view_matrix, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__model_view_matrix, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__model_view_matrix);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__MODEL_VIEW_MATRIX] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__POINT_DISTANCE_ATTENUATION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__point_distance_attenuation, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__point_distance_attenuation, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__point_distance_attenuation, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__point_distance_attenuation, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__point_distance_attenuation, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__point_distance_attenuation);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__POINT_DISTANCE_ATTENUATION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__POINT_FADE_THRESHOLD_SIZE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__point_fade_threshold_size, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__point_fade_threshold_size, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__point_fade_threshold_size, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__point_fade_threshold_size, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__point_fade_threshold_size, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__point_fade_threshold_size);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__POINT_FADE_THRESHOLD_SIZE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__POINT_SIZE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__point_size, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__point_size, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__point_size, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__point_size, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__point_size, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__point_size);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__POINT_SIZE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__POINT_SIZE_MIN] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__point_size_min, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__point_size_min, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__point_size_min, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__point_size_min, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__point_size_min, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__point_size_min);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__POINT_SIZE_MIN] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__POINT_SIZE_MAX] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__point_size_max, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__point_size_max, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__point_size_max, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__point_size_max, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__point_size_max, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__point_size_max);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__POINT_SIZE_MAX] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__POLYGON_OFFSET] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__polygon_offset, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__polygon_offset, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__polygon_offset, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__polygon_offset, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__polygon_offset, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__polygon_offset);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__POLYGON_OFFSET] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__PROJECTION_MATRIX] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__projection_matrix, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__projection_matrix, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__projection_matrix, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__projection_matrix, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__projection_matrix, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__projection_matrix);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__PROJECTION_MATRIX] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__SCISSOR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__scissor, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__scissor, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__scissor, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__scissor, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__scissor, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__scissor);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__SCISSOR] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__SHADE_MODEL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__shade_model, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__shade_model, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__shade_model, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__shade_model, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__shade_model, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__shade_model);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__SHADE_MODEL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__STENCIL_FUNC] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__stencil_func, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__stencil_func, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__stencil_func, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__stencil_func, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__stencil_func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__stencil_func);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__STENCIL_FUNC] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__STENCIL_FUNC__FUNC] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__stencil_func__func, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__stencil_func__func, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__stencil_func__func, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__stencil_func__func, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__stencil_func__func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__stencil_func__func);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__STENCIL_FUNC__FUNC] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__STENCIL_FUNC__REF] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__stencil_func__ref, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__stencil_func__ref, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__stencil_func__ref, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__stencil_func__ref, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__stencil_func__ref, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__stencil_func__ref);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__STENCIL_FUNC__REF] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__STENCIL_FUNC__MASK] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__stencil_func__mask, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__stencil_func__mask, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__stencil_func__mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__stencil_func__mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__stencil_func__mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__stencil_func__mask);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__STENCIL_FUNC__MASK] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__STENCIL_MASK] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__stencil_mask, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__stencil_mask, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__stencil_mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__stencil_mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__stencil_mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__stencil_mask);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__STENCIL_MASK] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__STENCIL_OP] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__stencil_op, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__stencil_op, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__stencil_op, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__stencil_op, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__stencil_op, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__stencil_op);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__STENCIL_OP] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__STENCIL_OP__FAIL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__stencil_op__fail, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__stencil_op__fail, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__stencil_op__fail, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__stencil_op__fail, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__stencil_op__fail, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__stencil_op__fail);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__STENCIL_OP__FAIL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__STENCIL_OP__ZFAIL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__stencil_op__zfail, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__stencil_op__zfail, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__stencil_op__zfail, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__stencil_op__zfail, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__stencil_op__zfail, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__stencil_op__zfail);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__STENCIL_OP__ZFAIL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__STENCIL_OP__ZPASS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__stencil_op__zpass, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__stencil_op__zpass, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__stencil_op__zpass, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__stencil_op__zpass, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__stencil_op__zpass, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__stencil_op__zpass);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__STENCIL_OP__ZPASS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TEXTURE_PIPELINE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__texture_pipeline, &ColladaParserAutoGen15Private::_data__texture_pipeline, &ColladaParserAutoGen15Private::_end__texture_pipeline, &ColladaParserAutoGen15Private::_preBegin__texture_pipeline, &ColladaParserAutoGen15Private::_preEnd__texture_pipeline, &ColladaParserAutoGen15Private::_freeAttributes__texture_pipeline);
mElementToNamespaceMap[HASH_ELEMENT_TEXTURE_PIPELINE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_VALUE____GLES_TEXTURE_PIPELINE_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__value____gles_texture_pipeline_type, &ColladaParserAutoGen15Private::_data__value____gles_texture_pipeline_type, &ColladaParserAutoGen15Private::_end__value____gles_texture_pipeline_type, &ColladaParserAutoGen15Private::_preBegin__value____gles_texture_pipeline_type, &ColladaParserAutoGen15Private::_preEnd__value____gles_texture_pipeline_type, &ColladaParserAutoGen15Private::_freeAttributes__value____gles_texture_pipeline_type);
mElementToNamespaceMap[HASH_ELEMENT_VALUE____GLES_TEXTURE_PIPELINE_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TEXCOMBINER] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__texcombiner, &ColladaParserAutoGen15Private::_data__texcombiner, &ColladaParserAutoGen15Private::_end__texcombiner, &ColladaParserAutoGen15Private::_preBegin__texcombiner, &ColladaParserAutoGen15Private::_preEnd__texcombiner, &ColladaParserAutoGen15Private::_freeAttributes__texcombiner);
mElementToNamespaceMap[HASH_ELEMENT_TEXCOMBINER] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CONSTANT____GLES_TEXTURE_CONSTANT_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__constant____gles_texture_constant_type, &ColladaParserAutoGen15Private::_data__constant____gles_texture_constant_type, &ColladaParserAutoGen15Private::_end__constant____gles_texture_constant_type, &ColladaParserAutoGen15Private::_preBegin__constant____gles_texture_constant_type, &ColladaParserAutoGen15Private::_preEnd__constant____gles_texture_constant_type, &ColladaParserAutoGen15Private::_freeAttributes__constant____gles_texture_constant_type);
mElementToNamespaceMap[HASH_ELEMENT_CONSTANT____GLES_TEXTURE_CONSTANT_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_RGB] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__RGB, &ColladaParserAutoGen15Private::_data__RGB, &ColladaParserAutoGen15Private::_end__RGB, &ColladaParserAutoGen15Private::_preBegin__RGB, &ColladaParserAutoGen15Private::_preEnd__RGB, &ColladaParserAutoGen15Private::_freeAttributes__RGB);
mElementToNamespaceMap[HASH_ELEMENT_RGB] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ARGUMENT____GLES_TEXCOMBINER_ARGUMENT_RGB_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__argument____gles_texcombiner_argument_rgb_type, &ColladaParserAutoGen15Private::_data__argument____gles_texcombiner_argument_rgb_type, &ColladaParserAutoGen15Private::_end__argument____gles_texcombiner_argument_rgb_type, &ColladaParserAutoGen15Private::_preBegin__argument____gles_texcombiner_argument_rgb_type, &ColladaParserAutoGen15Private::_preEnd__argument____gles_texcombiner_argument_rgb_type, &ColladaParserAutoGen15Private::_freeAttributes__argument____gles_texcombiner_argument_rgb_type);
mElementToNamespaceMap[HASH_ELEMENT_ARGUMENT____GLES_TEXCOMBINER_ARGUMENT_RGB_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ALPHA____GLES_TEXCOMBINER_COMMAND_ALPHA_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__alpha____gles_texcombiner_command_alpha_type, &ColladaParserAutoGen15Private::_data__alpha____gles_texcombiner_command_alpha_type, &ColladaParserAutoGen15Private::_end__alpha____gles_texcombiner_command_alpha_type, &ColladaParserAutoGen15Private::_preBegin__alpha____gles_texcombiner_command_alpha_type, &ColladaParserAutoGen15Private::_preEnd__alpha____gles_texcombiner_command_alpha_type, &ColladaParserAutoGen15Private::_freeAttributes__alpha____gles_texcombiner_command_alpha_type);
mElementToNamespaceMap[HASH_ELEMENT_ALPHA____GLES_TEXCOMBINER_COMMAND_ALPHA_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ARGUMENT____GLES_TEXCOMBINER_ARGUMENT_ALPHA_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__argument____gles_texcombiner_argument_alpha_type, &ColladaParserAutoGen15Private::_data__argument____gles_texcombiner_argument_alpha_type, &ColladaParserAutoGen15Private::_end__argument____gles_texcombiner_argument_alpha_type, &ColladaParserAutoGen15Private::_preBegin__argument____gles_texcombiner_argument_alpha_type, &ColladaParserAutoGen15Private::_preEnd__argument____gles_texcombiner_argument_alpha_type, &ColladaParserAutoGen15Private::_freeAttributes__argument____gles_texcombiner_argument_alpha_type);
mElementToNamespaceMap[HASH_ELEMENT_ARGUMENT____GLES_TEXCOMBINER_ARGUMENT_ALPHA_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TEXENV] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__texenv, &ColladaParserAutoGen15Private::_data__texenv, &ColladaParserAutoGen15Private::_end__texenv, &ColladaParserAutoGen15Private::_preBegin__texenv, &ColladaParserAutoGen15Private::_preEnd__texenv, &ColladaParserAutoGen15Private::_freeAttributes__texenv);
mElementToNamespaceMap[HASH_ELEMENT_TEXENV] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__ALPHA_TEST_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__alpha_test_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__alpha_test_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__alpha_test_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__alpha_test_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__alpha_test_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__alpha_test_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__ALPHA_TEST_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__BLEND_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__blend_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__blend_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__blend_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__blend_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__blend_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__blend_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__BLEND_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__CLIP_PLANE_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__clip_plane_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__clip_plane_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__clip_plane_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__clip_plane_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__clip_plane_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__clip_plane_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__CLIP_PLANE_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__COLOR_LOGIC_OP_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__color_logic_op_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__color_logic_op_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__color_logic_op_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__color_logic_op_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__color_logic_op_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__color_logic_op_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__COLOR_LOGIC_OP_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__COLOR_MATERIAL_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__color_material_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__color_material_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__color_material_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__color_material_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__color_material_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__color_material_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__COLOR_MATERIAL_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__CULL_FACE_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__cull_face_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__cull_face_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__cull_face_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__cull_face_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__cull_face_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__cull_face_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__CULL_FACE_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__DEPTH_TEST_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__depth_test_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__depth_test_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__depth_test_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__depth_test_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__depth_test_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__depth_test_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__DEPTH_TEST_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__DITHER_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__dither_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__dither_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__dither_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__dither_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__dither_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__dither_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__DITHER_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__FOG_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__fog_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__fog_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__fog_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__fog_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__fog_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__fog_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__FOG_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__light_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__light_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__light_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__light_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__light_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__light_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHTING_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__lighting_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__lighting_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__lighting_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__lighting_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__lighting_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__lighting_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHTING_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_MODEL_TWO_SIDE_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__light_model_two_side_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__light_model_two_side_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__light_model_two_side_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__light_model_two_side_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__light_model_two_side_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__light_model_two_side_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_MODEL_TWO_SIDE_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LINE_SMOOTH_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__line_smooth_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__line_smooth_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__line_smooth_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__line_smooth_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__line_smooth_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__line_smooth_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LINE_SMOOTH_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__MULTISAMPLE_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__multisample_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__multisample_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__multisample_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__multisample_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__multisample_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__multisample_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__MULTISAMPLE_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__NORMALIZE_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__normalize_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__normalize_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__normalize_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__normalize_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__normalize_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__normalize_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__NORMALIZE_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__POINT_SMOOTH_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__point_smooth_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__point_smooth_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__point_smooth_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__point_smooth_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__point_smooth_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__point_smooth_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__POINT_SMOOTH_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__POLYGON_OFFSET_FILL_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__polygon_offset_fill_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__polygon_offset_fill_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__polygon_offset_fill_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__polygon_offset_fill_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__polygon_offset_fill_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__polygon_offset_fill_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__POLYGON_OFFSET_FILL_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__RESCALE_NORMAL_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__rescale_normal_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__rescale_normal_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__rescale_normal_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__rescale_normal_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__rescale_normal_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__rescale_normal_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__RESCALE_NORMAL_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__SAMPLE_ALPHA_TO_COVERAGE_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__sample_alpha_to_coverage_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__sample_alpha_to_coverage_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__sample_alpha_to_coverage_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__sample_alpha_to_coverage_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__sample_alpha_to_coverage_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__sample_alpha_to_coverage_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__SAMPLE_ALPHA_TO_COVERAGE_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__SAMPLE_ALPHA_TO_ONE_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__sample_alpha_to_one_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__sample_alpha_to_one_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__sample_alpha_to_one_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__sample_alpha_to_one_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__sample_alpha_to_one_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__sample_alpha_to_one_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__SAMPLE_ALPHA_TO_ONE_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__SAMPLE_COVERAGE_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__sample_coverage_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__sample_coverage_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__sample_coverage_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__sample_coverage_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__sample_coverage_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__sample_coverage_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__SAMPLE_COVERAGE_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__SCISSOR_TEST_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__scissor_test_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__scissor_test_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__scissor_test_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__scissor_test_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__scissor_test_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__scissor_test_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__SCISSOR_TEST_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__STENCIL_TEST_ENABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__stencil_test_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__stencil_test_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__stencil_test_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__stencil_test_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__stencil_test_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__stencil_test_enable);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__STENCIL_TEST_ENABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__EVALUATE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__evaluate, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__evaluate, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__evaluate, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__evaluate, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__evaluate, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__evaluate);
mElementToNamespaceMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__EVALUATE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LIBRARY_FORCE_FIELDS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_force_fields, &ColladaParserAutoGen15Private::_data__library_force_fields, &ColladaParserAutoGen15Private::_end__library_force_fields, &ColladaParserAutoGen15Private::_preBegin__library_force_fields, &ColladaParserAutoGen15Private::_preEnd__library_force_fields, &ColladaParserAutoGen15Private::_freeAttributes__library_force_fields);
mElementToNamespaceMap[HASH_ELEMENT_LIBRARY_FORCE_FIELDS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FORCE_FIELD] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__force_field, &ColladaParserAutoGen15Private::_data__force_field, &ColladaParserAutoGen15Private::_end__force_field, &ColladaParserAutoGen15Private::_preBegin__force_field, &ColladaParserAutoGen15Private::_preEnd__force_field, &ColladaParserAutoGen15Private::_freeAttributes__force_field);
mElementToNamespaceMap[HASH_ELEMENT_FORCE_FIELD] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LIBRARY_IMAGES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_images, &ColladaParserAutoGen15Private::_data__library_images, &ColladaParserAutoGen15Private::_end__library_images, &ColladaParserAutoGen15Private::_preBegin__library_images, &ColladaParserAutoGen15Private::_preEnd__library_images, &ColladaParserAutoGen15Private::_freeAttributes__library_images);
mElementToNamespaceMap[HASH_ELEMENT_LIBRARY_IMAGES] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_IMAGE____IMAGE_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__image____image_type, &ColladaParserAutoGen15Private::_data__image____image_type, &ColladaParserAutoGen15Private::_end__image____image_type, &ColladaParserAutoGen15Private::_preBegin__image____image_type, &ColladaParserAutoGen15Private::_preEnd__image____image_type, &ColladaParserAutoGen15Private::_freeAttributes__image____image_type);
mElementToNamespaceMap[HASH_ELEMENT_IMAGE____IMAGE_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_RENDERABLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__renderable, &ColladaParserAutoGen15Private::_data__renderable, &ColladaParserAutoGen15Private::_end__renderable, &ColladaParserAutoGen15Private::_preBegin__renderable, &ColladaParserAutoGen15Private::_preEnd__renderable, &ColladaParserAutoGen15Private::_freeAttributes__renderable);
mElementToNamespaceMap[HASH_ELEMENT_RENDERABLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_IMAGE_TYPE____INIT_FROM] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__image_type____init_from, &ColladaParserAutoGen15Private::_data__image_type____init_from, &ColladaParserAutoGen15Private::_end__image_type____init_from, &ColladaParserAutoGen15Private::_preBegin__image_type____init_from, &ColladaParserAutoGen15Private::_preEnd__image_type____init_from, &ColladaParserAutoGen15Private::_freeAttributes__image_type____init_from);
mElementToNamespaceMap[HASH_ELEMENT_IMAGE_TYPE____INIT_FROM] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LIBRARY_IMAGES__IMAGE__INIT_FROM__HEX] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_images__image__init_from__hex, &ColladaParserAutoGen15Private::_data__library_images__image__init_from__hex, &ColladaParserAutoGen15Private::_end__library_images__image__init_from__hex, &ColladaParserAutoGen15Private::_preBegin__library_images__image__init_from__hex, &ColladaParserAutoGen15Private::_preEnd__library_images__image__init_from__hex, &ColladaParserAutoGen15Private::_freeAttributes__library_images__image__init_from__hex);
mElementToNamespaceMap[HASH_ELEMENT_LIBRARY_IMAGES__IMAGE__INIT_FROM__HEX] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CREATE_2D] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_2d, &ColladaParserAutoGen15Private::_data__create_2d, &ColladaParserAutoGen15Private::_end__create_2d, &ColladaParserAutoGen15Private::_preBegin__create_2d, &ColladaParserAutoGen15Private::_preEnd__create_2d, &ColladaParserAutoGen15Private::_freeAttributes__create_2d);
mElementToNamespaceMap[HASH_ELEMENT_CREATE_2D] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SIZE_EXACT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__size_exact, &ColladaParserAutoGen15Private::_data__size_exact, &ColladaParserAutoGen15Private::_end__size_exact, &ColladaParserAutoGen15Private::_preBegin__size_exact, &ColladaParserAutoGen15Private::_preEnd__size_exact, &ColladaParserAutoGen15Private::_freeAttributes__size_exact);
mElementToNamespaceMap[HASH_ELEMENT_SIZE_EXACT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SIZE_RATIO] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__size_ratio, &ColladaParserAutoGen15Private::_data__size_ratio, &ColladaParserAutoGen15Private::_end__size_ratio, &ColladaParserAutoGen15Private::_preBegin__size_ratio, &ColladaParserAutoGen15Private::_preEnd__size_ratio, &ColladaParserAutoGen15Private::_freeAttributes__size_ratio);
mElementToNamespaceMap[HASH_ELEMENT_SIZE_RATIO] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_MIPS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__mips, &ColladaParserAutoGen15Private::_data__mips, &ColladaParserAutoGen15Private::_end__mips, &ColladaParserAutoGen15Private::_preBegin__mips, &ColladaParserAutoGen15Private::_preEnd__mips, &ColladaParserAutoGen15Private::_freeAttributes__mips);
mElementToNamespaceMap[HASH_ELEMENT_MIPS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_UNNORMALIZED] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__unnormalized, &ColladaParserAutoGen15Private::_data__unnormalized, &ColladaParserAutoGen15Private::_end__unnormalized, &ColladaParserAutoGen15Private::_preBegin__unnormalized, &ColladaParserAutoGen15Private::_preEnd__unnormalized, &ColladaParserAutoGen15Private::_freeAttributes__unnormalized);
mElementToNamespaceMap[HASH_ELEMENT_UNNORMALIZED] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CREATE_2D__ARRAY] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_2d__array, &ColladaParserAutoGen15Private::_data__create_2d__array, &ColladaParserAutoGen15Private::_end__create_2d__array, &ColladaParserAutoGen15Private::_preBegin__create_2d__array, &ColladaParserAutoGen15Private::_preEnd__create_2d__array, &ColladaParserAutoGen15Private::_freeAttributes__create_2d__array);
mElementToNamespaceMap[HASH_ELEMENT_CREATE_2D__ARRAY] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CREATE_2D__FORMAT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_2d__format, &ColladaParserAutoGen15Private::_data__create_2d__format, &ColladaParserAutoGen15Private::_end__create_2d__format, &ColladaParserAutoGen15Private::_preBegin__create_2d__format, &ColladaParserAutoGen15Private::_preEnd__create_2d__format, &ColladaParserAutoGen15Private::_freeAttributes__create_2d__format);
mElementToNamespaceMap[HASH_ELEMENT_CREATE_2D__FORMAT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CREATE_2D__FORMAT__HINT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_2d__format__hint, &ColladaParserAutoGen15Private::_data__create_2d__format__hint, &ColladaParserAutoGen15Private::_end__create_2d__format__hint, &ColladaParserAutoGen15Private::_preBegin__create_2d__format__hint, &ColladaParserAutoGen15Private::_preEnd__create_2d__format__hint, &ColladaParserAutoGen15Private::_freeAttributes__create_2d__format__hint);
mElementToNamespaceMap[HASH_ELEMENT_CREATE_2D__FORMAT__HINT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_EXACT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__exact, &ColladaParserAutoGen15Private::_data__exact, &ColladaParserAutoGen15Private::_end__exact, &ColladaParserAutoGen15Private::_preBegin__exact, &ColladaParserAutoGen15Private::_preEnd__exact, &ColladaParserAutoGen15Private::_freeAttributes__exact);
mElementToNamespaceMap[HASH_ELEMENT_EXACT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CREATE_2D__INIT_FROM] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_2d__init_from, &ColladaParserAutoGen15Private::_data__create_2d__init_from, &ColladaParserAutoGen15Private::_end__create_2d__init_from, &ColladaParserAutoGen15Private::_preBegin__create_2d__init_from, &ColladaParserAutoGen15Private::_preEnd__create_2d__init_from, &ColladaParserAutoGen15Private::_freeAttributes__create_2d__init_from);
mElementToNamespaceMap[HASH_ELEMENT_CREATE_2D__INIT_FROM] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CREATE_3D] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_3d, &ColladaParserAutoGen15Private::_data__create_3d, &ColladaParserAutoGen15Private::_end__create_3d, &ColladaParserAutoGen15Private::_preBegin__create_3d, &ColladaParserAutoGen15Private::_preEnd__create_3d, &ColladaParserAutoGen15Private::_freeAttributes__create_3d);
mElementToNamespaceMap[HASH_ELEMENT_CREATE_3D] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CREATE_3D__SIZE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_3d__size, &ColladaParserAutoGen15Private::_data__create_3d__size, &ColladaParserAutoGen15Private::_end__create_3d__size, &ColladaParserAutoGen15Private::_preBegin__create_3d__size, &ColladaParserAutoGen15Private::_preEnd__create_3d__size, &ColladaParserAutoGen15Private::_freeAttributes__create_3d__size);
mElementToNamespaceMap[HASH_ELEMENT_CREATE_3D__SIZE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CREATE_3D__ARRAY] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_3d__array, &ColladaParserAutoGen15Private::_data__create_3d__array, &ColladaParserAutoGen15Private::_end__create_3d__array, &ColladaParserAutoGen15Private::_preBegin__create_3d__array, &ColladaParserAutoGen15Private::_preEnd__create_3d__array, &ColladaParserAutoGen15Private::_freeAttributes__create_3d__array);
mElementToNamespaceMap[HASH_ELEMENT_CREATE_3D__ARRAY] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CREATE_3D__FORMAT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_3d__format, &ColladaParserAutoGen15Private::_data__create_3d__format, &ColladaParserAutoGen15Private::_end__create_3d__format, &ColladaParserAutoGen15Private::_preBegin__create_3d__format, &ColladaParserAutoGen15Private::_preEnd__create_3d__format, &ColladaParserAutoGen15Private::_freeAttributes__create_3d__format);
mElementToNamespaceMap[HASH_ELEMENT_CREATE_3D__FORMAT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CREATE_3D__FORMAT__HINT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_3d__format__hint, &ColladaParserAutoGen15Private::_data__create_3d__format__hint, &ColladaParserAutoGen15Private::_end__create_3d__format__hint, &ColladaParserAutoGen15Private::_preBegin__create_3d__format__hint, &ColladaParserAutoGen15Private::_preEnd__create_3d__format__hint, &ColladaParserAutoGen15Private::_freeAttributes__create_3d__format__hint);
mElementToNamespaceMap[HASH_ELEMENT_CREATE_3D__FORMAT__HINT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CREATE_3D__INIT_FROM] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_3d__init_from, &ColladaParserAutoGen15Private::_data__create_3d__init_from, &ColladaParserAutoGen15Private::_end__create_3d__init_from, &ColladaParserAutoGen15Private::_preBegin__create_3d__init_from, &ColladaParserAutoGen15Private::_preEnd__create_3d__init_from, &ColladaParserAutoGen15Private::_freeAttributes__create_3d__init_from);
mElementToNamespaceMap[HASH_ELEMENT_CREATE_3D__INIT_FROM] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CREATE_CUBE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_cube, &ColladaParserAutoGen15Private::_data__create_cube, &ColladaParserAutoGen15Private::_end__create_cube, &ColladaParserAutoGen15Private::_preBegin__create_cube, &ColladaParserAutoGen15Private::_preEnd__create_cube, &ColladaParserAutoGen15Private::_freeAttributes__create_cube);
mElementToNamespaceMap[HASH_ELEMENT_CREATE_CUBE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CREATE_CUBE__SIZE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_cube__size, &ColladaParserAutoGen15Private::_data__create_cube__size, &ColladaParserAutoGen15Private::_end__create_cube__size, &ColladaParserAutoGen15Private::_preBegin__create_cube__size, &ColladaParserAutoGen15Private::_preEnd__create_cube__size, &ColladaParserAutoGen15Private::_freeAttributes__create_cube__size);
mElementToNamespaceMap[HASH_ELEMENT_CREATE_CUBE__SIZE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CREATE_CUBE__ARRAY] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_cube__array, &ColladaParserAutoGen15Private::_data__create_cube__array, &ColladaParserAutoGen15Private::_end__create_cube__array, &ColladaParserAutoGen15Private::_preBegin__create_cube__array, &ColladaParserAutoGen15Private::_preEnd__create_cube__array, &ColladaParserAutoGen15Private::_freeAttributes__create_cube__array);
mElementToNamespaceMap[HASH_ELEMENT_CREATE_CUBE__ARRAY] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CREATE_CUBE__FORMAT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_cube__format, &ColladaParserAutoGen15Private::_data__create_cube__format, &ColladaParserAutoGen15Private::_end__create_cube__format, &ColladaParserAutoGen15Private::_preBegin__create_cube__format, &ColladaParserAutoGen15Private::_preEnd__create_cube__format, &ColladaParserAutoGen15Private::_freeAttributes__create_cube__format);
mElementToNamespaceMap[HASH_ELEMENT_CREATE_CUBE__FORMAT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CREATE_CUBE__FORMAT__HINT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_cube__format__hint, &ColladaParserAutoGen15Private::_data__create_cube__format__hint, &ColladaParserAutoGen15Private::_end__create_cube__format__hint, &ColladaParserAutoGen15Private::_preBegin__create_cube__format__hint, &ColladaParserAutoGen15Private::_preEnd__create_cube__format__hint, &ColladaParserAutoGen15Private::_freeAttributes__create_cube__format__hint);
mElementToNamespaceMap[HASH_ELEMENT_CREATE_CUBE__FORMAT__HINT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CREATE_CUBE__INIT_FROM] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_cube__init_from, &ColladaParserAutoGen15Private::_data__create_cube__init_from, &ColladaParserAutoGen15Private::_end__create_cube__init_from, &ColladaParserAutoGen15Private::_preBegin__create_cube__init_from, &ColladaParserAutoGen15Private::_preEnd__create_cube__init_from, &ColladaParserAutoGen15Private::_freeAttributes__create_cube__init_from);
mElementToNamespaceMap[HASH_ELEMENT_CREATE_CUBE__INIT_FROM] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LIBRARY_LIGHTS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_lights, &ColladaParserAutoGen15Private::_data__library_lights, &ColladaParserAutoGen15Private::_end__library_lights, &ColladaParserAutoGen15Private::_preBegin__library_lights, &ColladaParserAutoGen15Private::_preEnd__library_lights, &ColladaParserAutoGen15Private::_freeAttributes__library_lights);
mElementToNamespaceMap[HASH_ELEMENT_LIBRARY_LIGHTS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LIGHT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__light, &ColladaParserAutoGen15Private::_data__light, &ColladaParserAutoGen15Private::_end__light, &ColladaParserAutoGen15Private::_preBegin__light, &ColladaParserAutoGen15Private::_preEnd__light, &ColladaParserAutoGen15Private::_freeAttributes__light);
mElementToNamespaceMap[HASH_ELEMENT_LIGHT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LIGHT_TYPE____TECHNIQUE_COMMON] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__light_type____technique_common, &ColladaParserAutoGen15Private::_data__light_type____technique_common, &ColladaParserAutoGen15Private::_end__light_type____technique_common, &ColladaParserAutoGen15Private::_preBegin__light_type____technique_common, &ColladaParserAutoGen15Private::_preEnd__light_type____technique_common, &ColladaParserAutoGen15Private::_freeAttributes__light_type____technique_common);
mElementToNamespaceMap[HASH_ELEMENT_LIGHT_TYPE____TECHNIQUE_COMMON] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LIGHT__TECHNIQUE_COMMON__AMBIENT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__light__technique_common__ambient, &ColladaParserAutoGen15Private::_data__light__technique_common__ambient, &ColladaParserAutoGen15Private::_end__light__technique_common__ambient, &ColladaParserAutoGen15Private::_preBegin__light__technique_common__ambient, &ColladaParserAutoGen15Private::_preEnd__light__technique_common__ambient, &ColladaParserAutoGen15Private::_freeAttributes__light__technique_common__ambient);
mElementToNamespaceMap[HASH_ELEMENT_LIGHT__TECHNIQUE_COMMON__AMBIENT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_COLOR____TARGETABLE_FLOAT3_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__color____targetable_float3_type, &ColladaParserAutoGen15Private::_data__color____targetable_float3_type, &ColladaParserAutoGen15Private::_end__color____targetable_float3_type, &ColladaParserAutoGen15Private::_preBegin__color____targetable_float3_type, &ColladaParserAutoGen15Private::_preEnd__color____targetable_float3_type, &ColladaParserAutoGen15Private::_freeAttributes__color____targetable_float3_type);
mElementToNamespaceMap[HASH_ELEMENT_COLOR____TARGETABLE_FLOAT3_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_DIRECTIONAL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__directional, &ColladaParserAutoGen15Private::_data__directional, &ColladaParserAutoGen15Private::_end__directional, &ColladaParserAutoGen15Private::_preBegin__directional, &ColladaParserAutoGen15Private::_preEnd__directional, &ColladaParserAutoGen15Private::_freeAttributes__directional);
mElementToNamespaceMap[HASH_ELEMENT_DIRECTIONAL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_POINT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__point, &ColladaParserAutoGen15Private::_data__point, &ColladaParserAutoGen15Private::_end__point, &ColladaParserAutoGen15Private::_preBegin__point, &ColladaParserAutoGen15Private::_preEnd__point, &ColladaParserAutoGen15Private::_freeAttributes__point);
mElementToNamespaceMap[HASH_ELEMENT_POINT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CONSTANT_ATTENUATION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__constant_attenuation, &ColladaParserAutoGen15Private::_data__constant_attenuation, &ColladaParserAutoGen15Private::_end__constant_attenuation, &ColladaParserAutoGen15Private::_preBegin__constant_attenuation, &ColladaParserAutoGen15Private::_preEnd__constant_attenuation, &ColladaParserAutoGen15Private::_freeAttributes__constant_attenuation);
mElementToNamespaceMap[HASH_ELEMENT_CONSTANT_ATTENUATION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LINEAR_ATTENUATION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__linear_attenuation, &ColladaParserAutoGen15Private::_data__linear_attenuation, &ColladaParserAutoGen15Private::_end__linear_attenuation, &ColladaParserAutoGen15Private::_preBegin__linear_attenuation, &ColladaParserAutoGen15Private::_preEnd__linear_attenuation, &ColladaParserAutoGen15Private::_freeAttributes__linear_attenuation);
mElementToNamespaceMap[HASH_ELEMENT_LINEAR_ATTENUATION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_QUADRATIC_ATTENUATION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__quadratic_attenuation, &ColladaParserAutoGen15Private::_data__quadratic_attenuation, &ColladaParserAutoGen15Private::_end__quadratic_attenuation, &ColladaParserAutoGen15Private::_preBegin__quadratic_attenuation, &ColladaParserAutoGen15Private::_preEnd__quadratic_attenuation, &ColladaParserAutoGen15Private::_freeAttributes__quadratic_attenuation);
mElementToNamespaceMap[HASH_ELEMENT_QUADRATIC_ATTENUATION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SPOT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__spot, &ColladaParserAutoGen15Private::_data__spot, &ColladaParserAutoGen15Private::_end__spot, &ColladaParserAutoGen15Private::_preBegin__spot, &ColladaParserAutoGen15Private::_preEnd__spot, &ColladaParserAutoGen15Private::_freeAttributes__spot);
mElementToNamespaceMap[HASH_ELEMENT_SPOT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FALLOFF_ANGLE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__falloff_angle, &ColladaParserAutoGen15Private::_data__falloff_angle, &ColladaParserAutoGen15Private::_end__falloff_angle, &ColladaParserAutoGen15Private::_preBegin__falloff_angle, &ColladaParserAutoGen15Private::_preEnd__falloff_angle, &ColladaParserAutoGen15Private::_freeAttributes__falloff_angle);
mElementToNamespaceMap[HASH_ELEMENT_FALLOFF_ANGLE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FALLOFF_EXPONENT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__falloff_exponent, &ColladaParserAutoGen15Private::_data__falloff_exponent, &ColladaParserAutoGen15Private::_end__falloff_exponent, &ColladaParserAutoGen15Private::_preBegin__falloff_exponent, &ColladaParserAutoGen15Private::_preEnd__falloff_exponent, &ColladaParserAutoGen15Private::_freeAttributes__falloff_exponent);
mElementToNamespaceMap[HASH_ELEMENT_FALLOFF_EXPONENT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LIBRARY_MATERIALS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_materials, &ColladaParserAutoGen15Private::_data__library_materials, &ColladaParserAutoGen15Private::_end__library_materials, &ColladaParserAutoGen15Private::_preBegin__library_materials, &ColladaParserAutoGen15Private::_preEnd__library_materials, &ColladaParserAutoGen15Private::_freeAttributes__library_materials);
mElementToNamespaceMap[HASH_ELEMENT_LIBRARY_MATERIALS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_MATERIAL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__material, &ColladaParserAutoGen15Private::_data__material, &ColladaParserAutoGen15Private::_end__material, &ColladaParserAutoGen15Private::_preBegin__material, &ColladaParserAutoGen15Private::_preEnd__material, &ColladaParserAutoGen15Private::_freeAttributes__material);
mElementToNamespaceMap[HASH_ELEMENT_MATERIAL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INSTANCE_EFFECT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_effect, &ColladaParserAutoGen15Private::_data__instance_effect, &ColladaParserAutoGen15Private::_end__instance_effect, &ColladaParserAutoGen15Private::_preBegin__instance_effect, &ColladaParserAutoGen15Private::_preEnd__instance_effect, &ColladaParserAutoGen15Private::_freeAttributes__instance_effect);
mElementToNamespaceMap[HASH_ELEMENT_INSTANCE_EFFECT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TECHNIQUE_HINT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique_hint, &ColladaParserAutoGen15Private::_data__technique_hint, &ColladaParserAutoGen15Private::_end__technique_hint, &ColladaParserAutoGen15Private::_preBegin__technique_hint, &ColladaParserAutoGen15Private::_preEnd__technique_hint, &ColladaParserAutoGen15Private::_freeAttributes__technique_hint);
mElementToNamespaceMap[HASH_ELEMENT_TECHNIQUE_HINT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INSTANCE_EFFECT_TYPE____SETPARAM] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_effect_type____setparam, &ColladaParserAutoGen15Private::_data__instance_effect_type____setparam, &ColladaParserAutoGen15Private::_end__instance_effect_type____setparam, &ColladaParserAutoGen15Private::_preBegin__instance_effect_type____setparam, &ColladaParserAutoGen15Private::_preEnd__instance_effect_type____setparam, &ColladaParserAutoGen15Private::_freeAttributes__instance_effect_type____setparam);
mElementToNamespaceMap[HASH_ELEMENT_INSTANCE_EFFECT_TYPE____SETPARAM] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SAMPLER_IMAGE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler_image, &ColladaParserAutoGen15Private::_data__sampler_image, &ColladaParserAutoGen15Private::_end__sampler_image, &ColladaParserAutoGen15Private::_preBegin__sampler_image, &ColladaParserAutoGen15Private::_preEnd__sampler_image, &ColladaParserAutoGen15Private::_freeAttributes__sampler_image);
mElementToNamespaceMap[HASH_ELEMENT_SAMPLER_IMAGE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SAMPLER_STATES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler_states, &ColladaParserAutoGen15Private::_data__sampler_states, &ColladaParserAutoGen15Private::_end__sampler_states, &ColladaParserAutoGen15Private::_preBegin__sampler_states, &ColladaParserAutoGen15Private::_preEnd__sampler_states, &ColladaParserAutoGen15Private::_freeAttributes__sampler_states);
mElementToNamespaceMap[HASH_ELEMENT_SAMPLER_STATES] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LIBRARY_NODES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_nodes, &ColladaParserAutoGen15Private::_data__library_nodes, &ColladaParserAutoGen15Private::_end__library_nodes, &ColladaParserAutoGen15Private::_preBegin__library_nodes, &ColladaParserAutoGen15Private::_preEnd__library_nodes, &ColladaParserAutoGen15Private::_freeAttributes__library_nodes);
mElementToNamespaceMap[HASH_ELEMENT_LIBRARY_NODES] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_NODE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__node, &ColladaParserAutoGen15Private::_data__node, &ColladaParserAutoGen15Private::_end__node, &ColladaParserAutoGen15Private::_preBegin__node, &ColladaParserAutoGen15Private::_preEnd__node, &ColladaParserAutoGen15Private::_freeAttributes__node);
mElementToNamespaceMap[HASH_ELEMENT_NODE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LOOKAT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__lookat, &ColladaParserAutoGen15Private::_data__lookat, &ColladaParserAutoGen15Private::_end__lookat, &ColladaParserAutoGen15Private::_preBegin__lookat, &ColladaParserAutoGen15Private::_preEnd__lookat, &ColladaParserAutoGen15Private::_freeAttributes__lookat);
mElementToNamespaceMap[HASH_ELEMENT_LOOKAT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_MATRIX____MATRIX_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__matrix____matrix_type, &ColladaParserAutoGen15Private::_data__matrix____matrix_type, &ColladaParserAutoGen15Private::_end__matrix____matrix_type, &ColladaParserAutoGen15Private::_preBegin__matrix____matrix_type, &ColladaParserAutoGen15Private::_preEnd__matrix____matrix_type, &ColladaParserAutoGen15Private::_freeAttributes__matrix____matrix_type);
mElementToNamespaceMap[HASH_ELEMENT_MATRIX____MATRIX_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ROTATE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__rotate, &ColladaParserAutoGen15Private::_data__rotate, &ColladaParserAutoGen15Private::_end__rotate, &ColladaParserAutoGen15Private::_preBegin__rotate, &ColladaParserAutoGen15Private::_preEnd__rotate, &ColladaParserAutoGen15Private::_freeAttributes__rotate);
mElementToNamespaceMap[HASH_ELEMENT_ROTATE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SCALE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__scale, &ColladaParserAutoGen15Private::_data__scale, &ColladaParserAutoGen15Private::_end__scale, &ColladaParserAutoGen15Private::_preBegin__scale, &ColladaParserAutoGen15Private::_preEnd__scale, &ColladaParserAutoGen15Private::_freeAttributes__scale);
mElementToNamespaceMap[HASH_ELEMENT_SCALE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SKEW] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__skew, &ColladaParserAutoGen15Private::_data__skew, &ColladaParserAutoGen15Private::_end__skew, &ColladaParserAutoGen15Private::_preBegin__skew, &ColladaParserAutoGen15Private::_preEnd__skew, &ColladaParserAutoGen15Private::_freeAttributes__skew);
mElementToNamespaceMap[HASH_ELEMENT_SKEW] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TRANSLATE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__translate, &ColladaParserAutoGen15Private::_data__translate, &ColladaParserAutoGen15Private::_end__translate, &ColladaParserAutoGen15Private::_preBegin__translate, &ColladaParserAutoGen15Private::_preEnd__translate, &ColladaParserAutoGen15Private::_freeAttributes__translate);
mElementToNamespaceMap[HASH_ELEMENT_TRANSLATE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INSTANCE_CAMERA] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_camera, &ColladaParserAutoGen15Private::_data__instance_camera, &ColladaParserAutoGen15Private::_end__instance_camera, &ColladaParserAutoGen15Private::_preBegin__instance_camera, &ColladaParserAutoGen15Private::_preEnd__instance_camera, &ColladaParserAutoGen15Private::_freeAttributes__instance_camera);
mElementToNamespaceMap[HASH_ELEMENT_INSTANCE_CAMERA] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INSTANCE_CONTROLLER] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_controller, &ColladaParserAutoGen15Private::_data__instance_controller, &ColladaParserAutoGen15Private::_end__instance_controller, &ColladaParserAutoGen15Private::_preBegin__instance_controller, &ColladaParserAutoGen15Private::_preEnd__instance_controller, &ColladaParserAutoGen15Private::_freeAttributes__instance_controller);
mElementToNamespaceMap[HASH_ELEMENT_INSTANCE_CONTROLLER] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SKELETON] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__skeleton, &ColladaParserAutoGen15Private::_data__skeleton, &ColladaParserAutoGen15Private::_end__skeleton, &ColladaParserAutoGen15Private::_preBegin__skeleton, &ColladaParserAutoGen15Private::_preEnd__skeleton, &ColladaParserAutoGen15Private::_freeAttributes__skeleton);
mElementToNamespaceMap[HASH_ELEMENT_SKELETON] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BIND_MATERIAL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__bind_material, &ColladaParserAutoGen15Private::_data__bind_material, &ColladaParserAutoGen15Private::_end__bind_material, &ColladaParserAutoGen15Private::_preBegin__bind_material, &ColladaParserAutoGen15Private::_preEnd__bind_material, &ColladaParserAutoGen15Private::_freeAttributes__bind_material);
mElementToNamespaceMap[HASH_ELEMENT_BIND_MATERIAL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BIND_MATERIAL_TYPE____TECHNIQUE_COMMON] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__bind_material_type____technique_common, &ColladaParserAutoGen15Private::_data__bind_material_type____technique_common, &ColladaParserAutoGen15Private::_end__bind_material_type____technique_common, &ColladaParserAutoGen15Private::_preBegin__bind_material_type____technique_common, &ColladaParserAutoGen15Private::_preEnd__bind_material_type____technique_common, &ColladaParserAutoGen15Private::_freeAttributes__bind_material_type____technique_common);
mElementToNamespaceMap[HASH_ELEMENT_BIND_MATERIAL_TYPE____TECHNIQUE_COMMON] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INSTANCE_MATERIAL____INSTANCE_MATERIAL_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_material____instance_material_type, &ColladaParserAutoGen15Private::_data__instance_material____instance_material_type, &ColladaParserAutoGen15Private::_end__instance_material____instance_material_type, &ColladaParserAutoGen15Private::_preBegin__instance_material____instance_material_type, &ColladaParserAutoGen15Private::_preEnd__instance_material____instance_material_type, &ColladaParserAutoGen15Private::_freeAttributes__instance_material____instance_material_type);
mElementToNamespaceMap[HASH_ELEMENT_INSTANCE_MATERIAL____INSTANCE_MATERIAL_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INSTANCE_MATERIAL_TYPE____BIND] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_material_type____bind, &ColladaParserAutoGen15Private::_data__instance_material_type____bind, &ColladaParserAutoGen15Private::_end__instance_material_type____bind, &ColladaParserAutoGen15Private::_preBegin__instance_material_type____bind, &ColladaParserAutoGen15Private::_preEnd__instance_material_type____bind, &ColladaParserAutoGen15Private::_freeAttributes__instance_material_type____bind);
mElementToNamespaceMap[HASH_ELEMENT_INSTANCE_MATERIAL_TYPE____BIND] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BIND_VERTEX_INPUT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__bind_vertex_input, &ColladaParserAutoGen15Private::_data__bind_vertex_input, &ColladaParserAutoGen15Private::_end__bind_vertex_input, &ColladaParserAutoGen15Private::_preBegin__bind_vertex_input, &ColladaParserAutoGen15Private::_preEnd__bind_vertex_input, &ColladaParserAutoGen15Private::_freeAttributes__bind_vertex_input);
mElementToNamespaceMap[HASH_ELEMENT_BIND_VERTEX_INPUT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INSTANCE_GEOMETRY] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_geometry, &ColladaParserAutoGen15Private::_data__instance_geometry, &ColladaParserAutoGen15Private::_end__instance_geometry, &ColladaParserAutoGen15Private::_preBegin__instance_geometry, &ColladaParserAutoGen15Private::_preEnd__instance_geometry, &ColladaParserAutoGen15Private::_freeAttributes__instance_geometry);
mElementToNamespaceMap[HASH_ELEMENT_INSTANCE_GEOMETRY] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INSTANCE_LIGHT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_light, &ColladaParserAutoGen15Private::_data__instance_light, &ColladaParserAutoGen15Private::_end__instance_light, &ColladaParserAutoGen15Private::_preBegin__instance_light, &ColladaParserAutoGen15Private::_preEnd__instance_light, &ColladaParserAutoGen15Private::_freeAttributes__instance_light);
mElementToNamespaceMap[HASH_ELEMENT_INSTANCE_LIGHT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INSTANCE_NODE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_node, &ColladaParserAutoGen15Private::_data__instance_node, &ColladaParserAutoGen15Private::_end__instance_node, &ColladaParserAutoGen15Private::_preBegin__instance_node, &ColladaParserAutoGen15Private::_preEnd__instance_node, &ColladaParserAutoGen15Private::_freeAttributes__instance_node);
mElementToNamespaceMap[HASH_ELEMENT_INSTANCE_NODE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_physics_materials, &ColladaParserAutoGen15Private::_data__library_physics_materials, &ColladaParserAutoGen15Private::_end__library_physics_materials, &ColladaParserAutoGen15Private::_preBegin__library_physics_materials, &ColladaParserAutoGen15Private::_preEnd__library_physics_materials, &ColladaParserAutoGen15Private::_freeAttributes__library_physics_materials);
mElementToNamespaceMap[HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PHYSICS_MATERIAL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__physics_material, &ColladaParserAutoGen15Private::_data__physics_material, &ColladaParserAutoGen15Private::_end__physics_material, &ColladaParserAutoGen15Private::_preBegin__physics_material, &ColladaParserAutoGen15Private::_preEnd__physics_material, &ColladaParserAutoGen15Private::_freeAttributes__physics_material);
mElementToNamespaceMap[HASH_ELEMENT_PHYSICS_MATERIAL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PHYSICS_MATERIAL_TYPE____TECHNIQUE_COMMON] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__physics_material_type____technique_common, &ColladaParserAutoGen15Private::_data__physics_material_type____technique_common, &ColladaParserAutoGen15Private::_end__physics_material_type____technique_common, &ColladaParserAutoGen15Private::_preBegin__physics_material_type____technique_common, &ColladaParserAutoGen15Private::_preEnd__physics_material_type____technique_common, &ColladaParserAutoGen15Private::_freeAttributes__physics_material_type____technique_common);
mElementToNamespaceMap[HASH_ELEMENT_PHYSICS_MATERIAL_TYPE____TECHNIQUE_COMMON] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_DYNAMIC_FRICTION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__dynamic_friction, &ColladaParserAutoGen15Private::_data__dynamic_friction, &ColladaParserAutoGen15Private::_end__dynamic_friction, &ColladaParserAutoGen15Private::_preBegin__dynamic_friction, &ColladaParserAutoGen15Private::_preEnd__dynamic_friction, &ColladaParserAutoGen15Private::_freeAttributes__dynamic_friction);
mElementToNamespaceMap[HASH_ELEMENT_DYNAMIC_FRICTION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_RESTITUTION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__restitution, &ColladaParserAutoGen15Private::_data__restitution, &ColladaParserAutoGen15Private::_end__restitution, &ColladaParserAutoGen15Private::_preBegin__restitution, &ColladaParserAutoGen15Private::_preEnd__restitution, &ColladaParserAutoGen15Private::_freeAttributes__restitution);
mElementToNamespaceMap[HASH_ELEMENT_RESTITUTION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_STATIC_FRICTION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__static_friction, &ColladaParserAutoGen15Private::_data__static_friction, &ColladaParserAutoGen15Private::_end__static_friction, &ColladaParserAutoGen15Private::_preBegin__static_friction, &ColladaParserAutoGen15Private::_preEnd__static_friction, &ColladaParserAutoGen15Private::_freeAttributes__static_friction);
mElementToNamespaceMap[HASH_ELEMENT_STATIC_FRICTION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LIBRARY_PHYSICS_MODELS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_physics_models, &ColladaParserAutoGen15Private::_data__library_physics_models, &ColladaParserAutoGen15Private::_end__library_physics_models, &ColladaParserAutoGen15Private::_preBegin__library_physics_models, &ColladaParserAutoGen15Private::_preEnd__library_physics_models, &ColladaParserAutoGen15Private::_freeAttributes__library_physics_models);
mElementToNamespaceMap[HASH_ELEMENT_LIBRARY_PHYSICS_MODELS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PHYSICS_MODEL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__physics_model, &ColladaParserAutoGen15Private::_data__physics_model, &ColladaParserAutoGen15Private::_end__physics_model, &ColladaParserAutoGen15Private::_preBegin__physics_model, &ColladaParserAutoGen15Private::_preEnd__physics_model, &ColladaParserAutoGen15Private::_freeAttributes__physics_model);
mElementToNamespaceMap[HASH_ELEMENT_PHYSICS_MODEL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_RIGID_BODY] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__rigid_body, &ColladaParserAutoGen15Private::_data__rigid_body, &ColladaParserAutoGen15Private::_end__rigid_body, &ColladaParserAutoGen15Private::_preBegin__rigid_body, &ColladaParserAutoGen15Private::_preEnd__rigid_body, &ColladaParserAutoGen15Private::_freeAttributes__rigid_body);
mElementToNamespaceMap[HASH_ELEMENT_RIGID_BODY] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_RIGID_BODY_TYPE____TECHNIQUE_COMMON] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__rigid_body_type____technique_common, &ColladaParserAutoGen15Private::_data__rigid_body_type____technique_common, &ColladaParserAutoGen15Private::_end__rigid_body_type____technique_common, &ColladaParserAutoGen15Private::_preBegin__rigid_body_type____technique_common, &ColladaParserAutoGen15Private::_preEnd__rigid_body_type____technique_common, &ColladaParserAutoGen15Private::_freeAttributes__rigid_body_type____technique_common);
mElementToNamespaceMap[HASH_ELEMENT_RIGID_BODY_TYPE____TECHNIQUE_COMMON] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_RIGID_BODY__TECHNIQUE_COMMON__DYNAMIC] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__rigid_body__technique_common__dynamic, &ColladaParserAutoGen15Private::_data__rigid_body__technique_common__dynamic, &ColladaParserAutoGen15Private::_end__rigid_body__technique_common__dynamic, &ColladaParserAutoGen15Private::_preBegin__rigid_body__technique_common__dynamic, &ColladaParserAutoGen15Private::_preEnd__rigid_body__technique_common__dynamic, &ColladaParserAutoGen15Private::_freeAttributes__rigid_body__technique_common__dynamic);
mElementToNamespaceMap[HASH_ELEMENT_RIGID_BODY__TECHNIQUE_COMMON__DYNAMIC] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_MASS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__mass, &ColladaParserAutoGen15Private::_data__mass, &ColladaParserAutoGen15Private::_end__mass, &ColladaParserAutoGen15Private::_preBegin__mass, &ColladaParserAutoGen15Private::_preEnd__mass, &ColladaParserAutoGen15Private::_freeAttributes__mass);
mElementToNamespaceMap[HASH_ELEMENT_MASS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_RIGID_BODY__TECHNIQUE_COMMON__MASS_FRAME] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__rigid_body__technique_common__mass_frame, &ColladaParserAutoGen15Private::_data__rigid_body__technique_common__mass_frame, &ColladaParserAutoGen15Private::_end__rigid_body__technique_common__mass_frame, &ColladaParserAutoGen15Private::_preBegin__rigid_body__technique_common__mass_frame, &ColladaParserAutoGen15Private::_preEnd__rigid_body__technique_common__mass_frame, &ColladaParserAutoGen15Private::_freeAttributes__rigid_body__technique_common__mass_frame);
mElementToNamespaceMap[HASH_ELEMENT_RIGID_BODY__TECHNIQUE_COMMON__MASS_FRAME] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INERTIA] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__inertia, &ColladaParserAutoGen15Private::_data__inertia, &ColladaParserAutoGen15Private::_end__inertia, &ColladaParserAutoGen15Private::_preBegin__inertia, &ColladaParserAutoGen15Private::_preEnd__inertia, &ColladaParserAutoGen15Private::_freeAttributes__inertia);
mElementToNamespaceMap[HASH_ELEMENT_INERTIA] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_physics_material, &ColladaParserAutoGen15Private::_data__instance_physics_material, &ColladaParserAutoGen15Private::_end__instance_physics_material, &ColladaParserAutoGen15Private::_preBegin__instance_physics_material, &ColladaParserAutoGen15Private::_preEnd__instance_physics_material, &ColladaParserAutoGen15Private::_freeAttributes__instance_physics_material);
mElementToNamespaceMap[HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_RIGID_BODY__TECHNIQUE_COMMON__SHAPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__rigid_body__technique_common__shape, &ColladaParserAutoGen15Private::_data__rigid_body__technique_common__shape, &ColladaParserAutoGen15Private::_end__rigid_body__technique_common__shape, &ColladaParserAutoGen15Private::_preBegin__rigid_body__technique_common__shape, &ColladaParserAutoGen15Private::_preEnd__rigid_body__technique_common__shape, &ColladaParserAutoGen15Private::_freeAttributes__rigid_body__technique_common__shape);
mElementToNamespaceMap[HASH_ELEMENT_RIGID_BODY__TECHNIQUE_COMMON__SHAPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_RIGID_BODY__TECHNIQUE_COMMON__SHAPE__HOLLOW] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__rigid_body__technique_common__shape__hollow, &ColladaParserAutoGen15Private::_data__rigid_body__technique_common__shape__hollow, &ColladaParserAutoGen15Private::_end__rigid_body__technique_common__shape__hollow, &ColladaParserAutoGen15Private::_preBegin__rigid_body__technique_common__shape__hollow, &ColladaParserAutoGen15Private::_preEnd__rigid_body__technique_common__shape__hollow, &ColladaParserAutoGen15Private::_freeAttributes__rigid_body__technique_common__shape__hollow);
mElementToNamespaceMap[HASH_ELEMENT_RIGID_BODY__TECHNIQUE_COMMON__SHAPE__HOLLOW] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_DENSITY] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__density, &ColladaParserAutoGen15Private::_data__density, &ColladaParserAutoGen15Private::_end__density, &ColladaParserAutoGen15Private::_preBegin__density, &ColladaParserAutoGen15Private::_preEnd__density, &ColladaParserAutoGen15Private::_freeAttributes__density);
mElementToNamespaceMap[HASH_ELEMENT_DENSITY] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BOX] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__box, &ColladaParserAutoGen15Private::_data__box, &ColladaParserAutoGen15Private::_end__box, &ColladaParserAutoGen15Private::_preBegin__box, &ColladaParserAutoGen15Private::_preEnd__box, &ColladaParserAutoGen15Private::_freeAttributes__box);
mElementToNamespaceMap[HASH_ELEMENT_BOX] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_HALF_EXTENTS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__half_extents, &ColladaParserAutoGen15Private::_data__half_extents, &ColladaParserAutoGen15Private::_end__half_extents, &ColladaParserAutoGen15Private::_preBegin__half_extents, &ColladaParserAutoGen15Private::_preEnd__half_extents, &ColladaParserAutoGen15Private::_freeAttributes__half_extents);
mElementToNamespaceMap[HASH_ELEMENT_HALF_EXTENTS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CYLINDER____CYLINDER_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__cylinder____cylinder_type, &ColladaParserAutoGen15Private::_data__cylinder____cylinder_type, &ColladaParserAutoGen15Private::_end__cylinder____cylinder_type, &ColladaParserAutoGen15Private::_preBegin__cylinder____cylinder_type, &ColladaParserAutoGen15Private::_preEnd__cylinder____cylinder_type, &ColladaParserAutoGen15Private::_freeAttributes__cylinder____cylinder_type);
mElementToNamespaceMap[HASH_ELEMENT_CYLINDER____CYLINDER_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_HEIGHT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__height, &ColladaParserAutoGen15Private::_data__height, &ColladaParserAutoGen15Private::_end__height, &ColladaParserAutoGen15Private::_preBegin__height, &ColladaParserAutoGen15Private::_preEnd__height, &ColladaParserAutoGen15Private::_freeAttributes__height);
mElementToNamespaceMap[HASH_ELEMENT_HEIGHT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_CAPSULE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__capsule, &ColladaParserAutoGen15Private::_data__capsule, &ColladaParserAutoGen15Private::_end__capsule, &ColladaParserAutoGen15Private::_preBegin__capsule, &ColladaParserAutoGen15Private::_preEnd__capsule, &ColladaParserAutoGen15Private::_freeAttributes__capsule);
mElementToNamespaceMap[HASH_ELEMENT_CAPSULE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_RADIUS____FLOAT3_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__radius____float3_type, &ColladaParserAutoGen15Private::_data__radius____float3_type, &ColladaParserAutoGen15Private::_end__radius____float3_type, &ColladaParserAutoGen15Private::_preBegin__radius____float3_type, &ColladaParserAutoGen15Private::_preEnd__radius____float3_type, &ColladaParserAutoGen15Private::_freeAttributes__radius____float3_type);
mElementToNamespaceMap[HASH_ELEMENT_RADIUS____FLOAT3_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_RIGID_CONSTRAINT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__rigid_constraint, &ColladaParserAutoGen15Private::_data__rigid_constraint, &ColladaParserAutoGen15Private::_end__rigid_constraint, &ColladaParserAutoGen15Private::_preBegin__rigid_constraint, &ColladaParserAutoGen15Private::_preEnd__rigid_constraint, &ColladaParserAutoGen15Private::_freeAttributes__rigid_constraint);
mElementToNamespaceMap[HASH_ELEMENT_RIGID_CONSTRAINT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_REF_ATTACHMENT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__ref_attachment, &ColladaParserAutoGen15Private::_data__ref_attachment, &ColladaParserAutoGen15Private::_end__ref_attachment, &ColladaParserAutoGen15Private::_preBegin__ref_attachment, &ColladaParserAutoGen15Private::_preEnd__ref_attachment, &ColladaParserAutoGen15Private::_freeAttributes__ref_attachment);
mElementToNamespaceMap[HASH_ELEMENT_REF_ATTACHMENT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ATTACHMENT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__attachment, &ColladaParserAutoGen15Private::_data__attachment, &ColladaParserAutoGen15Private::_end__attachment, &ColladaParserAutoGen15Private::_preBegin__attachment, &ColladaParserAutoGen15Private::_preEnd__attachment, &ColladaParserAutoGen15Private::_freeAttributes__attachment);
mElementToNamespaceMap[HASH_ELEMENT_ATTACHMENT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_RIGID_CONSTRAINT_TYPE____TECHNIQUE_COMMON] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__rigid_constraint_type____technique_common, &ColladaParserAutoGen15Private::_data__rigid_constraint_type____technique_common, &ColladaParserAutoGen15Private::_end__rigid_constraint_type____technique_common, &ColladaParserAutoGen15Private::_preBegin__rigid_constraint_type____technique_common, &ColladaParserAutoGen15Private::_preEnd__rigid_constraint_type____technique_common, &ColladaParserAutoGen15Private::_freeAttributes__rigid_constraint_type____technique_common);
mElementToNamespaceMap[HASH_ELEMENT_RIGID_CONSTRAINT_TYPE____TECHNIQUE_COMMON] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ENABLED] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__enabled, &ColladaParserAutoGen15Private::_data__enabled, &ColladaParserAutoGen15Private::_end__enabled, &ColladaParserAutoGen15Private::_preBegin__enabled, &ColladaParserAutoGen15Private::_preEnd__enabled, &ColladaParserAutoGen15Private::_freeAttributes__enabled);
mElementToNamespaceMap[HASH_ELEMENT_ENABLED] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INTERPENETRATE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__interpenetrate, &ColladaParserAutoGen15Private::_data__interpenetrate, &ColladaParserAutoGen15Private::_end__interpenetrate, &ColladaParserAutoGen15Private::_preBegin__interpenetrate, &ColladaParserAutoGen15Private::_preEnd__interpenetrate, &ColladaParserAutoGen15Private::_freeAttributes__interpenetrate);
mElementToNamespaceMap[HASH_ELEMENT_INTERPENETRATE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_RIGID_CONSTRAINT__TECHNIQUE_COMMON__LIMITS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__rigid_constraint__technique_common__limits, &ColladaParserAutoGen15Private::_data__rigid_constraint__technique_common__limits, &ColladaParserAutoGen15Private::_end__rigid_constraint__technique_common__limits, &ColladaParserAutoGen15Private::_preBegin__rigid_constraint__technique_common__limits, &ColladaParserAutoGen15Private::_preEnd__rigid_constraint__technique_common__limits, &ColladaParserAutoGen15Private::_freeAttributes__rigid_constraint__technique_common__limits);
mElementToNamespaceMap[HASH_ELEMENT_RIGID_CONSTRAINT__TECHNIQUE_COMMON__LIMITS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SWING_CONE_AND_TWIST] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__swing_cone_and_twist, &ColladaParserAutoGen15Private::_data__swing_cone_and_twist, &ColladaParserAutoGen15Private::_end__swing_cone_and_twist, &ColladaParserAutoGen15Private::_preBegin__swing_cone_and_twist, &ColladaParserAutoGen15Private::_preEnd__swing_cone_and_twist, &ColladaParserAutoGen15Private::_freeAttributes__swing_cone_and_twist);
mElementToNamespaceMap[HASH_ELEMENT_SWING_CONE_AND_TWIST] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_MIN____TARGETABLE_FLOAT3_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__min____targetable_float3_type, &ColladaParserAutoGen15Private::_data__min____targetable_float3_type, &ColladaParserAutoGen15Private::_end__min____targetable_float3_type, &ColladaParserAutoGen15Private::_preBegin__min____targetable_float3_type, &ColladaParserAutoGen15Private::_preEnd__min____targetable_float3_type, &ColladaParserAutoGen15Private::_freeAttributes__min____targetable_float3_type);
mElementToNamespaceMap[HASH_ELEMENT_MIN____TARGETABLE_FLOAT3_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_MAX____TARGETABLE_FLOAT3_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__max____targetable_float3_type, &ColladaParserAutoGen15Private::_data__max____targetable_float3_type, &ColladaParserAutoGen15Private::_end__max____targetable_float3_type, &ColladaParserAutoGen15Private::_preBegin__max____targetable_float3_type, &ColladaParserAutoGen15Private::_preEnd__max____targetable_float3_type, &ColladaParserAutoGen15Private::_freeAttributes__max____targetable_float3_type);
mElementToNamespaceMap[HASH_ELEMENT_MAX____TARGETABLE_FLOAT3_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_RIGID_CONSTRAINT__TECHNIQUE_COMMON__LIMITS__LINEAR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__rigid_constraint__technique_common__limits__linear, &ColladaParserAutoGen15Private::_data__rigid_constraint__technique_common__limits__linear, &ColladaParserAutoGen15Private::_end__rigid_constraint__technique_common__limits__linear, &ColladaParserAutoGen15Private::_preBegin__rigid_constraint__technique_common__limits__linear, &ColladaParserAutoGen15Private::_preEnd__rigid_constraint__technique_common__limits__linear, &ColladaParserAutoGen15Private::_freeAttributes__rigid_constraint__technique_common__limits__linear);
mElementToNamespaceMap[HASH_ELEMENT_RIGID_CONSTRAINT__TECHNIQUE_COMMON__LIMITS__LINEAR] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SPRING] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__spring, &ColladaParserAutoGen15Private::_data__spring, &ColladaParserAutoGen15Private::_end__spring, &ColladaParserAutoGen15Private::_preBegin__spring, &ColladaParserAutoGen15Private::_preEnd__spring, &ColladaParserAutoGen15Private::_freeAttributes__spring);
mElementToNamespaceMap[HASH_ELEMENT_SPRING] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ANGULAR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__angular, &ColladaParserAutoGen15Private::_data__angular, &ColladaParserAutoGen15Private::_end__angular, &ColladaParserAutoGen15Private::_preBegin__angular, &ColladaParserAutoGen15Private::_preEnd__angular, &ColladaParserAutoGen15Private::_freeAttributes__angular);
mElementToNamespaceMap[HASH_ELEMENT_ANGULAR] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_STIFFNESS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__stiffness, &ColladaParserAutoGen15Private::_data__stiffness, &ColladaParserAutoGen15Private::_end__stiffness, &ColladaParserAutoGen15Private::_preBegin__stiffness, &ColladaParserAutoGen15Private::_preEnd__stiffness, &ColladaParserAutoGen15Private::_freeAttributes__stiffness);
mElementToNamespaceMap[HASH_ELEMENT_STIFFNESS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_DAMPING] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__damping, &ColladaParserAutoGen15Private::_data__damping, &ColladaParserAutoGen15Private::_end__damping, &ColladaParserAutoGen15Private::_preBegin__damping, &ColladaParserAutoGen15Private::_preEnd__damping, &ColladaParserAutoGen15Private::_freeAttributes__damping);
mElementToNamespaceMap[HASH_ELEMENT_DAMPING] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TARGET_VALUE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__target_value, &ColladaParserAutoGen15Private::_data__target_value, &ColladaParserAutoGen15Private::_end__target_value, &ColladaParserAutoGen15Private::_preBegin__target_value, &ColladaParserAutoGen15Private::_preEnd__target_value, &ColladaParserAutoGen15Private::_freeAttributes__target_value);
mElementToNamespaceMap[HASH_ELEMENT_TARGET_VALUE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SPRING__LINEAR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__spring__linear, &ColladaParserAutoGen15Private::_data__spring__linear, &ColladaParserAutoGen15Private::_end__spring__linear, &ColladaParserAutoGen15Private::_preBegin__spring__linear, &ColladaParserAutoGen15Private::_preEnd__spring__linear, &ColladaParserAutoGen15Private::_freeAttributes__spring__linear);
mElementToNamespaceMap[HASH_ELEMENT_SPRING__LINEAR] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INSTANCE_PHYSICS_MODEL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_physics_model, &ColladaParserAutoGen15Private::_data__instance_physics_model, &ColladaParserAutoGen15Private::_end__instance_physics_model, &ColladaParserAutoGen15Private::_preBegin__instance_physics_model, &ColladaParserAutoGen15Private::_preEnd__instance_physics_model, &ColladaParserAutoGen15Private::_freeAttributes__instance_physics_model);
mElementToNamespaceMap[HASH_ELEMENT_INSTANCE_PHYSICS_MODEL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INSTANCE_FORCE_FIELD] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_force_field, &ColladaParserAutoGen15Private::_data__instance_force_field, &ColladaParserAutoGen15Private::_end__instance_force_field, &ColladaParserAutoGen15Private::_preBegin__instance_force_field, &ColladaParserAutoGen15Private::_preEnd__instance_force_field, &ColladaParserAutoGen15Private::_freeAttributes__instance_force_field);
mElementToNamespaceMap[HASH_ELEMENT_INSTANCE_FORCE_FIELD] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INSTANCE_RIGID_BODY] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_rigid_body, &ColladaParserAutoGen15Private::_data__instance_rigid_body, &ColladaParserAutoGen15Private::_end__instance_rigid_body, &ColladaParserAutoGen15Private::_preBegin__instance_rigid_body, &ColladaParserAutoGen15Private::_preEnd__instance_rigid_body, &ColladaParserAutoGen15Private::_freeAttributes__instance_rigid_body);
mElementToNamespaceMap[HASH_ELEMENT_INSTANCE_RIGID_BODY] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INSTANCE_RIGID_BODY_TYPE____TECHNIQUE_COMMON] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_rigid_body_type____technique_common, &ColladaParserAutoGen15Private::_data__instance_rigid_body_type____technique_common, &ColladaParserAutoGen15Private::_end__instance_rigid_body_type____technique_common, &ColladaParserAutoGen15Private::_preBegin__instance_rigid_body_type____technique_common, &ColladaParserAutoGen15Private::_preEnd__instance_rigid_body_type____technique_common, &ColladaParserAutoGen15Private::_freeAttributes__instance_rigid_body_type____technique_common);
mElementToNamespaceMap[HASH_ELEMENT_INSTANCE_RIGID_BODY_TYPE____TECHNIQUE_COMMON] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ANGULAR_VELOCITY] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__angular_velocity, &ColladaParserAutoGen15Private::_data__angular_velocity, &ColladaParserAutoGen15Private::_end__angular_velocity, &ColladaParserAutoGen15Private::_preBegin__angular_velocity, &ColladaParserAutoGen15Private::_preEnd__angular_velocity, &ColladaParserAutoGen15Private::_freeAttributes__angular_velocity);
mElementToNamespaceMap[HASH_ELEMENT_ANGULAR_VELOCITY] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_VELOCITY] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__velocity, &ColladaParserAutoGen15Private::_data__velocity, &ColladaParserAutoGen15Private::_end__velocity, &ColladaParserAutoGen15Private::_preBegin__velocity, &ColladaParserAutoGen15Private::_preEnd__velocity, &ColladaParserAutoGen15Private::_freeAttributes__velocity);
mElementToNamespaceMap[HASH_ELEMENT_VELOCITY] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INSTANCE_RIGID_BODY__TECHNIQUE_COMMON__DYNAMIC] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_rigid_body__technique_common__dynamic, &ColladaParserAutoGen15Private::_data__instance_rigid_body__technique_common__dynamic, &ColladaParserAutoGen15Private::_end__instance_rigid_body__technique_common__dynamic, &ColladaParserAutoGen15Private::_preBegin__instance_rigid_body__technique_common__dynamic, &ColladaParserAutoGen15Private::_preEnd__instance_rigid_body__technique_common__dynamic, &ColladaParserAutoGen15Private::_freeAttributes__instance_rigid_body__technique_common__dynamic);
mElementToNamespaceMap[HASH_ELEMENT_INSTANCE_RIGID_BODY__TECHNIQUE_COMMON__DYNAMIC] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INSTANCE_RIGID_BODY__TECHNIQUE_COMMON__MASS_FRAME] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_rigid_body__technique_common__mass_frame, &ColladaParserAutoGen15Private::_data__instance_rigid_body__technique_common__mass_frame, &ColladaParserAutoGen15Private::_end__instance_rigid_body__technique_common__mass_frame, &ColladaParserAutoGen15Private::_preBegin__instance_rigid_body__technique_common__mass_frame, &ColladaParserAutoGen15Private::_preEnd__instance_rigid_body__technique_common__mass_frame, &ColladaParserAutoGen15Private::_freeAttributes__instance_rigid_body__technique_common__mass_frame);
mElementToNamespaceMap[HASH_ELEMENT_INSTANCE_RIGID_BODY__TECHNIQUE_COMMON__MASS_FRAME] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INSTANCE_RIGID_BODY__TECHNIQUE_COMMON__SHAPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_rigid_body__technique_common__shape, &ColladaParserAutoGen15Private::_data__instance_rigid_body__technique_common__shape, &ColladaParserAutoGen15Private::_end__instance_rigid_body__technique_common__shape, &ColladaParserAutoGen15Private::_preBegin__instance_rigid_body__technique_common__shape, &ColladaParserAutoGen15Private::_preEnd__instance_rigid_body__technique_common__shape, &ColladaParserAutoGen15Private::_freeAttributes__instance_rigid_body__technique_common__shape);
mElementToNamespaceMap[HASH_ELEMENT_INSTANCE_RIGID_BODY__TECHNIQUE_COMMON__SHAPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INSTANCE_RIGID_BODY__TECHNIQUE_COMMON__SHAPE__HOLLOW] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_rigid_body__technique_common__shape__hollow, &ColladaParserAutoGen15Private::_data__instance_rigid_body__technique_common__shape__hollow, &ColladaParserAutoGen15Private::_end__instance_rigid_body__technique_common__shape__hollow, &ColladaParserAutoGen15Private::_preBegin__instance_rigid_body__technique_common__shape__hollow, &ColladaParserAutoGen15Private::_preEnd__instance_rigid_body__technique_common__shape__hollow, &ColladaParserAutoGen15Private::_freeAttributes__instance_rigid_body__technique_common__shape__hollow);
mElementToNamespaceMap[HASH_ELEMENT_INSTANCE_RIGID_BODY__TECHNIQUE_COMMON__SHAPE__HOLLOW] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INSTANCE_RIGID_CONSTRAINT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_rigid_constraint, &ColladaParserAutoGen15Private::_data__instance_rigid_constraint, &ColladaParserAutoGen15Private::_end__instance_rigid_constraint, &ColladaParserAutoGen15Private::_preBegin__instance_rigid_constraint, &ColladaParserAutoGen15Private::_preEnd__instance_rigid_constraint, &ColladaParserAutoGen15Private::_freeAttributes__instance_rigid_constraint);
mElementToNamespaceMap[HASH_ELEMENT_INSTANCE_RIGID_CONSTRAINT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LIBRARY_PHYSICS_SCENES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_physics_scenes, &ColladaParserAutoGen15Private::_data__library_physics_scenes, &ColladaParserAutoGen15Private::_end__library_physics_scenes, &ColladaParserAutoGen15Private::_preBegin__library_physics_scenes, &ColladaParserAutoGen15Private::_preEnd__library_physics_scenes, &ColladaParserAutoGen15Private::_freeAttributes__library_physics_scenes);
mElementToNamespaceMap[HASH_ELEMENT_LIBRARY_PHYSICS_SCENES] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PHYSICS_SCENE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__physics_scene, &ColladaParserAutoGen15Private::_data__physics_scene, &ColladaParserAutoGen15Private::_end__physics_scene, &ColladaParserAutoGen15Private::_preBegin__physics_scene, &ColladaParserAutoGen15Private::_preEnd__physics_scene, &ColladaParserAutoGen15Private::_freeAttributes__physics_scene);
mElementToNamespaceMap[HASH_ELEMENT_PHYSICS_SCENE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PHYSICS_SCENE_TYPE____TECHNIQUE_COMMON] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__physics_scene_type____technique_common, &ColladaParserAutoGen15Private::_data__physics_scene_type____technique_common, &ColladaParserAutoGen15Private::_end__physics_scene_type____technique_common, &ColladaParserAutoGen15Private::_preBegin__physics_scene_type____technique_common, &ColladaParserAutoGen15Private::_preEnd__physics_scene_type____technique_common, &ColladaParserAutoGen15Private::_freeAttributes__physics_scene_type____technique_common);
mElementToNamespaceMap[HASH_ELEMENT_PHYSICS_SCENE_TYPE____TECHNIQUE_COMMON] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_GRAVITY] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__gravity, &ColladaParserAutoGen15Private::_data__gravity, &ColladaParserAutoGen15Private::_end__gravity, &ColladaParserAutoGen15Private::_preBegin__gravity, &ColladaParserAutoGen15Private::_preEnd__gravity, &ColladaParserAutoGen15Private::_freeAttributes__gravity);
mElementToNamespaceMap[HASH_ELEMENT_GRAVITY] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TIME_STEP] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__time_step, &ColladaParserAutoGen15Private::_data__time_step, &ColladaParserAutoGen15Private::_end__time_step, &ColladaParserAutoGen15Private::_preBegin__time_step, &ColladaParserAutoGen15Private::_preEnd__time_step, &ColladaParserAutoGen15Private::_freeAttributes__time_step);
mElementToNamespaceMap[HASH_ELEMENT_TIME_STEP] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LIBRARY_VISUAL_SCENES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_visual_scenes, &ColladaParserAutoGen15Private::_data__library_visual_scenes, &ColladaParserAutoGen15Private::_end__library_visual_scenes, &ColladaParserAutoGen15Private::_preBegin__library_visual_scenes, &ColladaParserAutoGen15Private::_preEnd__library_visual_scenes, &ColladaParserAutoGen15Private::_freeAttributes__library_visual_scenes);
mElementToNamespaceMap[HASH_ELEMENT_LIBRARY_VISUAL_SCENES] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_VISUAL_SCENE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__visual_scene, &ColladaParserAutoGen15Private::_data__visual_scene, &ColladaParserAutoGen15Private::_end__visual_scene, &ColladaParserAutoGen15Private::_preBegin__visual_scene, &ColladaParserAutoGen15Private::_preEnd__visual_scene, &ColladaParserAutoGen15Private::_freeAttributes__visual_scene);
mElementToNamespaceMap[HASH_ELEMENT_VISUAL_SCENE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_EVALUATE_SCENE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__evaluate_scene, &ColladaParserAutoGen15Private::_data__evaluate_scene, &ColladaParserAutoGen15Private::_end__evaluate_scene, &ColladaParserAutoGen15Private::_preBegin__evaluate_scene, &ColladaParserAutoGen15Private::_preEnd__evaluate_scene, &ColladaParserAutoGen15Private::_freeAttributes__evaluate_scene);
mElementToNamespaceMap[HASH_ELEMENT_EVALUATE_SCENE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_RENDER] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__render, &ColladaParserAutoGen15Private::_data__render, &ColladaParserAutoGen15Private::_end__render, &ColladaParserAutoGen15Private::_preBegin__render, &ColladaParserAutoGen15Private::_preEnd__render, &ColladaParserAutoGen15Private::_freeAttributes__render);
mElementToNamespaceMap[HASH_ELEMENT_RENDER] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LAYER] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__layer, &ColladaParserAutoGen15Private::_data__layer, &ColladaParserAutoGen15Private::_end__layer, &ColladaParserAutoGen15Private::_preBegin__layer, &ColladaParserAutoGen15Private::_preEnd__layer, &ColladaParserAutoGen15Private::_freeAttributes__layer);
mElementToNamespaceMap[HASH_ELEMENT_LAYER] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_RENDER__INSTANCE_MATERIAL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__render__instance_material, &ColladaParserAutoGen15Private::_data__render__instance_material, &ColladaParserAutoGen15Private::_end__render__instance_material, &ColladaParserAutoGen15Private::_preBegin__render__instance_material, &ColladaParserAutoGen15Private::_preEnd__render__instance_material, &ColladaParserAutoGen15Private::_freeAttributes__render__instance_material);
mElementToNamespaceMap[HASH_ELEMENT_RENDER__INSTANCE_MATERIAL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TECHNIQUE_OVERRIDE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique_override, &ColladaParserAutoGen15Private::_data__technique_override, &ColladaParserAutoGen15Private::_end__technique_override, &ColladaParserAutoGen15Private::_preBegin__technique_override, &ColladaParserAutoGen15Private::_preEnd__technique_override, &ColladaParserAutoGen15Private::_freeAttributes__technique_override);
mElementToNamespaceMap[HASH_ELEMENT_TECHNIQUE_OVERRIDE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_RENDER__INSTANCE_MATERIAL__BIND] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__render__instance_material__bind, &ColladaParserAutoGen15Private::_data__render__instance_material__bind, &ColladaParserAutoGen15Private::_end__render__instance_material__bind, &ColladaParserAutoGen15Private::_preBegin__render__instance_material__bind, &ColladaParserAutoGen15Private::_preEnd__render__instance_material__bind, &ColladaParserAutoGen15Private::_freeAttributes__render__instance_material__bind);
mElementToNamespaceMap[HASH_ELEMENT_RENDER__INSTANCE_MATERIAL__BIND] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LIBRARY_JOINTS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_joints, &ColladaParserAutoGen15Private::_data__library_joints, &ColladaParserAutoGen15Private::_end__library_joints, &ColladaParserAutoGen15Private::_preBegin__library_joints, &ColladaParserAutoGen15Private::_preEnd__library_joints, &ColladaParserAutoGen15Private::_freeAttributes__library_joints);
mElementToNamespaceMap[HASH_ELEMENT_LIBRARY_JOINTS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_JOINT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__joint, &ColladaParserAutoGen15Private::_data__joint, &ColladaParserAutoGen15Private::_end__joint, &ColladaParserAutoGen15Private::_preBegin__joint, &ColladaParserAutoGen15Private::_preEnd__joint, &ColladaParserAutoGen15Private::_freeAttributes__joint);
mElementToNamespaceMap[HASH_ELEMENT_JOINT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PRISMATIC] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__prismatic, &ColladaParserAutoGen15Private::_data__prismatic, &ColladaParserAutoGen15Private::_end__prismatic, &ColladaParserAutoGen15Private::_preBegin__prismatic, &ColladaParserAutoGen15Private::_preEnd__prismatic, &ColladaParserAutoGen15Private::_freeAttributes__prismatic);
mElementToNamespaceMap[HASH_ELEMENT_PRISMATIC] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_AXIS____AXIS_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__axis____axis_type, &ColladaParserAutoGen15Private::_data__axis____axis_type, &ColladaParserAutoGen15Private::_end__axis____axis_type, &ColladaParserAutoGen15Private::_preBegin__axis____axis_type, &ColladaParserAutoGen15Private::_preEnd__axis____axis_type, &ColladaParserAutoGen15Private::_freeAttributes__axis____axis_type);
mElementToNamespaceMap[HASH_ELEMENT_AXIS____AXIS_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LIMITS____JOINT_LIMITS_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__limits____joint_limits_type, &ColladaParserAutoGen15Private::_data__limits____joint_limits_type, &ColladaParserAutoGen15Private::_end__limits____joint_limits_type, &ColladaParserAutoGen15Private::_preBegin__limits____joint_limits_type, &ColladaParserAutoGen15Private::_preEnd__limits____joint_limits_type, &ColladaParserAutoGen15Private::_freeAttributes__limits____joint_limits_type);
mElementToNamespaceMap[HASH_ELEMENT_LIMITS____JOINT_LIMITS_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_MIN____MINMAX_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__min____minmax_type, &ColladaParserAutoGen15Private::_data__min____minmax_type, &ColladaParserAutoGen15Private::_end__min____minmax_type, &ColladaParserAutoGen15Private::_preBegin__min____minmax_type, &ColladaParserAutoGen15Private::_preEnd__min____minmax_type, &ColladaParserAutoGen15Private::_freeAttributes__min____minmax_type);
mElementToNamespaceMap[HASH_ELEMENT_MIN____MINMAX_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_MAX____MINMAX_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__max____minmax_type, &ColladaParserAutoGen15Private::_data__max____minmax_type, &ColladaParserAutoGen15Private::_end__max____minmax_type, &ColladaParserAutoGen15Private::_preBegin__max____minmax_type, &ColladaParserAutoGen15Private::_preEnd__max____minmax_type, &ColladaParserAutoGen15Private::_freeAttributes__max____minmax_type);
mElementToNamespaceMap[HASH_ELEMENT_MAX____MINMAX_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_REVOLUTE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__revolute, &ColladaParserAutoGen15Private::_data__revolute, &ColladaParserAutoGen15Private::_end__revolute, &ColladaParserAutoGen15Private::_preBegin__revolute, &ColladaParserAutoGen15Private::_preEnd__revolute, &ColladaParserAutoGen15Private::_freeAttributes__revolute);
mElementToNamespaceMap[HASH_ELEMENT_REVOLUTE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LIBRARY_KINEMATICS_MODELS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_kinematics_models, &ColladaParserAutoGen15Private::_data__library_kinematics_models, &ColladaParserAutoGen15Private::_end__library_kinematics_models, &ColladaParserAutoGen15Private::_preBegin__library_kinematics_models, &ColladaParserAutoGen15Private::_preEnd__library_kinematics_models, &ColladaParserAutoGen15Private::_freeAttributes__library_kinematics_models);
mElementToNamespaceMap[HASH_ELEMENT_LIBRARY_KINEMATICS_MODELS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_KINEMATICS_MODEL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__kinematics_model, &ColladaParserAutoGen15Private::_data__kinematics_model, &ColladaParserAutoGen15Private::_end__kinematics_model, &ColladaParserAutoGen15Private::_preBegin__kinematics_model, &ColladaParserAutoGen15Private::_preEnd__kinematics_model, &ColladaParserAutoGen15Private::_freeAttributes__kinematics_model);
mElementToNamespaceMap[HASH_ELEMENT_KINEMATICS_MODEL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TECHNIQUE_COMMON____KINEMATICS_MODEL_TECHNIQUE_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique_common____kinematics_model_technique_type, &ColladaParserAutoGen15Private::_data__technique_common____kinematics_model_technique_type, &ColladaParserAutoGen15Private::_end__technique_common____kinematics_model_technique_type, &ColladaParserAutoGen15Private::_preBegin__technique_common____kinematics_model_technique_type, &ColladaParserAutoGen15Private::_preEnd__technique_common____kinematics_model_technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique_common____kinematics_model_technique_type);
mElementToNamespaceMap[HASH_ELEMENT_TECHNIQUE_COMMON____KINEMATICS_MODEL_TECHNIQUE_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_NEWPARAM____KINEMATICS_NEWPARAM_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__newparam____kinematics_newparam_type, &ColladaParserAutoGen15Private::_data__newparam____kinematics_newparam_type, &ColladaParserAutoGen15Private::_end__newparam____kinematics_newparam_type, &ColladaParserAutoGen15Private::_preBegin__newparam____kinematics_newparam_type, &ColladaParserAutoGen15Private::_preEnd__newparam____kinematics_newparam_type, &ColladaParserAutoGen15Private::_freeAttributes__newparam____kinematics_newparam_type);
mElementToNamespaceMap[HASH_ELEMENT_NEWPARAM____KINEMATICS_NEWPARAM_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INSTANCE_JOINT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_joint, &ColladaParserAutoGen15Private::_data__instance_joint, &ColladaParserAutoGen15Private::_end__instance_joint, &ColladaParserAutoGen15Private::_preBegin__instance_joint, &ColladaParserAutoGen15Private::_preEnd__instance_joint, &ColladaParserAutoGen15Private::_freeAttributes__instance_joint);
mElementToNamespaceMap[HASH_ELEMENT_INSTANCE_JOINT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LINK] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__link, &ColladaParserAutoGen15Private::_data__link, &ColladaParserAutoGen15Private::_end__link, &ColladaParserAutoGen15Private::_preBegin__link, &ColladaParserAutoGen15Private::_preEnd__link, &ColladaParserAutoGen15Private::_freeAttributes__link);
mElementToNamespaceMap[HASH_ELEMENT_LINK] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ATTACHMENT_FULL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__attachment_full, &ColladaParserAutoGen15Private::_data__attachment_full, &ColladaParserAutoGen15Private::_end__attachment_full, &ColladaParserAutoGen15Private::_preBegin__attachment_full, &ColladaParserAutoGen15Private::_preEnd__attachment_full, &ColladaParserAutoGen15Private::_freeAttributes__attachment_full);
mElementToNamespaceMap[HASH_ELEMENT_ATTACHMENT_FULL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ATTACHMENT_START] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__attachment_start, &ColladaParserAutoGen15Private::_data__attachment_start, &ColladaParserAutoGen15Private::_end__attachment_start, &ColladaParserAutoGen15Private::_preBegin__attachment_start, &ColladaParserAutoGen15Private::_preEnd__attachment_start, &ColladaParserAutoGen15Private::_freeAttributes__attachment_start);
mElementToNamespaceMap[HASH_ELEMENT_ATTACHMENT_START] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ATTACHMENT_END] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__attachment_end, &ColladaParserAutoGen15Private::_data__attachment_end, &ColladaParserAutoGen15Private::_end__attachment_end, &ColladaParserAutoGen15Private::_preBegin__attachment_end, &ColladaParserAutoGen15Private::_preEnd__attachment_end, &ColladaParserAutoGen15Private::_freeAttributes__attachment_end);
mElementToNamespaceMap[HASH_ELEMENT_ATTACHMENT_END] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FORMULA] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__formula, &ColladaParserAutoGen15Private::_data__formula, &ColladaParserAutoGen15Private::_end__formula, &ColladaParserAutoGen15Private::_preBegin__formula, &ColladaParserAutoGen15Private::_preEnd__formula, &ColladaParserAutoGen15Private::_freeAttributes__formula);
mElementToNamespaceMap[HASH_ELEMENT_FORMULA] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_NEWPARAM____FORMULA_NEWPARAM_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__newparam____formula_newparam_type, &ColladaParserAutoGen15Private::_data__newparam____formula_newparam_type, &ColladaParserAutoGen15Private::_end__newparam____formula_newparam_type, &ColladaParserAutoGen15Private::_preBegin__newparam____formula_newparam_type, &ColladaParserAutoGen15Private::_preEnd__newparam____formula_newparam_type, &ColladaParserAutoGen15Private::_freeAttributes__newparam____formula_newparam_type);
mElementToNamespaceMap[HASH_ELEMENT_NEWPARAM____FORMULA_NEWPARAM_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TARGET] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__target, &ColladaParserAutoGen15Private::_data__target, &ColladaParserAutoGen15Private::_end__target, &ColladaParserAutoGen15Private::_preBegin__target, &ColladaParserAutoGen15Private::_preEnd__target, &ColladaParserAutoGen15Private::_freeAttributes__target);
mElementToNamespaceMap[HASH_ELEMENT_TARGET] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PARAM____COMMON_PARAM_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__param____common_param_type, &ColladaParserAutoGen15Private::_data__param____common_param_type, &ColladaParserAutoGen15Private::_end__param____common_param_type, &ColladaParserAutoGen15Private::_preBegin__param____common_param_type, &ColladaParserAutoGen15Private::_preEnd__param____common_param_type, &ColladaParserAutoGen15Private::_freeAttributes__param____common_param_type);
mElementToNamespaceMap[HASH_ELEMENT_PARAM____COMMON_PARAM_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TECHNIQUE_COMMON____FORMULA_TECHNIQUE_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique_common____formula_technique_type, &ColladaParserAutoGen15Private::_data__technique_common____formula_technique_type, &ColladaParserAutoGen15Private::_end__technique_common____formula_technique_type, &ColladaParserAutoGen15Private::_preBegin__technique_common____formula_technique_type, &ColladaParserAutoGen15Private::_preEnd__technique_common____formula_technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique_common____formula_technique_type);
mElementToNamespaceMap[HASH_ELEMENT_TECHNIQUE_COMMON____FORMULA_TECHNIQUE_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_MATH] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__math, &ColladaParserAutoGen15Private::_data__math, &ColladaParserAutoGen15Private::_end__math, &ColladaParserAutoGen15Private::_preBegin__math, &ColladaParserAutoGen15Private::_preEnd__math, &ColladaParserAutoGen15Private::_freeAttributes__math);
mElementToNamespaceMap[HASH_ELEMENT_MATH] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_CN] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__cn, &ColladaParserAutoGen15Private::_data__cn, &ColladaParserAutoGen15Private::_end__cn, &ColladaParserAutoGen15Private::_preBegin__cn, &ColladaParserAutoGen15Private::_preEnd__cn, &ColladaParserAutoGen15Private::_freeAttributes__cn);
mElementToNamespaceMap[HASH_ELEMENT_CN] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_CI] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__ci, &ColladaParserAutoGen15Private::_data__ci, &ColladaParserAutoGen15Private::_end__ci, &ColladaParserAutoGen15Private::_preBegin__ci, &ColladaParserAutoGen15Private::_preEnd__ci, &ColladaParserAutoGen15Private::_freeAttributes__ci);
mElementToNamespaceMap[HASH_ELEMENT_CI] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_CSYMBOL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__csymbol, &ColladaParserAutoGen15Private::_data__csymbol, &ColladaParserAutoGen15Private::_end__csymbol, &ColladaParserAutoGen15Private::_preBegin__csymbol, &ColladaParserAutoGen15Private::_preEnd__csymbol, &ColladaParserAutoGen15Private::_freeAttributes__csymbol);
mElementToNamespaceMap[HASH_ELEMENT_CSYMBOL] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_ABS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__abs, &ColladaParserAutoGen15Private::_data__abs, &ColladaParserAutoGen15Private::_end__abs, &ColladaParserAutoGen15Private::_preBegin__abs, &ColladaParserAutoGen15Private::_preEnd__abs, &ColladaParserAutoGen15Private::_freeAttributes__abs);
mElementToNamespaceMap[HASH_ELEMENT_ABS] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_CONJUGATE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__conjugate, &ColladaParserAutoGen15Private::_data__conjugate, &ColladaParserAutoGen15Private::_end__conjugate, &ColladaParserAutoGen15Private::_preBegin__conjugate, &ColladaParserAutoGen15Private::_preEnd__conjugate, &ColladaParserAutoGen15Private::_freeAttributes__conjugate);
mElementToNamespaceMap[HASH_ELEMENT_CONJUGATE] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_FACTORIAL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__factorial, &ColladaParserAutoGen15Private::_data__factorial, &ColladaParserAutoGen15Private::_end__factorial, &ColladaParserAutoGen15Private::_preBegin__factorial, &ColladaParserAutoGen15Private::_preEnd__factorial, &ColladaParserAutoGen15Private::_freeAttributes__factorial);
mElementToNamespaceMap[HASH_ELEMENT_FACTORIAL] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_ARG] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__arg, &ColladaParserAutoGen15Private::_data__arg, &ColladaParserAutoGen15Private::_end__arg, &ColladaParserAutoGen15Private::_preBegin__arg, &ColladaParserAutoGen15Private::_preEnd__arg, &ColladaParserAutoGen15Private::_freeAttributes__arg);
mElementToNamespaceMap[HASH_ELEMENT_ARG] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_REAL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__real, &ColladaParserAutoGen15Private::_data__real, &ColladaParserAutoGen15Private::_end__real, &ColladaParserAutoGen15Private::_preBegin__real, &ColladaParserAutoGen15Private::_preEnd__real, &ColladaParserAutoGen15Private::_freeAttributes__real);
mElementToNamespaceMap[HASH_ELEMENT_REAL] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_IMAGINARY] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__imaginary, &ColladaParserAutoGen15Private::_data__imaginary, &ColladaParserAutoGen15Private::_end__imaginary, &ColladaParserAutoGen15Private::_preBegin__imaginary, &ColladaParserAutoGen15Private::_preEnd__imaginary, &ColladaParserAutoGen15Private::_freeAttributes__imaginary);
mElementToNamespaceMap[HASH_ELEMENT_IMAGINARY] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_FLOOR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__floor, &ColladaParserAutoGen15Private::_data__floor, &ColladaParserAutoGen15Private::_end__floor, &ColladaParserAutoGen15Private::_preBegin__floor, &ColladaParserAutoGen15Private::_preEnd__floor, &ColladaParserAutoGen15Private::_freeAttributes__floor);
mElementToNamespaceMap[HASH_ELEMENT_FLOOR] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_CEILING] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__ceiling, &ColladaParserAutoGen15Private::_data__ceiling, &ColladaParserAutoGen15Private::_end__ceiling, &ColladaParserAutoGen15Private::_preBegin__ceiling, &ColladaParserAutoGen15Private::_preEnd__ceiling, &ColladaParserAutoGen15Private::_freeAttributes__ceiling);
mElementToNamespaceMap[HASH_ELEMENT_CEILING] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_QUOTIENT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__quotient, &ColladaParserAutoGen15Private::_data__quotient, &ColladaParserAutoGen15Private::_end__quotient, &ColladaParserAutoGen15Private::_preBegin__quotient, &ColladaParserAutoGen15Private::_preEnd__quotient, &ColladaParserAutoGen15Private::_freeAttributes__quotient);
mElementToNamespaceMap[HASH_ELEMENT_QUOTIENT] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_DIVIDE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__divide, &ColladaParserAutoGen15Private::_data__divide, &ColladaParserAutoGen15Private::_end__divide, &ColladaParserAutoGen15Private::_preBegin__divide, &ColladaParserAutoGen15Private::_preEnd__divide, &ColladaParserAutoGen15Private::_freeAttributes__divide);
mElementToNamespaceMap[HASH_ELEMENT_DIVIDE] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_REM] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__rem, &ColladaParserAutoGen15Private::_data__rem, &ColladaParserAutoGen15Private::_end__rem, &ColladaParserAutoGen15Private::_preBegin__rem, &ColladaParserAutoGen15Private::_preEnd__rem, &ColladaParserAutoGen15Private::_freeAttributes__rem);
mElementToNamespaceMap[HASH_ELEMENT_REM] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_MINUS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__minus, &ColladaParserAutoGen15Private::_data__minus, &ColladaParserAutoGen15Private::_end__minus, &ColladaParserAutoGen15Private::_preBegin__minus, &ColladaParserAutoGen15Private::_preEnd__minus, &ColladaParserAutoGen15Private::_freeAttributes__minus);
mElementToNamespaceMap[HASH_ELEMENT_MINUS] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_PLUS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__plus, &ColladaParserAutoGen15Private::_data__plus, &ColladaParserAutoGen15Private::_end__plus, &ColladaParserAutoGen15Private::_preBegin__plus, &ColladaParserAutoGen15Private::_preEnd__plus, &ColladaParserAutoGen15Private::_freeAttributes__plus);
mElementToNamespaceMap[HASH_ELEMENT_PLUS] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_TIMES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__times, &ColladaParserAutoGen15Private::_data__times, &ColladaParserAutoGen15Private::_end__times, &ColladaParserAutoGen15Private::_preBegin__times, &ColladaParserAutoGen15Private::_preEnd__times, &ColladaParserAutoGen15Private::_freeAttributes__times);
mElementToNamespaceMap[HASH_ELEMENT_TIMES] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_POWER] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__power, &ColladaParserAutoGen15Private::_data__power, &ColladaParserAutoGen15Private::_end__power, &ColladaParserAutoGen15Private::_preBegin__power, &ColladaParserAutoGen15Private::_preEnd__power, &ColladaParserAutoGen15Private::_freeAttributes__power);
mElementToNamespaceMap[HASH_ELEMENT_POWER] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_ROOT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__root, &ColladaParserAutoGen15Private::_data__root, &ColladaParserAutoGen15Private::_end__root, &ColladaParserAutoGen15Private::_preBegin__root, &ColladaParserAutoGen15Private::_preEnd__root, &ColladaParserAutoGen15Private::_freeAttributes__root);
mElementToNamespaceMap[HASH_ELEMENT_ROOT] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_MAX____ARITH_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__max____Arith_type, &ColladaParserAutoGen15Private::_data__max____Arith_type, &ColladaParserAutoGen15Private::_end__max____Arith_type, &ColladaParserAutoGen15Private::_preBegin__max____Arith_type, &ColladaParserAutoGen15Private::_preEnd__max____Arith_type, &ColladaParserAutoGen15Private::_freeAttributes__max____Arith_type);
mElementToNamespaceMap[HASH_ELEMENT_MAX____ARITH_TYPE] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_MIN____ARITH_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__min____Arith_type, &ColladaParserAutoGen15Private::_data__min____Arith_type, &ColladaParserAutoGen15Private::_end__min____Arith_type, &ColladaParserAutoGen15Private::_preBegin__min____Arith_type, &ColladaParserAutoGen15Private::_preEnd__min____Arith_type, &ColladaParserAutoGen15Private::_freeAttributes__min____Arith_type);
mElementToNamespaceMap[HASH_ELEMENT_MIN____ARITH_TYPE] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_GCD] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__gcd, &ColladaParserAutoGen15Private::_data__gcd, &ColladaParserAutoGen15Private::_end__gcd, &ColladaParserAutoGen15Private::_preBegin__gcd, &ColladaParserAutoGen15Private::_preEnd__gcd, &ColladaParserAutoGen15Private::_freeAttributes__gcd);
mElementToNamespaceMap[HASH_ELEMENT_GCD] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_LCM] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__lcm, &ColladaParserAutoGen15Private::_data__lcm, &ColladaParserAutoGen15Private::_end__lcm, &ColladaParserAutoGen15Private::_preBegin__lcm, &ColladaParserAutoGen15Private::_preEnd__lcm, &ColladaParserAutoGen15Private::_freeAttributes__lcm);
mElementToNamespaceMap[HASH_ELEMENT_LCM] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_SUM] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__sum, &ColladaParserAutoGen15Private::_data__sum, &ColladaParserAutoGen15Private::_end__sum, &ColladaParserAutoGen15Private::_preBegin__sum, &ColladaParserAutoGen15Private::_preEnd__sum, &ColladaParserAutoGen15Private::_freeAttributes__sum);
mElementToNamespaceMap[HASH_ELEMENT_SUM] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_PRODUCT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__product, &ColladaParserAutoGen15Private::_data__product, &ColladaParserAutoGen15Private::_end__product, &ColladaParserAutoGen15Private::_preBegin__product, &ColladaParserAutoGen15Private::_preEnd__product, &ColladaParserAutoGen15Private::_freeAttributes__product);
mElementToNamespaceMap[HASH_ELEMENT_PRODUCT] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_COMPOSE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__compose, &ColladaParserAutoGen15Private::_data__compose, &ColladaParserAutoGen15Private::_end__compose, &ColladaParserAutoGen15Private::_preBegin__compose, &ColladaParserAutoGen15Private::_preEnd__compose, &ColladaParserAutoGen15Private::_freeAttributes__compose);
mElementToNamespaceMap[HASH_ELEMENT_COMPOSE] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_DOMAIN] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__domain, &ColladaParserAutoGen15Private::_data__domain, &ColladaParserAutoGen15Private::_end__domain, &ColladaParserAutoGen15Private::_preBegin__domain, &ColladaParserAutoGen15Private::_preEnd__domain, &ColladaParserAutoGen15Private::_freeAttributes__domain);
mElementToNamespaceMap[HASH_ELEMENT_DOMAIN] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_CODOMAIN] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__codomain, &ColladaParserAutoGen15Private::_data__codomain, &ColladaParserAutoGen15Private::_end__codomain, &ColladaParserAutoGen15Private::_preBegin__codomain, &ColladaParserAutoGen15Private::_preEnd__codomain, &ColladaParserAutoGen15Private::_freeAttributes__codomain);
mElementToNamespaceMap[HASH_ELEMENT_CODOMAIN] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_IMAGE____FUNCTIONS_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__image____Functions_type, &ColladaParserAutoGen15Private::_data__image____Functions_type, &ColladaParserAutoGen15Private::_end__image____Functions_type, &ColladaParserAutoGen15Private::_preBegin__image____Functions_type, &ColladaParserAutoGen15Private::_preEnd__image____Functions_type, &ColladaParserAutoGen15Private::_freeAttributes__image____Functions_type);
mElementToNamespaceMap[HASH_ELEMENT_IMAGE____FUNCTIONS_TYPE] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_DOMAINOFAPPLICATION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__domainofapplication, &ColladaParserAutoGen15Private::_data__domainofapplication, &ColladaParserAutoGen15Private::_end__domainofapplication, &ColladaParserAutoGen15Private::_preBegin__domainofapplication, &ColladaParserAutoGen15Private::_preEnd__domainofapplication, &ColladaParserAutoGen15Private::_freeAttributes__domainofapplication);
mElementToNamespaceMap[HASH_ELEMENT_DOMAINOFAPPLICATION] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_IDENT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__ident, &ColladaParserAutoGen15Private::_data__ident, &ColladaParserAutoGen15Private::_end__ident, &ColladaParserAutoGen15Private::_preBegin__ident, &ColladaParserAutoGen15Private::_preEnd__ident, &ColladaParserAutoGen15Private::_freeAttributes__ident);
mElementToNamespaceMap[HASH_ELEMENT_IDENT] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_AND] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__and, &ColladaParserAutoGen15Private::_data__and, &ColladaParserAutoGen15Private::_end__and, &ColladaParserAutoGen15Private::_preBegin__and, &ColladaParserAutoGen15Private::_preEnd__and, &ColladaParserAutoGen15Private::_freeAttributes__and);
mElementToNamespaceMap[HASH_ELEMENT_AND] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_OR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__or, &ColladaParserAutoGen15Private::_data__or, &ColladaParserAutoGen15Private::_end__or, &ColladaParserAutoGen15Private::_preBegin__or, &ColladaParserAutoGen15Private::_preEnd__or, &ColladaParserAutoGen15Private::_freeAttributes__or);
mElementToNamespaceMap[HASH_ELEMENT_OR] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_XOR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__xor, &ColladaParserAutoGen15Private::_data__xor, &ColladaParserAutoGen15Private::_end__xor, &ColladaParserAutoGen15Private::_preBegin__xor, &ColladaParserAutoGen15Private::_preEnd__xor, &ColladaParserAutoGen15Private::_freeAttributes__xor);
mElementToNamespaceMap[HASH_ELEMENT_XOR] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_NOT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__not, &ColladaParserAutoGen15Private::_data__not, &ColladaParserAutoGen15Private::_end__not, &ColladaParserAutoGen15Private::_preBegin__not, &ColladaParserAutoGen15Private::_preEnd__not, &ColladaParserAutoGen15Private::_freeAttributes__not);
mElementToNamespaceMap[HASH_ELEMENT_NOT] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_EXISTS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__exists, &ColladaParserAutoGen15Private::_data__exists, &ColladaParserAutoGen15Private::_end__exists, &ColladaParserAutoGen15Private::_preBegin__exists, &ColladaParserAutoGen15Private::_preEnd__exists, &ColladaParserAutoGen15Private::_freeAttributes__exists);
mElementToNamespaceMap[HASH_ELEMENT_EXISTS] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_FORALL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__forall, &ColladaParserAutoGen15Private::_data__forall, &ColladaParserAutoGen15Private::_end__forall, &ColladaParserAutoGen15Private::_preBegin__forall, &ColladaParserAutoGen15Private::_preEnd__forall, &ColladaParserAutoGen15Private::_freeAttributes__forall);
mElementToNamespaceMap[HASH_ELEMENT_FORALL] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_IMPLIES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__implies, &ColladaParserAutoGen15Private::_data__implies, &ColladaParserAutoGen15Private::_end__implies, &ColladaParserAutoGen15Private::_preBegin__implies, &ColladaParserAutoGen15Private::_preEnd__implies, &ColladaParserAutoGen15Private::_freeAttributes__implies);
mElementToNamespaceMap[HASH_ELEMENT_IMPLIES] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_NATURALNUMBERS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__naturalnumbers, &ColladaParserAutoGen15Private::_data__naturalnumbers, &ColladaParserAutoGen15Private::_end__naturalnumbers, &ColladaParserAutoGen15Private::_preBegin__naturalnumbers, &ColladaParserAutoGen15Private::_preEnd__naturalnumbers, &ColladaParserAutoGen15Private::_freeAttributes__naturalnumbers);
mElementToNamespaceMap[HASH_ELEMENT_NATURALNUMBERS] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_PRIMES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__primes, &ColladaParserAutoGen15Private::_data__primes, &ColladaParserAutoGen15Private::_end__primes, &ColladaParserAutoGen15Private::_preBegin__primes, &ColladaParserAutoGen15Private::_preEnd__primes, &ColladaParserAutoGen15Private::_freeAttributes__primes);
mElementToNamespaceMap[HASH_ELEMENT_PRIMES] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_INTEGERS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__integers, &ColladaParserAutoGen15Private::_data__integers, &ColladaParserAutoGen15Private::_end__integers, &ColladaParserAutoGen15Private::_preBegin__integers, &ColladaParserAutoGen15Private::_preEnd__integers, &ColladaParserAutoGen15Private::_freeAttributes__integers);
mElementToNamespaceMap[HASH_ELEMENT_INTEGERS] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_RATIONALS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__rationals, &ColladaParserAutoGen15Private::_data__rationals, &ColladaParserAutoGen15Private::_end__rationals, &ColladaParserAutoGen15Private::_preBegin__rationals, &ColladaParserAutoGen15Private::_preEnd__rationals, &ColladaParserAutoGen15Private::_freeAttributes__rationals);
mElementToNamespaceMap[HASH_ELEMENT_RATIONALS] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_REALS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__reals, &ColladaParserAutoGen15Private::_data__reals, &ColladaParserAutoGen15Private::_end__reals, &ColladaParserAutoGen15Private::_preBegin__reals, &ColladaParserAutoGen15Private::_preEnd__reals, &ColladaParserAutoGen15Private::_freeAttributes__reals);
mElementToNamespaceMap[HASH_ELEMENT_REALS] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_COMPLEXES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__complexes, &ColladaParserAutoGen15Private::_data__complexes, &ColladaParserAutoGen15Private::_end__complexes, &ColladaParserAutoGen15Private::_preBegin__complexes, &ColladaParserAutoGen15Private::_preEnd__complexes, &ColladaParserAutoGen15Private::_freeAttributes__complexes);
mElementToNamespaceMap[HASH_ELEMENT_COMPLEXES] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_EMPTYSET] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__emptyset, &ColladaParserAutoGen15Private::_data__emptyset, &ColladaParserAutoGen15Private::_end__emptyset, &ColladaParserAutoGen15Private::_preBegin__emptyset, &ColladaParserAutoGen15Private::_preEnd__emptyset, &ColladaParserAutoGen15Private::_freeAttributes__emptyset);
mElementToNamespaceMap[HASH_ELEMENT_EMPTYSET] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_EXPONENTIALE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__exponentiale, &ColladaParserAutoGen15Private::_data__exponentiale, &ColladaParserAutoGen15Private::_end__exponentiale, &ColladaParserAutoGen15Private::_preBegin__exponentiale, &ColladaParserAutoGen15Private::_preEnd__exponentiale, &ColladaParserAutoGen15Private::_freeAttributes__exponentiale);
mElementToNamespaceMap[HASH_ELEMENT_EXPONENTIALE] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_IMAGINARYI] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__imaginaryi, &ColladaParserAutoGen15Private::_data__imaginaryi, &ColladaParserAutoGen15Private::_end__imaginaryi, &ColladaParserAutoGen15Private::_preBegin__imaginaryi, &ColladaParserAutoGen15Private::_preEnd__imaginaryi, &ColladaParserAutoGen15Private::_freeAttributes__imaginaryi);
mElementToNamespaceMap[HASH_ELEMENT_IMAGINARYI] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_PI] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__pi, &ColladaParserAutoGen15Private::_data__pi, &ColladaParserAutoGen15Private::_end__pi, &ColladaParserAutoGen15Private::_preBegin__pi, &ColladaParserAutoGen15Private::_preEnd__pi, &ColladaParserAutoGen15Private::_freeAttributes__pi);
mElementToNamespaceMap[HASH_ELEMENT_PI] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_EULERGAMMA] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__eulergamma, &ColladaParserAutoGen15Private::_data__eulergamma, &ColladaParserAutoGen15Private::_end__eulergamma, &ColladaParserAutoGen15Private::_preBegin__eulergamma, &ColladaParserAutoGen15Private::_preEnd__eulergamma, &ColladaParserAutoGen15Private::_freeAttributes__eulergamma);
mElementToNamespaceMap[HASH_ELEMENT_EULERGAMMA] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_TRUE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__true, &ColladaParserAutoGen15Private::_data__true, &ColladaParserAutoGen15Private::_end__true, &ColladaParserAutoGen15Private::_preBegin__true, &ColladaParserAutoGen15Private::_preEnd__true, &ColladaParserAutoGen15Private::_freeAttributes__true);
mElementToNamespaceMap[HASH_ELEMENT_TRUE] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_FALSE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__false, &ColladaParserAutoGen15Private::_data__false, &ColladaParserAutoGen15Private::_end__false, &ColladaParserAutoGen15Private::_preBegin__false, &ColladaParserAutoGen15Private::_preEnd__false, &ColladaParserAutoGen15Private::_freeAttributes__false);
mElementToNamespaceMap[HASH_ELEMENT_FALSE] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_INFINITY] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__infinity, &ColladaParserAutoGen15Private::_data__infinity, &ColladaParserAutoGen15Private::_end__infinity, &ColladaParserAutoGen15Private::_preBegin__infinity, &ColladaParserAutoGen15Private::_preEnd__infinity, &ColladaParserAutoGen15Private::_freeAttributes__infinity);
mElementToNamespaceMap[HASH_ELEMENT_INFINITY] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_NOTANUMBER] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__notanumber, &ColladaParserAutoGen15Private::_data__notanumber, &ColladaParserAutoGen15Private::_end__notanumber, &ColladaParserAutoGen15Private::_preBegin__notanumber, &ColladaParserAutoGen15Private::_preEnd__notanumber, &ColladaParserAutoGen15Private::_freeAttributes__notanumber);
mElementToNamespaceMap[HASH_ELEMENT_NOTANUMBER] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_SET] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__set, &ColladaParserAutoGen15Private::_data__set, &ColladaParserAutoGen15Private::_end__set, &ColladaParserAutoGen15Private::_preBegin__set, &ColladaParserAutoGen15Private::_preEnd__set, &ColladaParserAutoGen15Private::_freeAttributes__set);
mElementToNamespaceMap[HASH_ELEMENT_SET] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_LIST] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__list, &ColladaParserAutoGen15Private::_data__list, &ColladaParserAutoGen15Private::_end__list, &ColladaParserAutoGen15Private::_preBegin__list, &ColladaParserAutoGen15Private::_preEnd__list, &ColladaParserAutoGen15Private::_freeAttributes__list);
mElementToNamespaceMap[HASH_ELEMENT_LIST] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_UNION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__union, &ColladaParserAutoGen15Private::_data__union, &ColladaParserAutoGen15Private::_end__union, &ColladaParserAutoGen15Private::_preBegin__union, &ColladaParserAutoGen15Private::_preEnd__union, &ColladaParserAutoGen15Private::_freeAttributes__union);
mElementToNamespaceMap[HASH_ELEMENT_UNION] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_INTERSECT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__intersect, &ColladaParserAutoGen15Private::_data__intersect, &ColladaParserAutoGen15Private::_end__intersect, &ColladaParserAutoGen15Private::_preBegin__intersect, &ColladaParserAutoGen15Private::_preEnd__intersect, &ColladaParserAutoGen15Private::_freeAttributes__intersect);
mElementToNamespaceMap[HASH_ELEMENT_INTERSECT] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_IN] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__in, &ColladaParserAutoGen15Private::_data__in, &ColladaParserAutoGen15Private::_end__in, &ColladaParserAutoGen15Private::_preBegin__in, &ColladaParserAutoGen15Private::_preEnd__in, &ColladaParserAutoGen15Private::_freeAttributes__in);
mElementToNamespaceMap[HASH_ELEMENT_IN] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_NOTIN] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__notin, &ColladaParserAutoGen15Private::_data__notin, &ColladaParserAutoGen15Private::_end__notin, &ColladaParserAutoGen15Private::_preBegin__notin, &ColladaParserAutoGen15Private::_preEnd__notin, &ColladaParserAutoGen15Private::_freeAttributes__notin);
mElementToNamespaceMap[HASH_ELEMENT_NOTIN] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_SUBSET] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__subset, &ColladaParserAutoGen15Private::_data__subset, &ColladaParserAutoGen15Private::_end__subset, &ColladaParserAutoGen15Private::_preBegin__subset, &ColladaParserAutoGen15Private::_preEnd__subset, &ColladaParserAutoGen15Private::_freeAttributes__subset);
mElementToNamespaceMap[HASH_ELEMENT_SUBSET] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_PRSUBSET] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__prsubset, &ColladaParserAutoGen15Private::_data__prsubset, &ColladaParserAutoGen15Private::_end__prsubset, &ColladaParserAutoGen15Private::_preBegin__prsubset, &ColladaParserAutoGen15Private::_preEnd__prsubset, &ColladaParserAutoGen15Private::_freeAttributes__prsubset);
mElementToNamespaceMap[HASH_ELEMENT_PRSUBSET] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_NOTSUBSET] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__notsubset, &ColladaParserAutoGen15Private::_data__notsubset, &ColladaParserAutoGen15Private::_end__notsubset, &ColladaParserAutoGen15Private::_preBegin__notsubset, &ColladaParserAutoGen15Private::_preEnd__notsubset, &ColladaParserAutoGen15Private::_freeAttributes__notsubset);
mElementToNamespaceMap[HASH_ELEMENT_NOTSUBSET] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_NOTPRSUBSET] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__notprsubset, &ColladaParserAutoGen15Private::_data__notprsubset, &ColladaParserAutoGen15Private::_end__notprsubset, &ColladaParserAutoGen15Private::_preBegin__notprsubset, &ColladaParserAutoGen15Private::_preEnd__notprsubset, &ColladaParserAutoGen15Private::_freeAttributes__notprsubset);
mElementToNamespaceMap[HASH_ELEMENT_NOTPRSUBSET] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_SETDIFF] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__setdiff, &ColladaParserAutoGen15Private::_data__setdiff, &ColladaParserAutoGen15Private::_end__setdiff, &ColladaParserAutoGen15Private::_preBegin__setdiff, &ColladaParserAutoGen15Private::_preEnd__setdiff, &ColladaParserAutoGen15Private::_freeAttributes__setdiff);
mElementToNamespaceMap[HASH_ELEMENT_SETDIFF] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_CARD] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__card, &ColladaParserAutoGen15Private::_data__card, &ColladaParserAutoGen15Private::_end__card, &ColladaParserAutoGen15Private::_preBegin__card, &ColladaParserAutoGen15Private::_preEnd__card, &ColladaParserAutoGen15Private::_freeAttributes__card);
mElementToNamespaceMap[HASH_ELEMENT_CARD] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_CARTESIANPRODUCT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__cartesianproduct, &ColladaParserAutoGen15Private::_data__cartesianproduct, &ColladaParserAutoGen15Private::_end__cartesianproduct, &ColladaParserAutoGen15Private::_preBegin__cartesianproduct, &ColladaParserAutoGen15Private::_preEnd__cartesianproduct, &ColladaParserAutoGen15Private::_freeAttributes__cartesianproduct);
mElementToNamespaceMap[HASH_ELEMENT_CARTESIANPRODUCT] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_EQ] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__eq, &ColladaParserAutoGen15Private::_data__eq, &ColladaParserAutoGen15Private::_end__eq, &ColladaParserAutoGen15Private::_preBegin__eq, &ColladaParserAutoGen15Private::_preEnd__eq, &ColladaParserAutoGen15Private::_freeAttributes__eq);
mElementToNamespaceMap[HASH_ELEMENT_EQ] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_NEQ] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__neq, &ColladaParserAutoGen15Private::_data__neq, &ColladaParserAutoGen15Private::_end__neq, &ColladaParserAutoGen15Private::_preBegin__neq, &ColladaParserAutoGen15Private::_preEnd__neq, &ColladaParserAutoGen15Private::_freeAttributes__neq);
mElementToNamespaceMap[HASH_ELEMENT_NEQ] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_LEQ] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__leq, &ColladaParserAutoGen15Private::_data__leq, &ColladaParserAutoGen15Private::_end__leq, &ColladaParserAutoGen15Private::_preBegin__leq, &ColladaParserAutoGen15Private::_preEnd__leq, &ColladaParserAutoGen15Private::_freeAttributes__leq);
mElementToNamespaceMap[HASH_ELEMENT_LEQ] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_LT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__lt, &ColladaParserAutoGen15Private::_data__lt, &ColladaParserAutoGen15Private::_end__lt, &ColladaParserAutoGen15Private::_preBegin__lt, &ColladaParserAutoGen15Private::_preEnd__lt, &ColladaParserAutoGen15Private::_freeAttributes__lt);
mElementToNamespaceMap[HASH_ELEMENT_LT] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_GEQ] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__geq, &ColladaParserAutoGen15Private::_data__geq, &ColladaParserAutoGen15Private::_end__geq, &ColladaParserAutoGen15Private::_preBegin__geq, &ColladaParserAutoGen15Private::_preEnd__geq, &ColladaParserAutoGen15Private::_freeAttributes__geq);
mElementToNamespaceMap[HASH_ELEMENT_GEQ] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_GT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__gt, &ColladaParserAutoGen15Private::_data__gt, &ColladaParserAutoGen15Private::_end__gt, &ColladaParserAutoGen15Private::_preBegin__gt, &ColladaParserAutoGen15Private::_preEnd__gt, &ColladaParserAutoGen15Private::_freeAttributes__gt);
mElementToNamespaceMap[HASH_ELEMENT_GT] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_EQUIVALENT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__equivalent, &ColladaParserAutoGen15Private::_data__equivalent, &ColladaParserAutoGen15Private::_end__equivalent, &ColladaParserAutoGen15Private::_preBegin__equivalent, &ColladaParserAutoGen15Private::_preEnd__equivalent, &ColladaParserAutoGen15Private::_freeAttributes__equivalent);
mElementToNamespaceMap[HASH_ELEMENT_EQUIVALENT] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_APPROX] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__approx, &ColladaParserAutoGen15Private::_data__approx, &ColladaParserAutoGen15Private::_end__approx, &ColladaParserAutoGen15Private::_preBegin__approx, &ColladaParserAutoGen15Private::_preEnd__approx, &ColladaParserAutoGen15Private::_freeAttributes__approx);
mElementToNamespaceMap[HASH_ELEMENT_APPROX] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_FACTOROF] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__factorof, &ColladaParserAutoGen15Private::_data__factorof, &ColladaParserAutoGen15Private::_end__factorof, &ColladaParserAutoGen15Private::_preBegin__factorof, &ColladaParserAutoGen15Private::_preEnd__factorof, &ColladaParserAutoGen15Private::_freeAttributes__factorof);
mElementToNamespaceMap[HASH_ELEMENT_FACTOROF] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_EXP] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__exp, &ColladaParserAutoGen15Private::_data__exp, &ColladaParserAutoGen15Private::_end__exp, &ColladaParserAutoGen15Private::_preBegin__exp, &ColladaParserAutoGen15Private::_preEnd__exp, &ColladaParserAutoGen15Private::_freeAttributes__exp);
mElementToNamespaceMap[HASH_ELEMENT_EXP] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_LN] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__ln, &ColladaParserAutoGen15Private::_data__ln, &ColladaParserAutoGen15Private::_end__ln, &ColladaParserAutoGen15Private::_preBegin__ln, &ColladaParserAutoGen15Private::_preEnd__ln, &ColladaParserAutoGen15Private::_freeAttributes__ln);
mElementToNamespaceMap[HASH_ELEMENT_LN] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_LOG] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__log, &ColladaParserAutoGen15Private::_data__log, &ColladaParserAutoGen15Private::_end__log, &ColladaParserAutoGen15Private::_preBegin__log, &ColladaParserAutoGen15Private::_preEnd__log, &ColladaParserAutoGen15Private::_freeAttributes__log);
mElementToNamespaceMap[HASH_ELEMENT_LOG] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_LOGBASE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__logbase, &ColladaParserAutoGen15Private::_data__logbase, &ColladaParserAutoGen15Private::_end__logbase, &ColladaParserAutoGen15Private::_preBegin__logbase, &ColladaParserAutoGen15Private::_preEnd__logbase, &ColladaParserAutoGen15Private::_freeAttributes__logbase);
mElementToNamespaceMap[HASH_ELEMENT_LOGBASE] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_SIN] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__sin, &ColladaParserAutoGen15Private::_data__sin, &ColladaParserAutoGen15Private::_end__sin, &ColladaParserAutoGen15Private::_preBegin__sin, &ColladaParserAutoGen15Private::_preEnd__sin, &ColladaParserAutoGen15Private::_freeAttributes__sin);
mElementToNamespaceMap[HASH_ELEMENT_SIN] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_COS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__cos, &ColladaParserAutoGen15Private::_data__cos, &ColladaParserAutoGen15Private::_end__cos, &ColladaParserAutoGen15Private::_preBegin__cos, &ColladaParserAutoGen15Private::_preEnd__cos, &ColladaParserAutoGen15Private::_freeAttributes__cos);
mElementToNamespaceMap[HASH_ELEMENT_COS] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_TAN] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__tan, &ColladaParserAutoGen15Private::_data__tan, &ColladaParserAutoGen15Private::_end__tan, &ColladaParserAutoGen15Private::_preBegin__tan, &ColladaParserAutoGen15Private::_preEnd__tan, &ColladaParserAutoGen15Private::_freeAttributes__tan);
mElementToNamespaceMap[HASH_ELEMENT_TAN] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_SEC] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__sec, &ColladaParserAutoGen15Private::_data__sec, &ColladaParserAutoGen15Private::_end__sec, &ColladaParserAutoGen15Private::_preBegin__sec, &ColladaParserAutoGen15Private::_preEnd__sec, &ColladaParserAutoGen15Private::_freeAttributes__sec);
mElementToNamespaceMap[HASH_ELEMENT_SEC] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_CSC] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__csc, &ColladaParserAutoGen15Private::_data__csc, &ColladaParserAutoGen15Private::_end__csc, &ColladaParserAutoGen15Private::_preBegin__csc, &ColladaParserAutoGen15Private::_preEnd__csc, &ColladaParserAutoGen15Private::_freeAttributes__csc);
mElementToNamespaceMap[HASH_ELEMENT_CSC] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_COT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__cot, &ColladaParserAutoGen15Private::_data__cot, &ColladaParserAutoGen15Private::_end__cot, &ColladaParserAutoGen15Private::_preBegin__cot, &ColladaParserAutoGen15Private::_preEnd__cot, &ColladaParserAutoGen15Private::_freeAttributes__cot);
mElementToNamespaceMap[HASH_ELEMENT_COT] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_ARCSIN] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__arcsin, &ColladaParserAutoGen15Private::_data__arcsin, &ColladaParserAutoGen15Private::_end__arcsin, &ColladaParserAutoGen15Private::_preBegin__arcsin, &ColladaParserAutoGen15Private::_preEnd__arcsin, &ColladaParserAutoGen15Private::_freeAttributes__arcsin);
mElementToNamespaceMap[HASH_ELEMENT_ARCSIN] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_ARCCOS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__arccos, &ColladaParserAutoGen15Private::_data__arccos, &ColladaParserAutoGen15Private::_end__arccos, &ColladaParserAutoGen15Private::_preBegin__arccos, &ColladaParserAutoGen15Private::_preEnd__arccos, &ColladaParserAutoGen15Private::_freeAttributes__arccos);
mElementToNamespaceMap[HASH_ELEMENT_ARCCOS] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_ARCTAN] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__arctan, &ColladaParserAutoGen15Private::_data__arctan, &ColladaParserAutoGen15Private::_end__arctan, &ColladaParserAutoGen15Private::_preBegin__arctan, &ColladaParserAutoGen15Private::_preEnd__arctan, &ColladaParserAutoGen15Private::_freeAttributes__arctan);
mElementToNamespaceMap[HASH_ELEMENT_ARCTAN] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_ARCSEC] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__arcsec, &ColladaParserAutoGen15Private::_data__arcsec, &ColladaParserAutoGen15Private::_end__arcsec, &ColladaParserAutoGen15Private::_preBegin__arcsec, &ColladaParserAutoGen15Private::_preEnd__arcsec, &ColladaParserAutoGen15Private::_freeAttributes__arcsec);
mElementToNamespaceMap[HASH_ELEMENT_ARCSEC] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_ARCCSC] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__arccsc, &ColladaParserAutoGen15Private::_data__arccsc, &ColladaParserAutoGen15Private::_end__arccsc, &ColladaParserAutoGen15Private::_preBegin__arccsc, &ColladaParserAutoGen15Private::_preEnd__arccsc, &ColladaParserAutoGen15Private::_freeAttributes__arccsc);
mElementToNamespaceMap[HASH_ELEMENT_ARCCSC] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_ARCCOT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__arccot, &ColladaParserAutoGen15Private::_data__arccot, &ColladaParserAutoGen15Private::_end__arccot, &ColladaParserAutoGen15Private::_preBegin__arccot, &ColladaParserAutoGen15Private::_preEnd__arccot, &ColladaParserAutoGen15Private::_freeAttributes__arccot);
mElementToNamespaceMap[HASH_ELEMENT_ARCCOT] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_SINH] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__sinh, &ColladaParserAutoGen15Private::_data__sinh, &ColladaParserAutoGen15Private::_end__sinh, &ColladaParserAutoGen15Private::_preBegin__sinh, &ColladaParserAutoGen15Private::_preEnd__sinh, &ColladaParserAutoGen15Private::_freeAttributes__sinh);
mElementToNamespaceMap[HASH_ELEMENT_SINH] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_COSH] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__cosh, &ColladaParserAutoGen15Private::_data__cosh, &ColladaParserAutoGen15Private::_end__cosh, &ColladaParserAutoGen15Private::_preBegin__cosh, &ColladaParserAutoGen15Private::_preEnd__cosh, &ColladaParserAutoGen15Private::_freeAttributes__cosh);
mElementToNamespaceMap[HASH_ELEMENT_COSH] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_TANH] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__tanh, &ColladaParserAutoGen15Private::_data__tanh, &ColladaParserAutoGen15Private::_end__tanh, &ColladaParserAutoGen15Private::_preBegin__tanh, &ColladaParserAutoGen15Private::_preEnd__tanh, &ColladaParserAutoGen15Private::_freeAttributes__tanh);
mElementToNamespaceMap[HASH_ELEMENT_TANH] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_SECH] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__sech, &ColladaParserAutoGen15Private::_data__sech, &ColladaParserAutoGen15Private::_end__sech, &ColladaParserAutoGen15Private::_preBegin__sech, &ColladaParserAutoGen15Private::_preEnd__sech, &ColladaParserAutoGen15Private::_freeAttributes__sech);
mElementToNamespaceMap[HASH_ELEMENT_SECH] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_CSCH] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__csch, &ColladaParserAutoGen15Private::_data__csch, &ColladaParserAutoGen15Private::_end__csch, &ColladaParserAutoGen15Private::_preBegin__csch, &ColladaParserAutoGen15Private::_preEnd__csch, &ColladaParserAutoGen15Private::_freeAttributes__csch);
mElementToNamespaceMap[HASH_ELEMENT_CSCH] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_COTH] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__coth, &ColladaParserAutoGen15Private::_data__coth, &ColladaParserAutoGen15Private::_end__coth, &ColladaParserAutoGen15Private::_preBegin__coth, &ColladaParserAutoGen15Private::_preEnd__coth, &ColladaParserAutoGen15Private::_freeAttributes__coth);
mElementToNamespaceMap[HASH_ELEMENT_COTH] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_ARCCOSH] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__arccosh, &ColladaParserAutoGen15Private::_data__arccosh, &ColladaParserAutoGen15Private::_end__arccosh, &ColladaParserAutoGen15Private::_preBegin__arccosh, &ColladaParserAutoGen15Private::_preEnd__arccosh, &ColladaParserAutoGen15Private::_freeAttributes__arccosh);
mElementToNamespaceMap[HASH_ELEMENT_ARCCOSH] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_ARCCOTH] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__arccoth, &ColladaParserAutoGen15Private::_data__arccoth, &ColladaParserAutoGen15Private::_end__arccoth, &ColladaParserAutoGen15Private::_preBegin__arccoth, &ColladaParserAutoGen15Private::_preEnd__arccoth, &ColladaParserAutoGen15Private::_freeAttributes__arccoth);
mElementToNamespaceMap[HASH_ELEMENT_ARCCOTH] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_ARCCSCH] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__arccsch, &ColladaParserAutoGen15Private::_data__arccsch, &ColladaParserAutoGen15Private::_end__arccsch, &ColladaParserAutoGen15Private::_preBegin__arccsch, &ColladaParserAutoGen15Private::_preEnd__arccsch, &ColladaParserAutoGen15Private::_freeAttributes__arccsch);
mElementToNamespaceMap[HASH_ELEMENT_ARCCSCH] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_ARCSECH] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__arcsech, &ColladaParserAutoGen15Private::_data__arcsech, &ColladaParserAutoGen15Private::_end__arcsech, &ColladaParserAutoGen15Private::_preBegin__arcsech, &ColladaParserAutoGen15Private::_preEnd__arcsech, &ColladaParserAutoGen15Private::_freeAttributes__arcsech);
mElementToNamespaceMap[HASH_ELEMENT_ARCSECH] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_ARCSINH] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__arcsinh, &ColladaParserAutoGen15Private::_data__arcsinh, &ColladaParserAutoGen15Private::_end__arcsinh, &ColladaParserAutoGen15Private::_preBegin__arcsinh, &ColladaParserAutoGen15Private::_preEnd__arcsinh, &ColladaParserAutoGen15Private::_freeAttributes__arcsinh);
mElementToNamespaceMap[HASH_ELEMENT_ARCSINH] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_ARCTANH] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__arctanh, &ColladaParserAutoGen15Private::_data__arctanh, &ColladaParserAutoGen15Private::_end__arctanh, &ColladaParserAutoGen15Private::_preBegin__arctanh, &ColladaParserAutoGen15Private::_preEnd__arctanh, &ColladaParserAutoGen15Private::_freeAttributes__arctanh);
mElementToNamespaceMap[HASH_ELEMENT_ARCTANH] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_INT____INT_TYPE____MATHML] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__int____int_type____mathml, &ColladaParserAutoGen15Private::_data__int____int_type____mathml, &ColladaParserAutoGen15Private::_end__int____int_type____mathml, &ColladaParserAutoGen15Private::_preBegin__int____int_type____mathml, &ColladaParserAutoGen15Private::_preEnd__int____int_type____mathml, &ColladaParserAutoGen15Private::_freeAttributes__int____int_type____mathml);
mElementToNamespaceMap[HASH_ELEMENT_INT____INT_TYPE____MATHML] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_DIFF] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__diff, &ColladaParserAutoGen15Private::_data__diff, &ColladaParserAutoGen15Private::_end__diff, &ColladaParserAutoGen15Private::_preBegin__diff, &ColladaParserAutoGen15Private::_preEnd__diff, &ColladaParserAutoGen15Private::_freeAttributes__diff);
mElementToNamespaceMap[HASH_ELEMENT_DIFF] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_PARTIALDIFF] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__partialdiff, &ColladaParserAutoGen15Private::_data__partialdiff, &ColladaParserAutoGen15Private::_end__partialdiff, &ColladaParserAutoGen15Private::_preBegin__partialdiff, &ColladaParserAutoGen15Private::_preEnd__partialdiff, &ColladaParserAutoGen15Private::_freeAttributes__partialdiff);
mElementToNamespaceMap[HASH_ELEMENT_PARTIALDIFF] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_LIMIT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__limit, &ColladaParserAutoGen15Private::_data__limit, &ColladaParserAutoGen15Private::_end__limit, &ColladaParserAutoGen15Private::_preBegin__limit, &ColladaParserAutoGen15Private::_preEnd__limit, &ColladaParserAutoGen15Private::_freeAttributes__limit);
mElementToNamespaceMap[HASH_ELEMENT_LIMIT] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_LOWLIMIT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__lowlimit, &ColladaParserAutoGen15Private::_data__lowlimit, &ColladaParserAutoGen15Private::_end__lowlimit, &ColladaParserAutoGen15Private::_preBegin__lowlimit, &ColladaParserAutoGen15Private::_preEnd__lowlimit, &ColladaParserAutoGen15Private::_freeAttributes__lowlimit);
mElementToNamespaceMap[HASH_ELEMENT_LOWLIMIT] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_UPLIMIT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__uplimit, &ColladaParserAutoGen15Private::_data__uplimit, &ColladaParserAutoGen15Private::_end__uplimit, &ColladaParserAutoGen15Private::_preBegin__uplimit, &ColladaParserAutoGen15Private::_preEnd__uplimit, &ColladaParserAutoGen15Private::_freeAttributes__uplimit);
mElementToNamespaceMap[HASH_ELEMENT_UPLIMIT] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_TENDSTO] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__tendsto, &ColladaParserAutoGen15Private::_data__tendsto, &ColladaParserAutoGen15Private::_end__tendsto, &ColladaParserAutoGen15Private::_preBegin__tendsto, &ColladaParserAutoGen15Private::_preEnd__tendsto, &ColladaParserAutoGen15Private::_freeAttributes__tendsto);
mElementToNamespaceMap[HASH_ELEMENT_TENDSTO] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_VECTOR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__vector, &ColladaParserAutoGen15Private::_data__vector, &ColladaParserAutoGen15Private::_end__vector, &ColladaParserAutoGen15Private::_preBegin__vector, &ColladaParserAutoGen15Private::_preEnd__vector, &ColladaParserAutoGen15Private::_freeAttributes__vector);
mElementToNamespaceMap[HASH_ELEMENT_VECTOR] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_MATRIX____MATRIX_TYPE____MATHML] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__matrix____matrix_type____mathml, &ColladaParserAutoGen15Private::_data__matrix____matrix_type____mathml, &ColladaParserAutoGen15Private::_end__matrix____matrix_type____mathml, &ColladaParserAutoGen15Private::_preBegin__matrix____matrix_type____mathml, &ColladaParserAutoGen15Private::_preEnd__matrix____matrix_type____mathml, &ColladaParserAutoGen15Private::_freeAttributes__matrix____matrix_type____mathml);
mElementToNamespaceMap[HASH_ELEMENT_MATRIX____MATRIX_TYPE____MATHML] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_MATRIXROW] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__matrixrow, &ColladaParserAutoGen15Private::_data__matrixrow, &ColladaParserAutoGen15Private::_end__matrixrow, &ColladaParserAutoGen15Private::_preBegin__matrixrow, &ColladaParserAutoGen15Private::_preEnd__matrixrow, &ColladaParserAutoGen15Private::_freeAttributes__matrixrow);
mElementToNamespaceMap[HASH_ELEMENT_MATRIXROW] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_DETERMINANT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__determinant, &ColladaParserAutoGen15Private::_data__determinant, &ColladaParserAutoGen15Private::_end__determinant, &ColladaParserAutoGen15Private::_preBegin__determinant, &ColladaParserAutoGen15Private::_preEnd__determinant, &ColladaParserAutoGen15Private::_freeAttributes__determinant);
mElementToNamespaceMap[HASH_ELEMENT_DETERMINANT] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_TRANSPOSE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__transpose, &ColladaParserAutoGen15Private::_data__transpose, &ColladaParserAutoGen15Private::_end__transpose, &ColladaParserAutoGen15Private::_preBegin__transpose, &ColladaParserAutoGen15Private::_preEnd__transpose, &ColladaParserAutoGen15Private::_freeAttributes__transpose);
mElementToNamespaceMap[HASH_ELEMENT_TRANSPOSE] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_SELECTOR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__selector, &ColladaParserAutoGen15Private::_data__selector, &ColladaParserAutoGen15Private::_end__selector, &ColladaParserAutoGen15Private::_preBegin__selector, &ColladaParserAutoGen15Private::_preEnd__selector, &ColladaParserAutoGen15Private::_freeAttributes__selector);
mElementToNamespaceMap[HASH_ELEMENT_SELECTOR] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_VECTORPRODUCT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__vectorproduct, &ColladaParserAutoGen15Private::_data__vectorproduct, &ColladaParserAutoGen15Private::_end__vectorproduct, &ColladaParserAutoGen15Private::_preBegin__vectorproduct, &ColladaParserAutoGen15Private::_preEnd__vectorproduct, &ColladaParserAutoGen15Private::_freeAttributes__vectorproduct);
mElementToNamespaceMap[HASH_ELEMENT_VECTORPRODUCT] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_SCALARPRODUCT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__scalarproduct, &ColladaParserAutoGen15Private::_data__scalarproduct, &ColladaParserAutoGen15Private::_end__scalarproduct, &ColladaParserAutoGen15Private::_preBegin__scalarproduct, &ColladaParserAutoGen15Private::_preEnd__scalarproduct, &ColladaParserAutoGen15Private::_freeAttributes__scalarproduct);
mElementToNamespaceMap[HASH_ELEMENT_SCALARPRODUCT] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_OUTERPRODUCT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__outerproduct, &ColladaParserAutoGen15Private::_data__outerproduct, &ColladaParserAutoGen15Private::_end__outerproduct, &ColladaParserAutoGen15Private::_preBegin__outerproduct, &ColladaParserAutoGen15Private::_preEnd__outerproduct, &ColladaParserAutoGen15Private::_freeAttributes__outerproduct);
mElementToNamespaceMap[HASH_ELEMENT_OUTERPRODUCT] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_DIVERGENCE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__divergence, &ColladaParserAutoGen15Private::_data__divergence, &ColladaParserAutoGen15Private::_end__divergence, &ColladaParserAutoGen15Private::_preBegin__divergence, &ColladaParserAutoGen15Private::_preEnd__divergence, &ColladaParserAutoGen15Private::_freeAttributes__divergence);
mElementToNamespaceMap[HASH_ELEMENT_DIVERGENCE] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_GRAD] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__grad, &ColladaParserAutoGen15Private::_data__grad, &ColladaParserAutoGen15Private::_end__grad, &ColladaParserAutoGen15Private::_preBegin__grad, &ColladaParserAutoGen15Private::_preEnd__grad, &ColladaParserAutoGen15Private::_freeAttributes__grad);
mElementToNamespaceMap[HASH_ELEMENT_GRAD] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_CURL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__curl, &ColladaParserAutoGen15Private::_data__curl, &ColladaParserAutoGen15Private::_end__curl, &ColladaParserAutoGen15Private::_preBegin__curl, &ColladaParserAutoGen15Private::_preEnd__curl, &ColladaParserAutoGen15Private::_freeAttributes__curl);
mElementToNamespaceMap[HASH_ELEMENT_CURL] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_LAPLACIAN] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__laplacian, &ColladaParserAutoGen15Private::_data__laplacian, &ColladaParserAutoGen15Private::_end__laplacian, &ColladaParserAutoGen15Private::_preBegin__laplacian, &ColladaParserAutoGen15Private::_preEnd__laplacian, &ColladaParserAutoGen15Private::_freeAttributes__laplacian);
mElementToNamespaceMap[HASH_ELEMENT_LAPLACIAN] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_MEAN] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__mean, &ColladaParserAutoGen15Private::_data__mean, &ColladaParserAutoGen15Private::_end__mean, &ColladaParserAutoGen15Private::_preBegin__mean, &ColladaParserAutoGen15Private::_preEnd__mean, &ColladaParserAutoGen15Private::_freeAttributes__mean);
mElementToNamespaceMap[HASH_ELEMENT_MEAN] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_SDEV] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__sdev, &ColladaParserAutoGen15Private::_data__sdev, &ColladaParserAutoGen15Private::_end__sdev, &ColladaParserAutoGen15Private::_preBegin__sdev, &ColladaParserAutoGen15Private::_preEnd__sdev, &ColladaParserAutoGen15Private::_freeAttributes__sdev);
mElementToNamespaceMap[HASH_ELEMENT_SDEV] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_VARIANCE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__variance, &ColladaParserAutoGen15Private::_data__variance, &ColladaParserAutoGen15Private::_end__variance, &ColladaParserAutoGen15Private::_preBegin__variance, &ColladaParserAutoGen15Private::_preEnd__variance, &ColladaParserAutoGen15Private::_freeAttributes__variance);
mElementToNamespaceMap[HASH_ELEMENT_VARIANCE] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_MEDIAN] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__median, &ColladaParserAutoGen15Private::_data__median, &ColladaParserAutoGen15Private::_end__median, &ColladaParserAutoGen15Private::_preBegin__median, &ColladaParserAutoGen15Private::_preEnd__median, &ColladaParserAutoGen15Private::_freeAttributes__median);
mElementToNamespaceMap[HASH_ELEMENT_MEDIAN] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_MODE____MODE_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__mode____mode_type, &ColladaParserAutoGen15Private::_data__mode____mode_type, &ColladaParserAutoGen15Private::_end__mode____mode_type, &ColladaParserAutoGen15Private::_preBegin__mode____mode_type, &ColladaParserAutoGen15Private::_preEnd__mode____mode_type, &ColladaParserAutoGen15Private::_freeAttributes__mode____mode_type);
mElementToNamespaceMap[HASH_ELEMENT_MODE____MODE_TYPE] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_MOMENT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__moment, &ColladaParserAutoGen15Private::_data__moment, &ColladaParserAutoGen15Private::_end__moment, &ColladaParserAutoGen15Private::_preBegin__moment, &ColladaParserAutoGen15Private::_preEnd__moment, &ColladaParserAutoGen15Private::_freeAttributes__moment);
mElementToNamespaceMap[HASH_ELEMENT_MOMENT] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_MOMENTABOUT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__momentabout, &ColladaParserAutoGen15Private::_data__momentabout, &ColladaParserAutoGen15Private::_end__momentabout, &ColladaParserAutoGen15Private::_preBegin__momentabout, &ColladaParserAutoGen15Private::_preEnd__momentabout, &ColladaParserAutoGen15Private::_freeAttributes__momentabout);
mElementToNamespaceMap[HASH_ELEMENT_MOMENTABOUT] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_APPLY] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__apply, &ColladaParserAutoGen15Private::_data__apply, &ColladaParserAutoGen15Private::_end__apply, &ColladaParserAutoGen15Private::_preBegin__apply, &ColladaParserAutoGen15Private::_preEnd__apply, &ColladaParserAutoGen15Private::_freeAttributes__apply);
mElementToNamespaceMap[HASH_ELEMENT_APPLY] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_INTERVAL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__interval, &ColladaParserAutoGen15Private::_data__interval, &ColladaParserAutoGen15Private::_end__interval, &ColladaParserAutoGen15Private::_preBegin__interval, &ColladaParserAutoGen15Private::_preEnd__interval, &ColladaParserAutoGen15Private::_freeAttributes__interval);
mElementToNamespaceMap[HASH_ELEMENT_INTERVAL] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_INVERSE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__inverse, &ColladaParserAutoGen15Private::_data__inverse, &ColladaParserAutoGen15Private::_end__inverse, &ColladaParserAutoGen15Private::_preBegin__inverse, &ColladaParserAutoGen15Private::_preEnd__inverse, &ColladaParserAutoGen15Private::_freeAttributes__inverse);
mElementToNamespaceMap[HASH_ELEMENT_INVERSE] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_CONDITION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__condition, &ColladaParserAutoGen15Private::_data__condition, &ColladaParserAutoGen15Private::_end__condition, &ColladaParserAutoGen15Private::_preBegin__condition, &ColladaParserAutoGen15Private::_preEnd__condition, &ColladaParserAutoGen15Private::_freeAttributes__condition);
mElementToNamespaceMap[HASH_ELEMENT_CONDITION] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_DECLARE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__declare, &ColladaParserAutoGen15Private::_data__declare, &ColladaParserAutoGen15Private::_end__declare, &ColladaParserAutoGen15Private::_preBegin__declare, &ColladaParserAutoGen15Private::_preEnd__declare, &ColladaParserAutoGen15Private::_freeAttributes__declare);
mElementToNamespaceMap[HASH_ELEMENT_DECLARE] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_LAMBDA] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__lambda, &ColladaParserAutoGen15Private::_data__lambda, &ColladaParserAutoGen15Private::_end__lambda, &ColladaParserAutoGen15Private::_preBegin__lambda, &ColladaParserAutoGen15Private::_preEnd__lambda, &ColladaParserAutoGen15Private::_freeAttributes__lambda);
mElementToNamespaceMap[HASH_ELEMENT_LAMBDA] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_PIECEWISE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__piecewise, &ColladaParserAutoGen15Private::_data__piecewise, &ColladaParserAutoGen15Private::_end__piecewise, &ColladaParserAutoGen15Private::_preBegin__piecewise, &ColladaParserAutoGen15Private::_preEnd__piecewise, &ColladaParserAutoGen15Private::_freeAttributes__piecewise);
mElementToNamespaceMap[HASH_ELEMENT_PIECEWISE] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_PIECE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__piece, &ColladaParserAutoGen15Private::_data__piece, &ColladaParserAutoGen15Private::_end__piece, &ColladaParserAutoGen15Private::_preBegin__piece, &ColladaParserAutoGen15Private::_preEnd__piece, &ColladaParserAutoGen15Private::_freeAttributes__piece);
mElementToNamespaceMap[HASH_ELEMENT_PIECE] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_BVAR] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__bvar, &ColladaParserAutoGen15Private::_data__bvar, &ColladaParserAutoGen15Private::_end__bvar, &ColladaParserAutoGen15Private::_preBegin__bvar, &ColladaParserAutoGen15Private::_preEnd__bvar, &ColladaParserAutoGen15Private::_freeAttributes__bvar);
mElementToNamespaceMap[HASH_ELEMENT_BVAR] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_DEGREE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__degree, &ColladaParserAutoGen15Private::_data__degree, &ColladaParserAutoGen15Private::_end__degree, &ColladaParserAutoGen15Private::_preBegin__degree, &ColladaParserAutoGen15Private::_preEnd__degree, &ColladaParserAutoGen15Private::_freeAttributes__degree);
mElementToNamespaceMap[HASH_ELEMENT_DEGREE] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_SEMANTICS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__semantics, &ColladaParserAutoGen15Private::_data__semantics, &ColladaParserAutoGen15Private::_end__semantics, &ColladaParserAutoGen15Private::_preBegin__semantics, &ColladaParserAutoGen15Private::_preEnd__semantics, &ColladaParserAutoGen15Private::_freeAttributes__semantics);
mElementToNamespaceMap[HASH_ELEMENT_SEMANTICS] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_ANNOTATION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__annotation, &ColladaParserAutoGen15Private::_data__annotation, &ColladaParserAutoGen15Private::_end__annotation, &ColladaParserAutoGen15Private::_preBegin__annotation, &ColladaParserAutoGen15Private::_preEnd__annotation, &ColladaParserAutoGen15Private::_freeAttributes__annotation);
mElementToNamespaceMap[HASH_ELEMENT_ANNOTATION] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_ANNOTATION_XML] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__annotation_xml, &ColladaParserAutoGen15Private::_data__annotation_xml, &ColladaParserAutoGen15Private::_end__annotation_xml, &ColladaParserAutoGen15Private::_preBegin__annotation_xml, &ColladaParserAutoGen15Private::_preEnd__annotation_xml, &ColladaParserAutoGen15Private::_freeAttributes__annotation_xml);
mElementToNamespaceMap[HASH_ELEMENT_ANNOTATION_XML] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_OTHERWISE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__otherwise, &ColladaParserAutoGen15Private::_data__otherwise, &ColladaParserAutoGen15Private::_end__otherwise, &ColladaParserAutoGen15Private::_preBegin__otherwise, &ColladaParserAutoGen15Private::_preEnd__otherwise, &ColladaParserAutoGen15Private::_freeAttributes__otherwise);
mElementToNamespaceMap[HASH_ELEMENT_OTHERWISE] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_SEP] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__sep, &ColladaParserAutoGen15Private::_data__sep, &ColladaParserAutoGen15Private::_end__sep, &ColladaParserAutoGen15Private::_preBegin__sep, &ColladaParserAutoGen15Private::_preEnd__sep, &ColladaParserAutoGen15Private::_freeAttributes__sep);
mElementToNamespaceMap[HASH_ELEMENT_SEP] = HASH_NAMESPACE_mathml;
mElementFunctionMap[HASH_ELEMENT_LIBRARY_ARTICULATED_SYSTEMS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_articulated_systems, &ColladaParserAutoGen15Private::_data__library_articulated_systems, &ColladaParserAutoGen15Private::_end__library_articulated_systems, &ColladaParserAutoGen15Private::_preBegin__library_articulated_systems, &ColladaParserAutoGen15Private::_preEnd__library_articulated_systems, &ColladaParserAutoGen15Private::_freeAttributes__library_articulated_systems);
mElementToNamespaceMap[HASH_ELEMENT_LIBRARY_ARTICULATED_SYSTEMS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ARTICULATED_SYSTEM] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__articulated_system, &ColladaParserAutoGen15Private::_data__articulated_system, &ColladaParserAutoGen15Private::_end__articulated_system, &ColladaParserAutoGen15Private::_preBegin__articulated_system, &ColladaParserAutoGen15Private::_preEnd__articulated_system, &ColladaParserAutoGen15Private::_freeAttributes__articulated_system);
mElementToNamespaceMap[HASH_ELEMENT_ARTICULATED_SYSTEM] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_KINEMATICS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__kinematics, &ColladaParserAutoGen15Private::_data__kinematics, &ColladaParserAutoGen15Private::_end__kinematics, &ColladaParserAutoGen15Private::_preBegin__kinematics, &ColladaParserAutoGen15Private::_preEnd__kinematics, &ColladaParserAutoGen15Private::_freeAttributes__kinematics);
mElementToNamespaceMap[HASH_ELEMENT_KINEMATICS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INSTANCE_KINEMATICS_MODEL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_kinematics_model, &ColladaParserAutoGen15Private::_data__instance_kinematics_model, &ColladaParserAutoGen15Private::_end__instance_kinematics_model, &ColladaParserAutoGen15Private::_preBegin__instance_kinematics_model, &ColladaParserAutoGen15Private::_preEnd__instance_kinematics_model, &ColladaParserAutoGen15Private::_freeAttributes__instance_kinematics_model);
mElementToNamespaceMap[HASH_ELEMENT_INSTANCE_KINEMATICS_MODEL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BIND____KINEMATICS_BIND_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__bind____kinematics_bind_type, &ColladaParserAutoGen15Private::_data__bind____kinematics_bind_type, &ColladaParserAutoGen15Private::_end__bind____kinematics_bind_type, &ColladaParserAutoGen15Private::_preBegin__bind____kinematics_bind_type, &ColladaParserAutoGen15Private::_preEnd__bind____kinematics_bind_type, &ColladaParserAutoGen15Private::_freeAttributes__bind____kinematics_bind_type);
mElementToNamespaceMap[HASH_ELEMENT_BIND____KINEMATICS_BIND_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_PARAM____KINEMATICS_PARAM_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__param____kinematics_param_type, &ColladaParserAutoGen15Private::_data__param____kinematics_param_type, &ColladaParserAutoGen15Private::_end__param____kinematics_param_type, &ColladaParserAutoGen15Private::_preBegin__param____kinematics_param_type, &ColladaParserAutoGen15Private::_preEnd__param____kinematics_param_type, &ColladaParserAutoGen15Private::_freeAttributes__param____kinematics_param_type);
mElementToNamespaceMap[HASH_ELEMENT_PARAM____KINEMATICS_PARAM_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SETPARAM____KINEMATICS_SETPARAM_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__setparam____kinematics_setparam_type, &ColladaParserAutoGen15Private::_data__setparam____kinematics_setparam_type, &ColladaParserAutoGen15Private::_end__setparam____kinematics_setparam_type, &ColladaParserAutoGen15Private::_preBegin__setparam____kinematics_setparam_type, &ColladaParserAutoGen15Private::_preEnd__setparam____kinematics_setparam_type, &ColladaParserAutoGen15Private::_freeAttributes__setparam____kinematics_setparam_type);
mElementToNamespaceMap[HASH_ELEMENT_SETPARAM____KINEMATICS_SETPARAM_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TECHNIQUE_COMMON____KINEMATICS_TECHNIQUE_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique_common____kinematics_technique_type, &ColladaParserAutoGen15Private::_data__technique_common____kinematics_technique_type, &ColladaParserAutoGen15Private::_end__technique_common____kinematics_technique_type, &ColladaParserAutoGen15Private::_preBegin__technique_common____kinematics_technique_type, &ColladaParserAutoGen15Private::_preEnd__technique_common____kinematics_technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique_common____kinematics_technique_type);
mElementToNamespaceMap[HASH_ELEMENT_TECHNIQUE_COMMON____KINEMATICS_TECHNIQUE_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_AXIS_INFO____KINEMATICS_AXIS_INFO_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__axis_info____kinematics_axis_info_type, &ColladaParserAutoGen15Private::_data__axis_info____kinematics_axis_info_type, &ColladaParserAutoGen15Private::_end__axis_info____kinematics_axis_info_type, &ColladaParserAutoGen15Private::_preBegin__axis_info____kinematics_axis_info_type, &ColladaParserAutoGen15Private::_preEnd__axis_info____kinematics_axis_info_type, &ColladaParserAutoGen15Private::_freeAttributes__axis_info____kinematics_axis_info_type);
mElementToNamespaceMap[HASH_ELEMENT_AXIS_INFO____KINEMATICS_AXIS_INFO_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ACTIVE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__active, &ColladaParserAutoGen15Private::_data__active, &ColladaParserAutoGen15Private::_end__active, &ColladaParserAutoGen15Private::_preBegin__active, &ColladaParserAutoGen15Private::_preEnd__active, &ColladaParserAutoGen15Private::_freeAttributes__active);
mElementToNamespaceMap[HASH_ELEMENT_ACTIVE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LOCKED] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__locked, &ColladaParserAutoGen15Private::_data__locked, &ColladaParserAutoGen15Private::_end__locked, &ColladaParserAutoGen15Private::_preBegin__locked, &ColladaParserAutoGen15Private::_preEnd__locked, &ColladaParserAutoGen15Private::_freeAttributes__locked);
mElementToNamespaceMap[HASH_ELEMENT_LOCKED] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INDEX] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__index, &ColladaParserAutoGen15Private::_data__index, &ColladaParserAutoGen15Private::_end__index, &ColladaParserAutoGen15Private::_preBegin__index, &ColladaParserAutoGen15Private::_preEnd__index, &ColladaParserAutoGen15Private::_freeAttributes__index);
mElementToNamespaceMap[HASH_ELEMENT_INDEX] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LIMITS____KINEMATICS_LIMITS_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__limits____kinematics_limits_type, &ColladaParserAutoGen15Private::_data__limits____kinematics_limits_type, &ColladaParserAutoGen15Private::_end__limits____kinematics_limits_type, &ColladaParserAutoGen15Private::_preBegin__limits____kinematics_limits_type, &ColladaParserAutoGen15Private::_preEnd__limits____kinematics_limits_type, &ColladaParserAutoGen15Private::_freeAttributes__limits____kinematics_limits_type);
mElementToNamespaceMap[HASH_ELEMENT_LIMITS____KINEMATICS_LIMITS_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_MIN____COMMON_FLOAT_OR_PARAM_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__min____common_float_or_param_type, &ColladaParserAutoGen15Private::_data__min____common_float_or_param_type, &ColladaParserAutoGen15Private::_end__min____common_float_or_param_type, &ColladaParserAutoGen15Private::_preBegin__min____common_float_or_param_type, &ColladaParserAutoGen15Private::_preEnd__min____common_float_or_param_type, &ColladaParserAutoGen15Private::_freeAttributes__min____common_float_or_param_type);
mElementToNamespaceMap[HASH_ELEMENT_MIN____COMMON_FLOAT_OR_PARAM_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_MAX____COMMON_FLOAT_OR_PARAM_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__max____common_float_or_param_type, &ColladaParserAutoGen15Private::_data__max____common_float_or_param_type, &ColladaParserAutoGen15Private::_end__max____common_float_or_param_type, &ColladaParserAutoGen15Private::_preBegin__max____common_float_or_param_type, &ColladaParserAutoGen15Private::_preEnd__max____common_float_or_param_type, &ColladaParserAutoGen15Private::_freeAttributes__max____common_float_or_param_type);
mElementToNamespaceMap[HASH_ELEMENT_MAX____COMMON_FLOAT_OR_PARAM_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FRAME_ORIGIN] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__frame_origin, &ColladaParserAutoGen15Private::_data__frame_origin, &ColladaParserAutoGen15Private::_end__frame_origin, &ColladaParserAutoGen15Private::_preBegin__frame_origin, &ColladaParserAutoGen15Private::_preEnd__frame_origin, &ColladaParserAutoGen15Private::_freeAttributes__frame_origin);
mElementToNamespaceMap[HASH_ELEMENT_FRAME_ORIGIN] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FRAME_TIP] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__frame_tip, &ColladaParserAutoGen15Private::_data__frame_tip, &ColladaParserAutoGen15Private::_end__frame_tip, &ColladaParserAutoGen15Private::_preBegin__frame_tip, &ColladaParserAutoGen15Private::_preEnd__frame_tip, &ColladaParserAutoGen15Private::_freeAttributes__frame_tip);
mElementToNamespaceMap[HASH_ELEMENT_FRAME_TIP] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FRAME_TCP] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__frame_tcp, &ColladaParserAutoGen15Private::_data__frame_tcp, &ColladaParserAutoGen15Private::_end__frame_tcp, &ColladaParserAutoGen15Private::_preBegin__frame_tcp, &ColladaParserAutoGen15Private::_preEnd__frame_tcp, &ColladaParserAutoGen15Private::_freeAttributes__frame_tcp);
mElementToNamespaceMap[HASH_ELEMENT_FRAME_TCP] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_FRAME_OBJECT] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__frame_object, &ColladaParserAutoGen15Private::_data__frame_object, &ColladaParserAutoGen15Private::_end__frame_object, &ColladaParserAutoGen15Private::_preBegin__frame_object, &ColladaParserAutoGen15Private::_preEnd__frame_object, &ColladaParserAutoGen15Private::_freeAttributes__frame_object);
mElementToNamespaceMap[HASH_ELEMENT_FRAME_OBJECT] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_MOTION] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__motion, &ColladaParserAutoGen15Private::_data__motion, &ColladaParserAutoGen15Private::_end__motion, &ColladaParserAutoGen15Private::_preBegin__motion, &ColladaParserAutoGen15Private::_preEnd__motion, &ColladaParserAutoGen15Private::_freeAttributes__motion);
mElementToNamespaceMap[HASH_ELEMENT_MOTION] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INSTANCE_ARTICULATED_SYSTEM] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_articulated_system, &ColladaParserAutoGen15Private::_data__instance_articulated_system, &ColladaParserAutoGen15Private::_end__instance_articulated_system, &ColladaParserAutoGen15Private::_preBegin__instance_articulated_system, &ColladaParserAutoGen15Private::_preEnd__instance_articulated_system, &ColladaParserAutoGen15Private::_freeAttributes__instance_articulated_system);
mElementToNamespaceMap[HASH_ELEMENT_INSTANCE_ARTICULATED_SYSTEM] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_TECHNIQUE_COMMON____MOTION_TECHNIQUE_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique_common____motion_technique_type, &ColladaParserAutoGen15Private::_data__technique_common____motion_technique_type, &ColladaParserAutoGen15Private::_end__technique_common____motion_technique_type, &ColladaParserAutoGen15Private::_preBegin__technique_common____motion_technique_type, &ColladaParserAutoGen15Private::_preEnd__technique_common____motion_technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique_common____motion_technique_type);
mElementToNamespaceMap[HASH_ELEMENT_TECHNIQUE_COMMON____MOTION_TECHNIQUE_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_AXIS_INFO____MOTION_AXIS_INFO_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__axis_info____motion_axis_info_type, &ColladaParserAutoGen15Private::_data__axis_info____motion_axis_info_type, &ColladaParserAutoGen15Private::_end__axis_info____motion_axis_info_type, &ColladaParserAutoGen15Private::_preBegin__axis_info____motion_axis_info_type, &ColladaParserAutoGen15Private::_preEnd__axis_info____motion_axis_info_type, &ColladaParserAutoGen15Private::_freeAttributes__axis_info____motion_axis_info_type);
mElementToNamespaceMap[HASH_ELEMENT_AXIS_INFO____MOTION_AXIS_INFO_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SPEED____COMMON_FLOAT_OR_PARAM_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__speed____common_float_or_param_type, &ColladaParserAutoGen15Private::_data__speed____common_float_or_param_type, &ColladaParserAutoGen15Private::_end__speed____common_float_or_param_type, &ColladaParserAutoGen15Private::_preBegin__speed____common_float_or_param_type, &ColladaParserAutoGen15Private::_preEnd__speed____common_float_or_param_type, &ColladaParserAutoGen15Private::_freeAttributes__speed____common_float_or_param_type);
mElementToNamespaceMap[HASH_ELEMENT_SPEED____COMMON_FLOAT_OR_PARAM_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ACCELERATION____COMMON_FLOAT_OR_PARAM_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__acceleration____common_float_or_param_type, &ColladaParserAutoGen15Private::_data__acceleration____common_float_or_param_type, &ColladaParserAutoGen15Private::_end__acceleration____common_float_or_param_type, &ColladaParserAutoGen15Private::_preBegin__acceleration____common_float_or_param_type, &ColladaParserAutoGen15Private::_preEnd__acceleration____common_float_or_param_type, &ColladaParserAutoGen15Private::_freeAttributes__acceleration____common_float_or_param_type);
mElementToNamespaceMap[HASH_ELEMENT_ACCELERATION____COMMON_FLOAT_OR_PARAM_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_DECELERATION____COMMON_FLOAT_OR_PARAM_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__deceleration____common_float_or_param_type, &ColladaParserAutoGen15Private::_data__deceleration____common_float_or_param_type, &ColladaParserAutoGen15Private::_end__deceleration____common_float_or_param_type, &ColladaParserAutoGen15Private::_preBegin__deceleration____common_float_or_param_type, &ColladaParserAutoGen15Private::_preEnd__deceleration____common_float_or_param_type, &ColladaParserAutoGen15Private::_freeAttributes__deceleration____common_float_or_param_type);
mElementToNamespaceMap[HASH_ELEMENT_DECELERATION____COMMON_FLOAT_OR_PARAM_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_JERK____COMMON_FLOAT_OR_PARAM_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__jerk____common_float_or_param_type, &ColladaParserAutoGen15Private::_data__jerk____common_float_or_param_type, &ColladaParserAutoGen15Private::_end__jerk____common_float_or_param_type, &ColladaParserAutoGen15Private::_preBegin__jerk____common_float_or_param_type, &ColladaParserAutoGen15Private::_preEnd__jerk____common_float_or_param_type, &ColladaParserAutoGen15Private::_freeAttributes__jerk____common_float_or_param_type);
mElementToNamespaceMap[HASH_ELEMENT_JERK____COMMON_FLOAT_OR_PARAM_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_EFFECTOR_INFO] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__effector_info, &ColladaParserAutoGen15Private::_data__effector_info, &ColladaParserAutoGen15Private::_end__effector_info, &ColladaParserAutoGen15Private::_preBegin__effector_info, &ColladaParserAutoGen15Private::_preEnd__effector_info, &ColladaParserAutoGen15Private::_freeAttributes__effector_info);
mElementToNamespaceMap[HASH_ELEMENT_EFFECTOR_INFO] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SPEED____COMMON_FLOAT2_OR_PARAM_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__speed____common_float2_or_param_type, &ColladaParserAutoGen15Private::_data__speed____common_float2_or_param_type, &ColladaParserAutoGen15Private::_end__speed____common_float2_or_param_type, &ColladaParserAutoGen15Private::_preBegin__speed____common_float2_or_param_type, &ColladaParserAutoGen15Private::_preEnd__speed____common_float2_or_param_type, &ColladaParserAutoGen15Private::_freeAttributes__speed____common_float2_or_param_type);
mElementToNamespaceMap[HASH_ELEMENT_SPEED____COMMON_FLOAT2_OR_PARAM_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_ACCELERATION____COMMON_FLOAT2_OR_PARAM_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__acceleration____common_float2_or_param_type, &ColladaParserAutoGen15Private::_data__acceleration____common_float2_or_param_type, &ColladaParserAutoGen15Private::_end__acceleration____common_float2_or_param_type, &ColladaParserAutoGen15Private::_preBegin__acceleration____common_float2_or_param_type, &ColladaParserAutoGen15Private::_preEnd__acceleration____common_float2_or_param_type, &ColladaParserAutoGen15Private::_freeAttributes__acceleration____common_float2_or_param_type);
mElementToNamespaceMap[HASH_ELEMENT_ACCELERATION____COMMON_FLOAT2_OR_PARAM_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_DECELERATION____COMMON_FLOAT2_OR_PARAM_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__deceleration____common_float2_or_param_type, &ColladaParserAutoGen15Private::_data__deceleration____common_float2_or_param_type, &ColladaParserAutoGen15Private::_end__deceleration____common_float2_or_param_type, &ColladaParserAutoGen15Private::_preBegin__deceleration____common_float2_or_param_type, &ColladaParserAutoGen15Private::_preEnd__deceleration____common_float2_or_param_type, &ColladaParserAutoGen15Private::_freeAttributes__deceleration____common_float2_or_param_type);
mElementToNamespaceMap[HASH_ELEMENT_DECELERATION____COMMON_FLOAT2_OR_PARAM_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_JERK____COMMON_FLOAT2_OR_PARAM_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__jerk____common_float2_or_param_type, &ColladaParserAutoGen15Private::_data__jerk____common_float2_or_param_type, &ColladaParserAutoGen15Private::_end__jerk____common_float2_or_param_type, &ColladaParserAutoGen15Private::_preBegin__jerk____common_float2_or_param_type, &ColladaParserAutoGen15Private::_preEnd__jerk____common_float2_or_param_type, &ColladaParserAutoGen15Private::_freeAttributes__jerk____common_float2_or_param_type);
mElementToNamespaceMap[HASH_ELEMENT_JERK____COMMON_FLOAT2_OR_PARAM_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LIBRARY_KINEMATICS_SCENES] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_kinematics_scenes, &ColladaParserAutoGen15Private::_data__library_kinematics_scenes, &ColladaParserAutoGen15Private::_end__library_kinematics_scenes, &ColladaParserAutoGen15Private::_preBegin__library_kinematics_scenes, &ColladaParserAutoGen15Private::_preEnd__library_kinematics_scenes, &ColladaParserAutoGen15Private::_freeAttributes__library_kinematics_scenes);
mElementToNamespaceMap[HASH_ELEMENT_LIBRARY_KINEMATICS_SCENES] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_KINEMATICS_SCENE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__kinematics_scene, &ColladaParserAutoGen15Private::_data__kinematics_scene, &ColladaParserAutoGen15Private::_end__kinematics_scene, &ColladaParserAutoGen15Private::_preBegin__kinematics_scene, &ColladaParserAutoGen15Private::_preEnd__kinematics_scene, &ColladaParserAutoGen15Private::_freeAttributes__kinematics_scene);
mElementToNamespaceMap[HASH_ELEMENT_KINEMATICS_SCENE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_LIBRARY_FORMULAS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_formulas, &ColladaParserAutoGen15Private::_data__library_formulas, &ColladaParserAutoGen15Private::_end__library_formulas, &ColladaParserAutoGen15Private::_preBegin__library_formulas, &ColladaParserAutoGen15Private::_preEnd__library_formulas, &ColladaParserAutoGen15Private::_freeAttributes__library_formulas);
mElementToNamespaceMap[HASH_ELEMENT_LIBRARY_FORMULAS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_SCENE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__scene, &ColladaParserAutoGen15Private::_data__scene, &ColladaParserAutoGen15Private::_end__scene, &ColladaParserAutoGen15Private::_preBegin__scene, &ColladaParserAutoGen15Private::_preEnd__scene, &ColladaParserAutoGen15Private::_freeAttributes__scene);
mElementToNamespaceMap[HASH_ELEMENT_SCENE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INSTANCE_PHYSICS_SCENE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_physics_scene, &ColladaParserAutoGen15Private::_data__instance_physics_scene, &ColladaParserAutoGen15Private::_end__instance_physics_scene, &ColladaParserAutoGen15Private::_preBegin__instance_physics_scene, &ColladaParserAutoGen15Private::_preEnd__instance_physics_scene, &ColladaParserAutoGen15Private::_freeAttributes__instance_physics_scene);
mElementToNamespaceMap[HASH_ELEMENT_INSTANCE_PHYSICS_SCENE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INSTANCE_VISUAL_SCENE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_visual_scene, &ColladaParserAutoGen15Private::_data__instance_visual_scene, &ColladaParserAutoGen15Private::_end__instance_visual_scene, &ColladaParserAutoGen15Private::_preBegin__instance_visual_scene, &ColladaParserAutoGen15Private::_preEnd__instance_visual_scene, &ColladaParserAutoGen15Private::_freeAttributes__instance_visual_scene);
mElementToNamespaceMap[HASH_ELEMENT_INSTANCE_VISUAL_SCENE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_INSTANCE_KINEMATICS_SCENE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_kinematics_scene, &ColladaParserAutoGen15Private::_data__instance_kinematics_scene, &ColladaParserAutoGen15Private::_end__instance_kinematics_scene, &ColladaParserAutoGen15Private::_preBegin__instance_kinematics_scene, &ColladaParserAutoGen15Private::_preEnd__instance_kinematics_scene, &ColladaParserAutoGen15Private::_freeAttributes__instance_kinematics_scene);
mElementToNamespaceMap[HASH_ELEMENT_INSTANCE_KINEMATICS_SCENE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BIND_KINEMATICS_MODEL] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__bind_kinematics_model, &ColladaParserAutoGen15Private::_data__bind_kinematics_model, &ColladaParserAutoGen15Private::_end__bind_kinematics_model, &ColladaParserAutoGen15Private::_preBegin__bind_kinematics_model, &ColladaParserAutoGen15Private::_preEnd__bind_kinematics_model, &ColladaParserAutoGen15Private::_freeAttributes__bind_kinematics_model);
mElementToNamespaceMap[HASH_ELEMENT_BIND_KINEMATICS_MODEL] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_BIND_JOINT_AXIS] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__bind_joint_axis, &ColladaParserAutoGen15Private::_data__bind_joint_axis, &ColladaParserAutoGen15Private::_end__bind_joint_axis, &ColladaParserAutoGen15Private::_preBegin__bind_joint_axis, &ColladaParserAutoGen15Private::_preEnd__bind_joint_axis, &ColladaParserAutoGen15Private::_freeAttributes__bind_joint_axis);
mElementToNamespaceMap[HASH_ELEMENT_BIND_JOINT_AXIS] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_AXIS____COMMON_SIDREF_OR_PARAM_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__axis____common_sidref_or_param_type, &ColladaParserAutoGen15Private::_data__axis____common_sidref_or_param_type, &ColladaParserAutoGen15Private::_end__axis____common_sidref_or_param_type, &ColladaParserAutoGen15Private::_preBegin__axis____common_sidref_or_param_type, &ColladaParserAutoGen15Private::_preEnd__axis____common_sidref_or_param_type, &ColladaParserAutoGen15Private::_freeAttributes__axis____common_sidref_or_param_type);
mElementToNamespaceMap[HASH_ELEMENT_AXIS____COMMON_SIDREF_OR_PARAM_TYPE] = HASH_NAMESPACE_COLLADA;
mElementFunctionMap[HASH_ELEMENT_VALUE____COMMON_FLOAT_OR_PARAM_TYPE] = FunctionStruct(&ColladaParserAutoGen15Private::_begin__value____common_float_or_param_type, &ColladaParserAutoGen15Private::_data__value____common_float_or_param_type, &ColladaParserAutoGen15Private::_end__value____common_float_or_param_type, &ColladaParserAutoGen15Private::_preBegin__value____common_float_or_param_type, &ColladaParserAutoGen15Private::_preEnd__value____common_float_or_param_type, &ColladaParserAutoGen15Private::_freeAttributes__value____common_float_or_param_type);
mElementToNamespaceMap[HASH_ELEMENT_VALUE____COMMON_FLOAT_OR_PARAM_TYPE] = HASH_NAMESPACE_COLLADA;
}


} // namespace
