/*
    Copyright (c) 2008-2009 NetAllied Systems GmbH

    This file is part of COLLADASaxFrameworkLoader.

    Licensed under the MIT Open Source License,
    for details please see LICENSE file or the website
    http://www.opensource.org/licenses/mit-license.php
*/


#include "COLLADASaxFWLStableHeaders.h"
#include "COLLADASaxFWLColladaParserAutoGen15FunctionMapFactory.h"

namespace COLLADASaxFWL15
{

//---------------------------------------------------------------------
const ColladaParserAutoGen15Private::ElementFunctionMap& FunctionMapFactory::createFunctionMap__COLLADA__1level() {
static ColladaParserAutoGen15Private::ElementFunctionMap functionMap;
static bool mapFilled = false;
if ( !mapFilled )
{
functionMap[HASH_ELEMENT_COLLADA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__COLLADA, &ColladaParserAutoGen15Private::_data__COLLADA, &ColladaParserAutoGen15Private::_end__COLLADA, &ColladaParserAutoGen15Private::_preBegin__COLLADA, &ColladaParserAutoGen15Private::_preEnd__COLLADA, &ColladaParserAutoGen15Private::_freeAttributes__COLLADA);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_animation_clips, &ColladaParserAutoGen15Private::_data__library_animation_clips, &ColladaParserAutoGen15Private::_end__library_animation_clips, &ColladaParserAutoGen15Private::_preBegin__library_animation_clips, &ColladaParserAutoGen15Private::_preEnd__library_animation_clips, &ColladaParserAutoGen15Private::_freeAttributes__library_animation_clips);
functionMap[HASH_ELEMENT_LIBRARY_ANIMATIONS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_animations, &ColladaParserAutoGen15Private::_data__library_animations, &ColladaParserAutoGen15Private::_end__library_animations, &ColladaParserAutoGen15Private::_preBegin__library_animations, &ColladaParserAutoGen15Private::_preEnd__library_animations, &ColladaParserAutoGen15Private::_freeAttributes__library_animations);
functionMap[HASH_ELEMENT_LIBRARY_ARTICULATED_SYSTEMS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_articulated_systems, &ColladaParserAutoGen15Private::_data__library_articulated_systems, &ColladaParserAutoGen15Private::_end__library_articulated_systems, &ColladaParserAutoGen15Private::_preBegin__library_articulated_systems, &ColladaParserAutoGen15Private::_preEnd__library_articulated_systems, &ColladaParserAutoGen15Private::_freeAttributes__library_articulated_systems);
functionMap[HASH_ELEMENT_LIBRARY_CAMERAS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_cameras, &ColladaParserAutoGen15Private::_data__library_cameras, &ColladaParserAutoGen15Private::_end__library_cameras, &ColladaParserAutoGen15Private::_preBegin__library_cameras, &ColladaParserAutoGen15Private::_preEnd__library_cameras, &ColladaParserAutoGen15Private::_freeAttributes__library_cameras);
functionMap[HASH_ELEMENT_LIBRARY_CONTROLLERS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_controllers, &ColladaParserAutoGen15Private::_data__library_controllers, &ColladaParserAutoGen15Private::_end__library_controllers, &ColladaParserAutoGen15Private::_preBegin__library_controllers, &ColladaParserAutoGen15Private::_preEnd__library_controllers, &ColladaParserAutoGen15Private::_freeAttributes__library_controllers);
functionMap[HASH_ELEMENT_LIBRARY_EFFECTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_effects, &ColladaParserAutoGen15Private::_data__library_effects, &ColladaParserAutoGen15Private::_end__library_effects, &ColladaParserAutoGen15Private::_preBegin__library_effects, &ColladaParserAutoGen15Private::_preEnd__library_effects, &ColladaParserAutoGen15Private::_freeAttributes__library_effects);
functionMap[HASH_ELEMENT_LIBRARY_FORCE_FIELDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_force_fields, &ColladaParserAutoGen15Private::_data__library_force_fields, &ColladaParserAutoGen15Private::_end__library_force_fields, &ColladaParserAutoGen15Private::_preBegin__library_force_fields, &ColladaParserAutoGen15Private::_preEnd__library_force_fields, &ColladaParserAutoGen15Private::_freeAttributes__library_force_fields);
functionMap[HASH_ELEMENT_LIBRARY_FORMULAS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_formulas, &ColladaParserAutoGen15Private::_data__library_formulas, &ColladaParserAutoGen15Private::_end__library_formulas, &ColladaParserAutoGen15Private::_preBegin__library_formulas, &ColladaParserAutoGen15Private::_preEnd__library_formulas, &ColladaParserAutoGen15Private::_freeAttributes__library_formulas);
functionMap[HASH_ELEMENT_LIBRARY_GEOMETRIES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_geometries, &ColladaParserAutoGen15Private::_data__library_geometries, &ColladaParserAutoGen15Private::_end__library_geometries, &ColladaParserAutoGen15Private::_preBegin__library_geometries, &ColladaParserAutoGen15Private::_preEnd__library_geometries, &ColladaParserAutoGen15Private::_freeAttributes__library_geometries);
functionMap[HASH_ELEMENT_LIBRARY_IMAGES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_images, &ColladaParserAutoGen15Private::_data__library_images, &ColladaParserAutoGen15Private::_end__library_images, &ColladaParserAutoGen15Private::_preBegin__library_images, &ColladaParserAutoGen15Private::_preEnd__library_images, &ColladaParserAutoGen15Private::_freeAttributes__library_images);
functionMap[HASH_ELEMENT_LIBRARY_JOINTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_joints, &ColladaParserAutoGen15Private::_data__library_joints, &ColladaParserAutoGen15Private::_end__library_joints, &ColladaParserAutoGen15Private::_preBegin__library_joints, &ColladaParserAutoGen15Private::_preEnd__library_joints, &ColladaParserAutoGen15Private::_freeAttributes__library_joints);
functionMap[HASH_ELEMENT_LIBRARY_KINEMATICS_MODELS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_kinematics_models, &ColladaParserAutoGen15Private::_data__library_kinematics_models, &ColladaParserAutoGen15Private::_end__library_kinematics_models, &ColladaParserAutoGen15Private::_preBegin__library_kinematics_models, &ColladaParserAutoGen15Private::_preEnd__library_kinematics_models, &ColladaParserAutoGen15Private::_freeAttributes__library_kinematics_models);
functionMap[HASH_ELEMENT_LIBRARY_KINEMATICS_SCENES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_kinematics_scenes, &ColladaParserAutoGen15Private::_data__library_kinematics_scenes, &ColladaParserAutoGen15Private::_end__library_kinematics_scenes, &ColladaParserAutoGen15Private::_preBegin__library_kinematics_scenes, &ColladaParserAutoGen15Private::_preEnd__library_kinematics_scenes, &ColladaParserAutoGen15Private::_freeAttributes__library_kinematics_scenes);
functionMap[HASH_ELEMENT_LIBRARY_LIGHTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_lights, &ColladaParserAutoGen15Private::_data__library_lights, &ColladaParserAutoGen15Private::_end__library_lights, &ColladaParserAutoGen15Private::_preBegin__library_lights, &ColladaParserAutoGen15Private::_preEnd__library_lights, &ColladaParserAutoGen15Private::_freeAttributes__library_lights);
functionMap[HASH_ELEMENT_LIBRARY_MATERIALS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_materials, &ColladaParserAutoGen15Private::_data__library_materials, &ColladaParserAutoGen15Private::_end__library_materials, &ColladaParserAutoGen15Private::_preBegin__library_materials, &ColladaParserAutoGen15Private::_preEnd__library_materials, &ColladaParserAutoGen15Private::_freeAttributes__library_materials);
functionMap[HASH_ELEMENT_LIBRARY_NODES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_nodes, &ColladaParserAutoGen15Private::_data__library_nodes, &ColladaParserAutoGen15Private::_end__library_nodes, &ColladaParserAutoGen15Private::_preBegin__library_nodes, &ColladaParserAutoGen15Private::_preEnd__library_nodes, &ColladaParserAutoGen15Private::_freeAttributes__library_nodes);
functionMap[HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_physics_materials, &ColladaParserAutoGen15Private::_data__library_physics_materials, &ColladaParserAutoGen15Private::_end__library_physics_materials, &ColladaParserAutoGen15Private::_preBegin__library_physics_materials, &ColladaParserAutoGen15Private::_preEnd__library_physics_materials, &ColladaParserAutoGen15Private::_freeAttributes__library_physics_materials);
functionMap[HASH_ELEMENT_LIBRARY_PHYSICS_MODELS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_physics_models, &ColladaParserAutoGen15Private::_data__library_physics_models, &ColladaParserAutoGen15Private::_end__library_physics_models, &ColladaParserAutoGen15Private::_preBegin__library_physics_models, &ColladaParserAutoGen15Private::_preEnd__library_physics_models, &ColladaParserAutoGen15Private::_freeAttributes__library_physics_models);
functionMap[HASH_ELEMENT_LIBRARY_PHYSICS_SCENES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_physics_scenes, &ColladaParserAutoGen15Private::_data__library_physics_scenes, &ColladaParserAutoGen15Private::_end__library_physics_scenes, &ColladaParserAutoGen15Private::_preBegin__library_physics_scenes, &ColladaParserAutoGen15Private::_preEnd__library_physics_scenes, &ColladaParserAutoGen15Private::_freeAttributes__library_physics_scenes);
functionMap[HASH_ELEMENT_LIBRARY_VISUAL_SCENES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_visual_scenes, &ColladaParserAutoGen15Private::_data__library_visual_scenes, &ColladaParserAutoGen15Private::_end__library_visual_scenes, &ColladaParserAutoGen15Private::_preBegin__library_visual_scenes, &ColladaParserAutoGen15Private::_preEnd__library_visual_scenes, &ColladaParserAutoGen15Private::_freeAttributes__library_visual_scenes);
functionMap[HASH_ELEMENT_SCENE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__scene, &ColladaParserAutoGen15Private::_data__scene, &ColladaParserAutoGen15Private::_end__scene, &ColladaParserAutoGen15Private::_preBegin__scene, &ColladaParserAutoGen15Private::_preEnd__scene, &ColladaParserAutoGen15Private::_freeAttributes__scene);
mapFilled = true;
}
return functionMap;
}

//---------------------------------------------------------------------
const ColladaParserAutoGen15Private::ElementFunctionMap& FunctionMapFactory::createFunctionMap__asset__allChildren() {
static ColladaParserAutoGen15Private::ElementFunctionMap functionMap;
static bool mapFilled = false;
if ( !mapFilled )
{
functionMap[HASH_ELEMENT_COLLADA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__COLLADA, &ColladaParserAutoGen15Private::_data__COLLADA, &ColladaParserAutoGen15Private::_end__COLLADA, &ColladaParserAutoGen15Private::_preBegin__COLLADA, &ColladaParserAutoGen15Private::_preEnd__COLLADA, &ColladaParserAutoGen15Private::_freeAttributes__COLLADA);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__ALPHA_FUNC] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__alpha_func, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__alpha_func, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__alpha_func, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__alpha_func, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__alpha_func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__alpha_func);
functionMap[HASH_ELEMENT_AMBIENT____FX_COMMON_COLOR_OR_TEXTURE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__ambient____fx_common_color_or_texture_type, &ColladaParserAutoGen15Private::_data__ambient____fx_common_color_or_texture_type, &ColladaParserAutoGen15Private::_end__ambient____fx_common_color_or_texture_type, &ColladaParserAutoGen15Private::_preBegin__ambient____fx_common_color_or_texture_type, &ColladaParserAutoGen15Private::_preEnd__ambient____fx_common_color_or_texture_type, &ColladaParserAutoGen15Private::_freeAttributes__ambient____fx_common_color_or_texture_type);
functionMap[HASH_ELEMENT_ANIMATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__animation, &ColladaParserAutoGen15Private::_data__animation, &ColladaParserAutoGen15Private::_end__animation, &ColladaParserAutoGen15Private::_preBegin__animation, &ColladaParserAutoGen15Private::_preEnd__animation, &ColladaParserAutoGen15Private::_freeAttributes__animation);
functionMap[HASH_ELEMENT_ANIMATION_CLIP] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__animation_clip, &ColladaParserAutoGen15Private::_data__animation_clip, &ColladaParserAutoGen15Private::_end__animation_clip, &ColladaParserAutoGen15Private::_preBegin__animation_clip, &ColladaParserAutoGen15Private::_preEnd__animation_clip, &ColladaParserAutoGen15Private::_freeAttributes__animation_clip);
functionMap[HASH_ELEMENT_ARRAY____GLSL_ARRAY_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__array____glsl_array_type, &ColladaParserAutoGen15Private::_data__array____glsl_array_type, &ColladaParserAutoGen15Private::_end__array____glsl_array_type, &ColladaParserAutoGen15Private::_preBegin__array____glsl_array_type, &ColladaParserAutoGen15Private::_preEnd__array____glsl_array_type, &ColladaParserAutoGen15Private::_freeAttributes__array____glsl_array_type);
functionMap[HASH_ELEMENT_ARTICULATED_SYSTEM] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__articulated_system, &ColladaParserAutoGen15Private::_data__articulated_system, &ColladaParserAutoGen15Private::_end__articulated_system, &ColladaParserAutoGen15Private::_preBegin__articulated_system, &ColladaParserAutoGen15Private::_preEnd__articulated_system, &ColladaParserAutoGen15Private::_freeAttributes__articulated_system);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_ATTACHMENT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__attachment, &ColladaParserAutoGen15Private::_data__attachment, &ColladaParserAutoGen15Private::_end__attachment, &ColladaParserAutoGen15Private::_preBegin__attachment, &ColladaParserAutoGen15Private::_preEnd__attachment, &ColladaParserAutoGen15Private::_freeAttributes__attachment);
functionMap[HASH_ELEMENT_AXIS_INFO____KINEMATICS_AXIS_INFO_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__axis_info____kinematics_axis_info_type, &ColladaParserAutoGen15Private::_data__axis_info____kinematics_axis_info_type, &ColladaParserAutoGen15Private::_end__axis_info____kinematics_axis_info_type, &ColladaParserAutoGen15Private::_preBegin__axis_info____kinematics_axis_info_type, &ColladaParserAutoGen15Private::_preEnd__axis_info____kinematics_axis_info_type, &ColladaParserAutoGen15Private::_freeAttributes__axis_info____kinematics_axis_info_type);
functionMap[HASH_ELEMENT_BIND_JOINT_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bind_joint_axis, &ColladaParserAutoGen15Private::_data__bind_joint_axis, &ColladaParserAutoGen15Private::_end__bind_joint_axis, &ColladaParserAutoGen15Private::_preBegin__bind_joint_axis, &ColladaParserAutoGen15Private::_preEnd__bind_joint_axis, &ColladaParserAutoGen15Private::_freeAttributes__bind_joint_axis);
functionMap[HASH_ELEMENT_BIND_MATERIAL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bind_material, &ColladaParserAutoGen15Private::_data__bind_material, &ColladaParserAutoGen15Private::_end__bind_material, &ColladaParserAutoGen15Private::_preBegin__bind_material, &ColladaParserAutoGen15Private::_preEnd__bind_material, &ColladaParserAutoGen15Private::_freeAttributes__bind_material);
functionMap[HASH_ELEMENT_GLES2_PROGRAM_TYPE____BIND_UNIFORM] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__gles2_program_type____bind_uniform, &ColladaParserAutoGen15Private::_data__gles2_program_type____bind_uniform, &ColladaParserAutoGen15Private::_end__gles2_program_type____bind_uniform, &ColladaParserAutoGen15Private::_preBegin__gles2_program_type____bind_uniform, &ColladaParserAutoGen15Private::_preEnd__gles2_program_type____bind_uniform, &ColladaParserAutoGen15Private::_freeAttributes__gles2_program_type____bind_uniform);
functionMap[HASH_ELEMENT_BLINN] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__blinn, &ColladaParserAutoGen15Private::_data__blinn, &ColladaParserAutoGen15Private::_end__blinn, &ColladaParserAutoGen15Private::_preBegin__blinn, &ColladaParserAutoGen15Private::_preEnd__blinn, &ColladaParserAutoGen15Private::_freeAttributes__blinn);
functionMap[HASH_ELEMENT_BOX] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__box, &ColladaParserAutoGen15Private::_data__box, &ColladaParserAutoGen15Private::_end__box, &ColladaParserAutoGen15Private::_preBegin__box, &ColladaParserAutoGen15Private::_preEnd__box, &ColladaParserAutoGen15Private::_freeAttributes__box);
functionMap[HASH_ELEMENT_BREP] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__brep, &ColladaParserAutoGen15Private::_data__brep, &ColladaParserAutoGen15Private::_end__brep, &ColladaParserAutoGen15Private::_preBegin__brep, &ColladaParserAutoGen15Private::_preEnd__brep, &ColladaParserAutoGen15Private::_freeAttributes__brep);
functionMap[HASH_ELEMENT_CAMERA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__camera, &ColladaParserAutoGen15Private::_data__camera, &ColladaParserAutoGen15Private::_end__camera, &ColladaParserAutoGen15Private::_preBegin__camera, &ColladaParserAutoGen15Private::_preEnd__camera, &ColladaParserAutoGen15Private::_freeAttributes__camera);
functionMap[HASH_ELEMENT_CAPSULE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__capsule, &ColladaParserAutoGen15Private::_data__capsule, &ColladaParserAutoGen15Private::_end__capsule, &ColladaParserAutoGen15Private::_preBegin__capsule, &ColladaParserAutoGen15Private::_preEnd__capsule, &ColladaParserAutoGen15Private::_freeAttributes__capsule);
functionMap[HASH_ELEMENT_CIRCLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__circle, &ColladaParserAutoGen15Private::_data__circle, &ColladaParserAutoGen15Private::_end__circle, &ColladaParserAutoGen15Private::_preBegin__circle, &ColladaParserAutoGen15Private::_preEnd__circle, &ColladaParserAutoGen15Private::_freeAttributes__circle);
functionMap[HASH_ELEMENT_COLOR_TARGET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__color_target, &ColladaParserAutoGen15Private::_data__color_target, &ColladaParserAutoGen15Private::_end__color_target, &ColladaParserAutoGen15Private::_preBegin__color_target, &ColladaParserAutoGen15Private::_preEnd__color_target, &ColladaParserAutoGen15Private::_freeAttributes__color_target);
functionMap[HASH_ELEMENT_CONE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__cone, &ColladaParserAutoGen15Private::_data__cone, &ColladaParserAutoGen15Private::_end__cone, &ColladaParserAutoGen15Private::_preBegin__cone, &ColladaParserAutoGen15Private::_preEnd__cone, &ColladaParserAutoGen15Private::_freeAttributes__cone);
functionMap[HASH_ELEMENT_PROFILE_COMMON__TECHNIQUE__CONSTANT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_COMMON__technique__constant, &ColladaParserAutoGen15Private::_data__profile_COMMON__technique__constant, &ColladaParserAutoGen15Private::_end__profile_COMMON__technique__constant, &ColladaParserAutoGen15Private::_preBegin__profile_COMMON__technique__constant, &ColladaParserAutoGen15Private::_preEnd__profile_COMMON__technique__constant, &ColladaParserAutoGen15Private::_freeAttributes__profile_COMMON__technique__constant);
functionMap[HASH_ELEMENT_SPLINE_TYPE____CONTROL_VERTICES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__spline_type____control_vertices, &ColladaParserAutoGen15Private::_data__spline_type____control_vertices, &ColladaParserAutoGen15Private::_end__spline_type____control_vertices, &ColladaParserAutoGen15Private::_preBegin__spline_type____control_vertices, &ColladaParserAutoGen15Private::_preEnd__spline_type____control_vertices, &ColladaParserAutoGen15Private::_freeAttributes__spline_type____control_vertices);
functionMap[HASH_ELEMENT_CONTROLLER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__controller, &ColladaParserAutoGen15Private::_data__controller, &ColladaParserAutoGen15Private::_end__controller, &ColladaParserAutoGen15Private::_preBegin__controller, &ColladaParserAutoGen15Private::_preEnd__controller, &ColladaParserAutoGen15Private::_freeAttributes__controller);
functionMap[HASH_ELEMENT_CONVEX_MESH] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__convex_mesh, &ColladaParserAutoGen15Private::_data__convex_mesh, &ColladaParserAutoGen15Private::_end__convex_mesh, &ColladaParserAutoGen15Private::_preBegin__convex_mesh, &ColladaParserAutoGen15Private::_preEnd__convex_mesh, &ColladaParserAutoGen15Private::_freeAttributes__convex_mesh);
functionMap[HASH_ELEMENT_CREATE_2D] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_2d, &ColladaParserAutoGen15Private::_data__create_2d, &ColladaParserAutoGen15Private::_end__create_2d, &ColladaParserAutoGen15Private::_preBegin__create_2d, &ColladaParserAutoGen15Private::_preEnd__create_2d, &ColladaParserAutoGen15Private::_freeAttributes__create_2d);
functionMap[HASH_ELEMENT_CREATE_3D] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_3d, &ColladaParserAutoGen15Private::_data__create_3d, &ColladaParserAutoGen15Private::_end__create_3d, &ColladaParserAutoGen15Private::_preBegin__create_3d, &ColladaParserAutoGen15Private::_preEnd__create_3d, &ColladaParserAutoGen15Private::_freeAttributes__create_3d);
functionMap[HASH_ELEMENT_CREATE_CUBE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_cube, &ColladaParserAutoGen15Private::_data__create_cube, &ColladaParserAutoGen15Private::_end__create_cube, &ColladaParserAutoGen15Private::_preBegin__create_cube, &ColladaParserAutoGen15Private::_preEnd__create_cube, &ColladaParserAutoGen15Private::_freeAttributes__create_cube);
functionMap[HASH_ELEMENT_CURVE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__curve, &ColladaParserAutoGen15Private::_data__curve, &ColladaParserAutoGen15Private::_end__curve, &ColladaParserAutoGen15Private::_preBegin__curve, &ColladaParserAutoGen15Private::_preEnd__curve, &ColladaParserAutoGen15Private::_freeAttributes__curve);
functionMap[HASH_ELEMENT_CURVES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__curves, &ColladaParserAutoGen15Private::_data__curves, &ColladaParserAutoGen15Private::_end__curves, &ColladaParserAutoGen15Private::_preBegin__curves, &ColladaParserAutoGen15Private::_preEnd__curves, &ColladaParserAutoGen15Private::_freeAttributes__curves);
functionMap[HASH_ELEMENT_SURFACE_TYPE____CYLINDER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__surface_type____cylinder, &ColladaParserAutoGen15Private::_data__surface_type____cylinder, &ColladaParserAutoGen15Private::_end__surface_type____cylinder, &ColladaParserAutoGen15Private::_preBegin__surface_type____cylinder, &ColladaParserAutoGen15Private::_preEnd__surface_type____cylinder, &ColladaParserAutoGen15Private::_freeAttributes__surface_type____cylinder);
functionMap[HASH_ELEMENT_DEPTH_TARGET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__depth_target, &ColladaParserAutoGen15Private::_data__depth_target, &ColladaParserAutoGen15Private::_end__depth_target, &ColladaParserAutoGen15Private::_preBegin__depth_target, &ColladaParserAutoGen15Private::_preEnd__depth_target, &ColladaParserAutoGen15Private::_freeAttributes__depth_target);
functionMap[HASH_ELEMENT_DIFFUSE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__diffuse, &ColladaParserAutoGen15Private::_data__diffuse, &ColladaParserAutoGen15Private::_end__diffuse, &ColladaParserAutoGen15Private::_preBegin__diffuse, &ColladaParserAutoGen15Private::_preEnd__diffuse, &ColladaParserAutoGen15Private::_freeAttributes__diffuse);
functionMap[HASH_ELEMENT_EDGES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__edges, &ColladaParserAutoGen15Private::_data__edges, &ColladaParserAutoGen15Private::_end__edges, &ColladaParserAutoGen15Private::_preBegin__edges, &ColladaParserAutoGen15Private::_preEnd__edges, &ColladaParserAutoGen15Private::_freeAttributes__edges);
functionMap[HASH_ELEMENT_EFFECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__effect, &ColladaParserAutoGen15Private::_data__effect, &ColladaParserAutoGen15Private::_end__effect, &ColladaParserAutoGen15Private::_preBegin__effect, &ColladaParserAutoGen15Private::_preEnd__effect, &ColladaParserAutoGen15Private::_freeAttributes__effect);
functionMap[HASH_ELEMENT_EFFECTOR_INFO] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__effector_info, &ColladaParserAutoGen15Private::_data__effector_info, &ColladaParserAutoGen15Private::_end__effector_info, &ColladaParserAutoGen15Private::_preBegin__effector_info, &ColladaParserAutoGen15Private::_preEnd__effector_info, &ColladaParserAutoGen15Private::_freeAttributes__effector_info);
functionMap[HASH_ELEMENT_ELLIPSE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__ellipse, &ColladaParserAutoGen15Private::_data__ellipse, &ColladaParserAutoGen15Private::_end__ellipse, &ColladaParserAutoGen15Private::_preBegin__ellipse, &ColladaParserAutoGen15Private::_preEnd__ellipse, &ColladaParserAutoGen15Private::_freeAttributes__ellipse);
functionMap[HASH_ELEMENT_EMISSION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__emission, &ColladaParserAutoGen15Private::_data__emission, &ColladaParserAutoGen15Private::_end__emission, &ColladaParserAutoGen15Private::_preBegin__emission, &ColladaParserAutoGen15Private::_preEnd__emission, &ColladaParserAutoGen15Private::_freeAttributes__emission);
functionMap[HASH_ELEMENT_GLES2_PASS_TYPE____EVALUATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__gles2_pass_type____evaluate, &ColladaParserAutoGen15Private::_data__gles2_pass_type____evaluate, &ColladaParserAutoGen15Private::_end__gles2_pass_type____evaluate, &ColladaParserAutoGen15Private::_preBegin__gles2_pass_type____evaluate, &ColladaParserAutoGen15Private::_preEnd__gles2_pass_type____evaluate, &ColladaParserAutoGen15Private::_freeAttributes__gles2_pass_type____evaluate);
functionMap[HASH_ELEMENT_EVALUATE_SCENE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__evaluate_scene, &ColladaParserAutoGen15Private::_data__evaluate_scene, &ColladaParserAutoGen15Private::_end__evaluate_scene, &ColladaParserAutoGen15Private::_preBegin__evaluate_scene, &ColladaParserAutoGen15Private::_preEnd__evaluate_scene, &ColladaParserAutoGen15Private::_freeAttributes__evaluate_scene);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_FACES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__faces, &ColladaParserAutoGen15Private::_data__faces, &ColladaParserAutoGen15Private::_end__faces, &ColladaParserAutoGen15Private::_preBegin__faces, &ColladaParserAutoGen15Private::_preEnd__faces, &ColladaParserAutoGen15Private::_freeAttributes__faces);
functionMap[HASH_ELEMENT_FORCE_FIELD] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__force_field, &ColladaParserAutoGen15Private::_data__force_field, &ColladaParserAutoGen15Private::_end__force_field, &ColladaParserAutoGen15Private::_preBegin__force_field, &ColladaParserAutoGen15Private::_preEnd__force_field, &ColladaParserAutoGen15Private::_freeAttributes__force_field);
functionMap[HASH_ELEMENT_FORMULA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__formula, &ColladaParserAutoGen15Private::_data__formula, &ColladaParserAutoGen15Private::_end__formula, &ColladaParserAutoGen15Private::_preBegin__formula, &ColladaParserAutoGen15Private::_preEnd__formula, &ColladaParserAutoGen15Private::_freeAttributes__formula);
functionMap[HASH_ELEMENT_GEOMETRY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__geometry, &ColladaParserAutoGen15Private::_data__geometry, &ColladaParserAutoGen15Private::_end__geometry, &ColladaParserAutoGen15Private::_preBegin__geometry, &ColladaParserAutoGen15Private::_preEnd__geometry, &ColladaParserAutoGen15Private::_freeAttributes__geometry);
functionMap[HASH_ELEMENT_HYPERBOLA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__hyperbola, &ColladaParserAutoGen15Private::_data__hyperbola, &ColladaParserAutoGen15Private::_end__hyperbola, &ColladaParserAutoGen15Private::_preBegin__hyperbola, &ColladaParserAutoGen15Private::_preEnd__hyperbola, &ColladaParserAutoGen15Private::_freeAttributes__hyperbola);
functionMap[HASH_ELEMENT_IMAGE____IMAGE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__image____image_type, &ColladaParserAutoGen15Private::_data__image____image_type, &ColladaParserAutoGen15Private::_end__image____image_type, &ColladaParserAutoGen15Private::_preBegin__image____image_type, &ColladaParserAutoGen15Private::_preEnd__image____image_type, &ColladaParserAutoGen15Private::_freeAttributes__image____image_type);
functionMap[HASH_ELEMENT_IMAGER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__imager, &ColladaParserAutoGen15Private::_data__imager, &ColladaParserAutoGen15Private::_end__imager, &ColladaParserAutoGen15Private::_preBegin__imager, &ColladaParserAutoGen15Private::_preEnd__imager, &ColladaParserAutoGen15Private::_freeAttributes__imager);
functionMap[HASH_ELEMENT_INSTANCE_ANIMATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_animation, &ColladaParserAutoGen15Private::_data__instance_animation, &ColladaParserAutoGen15Private::_end__instance_animation, &ColladaParserAutoGen15Private::_preBegin__instance_animation, &ColladaParserAutoGen15Private::_preEnd__instance_animation, &ColladaParserAutoGen15Private::_freeAttributes__instance_animation);
functionMap[HASH_ELEMENT_INSTANCE_ARTICULATED_SYSTEM] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_articulated_system, &ColladaParserAutoGen15Private::_data__instance_articulated_system, &ColladaParserAutoGen15Private::_end__instance_articulated_system, &ColladaParserAutoGen15Private::_preBegin__instance_articulated_system, &ColladaParserAutoGen15Private::_preEnd__instance_articulated_system, &ColladaParserAutoGen15Private::_freeAttributes__instance_articulated_system);
functionMap[HASH_ELEMENT_INSTANCE_CAMERA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_camera, &ColladaParserAutoGen15Private::_data__instance_camera, &ColladaParserAutoGen15Private::_end__instance_camera, &ColladaParserAutoGen15Private::_preBegin__instance_camera, &ColladaParserAutoGen15Private::_preEnd__instance_camera, &ColladaParserAutoGen15Private::_freeAttributes__instance_camera);
functionMap[HASH_ELEMENT_INSTANCE_CONTROLLER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_controller, &ColladaParserAutoGen15Private::_data__instance_controller, &ColladaParserAutoGen15Private::_end__instance_controller, &ColladaParserAutoGen15Private::_preBegin__instance_controller, &ColladaParserAutoGen15Private::_preEnd__instance_controller, &ColladaParserAutoGen15Private::_freeAttributes__instance_controller);
functionMap[HASH_ELEMENT_INSTANCE_EFFECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_effect, &ColladaParserAutoGen15Private::_data__instance_effect, &ColladaParserAutoGen15Private::_end__instance_effect, &ColladaParserAutoGen15Private::_preBegin__instance_effect, &ColladaParserAutoGen15Private::_preEnd__instance_effect, &ColladaParserAutoGen15Private::_freeAttributes__instance_effect);
functionMap[HASH_ELEMENT_INSTANCE_FORCE_FIELD] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_force_field, &ColladaParserAutoGen15Private::_data__instance_force_field, &ColladaParserAutoGen15Private::_end__instance_force_field, &ColladaParserAutoGen15Private::_preBegin__instance_force_field, &ColladaParserAutoGen15Private::_preEnd__instance_force_field, &ColladaParserAutoGen15Private::_freeAttributes__instance_force_field);
functionMap[HASH_ELEMENT_INSTANCE_FORMULA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_formula, &ColladaParserAutoGen15Private::_data__instance_formula, &ColladaParserAutoGen15Private::_end__instance_formula, &ColladaParserAutoGen15Private::_preBegin__instance_formula, &ColladaParserAutoGen15Private::_preEnd__instance_formula, &ColladaParserAutoGen15Private::_freeAttributes__instance_formula);
functionMap[HASH_ELEMENT_INSTANCE_GEOMETRY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_geometry, &ColladaParserAutoGen15Private::_data__instance_geometry, &ColladaParserAutoGen15Private::_end__instance_geometry, &ColladaParserAutoGen15Private::_preBegin__instance_geometry, &ColladaParserAutoGen15Private::_preEnd__instance_geometry, &ColladaParserAutoGen15Private::_freeAttributes__instance_geometry);
functionMap[HASH_ELEMENT_INSTANCE_IMAGE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_image, &ColladaParserAutoGen15Private::_data__instance_image, &ColladaParserAutoGen15Private::_end__instance_image, &ColladaParserAutoGen15Private::_preBegin__instance_image, &ColladaParserAutoGen15Private::_preEnd__instance_image, &ColladaParserAutoGen15Private::_freeAttributes__instance_image);
functionMap[HASH_ELEMENT_INSTANCE_JOINT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_joint, &ColladaParserAutoGen15Private::_data__instance_joint, &ColladaParserAutoGen15Private::_end__instance_joint, &ColladaParserAutoGen15Private::_preBegin__instance_joint, &ColladaParserAutoGen15Private::_preEnd__instance_joint, &ColladaParserAutoGen15Private::_freeAttributes__instance_joint);
functionMap[HASH_ELEMENT_INSTANCE_KINEMATICS_MODEL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_kinematics_model, &ColladaParserAutoGen15Private::_data__instance_kinematics_model, &ColladaParserAutoGen15Private::_end__instance_kinematics_model, &ColladaParserAutoGen15Private::_preBegin__instance_kinematics_model, &ColladaParserAutoGen15Private::_preEnd__instance_kinematics_model, &ColladaParserAutoGen15Private::_freeAttributes__instance_kinematics_model);
functionMap[HASH_ELEMENT_INSTANCE_KINEMATICS_SCENE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_kinematics_scene, &ColladaParserAutoGen15Private::_data__instance_kinematics_scene, &ColladaParserAutoGen15Private::_end__instance_kinematics_scene, &ColladaParserAutoGen15Private::_preBegin__instance_kinematics_scene, &ColladaParserAutoGen15Private::_preEnd__instance_kinematics_scene, &ColladaParserAutoGen15Private::_freeAttributes__instance_kinematics_scene);
functionMap[HASH_ELEMENT_INSTANCE_LIGHT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_light, &ColladaParserAutoGen15Private::_data__instance_light, &ColladaParserAutoGen15Private::_end__instance_light, &ColladaParserAutoGen15Private::_preBegin__instance_light, &ColladaParserAutoGen15Private::_preEnd__instance_light, &ColladaParserAutoGen15Private::_freeAttributes__instance_light);
functionMap[HASH_ELEMENT_INSTANCE_MATERIAL____INSTANCE_MATERIAL_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_material____instance_material_type, &ColladaParserAutoGen15Private::_data__instance_material____instance_material_type, &ColladaParserAutoGen15Private::_end__instance_material____instance_material_type, &ColladaParserAutoGen15Private::_preBegin__instance_material____instance_material_type, &ColladaParserAutoGen15Private::_preEnd__instance_material____instance_material_type, &ColladaParserAutoGen15Private::_freeAttributes__instance_material____instance_material_type);
functionMap[HASH_ELEMENT_INSTANCE_NODE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_node, &ColladaParserAutoGen15Private::_data__instance_node, &ColladaParserAutoGen15Private::_end__instance_node, &ColladaParserAutoGen15Private::_preBegin__instance_node, &ColladaParserAutoGen15Private::_preEnd__instance_node, &ColladaParserAutoGen15Private::_freeAttributes__instance_node);
functionMap[HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_physics_material, &ColladaParserAutoGen15Private::_data__instance_physics_material, &ColladaParserAutoGen15Private::_end__instance_physics_material, &ColladaParserAutoGen15Private::_preBegin__instance_physics_material, &ColladaParserAutoGen15Private::_preEnd__instance_physics_material, &ColladaParserAutoGen15Private::_freeAttributes__instance_physics_material);
functionMap[HASH_ELEMENT_INSTANCE_PHYSICS_MODEL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_physics_model, &ColladaParserAutoGen15Private::_data__instance_physics_model, &ColladaParserAutoGen15Private::_end__instance_physics_model, &ColladaParserAutoGen15Private::_preBegin__instance_physics_model, &ColladaParserAutoGen15Private::_preEnd__instance_physics_model, &ColladaParserAutoGen15Private::_freeAttributes__instance_physics_model);
functionMap[HASH_ELEMENT_INSTANCE_PHYSICS_SCENE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_physics_scene, &ColladaParserAutoGen15Private::_data__instance_physics_scene, &ColladaParserAutoGen15Private::_end__instance_physics_scene, &ColladaParserAutoGen15Private::_preBegin__instance_physics_scene, &ColladaParserAutoGen15Private::_preEnd__instance_physics_scene, &ColladaParserAutoGen15Private::_freeAttributes__instance_physics_scene);
functionMap[HASH_ELEMENT_INSTANCE_RIGID_BODY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_rigid_body, &ColladaParserAutoGen15Private::_data__instance_rigid_body, &ColladaParserAutoGen15Private::_end__instance_rigid_body, &ColladaParserAutoGen15Private::_preBegin__instance_rigid_body, &ColladaParserAutoGen15Private::_preEnd__instance_rigid_body, &ColladaParserAutoGen15Private::_freeAttributes__instance_rigid_body);
functionMap[HASH_ELEMENT_INSTANCE_RIGID_CONSTRAINT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_rigid_constraint, &ColladaParserAutoGen15Private::_data__instance_rigid_constraint, &ColladaParserAutoGen15Private::_end__instance_rigid_constraint, &ColladaParserAutoGen15Private::_preBegin__instance_rigid_constraint, &ColladaParserAutoGen15Private::_preEnd__instance_rigid_constraint, &ColladaParserAutoGen15Private::_freeAttributes__instance_rigid_constraint);
functionMap[HASH_ELEMENT_INSTANCE_VISUAL_SCENE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_visual_scene, &ColladaParserAutoGen15Private::_data__instance_visual_scene, &ColladaParserAutoGen15Private::_end__instance_visual_scene, &ColladaParserAutoGen15Private::_preBegin__instance_visual_scene, &ColladaParserAutoGen15Private::_preEnd__instance_visual_scene, &ColladaParserAutoGen15Private::_freeAttributes__instance_visual_scene);
functionMap[HASH_ELEMENT_JOINT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__joint, &ColladaParserAutoGen15Private::_data__joint, &ColladaParserAutoGen15Private::_end__joint, &ColladaParserAutoGen15Private::_preBegin__joint, &ColladaParserAutoGen15Private::_preEnd__joint, &ColladaParserAutoGen15Private::_freeAttributes__joint);
functionMap[HASH_ELEMENT_JOINTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__joints, &ColladaParserAutoGen15Private::_data__joints, &ColladaParserAutoGen15Private::_end__joints, &ColladaParserAutoGen15Private::_preBegin__joints, &ColladaParserAutoGen15Private::_preEnd__joints, &ColladaParserAutoGen15Private::_freeAttributes__joints);
functionMap[HASH_ELEMENT_KINEMATICS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__kinematics, &ColladaParserAutoGen15Private::_data__kinematics, &ColladaParserAutoGen15Private::_end__kinematics, &ColladaParserAutoGen15Private::_preBegin__kinematics, &ColladaParserAutoGen15Private::_preEnd__kinematics, &ColladaParserAutoGen15Private::_freeAttributes__kinematics);
functionMap[HASH_ELEMENT_KINEMATICS_MODEL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__kinematics_model, &ColladaParserAutoGen15Private::_data__kinematics_model, &ColladaParserAutoGen15Private::_end__kinematics_model, &ColladaParserAutoGen15Private::_preBegin__kinematics_model, &ColladaParserAutoGen15Private::_preEnd__kinematics_model, &ColladaParserAutoGen15Private::_freeAttributes__kinematics_model);
functionMap[HASH_ELEMENT_KINEMATICS_SCENE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__kinematics_scene, &ColladaParserAutoGen15Private::_data__kinematics_scene, &ColladaParserAutoGen15Private::_end__kinematics_scene, &ColladaParserAutoGen15Private::_preBegin__kinematics_scene, &ColladaParserAutoGen15Private::_preEnd__kinematics_scene, &ColladaParserAutoGen15Private::_freeAttributes__kinematics_scene);
functionMap[HASH_ELEMENT_LAMBERT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__lambert, &ColladaParserAutoGen15Private::_data__lambert, &ColladaParserAutoGen15Private::_end__lambert, &ColladaParserAutoGen15Private::_preBegin__lambert, &ColladaParserAutoGen15Private::_preEnd__lambert, &ColladaParserAutoGen15Private::_freeAttributes__lambert);
functionMap[HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_animation_clips, &ColladaParserAutoGen15Private::_data__library_animation_clips, &ColladaParserAutoGen15Private::_end__library_animation_clips, &ColladaParserAutoGen15Private::_preBegin__library_animation_clips, &ColladaParserAutoGen15Private::_preEnd__library_animation_clips, &ColladaParserAutoGen15Private::_freeAttributes__library_animation_clips);
functionMap[HASH_ELEMENT_LIBRARY_ANIMATIONS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_animations, &ColladaParserAutoGen15Private::_data__library_animations, &ColladaParserAutoGen15Private::_end__library_animations, &ColladaParserAutoGen15Private::_preBegin__library_animations, &ColladaParserAutoGen15Private::_preEnd__library_animations, &ColladaParserAutoGen15Private::_freeAttributes__library_animations);
functionMap[HASH_ELEMENT_LIBRARY_ARTICULATED_SYSTEMS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_articulated_systems, &ColladaParserAutoGen15Private::_data__library_articulated_systems, &ColladaParserAutoGen15Private::_end__library_articulated_systems, &ColladaParserAutoGen15Private::_preBegin__library_articulated_systems, &ColladaParserAutoGen15Private::_preEnd__library_articulated_systems, &ColladaParserAutoGen15Private::_freeAttributes__library_articulated_systems);
functionMap[HASH_ELEMENT_LIBRARY_CAMERAS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_cameras, &ColladaParserAutoGen15Private::_data__library_cameras, &ColladaParserAutoGen15Private::_end__library_cameras, &ColladaParserAutoGen15Private::_preBegin__library_cameras, &ColladaParserAutoGen15Private::_preEnd__library_cameras, &ColladaParserAutoGen15Private::_freeAttributes__library_cameras);
functionMap[HASH_ELEMENT_LIBRARY_CONTROLLERS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_controllers, &ColladaParserAutoGen15Private::_data__library_controllers, &ColladaParserAutoGen15Private::_end__library_controllers, &ColladaParserAutoGen15Private::_preBegin__library_controllers, &ColladaParserAutoGen15Private::_preEnd__library_controllers, &ColladaParserAutoGen15Private::_freeAttributes__library_controllers);
functionMap[HASH_ELEMENT_LIBRARY_EFFECTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_effects, &ColladaParserAutoGen15Private::_data__library_effects, &ColladaParserAutoGen15Private::_end__library_effects, &ColladaParserAutoGen15Private::_preBegin__library_effects, &ColladaParserAutoGen15Private::_preEnd__library_effects, &ColladaParserAutoGen15Private::_freeAttributes__library_effects);
functionMap[HASH_ELEMENT_LIBRARY_FORCE_FIELDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_force_fields, &ColladaParserAutoGen15Private::_data__library_force_fields, &ColladaParserAutoGen15Private::_end__library_force_fields, &ColladaParserAutoGen15Private::_preBegin__library_force_fields, &ColladaParserAutoGen15Private::_preEnd__library_force_fields, &ColladaParserAutoGen15Private::_freeAttributes__library_force_fields);
functionMap[HASH_ELEMENT_LIBRARY_FORMULAS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_formulas, &ColladaParserAutoGen15Private::_data__library_formulas, &ColladaParserAutoGen15Private::_end__library_formulas, &ColladaParserAutoGen15Private::_preBegin__library_formulas, &ColladaParserAutoGen15Private::_preEnd__library_formulas, &ColladaParserAutoGen15Private::_freeAttributes__library_formulas);
functionMap[HASH_ELEMENT_LIBRARY_GEOMETRIES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_geometries, &ColladaParserAutoGen15Private::_data__library_geometries, &ColladaParserAutoGen15Private::_end__library_geometries, &ColladaParserAutoGen15Private::_preBegin__library_geometries, &ColladaParserAutoGen15Private::_preEnd__library_geometries, &ColladaParserAutoGen15Private::_freeAttributes__library_geometries);
functionMap[HASH_ELEMENT_LIBRARY_IMAGES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_images, &ColladaParserAutoGen15Private::_data__library_images, &ColladaParserAutoGen15Private::_end__library_images, &ColladaParserAutoGen15Private::_preBegin__library_images, &ColladaParserAutoGen15Private::_preEnd__library_images, &ColladaParserAutoGen15Private::_freeAttributes__library_images);
functionMap[HASH_ELEMENT_LIBRARY_JOINTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_joints, &ColladaParserAutoGen15Private::_data__library_joints, &ColladaParserAutoGen15Private::_end__library_joints, &ColladaParserAutoGen15Private::_preBegin__library_joints, &ColladaParserAutoGen15Private::_preEnd__library_joints, &ColladaParserAutoGen15Private::_freeAttributes__library_joints);
functionMap[HASH_ELEMENT_LIBRARY_KINEMATICS_MODELS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_kinematics_models, &ColladaParserAutoGen15Private::_data__library_kinematics_models, &ColladaParserAutoGen15Private::_end__library_kinematics_models, &ColladaParserAutoGen15Private::_preBegin__library_kinematics_models, &ColladaParserAutoGen15Private::_preEnd__library_kinematics_models, &ColladaParserAutoGen15Private::_freeAttributes__library_kinematics_models);
functionMap[HASH_ELEMENT_LIBRARY_KINEMATICS_SCENES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_kinematics_scenes, &ColladaParserAutoGen15Private::_data__library_kinematics_scenes, &ColladaParserAutoGen15Private::_end__library_kinematics_scenes, &ColladaParserAutoGen15Private::_preBegin__library_kinematics_scenes, &ColladaParserAutoGen15Private::_preEnd__library_kinematics_scenes, &ColladaParserAutoGen15Private::_freeAttributes__library_kinematics_scenes);
functionMap[HASH_ELEMENT_LIBRARY_LIGHTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_lights, &ColladaParserAutoGen15Private::_data__library_lights, &ColladaParserAutoGen15Private::_end__library_lights, &ColladaParserAutoGen15Private::_preBegin__library_lights, &ColladaParserAutoGen15Private::_preEnd__library_lights, &ColladaParserAutoGen15Private::_freeAttributes__library_lights);
functionMap[HASH_ELEMENT_LIBRARY_MATERIALS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_materials, &ColladaParserAutoGen15Private::_data__library_materials, &ColladaParserAutoGen15Private::_end__library_materials, &ColladaParserAutoGen15Private::_preBegin__library_materials, &ColladaParserAutoGen15Private::_preEnd__library_materials, &ColladaParserAutoGen15Private::_freeAttributes__library_materials);
functionMap[HASH_ELEMENT_LIBRARY_NODES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_nodes, &ColladaParserAutoGen15Private::_data__library_nodes, &ColladaParserAutoGen15Private::_end__library_nodes, &ColladaParserAutoGen15Private::_preBegin__library_nodes, &ColladaParserAutoGen15Private::_preEnd__library_nodes, &ColladaParserAutoGen15Private::_freeAttributes__library_nodes);
functionMap[HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_physics_materials, &ColladaParserAutoGen15Private::_data__library_physics_materials, &ColladaParserAutoGen15Private::_end__library_physics_materials, &ColladaParserAutoGen15Private::_preBegin__library_physics_materials, &ColladaParserAutoGen15Private::_preEnd__library_physics_materials, &ColladaParserAutoGen15Private::_freeAttributes__library_physics_materials);
functionMap[HASH_ELEMENT_LIBRARY_PHYSICS_MODELS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_physics_models, &ColladaParserAutoGen15Private::_data__library_physics_models, &ColladaParserAutoGen15Private::_end__library_physics_models, &ColladaParserAutoGen15Private::_preBegin__library_physics_models, &ColladaParserAutoGen15Private::_preEnd__library_physics_models, &ColladaParserAutoGen15Private::_freeAttributes__library_physics_models);
functionMap[HASH_ELEMENT_LIBRARY_PHYSICS_SCENES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_physics_scenes, &ColladaParserAutoGen15Private::_data__library_physics_scenes, &ColladaParserAutoGen15Private::_end__library_physics_scenes, &ColladaParserAutoGen15Private::_preBegin__library_physics_scenes, &ColladaParserAutoGen15Private::_preEnd__library_physics_scenes, &ColladaParserAutoGen15Private::_freeAttributes__library_physics_scenes);
functionMap[HASH_ELEMENT_LIBRARY_VISUAL_SCENES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_visual_scenes, &ColladaParserAutoGen15Private::_data__library_visual_scenes, &ColladaParserAutoGen15Private::_end__library_visual_scenes, &ColladaParserAutoGen15Private::_preBegin__library_visual_scenes, &ColladaParserAutoGen15Private::_preEnd__library_visual_scenes, &ColladaParserAutoGen15Private::_freeAttributes__library_visual_scenes);
functionMap[HASH_ELEMENT_LIGHT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__light, &ColladaParserAutoGen15Private::_data__light, &ColladaParserAutoGen15Private::_end__light, &ColladaParserAutoGen15Private::_preBegin__light, &ColladaParserAutoGen15Private::_preEnd__light, &ColladaParserAutoGen15Private::_freeAttributes__light);
functionMap[HASH_ELEMENT_LINE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__line, &ColladaParserAutoGen15Private::_data__line, &ColladaParserAutoGen15Private::_end__line, &ColladaParserAutoGen15Private::_preBegin__line, &ColladaParserAutoGen15Private::_preEnd__line, &ColladaParserAutoGen15Private::_freeAttributes__line);
functionMap[HASH_ELEMENT_LINES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__lines, &ColladaParserAutoGen15Private::_data__lines, &ColladaParserAutoGen15Private::_end__lines, &ColladaParserAutoGen15Private::_preBegin__lines, &ColladaParserAutoGen15Private::_preEnd__lines, &ColladaParserAutoGen15Private::_freeAttributes__lines);
functionMap[HASH_ELEMENT_LINESTRIPS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__linestrips, &ColladaParserAutoGen15Private::_data__linestrips, &ColladaParserAutoGen15Private::_end__linestrips, &ColladaParserAutoGen15Private::_preBegin__linestrips, &ColladaParserAutoGen15Private::_preEnd__linestrips, &ColladaParserAutoGen15Private::_freeAttributes__linestrips);
functionMap[HASH_ELEMENT_MATERIAL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__material, &ColladaParserAutoGen15Private::_data__material, &ColladaParserAutoGen15Private::_end__material, &ColladaParserAutoGen15Private::_preBegin__material, &ColladaParserAutoGen15Private::_preEnd__material, &ColladaParserAutoGen15Private::_freeAttributes__material);
functionMap[HASH_ELEMENT_MESH] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__mesh, &ColladaParserAutoGen15Private::_data__mesh, &ColladaParserAutoGen15Private::_end__mesh, &ColladaParserAutoGen15Private::_preBegin__mesh, &ColladaParserAutoGen15Private::_preEnd__mesh, &ColladaParserAutoGen15Private::_freeAttributes__mesh);
functionMap[HASH_ELEMENT_MORPH] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__morph, &ColladaParserAutoGen15Private::_data__morph, &ColladaParserAutoGen15Private::_end__morph, &ColladaParserAutoGen15Private::_preBegin__morph, &ColladaParserAutoGen15Private::_preEnd__morph, &ColladaParserAutoGen15Private::_freeAttributes__morph);
functionMap[HASH_ELEMENT_MOTION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__motion, &ColladaParserAutoGen15Private::_data__motion, &ColladaParserAutoGen15Private::_end__motion, &ColladaParserAutoGen15Private::_preBegin__motion, &ColladaParserAutoGen15Private::_preEnd__motion, &ColladaParserAutoGen15Private::_freeAttributes__motion);
functionMap[HASH_ELEMENT_NEWPARAM____FX_NEWPARAM_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__newparam____fx_newparam_type, &ColladaParserAutoGen15Private::_data__newparam____fx_newparam_type, &ColladaParserAutoGen15Private::_end__newparam____fx_newparam_type, &ColladaParserAutoGen15Private::_preBegin__newparam____fx_newparam_type, &ColladaParserAutoGen15Private::_preEnd__newparam____fx_newparam_type, &ColladaParserAutoGen15Private::_freeAttributes__newparam____fx_newparam_type);
functionMap[HASH_ELEMENT_NODE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__node, &ColladaParserAutoGen15Private::_data__node, &ColladaParserAutoGen15Private::_end__node, &ColladaParserAutoGen15Private::_preBegin__node, &ColladaParserAutoGen15Private::_preEnd__node, &ColladaParserAutoGen15Private::_freeAttributes__node);
functionMap[HASH_ELEMENT_NURBS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__nurbs, &ColladaParserAutoGen15Private::_data__nurbs, &ColladaParserAutoGen15Private::_end__nurbs, &ColladaParserAutoGen15Private::_preBegin__nurbs, &ColladaParserAutoGen15Private::_preEnd__nurbs, &ColladaParserAutoGen15Private::_freeAttributes__nurbs);
functionMap[HASH_ELEMENT_NURBS_SURFACE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__nurbs_surface, &ColladaParserAutoGen15Private::_data__nurbs_surface, &ColladaParserAutoGen15Private::_end__nurbs_surface, &ColladaParserAutoGen15Private::_preBegin__nurbs_surface, &ColladaParserAutoGen15Private::_preEnd__nurbs_surface, &ColladaParserAutoGen15Private::_freeAttributes__nurbs_surface);
functionMap[HASH_ELEMENT_OPTICS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__optics, &ColladaParserAutoGen15Private::_data__optics, &ColladaParserAutoGen15Private::_end__optics, &ColladaParserAutoGen15Private::_preBegin__optics, &ColladaParserAutoGen15Private::_preEnd__optics, &ColladaParserAutoGen15Private::_freeAttributes__optics);
functionMap[HASH_ELEMENT_PARABOLA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__parabola, &ColladaParserAutoGen15Private::_data__parabola, &ColladaParserAutoGen15Private::_end__parabola, &ColladaParserAutoGen15Private::_preBegin__parabola, &ColladaParserAutoGen15Private::_preEnd__parabola, &ColladaParserAutoGen15Private::_freeAttributes__parabola);
functionMap[HASH_ELEMENT_PASS____GLES2_PASS_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__pass____gles2_pass_type, &ColladaParserAutoGen15Private::_data__pass____gles2_pass_type, &ColladaParserAutoGen15Private::_end__pass____gles2_pass_type, &ColladaParserAutoGen15Private::_preBegin__pass____gles2_pass_type, &ColladaParserAutoGen15Private::_preEnd__pass____gles2_pass_type, &ColladaParserAutoGen15Private::_freeAttributes__pass____gles2_pass_type);
functionMap[HASH_ELEMENT_PCURVES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__pcurves, &ColladaParserAutoGen15Private::_data__pcurves, &ColladaParserAutoGen15Private::_end__pcurves, &ColladaParserAutoGen15Private::_preBegin__pcurves, &ColladaParserAutoGen15Private::_preEnd__pcurves, &ColladaParserAutoGen15Private::_freeAttributes__pcurves);
functionMap[HASH_ELEMENT_PHONG] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__phong, &ColladaParserAutoGen15Private::_data__phong, &ColladaParserAutoGen15Private::_end__phong, &ColladaParserAutoGen15Private::_preBegin__phong, &ColladaParserAutoGen15Private::_preEnd__phong, &ColladaParserAutoGen15Private::_freeAttributes__phong);
functionMap[HASH_ELEMENT_PHYSICS_MATERIAL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__physics_material, &ColladaParserAutoGen15Private::_data__physics_material, &ColladaParserAutoGen15Private::_end__physics_material, &ColladaParserAutoGen15Private::_preBegin__physics_material, &ColladaParserAutoGen15Private::_preEnd__physics_material, &ColladaParserAutoGen15Private::_freeAttributes__physics_material);
functionMap[HASH_ELEMENT_PHYSICS_MODEL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__physics_model, &ColladaParserAutoGen15Private::_data__physics_model, &ColladaParserAutoGen15Private::_end__physics_model, &ColladaParserAutoGen15Private::_preBegin__physics_model, &ColladaParserAutoGen15Private::_preEnd__physics_model, &ColladaParserAutoGen15Private::_freeAttributes__physics_model);
functionMap[HASH_ELEMENT_PHYSICS_SCENE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__physics_scene, &ColladaParserAutoGen15Private::_data__physics_scene, &ColladaParserAutoGen15Private::_end__physics_scene, &ColladaParserAutoGen15Private::_preBegin__physics_scene, &ColladaParserAutoGen15Private::_preEnd__physics_scene, &ColladaParserAutoGen15Private::_freeAttributes__physics_scene);
functionMap[HASH_ELEMENT_PLANE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__plane, &ColladaParserAutoGen15Private::_data__plane, &ColladaParserAutoGen15Private::_end__plane, &ColladaParserAutoGen15Private::_preBegin__plane, &ColladaParserAutoGen15Private::_preEnd__plane, &ColladaParserAutoGen15Private::_freeAttributes__plane);
functionMap[HASH_ELEMENT_POLYGONS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__polygons, &ColladaParserAutoGen15Private::_data__polygons, &ColladaParserAutoGen15Private::_end__polygons, &ColladaParserAutoGen15Private::_preBegin__polygons, &ColladaParserAutoGen15Private::_preEnd__polygons, &ColladaParserAutoGen15Private::_freeAttributes__polygons);
functionMap[HASH_ELEMENT_POLYLIST] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__polylist, &ColladaParserAutoGen15Private::_data__polylist, &ColladaParserAutoGen15Private::_end__polylist, &ColladaParserAutoGen15Private::_preBegin__polylist, &ColladaParserAutoGen15Private::_preEnd__polylist, &ColladaParserAutoGen15Private::_freeAttributes__polylist);
functionMap[HASH_ELEMENT_PROFILE_BRIDGE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_BRIDGE, &ColladaParserAutoGen15Private::_data__profile_BRIDGE, &ColladaParserAutoGen15Private::_end__profile_BRIDGE, &ColladaParserAutoGen15Private::_preBegin__profile_BRIDGE, &ColladaParserAutoGen15Private::_preEnd__profile_BRIDGE, &ColladaParserAutoGen15Private::_freeAttributes__profile_BRIDGE);
functionMap[HASH_ELEMENT_PROFILE_CG] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_CG, &ColladaParserAutoGen15Private::_data__profile_CG, &ColladaParserAutoGen15Private::_end__profile_CG, &ColladaParserAutoGen15Private::_preBegin__profile_CG, &ColladaParserAutoGen15Private::_preEnd__profile_CG, &ColladaParserAutoGen15Private::_freeAttributes__profile_CG);
functionMap[HASH_ELEMENT_PROFILE_COMMON] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_COMMON, &ColladaParserAutoGen15Private::_data__profile_COMMON, &ColladaParserAutoGen15Private::_end__profile_COMMON, &ColladaParserAutoGen15Private::_preBegin__profile_COMMON, &ColladaParserAutoGen15Private::_preEnd__profile_COMMON, &ColladaParserAutoGen15Private::_freeAttributes__profile_COMMON);
functionMap[HASH_ELEMENT_PROFILE_GLES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES, &ColladaParserAutoGen15Private::_data__profile_GLES, &ColladaParserAutoGen15Private::_end__profile_GLES, &ColladaParserAutoGen15Private::_preBegin__profile_GLES, &ColladaParserAutoGen15Private::_preEnd__profile_GLES, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES);
functionMap[HASH_ELEMENT_PROFILE_GLES2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2, &ColladaParserAutoGen15Private::_data__profile_GLES2, &ColladaParserAutoGen15Private::_end__profile_GLES2, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2);
functionMap[HASH_ELEMENT_PROFILE_GLSL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL, &ColladaParserAutoGen15Private::_data__profile_GLSL, &ColladaParserAutoGen15Private::_end__profile_GLSL, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL);
functionMap[HASH_ELEMENT_PROGRAM____GLES2_PROGRAM_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__program____gles2_program_type, &ColladaParserAutoGen15Private::_data__program____gles2_program_type, &ColladaParserAutoGen15Private::_end__program____gles2_program_type, &ColladaParserAutoGen15Private::_preBegin__program____gles2_program_type, &ColladaParserAutoGen15Private::_preEnd__program____gles2_program_type, &ColladaParserAutoGen15Private::_freeAttributes__program____gles2_program_type);
functionMap[HASH_ELEMENT_REF_ATTACHMENT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__ref_attachment, &ColladaParserAutoGen15Private::_data__ref_attachment, &ColladaParserAutoGen15Private::_end__ref_attachment, &ColladaParserAutoGen15Private::_preBegin__ref_attachment, &ColladaParserAutoGen15Private::_preEnd__ref_attachment, &ColladaParserAutoGen15Private::_freeAttributes__ref_attachment);
functionMap[HASH_ELEMENT_REFLECTIVE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__reflective, &ColladaParserAutoGen15Private::_data__reflective, &ColladaParserAutoGen15Private::_end__reflective, &ColladaParserAutoGen15Private::_preBegin__reflective, &ColladaParserAutoGen15Private::_preEnd__reflective, &ColladaParserAutoGen15Private::_freeAttributes__reflective);
functionMap[HASH_ELEMENT_RENDER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__render, &ColladaParserAutoGen15Private::_data__render, &ColladaParserAutoGen15Private::_end__render, &ColladaParserAutoGen15Private::_preBegin__render, &ColladaParserAutoGen15Private::_preEnd__render, &ColladaParserAutoGen15Private::_freeAttributes__render);
functionMap[HASH_ELEMENT_RIGID_BODY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__rigid_body, &ColladaParserAutoGen15Private::_data__rigid_body, &ColladaParserAutoGen15Private::_end__rigid_body, &ColladaParserAutoGen15Private::_preBegin__rigid_body, &ColladaParserAutoGen15Private::_preEnd__rigid_body, &ColladaParserAutoGen15Private::_freeAttributes__rigid_body);
functionMap[HASH_ELEMENT_RIGID_CONSTRAINT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__rigid_constraint, &ColladaParserAutoGen15Private::_data__rigid_constraint, &ColladaParserAutoGen15Private::_end__rigid_constraint, &ColladaParserAutoGen15Private::_preBegin__rigid_constraint, &ColladaParserAutoGen15Private::_preEnd__rigid_constraint, &ColladaParserAutoGen15Private::_freeAttributes__rigid_constraint);
functionMap[HASH_ELEMENT_SAMPLE_COVERAGE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sample_coverage, &ColladaParserAutoGen15Private::_data__sample_coverage, &ColladaParserAutoGen15Private::_end__sample_coverage, &ColladaParserAutoGen15Private::_preBegin__sample_coverage, &ColladaParserAutoGen15Private::_preEnd__sample_coverage, &ColladaParserAutoGen15Private::_freeAttributes__sample_coverage);
functionMap[HASH_ELEMENT_SAMPLER1D] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler1D, &ColladaParserAutoGen15Private::_data__sampler1D, &ColladaParserAutoGen15Private::_end__sampler1D, &ColladaParserAutoGen15Private::_preBegin__sampler1D, &ColladaParserAutoGen15Private::_preEnd__sampler1D, &ColladaParserAutoGen15Private::_freeAttributes__sampler1D);
functionMap[HASH_ELEMENT_SAMPLER2D____FX_SAMPLER2D_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_data__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_end__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_preBegin__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_preEnd__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_freeAttributes__sampler2D____fx_sampler2D_type);
functionMap[HASH_ELEMENT_SAMPLER3D] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler3D, &ColladaParserAutoGen15Private::_data__sampler3D, &ColladaParserAutoGen15Private::_end__sampler3D, &ColladaParserAutoGen15Private::_preBegin__sampler3D, &ColladaParserAutoGen15Private::_preEnd__sampler3D, &ColladaParserAutoGen15Private::_freeAttributes__sampler3D);
functionMap[HASH_ELEMENT_SAMPLERCUBE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__samplerCUBE, &ColladaParserAutoGen15Private::_data__samplerCUBE, &ColladaParserAutoGen15Private::_end__samplerCUBE, &ColladaParserAutoGen15Private::_preBegin__samplerCUBE, &ColladaParserAutoGen15Private::_preEnd__samplerCUBE, &ColladaParserAutoGen15Private::_freeAttributes__samplerCUBE);
functionMap[HASH_ELEMENT_SAMPLERDEPTH] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__samplerDEPTH, &ColladaParserAutoGen15Private::_data__samplerDEPTH, &ColladaParserAutoGen15Private::_end__samplerDEPTH, &ColladaParserAutoGen15Private::_preBegin__samplerDEPTH, &ColladaParserAutoGen15Private::_preEnd__samplerDEPTH, &ColladaParserAutoGen15Private::_freeAttributes__samplerDEPTH);
functionMap[HASH_ELEMENT_SAMPLERRECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__samplerRECT, &ColladaParserAutoGen15Private::_data__samplerRECT, &ColladaParserAutoGen15Private::_end__samplerRECT, &ColladaParserAutoGen15Private::_preBegin__samplerRECT, &ColladaParserAutoGen15Private::_preEnd__samplerRECT, &ColladaParserAutoGen15Private::_freeAttributes__samplerRECT);
functionMap[HASH_ELEMENT_SAMPLER_IMAGE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler_image, &ColladaParserAutoGen15Private::_data__sampler_image, &ColladaParserAutoGen15Private::_end__sampler_image, &ColladaParserAutoGen15Private::_preBegin__sampler_image, &ColladaParserAutoGen15Private::_preEnd__sampler_image, &ColladaParserAutoGen15Private::_freeAttributes__sampler_image);
functionMap[HASH_ELEMENT_SAMPLER_STATES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler_states, &ColladaParserAutoGen15Private::_data__sampler_states, &ColladaParserAutoGen15Private::_end__sampler_states, &ColladaParserAutoGen15Private::_preBegin__sampler_states, &ColladaParserAutoGen15Private::_preEnd__sampler_states, &ColladaParserAutoGen15Private::_freeAttributes__sampler_states);
functionMap[HASH_ELEMENT_SCENE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__scene, &ColladaParserAutoGen15Private::_data__scene, &ColladaParserAutoGen15Private::_end__scene, &ColladaParserAutoGen15Private::_preBegin__scene, &ColladaParserAutoGen15Private::_preEnd__scene, &ColladaParserAutoGen15Private::_freeAttributes__scene);
functionMap[HASH_ELEMENT_PROFILE_GLES2__NEWPARAM__USERTYPE__SETPARAM] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__newparam__usertype__setparam, &ColladaParserAutoGen15Private::_data__profile_GLES2__newparam__usertype__setparam, &ColladaParserAutoGen15Private::_end__profile_GLES2__newparam__usertype__setparam, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__newparam__usertype__setparam, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__newparam__usertype__setparam, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__newparam__usertype__setparam);
functionMap[HASH_ELEMENT_PROFILE_CG__TECHNIQUE__PASS__PROGRAM__SHADER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_CG__technique__pass__program__shader, &ColladaParserAutoGen15Private::_data__profile_CG__technique__pass__program__shader, &ColladaParserAutoGen15Private::_end__profile_CG__technique__pass__program__shader, &ColladaParserAutoGen15Private::_preBegin__profile_CG__technique__pass__program__shader, &ColladaParserAutoGen15Private::_preEnd__profile_CG__technique__pass__program__shader, &ColladaParserAutoGen15Private::_freeAttributes__profile_CG__technique__pass__program__shader);
functionMap[HASH_ELEMENT_RIGID_BODY__TECHNIQUE_COMMON__SHAPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__rigid_body__technique_common__shape, &ColladaParserAutoGen15Private::_data__rigid_body__technique_common__shape, &ColladaParserAutoGen15Private::_end__rigid_body__technique_common__shape, &ColladaParserAutoGen15Private::_preBegin__rigid_body__technique_common__shape, &ColladaParserAutoGen15Private::_preEnd__rigid_body__technique_common__shape, &ColladaParserAutoGen15Private::_freeAttributes__rigid_body__technique_common__shape);
functionMap[HASH_ELEMENT_SHELLS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__shells, &ColladaParserAutoGen15Private::_data__shells, &ColladaParserAutoGen15Private::_end__shells, &ColladaParserAutoGen15Private::_preBegin__shells, &ColladaParserAutoGen15Private::_preEnd__shells, &ColladaParserAutoGen15Private::_freeAttributes__shells);
functionMap[HASH_ELEMENT_SKIN] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__skin, &ColladaParserAutoGen15Private::_data__skin, &ColladaParserAutoGen15Private::_end__skin, &ColladaParserAutoGen15Private::_preBegin__skin, &ColladaParserAutoGen15Private::_preEnd__skin, &ColladaParserAutoGen15Private::_freeAttributes__skin);
functionMap[HASH_ELEMENT_SOLIDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__solids, &ColladaParserAutoGen15Private::_data__solids, &ColladaParserAutoGen15Private::_end__solids, &ColladaParserAutoGen15Private::_preBegin__solids, &ColladaParserAutoGen15Private::_preEnd__solids, &ColladaParserAutoGen15Private::_freeAttributes__solids);
functionMap[HASH_ELEMENT_SOURCE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source, &ColladaParserAutoGen15Private::_data__source, &ColladaParserAutoGen15Private::_end__source, &ColladaParserAutoGen15Private::_preBegin__source, &ColladaParserAutoGen15Private::_preEnd__source, &ColladaParserAutoGen15Private::_freeAttributes__source);
functionMap[HASH_ELEMENT_SPECULAR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__specular, &ColladaParserAutoGen15Private::_data__specular, &ColladaParserAutoGen15Private::_end__specular, &ColladaParserAutoGen15Private::_preBegin__specular, &ColladaParserAutoGen15Private::_preEnd__specular, &ColladaParserAutoGen15Private::_freeAttributes__specular);
functionMap[HASH_ELEMENT_SPHERE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sphere, &ColladaParserAutoGen15Private::_data__sphere, &ColladaParserAutoGen15Private::_end__sphere, &ColladaParserAutoGen15Private::_preBegin__sphere, &ColladaParserAutoGen15Private::_preEnd__sphere, &ColladaParserAutoGen15Private::_freeAttributes__sphere);
functionMap[HASH_ELEMENT_SPLINE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__spline, &ColladaParserAutoGen15Private::_data__spline, &ColladaParserAutoGen15Private::_end__spline, &ColladaParserAutoGen15Private::_preBegin__spline, &ColladaParserAutoGen15Private::_preEnd__spline, &ColladaParserAutoGen15Private::_freeAttributes__spline);
functionMap[HASH_ELEMENT_GLES2_PASS_TYPE____STATES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__gles2_pass_type____states, &ColladaParserAutoGen15Private::_data__gles2_pass_type____states, &ColladaParserAutoGen15Private::_end__gles2_pass_type____states, &ColladaParserAutoGen15Private::_preBegin__gles2_pass_type____states, &ColladaParserAutoGen15Private::_preEnd__gles2_pass_type____states, &ColladaParserAutoGen15Private::_freeAttributes__gles2_pass_type____states);
functionMap[HASH_ELEMENT_STENCIL_TARGET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__stencil_target, &ColladaParserAutoGen15Private::_data__stencil_target, &ColladaParserAutoGen15Private::_end__stencil_target, &ColladaParserAutoGen15Private::_preBegin__stencil_target, &ColladaParserAutoGen15Private::_preEnd__stencil_target, &ColladaParserAutoGen15Private::_freeAttributes__stencil_target);
functionMap[HASH_ELEMENT_SURFACE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__surface, &ColladaParserAutoGen15Private::_data__surface, &ColladaParserAutoGen15Private::_end__surface, &ColladaParserAutoGen15Private::_preBegin__surface, &ColladaParserAutoGen15Private::_preEnd__surface, &ColladaParserAutoGen15Private::_freeAttributes__surface);
functionMap[HASH_ELEMENT_SURFACE_CURVES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__surface_curves, &ColladaParserAutoGen15Private::_data__surface_curves, &ColladaParserAutoGen15Private::_end__surface_curves, &ColladaParserAutoGen15Private::_preBegin__surface_curves, &ColladaParserAutoGen15Private::_preEnd__surface_curves, &ColladaParserAutoGen15Private::_freeAttributes__surface_curves);
functionMap[HASH_ELEMENT_SURFACES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__surfaces, &ColladaParserAutoGen15Private::_data__surfaces, &ColladaParserAutoGen15Private::_end__surfaces, &ColladaParserAutoGen15Private::_preBegin__surfaces, &ColladaParserAutoGen15Private::_preEnd__surfaces, &ColladaParserAutoGen15Private::_freeAttributes__surfaces);
functionMap[HASH_ELEMENT_SWEPT_SURFACE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__swept_surface, &ColladaParserAutoGen15Private::_data__swept_surface, &ColladaParserAutoGen15Private::_end__swept_surface, &ColladaParserAutoGen15Private::_preBegin__swept_surface, &ColladaParserAutoGen15Private::_preEnd__swept_surface, &ColladaParserAutoGen15Private::_freeAttributes__swept_surface);
functionMap[HASH_ELEMENT_TARGETS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__targets, &ColladaParserAutoGen15Private::_data__targets, &ColladaParserAutoGen15Private::_end__targets, &ColladaParserAutoGen15Private::_preBegin__targets, &ColladaParserAutoGen15Private::_preEnd__targets, &ColladaParserAutoGen15Private::_freeAttributes__targets);
functionMap[HASH_ELEMENT_PROFILE_GLES2_TYPE____TECHNIQUE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_gles2_type____technique, &ColladaParserAutoGen15Private::_data__profile_gles2_type____technique, &ColladaParserAutoGen15Private::_end__profile_gles2_type____technique, &ColladaParserAutoGen15Private::_preBegin__profile_gles2_type____technique, &ColladaParserAutoGen15Private::_preEnd__profile_gles2_type____technique, &ColladaParserAutoGen15Private::_freeAttributes__profile_gles2_type____technique);
functionMap[HASH_ELEMENT_RIGID_BODY_TYPE____TECHNIQUE_COMMON] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__rigid_body_type____technique_common, &ColladaParserAutoGen15Private::_data__rigid_body_type____technique_common, &ColladaParserAutoGen15Private::_end__rigid_body_type____technique_common, &ColladaParserAutoGen15Private::_preBegin__rigid_body_type____technique_common, &ColladaParserAutoGen15Private::_preEnd__rigid_body_type____technique_common, &ColladaParserAutoGen15Private::_freeAttributes__rigid_body_type____technique_common);
functionMap[HASH_ELEMENT_TEXCOMBINER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__texcombiner, &ColladaParserAutoGen15Private::_data__texcombiner, &ColladaParserAutoGen15Private::_end__texcombiner, &ColladaParserAutoGen15Private::_preBegin__texcombiner, &ColladaParserAutoGen15Private::_preEnd__texcombiner, &ColladaParserAutoGen15Private::_freeAttributes__texcombiner);
functionMap[HASH_ELEMENT_TEXENV] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__texenv, &ColladaParserAutoGen15Private::_data__texenv, &ColladaParserAutoGen15Private::_end__texenv, &ColladaParserAutoGen15Private::_preBegin__texenv, &ColladaParserAutoGen15Private::_preEnd__texenv, &ColladaParserAutoGen15Private::_freeAttributes__texenv);
functionMap[HASH_ELEMENT_TEXTURE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__texture, &ColladaParserAutoGen15Private::_data__texture, &ColladaParserAutoGen15Private::_end__texture, &ColladaParserAutoGen15Private::_preBegin__texture, &ColladaParserAutoGen15Private::_preEnd__texture, &ColladaParserAutoGen15Private::_freeAttributes__texture);
functionMap[HASH_ELEMENT_TEXTURE1D] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__texture1D, &ColladaParserAutoGen15Private::_data__texture1D, &ColladaParserAutoGen15Private::_end__texture1D, &ColladaParserAutoGen15Private::_preBegin__texture1D, &ColladaParserAutoGen15Private::_preEnd__texture1D, &ColladaParserAutoGen15Private::_freeAttributes__texture1D);
functionMap[HASH_ELEMENT_TEXTURE2D] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__texture2D, &ColladaParserAutoGen15Private::_data__texture2D, &ColladaParserAutoGen15Private::_end__texture2D, &ColladaParserAutoGen15Private::_preBegin__texture2D, &ColladaParserAutoGen15Private::_preEnd__texture2D, &ColladaParserAutoGen15Private::_freeAttributes__texture2D);
functionMap[HASH_ELEMENT_TEXTURE3D] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__texture3D, &ColladaParserAutoGen15Private::_data__texture3D, &ColladaParserAutoGen15Private::_end__texture3D, &ColladaParserAutoGen15Private::_preBegin__texture3D, &ColladaParserAutoGen15Private::_preEnd__texture3D, &ColladaParserAutoGen15Private::_freeAttributes__texture3D);
functionMap[HASH_ELEMENT_TEXTURECUBE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__textureCUBE, &ColladaParserAutoGen15Private::_data__textureCUBE, &ColladaParserAutoGen15Private::_end__textureCUBE, &ColladaParserAutoGen15Private::_preBegin__textureCUBE, &ColladaParserAutoGen15Private::_preEnd__textureCUBE, &ColladaParserAutoGen15Private::_freeAttributes__textureCUBE);
functionMap[HASH_ELEMENT_TEXTUREDEPTH] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__textureDEPTH, &ColladaParserAutoGen15Private::_data__textureDEPTH, &ColladaParserAutoGen15Private::_end__textureDEPTH, &ColladaParserAutoGen15Private::_preBegin__textureDEPTH, &ColladaParserAutoGen15Private::_preEnd__textureDEPTH, &ColladaParserAutoGen15Private::_freeAttributes__textureDEPTH);
functionMap[HASH_ELEMENT_TEXTURERECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__textureRECT, &ColladaParserAutoGen15Private::_data__textureRECT, &ColladaParserAutoGen15Private::_end__textureRECT, &ColladaParserAutoGen15Private::_preBegin__textureRECT, &ColladaParserAutoGen15Private::_preEnd__textureRECT, &ColladaParserAutoGen15Private::_freeAttributes__textureRECT);
functionMap[HASH_ELEMENT_TEXTURE_PIPELINE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__texture_pipeline, &ColladaParserAutoGen15Private::_data__texture_pipeline, &ColladaParserAutoGen15Private::_end__texture_pipeline, &ColladaParserAutoGen15Private::_preBegin__texture_pipeline, &ColladaParserAutoGen15Private::_preEnd__texture_pipeline, &ColladaParserAutoGen15Private::_freeAttributes__texture_pipeline);
functionMap[HASH_ELEMENT_TORUS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__torus, &ColladaParserAutoGen15Private::_data__torus, &ColladaParserAutoGen15Private::_end__torus, &ColladaParserAutoGen15Private::_preBegin__torus, &ColladaParserAutoGen15Private::_preEnd__torus, &ColladaParserAutoGen15Private::_freeAttributes__torus);
functionMap[HASH_ELEMENT_TRANSPARENT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__transparent, &ColladaParserAutoGen15Private::_data__transparent, &ColladaParserAutoGen15Private::_end__transparent, &ColladaParserAutoGen15Private::_preBegin__transparent, &ColladaParserAutoGen15Private::_preEnd__transparent, &ColladaParserAutoGen15Private::_freeAttributes__transparent);
functionMap[HASH_ELEMENT_TRIANGLES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__triangles, &ColladaParserAutoGen15Private::_data__triangles, &ColladaParserAutoGen15Private::_end__triangles, &ColladaParserAutoGen15Private::_preBegin__triangles, &ColladaParserAutoGen15Private::_preEnd__triangles, &ColladaParserAutoGen15Private::_freeAttributes__triangles);
functionMap[HASH_ELEMENT_TRIFANS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__trifans, &ColladaParserAutoGen15Private::_data__trifans, &ColladaParserAutoGen15Private::_end__trifans, &ColladaParserAutoGen15Private::_preBegin__trifans, &ColladaParserAutoGen15Private::_preEnd__trifans, &ColladaParserAutoGen15Private::_freeAttributes__trifans);
functionMap[HASH_ELEMENT_TRISTRIPS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__tristrips, &ColladaParserAutoGen15Private::_data__tristrips, &ColladaParserAutoGen15Private::_end__tristrips, &ColladaParserAutoGen15Private::_preBegin__tristrips, &ColladaParserAutoGen15Private::_preEnd__tristrips, &ColladaParserAutoGen15Private::_freeAttributes__tristrips);
functionMap[HASH_ELEMENT_PROFILE_GLES2__NEWPARAM__USERTYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__newparam__usertype, &ColladaParserAutoGen15Private::_data__profile_GLES2__newparam__usertype, &ColladaParserAutoGen15Private::_end__profile_GLES2__newparam__usertype, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__newparam__usertype, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__newparam__usertype, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__newparam__usertype);
functionMap[HASH_ELEMENT_VALUE____GLES_TEXTURE_PIPELINE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__value____gles_texture_pipeline_type, &ColladaParserAutoGen15Private::_data__value____gles_texture_pipeline_type, &ColladaParserAutoGen15Private::_end__value____gles_texture_pipeline_type, &ColladaParserAutoGen15Private::_preBegin__value____gles_texture_pipeline_type, &ColladaParserAutoGen15Private::_preEnd__value____gles_texture_pipeline_type, &ColladaParserAutoGen15Private::_freeAttributes__value____gles_texture_pipeline_type);
functionMap[HASH_ELEMENT_VERTEX_WEIGHTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__vertex_weights, &ColladaParserAutoGen15Private::_data__vertex_weights, &ColladaParserAutoGen15Private::_end__vertex_weights, &ColladaParserAutoGen15Private::_preBegin__vertex_weights, &ColladaParserAutoGen15Private::_preEnd__vertex_weights, &ColladaParserAutoGen15Private::_freeAttributes__vertex_weights);
functionMap[HASH_ELEMENT_VERTICES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__vertices, &ColladaParserAutoGen15Private::_data__vertices, &ColladaParserAutoGen15Private::_end__vertices, &ColladaParserAutoGen15Private::_preBegin__vertices, &ColladaParserAutoGen15Private::_preEnd__vertices, &ColladaParserAutoGen15Private::_freeAttributes__vertices);
functionMap[HASH_ELEMENT_VISUAL_SCENE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__visual_scene, &ColladaParserAutoGen15Private::_data__visual_scene, &ColladaParserAutoGen15Private::_end__visual_scene, &ColladaParserAutoGen15Private::_preBegin__visual_scene, &ColladaParserAutoGen15Private::_preEnd__visual_scene, &ColladaParserAutoGen15Private::_freeAttributes__visual_scene);
functionMap[HASH_ELEMENT_WIRES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__wires, &ColladaParserAutoGen15Private::_data__wires, &ColladaParserAutoGen15Private::_end__wires, &ColladaParserAutoGen15Private::_preBegin__wires, &ColladaParserAutoGen15Private::_preEnd__wires, &ColladaParserAutoGen15Private::_freeAttributes__wires);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_CONTRIBUTOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__contributor, &ColladaParserAutoGen15Private::_data__contributor, &ColladaParserAutoGen15Private::_end__contributor, &ColladaParserAutoGen15Private::_preBegin__contributor, &ColladaParserAutoGen15Private::_preEnd__contributor, &ColladaParserAutoGen15Private::_freeAttributes__contributor);
functionMap[HASH_ELEMENT_COVERAGE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__coverage, &ColladaParserAutoGen15Private::_data__coverage, &ColladaParserAutoGen15Private::_end__coverage, &ColladaParserAutoGen15Private::_preBegin__coverage, &ColladaParserAutoGen15Private::_preEnd__coverage, &ColladaParserAutoGen15Private::_freeAttributes__coverage);
functionMap[HASH_ELEMENT_CREATED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__created, &ColladaParserAutoGen15Private::_data__created, &ColladaParserAutoGen15Private::_end__created, &ColladaParserAutoGen15Private::_preBegin__created, &ColladaParserAutoGen15Private::_preEnd__created, &ColladaParserAutoGen15Private::_freeAttributes__created);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_AUTHOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author, &ColladaParserAutoGen15Private::_data__author, &ColladaParserAutoGen15Private::_end__author, &ColladaParserAutoGen15Private::_preBegin__author, &ColladaParserAutoGen15Private::_preEnd__author, &ColladaParserAutoGen15Private::_freeAttributes__author);
functionMap[HASH_ELEMENT_AUTHOR_EMAIL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_email, &ColladaParserAutoGen15Private::_data__author_email, &ColladaParserAutoGen15Private::_end__author_email, &ColladaParserAutoGen15Private::_preBegin__author_email, &ColladaParserAutoGen15Private::_preEnd__author_email, &ColladaParserAutoGen15Private::_freeAttributes__author_email);
functionMap[HASH_ELEMENT_AUTHOR_WEBSITE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_website, &ColladaParserAutoGen15Private::_data__author_website, &ColladaParserAutoGen15Private::_end__author_website, &ColladaParserAutoGen15Private::_preBegin__author_website, &ColladaParserAutoGen15Private::_preEnd__author_website, &ColladaParserAutoGen15Private::_freeAttributes__author_website);
functionMap[HASH_ELEMENT_AUTHORING_TOOL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__authoring_tool, &ColladaParserAutoGen15Private::_data__authoring_tool, &ColladaParserAutoGen15Private::_end__authoring_tool, &ColladaParserAutoGen15Private::_preBegin__authoring_tool, &ColladaParserAutoGen15Private::_preEnd__authoring_tool, &ColladaParserAutoGen15Private::_freeAttributes__authoring_tool);
functionMap[HASH_ELEMENT_COMMENTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__comments, &ColladaParserAutoGen15Private::_data__comments, &ColladaParserAutoGen15Private::_end__comments, &ColladaParserAutoGen15Private::_preBegin__comments, &ColladaParserAutoGen15Private::_preEnd__comments, &ColladaParserAutoGen15Private::_freeAttributes__comments);
functionMap[HASH_ELEMENT_COPYRIGHT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__copyright, &ColladaParserAutoGen15Private::_data__copyright, &ColladaParserAutoGen15Private::_end__copyright, &ColladaParserAutoGen15Private::_preBegin__copyright, &ColladaParserAutoGen15Private::_preEnd__copyright, &ColladaParserAutoGen15Private::_freeAttributes__copyright);
functionMap[HASH_ELEMENT_SOURCE_DATA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source_data, &ColladaParserAutoGen15Private::_data__source_data, &ColladaParserAutoGen15Private::_end__source_data, &ColladaParserAutoGen15Private::_preBegin__source_data, &ColladaParserAutoGen15Private::_preEnd__source_data, &ColladaParserAutoGen15Private::_freeAttributes__source_data);
functionMap[HASH_ELEMENT_GEOGRAPHIC_LOCATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__geographic_location, &ColladaParserAutoGen15Private::_data__geographic_location, &ColladaParserAutoGen15Private::_end__geographic_location, &ColladaParserAutoGen15Private::_preBegin__geographic_location, &ColladaParserAutoGen15Private::_preEnd__geographic_location, &ColladaParserAutoGen15Private::_freeAttributes__geographic_location);
functionMap[HASH_ELEMENT_ALTITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__altitude, &ColladaParserAutoGen15Private::_data__altitude, &ColladaParserAutoGen15Private::_end__altitude, &ColladaParserAutoGen15Private::_preBegin__altitude, &ColladaParserAutoGen15Private::_preEnd__altitude, &ColladaParserAutoGen15Private::_freeAttributes__altitude);
functionMap[HASH_ELEMENT_LATITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__latitude, &ColladaParserAutoGen15Private::_data__latitude, &ColladaParserAutoGen15Private::_end__latitude, &ColladaParserAutoGen15Private::_preBegin__latitude, &ColladaParserAutoGen15Private::_preEnd__latitude, &ColladaParserAutoGen15Private::_freeAttributes__latitude);
functionMap[HASH_ELEMENT_LONGITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__longitude, &ColladaParserAutoGen15Private::_data__longitude, &ColladaParserAutoGen15Private::_end__longitude, &ColladaParserAutoGen15Private::_preBegin__longitude, &ColladaParserAutoGen15Private::_preEnd__longitude, &ColladaParserAutoGen15Private::_freeAttributes__longitude);
mapFilled = true;
}
return functionMap;
}

//---------------------------------------------------------------------
const ColladaParserAutoGen15Private::ElementFunctionMap& FunctionMapFactory::createFunctionMap__scene__allChildren() {
static ColladaParserAutoGen15Private::ElementFunctionMap functionMap;
static bool mapFilled = false;
if ( !mapFilled )
{
functionMap[HASH_ELEMENT_COLLADA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__COLLADA, &ColladaParserAutoGen15Private::_data__COLLADA, &ColladaParserAutoGen15Private::_end__COLLADA, &ColladaParserAutoGen15Private::_preBegin__COLLADA, &ColladaParserAutoGen15Private::_preEnd__COLLADA, &ColladaParserAutoGen15Private::_freeAttributes__COLLADA);
functionMap[HASH_ELEMENT_SCENE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__scene, &ColladaParserAutoGen15Private::_data__scene, &ColladaParserAutoGen15Private::_end__scene, &ColladaParserAutoGen15Private::_preBegin__scene, &ColladaParserAutoGen15Private::_preEnd__scene, &ColladaParserAutoGen15Private::_freeAttributes__scene);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INSTANCE_KINEMATICS_SCENE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_kinematics_scene, &ColladaParserAutoGen15Private::_data__instance_kinematics_scene, &ColladaParserAutoGen15Private::_end__instance_kinematics_scene, &ColladaParserAutoGen15Private::_preBegin__instance_kinematics_scene, &ColladaParserAutoGen15Private::_preEnd__instance_kinematics_scene, &ColladaParserAutoGen15Private::_freeAttributes__instance_kinematics_scene);
functionMap[HASH_ELEMENT_INSTANCE_PHYSICS_SCENE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_physics_scene, &ColladaParserAutoGen15Private::_data__instance_physics_scene, &ColladaParserAutoGen15Private::_end__instance_physics_scene, &ColladaParserAutoGen15Private::_preBegin__instance_physics_scene, &ColladaParserAutoGen15Private::_preEnd__instance_physics_scene, &ColladaParserAutoGen15Private::_freeAttributes__instance_physics_scene);
functionMap[HASH_ELEMENT_INSTANCE_VISUAL_SCENE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_visual_scene, &ColladaParserAutoGen15Private::_data__instance_visual_scene, &ColladaParserAutoGen15Private::_end__instance_visual_scene, &ColladaParserAutoGen15Private::_preBegin__instance_visual_scene, &ColladaParserAutoGen15Private::_preEnd__instance_visual_scene, &ColladaParserAutoGen15Private::_freeAttributes__instance_visual_scene);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_CONTRIBUTOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__contributor, &ColladaParserAutoGen15Private::_data__contributor, &ColladaParserAutoGen15Private::_end__contributor, &ColladaParserAutoGen15Private::_preBegin__contributor, &ColladaParserAutoGen15Private::_preEnd__contributor, &ColladaParserAutoGen15Private::_freeAttributes__contributor);
functionMap[HASH_ELEMENT_COVERAGE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__coverage, &ColladaParserAutoGen15Private::_data__coverage, &ColladaParserAutoGen15Private::_end__coverage, &ColladaParserAutoGen15Private::_preBegin__coverage, &ColladaParserAutoGen15Private::_preEnd__coverage, &ColladaParserAutoGen15Private::_freeAttributes__coverage);
functionMap[HASH_ELEMENT_CREATED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__created, &ColladaParserAutoGen15Private::_data__created, &ColladaParserAutoGen15Private::_end__created, &ColladaParserAutoGen15Private::_preBegin__created, &ColladaParserAutoGen15Private::_preEnd__created, &ColladaParserAutoGen15Private::_freeAttributes__created);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_AUTHOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author, &ColladaParserAutoGen15Private::_data__author, &ColladaParserAutoGen15Private::_end__author, &ColladaParserAutoGen15Private::_preBegin__author, &ColladaParserAutoGen15Private::_preEnd__author, &ColladaParserAutoGen15Private::_freeAttributes__author);
functionMap[HASH_ELEMENT_AUTHOR_EMAIL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_email, &ColladaParserAutoGen15Private::_data__author_email, &ColladaParserAutoGen15Private::_end__author_email, &ColladaParserAutoGen15Private::_preBegin__author_email, &ColladaParserAutoGen15Private::_preEnd__author_email, &ColladaParserAutoGen15Private::_freeAttributes__author_email);
functionMap[HASH_ELEMENT_AUTHOR_WEBSITE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_website, &ColladaParserAutoGen15Private::_data__author_website, &ColladaParserAutoGen15Private::_end__author_website, &ColladaParserAutoGen15Private::_preBegin__author_website, &ColladaParserAutoGen15Private::_preEnd__author_website, &ColladaParserAutoGen15Private::_freeAttributes__author_website);
functionMap[HASH_ELEMENT_AUTHORING_TOOL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__authoring_tool, &ColladaParserAutoGen15Private::_data__authoring_tool, &ColladaParserAutoGen15Private::_end__authoring_tool, &ColladaParserAutoGen15Private::_preBegin__authoring_tool, &ColladaParserAutoGen15Private::_preEnd__authoring_tool, &ColladaParserAutoGen15Private::_freeAttributes__authoring_tool);
functionMap[HASH_ELEMENT_COMMENTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__comments, &ColladaParserAutoGen15Private::_data__comments, &ColladaParserAutoGen15Private::_end__comments, &ColladaParserAutoGen15Private::_preBegin__comments, &ColladaParserAutoGen15Private::_preEnd__comments, &ColladaParserAutoGen15Private::_freeAttributes__comments);
functionMap[HASH_ELEMENT_COPYRIGHT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__copyright, &ColladaParserAutoGen15Private::_data__copyright, &ColladaParserAutoGen15Private::_end__copyright, &ColladaParserAutoGen15Private::_preBegin__copyright, &ColladaParserAutoGen15Private::_preEnd__copyright, &ColladaParserAutoGen15Private::_freeAttributes__copyright);
functionMap[HASH_ELEMENT_SOURCE_DATA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source_data, &ColladaParserAutoGen15Private::_data__source_data, &ColladaParserAutoGen15Private::_end__source_data, &ColladaParserAutoGen15Private::_preBegin__source_data, &ColladaParserAutoGen15Private::_preEnd__source_data, &ColladaParserAutoGen15Private::_freeAttributes__source_data);
functionMap[HASH_ELEMENT_GEOGRAPHIC_LOCATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__geographic_location, &ColladaParserAutoGen15Private::_data__geographic_location, &ColladaParserAutoGen15Private::_end__geographic_location, &ColladaParserAutoGen15Private::_preBegin__geographic_location, &ColladaParserAutoGen15Private::_preEnd__geographic_location, &ColladaParserAutoGen15Private::_freeAttributes__geographic_location);
functionMap[HASH_ELEMENT_ALTITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__altitude, &ColladaParserAutoGen15Private::_data__altitude, &ColladaParserAutoGen15Private::_end__altitude, &ColladaParserAutoGen15Private::_preBegin__altitude, &ColladaParserAutoGen15Private::_preEnd__altitude, &ColladaParserAutoGen15Private::_freeAttributes__altitude);
functionMap[HASH_ELEMENT_LATITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__latitude, &ColladaParserAutoGen15Private::_data__latitude, &ColladaParserAutoGen15Private::_end__latitude, &ColladaParserAutoGen15Private::_preBegin__latitude, &ColladaParserAutoGen15Private::_preEnd__latitude, &ColladaParserAutoGen15Private::_freeAttributes__latitude);
functionMap[HASH_ELEMENT_LONGITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__longitude, &ColladaParserAutoGen15Private::_data__longitude, &ColladaParserAutoGen15Private::_end__longitude, &ColladaParserAutoGen15Private::_preBegin__longitude, &ColladaParserAutoGen15Private::_preEnd__longitude, &ColladaParserAutoGen15Private::_freeAttributes__longitude);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_BIND_JOINT_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bind_joint_axis, &ColladaParserAutoGen15Private::_data__bind_joint_axis, &ColladaParserAutoGen15Private::_end__bind_joint_axis, &ColladaParserAutoGen15Private::_preBegin__bind_joint_axis, &ColladaParserAutoGen15Private::_preEnd__bind_joint_axis, &ColladaParserAutoGen15Private::_freeAttributes__bind_joint_axis);
functionMap[HASH_ELEMENT_BIND_KINEMATICS_MODEL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bind_kinematics_model, &ColladaParserAutoGen15Private::_data__bind_kinematics_model, &ColladaParserAutoGen15Private::_end__bind_kinematics_model, &ColladaParserAutoGen15Private::_preBegin__bind_kinematics_model, &ColladaParserAutoGen15Private::_preEnd__bind_kinematics_model, &ColladaParserAutoGen15Private::_freeAttributes__bind_kinematics_model);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_NEWPARAM____KINEMATICS_NEWPARAM_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__newparam____kinematics_newparam_type, &ColladaParserAutoGen15Private::_data__newparam____kinematics_newparam_type, &ColladaParserAutoGen15Private::_end__newparam____kinematics_newparam_type, &ColladaParserAutoGen15Private::_preBegin__newparam____kinematics_newparam_type, &ColladaParserAutoGen15Private::_preEnd__newparam____kinematics_newparam_type, &ColladaParserAutoGen15Private::_freeAttributes__newparam____kinematics_newparam_type);
functionMap[HASH_ELEMENT_SETPARAM____KINEMATICS_SETPARAM_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__setparam____kinematics_setparam_type, &ColladaParserAutoGen15Private::_data__setparam____kinematics_setparam_type, &ColladaParserAutoGen15Private::_end__setparam____kinematics_setparam_type, &ColladaParserAutoGen15Private::_preBegin__setparam____kinematics_setparam_type, &ColladaParserAutoGen15Private::_preEnd__setparam____kinematics_setparam_type, &ColladaParserAutoGen15Private::_freeAttributes__setparam____kinematics_setparam_type);
functionMap[HASH_ELEMENT_AXIS____COMMON_SIDREF_OR_PARAM_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__axis____common_sidref_or_param_type, &ColladaParserAutoGen15Private::_data__axis____common_sidref_or_param_type, &ColladaParserAutoGen15Private::_end__axis____common_sidref_or_param_type, &ColladaParserAutoGen15Private::_preBegin__axis____common_sidref_or_param_type, &ColladaParserAutoGen15Private::_preEnd__axis____common_sidref_or_param_type, &ColladaParserAutoGen15Private::_freeAttributes__axis____common_sidref_or_param_type);
functionMap[HASH_ELEMENT_VALUE____COMMON_FLOAT_OR_PARAM_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__value____common_float_or_param_type, &ColladaParserAutoGen15Private::_data__value____common_float_or_param_type, &ColladaParserAutoGen15Private::_end__value____common_float_or_param_type, &ColladaParserAutoGen15Private::_preBegin__value____common_float_or_param_type, &ColladaParserAutoGen15Private::_preEnd__value____common_float_or_param_type, &ColladaParserAutoGen15Private::_freeAttributes__value____common_float_or_param_type);
functionMap[HASH_ELEMENT_SIDREF] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__SIDREF, &ColladaParserAutoGen15Private::_data__SIDREF, &ColladaParserAutoGen15Private::_end__SIDREF, &ColladaParserAutoGen15Private::_preBegin__SIDREF, &ColladaParserAutoGen15Private::_preEnd__SIDREF, &ColladaParserAutoGen15Private::_freeAttributes__SIDREF);
functionMap[HASH_ELEMENT_PARAM____COMMON_PARAM_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__param____common_param_type, &ColladaParserAutoGen15Private::_data__param____common_param_type, &ColladaParserAutoGen15Private::_end__param____common_param_type, &ColladaParserAutoGen15Private::_preBegin__param____common_param_type, &ColladaParserAutoGen15Private::_preEnd__param____common_param_type, &ColladaParserAutoGen15Private::_freeAttributes__param____common_param_type);
functionMap[HASH_ELEMENT_FLOAT____FLOAT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float____float_type, &ColladaParserAutoGen15Private::_data__float____float_type, &ColladaParserAutoGen15Private::_end__float____float_type, &ColladaParserAutoGen15Private::_preBegin__float____float_type, &ColladaParserAutoGen15Private::_preEnd__float____float_type, &ColladaParserAutoGen15Private::_freeAttributes__float____float_type);
functionMap[HASH_ELEMENT_PARAM____COMMON_PARAM_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__param____common_param_type, &ColladaParserAutoGen15Private::_data__param____common_param_type, &ColladaParserAutoGen15Private::_end__param____common_param_type, &ColladaParserAutoGen15Private::_preBegin__param____common_param_type, &ColladaParserAutoGen15Private::_preEnd__param____common_param_type, &ColladaParserAutoGen15Private::_freeAttributes__param____common_param_type);
functionMap[HASH_ELEMENT_SIDREF] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__SIDREF, &ColladaParserAutoGen15Private::_data__SIDREF, &ColladaParserAutoGen15Private::_end__SIDREF, &ColladaParserAutoGen15Private::_preBegin__SIDREF, &ColladaParserAutoGen15Private::_preEnd__SIDREF, &ColladaParserAutoGen15Private::_freeAttributes__SIDREF);
functionMap[HASH_ELEMENT_BOOL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool, &ColladaParserAutoGen15Private::_data__bool, &ColladaParserAutoGen15Private::_end__bool, &ColladaParserAutoGen15Private::_preBegin__bool, &ColladaParserAutoGen15Private::_preEnd__bool, &ColladaParserAutoGen15Private::_freeAttributes__bool);
functionMap[HASH_ELEMENT_FLOAT____FLOAT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float____float_type, &ColladaParserAutoGen15Private::_data__float____float_type, &ColladaParserAutoGen15Private::_end__float____float_type, &ColladaParserAutoGen15Private::_preBegin__float____float_type, &ColladaParserAutoGen15Private::_preEnd__float____float_type, &ColladaParserAutoGen15Private::_freeAttributes__float____float_type);
functionMap[HASH_ELEMENT_INT____INT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int____int_type, &ColladaParserAutoGen15Private::_data__int____int_type, &ColladaParserAutoGen15Private::_end__int____int_type, &ColladaParserAutoGen15Private::_preBegin__int____int_type, &ColladaParserAutoGen15Private::_preEnd__int____int_type, &ColladaParserAutoGen15Private::_freeAttributes__int____int_type);
functionMap[HASH_ELEMENT_SIDREF] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__SIDREF, &ColladaParserAutoGen15Private::_data__SIDREF, &ColladaParserAutoGen15Private::_end__SIDREF, &ColladaParserAutoGen15Private::_preBegin__SIDREF, &ColladaParserAutoGen15Private::_preEnd__SIDREF, &ColladaParserAutoGen15Private::_freeAttributes__SIDREF);
functionMap[HASH_ELEMENT_BOOL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool, &ColladaParserAutoGen15Private::_data__bool, &ColladaParserAutoGen15Private::_end__bool, &ColladaParserAutoGen15Private::_preBegin__bool, &ColladaParserAutoGen15Private::_preEnd__bool, &ColladaParserAutoGen15Private::_freeAttributes__bool);
functionMap[HASH_ELEMENT_CONNECT_PARAM] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__connect_param, &ColladaParserAutoGen15Private::_data__connect_param, &ColladaParserAutoGen15Private::_end__connect_param, &ColladaParserAutoGen15Private::_preBegin__connect_param, &ColladaParserAutoGen15Private::_preEnd__connect_param, &ColladaParserAutoGen15Private::_freeAttributes__connect_param);
functionMap[HASH_ELEMENT_FLOAT____FLOAT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float____float_type, &ColladaParserAutoGen15Private::_data__float____float_type, &ColladaParserAutoGen15Private::_end__float____float_type, &ColladaParserAutoGen15Private::_preBegin__float____float_type, &ColladaParserAutoGen15Private::_preEnd__float____float_type, &ColladaParserAutoGen15Private::_freeAttributes__float____float_type);
functionMap[HASH_ELEMENT_INT____INT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int____int_type, &ColladaParserAutoGen15Private::_data__int____int_type, &ColladaParserAutoGen15Private::_end__int____int_type, &ColladaParserAutoGen15Private::_preBegin__int____int_type, &ColladaParserAutoGen15Private::_preEnd__int____int_type, &ColladaParserAutoGen15Private::_freeAttributes__int____int_type);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
mapFilled = true;
}
return functionMap;
}

//---------------------------------------------------------------------
const ColladaParserAutoGen15Private::ElementFunctionMap& FunctionMapFactory::createFunctionMap__library_animation_clips__allChildren() {
static ColladaParserAutoGen15Private::ElementFunctionMap functionMap;
static bool mapFilled = false;
if ( !mapFilled )
{
functionMap[HASH_ELEMENT_COLLADA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__COLLADA, &ColladaParserAutoGen15Private::_data__COLLADA, &ColladaParserAutoGen15Private::_end__COLLADA, &ColladaParserAutoGen15Private::_preBegin__COLLADA, &ColladaParserAutoGen15Private::_preEnd__COLLADA, &ColladaParserAutoGen15Private::_freeAttributes__COLLADA);
functionMap[HASH_ELEMENT_LIBRARY_ANIMATION_CLIPS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_animation_clips, &ColladaParserAutoGen15Private::_data__library_animation_clips, &ColladaParserAutoGen15Private::_end__library_animation_clips, &ColladaParserAutoGen15Private::_preBegin__library_animation_clips, &ColladaParserAutoGen15Private::_preEnd__library_animation_clips, &ColladaParserAutoGen15Private::_freeAttributes__library_animation_clips);
functionMap[HASH_ELEMENT_ANIMATION_CLIP] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__animation_clip, &ColladaParserAutoGen15Private::_data__animation_clip, &ColladaParserAutoGen15Private::_end__animation_clip, &ColladaParserAutoGen15Private::_preBegin__animation_clip, &ColladaParserAutoGen15Private::_preEnd__animation_clip, &ColladaParserAutoGen15Private::_freeAttributes__animation_clip);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INSTANCE_ANIMATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_animation, &ColladaParserAutoGen15Private::_data__instance_animation, &ColladaParserAutoGen15Private::_end__instance_animation, &ColladaParserAutoGen15Private::_preBegin__instance_animation, &ColladaParserAutoGen15Private::_preEnd__instance_animation, &ColladaParserAutoGen15Private::_freeAttributes__instance_animation);
functionMap[HASH_ELEMENT_INSTANCE_FORMULA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_formula, &ColladaParserAutoGen15Private::_data__instance_formula, &ColladaParserAutoGen15Private::_end__instance_formula, &ColladaParserAutoGen15Private::_preBegin__instance_formula, &ColladaParserAutoGen15Private::_preEnd__instance_formula, &ColladaParserAutoGen15Private::_freeAttributes__instance_formula);
functionMap[HASH_ELEMENT_CONTRIBUTOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__contributor, &ColladaParserAutoGen15Private::_data__contributor, &ColladaParserAutoGen15Private::_end__contributor, &ColladaParserAutoGen15Private::_preBegin__contributor, &ColladaParserAutoGen15Private::_preEnd__contributor, &ColladaParserAutoGen15Private::_freeAttributes__contributor);
functionMap[HASH_ELEMENT_COVERAGE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__coverage, &ColladaParserAutoGen15Private::_data__coverage, &ColladaParserAutoGen15Private::_end__coverage, &ColladaParserAutoGen15Private::_preBegin__coverage, &ColladaParserAutoGen15Private::_preEnd__coverage, &ColladaParserAutoGen15Private::_freeAttributes__coverage);
functionMap[HASH_ELEMENT_CREATED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__created, &ColladaParserAutoGen15Private::_data__created, &ColladaParserAutoGen15Private::_end__created, &ColladaParserAutoGen15Private::_preBegin__created, &ColladaParserAutoGen15Private::_preEnd__created, &ColladaParserAutoGen15Private::_freeAttributes__created);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_AUTHOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author, &ColladaParserAutoGen15Private::_data__author, &ColladaParserAutoGen15Private::_end__author, &ColladaParserAutoGen15Private::_preBegin__author, &ColladaParserAutoGen15Private::_preEnd__author, &ColladaParserAutoGen15Private::_freeAttributes__author);
functionMap[HASH_ELEMENT_AUTHOR_EMAIL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_email, &ColladaParserAutoGen15Private::_data__author_email, &ColladaParserAutoGen15Private::_end__author_email, &ColladaParserAutoGen15Private::_preBegin__author_email, &ColladaParserAutoGen15Private::_preEnd__author_email, &ColladaParserAutoGen15Private::_freeAttributes__author_email);
functionMap[HASH_ELEMENT_AUTHOR_WEBSITE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_website, &ColladaParserAutoGen15Private::_data__author_website, &ColladaParserAutoGen15Private::_end__author_website, &ColladaParserAutoGen15Private::_preBegin__author_website, &ColladaParserAutoGen15Private::_preEnd__author_website, &ColladaParserAutoGen15Private::_freeAttributes__author_website);
functionMap[HASH_ELEMENT_AUTHORING_TOOL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__authoring_tool, &ColladaParserAutoGen15Private::_data__authoring_tool, &ColladaParserAutoGen15Private::_end__authoring_tool, &ColladaParserAutoGen15Private::_preBegin__authoring_tool, &ColladaParserAutoGen15Private::_preEnd__authoring_tool, &ColladaParserAutoGen15Private::_freeAttributes__authoring_tool);
functionMap[HASH_ELEMENT_COMMENTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__comments, &ColladaParserAutoGen15Private::_data__comments, &ColladaParserAutoGen15Private::_end__comments, &ColladaParserAutoGen15Private::_preBegin__comments, &ColladaParserAutoGen15Private::_preEnd__comments, &ColladaParserAutoGen15Private::_freeAttributes__comments);
functionMap[HASH_ELEMENT_COPYRIGHT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__copyright, &ColladaParserAutoGen15Private::_data__copyright, &ColladaParserAutoGen15Private::_end__copyright, &ColladaParserAutoGen15Private::_preBegin__copyright, &ColladaParserAutoGen15Private::_preEnd__copyright, &ColladaParserAutoGen15Private::_freeAttributes__copyright);
functionMap[HASH_ELEMENT_SOURCE_DATA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source_data, &ColladaParserAutoGen15Private::_data__source_data, &ColladaParserAutoGen15Private::_end__source_data, &ColladaParserAutoGen15Private::_preBegin__source_data, &ColladaParserAutoGen15Private::_preEnd__source_data, &ColladaParserAutoGen15Private::_freeAttributes__source_data);
functionMap[HASH_ELEMENT_GEOGRAPHIC_LOCATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__geographic_location, &ColladaParserAutoGen15Private::_data__geographic_location, &ColladaParserAutoGen15Private::_end__geographic_location, &ColladaParserAutoGen15Private::_preBegin__geographic_location, &ColladaParserAutoGen15Private::_preEnd__geographic_location, &ColladaParserAutoGen15Private::_freeAttributes__geographic_location);
functionMap[HASH_ELEMENT_ALTITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__altitude, &ColladaParserAutoGen15Private::_data__altitude, &ColladaParserAutoGen15Private::_end__altitude, &ColladaParserAutoGen15Private::_preBegin__altitude, &ColladaParserAutoGen15Private::_preEnd__altitude, &ColladaParserAutoGen15Private::_freeAttributes__altitude);
functionMap[HASH_ELEMENT_LATITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__latitude, &ColladaParserAutoGen15Private::_data__latitude, &ColladaParserAutoGen15Private::_end__latitude, &ColladaParserAutoGen15Private::_preBegin__latitude, &ColladaParserAutoGen15Private::_preEnd__latitude, &ColladaParserAutoGen15Private::_freeAttributes__latitude);
functionMap[HASH_ELEMENT_LONGITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__longitude, &ColladaParserAutoGen15Private::_data__longitude, &ColladaParserAutoGen15Private::_end__longitude, &ColladaParserAutoGen15Private::_preBegin__longitude, &ColladaParserAutoGen15Private::_preEnd__longitude, &ColladaParserAutoGen15Private::_freeAttributes__longitude);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_SETPARAM____FORMULA_SETPARAM_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__setparam____formula_setparam_type, &ColladaParserAutoGen15Private::_data__setparam____formula_setparam_type, &ColladaParserAutoGen15Private::_end__setparam____formula_setparam_type, &ColladaParserAutoGen15Private::_preBegin__setparam____formula_setparam_type, &ColladaParserAutoGen15Private::_preEnd__setparam____formula_setparam_type, &ColladaParserAutoGen15Private::_freeAttributes__setparam____formula_setparam_type);
functionMap[HASH_ELEMENT_SIDREF] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__SIDREF, &ColladaParserAutoGen15Private::_data__SIDREF, &ColladaParserAutoGen15Private::_end__SIDREF, &ColladaParserAutoGen15Private::_preBegin__SIDREF, &ColladaParserAutoGen15Private::_preEnd__SIDREF, &ColladaParserAutoGen15Private::_freeAttributes__SIDREF);
functionMap[HASH_ELEMENT_BOOL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool, &ColladaParserAutoGen15Private::_data__bool, &ColladaParserAutoGen15Private::_end__bool, &ColladaParserAutoGen15Private::_preBegin__bool, &ColladaParserAutoGen15Private::_preEnd__bool, &ColladaParserAutoGen15Private::_freeAttributes__bool);
functionMap[HASH_ELEMENT_CONNECT_PARAM] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__connect_param, &ColladaParserAutoGen15Private::_data__connect_param, &ColladaParserAutoGen15Private::_end__connect_param, &ColladaParserAutoGen15Private::_preBegin__connect_param, &ColladaParserAutoGen15Private::_preEnd__connect_param, &ColladaParserAutoGen15Private::_freeAttributes__connect_param);
functionMap[HASH_ELEMENT_FLOAT____FLOAT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float____float_type, &ColladaParserAutoGen15Private::_data__float____float_type, &ColladaParserAutoGen15Private::_end__float____float_type, &ColladaParserAutoGen15Private::_preBegin__float____float_type, &ColladaParserAutoGen15Private::_preEnd__float____float_type, &ColladaParserAutoGen15Private::_freeAttributes__float____float_type);
functionMap[HASH_ELEMENT_INT____INT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int____int_type, &ColladaParserAutoGen15Private::_data__int____int_type, &ColladaParserAutoGen15Private::_end__int____int_type, &ColladaParserAutoGen15Private::_preBegin__int____int_type, &ColladaParserAutoGen15Private::_preEnd__int____int_type, &ColladaParserAutoGen15Private::_freeAttributes__int____int_type);
mapFilled = true;
}
return functionMap;
}

//---------------------------------------------------------------------
const ColladaParserAutoGen15Private::ElementFunctionMap& FunctionMapFactory::createFunctionMap__library_animations__allChildren() {
static ColladaParserAutoGen15Private::ElementFunctionMap functionMap;
static bool mapFilled = false;
if ( !mapFilled )
{
functionMap[HASH_ELEMENT_COLLADA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__COLLADA, &ColladaParserAutoGen15Private::_data__COLLADA, &ColladaParserAutoGen15Private::_end__COLLADA, &ColladaParserAutoGen15Private::_preBegin__COLLADA, &ColladaParserAutoGen15Private::_preEnd__COLLADA, &ColladaParserAutoGen15Private::_freeAttributes__COLLADA);
functionMap[HASH_ELEMENT_LIBRARY_ANIMATIONS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_animations, &ColladaParserAutoGen15Private::_data__library_animations, &ColladaParserAutoGen15Private::_end__library_animations, &ColladaParserAutoGen15Private::_preBegin__library_animations, &ColladaParserAutoGen15Private::_preEnd__library_animations, &ColladaParserAutoGen15Private::_freeAttributes__library_animations);
functionMap[HASH_ELEMENT_ANIMATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__animation, &ColladaParserAutoGen15Private::_data__animation, &ColladaParserAutoGen15Private::_end__animation, &ColladaParserAutoGen15Private::_preBegin__animation, &ColladaParserAutoGen15Private::_preEnd__animation, &ColladaParserAutoGen15Private::_freeAttributes__animation);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_ANIMATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__animation, &ColladaParserAutoGen15Private::_data__animation, &ColladaParserAutoGen15Private::_end__animation, &ColladaParserAutoGen15Private::_preBegin__animation, &ColladaParserAutoGen15Private::_preEnd__animation, &ColladaParserAutoGen15Private::_freeAttributes__animation);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_CHANNEL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__channel, &ColladaParserAutoGen15Private::_data__channel, &ColladaParserAutoGen15Private::_end__channel, &ColladaParserAutoGen15Private::_preBegin__channel, &ColladaParserAutoGen15Private::_preEnd__channel, &ColladaParserAutoGen15Private::_freeAttributes__channel);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_SAMPLER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler, &ColladaParserAutoGen15Private::_data__sampler, &ColladaParserAutoGen15Private::_end__sampler, &ColladaParserAutoGen15Private::_preBegin__sampler, &ColladaParserAutoGen15Private::_preEnd__sampler, &ColladaParserAutoGen15Private::_freeAttributes__sampler);
functionMap[HASH_ELEMENT_SOURCE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source, &ColladaParserAutoGen15Private::_data__source, &ColladaParserAutoGen15Private::_end__source, &ColladaParserAutoGen15Private::_preBegin__source, &ColladaParserAutoGen15Private::_preEnd__source, &ColladaParserAutoGen15Private::_freeAttributes__source);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_CHANNEL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__channel, &ColladaParserAutoGen15Private::_data__channel, &ColladaParserAutoGen15Private::_end__channel, &ColladaParserAutoGen15Private::_preBegin__channel, &ColladaParserAutoGen15Private::_preEnd__channel, &ColladaParserAutoGen15Private::_freeAttributes__channel);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_SAMPLER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler, &ColladaParserAutoGen15Private::_data__sampler, &ColladaParserAutoGen15Private::_end__sampler, &ColladaParserAutoGen15Private::_preBegin__sampler, &ColladaParserAutoGen15Private::_preEnd__sampler, &ColladaParserAutoGen15Private::_freeAttributes__sampler);
functionMap[HASH_ELEMENT_SOURCE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source, &ColladaParserAutoGen15Private::_data__source, &ColladaParserAutoGen15Private::_end__source, &ColladaParserAutoGen15Private::_preBegin__source, &ColladaParserAutoGen15Private::_preEnd__source, &ColladaParserAutoGen15Private::_freeAttributes__source);
functionMap[HASH_ELEMENT_CONTRIBUTOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__contributor, &ColladaParserAutoGen15Private::_data__contributor, &ColladaParserAutoGen15Private::_end__contributor, &ColladaParserAutoGen15Private::_preBegin__contributor, &ColladaParserAutoGen15Private::_preEnd__contributor, &ColladaParserAutoGen15Private::_freeAttributes__contributor);
functionMap[HASH_ELEMENT_COVERAGE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__coverage, &ColladaParserAutoGen15Private::_data__coverage, &ColladaParserAutoGen15Private::_end__coverage, &ColladaParserAutoGen15Private::_preBegin__coverage, &ColladaParserAutoGen15Private::_preEnd__coverage, &ColladaParserAutoGen15Private::_freeAttributes__coverage);
functionMap[HASH_ELEMENT_CREATED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__created, &ColladaParserAutoGen15Private::_data__created, &ColladaParserAutoGen15Private::_end__created, &ColladaParserAutoGen15Private::_preBegin__created, &ColladaParserAutoGen15Private::_preEnd__created, &ColladaParserAutoGen15Private::_freeAttributes__created);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_AUTHOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author, &ColladaParserAutoGen15Private::_data__author, &ColladaParserAutoGen15Private::_end__author, &ColladaParserAutoGen15Private::_preBegin__author, &ColladaParserAutoGen15Private::_preEnd__author, &ColladaParserAutoGen15Private::_freeAttributes__author);
functionMap[HASH_ELEMENT_AUTHOR_EMAIL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_email, &ColladaParserAutoGen15Private::_data__author_email, &ColladaParserAutoGen15Private::_end__author_email, &ColladaParserAutoGen15Private::_preBegin__author_email, &ColladaParserAutoGen15Private::_preEnd__author_email, &ColladaParserAutoGen15Private::_freeAttributes__author_email);
functionMap[HASH_ELEMENT_AUTHOR_WEBSITE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_website, &ColladaParserAutoGen15Private::_data__author_website, &ColladaParserAutoGen15Private::_end__author_website, &ColladaParserAutoGen15Private::_preBegin__author_website, &ColladaParserAutoGen15Private::_preEnd__author_website, &ColladaParserAutoGen15Private::_freeAttributes__author_website);
functionMap[HASH_ELEMENT_AUTHORING_TOOL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__authoring_tool, &ColladaParserAutoGen15Private::_data__authoring_tool, &ColladaParserAutoGen15Private::_end__authoring_tool, &ColladaParserAutoGen15Private::_preBegin__authoring_tool, &ColladaParserAutoGen15Private::_preEnd__authoring_tool, &ColladaParserAutoGen15Private::_freeAttributes__authoring_tool);
functionMap[HASH_ELEMENT_COMMENTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__comments, &ColladaParserAutoGen15Private::_data__comments, &ColladaParserAutoGen15Private::_end__comments, &ColladaParserAutoGen15Private::_preBegin__comments, &ColladaParserAutoGen15Private::_preEnd__comments, &ColladaParserAutoGen15Private::_freeAttributes__comments);
functionMap[HASH_ELEMENT_COPYRIGHT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__copyright, &ColladaParserAutoGen15Private::_data__copyright, &ColladaParserAutoGen15Private::_end__copyright, &ColladaParserAutoGen15Private::_preBegin__copyright, &ColladaParserAutoGen15Private::_preEnd__copyright, &ColladaParserAutoGen15Private::_freeAttributes__copyright);
functionMap[HASH_ELEMENT_SOURCE_DATA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source_data, &ColladaParserAutoGen15Private::_data__source_data, &ColladaParserAutoGen15Private::_end__source_data, &ColladaParserAutoGen15Private::_preBegin__source_data, &ColladaParserAutoGen15Private::_preEnd__source_data, &ColladaParserAutoGen15Private::_freeAttributes__source_data);
functionMap[HASH_ELEMENT_GEOGRAPHIC_LOCATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__geographic_location, &ColladaParserAutoGen15Private::_data__geographic_location, &ColladaParserAutoGen15Private::_end__geographic_location, &ColladaParserAutoGen15Private::_preBegin__geographic_location, &ColladaParserAutoGen15Private::_preEnd__geographic_location, &ColladaParserAutoGen15Private::_freeAttributes__geographic_location);
functionMap[HASH_ELEMENT_ALTITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__altitude, &ColladaParserAutoGen15Private::_data__altitude, &ColladaParserAutoGen15Private::_end__altitude, &ColladaParserAutoGen15Private::_preBegin__altitude, &ColladaParserAutoGen15Private::_preEnd__altitude, &ColladaParserAutoGen15Private::_freeAttributes__altitude);
functionMap[HASH_ELEMENT_LATITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__latitude, &ColladaParserAutoGen15Private::_data__latitude, &ColladaParserAutoGen15Private::_end__latitude, &ColladaParserAutoGen15Private::_preBegin__latitude, &ColladaParserAutoGen15Private::_preEnd__latitude, &ColladaParserAutoGen15Private::_freeAttributes__latitude);
functionMap[HASH_ELEMENT_LONGITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__longitude, &ColladaParserAutoGen15Private::_data__longitude, &ColladaParserAutoGen15Private::_end__longitude, &ColladaParserAutoGen15Private::_preBegin__longitude, &ColladaParserAutoGen15Private::_preEnd__longitude, &ColladaParserAutoGen15Private::_freeAttributes__longitude);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_INPUT____INPUT_LOCAL_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__input____input_local_type, &ColladaParserAutoGen15Private::_data__input____input_local_type, &ColladaParserAutoGen15Private::_end__input____input_local_type, &ColladaParserAutoGen15Private::_preBegin__input____input_local_type, &ColladaParserAutoGen15Private::_preEnd__input____input_local_type, &ColladaParserAutoGen15Private::_freeAttributes__input____input_local_type);
functionMap[HASH_ELEMENT_IDREF_ARRAY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__IDREF_array, &ColladaParserAutoGen15Private::_data__IDREF_array, &ColladaParserAutoGen15Private::_end__IDREF_array, &ColladaParserAutoGen15Private::_preBegin__IDREF_array, &ColladaParserAutoGen15Private::_preEnd__IDREF_array, &ColladaParserAutoGen15Private::_freeAttributes__IDREF_array);
functionMap[HASH_ELEMENT_NAME_ARRAY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__Name_array, &ColladaParserAutoGen15Private::_data__Name_array, &ColladaParserAutoGen15Private::_end__Name_array, &ColladaParserAutoGen15Private::_preBegin__Name_array, &ColladaParserAutoGen15Private::_preEnd__Name_array, &ColladaParserAutoGen15Private::_freeAttributes__Name_array);
functionMap[HASH_ELEMENT_SIDREF_ARRAY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__SIDREF_array, &ColladaParserAutoGen15Private::_data__SIDREF_array, &ColladaParserAutoGen15Private::_end__SIDREF_array, &ColladaParserAutoGen15Private::_preBegin__SIDREF_array, &ColladaParserAutoGen15Private::_preEnd__SIDREF_array, &ColladaParserAutoGen15Private::_freeAttributes__SIDREF_array);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_BOOL_ARRAY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool_array, &ColladaParserAutoGen15Private::_data__bool_array, &ColladaParserAutoGen15Private::_end__bool_array, &ColladaParserAutoGen15Private::_preBegin__bool_array, &ColladaParserAutoGen15Private::_preEnd__bool_array, &ColladaParserAutoGen15Private::_freeAttributes__bool_array);
functionMap[HASH_ELEMENT_FLOAT_ARRAY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float_array, &ColladaParserAutoGen15Private::_data__float_array, &ColladaParserAutoGen15Private::_end__float_array, &ColladaParserAutoGen15Private::_preBegin__float_array, &ColladaParserAutoGen15Private::_preEnd__float_array, &ColladaParserAutoGen15Private::_freeAttributes__float_array);
functionMap[HASH_ELEMENT_INT_ARRAY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int_array, &ColladaParserAutoGen15Private::_data__int_array, &ColladaParserAutoGen15Private::_end__int_array, &ColladaParserAutoGen15Private::_preBegin__int_array, &ColladaParserAutoGen15Private::_preEnd__int_array, &ColladaParserAutoGen15Private::_freeAttributes__int_array);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_SOURCE_TYPE____TECHNIQUE_COMMON] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source_type____technique_common, &ColladaParserAutoGen15Private::_data__source_type____technique_common, &ColladaParserAutoGen15Private::_end__source_type____technique_common, &ColladaParserAutoGen15Private::_preBegin__source_type____technique_common, &ColladaParserAutoGen15Private::_preEnd__source_type____technique_common, &ColladaParserAutoGen15Private::_freeAttributes__source_type____technique_common);
functionMap[HASH_ELEMENT_TOKEN_ARRAY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__token_array, &ColladaParserAutoGen15Private::_data__token_array, &ColladaParserAutoGen15Private::_end__token_array, &ColladaParserAutoGen15Private::_preBegin__token_array, &ColladaParserAutoGen15Private::_preEnd__token_array, &ColladaParserAutoGen15Private::_freeAttributes__token_array);
functionMap[HASH_ELEMENT_ACCESSOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__accessor, &ColladaParserAutoGen15Private::_data__accessor, &ColladaParserAutoGen15Private::_end__accessor, &ColladaParserAutoGen15Private::_preBegin__accessor, &ColladaParserAutoGen15Private::_preEnd__accessor, &ColladaParserAutoGen15Private::_freeAttributes__accessor);
functionMap[HASH_ELEMENT_PARAM____PARAM_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__param____param_type, &ColladaParserAutoGen15Private::_data__param____param_type, &ColladaParserAutoGen15Private::_end__param____param_type, &ColladaParserAutoGen15Private::_preBegin__param____param_type, &ColladaParserAutoGen15Private::_preEnd__param____param_type, &ColladaParserAutoGen15Private::_freeAttributes__param____param_type);
mapFilled = true;
}
return functionMap;
}

//---------------------------------------------------------------------
const ColladaParserAutoGen15Private::ElementFunctionMap& FunctionMapFactory::createFunctionMap__library_cameras__allChildren() {
static ColladaParserAutoGen15Private::ElementFunctionMap functionMap;
static bool mapFilled = false;
if ( !mapFilled )
{
functionMap[HASH_ELEMENT_COLLADA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__COLLADA, &ColladaParserAutoGen15Private::_data__COLLADA, &ColladaParserAutoGen15Private::_end__COLLADA, &ColladaParserAutoGen15Private::_preBegin__COLLADA, &ColladaParserAutoGen15Private::_preEnd__COLLADA, &ColladaParserAutoGen15Private::_freeAttributes__COLLADA);
functionMap[HASH_ELEMENT_LIBRARY_CAMERAS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_cameras, &ColladaParserAutoGen15Private::_data__library_cameras, &ColladaParserAutoGen15Private::_end__library_cameras, &ColladaParserAutoGen15Private::_preBegin__library_cameras, &ColladaParserAutoGen15Private::_preEnd__library_cameras, &ColladaParserAutoGen15Private::_freeAttributes__library_cameras);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_CAMERA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__camera, &ColladaParserAutoGen15Private::_data__camera, &ColladaParserAutoGen15Private::_end__camera, &ColladaParserAutoGen15Private::_preBegin__camera, &ColladaParserAutoGen15Private::_preEnd__camera, &ColladaParserAutoGen15Private::_freeAttributes__camera);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_CONTRIBUTOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__contributor, &ColladaParserAutoGen15Private::_data__contributor, &ColladaParserAutoGen15Private::_end__contributor, &ColladaParserAutoGen15Private::_preBegin__contributor, &ColladaParserAutoGen15Private::_preEnd__contributor, &ColladaParserAutoGen15Private::_freeAttributes__contributor);
functionMap[HASH_ELEMENT_COVERAGE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__coverage, &ColladaParserAutoGen15Private::_data__coverage, &ColladaParserAutoGen15Private::_end__coverage, &ColladaParserAutoGen15Private::_preBegin__coverage, &ColladaParserAutoGen15Private::_preEnd__coverage, &ColladaParserAutoGen15Private::_freeAttributes__coverage);
functionMap[HASH_ELEMENT_CREATED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__created, &ColladaParserAutoGen15Private::_data__created, &ColladaParserAutoGen15Private::_end__created, &ColladaParserAutoGen15Private::_preBegin__created, &ColladaParserAutoGen15Private::_preEnd__created, &ColladaParserAutoGen15Private::_freeAttributes__created);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_AUTHOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author, &ColladaParserAutoGen15Private::_data__author, &ColladaParserAutoGen15Private::_end__author, &ColladaParserAutoGen15Private::_preBegin__author, &ColladaParserAutoGen15Private::_preEnd__author, &ColladaParserAutoGen15Private::_freeAttributes__author);
functionMap[HASH_ELEMENT_AUTHOR_EMAIL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_email, &ColladaParserAutoGen15Private::_data__author_email, &ColladaParserAutoGen15Private::_end__author_email, &ColladaParserAutoGen15Private::_preBegin__author_email, &ColladaParserAutoGen15Private::_preEnd__author_email, &ColladaParserAutoGen15Private::_freeAttributes__author_email);
functionMap[HASH_ELEMENT_AUTHOR_WEBSITE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_website, &ColladaParserAutoGen15Private::_data__author_website, &ColladaParserAutoGen15Private::_end__author_website, &ColladaParserAutoGen15Private::_preBegin__author_website, &ColladaParserAutoGen15Private::_preEnd__author_website, &ColladaParserAutoGen15Private::_freeAttributes__author_website);
functionMap[HASH_ELEMENT_AUTHORING_TOOL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__authoring_tool, &ColladaParserAutoGen15Private::_data__authoring_tool, &ColladaParserAutoGen15Private::_end__authoring_tool, &ColladaParserAutoGen15Private::_preBegin__authoring_tool, &ColladaParserAutoGen15Private::_preEnd__authoring_tool, &ColladaParserAutoGen15Private::_freeAttributes__authoring_tool);
functionMap[HASH_ELEMENT_COMMENTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__comments, &ColladaParserAutoGen15Private::_data__comments, &ColladaParserAutoGen15Private::_end__comments, &ColladaParserAutoGen15Private::_preBegin__comments, &ColladaParserAutoGen15Private::_preEnd__comments, &ColladaParserAutoGen15Private::_freeAttributes__comments);
functionMap[HASH_ELEMENT_COPYRIGHT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__copyright, &ColladaParserAutoGen15Private::_data__copyright, &ColladaParserAutoGen15Private::_end__copyright, &ColladaParserAutoGen15Private::_preBegin__copyright, &ColladaParserAutoGen15Private::_preEnd__copyright, &ColladaParserAutoGen15Private::_freeAttributes__copyright);
functionMap[HASH_ELEMENT_SOURCE_DATA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source_data, &ColladaParserAutoGen15Private::_data__source_data, &ColladaParserAutoGen15Private::_end__source_data, &ColladaParserAutoGen15Private::_preBegin__source_data, &ColladaParserAutoGen15Private::_preEnd__source_data, &ColladaParserAutoGen15Private::_freeAttributes__source_data);
functionMap[HASH_ELEMENT_GEOGRAPHIC_LOCATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__geographic_location, &ColladaParserAutoGen15Private::_data__geographic_location, &ColladaParserAutoGen15Private::_end__geographic_location, &ColladaParserAutoGen15Private::_preBegin__geographic_location, &ColladaParserAutoGen15Private::_preEnd__geographic_location, &ColladaParserAutoGen15Private::_freeAttributes__geographic_location);
functionMap[HASH_ELEMENT_ALTITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__altitude, &ColladaParserAutoGen15Private::_data__altitude, &ColladaParserAutoGen15Private::_end__altitude, &ColladaParserAutoGen15Private::_preBegin__altitude, &ColladaParserAutoGen15Private::_preEnd__altitude, &ColladaParserAutoGen15Private::_freeAttributes__altitude);
functionMap[HASH_ELEMENT_LATITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__latitude, &ColladaParserAutoGen15Private::_data__latitude, &ColladaParserAutoGen15Private::_end__latitude, &ColladaParserAutoGen15Private::_preBegin__latitude, &ColladaParserAutoGen15Private::_preEnd__latitude, &ColladaParserAutoGen15Private::_freeAttributes__latitude);
functionMap[HASH_ELEMENT_LONGITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__longitude, &ColladaParserAutoGen15Private::_data__longitude, &ColladaParserAutoGen15Private::_end__longitude, &ColladaParserAutoGen15Private::_preBegin__longitude, &ColladaParserAutoGen15Private::_preEnd__longitude, &ColladaParserAutoGen15Private::_freeAttributes__longitude);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_IMAGER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__imager, &ColladaParserAutoGen15Private::_data__imager, &ColladaParserAutoGen15Private::_end__imager, &ColladaParserAutoGen15Private::_preBegin__imager, &ColladaParserAutoGen15Private::_preEnd__imager, &ColladaParserAutoGen15Private::_freeAttributes__imager);
functionMap[HASH_ELEMENT_OPTICS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__optics, &ColladaParserAutoGen15Private::_data__optics, &ColladaParserAutoGen15Private::_end__optics, &ColladaParserAutoGen15Private::_preBegin__optics, &ColladaParserAutoGen15Private::_preEnd__optics, &ColladaParserAutoGen15Private::_freeAttributes__optics);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_OPTICS__TECHNIQUE_COMMON] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__optics__technique_common, &ColladaParserAutoGen15Private::_data__optics__technique_common, &ColladaParserAutoGen15Private::_end__optics__technique_common, &ColladaParserAutoGen15Private::_preBegin__optics__technique_common, &ColladaParserAutoGen15Private::_preEnd__optics__technique_common, &ColladaParserAutoGen15Private::_freeAttributes__optics__technique_common);
functionMap[HASH_ELEMENT_ORTHOGRAPHIC] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__orthographic, &ColladaParserAutoGen15Private::_data__orthographic, &ColladaParserAutoGen15Private::_end__orthographic, &ColladaParserAutoGen15Private::_preBegin__orthographic, &ColladaParserAutoGen15Private::_preEnd__orthographic, &ColladaParserAutoGen15Private::_freeAttributes__orthographic);
functionMap[HASH_ELEMENT_PERSPECTIVE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__perspective, &ColladaParserAutoGen15Private::_data__perspective, &ColladaParserAutoGen15Private::_end__perspective, &ColladaParserAutoGen15Private::_preBegin__perspective, &ColladaParserAutoGen15Private::_preEnd__perspective, &ColladaParserAutoGen15Private::_freeAttributes__perspective);
functionMap[HASH_ELEMENT_ASPECT_RATIO] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__aspect_ratio, &ColladaParserAutoGen15Private::_data__aspect_ratio, &ColladaParserAutoGen15Private::_end__aspect_ratio, &ColladaParserAutoGen15Private::_preBegin__aspect_ratio, &ColladaParserAutoGen15Private::_preEnd__aspect_ratio, &ColladaParserAutoGen15Private::_freeAttributes__aspect_ratio);
functionMap[HASH_ELEMENT_XMAG] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__xmag, &ColladaParserAutoGen15Private::_data__xmag, &ColladaParserAutoGen15Private::_end__xmag, &ColladaParserAutoGen15Private::_preBegin__xmag, &ColladaParserAutoGen15Private::_preEnd__xmag, &ColladaParserAutoGen15Private::_freeAttributes__xmag);
functionMap[HASH_ELEMENT_YMAG] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__ymag, &ColladaParserAutoGen15Private::_data__ymag, &ColladaParserAutoGen15Private::_end__ymag, &ColladaParserAutoGen15Private::_preBegin__ymag, &ColladaParserAutoGen15Private::_preEnd__ymag, &ColladaParserAutoGen15Private::_freeAttributes__ymag);
functionMap[HASH_ELEMENT_ZFAR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__zfar, &ColladaParserAutoGen15Private::_data__zfar, &ColladaParserAutoGen15Private::_end__zfar, &ColladaParserAutoGen15Private::_preBegin__zfar, &ColladaParserAutoGen15Private::_preEnd__zfar, &ColladaParserAutoGen15Private::_freeAttributes__zfar);
functionMap[HASH_ELEMENT_ZNEAR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__znear, &ColladaParserAutoGen15Private::_data__znear, &ColladaParserAutoGen15Private::_end__znear, &ColladaParserAutoGen15Private::_preBegin__znear, &ColladaParserAutoGen15Private::_preEnd__znear, &ColladaParserAutoGen15Private::_freeAttributes__znear);
functionMap[HASH_ELEMENT_ASPECT_RATIO] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__aspect_ratio, &ColladaParserAutoGen15Private::_data__aspect_ratio, &ColladaParserAutoGen15Private::_end__aspect_ratio, &ColladaParserAutoGen15Private::_preBegin__aspect_ratio, &ColladaParserAutoGen15Private::_preEnd__aspect_ratio, &ColladaParserAutoGen15Private::_freeAttributes__aspect_ratio);
functionMap[HASH_ELEMENT_XFOV] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__xfov, &ColladaParserAutoGen15Private::_data__xfov, &ColladaParserAutoGen15Private::_end__xfov, &ColladaParserAutoGen15Private::_preBegin__xfov, &ColladaParserAutoGen15Private::_preEnd__xfov, &ColladaParserAutoGen15Private::_freeAttributes__xfov);
functionMap[HASH_ELEMENT_YFOV] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__yfov, &ColladaParserAutoGen15Private::_data__yfov, &ColladaParserAutoGen15Private::_end__yfov, &ColladaParserAutoGen15Private::_preBegin__yfov, &ColladaParserAutoGen15Private::_preEnd__yfov, &ColladaParserAutoGen15Private::_freeAttributes__yfov);
functionMap[HASH_ELEMENT_ZFAR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__zfar, &ColladaParserAutoGen15Private::_data__zfar, &ColladaParserAutoGen15Private::_end__zfar, &ColladaParserAutoGen15Private::_preBegin__zfar, &ColladaParserAutoGen15Private::_preEnd__zfar, &ColladaParserAutoGen15Private::_freeAttributes__zfar);
functionMap[HASH_ELEMENT_ZNEAR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__znear, &ColladaParserAutoGen15Private::_data__znear, &ColladaParserAutoGen15Private::_end__znear, &ColladaParserAutoGen15Private::_preBegin__znear, &ColladaParserAutoGen15Private::_preEnd__znear, &ColladaParserAutoGen15Private::_freeAttributes__znear);
mapFilled = true;
}
return functionMap;
}

//---------------------------------------------------------------------
const ColladaParserAutoGen15Private::ElementFunctionMap& FunctionMapFactory::createFunctionMap__library_controllers__allChildren() {
static ColladaParserAutoGen15Private::ElementFunctionMap functionMap;
static bool mapFilled = false;
if ( !mapFilled )
{
functionMap[HASH_ELEMENT_COLLADA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__COLLADA, &ColladaParserAutoGen15Private::_data__COLLADA, &ColladaParserAutoGen15Private::_end__COLLADA, &ColladaParserAutoGen15Private::_preBegin__COLLADA, &ColladaParserAutoGen15Private::_preEnd__COLLADA, &ColladaParserAutoGen15Private::_freeAttributes__COLLADA);
functionMap[HASH_ELEMENT_LIBRARY_CONTROLLERS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_controllers, &ColladaParserAutoGen15Private::_data__library_controllers, &ColladaParserAutoGen15Private::_end__library_controllers, &ColladaParserAutoGen15Private::_preBegin__library_controllers, &ColladaParserAutoGen15Private::_preEnd__library_controllers, &ColladaParserAutoGen15Private::_freeAttributes__library_controllers);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_CONTROLLER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__controller, &ColladaParserAutoGen15Private::_data__controller, &ColladaParserAutoGen15Private::_end__controller, &ColladaParserAutoGen15Private::_preBegin__controller, &ColladaParserAutoGen15Private::_preEnd__controller, &ColladaParserAutoGen15Private::_freeAttributes__controller);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_CONTRIBUTOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__contributor, &ColladaParserAutoGen15Private::_data__contributor, &ColladaParserAutoGen15Private::_end__contributor, &ColladaParserAutoGen15Private::_preBegin__contributor, &ColladaParserAutoGen15Private::_preEnd__contributor, &ColladaParserAutoGen15Private::_freeAttributes__contributor);
functionMap[HASH_ELEMENT_COVERAGE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__coverage, &ColladaParserAutoGen15Private::_data__coverage, &ColladaParserAutoGen15Private::_end__coverage, &ColladaParserAutoGen15Private::_preBegin__coverage, &ColladaParserAutoGen15Private::_preEnd__coverage, &ColladaParserAutoGen15Private::_freeAttributes__coverage);
functionMap[HASH_ELEMENT_CREATED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__created, &ColladaParserAutoGen15Private::_data__created, &ColladaParserAutoGen15Private::_end__created, &ColladaParserAutoGen15Private::_preBegin__created, &ColladaParserAutoGen15Private::_preEnd__created, &ColladaParserAutoGen15Private::_freeAttributes__created);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_AUTHOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author, &ColladaParserAutoGen15Private::_data__author, &ColladaParserAutoGen15Private::_end__author, &ColladaParserAutoGen15Private::_preBegin__author, &ColladaParserAutoGen15Private::_preEnd__author, &ColladaParserAutoGen15Private::_freeAttributes__author);
functionMap[HASH_ELEMENT_AUTHOR_EMAIL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_email, &ColladaParserAutoGen15Private::_data__author_email, &ColladaParserAutoGen15Private::_end__author_email, &ColladaParserAutoGen15Private::_preBegin__author_email, &ColladaParserAutoGen15Private::_preEnd__author_email, &ColladaParserAutoGen15Private::_freeAttributes__author_email);
functionMap[HASH_ELEMENT_AUTHOR_WEBSITE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_website, &ColladaParserAutoGen15Private::_data__author_website, &ColladaParserAutoGen15Private::_end__author_website, &ColladaParserAutoGen15Private::_preBegin__author_website, &ColladaParserAutoGen15Private::_preEnd__author_website, &ColladaParserAutoGen15Private::_freeAttributes__author_website);
functionMap[HASH_ELEMENT_AUTHORING_TOOL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__authoring_tool, &ColladaParserAutoGen15Private::_data__authoring_tool, &ColladaParserAutoGen15Private::_end__authoring_tool, &ColladaParserAutoGen15Private::_preBegin__authoring_tool, &ColladaParserAutoGen15Private::_preEnd__authoring_tool, &ColladaParserAutoGen15Private::_freeAttributes__authoring_tool);
functionMap[HASH_ELEMENT_COMMENTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__comments, &ColladaParserAutoGen15Private::_data__comments, &ColladaParserAutoGen15Private::_end__comments, &ColladaParserAutoGen15Private::_preBegin__comments, &ColladaParserAutoGen15Private::_preEnd__comments, &ColladaParserAutoGen15Private::_freeAttributes__comments);
functionMap[HASH_ELEMENT_COPYRIGHT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__copyright, &ColladaParserAutoGen15Private::_data__copyright, &ColladaParserAutoGen15Private::_end__copyright, &ColladaParserAutoGen15Private::_preBegin__copyright, &ColladaParserAutoGen15Private::_preEnd__copyright, &ColladaParserAutoGen15Private::_freeAttributes__copyright);
functionMap[HASH_ELEMENT_SOURCE_DATA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source_data, &ColladaParserAutoGen15Private::_data__source_data, &ColladaParserAutoGen15Private::_end__source_data, &ColladaParserAutoGen15Private::_preBegin__source_data, &ColladaParserAutoGen15Private::_preEnd__source_data, &ColladaParserAutoGen15Private::_freeAttributes__source_data);
functionMap[HASH_ELEMENT_GEOGRAPHIC_LOCATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__geographic_location, &ColladaParserAutoGen15Private::_data__geographic_location, &ColladaParserAutoGen15Private::_end__geographic_location, &ColladaParserAutoGen15Private::_preBegin__geographic_location, &ColladaParserAutoGen15Private::_preEnd__geographic_location, &ColladaParserAutoGen15Private::_freeAttributes__geographic_location);
functionMap[HASH_ELEMENT_ALTITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__altitude, &ColladaParserAutoGen15Private::_data__altitude, &ColladaParserAutoGen15Private::_end__altitude, &ColladaParserAutoGen15Private::_preBegin__altitude, &ColladaParserAutoGen15Private::_preEnd__altitude, &ColladaParserAutoGen15Private::_freeAttributes__altitude);
functionMap[HASH_ELEMENT_LATITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__latitude, &ColladaParserAutoGen15Private::_data__latitude, &ColladaParserAutoGen15Private::_end__latitude, &ColladaParserAutoGen15Private::_preBegin__latitude, &ColladaParserAutoGen15Private::_preEnd__latitude, &ColladaParserAutoGen15Private::_freeAttributes__latitude);
functionMap[HASH_ELEMENT_LONGITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__longitude, &ColladaParserAutoGen15Private::_data__longitude, &ColladaParserAutoGen15Private::_end__longitude, &ColladaParserAutoGen15Private::_preBegin__longitude, &ColladaParserAutoGen15Private::_preEnd__longitude, &ColladaParserAutoGen15Private::_freeAttributes__longitude);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_MORPH] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__morph, &ColladaParserAutoGen15Private::_data__morph, &ColladaParserAutoGen15Private::_end__morph, &ColladaParserAutoGen15Private::_preBegin__morph, &ColladaParserAutoGen15Private::_preEnd__morph, &ColladaParserAutoGen15Private::_freeAttributes__morph);
functionMap[HASH_ELEMENT_SKIN] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__skin, &ColladaParserAutoGen15Private::_data__skin, &ColladaParserAutoGen15Private::_end__skin, &ColladaParserAutoGen15Private::_preBegin__skin, &ColladaParserAutoGen15Private::_preEnd__skin, &ColladaParserAutoGen15Private::_freeAttributes__skin);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_SOURCE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source, &ColladaParserAutoGen15Private::_data__source, &ColladaParserAutoGen15Private::_end__source, &ColladaParserAutoGen15Private::_preBegin__source, &ColladaParserAutoGen15Private::_preEnd__source, &ColladaParserAutoGen15Private::_freeAttributes__source);
functionMap[HASH_ELEMENT_TARGETS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__targets, &ColladaParserAutoGen15Private::_data__targets, &ColladaParserAutoGen15Private::_end__targets, &ColladaParserAutoGen15Private::_preBegin__targets, &ColladaParserAutoGen15Private::_preEnd__targets, &ColladaParserAutoGen15Private::_freeAttributes__targets);
functionMap[HASH_ELEMENT_IDREF_ARRAY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__IDREF_array, &ColladaParserAutoGen15Private::_data__IDREF_array, &ColladaParserAutoGen15Private::_end__IDREF_array, &ColladaParserAutoGen15Private::_preBegin__IDREF_array, &ColladaParserAutoGen15Private::_preEnd__IDREF_array, &ColladaParserAutoGen15Private::_freeAttributes__IDREF_array);
functionMap[HASH_ELEMENT_NAME_ARRAY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__Name_array, &ColladaParserAutoGen15Private::_data__Name_array, &ColladaParserAutoGen15Private::_end__Name_array, &ColladaParserAutoGen15Private::_preBegin__Name_array, &ColladaParserAutoGen15Private::_preEnd__Name_array, &ColladaParserAutoGen15Private::_freeAttributes__Name_array);
functionMap[HASH_ELEMENT_SIDREF_ARRAY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__SIDREF_array, &ColladaParserAutoGen15Private::_data__SIDREF_array, &ColladaParserAutoGen15Private::_end__SIDREF_array, &ColladaParserAutoGen15Private::_preBegin__SIDREF_array, &ColladaParserAutoGen15Private::_preEnd__SIDREF_array, &ColladaParserAutoGen15Private::_freeAttributes__SIDREF_array);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_BOOL_ARRAY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool_array, &ColladaParserAutoGen15Private::_data__bool_array, &ColladaParserAutoGen15Private::_end__bool_array, &ColladaParserAutoGen15Private::_preBegin__bool_array, &ColladaParserAutoGen15Private::_preEnd__bool_array, &ColladaParserAutoGen15Private::_freeAttributes__bool_array);
functionMap[HASH_ELEMENT_FLOAT_ARRAY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float_array, &ColladaParserAutoGen15Private::_data__float_array, &ColladaParserAutoGen15Private::_end__float_array, &ColladaParserAutoGen15Private::_preBegin__float_array, &ColladaParserAutoGen15Private::_preEnd__float_array, &ColladaParserAutoGen15Private::_freeAttributes__float_array);
functionMap[HASH_ELEMENT_INT_ARRAY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int_array, &ColladaParserAutoGen15Private::_data__int_array, &ColladaParserAutoGen15Private::_end__int_array, &ColladaParserAutoGen15Private::_preBegin__int_array, &ColladaParserAutoGen15Private::_preEnd__int_array, &ColladaParserAutoGen15Private::_freeAttributes__int_array);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_SOURCE_TYPE____TECHNIQUE_COMMON] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source_type____technique_common, &ColladaParserAutoGen15Private::_data__source_type____technique_common, &ColladaParserAutoGen15Private::_end__source_type____technique_common, &ColladaParserAutoGen15Private::_preBegin__source_type____technique_common, &ColladaParserAutoGen15Private::_preEnd__source_type____technique_common, &ColladaParserAutoGen15Private::_freeAttributes__source_type____technique_common);
functionMap[HASH_ELEMENT_TOKEN_ARRAY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__token_array, &ColladaParserAutoGen15Private::_data__token_array, &ColladaParserAutoGen15Private::_end__token_array, &ColladaParserAutoGen15Private::_preBegin__token_array, &ColladaParserAutoGen15Private::_preEnd__token_array, &ColladaParserAutoGen15Private::_freeAttributes__token_array);
functionMap[HASH_ELEMENT_ACCESSOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__accessor, &ColladaParserAutoGen15Private::_data__accessor, &ColladaParserAutoGen15Private::_end__accessor, &ColladaParserAutoGen15Private::_preBegin__accessor, &ColladaParserAutoGen15Private::_preEnd__accessor, &ColladaParserAutoGen15Private::_freeAttributes__accessor);
functionMap[HASH_ELEMENT_PARAM____PARAM_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__param____param_type, &ColladaParserAutoGen15Private::_data__param____param_type, &ColladaParserAutoGen15Private::_end__param____param_type, &ColladaParserAutoGen15Private::_preBegin__param____param_type, &ColladaParserAutoGen15Private::_preEnd__param____param_type, &ColladaParserAutoGen15Private::_freeAttributes__param____param_type);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INPUT____INPUT_LOCAL_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__input____input_local_type, &ColladaParserAutoGen15Private::_data__input____input_local_type, &ColladaParserAutoGen15Private::_end__input____input_local_type, &ColladaParserAutoGen15Private::_preBegin__input____input_local_type, &ColladaParserAutoGen15Private::_preEnd__input____input_local_type, &ColladaParserAutoGen15Private::_freeAttributes__input____input_local_type);
functionMap[HASH_ELEMENT_BIND_SHAPE_MATRIX] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bind_shape_matrix, &ColladaParserAutoGen15Private::_data__bind_shape_matrix, &ColladaParserAutoGen15Private::_end__bind_shape_matrix, &ColladaParserAutoGen15Private::_preBegin__bind_shape_matrix, &ColladaParserAutoGen15Private::_preEnd__bind_shape_matrix, &ColladaParserAutoGen15Private::_freeAttributes__bind_shape_matrix);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_JOINTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__joints, &ColladaParserAutoGen15Private::_data__joints, &ColladaParserAutoGen15Private::_end__joints, &ColladaParserAutoGen15Private::_preBegin__joints, &ColladaParserAutoGen15Private::_preEnd__joints, &ColladaParserAutoGen15Private::_freeAttributes__joints);
functionMap[HASH_ELEMENT_SOURCE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source, &ColladaParserAutoGen15Private::_data__source, &ColladaParserAutoGen15Private::_end__source, &ColladaParserAutoGen15Private::_preBegin__source, &ColladaParserAutoGen15Private::_preEnd__source, &ColladaParserAutoGen15Private::_freeAttributes__source);
functionMap[HASH_ELEMENT_VERTEX_WEIGHTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__vertex_weights, &ColladaParserAutoGen15Private::_data__vertex_weights, &ColladaParserAutoGen15Private::_end__vertex_weights, &ColladaParserAutoGen15Private::_preBegin__vertex_weights, &ColladaParserAutoGen15Private::_preEnd__vertex_weights, &ColladaParserAutoGen15Private::_freeAttributes__vertex_weights);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INPUT____INPUT_LOCAL_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__input____input_local_type, &ColladaParserAutoGen15Private::_data__input____input_local_type, &ColladaParserAutoGen15Private::_end__input____input_local_type, &ColladaParserAutoGen15Private::_preBegin__input____input_local_type, &ColladaParserAutoGen15Private::_preEnd__input____input_local_type, &ColladaParserAutoGen15Private::_freeAttributes__input____input_local_type);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INPUT____INPUT_LOCAL_OFFSET_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__input____input_local_offset_type, &ColladaParserAutoGen15Private::_data__input____input_local_offset_type, &ColladaParserAutoGen15Private::_end__input____input_local_offset_type, &ColladaParserAutoGen15Private::_preBegin__input____input_local_offset_type, &ColladaParserAutoGen15Private::_preEnd__input____input_local_offset_type, &ColladaParserAutoGen15Private::_freeAttributes__input____input_local_offset_type);
functionMap[HASH_ELEMENT_V] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__v, &ColladaParserAutoGen15Private::_data__v, &ColladaParserAutoGen15Private::_end__v, &ColladaParserAutoGen15Private::_preBegin__v, &ColladaParserAutoGen15Private::_preEnd__v, &ColladaParserAutoGen15Private::_freeAttributes__v);
functionMap[HASH_ELEMENT_VCOUNT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__vcount, &ColladaParserAutoGen15Private::_data__vcount, &ColladaParserAutoGen15Private::_end__vcount, &ColladaParserAutoGen15Private::_preBegin__vcount, &ColladaParserAutoGen15Private::_preEnd__vcount, &ColladaParserAutoGen15Private::_freeAttributes__vcount);
mapFilled = true;
}
return functionMap;
}

//---------------------------------------------------------------------
const ColladaParserAutoGen15Private::ElementFunctionMap& FunctionMapFactory::createFunctionMap__library_effects__allChildren() {
static ColladaParserAutoGen15Private::ElementFunctionMap functionMap;
static bool mapFilled = false;
if ( !mapFilled )
{
functionMap[HASH_ELEMENT_COLLADA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__COLLADA, &ColladaParserAutoGen15Private::_data__COLLADA, &ColladaParserAutoGen15Private::_end__COLLADA, &ColladaParserAutoGen15Private::_preBegin__COLLADA, &ColladaParserAutoGen15Private::_preEnd__COLLADA, &ColladaParserAutoGen15Private::_freeAttributes__COLLADA);
functionMap[HASH_ELEMENT_LIBRARY_EFFECTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_effects, &ColladaParserAutoGen15Private::_data__library_effects, &ColladaParserAutoGen15Private::_end__library_effects, &ColladaParserAutoGen15Private::_preBegin__library_effects, &ColladaParserAutoGen15Private::_preEnd__library_effects, &ColladaParserAutoGen15Private::_freeAttributes__library_effects);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EFFECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__effect, &ColladaParserAutoGen15Private::_data__effect, &ColladaParserAutoGen15Private::_end__effect, &ColladaParserAutoGen15Private::_preBegin__effect, &ColladaParserAutoGen15Private::_preEnd__effect, &ColladaParserAutoGen15Private::_freeAttributes__effect);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_CONTRIBUTOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__contributor, &ColladaParserAutoGen15Private::_data__contributor, &ColladaParserAutoGen15Private::_end__contributor, &ColladaParserAutoGen15Private::_preBegin__contributor, &ColladaParserAutoGen15Private::_preEnd__contributor, &ColladaParserAutoGen15Private::_freeAttributes__contributor);
functionMap[HASH_ELEMENT_COVERAGE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__coverage, &ColladaParserAutoGen15Private::_data__coverage, &ColladaParserAutoGen15Private::_end__coverage, &ColladaParserAutoGen15Private::_preBegin__coverage, &ColladaParserAutoGen15Private::_preEnd__coverage, &ColladaParserAutoGen15Private::_freeAttributes__coverage);
functionMap[HASH_ELEMENT_CREATED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__created, &ColladaParserAutoGen15Private::_data__created, &ColladaParserAutoGen15Private::_end__created, &ColladaParserAutoGen15Private::_preBegin__created, &ColladaParserAutoGen15Private::_preEnd__created, &ColladaParserAutoGen15Private::_freeAttributes__created);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_AUTHOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author, &ColladaParserAutoGen15Private::_data__author, &ColladaParserAutoGen15Private::_end__author, &ColladaParserAutoGen15Private::_preBegin__author, &ColladaParserAutoGen15Private::_preEnd__author, &ColladaParserAutoGen15Private::_freeAttributes__author);
functionMap[HASH_ELEMENT_AUTHOR_EMAIL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_email, &ColladaParserAutoGen15Private::_data__author_email, &ColladaParserAutoGen15Private::_end__author_email, &ColladaParserAutoGen15Private::_preBegin__author_email, &ColladaParserAutoGen15Private::_preEnd__author_email, &ColladaParserAutoGen15Private::_freeAttributes__author_email);
functionMap[HASH_ELEMENT_AUTHOR_WEBSITE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_website, &ColladaParserAutoGen15Private::_data__author_website, &ColladaParserAutoGen15Private::_end__author_website, &ColladaParserAutoGen15Private::_preBegin__author_website, &ColladaParserAutoGen15Private::_preEnd__author_website, &ColladaParserAutoGen15Private::_freeAttributes__author_website);
functionMap[HASH_ELEMENT_AUTHORING_TOOL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__authoring_tool, &ColladaParserAutoGen15Private::_data__authoring_tool, &ColladaParserAutoGen15Private::_end__authoring_tool, &ColladaParserAutoGen15Private::_preBegin__authoring_tool, &ColladaParserAutoGen15Private::_preEnd__authoring_tool, &ColladaParserAutoGen15Private::_freeAttributes__authoring_tool);
functionMap[HASH_ELEMENT_COMMENTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__comments, &ColladaParserAutoGen15Private::_data__comments, &ColladaParserAutoGen15Private::_end__comments, &ColladaParserAutoGen15Private::_preBegin__comments, &ColladaParserAutoGen15Private::_preEnd__comments, &ColladaParserAutoGen15Private::_freeAttributes__comments);
functionMap[HASH_ELEMENT_COPYRIGHT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__copyright, &ColladaParserAutoGen15Private::_data__copyright, &ColladaParserAutoGen15Private::_end__copyright, &ColladaParserAutoGen15Private::_preBegin__copyright, &ColladaParserAutoGen15Private::_preEnd__copyright, &ColladaParserAutoGen15Private::_freeAttributes__copyright);
functionMap[HASH_ELEMENT_SOURCE_DATA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source_data, &ColladaParserAutoGen15Private::_data__source_data, &ColladaParserAutoGen15Private::_end__source_data, &ColladaParserAutoGen15Private::_preBegin__source_data, &ColladaParserAutoGen15Private::_preEnd__source_data, &ColladaParserAutoGen15Private::_freeAttributes__source_data);
functionMap[HASH_ELEMENT_GEOGRAPHIC_LOCATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__geographic_location, &ColladaParserAutoGen15Private::_data__geographic_location, &ColladaParserAutoGen15Private::_end__geographic_location, &ColladaParserAutoGen15Private::_preBegin__geographic_location, &ColladaParserAutoGen15Private::_preEnd__geographic_location, &ColladaParserAutoGen15Private::_freeAttributes__geographic_location);
functionMap[HASH_ELEMENT_ALTITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__altitude, &ColladaParserAutoGen15Private::_data__altitude, &ColladaParserAutoGen15Private::_end__altitude, &ColladaParserAutoGen15Private::_preBegin__altitude, &ColladaParserAutoGen15Private::_preEnd__altitude, &ColladaParserAutoGen15Private::_freeAttributes__altitude);
functionMap[HASH_ELEMENT_LATITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__latitude, &ColladaParserAutoGen15Private::_data__latitude, &ColladaParserAutoGen15Private::_end__latitude, &ColladaParserAutoGen15Private::_preBegin__latitude, &ColladaParserAutoGen15Private::_preEnd__latitude, &ColladaParserAutoGen15Private::_freeAttributes__latitude);
functionMap[HASH_ELEMENT_LONGITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__longitude, &ColladaParserAutoGen15Private::_data__longitude, &ColladaParserAutoGen15Private::_end__longitude, &ColladaParserAutoGen15Private::_preBegin__longitude, &ColladaParserAutoGen15Private::_preEnd__longitude, &ColladaParserAutoGen15Private::_freeAttributes__longitude);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_ANNOTATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__annotate, &ColladaParserAutoGen15Private::_data__annotate, &ColladaParserAutoGen15Private::_end__annotate, &ColladaParserAutoGen15Private::_preBegin__annotate, &ColladaParserAutoGen15Private::_preEnd__annotate, &ColladaParserAutoGen15Private::_freeAttributes__annotate);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_NEWPARAM____FX_NEWPARAM_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__newparam____fx_newparam_type, &ColladaParserAutoGen15Private::_data__newparam____fx_newparam_type, &ColladaParserAutoGen15Private::_end__newparam____fx_newparam_type, &ColladaParserAutoGen15Private::_preBegin__newparam____fx_newparam_type, &ColladaParserAutoGen15Private::_preEnd__newparam____fx_newparam_type, &ColladaParserAutoGen15Private::_freeAttributes__newparam____fx_newparam_type);
functionMap[HASH_ELEMENT_PROFILE_BRIDGE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_BRIDGE, &ColladaParserAutoGen15Private::_data__profile_BRIDGE, &ColladaParserAutoGen15Private::_end__profile_BRIDGE, &ColladaParserAutoGen15Private::_preBegin__profile_BRIDGE, &ColladaParserAutoGen15Private::_preEnd__profile_BRIDGE, &ColladaParserAutoGen15Private::_freeAttributes__profile_BRIDGE);
functionMap[HASH_ELEMENT_PROFILE_CG] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_CG, &ColladaParserAutoGen15Private::_data__profile_CG, &ColladaParserAutoGen15Private::_end__profile_CG, &ColladaParserAutoGen15Private::_preBegin__profile_CG, &ColladaParserAutoGen15Private::_preEnd__profile_CG, &ColladaParserAutoGen15Private::_freeAttributes__profile_CG);
functionMap[HASH_ELEMENT_PROFILE_COMMON] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_COMMON, &ColladaParserAutoGen15Private::_data__profile_COMMON, &ColladaParserAutoGen15Private::_end__profile_COMMON, &ColladaParserAutoGen15Private::_preBegin__profile_COMMON, &ColladaParserAutoGen15Private::_preEnd__profile_COMMON, &ColladaParserAutoGen15Private::_freeAttributes__profile_COMMON);
functionMap[HASH_ELEMENT_PROFILE_GLES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES, &ColladaParserAutoGen15Private::_data__profile_GLES, &ColladaParserAutoGen15Private::_end__profile_GLES, &ColladaParserAutoGen15Private::_preBegin__profile_GLES, &ColladaParserAutoGen15Private::_preEnd__profile_GLES, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES);
functionMap[HASH_ELEMENT_PROFILE_GLES2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2, &ColladaParserAutoGen15Private::_data__profile_GLES2, &ColladaParserAutoGen15Private::_end__profile_GLES2, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2);
functionMap[HASH_ELEMENT_PROFILE_GLSL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL, &ColladaParserAutoGen15Private::_data__profile_GLSL, &ColladaParserAutoGen15Private::_end__profile_GLSL, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL);
functionMap[HASH_ELEMENT_BOOL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool, &ColladaParserAutoGen15Private::_data__bool, &ColladaParserAutoGen15Private::_end__bool, &ColladaParserAutoGen15Private::_preBegin__bool, &ColladaParserAutoGen15Private::_preEnd__bool, &ColladaParserAutoGen15Private::_freeAttributes__bool);
functionMap[HASH_ELEMENT_BOOL2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool2, &ColladaParserAutoGen15Private::_data__bool2, &ColladaParserAutoGen15Private::_end__bool2, &ColladaParserAutoGen15Private::_preBegin__bool2, &ColladaParserAutoGen15Private::_preEnd__bool2, &ColladaParserAutoGen15Private::_freeAttributes__bool2);
functionMap[HASH_ELEMENT_BOOL3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool3, &ColladaParserAutoGen15Private::_data__bool3, &ColladaParserAutoGen15Private::_end__bool3, &ColladaParserAutoGen15Private::_preBegin__bool3, &ColladaParserAutoGen15Private::_preEnd__bool3, &ColladaParserAutoGen15Private::_freeAttributes__bool3);
functionMap[HASH_ELEMENT_BOOL4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool4, &ColladaParserAutoGen15Private::_data__bool4, &ColladaParserAutoGen15Private::_end__bool4, &ColladaParserAutoGen15Private::_preBegin__bool4, &ColladaParserAutoGen15Private::_preEnd__bool4, &ColladaParserAutoGen15Private::_freeAttributes__bool4);
functionMap[HASH_ELEMENT_FLOAT____FLOAT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float____float_type, &ColladaParserAutoGen15Private::_data__float____float_type, &ColladaParserAutoGen15Private::_end__float____float_type, &ColladaParserAutoGen15Private::_preBegin__float____float_type, &ColladaParserAutoGen15Private::_preEnd__float____float_type, &ColladaParserAutoGen15Private::_freeAttributes__float____float_type);
functionMap[HASH_ELEMENT_FLOAT2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2, &ColladaParserAutoGen15Private::_data__float2, &ColladaParserAutoGen15Private::_end__float2, &ColladaParserAutoGen15Private::_preBegin__float2, &ColladaParserAutoGen15Private::_preEnd__float2, &ColladaParserAutoGen15Private::_freeAttributes__float2);
functionMap[HASH_ELEMENT_FLOAT2X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2x2, &ColladaParserAutoGen15Private::_data__float2x2, &ColladaParserAutoGen15Private::_end__float2x2, &ColladaParserAutoGen15Private::_preBegin__float2x2, &ColladaParserAutoGen15Private::_preEnd__float2x2, &ColladaParserAutoGen15Private::_freeAttributes__float2x2);
functionMap[HASH_ELEMENT_FLOAT3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3, &ColladaParserAutoGen15Private::_data__float3, &ColladaParserAutoGen15Private::_end__float3, &ColladaParserAutoGen15Private::_preBegin__float3, &ColladaParserAutoGen15Private::_preEnd__float3, &ColladaParserAutoGen15Private::_freeAttributes__float3);
functionMap[HASH_ELEMENT_FLOAT3X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3x3, &ColladaParserAutoGen15Private::_data__float3x3, &ColladaParserAutoGen15Private::_end__float3x3, &ColladaParserAutoGen15Private::_preBegin__float3x3, &ColladaParserAutoGen15Private::_preEnd__float3x3, &ColladaParserAutoGen15Private::_freeAttributes__float3x3);
functionMap[HASH_ELEMENT_FLOAT4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4, &ColladaParserAutoGen15Private::_data__float4, &ColladaParserAutoGen15Private::_end__float4, &ColladaParserAutoGen15Private::_preBegin__float4, &ColladaParserAutoGen15Private::_preEnd__float4, &ColladaParserAutoGen15Private::_freeAttributes__float4);
functionMap[HASH_ELEMENT_FLOAT4X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4x4, &ColladaParserAutoGen15Private::_data__float4x4, &ColladaParserAutoGen15Private::_end__float4x4, &ColladaParserAutoGen15Private::_preBegin__float4x4, &ColladaParserAutoGen15Private::_preEnd__float4x4, &ColladaParserAutoGen15Private::_freeAttributes__float4x4);
functionMap[HASH_ELEMENT_INT____INT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int____int_type, &ColladaParserAutoGen15Private::_data__int____int_type, &ColladaParserAutoGen15Private::_end__int____int_type, &ColladaParserAutoGen15Private::_preBegin__int____int_type, &ColladaParserAutoGen15Private::_preEnd__int____int_type, &ColladaParserAutoGen15Private::_freeAttributes__int____int_type);
functionMap[HASH_ELEMENT_INT2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int2, &ColladaParserAutoGen15Private::_data__int2, &ColladaParserAutoGen15Private::_end__int2, &ColladaParserAutoGen15Private::_preBegin__int2, &ColladaParserAutoGen15Private::_preEnd__int2, &ColladaParserAutoGen15Private::_freeAttributes__int2);
functionMap[HASH_ELEMENT_INT3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int3, &ColladaParserAutoGen15Private::_data__int3, &ColladaParserAutoGen15Private::_end__int3, &ColladaParserAutoGen15Private::_preBegin__int3, &ColladaParserAutoGen15Private::_preEnd__int3, &ColladaParserAutoGen15Private::_freeAttributes__int3);
functionMap[HASH_ELEMENT_INT4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int4, &ColladaParserAutoGen15Private::_data__int4, &ColladaParserAutoGen15Private::_end__int4, &ColladaParserAutoGen15Private::_preBegin__int4, &ColladaParserAutoGen15Private::_preEnd__int4, &ColladaParserAutoGen15Private::_freeAttributes__int4);
functionMap[HASH_ELEMENT_STRING] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__string, &ColladaParserAutoGen15Private::_data__string, &ColladaParserAutoGen15Private::_end__string, &ColladaParserAutoGen15Private::_preBegin__string, &ColladaParserAutoGen15Private::_preEnd__string, &ColladaParserAutoGen15Private::_freeAttributes__string);
functionMap[HASH_ELEMENT_ANNOTATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__annotate, &ColladaParserAutoGen15Private::_data__annotate, &ColladaParserAutoGen15Private::_end__annotate, &ColladaParserAutoGen15Private::_preBegin__annotate, &ColladaParserAutoGen15Private::_preEnd__annotate, &ColladaParserAutoGen15Private::_freeAttributes__annotate);
functionMap[HASH_ELEMENT_BOOL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool, &ColladaParserAutoGen15Private::_data__bool, &ColladaParserAutoGen15Private::_end__bool, &ColladaParserAutoGen15Private::_preBegin__bool, &ColladaParserAutoGen15Private::_preEnd__bool, &ColladaParserAutoGen15Private::_freeAttributes__bool);
functionMap[HASH_ELEMENT_BOOL2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool2, &ColladaParserAutoGen15Private::_data__bool2, &ColladaParserAutoGen15Private::_end__bool2, &ColladaParserAutoGen15Private::_preBegin__bool2, &ColladaParserAutoGen15Private::_preEnd__bool2, &ColladaParserAutoGen15Private::_freeAttributes__bool2);
functionMap[HASH_ELEMENT_BOOL3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool3, &ColladaParserAutoGen15Private::_data__bool3, &ColladaParserAutoGen15Private::_end__bool3, &ColladaParserAutoGen15Private::_preBegin__bool3, &ColladaParserAutoGen15Private::_preEnd__bool3, &ColladaParserAutoGen15Private::_freeAttributes__bool3);
functionMap[HASH_ELEMENT_BOOL4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool4, &ColladaParserAutoGen15Private::_data__bool4, &ColladaParserAutoGen15Private::_end__bool4, &ColladaParserAutoGen15Private::_preBegin__bool4, &ColladaParserAutoGen15Private::_preEnd__bool4, &ColladaParserAutoGen15Private::_freeAttributes__bool4);
functionMap[HASH_ELEMENT_ENUM____STRING] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__enum____string, &ColladaParserAutoGen15Private::_data__enum____string, &ColladaParserAutoGen15Private::_end__enum____string, &ColladaParserAutoGen15Private::_preBegin__enum____string, &ColladaParserAutoGen15Private::_preEnd__enum____string, &ColladaParserAutoGen15Private::_freeAttributes__enum____string);
functionMap[HASH_ELEMENT_FLOAT____FLOAT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float____float_type, &ColladaParserAutoGen15Private::_data__float____float_type, &ColladaParserAutoGen15Private::_end__float____float_type, &ColladaParserAutoGen15Private::_preBegin__float____float_type, &ColladaParserAutoGen15Private::_preEnd__float____float_type, &ColladaParserAutoGen15Private::_freeAttributes__float____float_type);
functionMap[HASH_ELEMENT_FLOAT2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2, &ColladaParserAutoGen15Private::_data__float2, &ColladaParserAutoGen15Private::_end__float2, &ColladaParserAutoGen15Private::_preBegin__float2, &ColladaParserAutoGen15Private::_preEnd__float2, &ColladaParserAutoGen15Private::_freeAttributes__float2);
functionMap[HASH_ELEMENT_FLOAT2X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2x1, &ColladaParserAutoGen15Private::_data__float2x1, &ColladaParserAutoGen15Private::_end__float2x1, &ColladaParserAutoGen15Private::_preBegin__float2x1, &ColladaParserAutoGen15Private::_preEnd__float2x1, &ColladaParserAutoGen15Private::_freeAttributes__float2x1);
functionMap[HASH_ELEMENT_FLOAT2X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2x2, &ColladaParserAutoGen15Private::_data__float2x2, &ColladaParserAutoGen15Private::_end__float2x2, &ColladaParserAutoGen15Private::_preBegin__float2x2, &ColladaParserAutoGen15Private::_preEnd__float2x2, &ColladaParserAutoGen15Private::_freeAttributes__float2x2);
functionMap[HASH_ELEMENT_FLOAT2X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2x3, &ColladaParserAutoGen15Private::_data__float2x3, &ColladaParserAutoGen15Private::_end__float2x3, &ColladaParserAutoGen15Private::_preBegin__float2x3, &ColladaParserAutoGen15Private::_preEnd__float2x3, &ColladaParserAutoGen15Private::_freeAttributes__float2x3);
functionMap[HASH_ELEMENT_FLOAT2X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2x4, &ColladaParserAutoGen15Private::_data__float2x4, &ColladaParserAutoGen15Private::_end__float2x4, &ColladaParserAutoGen15Private::_preBegin__float2x4, &ColladaParserAutoGen15Private::_preEnd__float2x4, &ColladaParserAutoGen15Private::_freeAttributes__float2x4);
functionMap[HASH_ELEMENT_FLOAT3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3, &ColladaParserAutoGen15Private::_data__float3, &ColladaParserAutoGen15Private::_end__float3, &ColladaParserAutoGen15Private::_preBegin__float3, &ColladaParserAutoGen15Private::_preEnd__float3, &ColladaParserAutoGen15Private::_freeAttributes__float3);
functionMap[HASH_ELEMENT_FLOAT3X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3x1, &ColladaParserAutoGen15Private::_data__float3x1, &ColladaParserAutoGen15Private::_end__float3x1, &ColladaParserAutoGen15Private::_preBegin__float3x1, &ColladaParserAutoGen15Private::_preEnd__float3x1, &ColladaParserAutoGen15Private::_freeAttributes__float3x1);
functionMap[HASH_ELEMENT_FLOAT3X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3x2, &ColladaParserAutoGen15Private::_data__float3x2, &ColladaParserAutoGen15Private::_end__float3x2, &ColladaParserAutoGen15Private::_preBegin__float3x2, &ColladaParserAutoGen15Private::_preEnd__float3x2, &ColladaParserAutoGen15Private::_freeAttributes__float3x2);
functionMap[HASH_ELEMENT_FLOAT3X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3x3, &ColladaParserAutoGen15Private::_data__float3x3, &ColladaParserAutoGen15Private::_end__float3x3, &ColladaParserAutoGen15Private::_preBegin__float3x3, &ColladaParserAutoGen15Private::_preEnd__float3x3, &ColladaParserAutoGen15Private::_freeAttributes__float3x3);
functionMap[HASH_ELEMENT_FLOAT3X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3x4, &ColladaParserAutoGen15Private::_data__float3x4, &ColladaParserAutoGen15Private::_end__float3x4, &ColladaParserAutoGen15Private::_preBegin__float3x4, &ColladaParserAutoGen15Private::_preEnd__float3x4, &ColladaParserAutoGen15Private::_freeAttributes__float3x4);
functionMap[HASH_ELEMENT_FLOAT4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4, &ColladaParserAutoGen15Private::_data__float4, &ColladaParserAutoGen15Private::_end__float4, &ColladaParserAutoGen15Private::_preBegin__float4, &ColladaParserAutoGen15Private::_preEnd__float4, &ColladaParserAutoGen15Private::_freeAttributes__float4);
functionMap[HASH_ELEMENT_FLOAT4X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4x1, &ColladaParserAutoGen15Private::_data__float4x1, &ColladaParserAutoGen15Private::_end__float4x1, &ColladaParserAutoGen15Private::_preBegin__float4x1, &ColladaParserAutoGen15Private::_preEnd__float4x1, &ColladaParserAutoGen15Private::_freeAttributes__float4x1);
functionMap[HASH_ELEMENT_FLOAT4X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4x2, &ColladaParserAutoGen15Private::_data__float4x2, &ColladaParserAutoGen15Private::_end__float4x2, &ColladaParserAutoGen15Private::_preBegin__float4x2, &ColladaParserAutoGen15Private::_preEnd__float4x2, &ColladaParserAutoGen15Private::_freeAttributes__float4x2);
functionMap[HASH_ELEMENT_FLOAT4X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4x3, &ColladaParserAutoGen15Private::_data__float4x3, &ColladaParserAutoGen15Private::_end__float4x3, &ColladaParserAutoGen15Private::_preBegin__float4x3, &ColladaParserAutoGen15Private::_preEnd__float4x3, &ColladaParserAutoGen15Private::_freeAttributes__float4x3);
functionMap[HASH_ELEMENT_FLOAT4X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4x4, &ColladaParserAutoGen15Private::_data__float4x4, &ColladaParserAutoGen15Private::_end__float4x4, &ColladaParserAutoGen15Private::_preBegin__float4x4, &ColladaParserAutoGen15Private::_preEnd__float4x4, &ColladaParserAutoGen15Private::_freeAttributes__float4x4);
functionMap[HASH_ELEMENT_INT____INT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int____int_type, &ColladaParserAutoGen15Private::_data__int____int_type, &ColladaParserAutoGen15Private::_end__int____int_type, &ColladaParserAutoGen15Private::_preBegin__int____int_type, &ColladaParserAutoGen15Private::_preEnd__int____int_type, &ColladaParserAutoGen15Private::_freeAttributes__int____int_type);
functionMap[HASH_ELEMENT_INT2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int2, &ColladaParserAutoGen15Private::_data__int2, &ColladaParserAutoGen15Private::_end__int2, &ColladaParserAutoGen15Private::_preBegin__int2, &ColladaParserAutoGen15Private::_preEnd__int2, &ColladaParserAutoGen15Private::_freeAttributes__int2);
functionMap[HASH_ELEMENT_INT3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int3, &ColladaParserAutoGen15Private::_data__int3, &ColladaParserAutoGen15Private::_end__int3, &ColladaParserAutoGen15Private::_preBegin__int3, &ColladaParserAutoGen15Private::_preEnd__int3, &ColladaParserAutoGen15Private::_freeAttributes__int3);
functionMap[HASH_ELEMENT_INT4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int4, &ColladaParserAutoGen15Private::_data__int4, &ColladaParserAutoGen15Private::_end__int4, &ColladaParserAutoGen15Private::_preBegin__int4, &ColladaParserAutoGen15Private::_preEnd__int4, &ColladaParserAutoGen15Private::_freeAttributes__int4);
functionMap[HASH_ELEMENT_MODIFIER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modifier, &ColladaParserAutoGen15Private::_data__modifier, &ColladaParserAutoGen15Private::_end__modifier, &ColladaParserAutoGen15Private::_preBegin__modifier, &ColladaParserAutoGen15Private::_preEnd__modifier, &ColladaParserAutoGen15Private::_freeAttributes__modifier);
functionMap[HASH_ELEMENT_SAMPLER1D] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler1D, &ColladaParserAutoGen15Private::_data__sampler1D, &ColladaParserAutoGen15Private::_end__sampler1D, &ColladaParserAutoGen15Private::_preBegin__sampler1D, &ColladaParserAutoGen15Private::_preEnd__sampler1D, &ColladaParserAutoGen15Private::_freeAttributes__sampler1D);
functionMap[HASH_ELEMENT_SAMPLER2D____FX_SAMPLER2D_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_data__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_end__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_preBegin__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_preEnd__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_freeAttributes__sampler2D____fx_sampler2D_type);
functionMap[HASH_ELEMENT_SAMPLER3D] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler3D, &ColladaParserAutoGen15Private::_data__sampler3D, &ColladaParserAutoGen15Private::_end__sampler3D, &ColladaParserAutoGen15Private::_preBegin__sampler3D, &ColladaParserAutoGen15Private::_preEnd__sampler3D, &ColladaParserAutoGen15Private::_freeAttributes__sampler3D);
functionMap[HASH_ELEMENT_SAMPLERCUBE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__samplerCUBE, &ColladaParserAutoGen15Private::_data__samplerCUBE, &ColladaParserAutoGen15Private::_end__samplerCUBE, &ColladaParserAutoGen15Private::_preBegin__samplerCUBE, &ColladaParserAutoGen15Private::_preEnd__samplerCUBE, &ColladaParserAutoGen15Private::_freeAttributes__samplerCUBE);
functionMap[HASH_ELEMENT_SAMPLERDEPTH] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__samplerDEPTH, &ColladaParserAutoGen15Private::_data__samplerDEPTH, &ColladaParserAutoGen15Private::_end__samplerDEPTH, &ColladaParserAutoGen15Private::_preBegin__samplerDEPTH, &ColladaParserAutoGen15Private::_preEnd__samplerDEPTH, &ColladaParserAutoGen15Private::_freeAttributes__samplerDEPTH);
functionMap[HASH_ELEMENT_SAMPLERRECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__samplerRECT, &ColladaParserAutoGen15Private::_data__samplerRECT, &ColladaParserAutoGen15Private::_end__samplerRECT, &ColladaParserAutoGen15Private::_preBegin__samplerRECT, &ColladaParserAutoGen15Private::_preEnd__samplerRECT, &ColladaParserAutoGen15Private::_freeAttributes__samplerRECT);
functionMap[HASH_ELEMENT_SEMANTIC____NCNAME] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__semantic____NCName, &ColladaParserAutoGen15Private::_data__semantic____NCName, &ColladaParserAutoGen15Private::_end__semantic____NCName, &ColladaParserAutoGen15Private::_preBegin__semantic____NCName, &ColladaParserAutoGen15Private::_preEnd__semantic____NCName, &ColladaParserAutoGen15Private::_freeAttributes__semantic____NCName);
functionMap[HASH_ELEMENT_BORDER_COLOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__border_color, &ColladaParserAutoGen15Private::_data__border_color, &ColladaParserAutoGen15Private::_end__border_color, &ColladaParserAutoGen15Private::_preBegin__border_color, &ColladaParserAutoGen15Private::_preEnd__border_color, &ColladaParserAutoGen15Private::_freeAttributes__border_color);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INSTANCE_IMAGE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_image, &ColladaParserAutoGen15Private::_data__instance_image, &ColladaParserAutoGen15Private::_end__instance_image, &ColladaParserAutoGen15Private::_preBegin__instance_image, &ColladaParserAutoGen15Private::_preEnd__instance_image, &ColladaParserAutoGen15Private::_freeAttributes__instance_image);
functionMap[HASH_ELEMENT_MAGFILTER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__magfilter, &ColladaParserAutoGen15Private::_data__magfilter, &ColladaParserAutoGen15Private::_end__magfilter, &ColladaParserAutoGen15Private::_preBegin__magfilter, &ColladaParserAutoGen15Private::_preEnd__magfilter, &ColladaParserAutoGen15Private::_freeAttributes__magfilter);
functionMap[HASH_ELEMENT_MAX_ANISOTROPY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__max_anisotropy, &ColladaParserAutoGen15Private::_data__max_anisotropy, &ColladaParserAutoGen15Private::_end__max_anisotropy, &ColladaParserAutoGen15Private::_preBegin__max_anisotropy, &ColladaParserAutoGen15Private::_preEnd__max_anisotropy, &ColladaParserAutoGen15Private::_freeAttributes__max_anisotropy);
functionMap[HASH_ELEMENT_MINFILTER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__minfilter, &ColladaParserAutoGen15Private::_data__minfilter, &ColladaParserAutoGen15Private::_end__minfilter, &ColladaParserAutoGen15Private::_preBegin__minfilter, &ColladaParserAutoGen15Private::_preEnd__minfilter, &ColladaParserAutoGen15Private::_freeAttributes__minfilter);
functionMap[HASH_ELEMENT_MIP_BIAS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__mip_bias, &ColladaParserAutoGen15Private::_data__mip_bias, &ColladaParserAutoGen15Private::_end__mip_bias, &ColladaParserAutoGen15Private::_preBegin__mip_bias, &ColladaParserAutoGen15Private::_preEnd__mip_bias, &ColladaParserAutoGen15Private::_freeAttributes__mip_bias);
functionMap[HASH_ELEMENT_MIP_MAX_LEVEL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__mip_max_level, &ColladaParserAutoGen15Private::_data__mip_max_level, &ColladaParserAutoGen15Private::_end__mip_max_level, &ColladaParserAutoGen15Private::_preBegin__mip_max_level, &ColladaParserAutoGen15Private::_preEnd__mip_max_level, &ColladaParserAutoGen15Private::_freeAttributes__mip_max_level);
functionMap[HASH_ELEMENT_MIP_MIN_LEVEL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__mip_min_level, &ColladaParserAutoGen15Private::_data__mip_min_level, &ColladaParserAutoGen15Private::_end__mip_min_level, &ColladaParserAutoGen15Private::_preBegin__mip_min_level, &ColladaParserAutoGen15Private::_preEnd__mip_min_level, &ColladaParserAutoGen15Private::_freeAttributes__mip_min_level);
functionMap[HASH_ELEMENT_MIPFILTER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__mipfilter, &ColladaParserAutoGen15Private::_data__mipfilter, &ColladaParserAutoGen15Private::_end__mipfilter, &ColladaParserAutoGen15Private::_preBegin__mipfilter, &ColladaParserAutoGen15Private::_preEnd__mipfilter, &ColladaParserAutoGen15Private::_freeAttributes__mipfilter);
functionMap[HASH_ELEMENT_WRAP_P] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__wrap_p, &ColladaParserAutoGen15Private::_data__wrap_p, &ColladaParserAutoGen15Private::_end__wrap_p, &ColladaParserAutoGen15Private::_preBegin__wrap_p, &ColladaParserAutoGen15Private::_preEnd__wrap_p, &ColladaParserAutoGen15Private::_freeAttributes__wrap_p);
functionMap[HASH_ELEMENT_WRAP_S____FX_SAMPLER_WRAP_ENUM] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__wrap_s____fx_sampler_wrap_enum, &ColladaParserAutoGen15Private::_data__wrap_s____fx_sampler_wrap_enum, &ColladaParserAutoGen15Private::_end__wrap_s____fx_sampler_wrap_enum, &ColladaParserAutoGen15Private::_preBegin__wrap_s____fx_sampler_wrap_enum, &ColladaParserAutoGen15Private::_preEnd__wrap_s____fx_sampler_wrap_enum, &ColladaParserAutoGen15Private::_freeAttributes__wrap_s____fx_sampler_wrap_enum);
functionMap[HASH_ELEMENT_WRAP_T____FX_SAMPLER_WRAP_ENUM] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__wrap_t____fx_sampler_wrap_enum, &ColladaParserAutoGen15Private::_data__wrap_t____fx_sampler_wrap_enum, &ColladaParserAutoGen15Private::_end__wrap_t____fx_sampler_wrap_enum, &ColladaParserAutoGen15Private::_preBegin__wrap_t____fx_sampler_wrap_enum, &ColladaParserAutoGen15Private::_preEnd__wrap_t____fx_sampler_wrap_enum, &ColladaParserAutoGen15Private::_freeAttributes__wrap_t____fx_sampler_wrap_enum);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_CODE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__code, &ColladaParserAutoGen15Private::_data__code, &ColladaParserAutoGen15Private::_end__code, &ColladaParserAutoGen15Private::_preBegin__code, &ColladaParserAutoGen15Private::_preEnd__code, &ColladaParserAutoGen15Private::_freeAttributes__code);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INCLUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__include, &ColladaParserAutoGen15Private::_data__include, &ColladaParserAutoGen15Private::_end__include, &ColladaParserAutoGen15Private::_preBegin__include, &ColladaParserAutoGen15Private::_preEnd__include, &ColladaParserAutoGen15Private::_freeAttributes__include);
functionMap[HASH_ELEMENT_NEWPARAM____CG_NEWPARAM_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__newparam____cg_newparam_type, &ColladaParserAutoGen15Private::_data__newparam____cg_newparam_type, &ColladaParserAutoGen15Private::_end__newparam____cg_newparam_type, &ColladaParserAutoGen15Private::_preBegin__newparam____cg_newparam_type, &ColladaParserAutoGen15Private::_preEnd__newparam____cg_newparam_type, &ColladaParserAutoGen15Private::_freeAttributes__newparam____cg_newparam_type);
functionMap[HASH_ELEMENT_PROFILE_CG_TYPE____TECHNIQUE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_cg_type____technique, &ColladaParserAutoGen15Private::_data__profile_cg_type____technique, &ColladaParserAutoGen15Private::_end__profile_cg_type____technique, &ColladaParserAutoGen15Private::_preBegin__profile_cg_type____technique, &ColladaParserAutoGen15Private::_preEnd__profile_cg_type____technique, &ColladaParserAutoGen15Private::_freeAttributes__profile_cg_type____technique);
functionMap[HASH_ELEMENT_ANNOTATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__annotate, &ColladaParserAutoGen15Private::_data__annotate, &ColladaParserAutoGen15Private::_end__annotate, &ColladaParserAutoGen15Private::_preBegin__annotate, &ColladaParserAutoGen15Private::_preEnd__annotate, &ColladaParserAutoGen15Private::_freeAttributes__annotate);
functionMap[HASH_ELEMENT_ARRAY____CG_ARRAY_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__array____cg_array_type, &ColladaParserAutoGen15Private::_data__array____cg_array_type, &ColladaParserAutoGen15Private::_end__array____cg_array_type, &ColladaParserAutoGen15Private::_preBegin__array____cg_array_type, &ColladaParserAutoGen15Private::_preEnd__array____cg_array_type, &ColladaParserAutoGen15Private::_freeAttributes__array____cg_array_type);
functionMap[HASH_ELEMENT_BOOL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool, &ColladaParserAutoGen15Private::_data__bool, &ColladaParserAutoGen15Private::_end__bool, &ColladaParserAutoGen15Private::_preBegin__bool, &ColladaParserAutoGen15Private::_preEnd__bool, &ColladaParserAutoGen15Private::_freeAttributes__bool);
functionMap[HASH_ELEMENT_BOOL2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool2, &ColladaParserAutoGen15Private::_data__bool2, &ColladaParserAutoGen15Private::_end__bool2, &ColladaParserAutoGen15Private::_preBegin__bool2, &ColladaParserAutoGen15Private::_preEnd__bool2, &ColladaParserAutoGen15Private::_freeAttributes__bool2);
functionMap[HASH_ELEMENT_BOOL2X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool2x1, &ColladaParserAutoGen15Private::_data__bool2x1, &ColladaParserAutoGen15Private::_end__bool2x1, &ColladaParserAutoGen15Private::_preBegin__bool2x1, &ColladaParserAutoGen15Private::_preEnd__bool2x1, &ColladaParserAutoGen15Private::_freeAttributes__bool2x1);
functionMap[HASH_ELEMENT_BOOL2X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool2x2, &ColladaParserAutoGen15Private::_data__bool2x2, &ColladaParserAutoGen15Private::_end__bool2x2, &ColladaParserAutoGen15Private::_preBegin__bool2x2, &ColladaParserAutoGen15Private::_preEnd__bool2x2, &ColladaParserAutoGen15Private::_freeAttributes__bool2x2);
functionMap[HASH_ELEMENT_BOOL2X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool2x3, &ColladaParserAutoGen15Private::_data__bool2x3, &ColladaParserAutoGen15Private::_end__bool2x3, &ColladaParserAutoGen15Private::_preBegin__bool2x3, &ColladaParserAutoGen15Private::_preEnd__bool2x3, &ColladaParserAutoGen15Private::_freeAttributes__bool2x3);
functionMap[HASH_ELEMENT_BOOL2X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool2x4, &ColladaParserAutoGen15Private::_data__bool2x4, &ColladaParserAutoGen15Private::_end__bool2x4, &ColladaParserAutoGen15Private::_preBegin__bool2x4, &ColladaParserAutoGen15Private::_preEnd__bool2x4, &ColladaParserAutoGen15Private::_freeAttributes__bool2x4);
functionMap[HASH_ELEMENT_BOOL3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool3, &ColladaParserAutoGen15Private::_data__bool3, &ColladaParserAutoGen15Private::_end__bool3, &ColladaParserAutoGen15Private::_preBegin__bool3, &ColladaParserAutoGen15Private::_preEnd__bool3, &ColladaParserAutoGen15Private::_freeAttributes__bool3);
functionMap[HASH_ELEMENT_BOOL3X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool3x1, &ColladaParserAutoGen15Private::_data__bool3x1, &ColladaParserAutoGen15Private::_end__bool3x1, &ColladaParserAutoGen15Private::_preBegin__bool3x1, &ColladaParserAutoGen15Private::_preEnd__bool3x1, &ColladaParserAutoGen15Private::_freeAttributes__bool3x1);
functionMap[HASH_ELEMENT_BOOL3X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool3x2, &ColladaParserAutoGen15Private::_data__bool3x2, &ColladaParserAutoGen15Private::_end__bool3x2, &ColladaParserAutoGen15Private::_preBegin__bool3x2, &ColladaParserAutoGen15Private::_preEnd__bool3x2, &ColladaParserAutoGen15Private::_freeAttributes__bool3x2);
functionMap[HASH_ELEMENT_BOOL3X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool3x3, &ColladaParserAutoGen15Private::_data__bool3x3, &ColladaParserAutoGen15Private::_end__bool3x3, &ColladaParserAutoGen15Private::_preBegin__bool3x3, &ColladaParserAutoGen15Private::_preEnd__bool3x3, &ColladaParserAutoGen15Private::_freeAttributes__bool3x3);
functionMap[HASH_ELEMENT_BOOL3X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool3x4, &ColladaParserAutoGen15Private::_data__bool3x4, &ColladaParserAutoGen15Private::_end__bool3x4, &ColladaParserAutoGen15Private::_preBegin__bool3x4, &ColladaParserAutoGen15Private::_preEnd__bool3x4, &ColladaParserAutoGen15Private::_freeAttributes__bool3x4);
functionMap[HASH_ELEMENT_BOOL4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool4, &ColladaParserAutoGen15Private::_data__bool4, &ColladaParserAutoGen15Private::_end__bool4, &ColladaParserAutoGen15Private::_preBegin__bool4, &ColladaParserAutoGen15Private::_preEnd__bool4, &ColladaParserAutoGen15Private::_freeAttributes__bool4);
functionMap[HASH_ELEMENT_BOOL4X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool4x1, &ColladaParserAutoGen15Private::_data__bool4x1, &ColladaParserAutoGen15Private::_end__bool4x1, &ColladaParserAutoGen15Private::_preBegin__bool4x1, &ColladaParserAutoGen15Private::_preEnd__bool4x1, &ColladaParserAutoGen15Private::_freeAttributes__bool4x1);
functionMap[HASH_ELEMENT_BOOL4X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool4x2, &ColladaParserAutoGen15Private::_data__bool4x2, &ColladaParserAutoGen15Private::_end__bool4x2, &ColladaParserAutoGen15Private::_preBegin__bool4x2, &ColladaParserAutoGen15Private::_preEnd__bool4x2, &ColladaParserAutoGen15Private::_freeAttributes__bool4x2);
functionMap[HASH_ELEMENT_BOOL4X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool4x3, &ColladaParserAutoGen15Private::_data__bool4x3, &ColladaParserAutoGen15Private::_end__bool4x3, &ColladaParserAutoGen15Private::_preBegin__bool4x3, &ColladaParserAutoGen15Private::_preEnd__bool4x3, &ColladaParserAutoGen15Private::_freeAttributes__bool4x3);
functionMap[HASH_ELEMENT_BOOL4X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool4x4, &ColladaParserAutoGen15Private::_data__bool4x4, &ColladaParserAutoGen15Private::_end__bool4x4, &ColladaParserAutoGen15Private::_preBegin__bool4x4, &ColladaParserAutoGen15Private::_preEnd__bool4x4, &ColladaParserAutoGen15Private::_freeAttributes__bool4x4);
functionMap[HASH_ELEMENT_ENUM____GL_ENUMERATION_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_data__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_end__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_preBegin__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_preEnd__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_freeAttributes__enum____gl_enumeration_type);
functionMap[HASH_ELEMENT_FIXED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed, &ColladaParserAutoGen15Private::_data__fixed, &ColladaParserAutoGen15Private::_end__fixed, &ColladaParserAutoGen15Private::_preBegin__fixed, &ColladaParserAutoGen15Private::_preEnd__fixed, &ColladaParserAutoGen15Private::_freeAttributes__fixed);
functionMap[HASH_ELEMENT_FIXED2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed2, &ColladaParserAutoGen15Private::_data__fixed2, &ColladaParserAutoGen15Private::_end__fixed2, &ColladaParserAutoGen15Private::_preBegin__fixed2, &ColladaParserAutoGen15Private::_preEnd__fixed2, &ColladaParserAutoGen15Private::_freeAttributes__fixed2);
functionMap[HASH_ELEMENT_FIXED2X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed2x1, &ColladaParserAutoGen15Private::_data__fixed2x1, &ColladaParserAutoGen15Private::_end__fixed2x1, &ColladaParserAutoGen15Private::_preBegin__fixed2x1, &ColladaParserAutoGen15Private::_preEnd__fixed2x1, &ColladaParserAutoGen15Private::_freeAttributes__fixed2x1);
functionMap[HASH_ELEMENT_FIXED2X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed2x2, &ColladaParserAutoGen15Private::_data__fixed2x2, &ColladaParserAutoGen15Private::_end__fixed2x2, &ColladaParserAutoGen15Private::_preBegin__fixed2x2, &ColladaParserAutoGen15Private::_preEnd__fixed2x2, &ColladaParserAutoGen15Private::_freeAttributes__fixed2x2);
functionMap[HASH_ELEMENT_FIXED2X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed2x3, &ColladaParserAutoGen15Private::_data__fixed2x3, &ColladaParserAutoGen15Private::_end__fixed2x3, &ColladaParserAutoGen15Private::_preBegin__fixed2x3, &ColladaParserAutoGen15Private::_preEnd__fixed2x3, &ColladaParserAutoGen15Private::_freeAttributes__fixed2x3);
functionMap[HASH_ELEMENT_FIXED2X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed2x4, &ColladaParserAutoGen15Private::_data__fixed2x4, &ColladaParserAutoGen15Private::_end__fixed2x4, &ColladaParserAutoGen15Private::_preBegin__fixed2x4, &ColladaParserAutoGen15Private::_preEnd__fixed2x4, &ColladaParserAutoGen15Private::_freeAttributes__fixed2x4);
functionMap[HASH_ELEMENT_FIXED3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed3, &ColladaParserAutoGen15Private::_data__fixed3, &ColladaParserAutoGen15Private::_end__fixed3, &ColladaParserAutoGen15Private::_preBegin__fixed3, &ColladaParserAutoGen15Private::_preEnd__fixed3, &ColladaParserAutoGen15Private::_freeAttributes__fixed3);
functionMap[HASH_ELEMENT_FIXED3X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed3x1, &ColladaParserAutoGen15Private::_data__fixed3x1, &ColladaParserAutoGen15Private::_end__fixed3x1, &ColladaParserAutoGen15Private::_preBegin__fixed3x1, &ColladaParserAutoGen15Private::_preEnd__fixed3x1, &ColladaParserAutoGen15Private::_freeAttributes__fixed3x1);
functionMap[HASH_ELEMENT_FIXED3X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed3x2, &ColladaParserAutoGen15Private::_data__fixed3x2, &ColladaParserAutoGen15Private::_end__fixed3x2, &ColladaParserAutoGen15Private::_preBegin__fixed3x2, &ColladaParserAutoGen15Private::_preEnd__fixed3x2, &ColladaParserAutoGen15Private::_freeAttributes__fixed3x2);
functionMap[HASH_ELEMENT_FIXED3X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed3x3, &ColladaParserAutoGen15Private::_data__fixed3x3, &ColladaParserAutoGen15Private::_end__fixed3x3, &ColladaParserAutoGen15Private::_preBegin__fixed3x3, &ColladaParserAutoGen15Private::_preEnd__fixed3x3, &ColladaParserAutoGen15Private::_freeAttributes__fixed3x3);
functionMap[HASH_ELEMENT_FIXED3X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed3x4, &ColladaParserAutoGen15Private::_data__fixed3x4, &ColladaParserAutoGen15Private::_end__fixed3x4, &ColladaParserAutoGen15Private::_preBegin__fixed3x4, &ColladaParserAutoGen15Private::_preEnd__fixed3x4, &ColladaParserAutoGen15Private::_freeAttributes__fixed3x4);
functionMap[HASH_ELEMENT_FIXED4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed4, &ColladaParserAutoGen15Private::_data__fixed4, &ColladaParserAutoGen15Private::_end__fixed4, &ColladaParserAutoGen15Private::_preBegin__fixed4, &ColladaParserAutoGen15Private::_preEnd__fixed4, &ColladaParserAutoGen15Private::_freeAttributes__fixed4);
functionMap[HASH_ELEMENT_FIXED4X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed4x1, &ColladaParserAutoGen15Private::_data__fixed4x1, &ColladaParserAutoGen15Private::_end__fixed4x1, &ColladaParserAutoGen15Private::_preBegin__fixed4x1, &ColladaParserAutoGen15Private::_preEnd__fixed4x1, &ColladaParserAutoGen15Private::_freeAttributes__fixed4x1);
functionMap[HASH_ELEMENT_FIXED4X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed4x2, &ColladaParserAutoGen15Private::_data__fixed4x2, &ColladaParserAutoGen15Private::_end__fixed4x2, &ColladaParserAutoGen15Private::_preBegin__fixed4x2, &ColladaParserAutoGen15Private::_preEnd__fixed4x2, &ColladaParserAutoGen15Private::_freeAttributes__fixed4x2);
functionMap[HASH_ELEMENT_FIXED4X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed4x3, &ColladaParserAutoGen15Private::_data__fixed4x3, &ColladaParserAutoGen15Private::_end__fixed4x3, &ColladaParserAutoGen15Private::_preBegin__fixed4x3, &ColladaParserAutoGen15Private::_preEnd__fixed4x3, &ColladaParserAutoGen15Private::_freeAttributes__fixed4x3);
functionMap[HASH_ELEMENT_FIXED4X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed4x4, &ColladaParserAutoGen15Private::_data__fixed4x4, &ColladaParserAutoGen15Private::_end__fixed4x4, &ColladaParserAutoGen15Private::_preBegin__fixed4x4, &ColladaParserAutoGen15Private::_preEnd__fixed4x4, &ColladaParserAutoGen15Private::_freeAttributes__fixed4x4);
functionMap[HASH_ELEMENT_FLOAT____FLOAT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float____float_type, &ColladaParserAutoGen15Private::_data__float____float_type, &ColladaParserAutoGen15Private::_end__float____float_type, &ColladaParserAutoGen15Private::_preBegin__float____float_type, &ColladaParserAutoGen15Private::_preEnd__float____float_type, &ColladaParserAutoGen15Private::_freeAttributes__float____float_type);
functionMap[HASH_ELEMENT_FLOAT2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2, &ColladaParserAutoGen15Private::_data__float2, &ColladaParserAutoGen15Private::_end__float2, &ColladaParserAutoGen15Private::_preBegin__float2, &ColladaParserAutoGen15Private::_preEnd__float2, &ColladaParserAutoGen15Private::_freeAttributes__float2);
functionMap[HASH_ELEMENT_FLOAT2X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2x1, &ColladaParserAutoGen15Private::_data__float2x1, &ColladaParserAutoGen15Private::_end__float2x1, &ColladaParserAutoGen15Private::_preBegin__float2x1, &ColladaParserAutoGen15Private::_preEnd__float2x1, &ColladaParserAutoGen15Private::_freeAttributes__float2x1);
functionMap[HASH_ELEMENT_FLOAT2X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2x2, &ColladaParserAutoGen15Private::_data__float2x2, &ColladaParserAutoGen15Private::_end__float2x2, &ColladaParserAutoGen15Private::_preBegin__float2x2, &ColladaParserAutoGen15Private::_preEnd__float2x2, &ColladaParserAutoGen15Private::_freeAttributes__float2x2);
functionMap[HASH_ELEMENT_FLOAT2X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2x3, &ColladaParserAutoGen15Private::_data__float2x3, &ColladaParserAutoGen15Private::_end__float2x3, &ColladaParserAutoGen15Private::_preBegin__float2x3, &ColladaParserAutoGen15Private::_preEnd__float2x3, &ColladaParserAutoGen15Private::_freeAttributes__float2x3);
functionMap[HASH_ELEMENT_FLOAT2X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2x4, &ColladaParserAutoGen15Private::_data__float2x4, &ColladaParserAutoGen15Private::_end__float2x4, &ColladaParserAutoGen15Private::_preBegin__float2x4, &ColladaParserAutoGen15Private::_preEnd__float2x4, &ColladaParserAutoGen15Private::_freeAttributes__float2x4);
functionMap[HASH_ELEMENT_FLOAT3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3, &ColladaParserAutoGen15Private::_data__float3, &ColladaParserAutoGen15Private::_end__float3, &ColladaParserAutoGen15Private::_preBegin__float3, &ColladaParserAutoGen15Private::_preEnd__float3, &ColladaParserAutoGen15Private::_freeAttributes__float3);
functionMap[HASH_ELEMENT_FLOAT3X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3x1, &ColladaParserAutoGen15Private::_data__float3x1, &ColladaParserAutoGen15Private::_end__float3x1, &ColladaParserAutoGen15Private::_preBegin__float3x1, &ColladaParserAutoGen15Private::_preEnd__float3x1, &ColladaParserAutoGen15Private::_freeAttributes__float3x1);
functionMap[HASH_ELEMENT_FLOAT3X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3x2, &ColladaParserAutoGen15Private::_data__float3x2, &ColladaParserAutoGen15Private::_end__float3x2, &ColladaParserAutoGen15Private::_preBegin__float3x2, &ColladaParserAutoGen15Private::_preEnd__float3x2, &ColladaParserAutoGen15Private::_freeAttributes__float3x2);
functionMap[HASH_ELEMENT_FLOAT3X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3x3, &ColladaParserAutoGen15Private::_data__float3x3, &ColladaParserAutoGen15Private::_end__float3x3, &ColladaParserAutoGen15Private::_preBegin__float3x3, &ColladaParserAutoGen15Private::_preEnd__float3x3, &ColladaParserAutoGen15Private::_freeAttributes__float3x3);
functionMap[HASH_ELEMENT_FLOAT3X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3x4, &ColladaParserAutoGen15Private::_data__float3x4, &ColladaParserAutoGen15Private::_end__float3x4, &ColladaParserAutoGen15Private::_preBegin__float3x4, &ColladaParserAutoGen15Private::_preEnd__float3x4, &ColladaParserAutoGen15Private::_freeAttributes__float3x4);
functionMap[HASH_ELEMENT_FLOAT4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4, &ColladaParserAutoGen15Private::_data__float4, &ColladaParserAutoGen15Private::_end__float4, &ColladaParserAutoGen15Private::_preBegin__float4, &ColladaParserAutoGen15Private::_preEnd__float4, &ColladaParserAutoGen15Private::_freeAttributes__float4);
functionMap[HASH_ELEMENT_FLOAT4X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4x1, &ColladaParserAutoGen15Private::_data__float4x1, &ColladaParserAutoGen15Private::_end__float4x1, &ColladaParserAutoGen15Private::_preBegin__float4x1, &ColladaParserAutoGen15Private::_preEnd__float4x1, &ColladaParserAutoGen15Private::_freeAttributes__float4x1);
functionMap[HASH_ELEMENT_FLOAT4X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4x2, &ColladaParserAutoGen15Private::_data__float4x2, &ColladaParserAutoGen15Private::_end__float4x2, &ColladaParserAutoGen15Private::_preBegin__float4x2, &ColladaParserAutoGen15Private::_preEnd__float4x2, &ColladaParserAutoGen15Private::_freeAttributes__float4x2);
functionMap[HASH_ELEMENT_FLOAT4X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4x3, &ColladaParserAutoGen15Private::_data__float4x3, &ColladaParserAutoGen15Private::_end__float4x3, &ColladaParserAutoGen15Private::_preBegin__float4x3, &ColladaParserAutoGen15Private::_preEnd__float4x3, &ColladaParserAutoGen15Private::_freeAttributes__float4x3);
functionMap[HASH_ELEMENT_FLOAT4X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4x4, &ColladaParserAutoGen15Private::_data__float4x4, &ColladaParserAutoGen15Private::_end__float4x4, &ColladaParserAutoGen15Private::_preBegin__float4x4, &ColladaParserAutoGen15Private::_preEnd__float4x4, &ColladaParserAutoGen15Private::_freeAttributes__float4x4);
functionMap[HASH_ELEMENT_HALF] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half, &ColladaParserAutoGen15Private::_data__half, &ColladaParserAutoGen15Private::_end__half, &ColladaParserAutoGen15Private::_preBegin__half, &ColladaParserAutoGen15Private::_preEnd__half, &ColladaParserAutoGen15Private::_freeAttributes__half);
functionMap[HASH_ELEMENT_HALF2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half2, &ColladaParserAutoGen15Private::_data__half2, &ColladaParserAutoGen15Private::_end__half2, &ColladaParserAutoGen15Private::_preBegin__half2, &ColladaParserAutoGen15Private::_preEnd__half2, &ColladaParserAutoGen15Private::_freeAttributes__half2);
functionMap[HASH_ELEMENT_HALF2X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half2x1, &ColladaParserAutoGen15Private::_data__half2x1, &ColladaParserAutoGen15Private::_end__half2x1, &ColladaParserAutoGen15Private::_preBegin__half2x1, &ColladaParserAutoGen15Private::_preEnd__half2x1, &ColladaParserAutoGen15Private::_freeAttributes__half2x1);
functionMap[HASH_ELEMENT_HALF2X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half2x2, &ColladaParserAutoGen15Private::_data__half2x2, &ColladaParserAutoGen15Private::_end__half2x2, &ColladaParserAutoGen15Private::_preBegin__half2x2, &ColladaParserAutoGen15Private::_preEnd__half2x2, &ColladaParserAutoGen15Private::_freeAttributes__half2x2);
functionMap[HASH_ELEMENT_HALF2X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half2x3, &ColladaParserAutoGen15Private::_data__half2x3, &ColladaParserAutoGen15Private::_end__half2x3, &ColladaParserAutoGen15Private::_preBegin__half2x3, &ColladaParserAutoGen15Private::_preEnd__half2x3, &ColladaParserAutoGen15Private::_freeAttributes__half2x3);
functionMap[HASH_ELEMENT_HALF2X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half2x4, &ColladaParserAutoGen15Private::_data__half2x4, &ColladaParserAutoGen15Private::_end__half2x4, &ColladaParserAutoGen15Private::_preBegin__half2x4, &ColladaParserAutoGen15Private::_preEnd__half2x4, &ColladaParserAutoGen15Private::_freeAttributes__half2x4);
functionMap[HASH_ELEMENT_HALF3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half3, &ColladaParserAutoGen15Private::_data__half3, &ColladaParserAutoGen15Private::_end__half3, &ColladaParserAutoGen15Private::_preBegin__half3, &ColladaParserAutoGen15Private::_preEnd__half3, &ColladaParserAutoGen15Private::_freeAttributes__half3);
functionMap[HASH_ELEMENT_HALF3X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half3x1, &ColladaParserAutoGen15Private::_data__half3x1, &ColladaParserAutoGen15Private::_end__half3x1, &ColladaParserAutoGen15Private::_preBegin__half3x1, &ColladaParserAutoGen15Private::_preEnd__half3x1, &ColladaParserAutoGen15Private::_freeAttributes__half3x1);
functionMap[HASH_ELEMENT_HALF3X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half3x2, &ColladaParserAutoGen15Private::_data__half3x2, &ColladaParserAutoGen15Private::_end__half3x2, &ColladaParserAutoGen15Private::_preBegin__half3x2, &ColladaParserAutoGen15Private::_preEnd__half3x2, &ColladaParserAutoGen15Private::_freeAttributes__half3x2);
functionMap[HASH_ELEMENT_HALF3X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half3x3, &ColladaParserAutoGen15Private::_data__half3x3, &ColladaParserAutoGen15Private::_end__half3x3, &ColladaParserAutoGen15Private::_preBegin__half3x3, &ColladaParserAutoGen15Private::_preEnd__half3x3, &ColladaParserAutoGen15Private::_freeAttributes__half3x3);
functionMap[HASH_ELEMENT_HALF3X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half3x4, &ColladaParserAutoGen15Private::_data__half3x4, &ColladaParserAutoGen15Private::_end__half3x4, &ColladaParserAutoGen15Private::_preBegin__half3x4, &ColladaParserAutoGen15Private::_preEnd__half3x4, &ColladaParserAutoGen15Private::_freeAttributes__half3x4);
functionMap[HASH_ELEMENT_HALF4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half4, &ColladaParserAutoGen15Private::_data__half4, &ColladaParserAutoGen15Private::_end__half4, &ColladaParserAutoGen15Private::_preBegin__half4, &ColladaParserAutoGen15Private::_preEnd__half4, &ColladaParserAutoGen15Private::_freeAttributes__half4);
functionMap[HASH_ELEMENT_HALF4X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half4x1, &ColladaParserAutoGen15Private::_data__half4x1, &ColladaParserAutoGen15Private::_end__half4x1, &ColladaParserAutoGen15Private::_preBegin__half4x1, &ColladaParserAutoGen15Private::_preEnd__half4x1, &ColladaParserAutoGen15Private::_freeAttributes__half4x1);
functionMap[HASH_ELEMENT_HALF4X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half4x2, &ColladaParserAutoGen15Private::_data__half4x2, &ColladaParserAutoGen15Private::_end__half4x2, &ColladaParserAutoGen15Private::_preBegin__half4x2, &ColladaParserAutoGen15Private::_preEnd__half4x2, &ColladaParserAutoGen15Private::_freeAttributes__half4x2);
functionMap[HASH_ELEMENT_HALF4X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half4x3, &ColladaParserAutoGen15Private::_data__half4x3, &ColladaParserAutoGen15Private::_end__half4x3, &ColladaParserAutoGen15Private::_preBegin__half4x3, &ColladaParserAutoGen15Private::_preEnd__half4x3, &ColladaParserAutoGen15Private::_freeAttributes__half4x3);
functionMap[HASH_ELEMENT_HALF4X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half4x4, &ColladaParserAutoGen15Private::_data__half4x4, &ColladaParserAutoGen15Private::_end__half4x4, &ColladaParserAutoGen15Private::_preBegin__half4x4, &ColladaParserAutoGen15Private::_preEnd__half4x4, &ColladaParserAutoGen15Private::_freeAttributes__half4x4);
functionMap[HASH_ELEMENT_INT____INT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int____int_type, &ColladaParserAutoGen15Private::_data__int____int_type, &ColladaParserAutoGen15Private::_end__int____int_type, &ColladaParserAutoGen15Private::_preBegin__int____int_type, &ColladaParserAutoGen15Private::_preEnd__int____int_type, &ColladaParserAutoGen15Private::_freeAttributes__int____int_type);
functionMap[HASH_ELEMENT_INT2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int2, &ColladaParserAutoGen15Private::_data__int2, &ColladaParserAutoGen15Private::_end__int2, &ColladaParserAutoGen15Private::_preBegin__int2, &ColladaParserAutoGen15Private::_preEnd__int2, &ColladaParserAutoGen15Private::_freeAttributes__int2);
functionMap[HASH_ELEMENT_INT2X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int2x1, &ColladaParserAutoGen15Private::_data__int2x1, &ColladaParserAutoGen15Private::_end__int2x1, &ColladaParserAutoGen15Private::_preBegin__int2x1, &ColladaParserAutoGen15Private::_preEnd__int2x1, &ColladaParserAutoGen15Private::_freeAttributes__int2x1);
functionMap[HASH_ELEMENT_INT2X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int2x2, &ColladaParserAutoGen15Private::_data__int2x2, &ColladaParserAutoGen15Private::_end__int2x2, &ColladaParserAutoGen15Private::_preBegin__int2x2, &ColladaParserAutoGen15Private::_preEnd__int2x2, &ColladaParserAutoGen15Private::_freeAttributes__int2x2);
functionMap[HASH_ELEMENT_INT2X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int2x3, &ColladaParserAutoGen15Private::_data__int2x3, &ColladaParserAutoGen15Private::_end__int2x3, &ColladaParserAutoGen15Private::_preBegin__int2x3, &ColladaParserAutoGen15Private::_preEnd__int2x3, &ColladaParserAutoGen15Private::_freeAttributes__int2x3);
functionMap[HASH_ELEMENT_INT2X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int2x4, &ColladaParserAutoGen15Private::_data__int2x4, &ColladaParserAutoGen15Private::_end__int2x4, &ColladaParserAutoGen15Private::_preBegin__int2x4, &ColladaParserAutoGen15Private::_preEnd__int2x4, &ColladaParserAutoGen15Private::_freeAttributes__int2x4);
functionMap[HASH_ELEMENT_INT3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int3, &ColladaParserAutoGen15Private::_data__int3, &ColladaParserAutoGen15Private::_end__int3, &ColladaParserAutoGen15Private::_preBegin__int3, &ColladaParserAutoGen15Private::_preEnd__int3, &ColladaParserAutoGen15Private::_freeAttributes__int3);
functionMap[HASH_ELEMENT_INT3X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int3x1, &ColladaParserAutoGen15Private::_data__int3x1, &ColladaParserAutoGen15Private::_end__int3x1, &ColladaParserAutoGen15Private::_preBegin__int3x1, &ColladaParserAutoGen15Private::_preEnd__int3x1, &ColladaParserAutoGen15Private::_freeAttributes__int3x1);
functionMap[HASH_ELEMENT_INT3X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int3x2, &ColladaParserAutoGen15Private::_data__int3x2, &ColladaParserAutoGen15Private::_end__int3x2, &ColladaParserAutoGen15Private::_preBegin__int3x2, &ColladaParserAutoGen15Private::_preEnd__int3x2, &ColladaParserAutoGen15Private::_freeAttributes__int3x2);
functionMap[HASH_ELEMENT_INT3X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int3x3, &ColladaParserAutoGen15Private::_data__int3x3, &ColladaParserAutoGen15Private::_end__int3x3, &ColladaParserAutoGen15Private::_preBegin__int3x3, &ColladaParserAutoGen15Private::_preEnd__int3x3, &ColladaParserAutoGen15Private::_freeAttributes__int3x3);
functionMap[HASH_ELEMENT_INT3X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int3x4, &ColladaParserAutoGen15Private::_data__int3x4, &ColladaParserAutoGen15Private::_end__int3x4, &ColladaParserAutoGen15Private::_preBegin__int3x4, &ColladaParserAutoGen15Private::_preEnd__int3x4, &ColladaParserAutoGen15Private::_freeAttributes__int3x4);
functionMap[HASH_ELEMENT_INT4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int4, &ColladaParserAutoGen15Private::_data__int4, &ColladaParserAutoGen15Private::_end__int4, &ColladaParserAutoGen15Private::_preBegin__int4, &ColladaParserAutoGen15Private::_preEnd__int4, &ColladaParserAutoGen15Private::_freeAttributes__int4);
functionMap[HASH_ELEMENT_INT4X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int4x1, &ColladaParserAutoGen15Private::_data__int4x1, &ColladaParserAutoGen15Private::_end__int4x1, &ColladaParserAutoGen15Private::_preBegin__int4x1, &ColladaParserAutoGen15Private::_preEnd__int4x1, &ColladaParserAutoGen15Private::_freeAttributes__int4x1);
functionMap[HASH_ELEMENT_INT4X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int4x2, &ColladaParserAutoGen15Private::_data__int4x2, &ColladaParserAutoGen15Private::_end__int4x2, &ColladaParserAutoGen15Private::_preBegin__int4x2, &ColladaParserAutoGen15Private::_preEnd__int4x2, &ColladaParserAutoGen15Private::_freeAttributes__int4x2);
functionMap[HASH_ELEMENT_INT4X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int4x3, &ColladaParserAutoGen15Private::_data__int4x3, &ColladaParserAutoGen15Private::_end__int4x3, &ColladaParserAutoGen15Private::_preBegin__int4x3, &ColladaParserAutoGen15Private::_preEnd__int4x3, &ColladaParserAutoGen15Private::_freeAttributes__int4x3);
functionMap[HASH_ELEMENT_INT4X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int4x4, &ColladaParserAutoGen15Private::_data__int4x4, &ColladaParserAutoGen15Private::_end__int4x4, &ColladaParserAutoGen15Private::_preBegin__int4x4, &ColladaParserAutoGen15Private::_preEnd__int4x4, &ColladaParserAutoGen15Private::_freeAttributes__int4x4);
functionMap[HASH_ELEMENT_MODIFIER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modifier, &ColladaParserAutoGen15Private::_data__modifier, &ColladaParserAutoGen15Private::_end__modifier, &ColladaParserAutoGen15Private::_preBegin__modifier, &ColladaParserAutoGen15Private::_preEnd__modifier, &ColladaParserAutoGen15Private::_freeAttributes__modifier);
functionMap[HASH_ELEMENT_SAMPLER1D] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler1D, &ColladaParserAutoGen15Private::_data__sampler1D, &ColladaParserAutoGen15Private::_end__sampler1D, &ColladaParserAutoGen15Private::_preBegin__sampler1D, &ColladaParserAutoGen15Private::_preEnd__sampler1D, &ColladaParserAutoGen15Private::_freeAttributes__sampler1D);
functionMap[HASH_ELEMENT_SAMPLER2D____FX_SAMPLER2D_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_data__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_end__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_preBegin__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_preEnd__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_freeAttributes__sampler2D____fx_sampler2D_type);
functionMap[HASH_ELEMENT_SAMPLER3D] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler3D, &ColladaParserAutoGen15Private::_data__sampler3D, &ColladaParserAutoGen15Private::_end__sampler3D, &ColladaParserAutoGen15Private::_preBegin__sampler3D, &ColladaParserAutoGen15Private::_preEnd__sampler3D, &ColladaParserAutoGen15Private::_freeAttributes__sampler3D);
functionMap[HASH_ELEMENT_SAMPLERCUBE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__samplerCUBE, &ColladaParserAutoGen15Private::_data__samplerCUBE, &ColladaParserAutoGen15Private::_end__samplerCUBE, &ColladaParserAutoGen15Private::_preBegin__samplerCUBE, &ColladaParserAutoGen15Private::_preEnd__samplerCUBE, &ColladaParserAutoGen15Private::_freeAttributes__samplerCUBE);
functionMap[HASH_ELEMENT_SAMPLERDEPTH] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__samplerDEPTH, &ColladaParserAutoGen15Private::_data__samplerDEPTH, &ColladaParserAutoGen15Private::_end__samplerDEPTH, &ColladaParserAutoGen15Private::_preBegin__samplerDEPTH, &ColladaParserAutoGen15Private::_preEnd__samplerDEPTH, &ColladaParserAutoGen15Private::_freeAttributes__samplerDEPTH);
functionMap[HASH_ELEMENT_SAMPLERRECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__samplerRECT, &ColladaParserAutoGen15Private::_data__samplerRECT, &ColladaParserAutoGen15Private::_end__samplerRECT, &ColladaParserAutoGen15Private::_preBegin__samplerRECT, &ColladaParserAutoGen15Private::_preEnd__samplerRECT, &ColladaParserAutoGen15Private::_freeAttributes__samplerRECT);
functionMap[HASH_ELEMENT_SEMANTIC____NCNAME] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__semantic____NCName, &ColladaParserAutoGen15Private::_data__semantic____NCName, &ColladaParserAutoGen15Private::_end__semantic____NCName, &ColladaParserAutoGen15Private::_preBegin__semantic____NCName, &ColladaParserAutoGen15Private::_preEnd__semantic____NCName, &ColladaParserAutoGen15Private::_freeAttributes__semantic____NCName);
functionMap[HASH_ELEMENT_STRING] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__string, &ColladaParserAutoGen15Private::_data__string, &ColladaParserAutoGen15Private::_end__string, &ColladaParserAutoGen15Private::_preBegin__string, &ColladaParserAutoGen15Private::_preEnd__string, &ColladaParserAutoGen15Private::_freeAttributes__string);
functionMap[HASH_ELEMENT_USERTYPE____CG_USER_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__usertype____cg_user_type, &ColladaParserAutoGen15Private::_data__usertype____cg_user_type, &ColladaParserAutoGen15Private::_end__usertype____cg_user_type, &ColladaParserAutoGen15Private::_preBegin__usertype____cg_user_type, &ColladaParserAutoGen15Private::_preEnd__usertype____cg_user_type, &ColladaParserAutoGen15Private::_freeAttributes__usertype____cg_user_type);
functionMap[HASH_ELEMENT_BOOL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool, &ColladaParserAutoGen15Private::_data__bool, &ColladaParserAutoGen15Private::_end__bool, &ColladaParserAutoGen15Private::_preBegin__bool, &ColladaParserAutoGen15Private::_preEnd__bool, &ColladaParserAutoGen15Private::_freeAttributes__bool);
functionMap[HASH_ELEMENT_BOOL2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool2, &ColladaParserAutoGen15Private::_data__bool2, &ColladaParserAutoGen15Private::_end__bool2, &ColladaParserAutoGen15Private::_preBegin__bool2, &ColladaParserAutoGen15Private::_preEnd__bool2, &ColladaParserAutoGen15Private::_freeAttributes__bool2);
functionMap[HASH_ELEMENT_BOOL2X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool2x1, &ColladaParserAutoGen15Private::_data__bool2x1, &ColladaParserAutoGen15Private::_end__bool2x1, &ColladaParserAutoGen15Private::_preBegin__bool2x1, &ColladaParserAutoGen15Private::_preEnd__bool2x1, &ColladaParserAutoGen15Private::_freeAttributes__bool2x1);
functionMap[HASH_ELEMENT_BOOL2X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool2x2, &ColladaParserAutoGen15Private::_data__bool2x2, &ColladaParserAutoGen15Private::_end__bool2x2, &ColladaParserAutoGen15Private::_preBegin__bool2x2, &ColladaParserAutoGen15Private::_preEnd__bool2x2, &ColladaParserAutoGen15Private::_freeAttributes__bool2x2);
functionMap[HASH_ELEMENT_BOOL2X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool2x3, &ColladaParserAutoGen15Private::_data__bool2x3, &ColladaParserAutoGen15Private::_end__bool2x3, &ColladaParserAutoGen15Private::_preBegin__bool2x3, &ColladaParserAutoGen15Private::_preEnd__bool2x3, &ColladaParserAutoGen15Private::_freeAttributes__bool2x3);
functionMap[HASH_ELEMENT_BOOL2X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool2x4, &ColladaParserAutoGen15Private::_data__bool2x4, &ColladaParserAutoGen15Private::_end__bool2x4, &ColladaParserAutoGen15Private::_preBegin__bool2x4, &ColladaParserAutoGen15Private::_preEnd__bool2x4, &ColladaParserAutoGen15Private::_freeAttributes__bool2x4);
functionMap[HASH_ELEMENT_BOOL3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool3, &ColladaParserAutoGen15Private::_data__bool3, &ColladaParserAutoGen15Private::_end__bool3, &ColladaParserAutoGen15Private::_preBegin__bool3, &ColladaParserAutoGen15Private::_preEnd__bool3, &ColladaParserAutoGen15Private::_freeAttributes__bool3);
functionMap[HASH_ELEMENT_BOOL3X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool3x1, &ColladaParserAutoGen15Private::_data__bool3x1, &ColladaParserAutoGen15Private::_end__bool3x1, &ColladaParserAutoGen15Private::_preBegin__bool3x1, &ColladaParserAutoGen15Private::_preEnd__bool3x1, &ColladaParserAutoGen15Private::_freeAttributes__bool3x1);
functionMap[HASH_ELEMENT_BOOL3X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool3x2, &ColladaParserAutoGen15Private::_data__bool3x2, &ColladaParserAutoGen15Private::_end__bool3x2, &ColladaParserAutoGen15Private::_preBegin__bool3x2, &ColladaParserAutoGen15Private::_preEnd__bool3x2, &ColladaParserAutoGen15Private::_freeAttributes__bool3x2);
functionMap[HASH_ELEMENT_BOOL3X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool3x3, &ColladaParserAutoGen15Private::_data__bool3x3, &ColladaParserAutoGen15Private::_end__bool3x3, &ColladaParserAutoGen15Private::_preBegin__bool3x3, &ColladaParserAutoGen15Private::_preEnd__bool3x3, &ColladaParserAutoGen15Private::_freeAttributes__bool3x3);
functionMap[HASH_ELEMENT_BOOL3X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool3x4, &ColladaParserAutoGen15Private::_data__bool3x4, &ColladaParserAutoGen15Private::_end__bool3x4, &ColladaParserAutoGen15Private::_preBegin__bool3x4, &ColladaParserAutoGen15Private::_preEnd__bool3x4, &ColladaParserAutoGen15Private::_freeAttributes__bool3x4);
functionMap[HASH_ELEMENT_BOOL4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool4, &ColladaParserAutoGen15Private::_data__bool4, &ColladaParserAutoGen15Private::_end__bool4, &ColladaParserAutoGen15Private::_preBegin__bool4, &ColladaParserAutoGen15Private::_preEnd__bool4, &ColladaParserAutoGen15Private::_freeAttributes__bool4);
functionMap[HASH_ELEMENT_BOOL4X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool4x1, &ColladaParserAutoGen15Private::_data__bool4x1, &ColladaParserAutoGen15Private::_end__bool4x1, &ColladaParserAutoGen15Private::_preBegin__bool4x1, &ColladaParserAutoGen15Private::_preEnd__bool4x1, &ColladaParserAutoGen15Private::_freeAttributes__bool4x1);
functionMap[HASH_ELEMENT_BOOL4X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool4x2, &ColladaParserAutoGen15Private::_data__bool4x2, &ColladaParserAutoGen15Private::_end__bool4x2, &ColladaParserAutoGen15Private::_preBegin__bool4x2, &ColladaParserAutoGen15Private::_preEnd__bool4x2, &ColladaParserAutoGen15Private::_freeAttributes__bool4x2);
functionMap[HASH_ELEMENT_BOOL4X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool4x3, &ColladaParserAutoGen15Private::_data__bool4x3, &ColladaParserAutoGen15Private::_end__bool4x3, &ColladaParserAutoGen15Private::_preBegin__bool4x3, &ColladaParserAutoGen15Private::_preEnd__bool4x3, &ColladaParserAutoGen15Private::_freeAttributes__bool4x3);
functionMap[HASH_ELEMENT_BOOL4X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool4x4, &ColladaParserAutoGen15Private::_data__bool4x4, &ColladaParserAutoGen15Private::_end__bool4x4, &ColladaParserAutoGen15Private::_preBegin__bool4x4, &ColladaParserAutoGen15Private::_preEnd__bool4x4, &ColladaParserAutoGen15Private::_freeAttributes__bool4x4);
functionMap[HASH_ELEMENT_ENUM____GL_ENUMERATION_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_data__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_end__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_preBegin__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_preEnd__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_freeAttributes__enum____gl_enumeration_type);
functionMap[HASH_ELEMENT_FIXED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed, &ColladaParserAutoGen15Private::_data__fixed, &ColladaParserAutoGen15Private::_end__fixed, &ColladaParserAutoGen15Private::_preBegin__fixed, &ColladaParserAutoGen15Private::_preEnd__fixed, &ColladaParserAutoGen15Private::_freeAttributes__fixed);
functionMap[HASH_ELEMENT_FIXED2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed2, &ColladaParserAutoGen15Private::_data__fixed2, &ColladaParserAutoGen15Private::_end__fixed2, &ColladaParserAutoGen15Private::_preBegin__fixed2, &ColladaParserAutoGen15Private::_preEnd__fixed2, &ColladaParserAutoGen15Private::_freeAttributes__fixed2);
functionMap[HASH_ELEMENT_FIXED2X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed2x1, &ColladaParserAutoGen15Private::_data__fixed2x1, &ColladaParserAutoGen15Private::_end__fixed2x1, &ColladaParserAutoGen15Private::_preBegin__fixed2x1, &ColladaParserAutoGen15Private::_preEnd__fixed2x1, &ColladaParserAutoGen15Private::_freeAttributes__fixed2x1);
functionMap[HASH_ELEMENT_FIXED2X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed2x2, &ColladaParserAutoGen15Private::_data__fixed2x2, &ColladaParserAutoGen15Private::_end__fixed2x2, &ColladaParserAutoGen15Private::_preBegin__fixed2x2, &ColladaParserAutoGen15Private::_preEnd__fixed2x2, &ColladaParserAutoGen15Private::_freeAttributes__fixed2x2);
functionMap[HASH_ELEMENT_FIXED2X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed2x3, &ColladaParserAutoGen15Private::_data__fixed2x3, &ColladaParserAutoGen15Private::_end__fixed2x3, &ColladaParserAutoGen15Private::_preBegin__fixed2x3, &ColladaParserAutoGen15Private::_preEnd__fixed2x3, &ColladaParserAutoGen15Private::_freeAttributes__fixed2x3);
functionMap[HASH_ELEMENT_FIXED2X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed2x4, &ColladaParserAutoGen15Private::_data__fixed2x4, &ColladaParserAutoGen15Private::_end__fixed2x4, &ColladaParserAutoGen15Private::_preBegin__fixed2x4, &ColladaParserAutoGen15Private::_preEnd__fixed2x4, &ColladaParserAutoGen15Private::_freeAttributes__fixed2x4);
functionMap[HASH_ELEMENT_FIXED3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed3, &ColladaParserAutoGen15Private::_data__fixed3, &ColladaParserAutoGen15Private::_end__fixed3, &ColladaParserAutoGen15Private::_preBegin__fixed3, &ColladaParserAutoGen15Private::_preEnd__fixed3, &ColladaParserAutoGen15Private::_freeAttributes__fixed3);
functionMap[HASH_ELEMENT_FIXED3X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed3x1, &ColladaParserAutoGen15Private::_data__fixed3x1, &ColladaParserAutoGen15Private::_end__fixed3x1, &ColladaParserAutoGen15Private::_preBegin__fixed3x1, &ColladaParserAutoGen15Private::_preEnd__fixed3x1, &ColladaParserAutoGen15Private::_freeAttributes__fixed3x1);
functionMap[HASH_ELEMENT_FIXED3X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed3x2, &ColladaParserAutoGen15Private::_data__fixed3x2, &ColladaParserAutoGen15Private::_end__fixed3x2, &ColladaParserAutoGen15Private::_preBegin__fixed3x2, &ColladaParserAutoGen15Private::_preEnd__fixed3x2, &ColladaParserAutoGen15Private::_freeAttributes__fixed3x2);
functionMap[HASH_ELEMENT_FIXED3X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed3x3, &ColladaParserAutoGen15Private::_data__fixed3x3, &ColladaParserAutoGen15Private::_end__fixed3x3, &ColladaParserAutoGen15Private::_preBegin__fixed3x3, &ColladaParserAutoGen15Private::_preEnd__fixed3x3, &ColladaParserAutoGen15Private::_freeAttributes__fixed3x3);
functionMap[HASH_ELEMENT_FIXED3X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed3x4, &ColladaParserAutoGen15Private::_data__fixed3x4, &ColladaParserAutoGen15Private::_end__fixed3x4, &ColladaParserAutoGen15Private::_preBegin__fixed3x4, &ColladaParserAutoGen15Private::_preEnd__fixed3x4, &ColladaParserAutoGen15Private::_freeAttributes__fixed3x4);
functionMap[HASH_ELEMENT_FIXED4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed4, &ColladaParserAutoGen15Private::_data__fixed4, &ColladaParserAutoGen15Private::_end__fixed4, &ColladaParserAutoGen15Private::_preBegin__fixed4, &ColladaParserAutoGen15Private::_preEnd__fixed4, &ColladaParserAutoGen15Private::_freeAttributes__fixed4);
functionMap[HASH_ELEMENT_FIXED4X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed4x1, &ColladaParserAutoGen15Private::_data__fixed4x1, &ColladaParserAutoGen15Private::_end__fixed4x1, &ColladaParserAutoGen15Private::_preBegin__fixed4x1, &ColladaParserAutoGen15Private::_preEnd__fixed4x1, &ColladaParserAutoGen15Private::_freeAttributes__fixed4x1);
functionMap[HASH_ELEMENT_FIXED4X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed4x2, &ColladaParserAutoGen15Private::_data__fixed4x2, &ColladaParserAutoGen15Private::_end__fixed4x2, &ColladaParserAutoGen15Private::_preBegin__fixed4x2, &ColladaParserAutoGen15Private::_preEnd__fixed4x2, &ColladaParserAutoGen15Private::_freeAttributes__fixed4x2);
functionMap[HASH_ELEMENT_FIXED4X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed4x3, &ColladaParserAutoGen15Private::_data__fixed4x3, &ColladaParserAutoGen15Private::_end__fixed4x3, &ColladaParserAutoGen15Private::_preBegin__fixed4x3, &ColladaParserAutoGen15Private::_preEnd__fixed4x3, &ColladaParserAutoGen15Private::_freeAttributes__fixed4x3);
functionMap[HASH_ELEMENT_FIXED4X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fixed4x4, &ColladaParserAutoGen15Private::_data__fixed4x4, &ColladaParserAutoGen15Private::_end__fixed4x4, &ColladaParserAutoGen15Private::_preBegin__fixed4x4, &ColladaParserAutoGen15Private::_preEnd__fixed4x4, &ColladaParserAutoGen15Private::_freeAttributes__fixed4x4);
functionMap[HASH_ELEMENT_FLOAT____FLOAT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float____float_type, &ColladaParserAutoGen15Private::_data__float____float_type, &ColladaParserAutoGen15Private::_end__float____float_type, &ColladaParserAutoGen15Private::_preBegin__float____float_type, &ColladaParserAutoGen15Private::_preEnd__float____float_type, &ColladaParserAutoGen15Private::_freeAttributes__float____float_type);
functionMap[HASH_ELEMENT_FLOAT2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2, &ColladaParserAutoGen15Private::_data__float2, &ColladaParserAutoGen15Private::_end__float2, &ColladaParserAutoGen15Private::_preBegin__float2, &ColladaParserAutoGen15Private::_preEnd__float2, &ColladaParserAutoGen15Private::_freeAttributes__float2);
functionMap[HASH_ELEMENT_FLOAT2X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2x1, &ColladaParserAutoGen15Private::_data__float2x1, &ColladaParserAutoGen15Private::_end__float2x1, &ColladaParserAutoGen15Private::_preBegin__float2x1, &ColladaParserAutoGen15Private::_preEnd__float2x1, &ColladaParserAutoGen15Private::_freeAttributes__float2x1);
functionMap[HASH_ELEMENT_FLOAT2X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2x2, &ColladaParserAutoGen15Private::_data__float2x2, &ColladaParserAutoGen15Private::_end__float2x2, &ColladaParserAutoGen15Private::_preBegin__float2x2, &ColladaParserAutoGen15Private::_preEnd__float2x2, &ColladaParserAutoGen15Private::_freeAttributes__float2x2);
functionMap[HASH_ELEMENT_FLOAT2X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2x3, &ColladaParserAutoGen15Private::_data__float2x3, &ColladaParserAutoGen15Private::_end__float2x3, &ColladaParserAutoGen15Private::_preBegin__float2x3, &ColladaParserAutoGen15Private::_preEnd__float2x3, &ColladaParserAutoGen15Private::_freeAttributes__float2x3);
functionMap[HASH_ELEMENT_FLOAT2X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2x4, &ColladaParserAutoGen15Private::_data__float2x4, &ColladaParserAutoGen15Private::_end__float2x4, &ColladaParserAutoGen15Private::_preBegin__float2x4, &ColladaParserAutoGen15Private::_preEnd__float2x4, &ColladaParserAutoGen15Private::_freeAttributes__float2x4);
functionMap[HASH_ELEMENT_FLOAT3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3, &ColladaParserAutoGen15Private::_data__float3, &ColladaParserAutoGen15Private::_end__float3, &ColladaParserAutoGen15Private::_preBegin__float3, &ColladaParserAutoGen15Private::_preEnd__float3, &ColladaParserAutoGen15Private::_freeAttributes__float3);
functionMap[HASH_ELEMENT_FLOAT3X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3x1, &ColladaParserAutoGen15Private::_data__float3x1, &ColladaParserAutoGen15Private::_end__float3x1, &ColladaParserAutoGen15Private::_preBegin__float3x1, &ColladaParserAutoGen15Private::_preEnd__float3x1, &ColladaParserAutoGen15Private::_freeAttributes__float3x1);
functionMap[HASH_ELEMENT_FLOAT3X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3x2, &ColladaParserAutoGen15Private::_data__float3x2, &ColladaParserAutoGen15Private::_end__float3x2, &ColladaParserAutoGen15Private::_preBegin__float3x2, &ColladaParserAutoGen15Private::_preEnd__float3x2, &ColladaParserAutoGen15Private::_freeAttributes__float3x2);
functionMap[HASH_ELEMENT_FLOAT3X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3x3, &ColladaParserAutoGen15Private::_data__float3x3, &ColladaParserAutoGen15Private::_end__float3x3, &ColladaParserAutoGen15Private::_preBegin__float3x3, &ColladaParserAutoGen15Private::_preEnd__float3x3, &ColladaParserAutoGen15Private::_freeAttributes__float3x3);
functionMap[HASH_ELEMENT_FLOAT3X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3x4, &ColladaParserAutoGen15Private::_data__float3x4, &ColladaParserAutoGen15Private::_end__float3x4, &ColladaParserAutoGen15Private::_preBegin__float3x4, &ColladaParserAutoGen15Private::_preEnd__float3x4, &ColladaParserAutoGen15Private::_freeAttributes__float3x4);
functionMap[HASH_ELEMENT_FLOAT4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4, &ColladaParserAutoGen15Private::_data__float4, &ColladaParserAutoGen15Private::_end__float4, &ColladaParserAutoGen15Private::_preBegin__float4, &ColladaParserAutoGen15Private::_preEnd__float4, &ColladaParserAutoGen15Private::_freeAttributes__float4);
functionMap[HASH_ELEMENT_FLOAT4X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4x1, &ColladaParserAutoGen15Private::_data__float4x1, &ColladaParserAutoGen15Private::_end__float4x1, &ColladaParserAutoGen15Private::_preBegin__float4x1, &ColladaParserAutoGen15Private::_preEnd__float4x1, &ColladaParserAutoGen15Private::_freeAttributes__float4x1);
functionMap[HASH_ELEMENT_FLOAT4X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4x2, &ColladaParserAutoGen15Private::_data__float4x2, &ColladaParserAutoGen15Private::_end__float4x2, &ColladaParserAutoGen15Private::_preBegin__float4x2, &ColladaParserAutoGen15Private::_preEnd__float4x2, &ColladaParserAutoGen15Private::_freeAttributes__float4x2);
functionMap[HASH_ELEMENT_FLOAT4X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4x3, &ColladaParserAutoGen15Private::_data__float4x3, &ColladaParserAutoGen15Private::_end__float4x3, &ColladaParserAutoGen15Private::_preBegin__float4x3, &ColladaParserAutoGen15Private::_preEnd__float4x3, &ColladaParserAutoGen15Private::_freeAttributes__float4x3);
functionMap[HASH_ELEMENT_FLOAT4X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4x4, &ColladaParserAutoGen15Private::_data__float4x4, &ColladaParserAutoGen15Private::_end__float4x4, &ColladaParserAutoGen15Private::_preBegin__float4x4, &ColladaParserAutoGen15Private::_preEnd__float4x4, &ColladaParserAutoGen15Private::_freeAttributes__float4x4);
functionMap[HASH_ELEMENT_HALF] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half, &ColladaParserAutoGen15Private::_data__half, &ColladaParserAutoGen15Private::_end__half, &ColladaParserAutoGen15Private::_preBegin__half, &ColladaParserAutoGen15Private::_preEnd__half, &ColladaParserAutoGen15Private::_freeAttributes__half);
functionMap[HASH_ELEMENT_HALF2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half2, &ColladaParserAutoGen15Private::_data__half2, &ColladaParserAutoGen15Private::_end__half2, &ColladaParserAutoGen15Private::_preBegin__half2, &ColladaParserAutoGen15Private::_preEnd__half2, &ColladaParserAutoGen15Private::_freeAttributes__half2);
functionMap[HASH_ELEMENT_HALF2X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half2x1, &ColladaParserAutoGen15Private::_data__half2x1, &ColladaParserAutoGen15Private::_end__half2x1, &ColladaParserAutoGen15Private::_preBegin__half2x1, &ColladaParserAutoGen15Private::_preEnd__half2x1, &ColladaParserAutoGen15Private::_freeAttributes__half2x1);
functionMap[HASH_ELEMENT_HALF2X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half2x2, &ColladaParserAutoGen15Private::_data__half2x2, &ColladaParserAutoGen15Private::_end__half2x2, &ColladaParserAutoGen15Private::_preBegin__half2x2, &ColladaParserAutoGen15Private::_preEnd__half2x2, &ColladaParserAutoGen15Private::_freeAttributes__half2x2);
functionMap[HASH_ELEMENT_HALF2X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half2x3, &ColladaParserAutoGen15Private::_data__half2x3, &ColladaParserAutoGen15Private::_end__half2x3, &ColladaParserAutoGen15Private::_preBegin__half2x3, &ColladaParserAutoGen15Private::_preEnd__half2x3, &ColladaParserAutoGen15Private::_freeAttributes__half2x3);
functionMap[HASH_ELEMENT_HALF2X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half2x4, &ColladaParserAutoGen15Private::_data__half2x4, &ColladaParserAutoGen15Private::_end__half2x4, &ColladaParserAutoGen15Private::_preBegin__half2x4, &ColladaParserAutoGen15Private::_preEnd__half2x4, &ColladaParserAutoGen15Private::_freeAttributes__half2x4);
functionMap[HASH_ELEMENT_HALF3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half3, &ColladaParserAutoGen15Private::_data__half3, &ColladaParserAutoGen15Private::_end__half3, &ColladaParserAutoGen15Private::_preBegin__half3, &ColladaParserAutoGen15Private::_preEnd__half3, &ColladaParserAutoGen15Private::_freeAttributes__half3);
functionMap[HASH_ELEMENT_HALF3X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half3x1, &ColladaParserAutoGen15Private::_data__half3x1, &ColladaParserAutoGen15Private::_end__half3x1, &ColladaParserAutoGen15Private::_preBegin__half3x1, &ColladaParserAutoGen15Private::_preEnd__half3x1, &ColladaParserAutoGen15Private::_freeAttributes__half3x1);
functionMap[HASH_ELEMENT_HALF3X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half3x2, &ColladaParserAutoGen15Private::_data__half3x2, &ColladaParserAutoGen15Private::_end__half3x2, &ColladaParserAutoGen15Private::_preBegin__half3x2, &ColladaParserAutoGen15Private::_preEnd__half3x2, &ColladaParserAutoGen15Private::_freeAttributes__half3x2);
functionMap[HASH_ELEMENT_HALF3X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half3x3, &ColladaParserAutoGen15Private::_data__half3x3, &ColladaParserAutoGen15Private::_end__half3x3, &ColladaParserAutoGen15Private::_preBegin__half3x3, &ColladaParserAutoGen15Private::_preEnd__half3x3, &ColladaParserAutoGen15Private::_freeAttributes__half3x3);
functionMap[HASH_ELEMENT_HALF3X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half3x4, &ColladaParserAutoGen15Private::_data__half3x4, &ColladaParserAutoGen15Private::_end__half3x4, &ColladaParserAutoGen15Private::_preBegin__half3x4, &ColladaParserAutoGen15Private::_preEnd__half3x4, &ColladaParserAutoGen15Private::_freeAttributes__half3x4);
functionMap[HASH_ELEMENT_HALF4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half4, &ColladaParserAutoGen15Private::_data__half4, &ColladaParserAutoGen15Private::_end__half4, &ColladaParserAutoGen15Private::_preBegin__half4, &ColladaParserAutoGen15Private::_preEnd__half4, &ColladaParserAutoGen15Private::_freeAttributes__half4);
functionMap[HASH_ELEMENT_HALF4X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half4x1, &ColladaParserAutoGen15Private::_data__half4x1, &ColladaParserAutoGen15Private::_end__half4x1, &ColladaParserAutoGen15Private::_preBegin__half4x1, &ColladaParserAutoGen15Private::_preEnd__half4x1, &ColladaParserAutoGen15Private::_freeAttributes__half4x1);
functionMap[HASH_ELEMENT_HALF4X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half4x2, &ColladaParserAutoGen15Private::_data__half4x2, &ColladaParserAutoGen15Private::_end__half4x2, &ColladaParserAutoGen15Private::_preBegin__half4x2, &ColladaParserAutoGen15Private::_preEnd__half4x2, &ColladaParserAutoGen15Private::_freeAttributes__half4x2);
functionMap[HASH_ELEMENT_HALF4X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half4x3, &ColladaParserAutoGen15Private::_data__half4x3, &ColladaParserAutoGen15Private::_end__half4x3, &ColladaParserAutoGen15Private::_preBegin__half4x3, &ColladaParserAutoGen15Private::_preEnd__half4x3, &ColladaParserAutoGen15Private::_freeAttributes__half4x3);
functionMap[HASH_ELEMENT_HALF4X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half4x4, &ColladaParserAutoGen15Private::_data__half4x4, &ColladaParserAutoGen15Private::_end__half4x4, &ColladaParserAutoGen15Private::_preBegin__half4x4, &ColladaParserAutoGen15Private::_preEnd__half4x4, &ColladaParserAutoGen15Private::_freeAttributes__half4x4);
functionMap[HASH_ELEMENT_INT____INT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int____int_type, &ColladaParserAutoGen15Private::_data__int____int_type, &ColladaParserAutoGen15Private::_end__int____int_type, &ColladaParserAutoGen15Private::_preBegin__int____int_type, &ColladaParserAutoGen15Private::_preEnd__int____int_type, &ColladaParserAutoGen15Private::_freeAttributes__int____int_type);
functionMap[HASH_ELEMENT_INT2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int2, &ColladaParserAutoGen15Private::_data__int2, &ColladaParserAutoGen15Private::_end__int2, &ColladaParserAutoGen15Private::_preBegin__int2, &ColladaParserAutoGen15Private::_preEnd__int2, &ColladaParserAutoGen15Private::_freeAttributes__int2);
functionMap[HASH_ELEMENT_INT2X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int2x1, &ColladaParserAutoGen15Private::_data__int2x1, &ColladaParserAutoGen15Private::_end__int2x1, &ColladaParserAutoGen15Private::_preBegin__int2x1, &ColladaParserAutoGen15Private::_preEnd__int2x1, &ColladaParserAutoGen15Private::_freeAttributes__int2x1);
functionMap[HASH_ELEMENT_INT2X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int2x2, &ColladaParserAutoGen15Private::_data__int2x2, &ColladaParserAutoGen15Private::_end__int2x2, &ColladaParserAutoGen15Private::_preBegin__int2x2, &ColladaParserAutoGen15Private::_preEnd__int2x2, &ColladaParserAutoGen15Private::_freeAttributes__int2x2);
functionMap[HASH_ELEMENT_INT2X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int2x3, &ColladaParserAutoGen15Private::_data__int2x3, &ColladaParserAutoGen15Private::_end__int2x3, &ColladaParserAutoGen15Private::_preBegin__int2x3, &ColladaParserAutoGen15Private::_preEnd__int2x3, &ColladaParserAutoGen15Private::_freeAttributes__int2x3);
functionMap[HASH_ELEMENT_INT2X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int2x4, &ColladaParserAutoGen15Private::_data__int2x4, &ColladaParserAutoGen15Private::_end__int2x4, &ColladaParserAutoGen15Private::_preBegin__int2x4, &ColladaParserAutoGen15Private::_preEnd__int2x4, &ColladaParserAutoGen15Private::_freeAttributes__int2x4);
functionMap[HASH_ELEMENT_INT3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int3, &ColladaParserAutoGen15Private::_data__int3, &ColladaParserAutoGen15Private::_end__int3, &ColladaParserAutoGen15Private::_preBegin__int3, &ColladaParserAutoGen15Private::_preEnd__int3, &ColladaParserAutoGen15Private::_freeAttributes__int3);
functionMap[HASH_ELEMENT_INT3X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int3x1, &ColladaParserAutoGen15Private::_data__int3x1, &ColladaParserAutoGen15Private::_end__int3x1, &ColladaParserAutoGen15Private::_preBegin__int3x1, &ColladaParserAutoGen15Private::_preEnd__int3x1, &ColladaParserAutoGen15Private::_freeAttributes__int3x1);
functionMap[HASH_ELEMENT_INT3X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int3x2, &ColladaParserAutoGen15Private::_data__int3x2, &ColladaParserAutoGen15Private::_end__int3x2, &ColladaParserAutoGen15Private::_preBegin__int3x2, &ColladaParserAutoGen15Private::_preEnd__int3x2, &ColladaParserAutoGen15Private::_freeAttributes__int3x2);
functionMap[HASH_ELEMENT_INT3X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int3x3, &ColladaParserAutoGen15Private::_data__int3x3, &ColladaParserAutoGen15Private::_end__int3x3, &ColladaParserAutoGen15Private::_preBegin__int3x3, &ColladaParserAutoGen15Private::_preEnd__int3x3, &ColladaParserAutoGen15Private::_freeAttributes__int3x3);
functionMap[HASH_ELEMENT_INT3X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int3x4, &ColladaParserAutoGen15Private::_data__int3x4, &ColladaParserAutoGen15Private::_end__int3x4, &ColladaParserAutoGen15Private::_preBegin__int3x4, &ColladaParserAutoGen15Private::_preEnd__int3x4, &ColladaParserAutoGen15Private::_freeAttributes__int3x4);
functionMap[HASH_ELEMENT_INT4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int4, &ColladaParserAutoGen15Private::_data__int4, &ColladaParserAutoGen15Private::_end__int4, &ColladaParserAutoGen15Private::_preBegin__int4, &ColladaParserAutoGen15Private::_preEnd__int4, &ColladaParserAutoGen15Private::_freeAttributes__int4);
functionMap[HASH_ELEMENT_INT4X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int4x1, &ColladaParserAutoGen15Private::_data__int4x1, &ColladaParserAutoGen15Private::_end__int4x1, &ColladaParserAutoGen15Private::_preBegin__int4x1, &ColladaParserAutoGen15Private::_preEnd__int4x1, &ColladaParserAutoGen15Private::_freeAttributes__int4x1);
functionMap[HASH_ELEMENT_INT4X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int4x2, &ColladaParserAutoGen15Private::_data__int4x2, &ColladaParserAutoGen15Private::_end__int4x2, &ColladaParserAutoGen15Private::_preBegin__int4x2, &ColladaParserAutoGen15Private::_preEnd__int4x2, &ColladaParserAutoGen15Private::_freeAttributes__int4x2);
functionMap[HASH_ELEMENT_INT4X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int4x3, &ColladaParserAutoGen15Private::_data__int4x3, &ColladaParserAutoGen15Private::_end__int4x3, &ColladaParserAutoGen15Private::_preBegin__int4x3, &ColladaParserAutoGen15Private::_preEnd__int4x3, &ColladaParserAutoGen15Private::_freeAttributes__int4x3);
functionMap[HASH_ELEMENT_INT4X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int4x4, &ColladaParserAutoGen15Private::_data__int4x4, &ColladaParserAutoGen15Private::_end__int4x4, &ColladaParserAutoGen15Private::_preBegin__int4x4, &ColladaParserAutoGen15Private::_preEnd__int4x4, &ColladaParserAutoGen15Private::_freeAttributes__int4x4);
functionMap[HASH_ELEMENT_SAMPLER1D] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler1D, &ColladaParserAutoGen15Private::_data__sampler1D, &ColladaParserAutoGen15Private::_end__sampler1D, &ColladaParserAutoGen15Private::_preBegin__sampler1D, &ColladaParserAutoGen15Private::_preEnd__sampler1D, &ColladaParserAutoGen15Private::_freeAttributes__sampler1D);
functionMap[HASH_ELEMENT_SAMPLER2D____FX_SAMPLER2D_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_data__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_end__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_preBegin__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_preEnd__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_freeAttributes__sampler2D____fx_sampler2D_type);
functionMap[HASH_ELEMENT_SAMPLER3D] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler3D, &ColladaParserAutoGen15Private::_data__sampler3D, &ColladaParserAutoGen15Private::_end__sampler3D, &ColladaParserAutoGen15Private::_preBegin__sampler3D, &ColladaParserAutoGen15Private::_preEnd__sampler3D, &ColladaParserAutoGen15Private::_freeAttributes__sampler3D);
functionMap[HASH_ELEMENT_SAMPLERCUBE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__samplerCUBE, &ColladaParserAutoGen15Private::_data__samplerCUBE, &ColladaParserAutoGen15Private::_end__samplerCUBE, &ColladaParserAutoGen15Private::_preBegin__samplerCUBE, &ColladaParserAutoGen15Private::_preEnd__samplerCUBE, &ColladaParserAutoGen15Private::_freeAttributes__samplerCUBE);
functionMap[HASH_ELEMENT_SAMPLERDEPTH] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__samplerDEPTH, &ColladaParserAutoGen15Private::_data__samplerDEPTH, &ColladaParserAutoGen15Private::_end__samplerDEPTH, &ColladaParserAutoGen15Private::_preBegin__samplerDEPTH, &ColladaParserAutoGen15Private::_preEnd__samplerDEPTH, &ColladaParserAutoGen15Private::_freeAttributes__samplerDEPTH);
functionMap[HASH_ELEMENT_SAMPLERRECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__samplerRECT, &ColladaParserAutoGen15Private::_data__samplerRECT, &ColladaParserAutoGen15Private::_end__samplerRECT, &ColladaParserAutoGen15Private::_preBegin__samplerRECT, &ColladaParserAutoGen15Private::_preEnd__samplerRECT, &ColladaParserAutoGen15Private::_freeAttributes__samplerRECT);
functionMap[HASH_ELEMENT_STRING] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__string, &ColladaParserAutoGen15Private::_data__string, &ColladaParserAutoGen15Private::_end__string, &ColladaParserAutoGen15Private::_preBegin__string, &ColladaParserAutoGen15Private::_preEnd__string, &ColladaParserAutoGen15Private::_freeAttributes__string);
functionMap[HASH_ELEMENT_USERTYPE____CG_USER_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__usertype____cg_user_type, &ColladaParserAutoGen15Private::_data__usertype____cg_user_type, &ColladaParserAutoGen15Private::_end__usertype____cg_user_type, &ColladaParserAutoGen15Private::_preBegin__usertype____cg_user_type, &ColladaParserAutoGen15Private::_preEnd__usertype____cg_user_type, &ColladaParserAutoGen15Private::_freeAttributes__usertype____cg_user_type);
functionMap[HASH_ELEMENT_SETPARAM____CG_SETPARAM_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__setparam____cg_setparam_type, &ColladaParserAutoGen15Private::_data__setparam____cg_setparam_type, &ColladaParserAutoGen15Private::_end__setparam____cg_setparam_type, &ColladaParserAutoGen15Private::_preBegin__setparam____cg_setparam_type, &ColladaParserAutoGen15Private::_preEnd__setparam____cg_setparam_type, &ColladaParserAutoGen15Private::_freeAttributes__setparam____cg_setparam_type);
functionMap[HASH_ELEMENT_ANNOTATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__annotate, &ColladaParserAutoGen15Private::_data__annotate, &ColladaParserAutoGen15Private::_end__annotate, &ColladaParserAutoGen15Private::_preBegin__annotate, &ColladaParserAutoGen15Private::_preEnd__annotate, &ColladaParserAutoGen15Private::_freeAttributes__annotate);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_PASS____CG_PASS_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__pass____cg_pass_type, &ColladaParserAutoGen15Private::_data__pass____cg_pass_type, &ColladaParserAutoGen15Private::_end__pass____cg_pass_type, &ColladaParserAutoGen15Private::_preBegin__pass____cg_pass_type, &ColladaParserAutoGen15Private::_preEnd__pass____cg_pass_type, &ColladaParserAutoGen15Private::_freeAttributes__pass____cg_pass_type);
functionMap[HASH_ELEMENT_ANNOTATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__annotate, &ColladaParserAutoGen15Private::_data__annotate, &ColladaParserAutoGen15Private::_end__annotate, &ColladaParserAutoGen15Private::_preBegin__annotate, &ColladaParserAutoGen15Private::_preEnd__annotate, &ColladaParserAutoGen15Private::_freeAttributes__annotate);
functionMap[HASH_ELEMENT_CG_PASS_TYPE____EVALUATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__cg_pass_type____evaluate, &ColladaParserAutoGen15Private::_data__cg_pass_type____evaluate, &ColladaParserAutoGen15Private::_end__cg_pass_type____evaluate, &ColladaParserAutoGen15Private::_preBegin__cg_pass_type____evaluate, &ColladaParserAutoGen15Private::_preEnd__cg_pass_type____evaluate, &ColladaParserAutoGen15Private::_freeAttributes__cg_pass_type____evaluate);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_CG_PASS_TYPE____PROGRAM] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__cg_pass_type____program, &ColladaParserAutoGen15Private::_data__cg_pass_type____program, &ColladaParserAutoGen15Private::_end__cg_pass_type____program, &ColladaParserAutoGen15Private::_preBegin__cg_pass_type____program, &ColladaParserAutoGen15Private::_preEnd__cg_pass_type____program, &ColladaParserAutoGen15Private::_freeAttributes__cg_pass_type____program);
functionMap[HASH_ELEMENT_CG_PASS_TYPE____STATES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__cg_pass_type____states, &ColladaParserAutoGen15Private::_data__cg_pass_type____states, &ColladaParserAutoGen15Private::_end__cg_pass_type____states, &ColladaParserAutoGen15Private::_preBegin__cg_pass_type____states, &ColladaParserAutoGen15Private::_preEnd__cg_pass_type____states, &ColladaParserAutoGen15Private::_freeAttributes__cg_pass_type____states);
functionMap[HASH_ELEMENT_COLOR_CLEAR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__color_clear, &ColladaParserAutoGen15Private::_data__color_clear, &ColladaParserAutoGen15Private::_end__color_clear, &ColladaParserAutoGen15Private::_preBegin__color_clear, &ColladaParserAutoGen15Private::_preEnd__color_clear, &ColladaParserAutoGen15Private::_freeAttributes__color_clear);
functionMap[HASH_ELEMENT_COLOR_TARGET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__color_target, &ColladaParserAutoGen15Private::_data__color_target, &ColladaParserAutoGen15Private::_end__color_target, &ColladaParserAutoGen15Private::_preBegin__color_target, &ColladaParserAutoGen15Private::_preEnd__color_target, &ColladaParserAutoGen15Private::_freeAttributes__color_target);
functionMap[HASH_ELEMENT_DEPTH_CLEAR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__depth_clear, &ColladaParserAutoGen15Private::_data__depth_clear, &ColladaParserAutoGen15Private::_end__depth_clear, &ColladaParserAutoGen15Private::_preBegin__depth_clear, &ColladaParserAutoGen15Private::_preEnd__depth_clear, &ColladaParserAutoGen15Private::_freeAttributes__depth_clear);
functionMap[HASH_ELEMENT_DEPTH_TARGET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__depth_target, &ColladaParserAutoGen15Private::_data__depth_target, &ColladaParserAutoGen15Private::_end__depth_target, &ColladaParserAutoGen15Private::_preBegin__depth_target, &ColladaParserAutoGen15Private::_preEnd__depth_target, &ColladaParserAutoGen15Private::_freeAttributes__depth_target);
functionMap[HASH_ELEMENT_DRAW] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__draw, &ColladaParserAutoGen15Private::_data__draw, &ColladaParserAutoGen15Private::_end__draw, &ColladaParserAutoGen15Private::_preBegin__draw, &ColladaParserAutoGen15Private::_preEnd__draw, &ColladaParserAutoGen15Private::_freeAttributes__draw);
functionMap[HASH_ELEMENT_STENCIL_CLEAR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__stencil_clear, &ColladaParserAutoGen15Private::_data__stencil_clear, &ColladaParserAutoGen15Private::_end__stencil_clear, &ColladaParserAutoGen15Private::_preBegin__stencil_clear, &ColladaParserAutoGen15Private::_preEnd__stencil_clear, &ColladaParserAutoGen15Private::_freeAttributes__stencil_clear);
functionMap[HASH_ELEMENT_STENCIL_TARGET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__stencil_target, &ColladaParserAutoGen15Private::_data__stencil_target, &ColladaParserAutoGen15Private::_end__stencil_target, &ColladaParserAutoGen15Private::_preBegin__stencil_target, &ColladaParserAutoGen15Private::_preEnd__stencil_target, &ColladaParserAutoGen15Private::_freeAttributes__stencil_target);
functionMap[HASH_ELEMENT_INSTANCE_IMAGE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_image, &ColladaParserAutoGen15Private::_data__instance_image, &ColladaParserAutoGen15Private::_end__instance_image, &ColladaParserAutoGen15Private::_preBegin__instance_image, &ColladaParserAutoGen15Private::_preEnd__instance_image, &ColladaParserAutoGen15Private::_freeAttributes__instance_image);
functionMap[HASH_ELEMENT_FX_COLORTARGET_TYPE____PARAM] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fx_colortarget_type____param, &ColladaParserAutoGen15Private::_data__fx_colortarget_type____param, &ColladaParserAutoGen15Private::_end__fx_colortarget_type____param, &ColladaParserAutoGen15Private::_preBegin__fx_colortarget_type____param, &ColladaParserAutoGen15Private::_preEnd__fx_colortarget_type____param, &ColladaParserAutoGen15Private::_freeAttributes__fx_colortarget_type____param);
functionMap[HASH_ELEMENT_PROFILE_CG__TECHNIQUE__PASS__PROGRAM__SHADER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_CG__technique__pass__program__shader, &ColladaParserAutoGen15Private::_data__profile_CG__technique__pass__program__shader, &ColladaParserAutoGen15Private::_end__profile_CG__technique__pass__program__shader, &ColladaParserAutoGen15Private::_preBegin__profile_CG__technique__pass__program__shader, &ColladaParserAutoGen15Private::_preEnd__profile_CG__technique__pass__program__shader, &ColladaParserAutoGen15Private::_freeAttributes__profile_CG__technique__pass__program__shader);
functionMap[HASH_ELEMENT_PROFILE_CG__TECHNIQUE__PASS__PROGRAM__SHADER__BIND_UNIFORM] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_CG__technique__pass__program__shader__bind_uniform, &ColladaParserAutoGen15Private::_data__profile_CG__technique__pass__program__shader__bind_uniform, &ColladaParserAutoGen15Private::_end__profile_CG__technique__pass__program__shader__bind_uniform, &ColladaParserAutoGen15Private::_preBegin__profile_CG__technique__pass__program__shader__bind_uniform, &ColladaParserAutoGen15Private::_preEnd__profile_CG__technique__pass__program__shader__bind_uniform, &ColladaParserAutoGen15Private::_freeAttributes__profile_CG__technique__pass__program__shader__bind_uniform);
functionMap[HASH_ELEMENT_COMPILER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__compiler, &ColladaParserAutoGen15Private::_data__compiler, &ColladaParserAutoGen15Private::_end__compiler, &ColladaParserAutoGen15Private::_preBegin__compiler, &ColladaParserAutoGen15Private::_preEnd__compiler, &ColladaParserAutoGen15Private::_freeAttributes__compiler);
functionMap[HASH_ELEMENT_PROFILE_CG__TECHNIQUE__PASS__PROGRAM__SHADER__SOURCES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_CG__technique__pass__program__shader__sources, &ColladaParserAutoGen15Private::_data__profile_CG__technique__pass__program__shader__sources, &ColladaParserAutoGen15Private::_end__profile_CG__technique__pass__program__shader__sources, &ColladaParserAutoGen15Private::_preBegin__profile_CG__technique__pass__program__shader__sources, &ColladaParserAutoGen15Private::_preEnd__profile_CG__technique__pass__program__shader__sources, &ColladaParserAutoGen15Private::_freeAttributes__profile_CG__technique__pass__program__shader__sources);
functionMap[HASH_ELEMENT_PROFILE_CG__TECHNIQUE__PASS__PROGRAM__SHADER__BIND_UNIFORM__PARAM] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_CG__technique__pass__program__shader__bind_uniform__param, &ColladaParserAutoGen15Private::_data__profile_CG__technique__pass__program__shader__bind_uniform__param, &ColladaParserAutoGen15Private::_end__profile_CG__technique__pass__program__shader__bind_uniform__param, &ColladaParserAutoGen15Private::_preBegin__profile_CG__technique__pass__program__shader__bind_uniform__param, &ColladaParserAutoGen15Private::_preEnd__profile_CG__technique__pass__program__shader__bind_uniform__param, &ColladaParserAutoGen15Private::_freeAttributes__profile_CG__technique__pass__program__shader__bind_uniform__param);
functionMap[HASH_ELEMENT_BINARY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__binary, &ColladaParserAutoGen15Private::_data__binary, &ColladaParserAutoGen15Private::_end__binary, &ColladaParserAutoGen15Private::_preBegin__binary, &ColladaParserAutoGen15Private::_preEnd__binary, &ColladaParserAutoGen15Private::_freeAttributes__binary);
functionMap[HASH_ELEMENT_BINARY__HEX] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__binary__hex, &ColladaParserAutoGen15Private::_data__binary__hex, &ColladaParserAutoGen15Private::_end__binary__hex, &ColladaParserAutoGen15Private::_preBegin__binary__hex, &ColladaParserAutoGen15Private::_preEnd__binary__hex, &ColladaParserAutoGen15Private::_freeAttributes__binary__hex);
functionMap[HASH_ELEMENT_REF____ANYURI] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__ref____anyURI, &ColladaParserAutoGen15Private::_data__ref____anyURI, &ColladaParserAutoGen15Private::_end__ref____anyURI, &ColladaParserAutoGen15Private::_preBegin__ref____anyURI, &ColladaParserAutoGen15Private::_preEnd__ref____anyURI, &ColladaParserAutoGen15Private::_freeAttributes__ref____anyURI);
functionMap[HASH_ELEMENT_IMPORT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__import, &ColladaParserAutoGen15Private::_data__import, &ColladaParserAutoGen15Private::_end__import, &ColladaParserAutoGen15Private::_preBegin__import, &ColladaParserAutoGen15Private::_preEnd__import, &ColladaParserAutoGen15Private::_freeAttributes__import);
functionMap[HASH_ELEMENT_INLINE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__inline, &ColladaParserAutoGen15Private::_data__inline, &ColladaParserAutoGen15Private::_end__inline, &ColladaParserAutoGen15Private::_preBegin__inline, &ColladaParserAutoGen15Private::_preEnd__inline, &ColladaParserAutoGen15Private::_freeAttributes__inline);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__ALPHA_FUNC] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__alpha_func, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__alpha_func, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__alpha_func, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__alpha_func, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__alpha_func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__alpha_func);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__ALPHA_TEST_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__alpha_test_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__alpha_test_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__alpha_test_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__alpha_test_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__alpha_test_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__alpha_test_enable);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_COLOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__blend_color, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__blend_color, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__blend_color, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__blend_color, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__blend_color, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__blend_color);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__blend_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__blend_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__blend_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__blend_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__blend_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__blend_enable);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_EQUATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__blend_equation, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__blend_equation, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__blend_equation, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__blend_equation, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__blend_equation, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__blend_equation);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_EQUATION_SEPARATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__blend_equation_separate, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__blend_equation_separate, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__blend_equation_separate, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__blend_equation_separate, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__blend_equation_separate, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__blend_equation_separate);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_FUNC] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__blend_func, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__blend_func, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__blend_func, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__blend_func, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__blend_func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__blend_func);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_FUNC_SEPARATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__blend_func_separate, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__blend_func_separate, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__blend_func_separate, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__blend_func_separate, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__blend_func_separate, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__blend_func_separate);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__CLIP_PLANE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__clip_plane, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__clip_plane, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__clip_plane, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__clip_plane, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__clip_plane, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__clip_plane);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__CLIP_PLANE_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__clip_plane_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__clip_plane_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__clip_plane_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__clip_plane_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__clip_plane_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__clip_plane_enable);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__COLOR_LOGIC_OP_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__color_logic_op_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__color_logic_op_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__color_logic_op_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__color_logic_op_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__color_logic_op_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__color_logic_op_enable);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__COLOR_MASK] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__color_mask, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__color_mask, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__color_mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__color_mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__color_mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__color_mask);
functionMap[HASH_ELEMENT_COLOR_MATERIAL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__color_material, &ColladaParserAutoGen15Private::_data__color_material, &ColladaParserAutoGen15Private::_end__color_material, &ColladaParserAutoGen15Private::_preBegin__color_material, &ColladaParserAutoGen15Private::_preEnd__color_material, &ColladaParserAutoGen15Private::_freeAttributes__color_material);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__COLOR_MATERIAL_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__color_material_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__color_material_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__color_material_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__color_material_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__color_material_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__color_material_enable);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__CULL_FACE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__cull_face, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__cull_face, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__cull_face, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__cull_face, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__cull_face, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__cull_face);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__CULL_FACE_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__cull_face_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__cull_face_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__cull_face_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__cull_face_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__cull_face_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__cull_face_enable);
functionMap[HASH_ELEMENT_DEPTH_BOUNDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__depth_bounds, &ColladaParserAutoGen15Private::_data__depth_bounds, &ColladaParserAutoGen15Private::_end__depth_bounds, &ColladaParserAutoGen15Private::_preBegin__depth_bounds, &ColladaParserAutoGen15Private::_preEnd__depth_bounds, &ColladaParserAutoGen15Private::_freeAttributes__depth_bounds);
functionMap[HASH_ELEMENT_DEPTH_BOUNDS_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__depth_bounds_enable, &ColladaParserAutoGen15Private::_data__depth_bounds_enable, &ColladaParserAutoGen15Private::_end__depth_bounds_enable, &ColladaParserAutoGen15Private::_preBegin__depth_bounds_enable, &ColladaParserAutoGen15Private::_preEnd__depth_bounds_enable, &ColladaParserAutoGen15Private::_freeAttributes__depth_bounds_enable);
functionMap[HASH_ELEMENT_DEPTH_CLAMP_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__depth_clamp_enable, &ColladaParserAutoGen15Private::_data__depth_clamp_enable, &ColladaParserAutoGen15Private::_end__depth_clamp_enable, &ColladaParserAutoGen15Private::_preBegin__depth_clamp_enable, &ColladaParserAutoGen15Private::_preEnd__depth_clamp_enable, &ColladaParserAutoGen15Private::_freeAttributes__depth_clamp_enable);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__DEPTH_FUNC] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__depth_func, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__depth_func, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__depth_func, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__depth_func, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__depth_func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__depth_func);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__DEPTH_MASK] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__depth_mask, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__depth_mask, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__depth_mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__depth_mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__depth_mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__depth_mask);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__DEPTH_RANGE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__depth_range, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__depth_range, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__depth_range, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__depth_range, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__depth_range, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__depth_range);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__DEPTH_TEST_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__depth_test_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__depth_test_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__depth_test_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__depth_test_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__depth_test_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__depth_test_enable);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__DITHER_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__dither_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__dither_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__dither_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__dither_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__dither_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__dither_enable);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__FOG_COLOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__fog_color, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__fog_color, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__fog_color, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__fog_color, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__fog_color, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__fog_color);
functionMap[HASH_ELEMENT_FOG_COORD_SRC] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fog_coord_src, &ColladaParserAutoGen15Private::_data__fog_coord_src, &ColladaParserAutoGen15Private::_end__fog_coord_src, &ColladaParserAutoGen15Private::_preBegin__fog_coord_src, &ColladaParserAutoGen15Private::_preEnd__fog_coord_src, &ColladaParserAutoGen15Private::_freeAttributes__fog_coord_src);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__FOG_DENSITY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__fog_density, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__fog_density, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__fog_density, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__fog_density, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__fog_density, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__fog_density);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__FOG_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__fog_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__fog_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__fog_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__fog_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__fog_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__fog_enable);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__FOG_END] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__fog_end, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__fog_end, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__fog_end, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__fog_end, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__fog_end, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__fog_end);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__FOG_MODE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__fog_mode, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__fog_mode, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__fog_mode, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__fog_mode, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__fog_mode, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__fog_mode);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__FOG_START] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__fog_start, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__fog_start, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__fog_start, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__fog_start, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__fog_start, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__fog_start);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__FRONT_FACE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__front_face, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__front_face, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__front_face, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__front_face, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__front_face, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__front_face);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_AMBIENT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__light_ambient, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__light_ambient, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__light_ambient, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__light_ambient, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__light_ambient, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__light_ambient);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_CONSTANT_ATTENUATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__light_constant_attenuation, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__light_constant_attenuation, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__light_constant_attenuation, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__light_constant_attenuation, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__light_constant_attenuation, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__light_constant_attenuation);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_DIFFUSE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__light_diffuse, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__light_diffuse, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__light_diffuse, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__light_diffuse, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__light_diffuse, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__light_diffuse);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__light_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__light_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__light_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__light_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__light_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__light_enable);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_LINEAR_ATTENUATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__light_linear_attenuation, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__light_linear_attenuation, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__light_linear_attenuation, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__light_linear_attenuation, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__light_linear_attenuation, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__light_linear_attenuation);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_MODEL_AMBIENT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__light_model_ambient, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__light_model_ambient, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__light_model_ambient, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__light_model_ambient, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__light_model_ambient, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__light_model_ambient);
functionMap[HASH_ELEMENT_LIGHT_MODEL_COLOR_CONTROL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__light_model_color_control, &ColladaParserAutoGen15Private::_data__light_model_color_control, &ColladaParserAutoGen15Private::_end__light_model_color_control, &ColladaParserAutoGen15Private::_preBegin__light_model_color_control, &ColladaParserAutoGen15Private::_preEnd__light_model_color_control, &ColladaParserAutoGen15Private::_freeAttributes__light_model_color_control);
functionMap[HASH_ELEMENT_LIGHT_MODEL_LOCAL_VIEWER_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__light_model_local_viewer_enable, &ColladaParserAutoGen15Private::_data__light_model_local_viewer_enable, &ColladaParserAutoGen15Private::_end__light_model_local_viewer_enable, &ColladaParserAutoGen15Private::_preBegin__light_model_local_viewer_enable, &ColladaParserAutoGen15Private::_preEnd__light_model_local_viewer_enable, &ColladaParserAutoGen15Private::_freeAttributes__light_model_local_viewer_enable);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_MODEL_TWO_SIDE_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__light_model_two_side_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__light_model_two_side_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__light_model_two_side_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__light_model_two_side_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__light_model_two_side_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__light_model_two_side_enable);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_POSITION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__light_position, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__light_position, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__light_position, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__light_position, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__light_position, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__light_position);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_QUADRATIC_ATTENUATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__light_quadratic_attenuation, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__light_quadratic_attenuation, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__light_quadratic_attenuation, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__light_quadratic_attenuation, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__light_quadratic_attenuation, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__light_quadratic_attenuation);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_SPECULAR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__light_specular, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__light_specular, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__light_specular, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__light_specular, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__light_specular, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__light_specular);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_SPOT_CUTOFF] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__light_spot_cutoff, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__light_spot_cutoff, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__light_spot_cutoff, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__light_spot_cutoff, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__light_spot_cutoff, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__light_spot_cutoff);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_SPOT_DIRECTION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__light_spot_direction, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__light_spot_direction, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__light_spot_direction, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__light_spot_direction, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__light_spot_direction, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__light_spot_direction);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHT_SPOT_EXPONENT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__light_spot_exponent, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__light_spot_exponent, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__light_spot_exponent, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__light_spot_exponent, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__light_spot_exponent, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__light_spot_exponent);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LIGHTING_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__lighting_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__lighting_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__lighting_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__lighting_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__lighting_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__lighting_enable);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LINE_SMOOTH_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__line_smooth_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__line_smooth_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__line_smooth_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__line_smooth_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__line_smooth_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__line_smooth_enable);
functionMap[HASH_ELEMENT_LINE_STIPPLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__line_stipple, &ColladaParserAutoGen15Private::_data__line_stipple, &ColladaParserAutoGen15Private::_end__line_stipple, &ColladaParserAutoGen15Private::_preBegin__line_stipple, &ColladaParserAutoGen15Private::_preEnd__line_stipple, &ColladaParserAutoGen15Private::_freeAttributes__line_stipple);
functionMap[HASH_ELEMENT_LINE_STIPPLE_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__line_stipple_enable, &ColladaParserAutoGen15Private::_data__line_stipple_enable, &ColladaParserAutoGen15Private::_end__line_stipple_enable, &ColladaParserAutoGen15Private::_preBegin__line_stipple_enable, &ColladaParserAutoGen15Private::_preEnd__line_stipple_enable, &ColladaParserAutoGen15Private::_freeAttributes__line_stipple_enable);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LINE_WIDTH] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__line_width, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__line_width, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__line_width, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__line_width, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__line_width, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__line_width);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__LOGIC_OP] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__logic_op, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__logic_op, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__logic_op, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__logic_op, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__logic_op, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__logic_op);
functionMap[HASH_ELEMENT_LOGIC_OP_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__logic_op_enable, &ColladaParserAutoGen15Private::_data__logic_op_enable, &ColladaParserAutoGen15Private::_end__logic_op_enable, &ColladaParserAutoGen15Private::_preBegin__logic_op_enable, &ColladaParserAutoGen15Private::_preEnd__logic_op_enable, &ColladaParserAutoGen15Private::_freeAttributes__logic_op_enable);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__MATERIAL_AMBIENT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__material_ambient, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__material_ambient, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__material_ambient, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__material_ambient, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__material_ambient, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__material_ambient);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__MATERIAL_DIFFUSE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__material_diffuse, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__material_diffuse, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__material_diffuse, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__material_diffuse, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__material_diffuse, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__material_diffuse);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__MATERIAL_EMISSION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__material_emission, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__material_emission, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__material_emission, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__material_emission, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__material_emission, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__material_emission);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__MATERIAL_SHININESS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__material_shininess, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__material_shininess, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__material_shininess, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__material_shininess, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__material_shininess, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__material_shininess);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__MATERIAL_SPECULAR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__material_specular, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__material_specular, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__material_specular, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__material_specular, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__material_specular, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__material_specular);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__MODEL_VIEW_MATRIX] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__model_view_matrix, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__model_view_matrix, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__model_view_matrix, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__model_view_matrix, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__model_view_matrix, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__model_view_matrix);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__MULTISAMPLE_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__multisample_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__multisample_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__multisample_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__multisample_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__multisample_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__multisample_enable);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__NORMALIZE_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__normalize_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__normalize_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__normalize_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__normalize_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__normalize_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__normalize_enable);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__POINT_DISTANCE_ATTENUATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__point_distance_attenuation, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__point_distance_attenuation, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__point_distance_attenuation, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__point_distance_attenuation, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__point_distance_attenuation, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__point_distance_attenuation);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__POINT_FADE_THRESHOLD_SIZE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__point_fade_threshold_size, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__point_fade_threshold_size, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__point_fade_threshold_size, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__point_fade_threshold_size, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__point_fade_threshold_size, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__point_fade_threshold_size);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__POINT_SIZE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__point_size, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__point_size, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__point_size, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__point_size, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__point_size, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__point_size);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__POINT_SIZE_MAX] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__point_size_max, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__point_size_max, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__point_size_max, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__point_size_max, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__point_size_max, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__point_size_max);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__POINT_SIZE_MIN] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__point_size_min, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__point_size_min, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__point_size_min, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__point_size_min, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__point_size_min, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__point_size_min);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__POINT_SMOOTH_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__point_smooth_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__point_smooth_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__point_smooth_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__point_smooth_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__point_smooth_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__point_smooth_enable);
functionMap[HASH_ELEMENT_POLYGON_MODE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__polygon_mode, &ColladaParserAutoGen15Private::_data__polygon_mode, &ColladaParserAutoGen15Private::_end__polygon_mode, &ColladaParserAutoGen15Private::_preBegin__polygon_mode, &ColladaParserAutoGen15Private::_preEnd__polygon_mode, &ColladaParserAutoGen15Private::_freeAttributes__polygon_mode);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__POLYGON_OFFSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__polygon_offset, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__polygon_offset, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__polygon_offset, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__polygon_offset, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__polygon_offset, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__polygon_offset);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__POLYGON_OFFSET_FILL_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__polygon_offset_fill_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__polygon_offset_fill_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__polygon_offset_fill_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__polygon_offset_fill_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__polygon_offset_fill_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__polygon_offset_fill_enable);
functionMap[HASH_ELEMENT_POLYGON_OFFSET_LINE_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__polygon_offset_line_enable, &ColladaParserAutoGen15Private::_data__polygon_offset_line_enable, &ColladaParserAutoGen15Private::_end__polygon_offset_line_enable, &ColladaParserAutoGen15Private::_preBegin__polygon_offset_line_enable, &ColladaParserAutoGen15Private::_preEnd__polygon_offset_line_enable, &ColladaParserAutoGen15Private::_freeAttributes__polygon_offset_line_enable);
functionMap[HASH_ELEMENT_POLYGON_OFFSET_POINT_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__polygon_offset_point_enable, &ColladaParserAutoGen15Private::_data__polygon_offset_point_enable, &ColladaParserAutoGen15Private::_end__polygon_offset_point_enable, &ColladaParserAutoGen15Private::_preBegin__polygon_offset_point_enable, &ColladaParserAutoGen15Private::_preEnd__polygon_offset_point_enable, &ColladaParserAutoGen15Private::_freeAttributes__polygon_offset_point_enable);
functionMap[HASH_ELEMENT_POLYGON_SMOOTH_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__polygon_smooth_enable, &ColladaParserAutoGen15Private::_data__polygon_smooth_enable, &ColladaParserAutoGen15Private::_end__polygon_smooth_enable, &ColladaParserAutoGen15Private::_preBegin__polygon_smooth_enable, &ColladaParserAutoGen15Private::_preEnd__polygon_smooth_enable, &ColladaParserAutoGen15Private::_freeAttributes__polygon_smooth_enable);
functionMap[HASH_ELEMENT_POLYGON_STIPPLE_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__polygon_stipple_enable, &ColladaParserAutoGen15Private::_data__polygon_stipple_enable, &ColladaParserAutoGen15Private::_end__polygon_stipple_enable, &ColladaParserAutoGen15Private::_preBegin__polygon_stipple_enable, &ColladaParserAutoGen15Private::_preEnd__polygon_stipple_enable, &ColladaParserAutoGen15Private::_freeAttributes__polygon_stipple_enable);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__PROJECTION_MATRIX] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__projection_matrix, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__projection_matrix, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__projection_matrix, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__projection_matrix, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__projection_matrix, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__projection_matrix);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__RESCALE_NORMAL_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__rescale_normal_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__rescale_normal_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__rescale_normal_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__rescale_normal_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__rescale_normal_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__rescale_normal_enable);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__SAMPLE_ALPHA_TO_COVERAGE_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__sample_alpha_to_coverage_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__sample_alpha_to_coverage_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__sample_alpha_to_coverage_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__sample_alpha_to_coverage_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__sample_alpha_to_coverage_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__sample_alpha_to_coverage_enable);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__SAMPLE_ALPHA_TO_ONE_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__sample_alpha_to_one_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__sample_alpha_to_one_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__sample_alpha_to_one_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__sample_alpha_to_one_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__sample_alpha_to_one_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__sample_alpha_to_one_enable);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__SAMPLE_COVERAGE_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__sample_coverage_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__sample_coverage_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__sample_coverage_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__sample_coverage_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__sample_coverage_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__sample_coverage_enable);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__SCISSOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__scissor, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__scissor, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__scissor, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__scissor, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__scissor, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__scissor);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__SCISSOR_TEST_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__scissor_test_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__scissor_test_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__scissor_test_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__scissor_test_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__scissor_test_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__scissor_test_enable);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__SHADE_MODEL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__shade_model, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__shade_model, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__shade_model, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__shade_model, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__shade_model, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__shade_model);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_FUNC] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_func, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_func, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_func, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_func, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_func);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_FUNC_SEPARATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_func_separate, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_func_separate, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_func_separate, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_func_separate, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_func_separate, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_func_separate);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_MASK] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_mask, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_mask, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_mask);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_MASK_SEPARATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_mask_separate, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_mask_separate, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_mask_separate, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_mask_separate, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_mask_separate, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_mask_separate);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_OP] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_op, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_op, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_op, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_op, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_op, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_op);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_OP_SEPARATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_op_separate, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_op_separate, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_op_separate, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_op_separate, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_op_separate, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_op_separate);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_TEST_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_test_enable, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_test_enable, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_test_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_test_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_test_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_test_enable);
functionMap[HASH_ELEMENT_TEXTURE1D] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__texture1D, &ColladaParserAutoGen15Private::_data__texture1D, &ColladaParserAutoGen15Private::_end__texture1D, &ColladaParserAutoGen15Private::_preBegin__texture1D, &ColladaParserAutoGen15Private::_preEnd__texture1D, &ColladaParserAutoGen15Private::_freeAttributes__texture1D);
functionMap[HASH_ELEMENT_TEXTURE1D_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__texture1D_enable, &ColladaParserAutoGen15Private::_data__texture1D_enable, &ColladaParserAutoGen15Private::_end__texture1D_enable, &ColladaParserAutoGen15Private::_preBegin__texture1D_enable, &ColladaParserAutoGen15Private::_preEnd__texture1D_enable, &ColladaParserAutoGen15Private::_freeAttributes__texture1D_enable);
functionMap[HASH_ELEMENT_TEXTURE2D] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__texture2D, &ColladaParserAutoGen15Private::_data__texture2D, &ColladaParserAutoGen15Private::_end__texture2D, &ColladaParserAutoGen15Private::_preBegin__texture2D, &ColladaParserAutoGen15Private::_preEnd__texture2D, &ColladaParserAutoGen15Private::_freeAttributes__texture2D);
functionMap[HASH_ELEMENT_TEXTURE2D_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__texture2D_enable, &ColladaParserAutoGen15Private::_data__texture2D_enable, &ColladaParserAutoGen15Private::_end__texture2D_enable, &ColladaParserAutoGen15Private::_preBegin__texture2D_enable, &ColladaParserAutoGen15Private::_preEnd__texture2D_enable, &ColladaParserAutoGen15Private::_freeAttributes__texture2D_enable);
functionMap[HASH_ELEMENT_TEXTURE3D] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__texture3D, &ColladaParserAutoGen15Private::_data__texture3D, &ColladaParserAutoGen15Private::_end__texture3D, &ColladaParserAutoGen15Private::_preBegin__texture3D, &ColladaParserAutoGen15Private::_preEnd__texture3D, &ColladaParserAutoGen15Private::_freeAttributes__texture3D);
functionMap[HASH_ELEMENT_TEXTURE3D_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__texture3D_enable, &ColladaParserAutoGen15Private::_data__texture3D_enable, &ColladaParserAutoGen15Private::_end__texture3D_enable, &ColladaParserAutoGen15Private::_preBegin__texture3D_enable, &ColladaParserAutoGen15Private::_preEnd__texture3D_enable, &ColladaParserAutoGen15Private::_freeAttributes__texture3D_enable);
functionMap[HASH_ELEMENT_TEXTURECUBE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__textureCUBE, &ColladaParserAutoGen15Private::_data__textureCUBE, &ColladaParserAutoGen15Private::_end__textureCUBE, &ColladaParserAutoGen15Private::_preBegin__textureCUBE, &ColladaParserAutoGen15Private::_preEnd__textureCUBE, &ColladaParserAutoGen15Private::_freeAttributes__textureCUBE);
functionMap[HASH_ELEMENT_TEXTURECUBE_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__textureCUBE_enable, &ColladaParserAutoGen15Private::_data__textureCUBE_enable, &ColladaParserAutoGen15Private::_end__textureCUBE_enable, &ColladaParserAutoGen15Private::_preBegin__textureCUBE_enable, &ColladaParserAutoGen15Private::_preEnd__textureCUBE_enable, &ColladaParserAutoGen15Private::_freeAttributes__textureCUBE_enable);
functionMap[HASH_ELEMENT_TEXTUREDEPTH] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__textureDEPTH, &ColladaParserAutoGen15Private::_data__textureDEPTH, &ColladaParserAutoGen15Private::_end__textureDEPTH, &ColladaParserAutoGen15Private::_preBegin__textureDEPTH, &ColladaParserAutoGen15Private::_preEnd__textureDEPTH, &ColladaParserAutoGen15Private::_freeAttributes__textureDEPTH);
functionMap[HASH_ELEMENT_TEXTUREDEPTH_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__textureDEPTH_enable, &ColladaParserAutoGen15Private::_data__textureDEPTH_enable, &ColladaParserAutoGen15Private::_end__textureDEPTH_enable, &ColladaParserAutoGen15Private::_preBegin__textureDEPTH_enable, &ColladaParserAutoGen15Private::_preEnd__textureDEPTH_enable, &ColladaParserAutoGen15Private::_freeAttributes__textureDEPTH_enable);
functionMap[HASH_ELEMENT_TEXTURERECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__textureRECT, &ColladaParserAutoGen15Private::_data__textureRECT, &ColladaParserAutoGen15Private::_end__textureRECT, &ColladaParserAutoGen15Private::_preBegin__textureRECT, &ColladaParserAutoGen15Private::_preEnd__textureRECT, &ColladaParserAutoGen15Private::_freeAttributes__textureRECT);
functionMap[HASH_ELEMENT_TEXTURERECT_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__textureRECT_enable, &ColladaParserAutoGen15Private::_data__textureRECT_enable, &ColladaParserAutoGen15Private::_end__textureRECT_enable, &ColladaParserAutoGen15Private::_preBegin__textureRECT_enable, &ColladaParserAutoGen15Private::_preEnd__textureRECT_enable, &ColladaParserAutoGen15Private::_freeAttributes__textureRECT_enable);
functionMap[HASH_ELEMENT_TEXTURE_ENV_COLOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__texture_env_color, &ColladaParserAutoGen15Private::_data__texture_env_color, &ColladaParserAutoGen15Private::_end__texture_env_color, &ColladaParserAutoGen15Private::_preBegin__texture_env_color, &ColladaParserAutoGen15Private::_preEnd__texture_env_color, &ColladaParserAutoGen15Private::_freeAttributes__texture_env_color);
functionMap[HASH_ELEMENT_TEXTURE_ENV_MODE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__texture_env_mode, &ColladaParserAutoGen15Private::_data__texture_env_mode, &ColladaParserAutoGen15Private::_end__texture_env_mode, &ColladaParserAutoGen15Private::_preBegin__texture_env_mode, &ColladaParserAutoGen15Private::_preEnd__texture_env_mode, &ColladaParserAutoGen15Private::_freeAttributes__texture_env_mode);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__ALPHA_FUNC__FUNC] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__alpha_func__func, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__alpha_func__func, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__alpha_func__func, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__alpha_func__func, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__alpha_func__func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__alpha_func__func);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__ALPHA_FUNC__VALUE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__alpha_func__value, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__alpha_func__value, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__alpha_func__value, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__alpha_func__value, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__alpha_func__value, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__alpha_func__value);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_EQUATION_SEPARATE__ALPHA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__blend_equation_separate__alpha, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__blend_equation_separate__alpha, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__blend_equation_separate__alpha, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__blend_equation_separate__alpha, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__blend_equation_separate__alpha, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__blend_equation_separate__alpha);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_EQUATION_SEPARATE__RGB] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__blend_equation_separate__rgb, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__blend_equation_separate__rgb, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__blend_equation_separate__rgb, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__blend_equation_separate__rgb, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__blend_equation_separate__rgb, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__blend_equation_separate__rgb);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_FUNC__DEST] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__blend_func__dest, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__blend_func__dest, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__blend_func__dest, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__blend_func__dest, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__blend_func__dest, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__blend_func__dest);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_FUNC__SRC] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__blend_func__src, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__blend_func__src, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__blend_func__src, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__blend_func__src, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__blend_func__src, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__blend_func__src);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_FUNC_SEPARATE__DEST_ALPHA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__blend_func_separate__dest_alpha, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__blend_func_separate__dest_alpha, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__blend_func_separate__dest_alpha, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__blend_func_separate__dest_alpha, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__blend_func_separate__dest_alpha, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__blend_func_separate__dest_alpha);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_FUNC_SEPARATE__DEST_RGB] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__blend_func_separate__dest_rgb, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__blend_func_separate__dest_rgb, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__blend_func_separate__dest_rgb, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__blend_func_separate__dest_rgb, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__blend_func_separate__dest_rgb, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__blend_func_separate__dest_rgb);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_FUNC_SEPARATE__SRC_ALPHA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__blend_func_separate__src_alpha, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__blend_func_separate__src_alpha, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__blend_func_separate__src_alpha, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__blend_func_separate__src_alpha, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__blend_func_separate__src_alpha, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__blend_func_separate__src_alpha);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__BLEND_FUNC_SEPARATE__SRC_RGB] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__blend_func_separate__src_rgb, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__blend_func_separate__src_rgb, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__blend_func_separate__src_rgb, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__blend_func_separate__src_rgb, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__blend_func_separate__src_rgb, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__blend_func_separate__src_rgb);
functionMap[HASH_ELEMENT_COLOR_MATERIAL__FACE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__color_material__face, &ColladaParserAutoGen15Private::_data__color_material__face, &ColladaParserAutoGen15Private::_end__color_material__face, &ColladaParserAutoGen15Private::_preBegin__color_material__face, &ColladaParserAutoGen15Private::_preEnd__color_material__face, &ColladaParserAutoGen15Private::_freeAttributes__color_material__face);
functionMap[HASH_ELEMENT_COLOR_MATERIAL__MODE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__color_material__mode, &ColladaParserAutoGen15Private::_data__color_material__mode, &ColladaParserAutoGen15Private::_end__color_material__mode, &ColladaParserAutoGen15Private::_preBegin__color_material__mode, &ColladaParserAutoGen15Private::_preEnd__color_material__mode, &ColladaParserAutoGen15Private::_freeAttributes__color_material__mode);
functionMap[HASH_ELEMENT_POLYGON_MODE__FACE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__polygon_mode__face, &ColladaParserAutoGen15Private::_data__polygon_mode__face, &ColladaParserAutoGen15Private::_end__polygon_mode__face, &ColladaParserAutoGen15Private::_preBegin__polygon_mode__face, &ColladaParserAutoGen15Private::_preEnd__polygon_mode__face, &ColladaParserAutoGen15Private::_freeAttributes__polygon_mode__face);
functionMap[HASH_ELEMENT_POLYGON_MODE__MODE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__polygon_mode__mode, &ColladaParserAutoGen15Private::_data__polygon_mode__mode, &ColladaParserAutoGen15Private::_end__polygon_mode__mode, &ColladaParserAutoGen15Private::_preBegin__polygon_mode__mode, &ColladaParserAutoGen15Private::_preEnd__polygon_mode__mode, &ColladaParserAutoGen15Private::_freeAttributes__polygon_mode__mode);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_FUNC__FUNC] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_func__func, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_func__func, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_func__func, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_func__func, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_func__func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_func__func);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_FUNC__MASK] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_func__mask, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_func__mask, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_func__mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_func__mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_func__mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_func__mask);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_FUNC__REF] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_func__ref, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_func__ref, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_func__ref, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_func__ref, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_func__ref, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_func__ref);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_FUNC_SEPARATE__BACK] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_func_separate__back, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_func_separate__back, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_func_separate__back, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_func_separate__back, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_func_separate__back, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_func_separate__back);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_FUNC_SEPARATE__FRONT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_func_separate__front, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_func_separate__front, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_func_separate__front, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_func_separate__front, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_func_separate__front, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_func_separate__front);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_FUNC_SEPARATE__MASK] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_func_separate__mask, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_func_separate__mask, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_func_separate__mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_func_separate__mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_func_separate__mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_func_separate__mask);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_FUNC_SEPARATE__REF] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_func_separate__ref, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_func_separate__ref, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_func_separate__ref, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_func_separate__ref, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_func_separate__ref, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_func_separate__ref);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_MASK_SEPARATE__FACE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_mask_separate__face, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_mask_separate__face, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_mask_separate__face, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_mask_separate__face, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_mask_separate__face, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_mask_separate__face);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_MASK_SEPARATE__MASK] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_mask_separate__mask, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_mask_separate__mask, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_mask_separate__mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_mask_separate__mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_mask_separate__mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_mask_separate__mask);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_OP__FAIL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_op__fail, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_op__fail, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_op__fail, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_op__fail, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_op__fail, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_op__fail);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_OP__ZFAIL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_op__zfail, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_op__zfail, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_op__zfail, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_op__zfail, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_op__zfail, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_op__zfail);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_OP__ZPASS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_op__zpass, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_op__zpass, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_op__zpass, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_op__zpass, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_op__zpass, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_op__zpass);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_OP_SEPARATE__FACE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_op_separate__face, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_op_separate__face, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_op_separate__face, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_op_separate__face, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_op_separate__face, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_op_separate__face);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_OP_SEPARATE__FAIL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_op_separate__fail, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_op_separate__fail, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_op_separate__fail, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_op_separate__fail, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_op_separate__fail, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_op_separate__fail);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_OP_SEPARATE__ZFAIL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_op_separate__zfail, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_op_separate__zfail, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_op_separate__zfail, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_op_separate__zfail, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_op_separate__zfail, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_op_separate__zfail);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES__STENCIL_OP_SEPARATE__ZPASS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states__stencil_op_separate__zpass, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states__stencil_op_separate__zpass, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states__stencil_op_separate__zpass, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states__stencil_op_separate__zpass, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states__stencil_op_separate__zpass, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states__stencil_op_separate__zpass);
functionMap[HASH_ELEMENT_PARAM____NCNAME] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__param____NCName, &ColladaParserAutoGen15Private::_data__param____NCName, &ColladaParserAutoGen15Private::_end__param____NCName, &ColladaParserAutoGen15Private::_preBegin__param____NCName, &ColladaParserAutoGen15Private::_preEnd__param____NCName, &ColladaParserAutoGen15Private::_freeAttributes__param____NCName);
functionMap[HASH_ELEMENT_VALUE____FX_SAMPLER1D_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__value____fx_sampler1D_type, &ColladaParserAutoGen15Private::_data__value____fx_sampler1D_type, &ColladaParserAutoGen15Private::_end__value____fx_sampler1D_type, &ColladaParserAutoGen15Private::_preBegin__value____fx_sampler1D_type, &ColladaParserAutoGen15Private::_preEnd__value____fx_sampler1D_type, &ColladaParserAutoGen15Private::_freeAttributes__value____fx_sampler1D_type);
functionMap[HASH_ELEMENT_PARAM____NCNAME] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__param____NCName, &ColladaParserAutoGen15Private::_data__param____NCName, &ColladaParserAutoGen15Private::_end__param____NCName, &ColladaParserAutoGen15Private::_preBegin__param____NCName, &ColladaParserAutoGen15Private::_preEnd__param____NCName, &ColladaParserAutoGen15Private::_freeAttributes__param____NCName);
functionMap[HASH_ELEMENT_VALUE____FX_SAMPLER2D_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__value____fx_sampler2D_type, &ColladaParserAutoGen15Private::_data__value____fx_sampler2D_type, &ColladaParserAutoGen15Private::_end__value____fx_sampler2D_type, &ColladaParserAutoGen15Private::_preBegin__value____fx_sampler2D_type, &ColladaParserAutoGen15Private::_preEnd__value____fx_sampler2D_type, &ColladaParserAutoGen15Private::_freeAttributes__value____fx_sampler2D_type);
functionMap[HASH_ELEMENT_PARAM____NCNAME] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__param____NCName, &ColladaParserAutoGen15Private::_data__param____NCName, &ColladaParserAutoGen15Private::_end__param____NCName, &ColladaParserAutoGen15Private::_preBegin__param____NCName, &ColladaParserAutoGen15Private::_preEnd__param____NCName, &ColladaParserAutoGen15Private::_freeAttributes__param____NCName);
functionMap[HASH_ELEMENT_VALUE____FX_SAMPLER3D_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__value____fx_sampler3D_type, &ColladaParserAutoGen15Private::_data__value____fx_sampler3D_type, &ColladaParserAutoGen15Private::_end__value____fx_sampler3D_type, &ColladaParserAutoGen15Private::_preBegin__value____fx_sampler3D_type, &ColladaParserAutoGen15Private::_preEnd__value____fx_sampler3D_type, &ColladaParserAutoGen15Private::_freeAttributes__value____fx_sampler3D_type);
functionMap[HASH_ELEMENT_PARAM____NCNAME] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__param____NCName, &ColladaParserAutoGen15Private::_data__param____NCName, &ColladaParserAutoGen15Private::_end__param____NCName, &ColladaParserAutoGen15Private::_preBegin__param____NCName, &ColladaParserAutoGen15Private::_preEnd__param____NCName, &ColladaParserAutoGen15Private::_freeAttributes__param____NCName);
functionMap[HASH_ELEMENT_VALUE____FX_SAMPLERCUBE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__value____fx_samplerCUBE_type, &ColladaParserAutoGen15Private::_data__value____fx_samplerCUBE_type, &ColladaParserAutoGen15Private::_end__value____fx_samplerCUBE_type, &ColladaParserAutoGen15Private::_preBegin__value____fx_samplerCUBE_type, &ColladaParserAutoGen15Private::_preEnd__value____fx_samplerCUBE_type, &ColladaParserAutoGen15Private::_freeAttributes__value____fx_samplerCUBE_type);
functionMap[HASH_ELEMENT_PARAM____NCNAME] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__param____NCName, &ColladaParserAutoGen15Private::_data__param____NCName, &ColladaParserAutoGen15Private::_end__param____NCName, &ColladaParserAutoGen15Private::_preBegin__param____NCName, &ColladaParserAutoGen15Private::_preEnd__param____NCName, &ColladaParserAutoGen15Private::_freeAttributes__param____NCName);
functionMap[HASH_ELEMENT_VALUE____FX_SAMPLERDEPTH_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__value____fx_samplerDEPTH_type, &ColladaParserAutoGen15Private::_data__value____fx_samplerDEPTH_type, &ColladaParserAutoGen15Private::_end__value____fx_samplerDEPTH_type, &ColladaParserAutoGen15Private::_preBegin__value____fx_samplerDEPTH_type, &ColladaParserAutoGen15Private::_preEnd__value____fx_samplerDEPTH_type, &ColladaParserAutoGen15Private::_freeAttributes__value____fx_samplerDEPTH_type);
functionMap[HASH_ELEMENT_PARAM____NCNAME] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__param____NCName, &ColladaParserAutoGen15Private::_data__param____NCName, &ColladaParserAutoGen15Private::_end__param____NCName, &ColladaParserAutoGen15Private::_preBegin__param____NCName, &ColladaParserAutoGen15Private::_preEnd__param____NCName, &ColladaParserAutoGen15Private::_freeAttributes__param____NCName);
functionMap[HASH_ELEMENT_VALUE____FX_SAMPLERRECT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__value____fx_samplerRECT_type, &ColladaParserAutoGen15Private::_data__value____fx_samplerRECT_type, &ColladaParserAutoGen15Private::_end__value____fx_samplerRECT_type, &ColladaParserAutoGen15Private::_preBegin__value____fx_samplerRECT_type, &ColladaParserAutoGen15Private::_preEnd__value____fx_samplerRECT_type, &ColladaParserAutoGen15Private::_freeAttributes__value____fx_samplerRECT_type);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_NEWPARAM____FX_COMMON_NEWPARAM_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__newparam____fx_common_newparam_type, &ColladaParserAutoGen15Private::_data__newparam____fx_common_newparam_type, &ColladaParserAutoGen15Private::_end__newparam____fx_common_newparam_type, &ColladaParserAutoGen15Private::_preBegin__newparam____fx_common_newparam_type, &ColladaParserAutoGen15Private::_preEnd__newparam____fx_common_newparam_type, &ColladaParserAutoGen15Private::_freeAttributes__newparam____fx_common_newparam_type);
functionMap[HASH_ELEMENT_PROFILE_COMMON_TYPE____TECHNIQUE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_common_type____technique, &ColladaParserAutoGen15Private::_data__profile_common_type____technique, &ColladaParserAutoGen15Private::_end__profile_common_type____technique, &ColladaParserAutoGen15Private::_preBegin__profile_common_type____technique, &ColladaParserAutoGen15Private::_preEnd__profile_common_type____technique, &ColladaParserAutoGen15Private::_freeAttributes__profile_common_type____technique);
functionMap[HASH_ELEMENT_FLOAT____FLOAT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float____float_type, &ColladaParserAutoGen15Private::_data__float____float_type, &ColladaParserAutoGen15Private::_end__float____float_type, &ColladaParserAutoGen15Private::_preBegin__float____float_type, &ColladaParserAutoGen15Private::_preEnd__float____float_type, &ColladaParserAutoGen15Private::_freeAttributes__float____float_type);
functionMap[HASH_ELEMENT_FLOAT2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2, &ColladaParserAutoGen15Private::_data__float2, &ColladaParserAutoGen15Private::_end__float2, &ColladaParserAutoGen15Private::_preBegin__float2, &ColladaParserAutoGen15Private::_preEnd__float2, &ColladaParserAutoGen15Private::_freeAttributes__float2);
functionMap[HASH_ELEMENT_FLOAT3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3, &ColladaParserAutoGen15Private::_data__float3, &ColladaParserAutoGen15Private::_end__float3, &ColladaParserAutoGen15Private::_preBegin__float3, &ColladaParserAutoGen15Private::_preEnd__float3, &ColladaParserAutoGen15Private::_freeAttributes__float3);
functionMap[HASH_ELEMENT_FLOAT4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4, &ColladaParserAutoGen15Private::_data__float4, &ColladaParserAutoGen15Private::_end__float4, &ColladaParserAutoGen15Private::_preBegin__float4, &ColladaParserAutoGen15Private::_preEnd__float4, &ColladaParserAutoGen15Private::_freeAttributes__float4);
functionMap[HASH_ELEMENT_SAMPLER2D____FX_SAMPLER2D_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_data__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_end__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_preBegin__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_preEnd__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_freeAttributes__sampler2D____fx_sampler2D_type);
functionMap[HASH_ELEMENT_SEMANTIC____NCNAME] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__semantic____NCName, &ColladaParserAutoGen15Private::_data__semantic____NCName, &ColladaParserAutoGen15Private::_end__semantic____NCName, &ColladaParserAutoGen15Private::_preBegin__semantic____NCName, &ColladaParserAutoGen15Private::_preEnd__semantic____NCName, &ColladaParserAutoGen15Private::_freeAttributes__semantic____NCName);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_BLINN] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__blinn, &ColladaParserAutoGen15Private::_data__blinn, &ColladaParserAutoGen15Private::_end__blinn, &ColladaParserAutoGen15Private::_preBegin__blinn, &ColladaParserAutoGen15Private::_preEnd__blinn, &ColladaParserAutoGen15Private::_freeAttributes__blinn);
functionMap[HASH_ELEMENT_PROFILE_COMMON__TECHNIQUE__CONSTANT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_COMMON__technique__constant, &ColladaParserAutoGen15Private::_data__profile_COMMON__technique__constant, &ColladaParserAutoGen15Private::_end__profile_COMMON__technique__constant, &ColladaParserAutoGen15Private::_preBegin__profile_COMMON__technique__constant, &ColladaParserAutoGen15Private::_preEnd__profile_COMMON__technique__constant, &ColladaParserAutoGen15Private::_freeAttributes__profile_COMMON__technique__constant);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_LAMBERT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__lambert, &ColladaParserAutoGen15Private::_data__lambert, &ColladaParserAutoGen15Private::_end__lambert, &ColladaParserAutoGen15Private::_preBegin__lambert, &ColladaParserAutoGen15Private::_preEnd__lambert, &ColladaParserAutoGen15Private::_freeAttributes__lambert);
functionMap[HASH_ELEMENT_PHONG] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__phong, &ColladaParserAutoGen15Private::_data__phong, &ColladaParserAutoGen15Private::_end__phong, &ColladaParserAutoGen15Private::_preBegin__phong, &ColladaParserAutoGen15Private::_preEnd__phong, &ColladaParserAutoGen15Private::_freeAttributes__phong);
functionMap[HASH_ELEMENT_AMBIENT____FX_COMMON_COLOR_OR_TEXTURE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__ambient____fx_common_color_or_texture_type, &ColladaParserAutoGen15Private::_data__ambient____fx_common_color_or_texture_type, &ColladaParserAutoGen15Private::_end__ambient____fx_common_color_or_texture_type, &ColladaParserAutoGen15Private::_preBegin__ambient____fx_common_color_or_texture_type, &ColladaParserAutoGen15Private::_preEnd__ambient____fx_common_color_or_texture_type, &ColladaParserAutoGen15Private::_freeAttributes__ambient____fx_common_color_or_texture_type);
functionMap[HASH_ELEMENT_DIFFUSE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__diffuse, &ColladaParserAutoGen15Private::_data__diffuse, &ColladaParserAutoGen15Private::_end__diffuse, &ColladaParserAutoGen15Private::_preBegin__diffuse, &ColladaParserAutoGen15Private::_preEnd__diffuse, &ColladaParserAutoGen15Private::_freeAttributes__diffuse);
functionMap[HASH_ELEMENT_EMISSION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__emission, &ColladaParserAutoGen15Private::_data__emission, &ColladaParserAutoGen15Private::_end__emission, &ColladaParserAutoGen15Private::_preBegin__emission, &ColladaParserAutoGen15Private::_preEnd__emission, &ColladaParserAutoGen15Private::_freeAttributes__emission);
functionMap[HASH_ELEMENT_INDEX_OF_REFRACTION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__index_of_refraction, &ColladaParserAutoGen15Private::_data__index_of_refraction, &ColladaParserAutoGen15Private::_end__index_of_refraction, &ColladaParserAutoGen15Private::_preBegin__index_of_refraction, &ColladaParserAutoGen15Private::_preEnd__index_of_refraction, &ColladaParserAutoGen15Private::_freeAttributes__index_of_refraction);
functionMap[HASH_ELEMENT_REFLECTIVE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__reflective, &ColladaParserAutoGen15Private::_data__reflective, &ColladaParserAutoGen15Private::_end__reflective, &ColladaParserAutoGen15Private::_preBegin__reflective, &ColladaParserAutoGen15Private::_preEnd__reflective, &ColladaParserAutoGen15Private::_freeAttributes__reflective);
functionMap[HASH_ELEMENT_REFLECTIVITY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__reflectivity, &ColladaParserAutoGen15Private::_data__reflectivity, &ColladaParserAutoGen15Private::_end__reflectivity, &ColladaParserAutoGen15Private::_preBegin__reflectivity, &ColladaParserAutoGen15Private::_preEnd__reflectivity, &ColladaParserAutoGen15Private::_freeAttributes__reflectivity);
functionMap[HASH_ELEMENT_SHININESS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__shininess, &ColladaParserAutoGen15Private::_data__shininess, &ColladaParserAutoGen15Private::_end__shininess, &ColladaParserAutoGen15Private::_preBegin__shininess, &ColladaParserAutoGen15Private::_preEnd__shininess, &ColladaParserAutoGen15Private::_freeAttributes__shininess);
functionMap[HASH_ELEMENT_SPECULAR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__specular, &ColladaParserAutoGen15Private::_data__specular, &ColladaParserAutoGen15Private::_end__specular, &ColladaParserAutoGen15Private::_preBegin__specular, &ColladaParserAutoGen15Private::_preEnd__specular, &ColladaParserAutoGen15Private::_freeAttributes__specular);
functionMap[HASH_ELEMENT_TRANSPARENCY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__transparency, &ColladaParserAutoGen15Private::_data__transparency, &ColladaParserAutoGen15Private::_end__transparency, &ColladaParserAutoGen15Private::_preBegin__transparency, &ColladaParserAutoGen15Private::_preEnd__transparency, &ColladaParserAutoGen15Private::_freeAttributes__transparency);
functionMap[HASH_ELEMENT_TRANSPARENT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__transparent, &ColladaParserAutoGen15Private::_data__transparent, &ColladaParserAutoGen15Private::_end__transparent, &ColladaParserAutoGen15Private::_preBegin__transparent, &ColladaParserAutoGen15Private::_preEnd__transparent, &ColladaParserAutoGen15Private::_freeAttributes__transparent);
functionMap[HASH_ELEMENT_FX_COMMON_COLOR_OR_TEXTURE_TYPE____COLOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fx_common_color_or_texture_type____color, &ColladaParserAutoGen15Private::_data__fx_common_color_or_texture_type____color, &ColladaParserAutoGen15Private::_end__fx_common_color_or_texture_type____color, &ColladaParserAutoGen15Private::_preBegin__fx_common_color_or_texture_type____color, &ColladaParserAutoGen15Private::_preEnd__fx_common_color_or_texture_type____color, &ColladaParserAutoGen15Private::_freeAttributes__fx_common_color_or_texture_type____color);
functionMap[HASH_ELEMENT_FX_COMMON_COLOR_OR_TEXTURE_TYPE____PARAM] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fx_common_color_or_texture_type____param, &ColladaParserAutoGen15Private::_data__fx_common_color_or_texture_type____param, &ColladaParserAutoGen15Private::_end__fx_common_color_or_texture_type____param, &ColladaParserAutoGen15Private::_preBegin__fx_common_color_or_texture_type____param, &ColladaParserAutoGen15Private::_preEnd__fx_common_color_or_texture_type____param, &ColladaParserAutoGen15Private::_freeAttributes__fx_common_color_or_texture_type____param);
functionMap[HASH_ELEMENT_TEXTURE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__texture, &ColladaParserAutoGen15Private::_data__texture, &ColladaParserAutoGen15Private::_end__texture, &ColladaParserAutoGen15Private::_preBegin__texture, &ColladaParserAutoGen15Private::_preEnd__texture, &ColladaParserAutoGen15Private::_freeAttributes__texture);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_FX_COMMON_FLOAT_OR_PARAM_TYPE____FLOAT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fx_common_float_or_param_type____float, &ColladaParserAutoGen15Private::_data__fx_common_float_or_param_type____float, &ColladaParserAutoGen15Private::_end__fx_common_float_or_param_type____float, &ColladaParserAutoGen15Private::_preBegin__fx_common_float_or_param_type____float, &ColladaParserAutoGen15Private::_preEnd__fx_common_float_or_param_type____float, &ColladaParserAutoGen15Private::_freeAttributes__fx_common_float_or_param_type____float);
functionMap[HASH_ELEMENT_FX_COMMON_FLOAT_OR_PARAM_TYPE____PARAM] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__fx_common_float_or_param_type____param, &ColladaParserAutoGen15Private::_data__fx_common_float_or_param_type____param, &ColladaParserAutoGen15Private::_end__fx_common_float_or_param_type____param, &ColladaParserAutoGen15Private::_preBegin__fx_common_float_or_param_type____param, &ColladaParserAutoGen15Private::_preEnd__fx_common_float_or_param_type____param, &ColladaParserAutoGen15Private::_freeAttributes__fx_common_float_or_param_type____param);
functionMap[HASH_ELEMENT_EMISSION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__emission, &ColladaParserAutoGen15Private::_data__emission, &ColladaParserAutoGen15Private::_end__emission, &ColladaParserAutoGen15Private::_preBegin__emission, &ColladaParserAutoGen15Private::_preEnd__emission, &ColladaParserAutoGen15Private::_freeAttributes__emission);
functionMap[HASH_ELEMENT_INDEX_OF_REFRACTION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__index_of_refraction, &ColladaParserAutoGen15Private::_data__index_of_refraction, &ColladaParserAutoGen15Private::_end__index_of_refraction, &ColladaParserAutoGen15Private::_preBegin__index_of_refraction, &ColladaParserAutoGen15Private::_preEnd__index_of_refraction, &ColladaParserAutoGen15Private::_freeAttributes__index_of_refraction);
functionMap[HASH_ELEMENT_REFLECTIVE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__reflective, &ColladaParserAutoGen15Private::_data__reflective, &ColladaParserAutoGen15Private::_end__reflective, &ColladaParserAutoGen15Private::_preBegin__reflective, &ColladaParserAutoGen15Private::_preEnd__reflective, &ColladaParserAutoGen15Private::_freeAttributes__reflective);
functionMap[HASH_ELEMENT_REFLECTIVITY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__reflectivity, &ColladaParserAutoGen15Private::_data__reflectivity, &ColladaParserAutoGen15Private::_end__reflectivity, &ColladaParserAutoGen15Private::_preBegin__reflectivity, &ColladaParserAutoGen15Private::_preEnd__reflectivity, &ColladaParserAutoGen15Private::_freeAttributes__reflectivity);
functionMap[HASH_ELEMENT_TRANSPARENCY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__transparency, &ColladaParserAutoGen15Private::_data__transparency, &ColladaParserAutoGen15Private::_end__transparency, &ColladaParserAutoGen15Private::_preBegin__transparency, &ColladaParserAutoGen15Private::_preEnd__transparency, &ColladaParserAutoGen15Private::_freeAttributes__transparency);
functionMap[HASH_ELEMENT_TRANSPARENT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__transparent, &ColladaParserAutoGen15Private::_data__transparent, &ColladaParserAutoGen15Private::_end__transparent, &ColladaParserAutoGen15Private::_preBegin__transparent, &ColladaParserAutoGen15Private::_preEnd__transparent, &ColladaParserAutoGen15Private::_freeAttributes__transparent);
functionMap[HASH_ELEMENT_AMBIENT____FX_COMMON_COLOR_OR_TEXTURE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__ambient____fx_common_color_or_texture_type, &ColladaParserAutoGen15Private::_data__ambient____fx_common_color_or_texture_type, &ColladaParserAutoGen15Private::_end__ambient____fx_common_color_or_texture_type, &ColladaParserAutoGen15Private::_preBegin__ambient____fx_common_color_or_texture_type, &ColladaParserAutoGen15Private::_preEnd__ambient____fx_common_color_or_texture_type, &ColladaParserAutoGen15Private::_freeAttributes__ambient____fx_common_color_or_texture_type);
functionMap[HASH_ELEMENT_DIFFUSE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__diffuse, &ColladaParserAutoGen15Private::_data__diffuse, &ColladaParserAutoGen15Private::_end__diffuse, &ColladaParserAutoGen15Private::_preBegin__diffuse, &ColladaParserAutoGen15Private::_preEnd__diffuse, &ColladaParserAutoGen15Private::_freeAttributes__diffuse);
functionMap[HASH_ELEMENT_EMISSION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__emission, &ColladaParserAutoGen15Private::_data__emission, &ColladaParserAutoGen15Private::_end__emission, &ColladaParserAutoGen15Private::_preBegin__emission, &ColladaParserAutoGen15Private::_preEnd__emission, &ColladaParserAutoGen15Private::_freeAttributes__emission);
functionMap[HASH_ELEMENT_INDEX_OF_REFRACTION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__index_of_refraction, &ColladaParserAutoGen15Private::_data__index_of_refraction, &ColladaParserAutoGen15Private::_end__index_of_refraction, &ColladaParserAutoGen15Private::_preBegin__index_of_refraction, &ColladaParserAutoGen15Private::_preEnd__index_of_refraction, &ColladaParserAutoGen15Private::_freeAttributes__index_of_refraction);
functionMap[HASH_ELEMENT_REFLECTIVE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__reflective, &ColladaParserAutoGen15Private::_data__reflective, &ColladaParserAutoGen15Private::_end__reflective, &ColladaParserAutoGen15Private::_preBegin__reflective, &ColladaParserAutoGen15Private::_preEnd__reflective, &ColladaParserAutoGen15Private::_freeAttributes__reflective);
functionMap[HASH_ELEMENT_REFLECTIVITY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__reflectivity, &ColladaParserAutoGen15Private::_data__reflectivity, &ColladaParserAutoGen15Private::_end__reflectivity, &ColladaParserAutoGen15Private::_preBegin__reflectivity, &ColladaParserAutoGen15Private::_preEnd__reflectivity, &ColladaParserAutoGen15Private::_freeAttributes__reflectivity);
functionMap[HASH_ELEMENT_TRANSPARENCY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__transparency, &ColladaParserAutoGen15Private::_data__transparency, &ColladaParserAutoGen15Private::_end__transparency, &ColladaParserAutoGen15Private::_preBegin__transparency, &ColladaParserAutoGen15Private::_preEnd__transparency, &ColladaParserAutoGen15Private::_freeAttributes__transparency);
functionMap[HASH_ELEMENT_TRANSPARENT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__transparent, &ColladaParserAutoGen15Private::_data__transparent, &ColladaParserAutoGen15Private::_end__transparent, &ColladaParserAutoGen15Private::_preBegin__transparent, &ColladaParserAutoGen15Private::_preEnd__transparent, &ColladaParserAutoGen15Private::_freeAttributes__transparent);
functionMap[HASH_ELEMENT_AMBIENT____FX_COMMON_COLOR_OR_TEXTURE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__ambient____fx_common_color_or_texture_type, &ColladaParserAutoGen15Private::_data__ambient____fx_common_color_or_texture_type, &ColladaParserAutoGen15Private::_end__ambient____fx_common_color_or_texture_type, &ColladaParserAutoGen15Private::_preBegin__ambient____fx_common_color_or_texture_type, &ColladaParserAutoGen15Private::_preEnd__ambient____fx_common_color_or_texture_type, &ColladaParserAutoGen15Private::_freeAttributes__ambient____fx_common_color_or_texture_type);
functionMap[HASH_ELEMENT_DIFFUSE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__diffuse, &ColladaParserAutoGen15Private::_data__diffuse, &ColladaParserAutoGen15Private::_end__diffuse, &ColladaParserAutoGen15Private::_preBegin__diffuse, &ColladaParserAutoGen15Private::_preEnd__diffuse, &ColladaParserAutoGen15Private::_freeAttributes__diffuse);
functionMap[HASH_ELEMENT_EMISSION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__emission, &ColladaParserAutoGen15Private::_data__emission, &ColladaParserAutoGen15Private::_end__emission, &ColladaParserAutoGen15Private::_preBegin__emission, &ColladaParserAutoGen15Private::_preEnd__emission, &ColladaParserAutoGen15Private::_freeAttributes__emission);
functionMap[HASH_ELEMENT_INDEX_OF_REFRACTION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__index_of_refraction, &ColladaParserAutoGen15Private::_data__index_of_refraction, &ColladaParserAutoGen15Private::_end__index_of_refraction, &ColladaParserAutoGen15Private::_preBegin__index_of_refraction, &ColladaParserAutoGen15Private::_preEnd__index_of_refraction, &ColladaParserAutoGen15Private::_freeAttributes__index_of_refraction);
functionMap[HASH_ELEMENT_REFLECTIVE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__reflective, &ColladaParserAutoGen15Private::_data__reflective, &ColladaParserAutoGen15Private::_end__reflective, &ColladaParserAutoGen15Private::_preBegin__reflective, &ColladaParserAutoGen15Private::_preEnd__reflective, &ColladaParserAutoGen15Private::_freeAttributes__reflective);
functionMap[HASH_ELEMENT_REFLECTIVITY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__reflectivity, &ColladaParserAutoGen15Private::_data__reflectivity, &ColladaParserAutoGen15Private::_end__reflectivity, &ColladaParserAutoGen15Private::_preBegin__reflectivity, &ColladaParserAutoGen15Private::_preEnd__reflectivity, &ColladaParserAutoGen15Private::_freeAttributes__reflectivity);
functionMap[HASH_ELEMENT_SHININESS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__shininess, &ColladaParserAutoGen15Private::_data__shininess, &ColladaParserAutoGen15Private::_end__shininess, &ColladaParserAutoGen15Private::_preBegin__shininess, &ColladaParserAutoGen15Private::_preEnd__shininess, &ColladaParserAutoGen15Private::_freeAttributes__shininess);
functionMap[HASH_ELEMENT_SPECULAR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__specular, &ColladaParserAutoGen15Private::_data__specular, &ColladaParserAutoGen15Private::_end__specular, &ColladaParserAutoGen15Private::_preBegin__specular, &ColladaParserAutoGen15Private::_preEnd__specular, &ColladaParserAutoGen15Private::_freeAttributes__specular);
functionMap[HASH_ELEMENT_TRANSPARENCY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__transparency, &ColladaParserAutoGen15Private::_data__transparency, &ColladaParserAutoGen15Private::_end__transparency, &ColladaParserAutoGen15Private::_preBegin__transparency, &ColladaParserAutoGen15Private::_preEnd__transparency, &ColladaParserAutoGen15Private::_freeAttributes__transparency);
functionMap[HASH_ELEMENT_TRANSPARENT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__transparent, &ColladaParserAutoGen15Private::_data__transparent, &ColladaParserAutoGen15Private::_end__transparent, &ColladaParserAutoGen15Private::_preBegin__transparent, &ColladaParserAutoGen15Private::_preEnd__transparent, &ColladaParserAutoGen15Private::_freeAttributes__transparent);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_NEWPARAM____GLES_NEWPARAM_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__newparam____gles_newparam_type, &ColladaParserAutoGen15Private::_data__newparam____gles_newparam_type, &ColladaParserAutoGen15Private::_end__newparam____gles_newparam_type, &ColladaParserAutoGen15Private::_preBegin__newparam____gles_newparam_type, &ColladaParserAutoGen15Private::_preEnd__newparam____gles_newparam_type, &ColladaParserAutoGen15Private::_freeAttributes__newparam____gles_newparam_type);
functionMap[HASH_ELEMENT_PROFILE_GLES_TYPE____TECHNIQUE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_gles_type____technique, &ColladaParserAutoGen15Private::_data__profile_gles_type____technique, &ColladaParserAutoGen15Private::_end__profile_gles_type____technique, &ColladaParserAutoGen15Private::_preBegin__profile_gles_type____technique, &ColladaParserAutoGen15Private::_preEnd__profile_gles_type____technique, &ColladaParserAutoGen15Private::_freeAttributes__profile_gles_type____technique);
functionMap[HASH_ELEMENT_ANNOTATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__annotate, &ColladaParserAutoGen15Private::_data__annotate, &ColladaParserAutoGen15Private::_end__annotate, &ColladaParserAutoGen15Private::_preBegin__annotate, &ColladaParserAutoGen15Private::_preEnd__annotate, &ColladaParserAutoGen15Private::_freeAttributes__annotate);
functionMap[HASH_ELEMENT_BOOL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool, &ColladaParserAutoGen15Private::_data__bool, &ColladaParserAutoGen15Private::_end__bool, &ColladaParserAutoGen15Private::_preBegin__bool, &ColladaParserAutoGen15Private::_preEnd__bool, &ColladaParserAutoGen15Private::_freeAttributes__bool);
functionMap[HASH_ELEMENT_BOOL2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool2, &ColladaParserAutoGen15Private::_data__bool2, &ColladaParserAutoGen15Private::_end__bool2, &ColladaParserAutoGen15Private::_preBegin__bool2, &ColladaParserAutoGen15Private::_preEnd__bool2, &ColladaParserAutoGen15Private::_freeAttributes__bool2);
functionMap[HASH_ELEMENT_BOOL3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool3, &ColladaParserAutoGen15Private::_data__bool3, &ColladaParserAutoGen15Private::_end__bool3, &ColladaParserAutoGen15Private::_preBegin__bool3, &ColladaParserAutoGen15Private::_preEnd__bool3, &ColladaParserAutoGen15Private::_freeAttributes__bool3);
functionMap[HASH_ELEMENT_BOOL4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool4, &ColladaParserAutoGen15Private::_data__bool4, &ColladaParserAutoGen15Private::_end__bool4, &ColladaParserAutoGen15Private::_preBegin__bool4, &ColladaParserAutoGen15Private::_preEnd__bool4, &ColladaParserAutoGen15Private::_freeAttributes__bool4);
functionMap[HASH_ELEMENT_ENUM____GLES_ENUMERATION_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__enum____gles_enumeration_type, &ColladaParserAutoGen15Private::_data__enum____gles_enumeration_type, &ColladaParserAutoGen15Private::_end__enum____gles_enumeration_type, &ColladaParserAutoGen15Private::_preBegin__enum____gles_enumeration_type, &ColladaParserAutoGen15Private::_preEnd__enum____gles_enumeration_type, &ColladaParserAutoGen15Private::_freeAttributes__enum____gles_enumeration_type);
functionMap[HASH_ELEMENT_FLOAT____FLOAT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float____float_type, &ColladaParserAutoGen15Private::_data__float____float_type, &ColladaParserAutoGen15Private::_end__float____float_type, &ColladaParserAutoGen15Private::_preBegin__float____float_type, &ColladaParserAutoGen15Private::_preEnd__float____float_type, &ColladaParserAutoGen15Private::_freeAttributes__float____float_type);
functionMap[HASH_ELEMENT_FLOAT1X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float1x1, &ColladaParserAutoGen15Private::_data__float1x1, &ColladaParserAutoGen15Private::_end__float1x1, &ColladaParserAutoGen15Private::_preBegin__float1x1, &ColladaParserAutoGen15Private::_preEnd__float1x1, &ColladaParserAutoGen15Private::_freeAttributes__float1x1);
functionMap[HASH_ELEMENT_FLOAT1X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float1x2, &ColladaParserAutoGen15Private::_data__float1x2, &ColladaParserAutoGen15Private::_end__float1x2, &ColladaParserAutoGen15Private::_preBegin__float1x2, &ColladaParserAutoGen15Private::_preEnd__float1x2, &ColladaParserAutoGen15Private::_freeAttributes__float1x2);
functionMap[HASH_ELEMENT_FLOAT1X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float1x3, &ColladaParserAutoGen15Private::_data__float1x3, &ColladaParserAutoGen15Private::_end__float1x3, &ColladaParserAutoGen15Private::_preBegin__float1x3, &ColladaParserAutoGen15Private::_preEnd__float1x3, &ColladaParserAutoGen15Private::_freeAttributes__float1x3);
functionMap[HASH_ELEMENT_FLOAT1X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float1x4, &ColladaParserAutoGen15Private::_data__float1x4, &ColladaParserAutoGen15Private::_end__float1x4, &ColladaParserAutoGen15Private::_preBegin__float1x4, &ColladaParserAutoGen15Private::_preEnd__float1x4, &ColladaParserAutoGen15Private::_freeAttributes__float1x4);
functionMap[HASH_ELEMENT_FLOAT2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2, &ColladaParserAutoGen15Private::_data__float2, &ColladaParserAutoGen15Private::_end__float2, &ColladaParserAutoGen15Private::_preBegin__float2, &ColladaParserAutoGen15Private::_preEnd__float2, &ColladaParserAutoGen15Private::_freeAttributes__float2);
functionMap[HASH_ELEMENT_FLOAT2X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2x1, &ColladaParserAutoGen15Private::_data__float2x1, &ColladaParserAutoGen15Private::_end__float2x1, &ColladaParserAutoGen15Private::_preBegin__float2x1, &ColladaParserAutoGen15Private::_preEnd__float2x1, &ColladaParserAutoGen15Private::_freeAttributes__float2x1);
functionMap[HASH_ELEMENT_FLOAT2X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2x2, &ColladaParserAutoGen15Private::_data__float2x2, &ColladaParserAutoGen15Private::_end__float2x2, &ColladaParserAutoGen15Private::_preBegin__float2x2, &ColladaParserAutoGen15Private::_preEnd__float2x2, &ColladaParserAutoGen15Private::_freeAttributes__float2x2);
functionMap[HASH_ELEMENT_FLOAT2X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2x3, &ColladaParserAutoGen15Private::_data__float2x3, &ColladaParserAutoGen15Private::_end__float2x3, &ColladaParserAutoGen15Private::_preBegin__float2x3, &ColladaParserAutoGen15Private::_preEnd__float2x3, &ColladaParserAutoGen15Private::_freeAttributes__float2x3);
functionMap[HASH_ELEMENT_FLOAT2X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2x4, &ColladaParserAutoGen15Private::_data__float2x4, &ColladaParserAutoGen15Private::_end__float2x4, &ColladaParserAutoGen15Private::_preBegin__float2x4, &ColladaParserAutoGen15Private::_preEnd__float2x4, &ColladaParserAutoGen15Private::_freeAttributes__float2x4);
functionMap[HASH_ELEMENT_FLOAT3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3, &ColladaParserAutoGen15Private::_data__float3, &ColladaParserAutoGen15Private::_end__float3, &ColladaParserAutoGen15Private::_preBegin__float3, &ColladaParserAutoGen15Private::_preEnd__float3, &ColladaParserAutoGen15Private::_freeAttributes__float3);
functionMap[HASH_ELEMENT_FLOAT3X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3x1, &ColladaParserAutoGen15Private::_data__float3x1, &ColladaParserAutoGen15Private::_end__float3x1, &ColladaParserAutoGen15Private::_preBegin__float3x1, &ColladaParserAutoGen15Private::_preEnd__float3x1, &ColladaParserAutoGen15Private::_freeAttributes__float3x1);
functionMap[HASH_ELEMENT_FLOAT3X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3x2, &ColladaParserAutoGen15Private::_data__float3x2, &ColladaParserAutoGen15Private::_end__float3x2, &ColladaParserAutoGen15Private::_preBegin__float3x2, &ColladaParserAutoGen15Private::_preEnd__float3x2, &ColladaParserAutoGen15Private::_freeAttributes__float3x2);
functionMap[HASH_ELEMENT_FLOAT3X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3x3, &ColladaParserAutoGen15Private::_data__float3x3, &ColladaParserAutoGen15Private::_end__float3x3, &ColladaParserAutoGen15Private::_preBegin__float3x3, &ColladaParserAutoGen15Private::_preEnd__float3x3, &ColladaParserAutoGen15Private::_freeAttributes__float3x3);
functionMap[HASH_ELEMENT_FLOAT3X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3x4, &ColladaParserAutoGen15Private::_data__float3x4, &ColladaParserAutoGen15Private::_end__float3x4, &ColladaParserAutoGen15Private::_preBegin__float3x4, &ColladaParserAutoGen15Private::_preEnd__float3x4, &ColladaParserAutoGen15Private::_freeAttributes__float3x4);
functionMap[HASH_ELEMENT_FLOAT4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4, &ColladaParserAutoGen15Private::_data__float4, &ColladaParserAutoGen15Private::_end__float4, &ColladaParserAutoGen15Private::_preBegin__float4, &ColladaParserAutoGen15Private::_preEnd__float4, &ColladaParserAutoGen15Private::_freeAttributes__float4);
functionMap[HASH_ELEMENT_FLOAT4X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4x1, &ColladaParserAutoGen15Private::_data__float4x1, &ColladaParserAutoGen15Private::_end__float4x1, &ColladaParserAutoGen15Private::_preBegin__float4x1, &ColladaParserAutoGen15Private::_preEnd__float4x1, &ColladaParserAutoGen15Private::_freeAttributes__float4x1);
functionMap[HASH_ELEMENT_FLOAT4X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4x2, &ColladaParserAutoGen15Private::_data__float4x2, &ColladaParserAutoGen15Private::_end__float4x2, &ColladaParserAutoGen15Private::_preBegin__float4x2, &ColladaParserAutoGen15Private::_preEnd__float4x2, &ColladaParserAutoGen15Private::_freeAttributes__float4x2);
functionMap[HASH_ELEMENT_FLOAT4X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4x3, &ColladaParserAutoGen15Private::_data__float4x3, &ColladaParserAutoGen15Private::_end__float4x3, &ColladaParserAutoGen15Private::_preBegin__float4x3, &ColladaParserAutoGen15Private::_preEnd__float4x3, &ColladaParserAutoGen15Private::_freeAttributes__float4x3);
functionMap[HASH_ELEMENT_FLOAT4X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4x4, &ColladaParserAutoGen15Private::_data__float4x4, &ColladaParserAutoGen15Private::_end__float4x4, &ColladaParserAutoGen15Private::_preBegin__float4x4, &ColladaParserAutoGen15Private::_preEnd__float4x4, &ColladaParserAutoGen15Private::_freeAttributes__float4x4);
functionMap[HASH_ELEMENT_INT____INT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int____int_type, &ColladaParserAutoGen15Private::_data__int____int_type, &ColladaParserAutoGen15Private::_end__int____int_type, &ColladaParserAutoGen15Private::_preBegin__int____int_type, &ColladaParserAutoGen15Private::_preEnd__int____int_type, &ColladaParserAutoGen15Private::_freeAttributes__int____int_type);
functionMap[HASH_ELEMENT_INT2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int2, &ColladaParserAutoGen15Private::_data__int2, &ColladaParserAutoGen15Private::_end__int2, &ColladaParserAutoGen15Private::_preBegin__int2, &ColladaParserAutoGen15Private::_preEnd__int2, &ColladaParserAutoGen15Private::_freeAttributes__int2);
functionMap[HASH_ELEMENT_INT3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int3, &ColladaParserAutoGen15Private::_data__int3, &ColladaParserAutoGen15Private::_end__int3, &ColladaParserAutoGen15Private::_preBegin__int3, &ColladaParserAutoGen15Private::_preEnd__int3, &ColladaParserAutoGen15Private::_freeAttributes__int3);
functionMap[HASH_ELEMENT_INT4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int4, &ColladaParserAutoGen15Private::_data__int4, &ColladaParserAutoGen15Private::_end__int4, &ColladaParserAutoGen15Private::_preBegin__int4, &ColladaParserAutoGen15Private::_preEnd__int4, &ColladaParserAutoGen15Private::_freeAttributes__int4);
functionMap[HASH_ELEMENT_MODIFIER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modifier, &ColladaParserAutoGen15Private::_data__modifier, &ColladaParserAutoGen15Private::_end__modifier, &ColladaParserAutoGen15Private::_preBegin__modifier, &ColladaParserAutoGen15Private::_preEnd__modifier, &ColladaParserAutoGen15Private::_freeAttributes__modifier);
functionMap[HASH_ELEMENT_SAMPLER2D____GLES_SAMPLER_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler2D____gles_sampler_type, &ColladaParserAutoGen15Private::_data__sampler2D____gles_sampler_type, &ColladaParserAutoGen15Private::_end__sampler2D____gles_sampler_type, &ColladaParserAutoGen15Private::_preBegin__sampler2D____gles_sampler_type, &ColladaParserAutoGen15Private::_preEnd__sampler2D____gles_sampler_type, &ColladaParserAutoGen15Private::_freeAttributes__sampler2D____gles_sampler_type);
functionMap[HASH_ELEMENT_SEMANTIC____NCNAME] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__semantic____NCName, &ColladaParserAutoGen15Private::_data__semantic____NCName, &ColladaParserAutoGen15Private::_end__semantic____NCName, &ColladaParserAutoGen15Private::_preBegin__semantic____NCName, &ColladaParserAutoGen15Private::_preEnd__semantic____NCName, &ColladaParserAutoGen15Private::_freeAttributes__semantic____NCName);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INSTANCE_IMAGE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_image, &ColladaParserAutoGen15Private::_data__instance_image, &ColladaParserAutoGen15Private::_end__instance_image, &ColladaParserAutoGen15Private::_preBegin__instance_image, &ColladaParserAutoGen15Private::_preEnd__instance_image, &ColladaParserAutoGen15Private::_freeAttributes__instance_image);
functionMap[HASH_ELEMENT_MAGFILTER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__magfilter, &ColladaParserAutoGen15Private::_data__magfilter, &ColladaParserAutoGen15Private::_end__magfilter, &ColladaParserAutoGen15Private::_preBegin__magfilter, &ColladaParserAutoGen15Private::_preEnd__magfilter, &ColladaParserAutoGen15Private::_freeAttributes__magfilter);
functionMap[HASH_ELEMENT_MINFILTER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__minfilter, &ColladaParserAutoGen15Private::_data__minfilter, &ColladaParserAutoGen15Private::_end__minfilter, &ColladaParserAutoGen15Private::_preBegin__minfilter, &ColladaParserAutoGen15Private::_preEnd__minfilter, &ColladaParserAutoGen15Private::_freeAttributes__minfilter);
functionMap[HASH_ELEMENT_MIP_BIAS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__mip_bias, &ColladaParserAutoGen15Private::_data__mip_bias, &ColladaParserAutoGen15Private::_end__mip_bias, &ColladaParserAutoGen15Private::_preBegin__mip_bias, &ColladaParserAutoGen15Private::_preEnd__mip_bias, &ColladaParserAutoGen15Private::_freeAttributes__mip_bias);
functionMap[HASH_ELEMENT_MIP_MAX_LEVEL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__mip_max_level, &ColladaParserAutoGen15Private::_data__mip_max_level, &ColladaParserAutoGen15Private::_end__mip_max_level, &ColladaParserAutoGen15Private::_preBegin__mip_max_level, &ColladaParserAutoGen15Private::_preEnd__mip_max_level, &ColladaParserAutoGen15Private::_freeAttributes__mip_max_level);
functionMap[HASH_ELEMENT_MIPFILTER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__mipfilter, &ColladaParserAutoGen15Private::_data__mipfilter, &ColladaParserAutoGen15Private::_end__mipfilter, &ColladaParserAutoGen15Private::_preBegin__mipfilter, &ColladaParserAutoGen15Private::_preEnd__mipfilter, &ColladaParserAutoGen15Private::_freeAttributes__mipfilter);
functionMap[HASH_ELEMENT_TEXCOORD] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__texcoord, &ColladaParserAutoGen15Private::_data__texcoord, &ColladaParserAutoGen15Private::_end__texcoord, &ColladaParserAutoGen15Private::_preBegin__texcoord, &ColladaParserAutoGen15Private::_preEnd__texcoord, &ColladaParserAutoGen15Private::_freeAttributes__texcoord);
functionMap[HASH_ELEMENT_WRAP_S____GLES_SAMPLER_WRAP_ENUM] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__wrap_s____gles_sampler_wrap_enum, &ColladaParserAutoGen15Private::_data__wrap_s____gles_sampler_wrap_enum, &ColladaParserAutoGen15Private::_end__wrap_s____gles_sampler_wrap_enum, &ColladaParserAutoGen15Private::_preBegin__wrap_s____gles_sampler_wrap_enum, &ColladaParserAutoGen15Private::_preEnd__wrap_s____gles_sampler_wrap_enum, &ColladaParserAutoGen15Private::_freeAttributes__wrap_s____gles_sampler_wrap_enum);
functionMap[HASH_ELEMENT_WRAP_T____GLES_SAMPLER_WRAP_ENUM] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__wrap_t____gles_sampler_wrap_enum, &ColladaParserAutoGen15Private::_data__wrap_t____gles_sampler_wrap_enum, &ColladaParserAutoGen15Private::_end__wrap_t____gles_sampler_wrap_enum, &ColladaParserAutoGen15Private::_preBegin__wrap_t____gles_sampler_wrap_enum, &ColladaParserAutoGen15Private::_preEnd__wrap_t____gles_sampler_wrap_enum, &ColladaParserAutoGen15Private::_freeAttributes__wrap_t____gles_sampler_wrap_enum);
functionMap[HASH_ELEMENT_ANNOTATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__annotate, &ColladaParserAutoGen15Private::_data__annotate, &ColladaParserAutoGen15Private::_end__annotate, &ColladaParserAutoGen15Private::_preBegin__annotate, &ColladaParserAutoGen15Private::_preEnd__annotate, &ColladaParserAutoGen15Private::_freeAttributes__annotate);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass);
functionMap[HASH_ELEMENT_ANNOTATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__annotate, &ColladaParserAutoGen15Private::_data__annotate, &ColladaParserAutoGen15Private::_end__annotate, &ColladaParserAutoGen15Private::_preBegin__annotate, &ColladaParserAutoGen15Private::_preEnd__annotate, &ColladaParserAutoGen15Private::_freeAttributes__annotate);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__EVALUATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__evaluate, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__evaluate, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__evaluate, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__evaluate, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__evaluate, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__evaluate);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states);
functionMap[HASH_ELEMENT_COLOR_CLEAR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__color_clear, &ColladaParserAutoGen15Private::_data__color_clear, &ColladaParserAutoGen15Private::_end__color_clear, &ColladaParserAutoGen15Private::_preBegin__color_clear, &ColladaParserAutoGen15Private::_preEnd__color_clear, &ColladaParserAutoGen15Private::_freeAttributes__color_clear);
functionMap[HASH_ELEMENT_COLOR_TARGET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__color_target, &ColladaParserAutoGen15Private::_data__color_target, &ColladaParserAutoGen15Private::_end__color_target, &ColladaParserAutoGen15Private::_preBegin__color_target, &ColladaParserAutoGen15Private::_preEnd__color_target, &ColladaParserAutoGen15Private::_freeAttributes__color_target);
functionMap[HASH_ELEMENT_DEPTH_CLEAR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__depth_clear, &ColladaParserAutoGen15Private::_data__depth_clear, &ColladaParserAutoGen15Private::_end__depth_clear, &ColladaParserAutoGen15Private::_preBegin__depth_clear, &ColladaParserAutoGen15Private::_preEnd__depth_clear, &ColladaParserAutoGen15Private::_freeAttributes__depth_clear);
functionMap[HASH_ELEMENT_DEPTH_TARGET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__depth_target, &ColladaParserAutoGen15Private::_data__depth_target, &ColladaParserAutoGen15Private::_end__depth_target, &ColladaParserAutoGen15Private::_preBegin__depth_target, &ColladaParserAutoGen15Private::_preEnd__depth_target, &ColladaParserAutoGen15Private::_freeAttributes__depth_target);
functionMap[HASH_ELEMENT_DRAW] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__draw, &ColladaParserAutoGen15Private::_data__draw, &ColladaParserAutoGen15Private::_end__draw, &ColladaParserAutoGen15Private::_preBegin__draw, &ColladaParserAutoGen15Private::_preEnd__draw, &ColladaParserAutoGen15Private::_freeAttributes__draw);
functionMap[HASH_ELEMENT_STENCIL_CLEAR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__stencil_clear, &ColladaParserAutoGen15Private::_data__stencil_clear, &ColladaParserAutoGen15Private::_end__stencil_clear, &ColladaParserAutoGen15Private::_preBegin__stencil_clear, &ColladaParserAutoGen15Private::_preEnd__stencil_clear, &ColladaParserAutoGen15Private::_freeAttributes__stencil_clear);
functionMap[HASH_ELEMENT_STENCIL_TARGET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__stencil_target, &ColladaParserAutoGen15Private::_data__stencil_target, &ColladaParserAutoGen15Private::_end__stencil_target, &ColladaParserAutoGen15Private::_preBegin__stencil_target, &ColladaParserAutoGen15Private::_preEnd__stencil_target, &ColladaParserAutoGen15Private::_freeAttributes__stencil_target);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__ALPHA_FUNC] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__alpha_func, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__alpha_func, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__alpha_func, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__alpha_func, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__alpha_func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__alpha_func);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__ALPHA_TEST_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__alpha_test_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__alpha_test_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__alpha_test_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__alpha_test_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__alpha_test_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__alpha_test_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__BLEND_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__blend_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__blend_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__blend_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__blend_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__blend_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__blend_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__BLEND_FUNC] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__blend_func, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__blend_func, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__blend_func, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__blend_func, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__blend_func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__blend_func);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__CLIP_PLANE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__clip_plane, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__clip_plane, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__clip_plane, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__clip_plane, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__clip_plane, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__clip_plane);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__CLIP_PLANE_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__clip_plane_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__clip_plane_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__clip_plane_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__clip_plane_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__clip_plane_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__clip_plane_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__COLOR_LOGIC_OP_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__color_logic_op_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__color_logic_op_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__color_logic_op_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__color_logic_op_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__color_logic_op_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__color_logic_op_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__COLOR_MASK] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__color_mask, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__color_mask, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__color_mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__color_mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__color_mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__color_mask);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__COLOR_MATERIAL_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__color_material_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__color_material_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__color_material_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__color_material_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__color_material_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__color_material_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__CULL_FACE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__cull_face, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__cull_face, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__cull_face, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__cull_face, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__cull_face, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__cull_face);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__CULL_FACE_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__cull_face_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__cull_face_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__cull_face_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__cull_face_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__cull_face_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__cull_face_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__DEPTH_FUNC] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__depth_func, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__depth_func, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__depth_func, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__depth_func, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__depth_func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__depth_func);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__DEPTH_MASK] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__depth_mask, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__depth_mask, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__depth_mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__depth_mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__depth_mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__depth_mask);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__DEPTH_RANGE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__depth_range, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__depth_range, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__depth_range, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__depth_range, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__depth_range, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__depth_range);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__DEPTH_TEST_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__depth_test_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__depth_test_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__depth_test_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__depth_test_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__depth_test_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__depth_test_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__DITHER_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__dither_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__dither_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__dither_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__dither_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__dither_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__dither_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__FOG_COLOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__fog_color, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__fog_color, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__fog_color, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__fog_color, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__fog_color, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__fog_color);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__FOG_DENSITY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__fog_density, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__fog_density, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__fog_density, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__fog_density, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__fog_density, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__fog_density);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__FOG_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__fog_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__fog_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__fog_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__fog_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__fog_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__fog_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__FOG_END] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__fog_end, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__fog_end, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__fog_end, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__fog_end, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__fog_end, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__fog_end);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__FOG_MODE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__fog_mode, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__fog_mode, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__fog_mode, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__fog_mode, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__fog_mode, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__fog_mode);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__FOG_START] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__fog_start, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__fog_start, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__fog_start, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__fog_start, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__fog_start, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__fog_start);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__FRONT_FACE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__front_face, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__front_face, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__front_face, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__front_face, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__front_face, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__front_face);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_AMBIENT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__light_ambient, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__light_ambient, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__light_ambient, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__light_ambient, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__light_ambient, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__light_ambient);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_CONSTANT_ATTENUATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__light_constant_attenuation, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__light_constant_attenuation, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__light_constant_attenuation, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__light_constant_attenuation, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__light_constant_attenuation, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__light_constant_attenuation);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_DIFFUSE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__light_diffuse, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__light_diffuse, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__light_diffuse, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__light_diffuse, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__light_diffuse, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__light_diffuse);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__light_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__light_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__light_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__light_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__light_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__light_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_LINEAR_ATTENUATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__light_linear_attenuation, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__light_linear_attenuation, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__light_linear_attenuation, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__light_linear_attenuation, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__light_linear_attenuation, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__light_linear_attenuation);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_MODEL_AMBIENT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__light_model_ambient, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__light_model_ambient, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__light_model_ambient, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__light_model_ambient, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__light_model_ambient, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__light_model_ambient);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_MODEL_TWO_SIDE_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__light_model_two_side_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__light_model_two_side_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__light_model_two_side_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__light_model_two_side_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__light_model_two_side_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__light_model_two_side_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_POSITION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__light_position, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__light_position, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__light_position, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__light_position, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__light_position, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__light_position);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_QUADRATIC_ATTENUATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__light_quadratic_attenuation, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__light_quadratic_attenuation, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__light_quadratic_attenuation, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__light_quadratic_attenuation, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__light_quadratic_attenuation, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__light_quadratic_attenuation);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_SPECULAR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__light_specular, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__light_specular, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__light_specular, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__light_specular, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__light_specular, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__light_specular);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_SPOT_CUTOFF] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__light_spot_cutoff, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__light_spot_cutoff, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__light_spot_cutoff, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__light_spot_cutoff, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__light_spot_cutoff, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__light_spot_cutoff);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_SPOT_DIRECTION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__light_spot_direction, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__light_spot_direction, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__light_spot_direction, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__light_spot_direction, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__light_spot_direction, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__light_spot_direction);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHT_SPOT_EXPONENT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__light_spot_exponent, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__light_spot_exponent, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__light_spot_exponent, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__light_spot_exponent, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__light_spot_exponent, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__light_spot_exponent);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LIGHTING_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__lighting_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__lighting_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__lighting_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__lighting_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__lighting_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__lighting_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LINE_SMOOTH_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__line_smooth_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__line_smooth_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__line_smooth_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__line_smooth_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__line_smooth_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__line_smooth_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LINE_WIDTH] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__line_width, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__line_width, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__line_width, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__line_width, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__line_width, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__line_width);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__LOGIC_OP] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__logic_op, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__logic_op, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__logic_op, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__logic_op, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__logic_op, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__logic_op);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__MATERIAL_AMBIENT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__material_ambient, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__material_ambient, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__material_ambient, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__material_ambient, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__material_ambient, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__material_ambient);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__MATERIAL_DIFFUSE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__material_diffuse, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__material_diffuse, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__material_diffuse, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__material_diffuse, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__material_diffuse, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__material_diffuse);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__MATERIAL_EMISSION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__material_emission, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__material_emission, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__material_emission, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__material_emission, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__material_emission, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__material_emission);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__MATERIAL_SHININESS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__material_shininess, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__material_shininess, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__material_shininess, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__material_shininess, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__material_shininess, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__material_shininess);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__MATERIAL_SPECULAR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__material_specular, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__material_specular, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__material_specular, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__material_specular, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__material_specular, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__material_specular);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__MODEL_VIEW_MATRIX] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__model_view_matrix, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__model_view_matrix, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__model_view_matrix, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__model_view_matrix, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__model_view_matrix, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__model_view_matrix);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__MULTISAMPLE_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__multisample_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__multisample_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__multisample_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__multisample_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__multisample_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__multisample_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__NORMALIZE_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__normalize_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__normalize_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__normalize_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__normalize_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__normalize_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__normalize_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__POINT_DISTANCE_ATTENUATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__point_distance_attenuation, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__point_distance_attenuation, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__point_distance_attenuation, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__point_distance_attenuation, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__point_distance_attenuation, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__point_distance_attenuation);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__POINT_FADE_THRESHOLD_SIZE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__point_fade_threshold_size, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__point_fade_threshold_size, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__point_fade_threshold_size, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__point_fade_threshold_size, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__point_fade_threshold_size, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__point_fade_threshold_size);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__POINT_SIZE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__point_size, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__point_size, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__point_size, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__point_size, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__point_size, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__point_size);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__POINT_SIZE_MAX] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__point_size_max, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__point_size_max, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__point_size_max, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__point_size_max, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__point_size_max, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__point_size_max);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__POINT_SIZE_MIN] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__point_size_min, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__point_size_min, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__point_size_min, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__point_size_min, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__point_size_min, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__point_size_min);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__POINT_SMOOTH_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__point_smooth_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__point_smooth_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__point_smooth_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__point_smooth_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__point_smooth_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__point_smooth_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__POLYGON_OFFSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__polygon_offset, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__polygon_offset, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__polygon_offset, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__polygon_offset, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__polygon_offset, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__polygon_offset);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__POLYGON_OFFSET_FILL_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__polygon_offset_fill_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__polygon_offset_fill_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__polygon_offset_fill_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__polygon_offset_fill_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__polygon_offset_fill_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__polygon_offset_fill_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__PROJECTION_MATRIX] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__projection_matrix, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__projection_matrix, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__projection_matrix, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__projection_matrix, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__projection_matrix, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__projection_matrix);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__RESCALE_NORMAL_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__rescale_normal_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__rescale_normal_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__rescale_normal_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__rescale_normal_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__rescale_normal_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__rescale_normal_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__SAMPLE_ALPHA_TO_COVERAGE_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__sample_alpha_to_coverage_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__sample_alpha_to_coverage_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__sample_alpha_to_coverage_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__sample_alpha_to_coverage_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__sample_alpha_to_coverage_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__sample_alpha_to_coverage_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__SAMPLE_ALPHA_TO_ONE_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__sample_alpha_to_one_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__sample_alpha_to_one_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__sample_alpha_to_one_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__sample_alpha_to_one_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__sample_alpha_to_one_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__sample_alpha_to_one_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__SAMPLE_COVERAGE_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__sample_coverage_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__sample_coverage_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__sample_coverage_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__sample_coverage_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__sample_coverage_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__sample_coverage_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__SCISSOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__scissor, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__scissor, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__scissor, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__scissor, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__scissor, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__scissor);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__SCISSOR_TEST_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__scissor_test_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__scissor_test_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__scissor_test_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__scissor_test_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__scissor_test_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__scissor_test_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__SHADE_MODEL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__shade_model, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__shade_model, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__shade_model, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__shade_model, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__shade_model, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__shade_model);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__STENCIL_FUNC] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__stencil_func, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__stencil_func, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__stencil_func, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__stencil_func, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__stencil_func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__stencil_func);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__STENCIL_MASK] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__stencil_mask, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__stencil_mask, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__stencil_mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__stencil_mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__stencil_mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__stencil_mask);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__STENCIL_OP] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__stencil_op, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__stencil_op, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__stencil_op, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__stencil_op, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__stencil_op, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__stencil_op);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__STENCIL_TEST_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__stencil_test_enable, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__stencil_test_enable, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__stencil_test_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__stencil_test_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__stencil_test_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__stencil_test_enable);
functionMap[HASH_ELEMENT_TEXTURE_PIPELINE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__texture_pipeline, &ColladaParserAutoGen15Private::_data__texture_pipeline, &ColladaParserAutoGen15Private::_end__texture_pipeline, &ColladaParserAutoGen15Private::_preBegin__texture_pipeline, &ColladaParserAutoGen15Private::_preEnd__texture_pipeline, &ColladaParserAutoGen15Private::_freeAttributes__texture_pipeline);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__ALPHA_FUNC__FUNC] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__alpha_func__func, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__alpha_func__func, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__alpha_func__func, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__alpha_func__func, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__alpha_func__func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__alpha_func__func);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__ALPHA_FUNC__VALUE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__alpha_func__value, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__alpha_func__value, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__alpha_func__value, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__alpha_func__value, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__alpha_func__value, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__alpha_func__value);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__BLEND_FUNC__DEST] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__blend_func__dest, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__blend_func__dest, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__blend_func__dest, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__blend_func__dest, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__blend_func__dest, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__blend_func__dest);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__BLEND_FUNC__SRC] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__blend_func__src, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__blend_func__src, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__blend_func__src, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__blend_func__src, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__blend_func__src, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__blend_func__src);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__STENCIL_FUNC__FUNC] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__stencil_func__func, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__stencil_func__func, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__stencil_func__func, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__stencil_func__func, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__stencil_func__func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__stencil_func__func);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__STENCIL_FUNC__MASK] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__stencil_func__mask, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__stencil_func__mask, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__stencil_func__mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__stencil_func__mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__stencil_func__mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__stencil_func__mask);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__STENCIL_FUNC__REF] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__stencil_func__ref, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__stencil_func__ref, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__stencil_func__ref, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__stencil_func__ref, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__stencil_func__ref, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__stencil_func__ref);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__STENCIL_OP__FAIL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__stencil_op__fail, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__stencil_op__fail, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__stencil_op__fail, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__stencil_op__fail, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__stencil_op__fail, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__stencil_op__fail);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__STENCIL_OP__ZFAIL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__stencil_op__zfail, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__stencil_op__zfail, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__stencil_op__zfail, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__stencil_op__zfail, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__stencil_op__zfail, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__stencil_op__zfail);
functionMap[HASH_ELEMENT_PROFILE_GLES__TECHNIQUE__PASS__STATES__STENCIL_OP__ZPASS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES__technique__pass__states__stencil_op__zpass, &ColladaParserAutoGen15Private::_data__profile_GLES__technique__pass__states__stencil_op__zpass, &ColladaParserAutoGen15Private::_end__profile_GLES__technique__pass__states__stencil_op__zpass, &ColladaParserAutoGen15Private::_preBegin__profile_GLES__technique__pass__states__stencil_op__zpass, &ColladaParserAutoGen15Private::_preEnd__profile_GLES__technique__pass__states__stencil_op__zpass, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES__technique__pass__states__stencil_op__zpass);
functionMap[HASH_ELEMENT_VALUE____GLES_TEXTURE_PIPELINE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__value____gles_texture_pipeline_type, &ColladaParserAutoGen15Private::_data__value____gles_texture_pipeline_type, &ColladaParserAutoGen15Private::_end__value____gles_texture_pipeline_type, &ColladaParserAutoGen15Private::_preBegin__value____gles_texture_pipeline_type, &ColladaParserAutoGen15Private::_preEnd__value____gles_texture_pipeline_type, &ColladaParserAutoGen15Private::_freeAttributes__value____gles_texture_pipeline_type);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_TEXCOMBINER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__texcombiner, &ColladaParserAutoGen15Private::_data__texcombiner, &ColladaParserAutoGen15Private::_end__texcombiner, &ColladaParserAutoGen15Private::_preBegin__texcombiner, &ColladaParserAutoGen15Private::_preEnd__texcombiner, &ColladaParserAutoGen15Private::_freeAttributes__texcombiner);
functionMap[HASH_ELEMENT_TEXENV] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__texenv, &ColladaParserAutoGen15Private::_data__texenv, &ColladaParserAutoGen15Private::_end__texenv, &ColladaParserAutoGen15Private::_preBegin__texenv, &ColladaParserAutoGen15Private::_preEnd__texenv, &ColladaParserAutoGen15Private::_freeAttributes__texenv);
functionMap[HASH_ELEMENT_RGB] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__RGB, &ColladaParserAutoGen15Private::_data__RGB, &ColladaParserAutoGen15Private::_end__RGB, &ColladaParserAutoGen15Private::_preBegin__RGB, &ColladaParserAutoGen15Private::_preEnd__RGB, &ColladaParserAutoGen15Private::_freeAttributes__RGB);
functionMap[HASH_ELEMENT_ALPHA____GLES_TEXCOMBINER_COMMAND_ALPHA_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__alpha____gles_texcombiner_command_alpha_type, &ColladaParserAutoGen15Private::_data__alpha____gles_texcombiner_command_alpha_type, &ColladaParserAutoGen15Private::_end__alpha____gles_texcombiner_command_alpha_type, &ColladaParserAutoGen15Private::_preBegin__alpha____gles_texcombiner_command_alpha_type, &ColladaParserAutoGen15Private::_preEnd__alpha____gles_texcombiner_command_alpha_type, &ColladaParserAutoGen15Private::_freeAttributes__alpha____gles_texcombiner_command_alpha_type);
functionMap[HASH_ELEMENT_CONSTANT____GLES_TEXTURE_CONSTANT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__constant____gles_texture_constant_type, &ColladaParserAutoGen15Private::_data__constant____gles_texture_constant_type, &ColladaParserAutoGen15Private::_end__constant____gles_texture_constant_type, &ColladaParserAutoGen15Private::_preBegin__constant____gles_texture_constant_type, &ColladaParserAutoGen15Private::_preEnd__constant____gles_texture_constant_type, &ColladaParserAutoGen15Private::_freeAttributes__constant____gles_texture_constant_type);
functionMap[HASH_ELEMENT_ARGUMENT____GLES_TEXCOMBINER_ARGUMENT_RGB_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__argument____gles_texcombiner_argument_rgb_type, &ColladaParserAutoGen15Private::_data__argument____gles_texcombiner_argument_rgb_type, &ColladaParserAutoGen15Private::_end__argument____gles_texcombiner_argument_rgb_type, &ColladaParserAutoGen15Private::_preBegin__argument____gles_texcombiner_argument_rgb_type, &ColladaParserAutoGen15Private::_preEnd__argument____gles_texcombiner_argument_rgb_type, &ColladaParserAutoGen15Private::_freeAttributes__argument____gles_texcombiner_argument_rgb_type);
functionMap[HASH_ELEMENT_ARGUMENT____GLES_TEXCOMBINER_ARGUMENT_ALPHA_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__argument____gles_texcombiner_argument_alpha_type, &ColladaParserAutoGen15Private::_data__argument____gles_texcombiner_argument_alpha_type, &ColladaParserAutoGen15Private::_end__argument____gles_texcombiner_argument_alpha_type, &ColladaParserAutoGen15Private::_preBegin__argument____gles_texcombiner_argument_alpha_type, &ColladaParserAutoGen15Private::_preEnd__argument____gles_texcombiner_argument_alpha_type, &ColladaParserAutoGen15Private::_freeAttributes__argument____gles_texcombiner_argument_alpha_type);
functionMap[HASH_ELEMENT_CONSTANT____GLES_TEXTURE_CONSTANT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__constant____gles_texture_constant_type, &ColladaParserAutoGen15Private::_data__constant____gles_texture_constant_type, &ColladaParserAutoGen15Private::_end__constant____gles_texture_constant_type, &ColladaParserAutoGen15Private::_preBegin__constant____gles_texture_constant_type, &ColladaParserAutoGen15Private::_preEnd__constant____gles_texture_constant_type, &ColladaParserAutoGen15Private::_freeAttributes__constant____gles_texture_constant_type);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_CODE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__code, &ColladaParserAutoGen15Private::_data__code, &ColladaParserAutoGen15Private::_end__code, &ColladaParserAutoGen15Private::_preBegin__code, &ColladaParserAutoGen15Private::_preEnd__code, &ColladaParserAutoGen15Private::_freeAttributes__code);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INCLUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__include, &ColladaParserAutoGen15Private::_data__include, &ColladaParserAutoGen15Private::_end__include, &ColladaParserAutoGen15Private::_preBegin__include, &ColladaParserAutoGen15Private::_preEnd__include, &ColladaParserAutoGen15Private::_freeAttributes__include);
functionMap[HASH_ELEMENT_PROFILE_GLES2_TYPE____NEWPARAM] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_gles2_type____newparam, &ColladaParserAutoGen15Private::_data__profile_gles2_type____newparam, &ColladaParserAutoGen15Private::_end__profile_gles2_type____newparam, &ColladaParserAutoGen15Private::_preBegin__profile_gles2_type____newparam, &ColladaParserAutoGen15Private::_preEnd__profile_gles2_type____newparam, &ColladaParserAutoGen15Private::_freeAttributes__profile_gles2_type____newparam);
functionMap[HASH_ELEMENT_PROFILE_GLES2_TYPE____TECHNIQUE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_gles2_type____technique, &ColladaParserAutoGen15Private::_data__profile_gles2_type____technique, &ColladaParserAutoGen15Private::_end__profile_gles2_type____technique, &ColladaParserAutoGen15Private::_preBegin__profile_gles2_type____technique, &ColladaParserAutoGen15Private::_preEnd__profile_gles2_type____technique, &ColladaParserAutoGen15Private::_freeAttributes__profile_gles2_type____technique);
functionMap[HASH_ELEMENT_ANNOTATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__annotate, &ColladaParserAutoGen15Private::_data__annotate, &ColladaParserAutoGen15Private::_end__annotate, &ColladaParserAutoGen15Private::_preBegin__annotate, &ColladaParserAutoGen15Private::_preEnd__annotate, &ColladaParserAutoGen15Private::_freeAttributes__annotate);
functionMap[HASH_ELEMENT_PROFILE_GLES2__NEWPARAM__USERTYPE__SETPARAM__ARRAY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__newparam__usertype__setparam__array, &ColladaParserAutoGen15Private::_data__profile_GLES2__newparam__usertype__setparam__array, &ColladaParserAutoGen15Private::_end__profile_GLES2__newparam__usertype__setparam__array, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__newparam__usertype__setparam__array, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__newparam__usertype__setparam__array, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__newparam__usertype__setparam__array);
functionMap[HASH_ELEMENT_BOOL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool, &ColladaParserAutoGen15Private::_data__bool, &ColladaParserAutoGen15Private::_end__bool, &ColladaParserAutoGen15Private::_preBegin__bool, &ColladaParserAutoGen15Private::_preEnd__bool, &ColladaParserAutoGen15Private::_freeAttributes__bool);
functionMap[HASH_ELEMENT_BVEC2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bvec2, &ColladaParserAutoGen15Private::_data__bvec2, &ColladaParserAutoGen15Private::_end__bvec2, &ColladaParserAutoGen15Private::_preBegin__bvec2, &ColladaParserAutoGen15Private::_preEnd__bvec2, &ColladaParserAutoGen15Private::_freeAttributes__bvec2);
functionMap[HASH_ELEMENT_BVEC3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bvec3, &ColladaParserAutoGen15Private::_data__bvec3, &ColladaParserAutoGen15Private::_end__bvec3, &ColladaParserAutoGen15Private::_preBegin__bvec3, &ColladaParserAutoGen15Private::_preEnd__bvec3, &ColladaParserAutoGen15Private::_freeAttributes__bvec3);
functionMap[HASH_ELEMENT_BVEC4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bvec4, &ColladaParserAutoGen15Private::_data__bvec4, &ColladaParserAutoGen15Private::_end__bvec4, &ColladaParserAutoGen15Private::_preBegin__bvec4, &ColladaParserAutoGen15Private::_preEnd__bvec4, &ColladaParserAutoGen15Private::_freeAttributes__bvec4);
functionMap[HASH_ELEMENT_ENUM____GL_ENUMERATION_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_data__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_end__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_preBegin__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_preEnd__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_freeAttributes__enum____gl_enumeration_type);
functionMap[HASH_ELEMENT_FLOAT____FLOAT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float____float_type, &ColladaParserAutoGen15Private::_data__float____float_type, &ColladaParserAutoGen15Private::_end__float____float_type, &ColladaParserAutoGen15Private::_preBegin__float____float_type, &ColladaParserAutoGen15Private::_preEnd__float____float_type, &ColladaParserAutoGen15Private::_freeAttributes__float____float_type);
functionMap[HASH_ELEMENT_INT____INT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int____int_type, &ColladaParserAutoGen15Private::_data__int____int_type, &ColladaParserAutoGen15Private::_end__int____int_type, &ColladaParserAutoGen15Private::_preBegin__int____int_type, &ColladaParserAutoGen15Private::_preEnd__int____int_type, &ColladaParserAutoGen15Private::_freeAttributes__int____int_type);
functionMap[HASH_ELEMENT_IVEC2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__ivec2, &ColladaParserAutoGen15Private::_data__ivec2, &ColladaParserAutoGen15Private::_end__ivec2, &ColladaParserAutoGen15Private::_preBegin__ivec2, &ColladaParserAutoGen15Private::_preEnd__ivec2, &ColladaParserAutoGen15Private::_freeAttributes__ivec2);
functionMap[HASH_ELEMENT_IVEC3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__ivec3, &ColladaParserAutoGen15Private::_data__ivec3, &ColladaParserAutoGen15Private::_end__ivec3, &ColladaParserAutoGen15Private::_preBegin__ivec3, &ColladaParserAutoGen15Private::_preEnd__ivec3, &ColladaParserAutoGen15Private::_freeAttributes__ivec3);
functionMap[HASH_ELEMENT_IVEC4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__ivec4, &ColladaParserAutoGen15Private::_data__ivec4, &ColladaParserAutoGen15Private::_end__ivec4, &ColladaParserAutoGen15Private::_preBegin__ivec4, &ColladaParserAutoGen15Private::_preEnd__ivec4, &ColladaParserAutoGen15Private::_freeAttributes__ivec4);
functionMap[HASH_ELEMENT_MAT2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__mat2, &ColladaParserAutoGen15Private::_data__mat2, &ColladaParserAutoGen15Private::_end__mat2, &ColladaParserAutoGen15Private::_preBegin__mat2, &ColladaParserAutoGen15Private::_preEnd__mat2, &ColladaParserAutoGen15Private::_freeAttributes__mat2);
functionMap[HASH_ELEMENT_MAT3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__mat3, &ColladaParserAutoGen15Private::_data__mat3, &ColladaParserAutoGen15Private::_end__mat3, &ColladaParserAutoGen15Private::_preBegin__mat3, &ColladaParserAutoGen15Private::_preEnd__mat3, &ColladaParserAutoGen15Private::_freeAttributes__mat3);
functionMap[HASH_ELEMENT_MAT4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__mat4, &ColladaParserAutoGen15Private::_data__mat4, &ColladaParserAutoGen15Private::_end__mat4, &ColladaParserAutoGen15Private::_preBegin__mat4, &ColladaParserAutoGen15Private::_preEnd__mat4, &ColladaParserAutoGen15Private::_freeAttributes__mat4);
functionMap[HASH_ELEMENT_MODIFIER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modifier, &ColladaParserAutoGen15Private::_data__modifier, &ColladaParserAutoGen15Private::_end__modifier, &ColladaParserAutoGen15Private::_preBegin__modifier, &ColladaParserAutoGen15Private::_preEnd__modifier, &ColladaParserAutoGen15Private::_freeAttributes__modifier);
functionMap[HASH_ELEMENT_SAMPLER2D____FX_SAMPLER2D_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_data__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_end__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_preBegin__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_preEnd__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_freeAttributes__sampler2D____fx_sampler2D_type);
functionMap[HASH_ELEMENT_SAMPLER3D] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler3D, &ColladaParserAutoGen15Private::_data__sampler3D, &ColladaParserAutoGen15Private::_end__sampler3D, &ColladaParserAutoGen15Private::_preBegin__sampler3D, &ColladaParserAutoGen15Private::_preEnd__sampler3D, &ColladaParserAutoGen15Private::_freeAttributes__sampler3D);
functionMap[HASH_ELEMENT_SAMPLERCUBE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__samplerCUBE, &ColladaParserAutoGen15Private::_data__samplerCUBE, &ColladaParserAutoGen15Private::_end__samplerCUBE, &ColladaParserAutoGen15Private::_preBegin__samplerCUBE, &ColladaParserAutoGen15Private::_preEnd__samplerCUBE, &ColladaParserAutoGen15Private::_freeAttributes__samplerCUBE);
functionMap[HASH_ELEMENT_SAMPLERDEPTH] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__samplerDEPTH, &ColladaParserAutoGen15Private::_data__samplerDEPTH, &ColladaParserAutoGen15Private::_end__samplerDEPTH, &ColladaParserAutoGen15Private::_preBegin__samplerDEPTH, &ColladaParserAutoGen15Private::_preEnd__samplerDEPTH, &ColladaParserAutoGen15Private::_freeAttributes__samplerDEPTH);
functionMap[HASH_ELEMENT_SEMANTIC____NCNAME] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__semantic____NCName, &ColladaParserAutoGen15Private::_data__semantic____NCName, &ColladaParserAutoGen15Private::_end__semantic____NCName, &ColladaParserAutoGen15Private::_preBegin__semantic____NCName, &ColladaParserAutoGen15Private::_preEnd__semantic____NCName, &ColladaParserAutoGen15Private::_freeAttributes__semantic____NCName);
functionMap[HASH_ELEMENT_PROFILE_GLES2__NEWPARAM__USERTYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__newparam__usertype, &ColladaParserAutoGen15Private::_data__profile_GLES2__newparam__usertype, &ColladaParserAutoGen15Private::_end__profile_GLES2__newparam__usertype, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__newparam__usertype, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__newparam__usertype, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__newparam__usertype);
functionMap[HASH_ELEMENT_VEC2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__vec2, &ColladaParserAutoGen15Private::_data__vec2, &ColladaParserAutoGen15Private::_end__vec2, &ColladaParserAutoGen15Private::_preBegin__vec2, &ColladaParserAutoGen15Private::_preEnd__vec2, &ColladaParserAutoGen15Private::_freeAttributes__vec2);
functionMap[HASH_ELEMENT_VEC3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__vec3, &ColladaParserAutoGen15Private::_data__vec3, &ColladaParserAutoGen15Private::_end__vec3, &ColladaParserAutoGen15Private::_preBegin__vec3, &ColladaParserAutoGen15Private::_preEnd__vec3, &ColladaParserAutoGen15Private::_freeAttributes__vec3);
functionMap[HASH_ELEMENT_VEC4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__vec4, &ColladaParserAutoGen15Private::_data__vec4, &ColladaParserAutoGen15Private::_end__vec4, &ColladaParserAutoGen15Private::_preBegin__vec4, &ColladaParserAutoGen15Private::_preEnd__vec4, &ColladaParserAutoGen15Private::_freeAttributes__vec4);
functionMap[HASH_ELEMENT_BOOL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool, &ColladaParserAutoGen15Private::_data__bool, &ColladaParserAutoGen15Private::_end__bool, &ColladaParserAutoGen15Private::_preBegin__bool, &ColladaParserAutoGen15Private::_preEnd__bool, &ColladaParserAutoGen15Private::_freeAttributes__bool);
functionMap[HASH_ELEMENT_BVEC2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bvec2, &ColladaParserAutoGen15Private::_data__bvec2, &ColladaParserAutoGen15Private::_end__bvec2, &ColladaParserAutoGen15Private::_preBegin__bvec2, &ColladaParserAutoGen15Private::_preEnd__bvec2, &ColladaParserAutoGen15Private::_freeAttributes__bvec2);
functionMap[HASH_ELEMENT_BVEC3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bvec3, &ColladaParserAutoGen15Private::_data__bvec3, &ColladaParserAutoGen15Private::_end__bvec3, &ColladaParserAutoGen15Private::_preBegin__bvec3, &ColladaParserAutoGen15Private::_preEnd__bvec3, &ColladaParserAutoGen15Private::_freeAttributes__bvec3);
functionMap[HASH_ELEMENT_BVEC4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bvec4, &ColladaParserAutoGen15Private::_data__bvec4, &ColladaParserAutoGen15Private::_end__bvec4, &ColladaParserAutoGen15Private::_preBegin__bvec4, &ColladaParserAutoGen15Private::_preEnd__bvec4, &ColladaParserAutoGen15Private::_freeAttributes__bvec4);
functionMap[HASH_ELEMENT_ENUM____GL_ENUMERATION_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_data__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_end__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_preBegin__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_preEnd__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_freeAttributes__enum____gl_enumeration_type);
functionMap[HASH_ELEMENT_FLOAT____FLOAT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float____float_type, &ColladaParserAutoGen15Private::_data__float____float_type, &ColladaParserAutoGen15Private::_end__float____float_type, &ColladaParserAutoGen15Private::_preBegin__float____float_type, &ColladaParserAutoGen15Private::_preEnd__float____float_type, &ColladaParserAutoGen15Private::_freeAttributes__float____float_type);
functionMap[HASH_ELEMENT_INT____INT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int____int_type, &ColladaParserAutoGen15Private::_data__int____int_type, &ColladaParserAutoGen15Private::_end__int____int_type, &ColladaParserAutoGen15Private::_preBegin__int____int_type, &ColladaParserAutoGen15Private::_preEnd__int____int_type, &ColladaParserAutoGen15Private::_freeAttributes__int____int_type);
functionMap[HASH_ELEMENT_IVEC2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__ivec2, &ColladaParserAutoGen15Private::_data__ivec2, &ColladaParserAutoGen15Private::_end__ivec2, &ColladaParserAutoGen15Private::_preBegin__ivec2, &ColladaParserAutoGen15Private::_preEnd__ivec2, &ColladaParserAutoGen15Private::_freeAttributes__ivec2);
functionMap[HASH_ELEMENT_IVEC3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__ivec3, &ColladaParserAutoGen15Private::_data__ivec3, &ColladaParserAutoGen15Private::_end__ivec3, &ColladaParserAutoGen15Private::_preBegin__ivec3, &ColladaParserAutoGen15Private::_preEnd__ivec3, &ColladaParserAutoGen15Private::_freeAttributes__ivec3);
functionMap[HASH_ELEMENT_IVEC4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__ivec4, &ColladaParserAutoGen15Private::_data__ivec4, &ColladaParserAutoGen15Private::_end__ivec4, &ColladaParserAutoGen15Private::_preBegin__ivec4, &ColladaParserAutoGen15Private::_preEnd__ivec4, &ColladaParserAutoGen15Private::_freeAttributes__ivec4);
functionMap[HASH_ELEMENT_MAT2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__mat2, &ColladaParserAutoGen15Private::_data__mat2, &ColladaParserAutoGen15Private::_end__mat2, &ColladaParserAutoGen15Private::_preBegin__mat2, &ColladaParserAutoGen15Private::_preEnd__mat2, &ColladaParserAutoGen15Private::_freeAttributes__mat2);
functionMap[HASH_ELEMENT_MAT3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__mat3, &ColladaParserAutoGen15Private::_data__mat3, &ColladaParserAutoGen15Private::_end__mat3, &ColladaParserAutoGen15Private::_preBegin__mat3, &ColladaParserAutoGen15Private::_preEnd__mat3, &ColladaParserAutoGen15Private::_freeAttributes__mat3);
functionMap[HASH_ELEMENT_MAT4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__mat4, &ColladaParserAutoGen15Private::_data__mat4, &ColladaParserAutoGen15Private::_end__mat4, &ColladaParserAutoGen15Private::_preBegin__mat4, &ColladaParserAutoGen15Private::_preEnd__mat4, &ColladaParserAutoGen15Private::_freeAttributes__mat4);
functionMap[HASH_ELEMENT_SAMPLER2D____FX_SAMPLER2D_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_data__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_end__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_preBegin__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_preEnd__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_freeAttributes__sampler2D____fx_sampler2D_type);
functionMap[HASH_ELEMENT_SAMPLER3D] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler3D, &ColladaParserAutoGen15Private::_data__sampler3D, &ColladaParserAutoGen15Private::_end__sampler3D, &ColladaParserAutoGen15Private::_preBegin__sampler3D, &ColladaParserAutoGen15Private::_preEnd__sampler3D, &ColladaParserAutoGen15Private::_freeAttributes__sampler3D);
functionMap[HASH_ELEMENT_SAMPLERCUBE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__samplerCUBE, &ColladaParserAutoGen15Private::_data__samplerCUBE, &ColladaParserAutoGen15Private::_end__samplerCUBE, &ColladaParserAutoGen15Private::_preBegin__samplerCUBE, &ColladaParserAutoGen15Private::_preEnd__samplerCUBE, &ColladaParserAutoGen15Private::_freeAttributes__samplerCUBE);
functionMap[HASH_ELEMENT_SAMPLERDEPTH] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__samplerDEPTH, &ColladaParserAutoGen15Private::_data__samplerDEPTH, &ColladaParserAutoGen15Private::_end__samplerDEPTH, &ColladaParserAutoGen15Private::_preBegin__samplerDEPTH, &ColladaParserAutoGen15Private::_preEnd__samplerDEPTH, &ColladaParserAutoGen15Private::_freeAttributes__samplerDEPTH);
functionMap[HASH_ELEMENT_PROFILE_GLES2__NEWPARAM__USERTYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__newparam__usertype, &ColladaParserAutoGen15Private::_data__profile_GLES2__newparam__usertype, &ColladaParserAutoGen15Private::_end__profile_GLES2__newparam__usertype, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__newparam__usertype, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__newparam__usertype, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__newparam__usertype);
functionMap[HASH_ELEMENT_VEC2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__vec2, &ColladaParserAutoGen15Private::_data__vec2, &ColladaParserAutoGen15Private::_end__vec2, &ColladaParserAutoGen15Private::_preBegin__vec2, &ColladaParserAutoGen15Private::_preEnd__vec2, &ColladaParserAutoGen15Private::_freeAttributes__vec2);
functionMap[HASH_ELEMENT_VEC3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__vec3, &ColladaParserAutoGen15Private::_data__vec3, &ColladaParserAutoGen15Private::_end__vec3, &ColladaParserAutoGen15Private::_preBegin__vec3, &ColladaParserAutoGen15Private::_preEnd__vec3, &ColladaParserAutoGen15Private::_freeAttributes__vec3);
functionMap[HASH_ELEMENT_VEC4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__vec4, &ColladaParserAutoGen15Private::_data__vec4, &ColladaParserAutoGen15Private::_end__vec4, &ColladaParserAutoGen15Private::_preBegin__vec4, &ColladaParserAutoGen15Private::_preEnd__vec4, &ColladaParserAutoGen15Private::_freeAttributes__vec4);
functionMap[HASH_ELEMENT_PROFILE_GLES2__NEWPARAM__USERTYPE__SETPARAM] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__newparam__usertype__setparam, &ColladaParserAutoGen15Private::_data__profile_GLES2__newparam__usertype__setparam, &ColladaParserAutoGen15Private::_end__profile_GLES2__newparam__usertype__setparam, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__newparam__usertype__setparam, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__newparam__usertype__setparam, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__newparam__usertype__setparam);
functionMap[HASH_ELEMENT_VEC2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__vec2, &ColladaParserAutoGen15Private::_data__vec2, &ColladaParserAutoGen15Private::_end__vec2, &ColladaParserAutoGen15Private::_preBegin__vec2, &ColladaParserAutoGen15Private::_preEnd__vec2, &ColladaParserAutoGen15Private::_freeAttributes__vec2);
functionMap[HASH_ELEMENT_VEC3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__vec3, &ColladaParserAutoGen15Private::_data__vec3, &ColladaParserAutoGen15Private::_end__vec3, &ColladaParserAutoGen15Private::_preBegin__vec3, &ColladaParserAutoGen15Private::_preEnd__vec3, &ColladaParserAutoGen15Private::_freeAttributes__vec3);
functionMap[HASH_ELEMENT_VEC4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__vec4, &ColladaParserAutoGen15Private::_data__vec4, &ColladaParserAutoGen15Private::_end__vec4, &ColladaParserAutoGen15Private::_preBegin__vec4, &ColladaParserAutoGen15Private::_preEnd__vec4, &ColladaParserAutoGen15Private::_freeAttributes__vec4);
functionMap[HASH_ELEMENT_ANNOTATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__annotate, &ColladaParserAutoGen15Private::_data__annotate, &ColladaParserAutoGen15Private::_end__annotate, &ColladaParserAutoGen15Private::_preBegin__annotate, &ColladaParserAutoGen15Private::_preEnd__annotate, &ColladaParserAutoGen15Private::_freeAttributes__annotate);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_PASS____GLES2_PASS_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__pass____gles2_pass_type, &ColladaParserAutoGen15Private::_data__pass____gles2_pass_type, &ColladaParserAutoGen15Private::_end__pass____gles2_pass_type, &ColladaParserAutoGen15Private::_preBegin__pass____gles2_pass_type, &ColladaParserAutoGen15Private::_preEnd__pass____gles2_pass_type, &ColladaParserAutoGen15Private::_freeAttributes__pass____gles2_pass_type);
functionMap[HASH_ELEMENT_ANNOTATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__annotate, &ColladaParserAutoGen15Private::_data__annotate, &ColladaParserAutoGen15Private::_end__annotate, &ColladaParserAutoGen15Private::_preBegin__annotate, &ColladaParserAutoGen15Private::_preEnd__annotate, &ColladaParserAutoGen15Private::_freeAttributes__annotate);
functionMap[HASH_ELEMENT_GLES2_PASS_TYPE____EVALUATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__gles2_pass_type____evaluate, &ColladaParserAutoGen15Private::_data__gles2_pass_type____evaluate, &ColladaParserAutoGen15Private::_end__gles2_pass_type____evaluate, &ColladaParserAutoGen15Private::_preBegin__gles2_pass_type____evaluate, &ColladaParserAutoGen15Private::_preEnd__gles2_pass_type____evaluate, &ColladaParserAutoGen15Private::_freeAttributes__gles2_pass_type____evaluate);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_PROGRAM____GLES2_PROGRAM_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__program____gles2_program_type, &ColladaParserAutoGen15Private::_data__program____gles2_program_type, &ColladaParserAutoGen15Private::_end__program____gles2_program_type, &ColladaParserAutoGen15Private::_preBegin__program____gles2_program_type, &ColladaParserAutoGen15Private::_preEnd__program____gles2_program_type, &ColladaParserAutoGen15Private::_freeAttributes__program____gles2_program_type);
functionMap[HASH_ELEMENT_GLES2_PASS_TYPE____STATES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__gles2_pass_type____states, &ColladaParserAutoGen15Private::_data__gles2_pass_type____states, &ColladaParserAutoGen15Private::_end__gles2_pass_type____states, &ColladaParserAutoGen15Private::_preBegin__gles2_pass_type____states, &ColladaParserAutoGen15Private::_preEnd__gles2_pass_type____states, &ColladaParserAutoGen15Private::_freeAttributes__gles2_pass_type____states);
functionMap[HASH_ELEMENT_COLOR_CLEAR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__color_clear, &ColladaParserAutoGen15Private::_data__color_clear, &ColladaParserAutoGen15Private::_end__color_clear, &ColladaParserAutoGen15Private::_preBegin__color_clear, &ColladaParserAutoGen15Private::_preEnd__color_clear, &ColladaParserAutoGen15Private::_freeAttributes__color_clear);
functionMap[HASH_ELEMENT_COLOR_TARGET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__color_target, &ColladaParserAutoGen15Private::_data__color_target, &ColladaParserAutoGen15Private::_end__color_target, &ColladaParserAutoGen15Private::_preBegin__color_target, &ColladaParserAutoGen15Private::_preEnd__color_target, &ColladaParserAutoGen15Private::_freeAttributes__color_target);
functionMap[HASH_ELEMENT_DEPTH_CLEAR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__depth_clear, &ColladaParserAutoGen15Private::_data__depth_clear, &ColladaParserAutoGen15Private::_end__depth_clear, &ColladaParserAutoGen15Private::_preBegin__depth_clear, &ColladaParserAutoGen15Private::_preEnd__depth_clear, &ColladaParserAutoGen15Private::_freeAttributes__depth_clear);
functionMap[HASH_ELEMENT_DEPTH_TARGET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__depth_target, &ColladaParserAutoGen15Private::_data__depth_target, &ColladaParserAutoGen15Private::_end__depth_target, &ColladaParserAutoGen15Private::_preBegin__depth_target, &ColladaParserAutoGen15Private::_preEnd__depth_target, &ColladaParserAutoGen15Private::_freeAttributes__depth_target);
functionMap[HASH_ELEMENT_DRAW] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__draw, &ColladaParserAutoGen15Private::_data__draw, &ColladaParserAutoGen15Private::_end__draw, &ColladaParserAutoGen15Private::_preBegin__draw, &ColladaParserAutoGen15Private::_preEnd__draw, &ColladaParserAutoGen15Private::_freeAttributes__draw);
functionMap[HASH_ELEMENT_STENCIL_CLEAR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__stencil_clear, &ColladaParserAutoGen15Private::_data__stencil_clear, &ColladaParserAutoGen15Private::_end__stencil_clear, &ColladaParserAutoGen15Private::_preBegin__stencil_clear, &ColladaParserAutoGen15Private::_preEnd__stencil_clear, &ColladaParserAutoGen15Private::_freeAttributes__stencil_clear);
functionMap[HASH_ELEMENT_STENCIL_TARGET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__stencil_target, &ColladaParserAutoGen15Private::_data__stencil_target, &ColladaParserAutoGen15Private::_end__stencil_target, &ColladaParserAutoGen15Private::_preBegin__stencil_target, &ColladaParserAutoGen15Private::_preEnd__stencil_target, &ColladaParserAutoGen15Private::_freeAttributes__stencil_target);
functionMap[HASH_ELEMENT_GLES2_PROGRAM_TYPE____BIND_ATTRIBUTE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__gles2_program_type____bind_attribute, &ColladaParserAutoGen15Private::_data__gles2_program_type____bind_attribute, &ColladaParserAutoGen15Private::_end__gles2_program_type____bind_attribute, &ColladaParserAutoGen15Private::_preBegin__gles2_program_type____bind_attribute, &ColladaParserAutoGen15Private::_preEnd__gles2_program_type____bind_attribute, &ColladaParserAutoGen15Private::_freeAttributes__gles2_program_type____bind_attribute);
functionMap[HASH_ELEMENT_GLES2_PROGRAM_TYPE____BIND_UNIFORM] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__gles2_program_type____bind_uniform, &ColladaParserAutoGen15Private::_data__gles2_program_type____bind_uniform, &ColladaParserAutoGen15Private::_end__gles2_program_type____bind_uniform, &ColladaParserAutoGen15Private::_preBegin__gles2_program_type____bind_uniform, &ColladaParserAutoGen15Private::_preEnd__gles2_program_type____bind_uniform, &ColladaParserAutoGen15Private::_freeAttributes__gles2_program_type____bind_uniform);
functionMap[HASH_ELEMENT_LINKER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__linker, &ColladaParserAutoGen15Private::_data__linker, &ColladaParserAutoGen15Private::_end__linker, &ColladaParserAutoGen15Private::_preBegin__linker, &ColladaParserAutoGen15Private::_preEnd__linker, &ColladaParserAutoGen15Private::_freeAttributes__linker);
functionMap[HASH_ELEMENT_SHADER____GLES2_SHADER_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__shader____gles2_shader_type, &ColladaParserAutoGen15Private::_data__shader____gles2_shader_type, &ColladaParserAutoGen15Private::_end__shader____gles2_shader_type, &ColladaParserAutoGen15Private::_preBegin__shader____gles2_shader_type, &ColladaParserAutoGen15Private::_preEnd__shader____gles2_shader_type, &ColladaParserAutoGen15Private::_freeAttributes__shader____gles2_shader_type);
functionMap[HASH_ELEMENT_SEMANTIC____TOKEN] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__semantic____token, &ColladaParserAutoGen15Private::_data__semantic____token, &ColladaParserAutoGen15Private::_end__semantic____token, &ColladaParserAutoGen15Private::_preBegin__semantic____token, &ColladaParserAutoGen15Private::_preEnd__semantic____token, &ColladaParserAutoGen15Private::_freeAttributes__semantic____token);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__PROGRAM__BIND_UNIFORM__PARAM] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__program__bind_uniform__param, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__program__bind_uniform__param, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__program__bind_uniform__param, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__program__bind_uniform__param, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__program__bind_uniform__param, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__program__bind_uniform__param);
functionMap[HASH_ELEMENT_COMPILER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__compiler, &ColladaParserAutoGen15Private::_data__compiler, &ColladaParserAutoGen15Private::_end__compiler, &ColladaParserAutoGen15Private::_preBegin__compiler, &ColladaParserAutoGen15Private::_preEnd__compiler, &ColladaParserAutoGen15Private::_freeAttributes__compiler);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_GLES2_SHADER_TYPE____SOURCES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__gles2_shader_type____sources, &ColladaParserAutoGen15Private::_data__gles2_shader_type____sources, &ColladaParserAutoGen15Private::_end__gles2_shader_type____sources, &ColladaParserAutoGen15Private::_preBegin__gles2_shader_type____sources, &ColladaParserAutoGen15Private::_preEnd__gles2_shader_type____sources, &ColladaParserAutoGen15Private::_freeAttributes__gles2_shader_type____sources);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_COLOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__blend_color, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__blend_color, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__blend_color, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__blend_color, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__blend_color, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__blend_color);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__blend_enable, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__blend_enable, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__blend_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__blend_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__blend_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__blend_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_EQUATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__blend_equation, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__blend_equation, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__blend_equation, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__blend_equation, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__blend_equation, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__blend_equation);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_EQUATION_SEPARATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__blend_equation_separate, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__blend_equation_separate, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__blend_equation_separate, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__blend_equation_separate, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__blend_equation_separate, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__blend_equation_separate);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_FUNC] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__blend_func, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__blend_func, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__blend_func, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__blend_func, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__blend_func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__blend_func);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_FUNC_SEPARATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__blend_func_separate, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__blend_func_separate, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__blend_func_separate, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__blend_func_separate, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__blend_func_separate, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__blend_func_separate);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__COLOR_MASK] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__color_mask, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__color_mask, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__color_mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__color_mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__color_mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__color_mask);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__CULL_FACE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__cull_face, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__cull_face, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__cull_face, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__cull_face, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__cull_face, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__cull_face);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__CULL_FACE_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__cull_face_enable, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__cull_face_enable, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__cull_face_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__cull_face_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__cull_face_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__cull_face_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__DEPTH_FUNC] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__depth_func, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__depth_func, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__depth_func, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__depth_func, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__depth_func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__depth_func);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__DEPTH_MASK] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__depth_mask, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__depth_mask, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__depth_mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__depth_mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__depth_mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__depth_mask);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__DEPTH_RANGE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__depth_range, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__depth_range, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__depth_range, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__depth_range, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__depth_range, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__depth_range);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__DEPTH_TEST_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__depth_test_enable, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__depth_test_enable, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__depth_test_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__depth_test_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__depth_test_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__depth_test_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__DITHER_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__dither_enable, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__dither_enable, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__dither_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__dither_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__dither_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__dither_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__FRONT_FACE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__front_face, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__front_face, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__front_face, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__front_face, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__front_face, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__front_face);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__LINE_WIDTH] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__line_width, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__line_width, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__line_width, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__line_width, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__line_width, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__line_width);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__POINT_SIZE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__point_size, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__point_size, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__point_size, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__point_size, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__point_size, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__point_size);
functionMap[HASH_ELEMENT_POINT_SIZE_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__point_size_enable, &ColladaParserAutoGen15Private::_data__point_size_enable, &ColladaParserAutoGen15Private::_end__point_size_enable, &ColladaParserAutoGen15Private::_preBegin__point_size_enable, &ColladaParserAutoGen15Private::_preEnd__point_size_enable, &ColladaParserAutoGen15Private::_freeAttributes__point_size_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__POLYGON_OFFSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__polygon_offset, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__polygon_offset, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__polygon_offset, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__polygon_offset, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__polygon_offset, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__polygon_offset);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__POLYGON_OFFSET_FILL_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__polygon_offset_fill_enable, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__polygon_offset_fill_enable, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__polygon_offset_fill_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__polygon_offset_fill_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__polygon_offset_fill_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__polygon_offset_fill_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__SAMPLE_ALPHA_TO_COVERAGE_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__sample_alpha_to_coverage_enable, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__sample_alpha_to_coverage_enable, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__sample_alpha_to_coverage_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__sample_alpha_to_coverage_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__sample_alpha_to_coverage_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__sample_alpha_to_coverage_enable);
functionMap[HASH_ELEMENT_SAMPLE_COVERAGE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sample_coverage, &ColladaParserAutoGen15Private::_data__sample_coverage, &ColladaParserAutoGen15Private::_end__sample_coverage, &ColladaParserAutoGen15Private::_preBegin__sample_coverage, &ColladaParserAutoGen15Private::_preEnd__sample_coverage, &ColladaParserAutoGen15Private::_freeAttributes__sample_coverage);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__SAMPLE_COVERAGE_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__sample_coverage_enable, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__sample_coverage_enable, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__sample_coverage_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__sample_coverage_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__sample_coverage_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__sample_coverage_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__SCISSOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__scissor, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__scissor, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__scissor, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__scissor, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__scissor, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__scissor);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__SCISSOR_TEST_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__scissor_test_enable, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__scissor_test_enable, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__scissor_test_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__scissor_test_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__scissor_test_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__scissor_test_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_FUNC] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_func, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_func, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_func, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_func, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_func);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_FUNC_SEPARATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_func_separate, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_func_separate, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_func_separate, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_func_separate, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_func_separate, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_func_separate);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_MASK] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_mask, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_mask, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_mask);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_MASK_SEPARATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_mask_separate, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_mask_separate, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_mask_separate, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_mask_separate, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_mask_separate, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_mask_separate);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_OP] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_op, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_op, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_op, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_op, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_op, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_op);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_OP_SEPARATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_op_separate, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_op_separate, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_op_separate, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_op_separate, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_op_separate, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_op_separate);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_TEST_ENABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_test_enable, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_test_enable, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_test_enable, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_test_enable, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_test_enable, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_test_enable);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_EQUATION_SEPARATE__ALPHA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__blend_equation_separate__alpha, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__blend_equation_separate__alpha, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__blend_equation_separate__alpha, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__blend_equation_separate__alpha, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__blend_equation_separate__alpha, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__blend_equation_separate__alpha);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_EQUATION_SEPARATE__RGB] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__blend_equation_separate__rgb, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__blend_equation_separate__rgb, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__blend_equation_separate__rgb, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__blend_equation_separate__rgb, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__blend_equation_separate__rgb, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__blend_equation_separate__rgb);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_FUNC__DEST] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__blend_func__dest, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__blend_func__dest, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__blend_func__dest, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__blend_func__dest, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__blend_func__dest, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__blend_func__dest);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_FUNC__SRC] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__blend_func__src, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__blend_func__src, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__blend_func__src, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__blend_func__src, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__blend_func__src, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__blend_func__src);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_FUNC_SEPARATE__DEST_ALPHA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__blend_func_separate__dest_alpha, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__blend_func_separate__dest_alpha, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__blend_func_separate__dest_alpha, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__blend_func_separate__dest_alpha, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__blend_func_separate__dest_alpha, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__blend_func_separate__dest_alpha);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_FUNC_SEPARATE__DEST_RGB] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__blend_func_separate__dest_rgb, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__blend_func_separate__dest_rgb, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__blend_func_separate__dest_rgb, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__blend_func_separate__dest_rgb, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__blend_func_separate__dest_rgb, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__blend_func_separate__dest_rgb);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_FUNC_SEPARATE__SRC_ALPHA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__blend_func_separate__src_alpha, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__blend_func_separate__src_alpha, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__blend_func_separate__src_alpha, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__blend_func_separate__src_alpha, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__blend_func_separate__src_alpha, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__blend_func_separate__src_alpha);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__BLEND_FUNC_SEPARATE__SRC_RGB] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__blend_func_separate__src_rgb, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__blend_func_separate__src_rgb, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__blend_func_separate__src_rgb, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__blend_func_separate__src_rgb, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__blend_func_separate__src_rgb, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__blend_func_separate__src_rgb);
functionMap[HASH_ELEMENT_INVERT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__invert, &ColladaParserAutoGen15Private::_data__invert, &ColladaParserAutoGen15Private::_end__invert, &ColladaParserAutoGen15Private::_preBegin__invert, &ColladaParserAutoGen15Private::_preEnd__invert, &ColladaParserAutoGen15Private::_freeAttributes__invert);
functionMap[HASH_ELEMENT_SAMPLE_COVERAGE__VALUE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sample_coverage__value, &ColladaParserAutoGen15Private::_data__sample_coverage__value, &ColladaParserAutoGen15Private::_end__sample_coverage__value, &ColladaParserAutoGen15Private::_preBegin__sample_coverage__value, &ColladaParserAutoGen15Private::_preEnd__sample_coverage__value, &ColladaParserAutoGen15Private::_freeAttributes__sample_coverage__value);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_FUNC__FUNC] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_func__func, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_func__func, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_func__func, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_func__func, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_func__func, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_func__func);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_FUNC__MASK] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_func__mask, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_func__mask, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_func__mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_func__mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_func__mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_func__mask);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_FUNC__REF] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_func__ref, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_func__ref, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_func__ref, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_func__ref, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_func__ref, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_func__ref);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_FUNC_SEPARATE__BACK] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_func_separate__back, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_func_separate__back, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_func_separate__back, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_func_separate__back, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_func_separate__back, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_func_separate__back);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_FUNC_SEPARATE__FRONT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_func_separate__front, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_func_separate__front, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_func_separate__front, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_func_separate__front, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_func_separate__front, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_func_separate__front);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_FUNC_SEPARATE__MASK] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_func_separate__mask, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_func_separate__mask, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_func_separate__mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_func_separate__mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_func_separate__mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_func_separate__mask);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_FUNC_SEPARATE__REF] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_func_separate__ref, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_func_separate__ref, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_func_separate__ref, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_func_separate__ref, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_func_separate__ref, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_func_separate__ref);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_MASK_SEPARATE__FACE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_mask_separate__face, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_mask_separate__face, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_mask_separate__face, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_mask_separate__face, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_mask_separate__face, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_mask_separate__face);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_MASK_SEPARATE__MASK] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_mask_separate__mask, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_mask_separate__mask, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_mask_separate__mask, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_mask_separate__mask, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_mask_separate__mask, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_mask_separate__mask);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_OP__FAIL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_op__fail, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_op__fail, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_op__fail, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_op__fail, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_op__fail, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_op__fail);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_OP__ZFAIL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_op__zfail, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_op__zfail, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_op__zfail, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_op__zfail, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_op__zfail, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_op__zfail);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_OP__ZPASS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_op__zpass, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_op__zpass, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_op__zpass, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_op__zpass, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_op__zpass, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_op__zpass);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_OP_SEPARATE__FACE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_op_separate__face, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_op_separate__face, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_op_separate__face, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_op_separate__face, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_op_separate__face, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_op_separate__face);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_OP_SEPARATE__FAIL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_op_separate__fail, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_op_separate__fail, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_op_separate__fail, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_op_separate__fail, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_op_separate__fail, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_op_separate__fail);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_OP_SEPARATE__ZFAIL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_op_separate__zfail, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_op_separate__zfail, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_op_separate__zfail, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_op_separate__zfail, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_op_separate__zfail, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_op_separate__zfail);
functionMap[HASH_ELEMENT_PROFILE_GLES2__TECHNIQUE__PASS__STATES__STENCIL_OP_SEPARATE__ZPASS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLES2__technique__pass__states__stencil_op_separate__zpass, &ColladaParserAutoGen15Private::_data__profile_GLES2__technique__pass__states__stencil_op_separate__zpass, &ColladaParserAutoGen15Private::_end__profile_GLES2__technique__pass__states__stencil_op_separate__zpass, &ColladaParserAutoGen15Private::_preBegin__profile_GLES2__technique__pass__states__stencil_op_separate__zpass, &ColladaParserAutoGen15Private::_preEnd__profile_GLES2__technique__pass__states__stencil_op_separate__zpass, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLES2__technique__pass__states__stencil_op_separate__zpass);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_CODE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__code, &ColladaParserAutoGen15Private::_data__code, &ColladaParserAutoGen15Private::_end__code, &ColladaParserAutoGen15Private::_preBegin__code, &ColladaParserAutoGen15Private::_preEnd__code, &ColladaParserAutoGen15Private::_freeAttributes__code);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INCLUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__include, &ColladaParserAutoGen15Private::_data__include, &ColladaParserAutoGen15Private::_end__include, &ColladaParserAutoGen15Private::_preBegin__include, &ColladaParserAutoGen15Private::_preEnd__include, &ColladaParserAutoGen15Private::_freeAttributes__include);
functionMap[HASH_ELEMENT_NEWPARAM____GLSL_NEWPARAM_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__newparam____glsl_newparam_type, &ColladaParserAutoGen15Private::_data__newparam____glsl_newparam_type, &ColladaParserAutoGen15Private::_end__newparam____glsl_newparam_type, &ColladaParserAutoGen15Private::_preBegin__newparam____glsl_newparam_type, &ColladaParserAutoGen15Private::_preEnd__newparam____glsl_newparam_type, &ColladaParserAutoGen15Private::_freeAttributes__newparam____glsl_newparam_type);
functionMap[HASH_ELEMENT_PROFILE_GLSL_TYPE____TECHNIQUE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_glsl_type____technique, &ColladaParserAutoGen15Private::_data__profile_glsl_type____technique, &ColladaParserAutoGen15Private::_end__profile_glsl_type____technique, &ColladaParserAutoGen15Private::_preBegin__profile_glsl_type____technique, &ColladaParserAutoGen15Private::_preEnd__profile_glsl_type____technique, &ColladaParserAutoGen15Private::_freeAttributes__profile_glsl_type____technique);
functionMap[HASH_ELEMENT_ANNOTATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__annotate, &ColladaParserAutoGen15Private::_data__annotate, &ColladaParserAutoGen15Private::_end__annotate, &ColladaParserAutoGen15Private::_preBegin__annotate, &ColladaParserAutoGen15Private::_preEnd__annotate, &ColladaParserAutoGen15Private::_freeAttributes__annotate);
functionMap[HASH_ELEMENT_ARRAY____GLSL_ARRAY_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__array____glsl_array_type, &ColladaParserAutoGen15Private::_data__array____glsl_array_type, &ColladaParserAutoGen15Private::_end__array____glsl_array_type, &ColladaParserAutoGen15Private::_preBegin__array____glsl_array_type, &ColladaParserAutoGen15Private::_preEnd__array____glsl_array_type, &ColladaParserAutoGen15Private::_freeAttributes__array____glsl_array_type);
functionMap[HASH_ELEMENT_BOOL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool, &ColladaParserAutoGen15Private::_data__bool, &ColladaParserAutoGen15Private::_end__bool, &ColladaParserAutoGen15Private::_preBegin__bool, &ColladaParserAutoGen15Private::_preEnd__bool, &ColladaParserAutoGen15Private::_freeAttributes__bool);
functionMap[HASH_ELEMENT_BOOL2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool2, &ColladaParserAutoGen15Private::_data__bool2, &ColladaParserAutoGen15Private::_end__bool2, &ColladaParserAutoGen15Private::_preBegin__bool2, &ColladaParserAutoGen15Private::_preEnd__bool2, &ColladaParserAutoGen15Private::_freeAttributes__bool2);
functionMap[HASH_ELEMENT_BOOL3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool3, &ColladaParserAutoGen15Private::_data__bool3, &ColladaParserAutoGen15Private::_end__bool3, &ColladaParserAutoGen15Private::_preBegin__bool3, &ColladaParserAutoGen15Private::_preEnd__bool3, &ColladaParserAutoGen15Private::_freeAttributes__bool3);
functionMap[HASH_ELEMENT_BOOL4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool4, &ColladaParserAutoGen15Private::_data__bool4, &ColladaParserAutoGen15Private::_end__bool4, &ColladaParserAutoGen15Private::_preBegin__bool4, &ColladaParserAutoGen15Private::_preEnd__bool4, &ColladaParserAutoGen15Private::_freeAttributes__bool4);
functionMap[HASH_ELEMENT_ENUM____GL_ENUMERATION_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_data__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_end__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_preBegin__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_preEnd__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_freeAttributes__enum____gl_enumeration_type);
functionMap[HASH_ELEMENT_FLOAT____FLOAT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float____float_type, &ColladaParserAutoGen15Private::_data__float____float_type, &ColladaParserAutoGen15Private::_end__float____float_type, &ColladaParserAutoGen15Private::_preBegin__float____float_type, &ColladaParserAutoGen15Private::_preEnd__float____float_type, &ColladaParserAutoGen15Private::_freeAttributes__float____float_type);
functionMap[HASH_ELEMENT_FLOAT2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2, &ColladaParserAutoGen15Private::_data__float2, &ColladaParserAutoGen15Private::_end__float2, &ColladaParserAutoGen15Private::_preBegin__float2, &ColladaParserAutoGen15Private::_preEnd__float2, &ColladaParserAutoGen15Private::_freeAttributes__float2);
functionMap[HASH_ELEMENT_FLOAT2X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2x2, &ColladaParserAutoGen15Private::_data__float2x2, &ColladaParserAutoGen15Private::_end__float2x2, &ColladaParserAutoGen15Private::_preBegin__float2x2, &ColladaParserAutoGen15Private::_preEnd__float2x2, &ColladaParserAutoGen15Private::_freeAttributes__float2x2);
functionMap[HASH_ELEMENT_FLOAT3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3, &ColladaParserAutoGen15Private::_data__float3, &ColladaParserAutoGen15Private::_end__float3, &ColladaParserAutoGen15Private::_preBegin__float3, &ColladaParserAutoGen15Private::_preEnd__float3, &ColladaParserAutoGen15Private::_freeAttributes__float3);
functionMap[HASH_ELEMENT_FLOAT3X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3x3, &ColladaParserAutoGen15Private::_data__float3x3, &ColladaParserAutoGen15Private::_end__float3x3, &ColladaParserAutoGen15Private::_preBegin__float3x3, &ColladaParserAutoGen15Private::_preEnd__float3x3, &ColladaParserAutoGen15Private::_freeAttributes__float3x3);
functionMap[HASH_ELEMENT_FLOAT4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4, &ColladaParserAutoGen15Private::_data__float4, &ColladaParserAutoGen15Private::_end__float4, &ColladaParserAutoGen15Private::_preBegin__float4, &ColladaParserAutoGen15Private::_preEnd__float4, &ColladaParserAutoGen15Private::_freeAttributes__float4);
functionMap[HASH_ELEMENT_FLOAT4X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4x4, &ColladaParserAutoGen15Private::_data__float4x4, &ColladaParserAutoGen15Private::_end__float4x4, &ColladaParserAutoGen15Private::_preBegin__float4x4, &ColladaParserAutoGen15Private::_preEnd__float4x4, &ColladaParserAutoGen15Private::_freeAttributes__float4x4);
functionMap[HASH_ELEMENT_INT____INT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int____int_type, &ColladaParserAutoGen15Private::_data__int____int_type, &ColladaParserAutoGen15Private::_end__int____int_type, &ColladaParserAutoGen15Private::_preBegin__int____int_type, &ColladaParserAutoGen15Private::_preEnd__int____int_type, &ColladaParserAutoGen15Private::_freeAttributes__int____int_type);
functionMap[HASH_ELEMENT_INT2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int2, &ColladaParserAutoGen15Private::_data__int2, &ColladaParserAutoGen15Private::_end__int2, &ColladaParserAutoGen15Private::_preBegin__int2, &ColladaParserAutoGen15Private::_preEnd__int2, &ColladaParserAutoGen15Private::_freeAttributes__int2);
functionMap[HASH_ELEMENT_INT3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int3, &ColladaParserAutoGen15Private::_data__int3, &ColladaParserAutoGen15Private::_end__int3, &ColladaParserAutoGen15Private::_preBegin__int3, &ColladaParserAutoGen15Private::_preEnd__int3, &ColladaParserAutoGen15Private::_freeAttributes__int3);
functionMap[HASH_ELEMENT_INT4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int4, &ColladaParserAutoGen15Private::_data__int4, &ColladaParserAutoGen15Private::_end__int4, &ColladaParserAutoGen15Private::_preBegin__int4, &ColladaParserAutoGen15Private::_preEnd__int4, &ColladaParserAutoGen15Private::_freeAttributes__int4);
functionMap[HASH_ELEMENT_MODIFIER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modifier, &ColladaParserAutoGen15Private::_data__modifier, &ColladaParserAutoGen15Private::_end__modifier, &ColladaParserAutoGen15Private::_preBegin__modifier, &ColladaParserAutoGen15Private::_preEnd__modifier, &ColladaParserAutoGen15Private::_freeAttributes__modifier);
functionMap[HASH_ELEMENT_SAMPLER1D] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler1D, &ColladaParserAutoGen15Private::_data__sampler1D, &ColladaParserAutoGen15Private::_end__sampler1D, &ColladaParserAutoGen15Private::_preBegin__sampler1D, &ColladaParserAutoGen15Private::_preEnd__sampler1D, &ColladaParserAutoGen15Private::_freeAttributes__sampler1D);
functionMap[HASH_ELEMENT_SAMPLER2D____FX_SAMPLER2D_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_data__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_end__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_preBegin__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_preEnd__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_freeAttributes__sampler2D____fx_sampler2D_type);
functionMap[HASH_ELEMENT_SAMPLER3D] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler3D, &ColladaParserAutoGen15Private::_data__sampler3D, &ColladaParserAutoGen15Private::_end__sampler3D, &ColladaParserAutoGen15Private::_preBegin__sampler3D, &ColladaParserAutoGen15Private::_preEnd__sampler3D, &ColladaParserAutoGen15Private::_freeAttributes__sampler3D);
functionMap[HASH_ELEMENT_SAMPLERCUBE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__samplerCUBE, &ColladaParserAutoGen15Private::_data__samplerCUBE, &ColladaParserAutoGen15Private::_end__samplerCUBE, &ColladaParserAutoGen15Private::_preBegin__samplerCUBE, &ColladaParserAutoGen15Private::_preEnd__samplerCUBE, &ColladaParserAutoGen15Private::_freeAttributes__samplerCUBE);
functionMap[HASH_ELEMENT_SAMPLERDEPTH] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__samplerDEPTH, &ColladaParserAutoGen15Private::_data__samplerDEPTH, &ColladaParserAutoGen15Private::_end__samplerDEPTH, &ColladaParserAutoGen15Private::_preBegin__samplerDEPTH, &ColladaParserAutoGen15Private::_preEnd__samplerDEPTH, &ColladaParserAutoGen15Private::_freeAttributes__samplerDEPTH);
functionMap[HASH_ELEMENT_SAMPLERRECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__samplerRECT, &ColladaParserAutoGen15Private::_data__samplerRECT, &ColladaParserAutoGen15Private::_end__samplerRECT, &ColladaParserAutoGen15Private::_preBegin__samplerRECT, &ColladaParserAutoGen15Private::_preEnd__samplerRECT, &ColladaParserAutoGen15Private::_freeAttributes__samplerRECT);
functionMap[HASH_ELEMENT_SEMANTIC____NCNAME] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__semantic____NCName, &ColladaParserAutoGen15Private::_data__semantic____NCName, &ColladaParserAutoGen15Private::_end__semantic____NCName, &ColladaParserAutoGen15Private::_preBegin__semantic____NCName, &ColladaParserAutoGen15Private::_preEnd__semantic____NCName, &ColladaParserAutoGen15Private::_freeAttributes__semantic____NCName);
functionMap[HASH_ELEMENT_BOOL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool, &ColladaParserAutoGen15Private::_data__bool, &ColladaParserAutoGen15Private::_end__bool, &ColladaParserAutoGen15Private::_preBegin__bool, &ColladaParserAutoGen15Private::_preEnd__bool, &ColladaParserAutoGen15Private::_freeAttributes__bool);
functionMap[HASH_ELEMENT_BOOL2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool2, &ColladaParserAutoGen15Private::_data__bool2, &ColladaParserAutoGen15Private::_end__bool2, &ColladaParserAutoGen15Private::_preBegin__bool2, &ColladaParserAutoGen15Private::_preEnd__bool2, &ColladaParserAutoGen15Private::_freeAttributes__bool2);
functionMap[HASH_ELEMENT_BOOL3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool3, &ColladaParserAutoGen15Private::_data__bool3, &ColladaParserAutoGen15Private::_end__bool3, &ColladaParserAutoGen15Private::_preBegin__bool3, &ColladaParserAutoGen15Private::_preEnd__bool3, &ColladaParserAutoGen15Private::_freeAttributes__bool3);
functionMap[HASH_ELEMENT_BOOL4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool4, &ColladaParserAutoGen15Private::_data__bool4, &ColladaParserAutoGen15Private::_end__bool4, &ColladaParserAutoGen15Private::_preBegin__bool4, &ColladaParserAutoGen15Private::_preEnd__bool4, &ColladaParserAutoGen15Private::_freeAttributes__bool4);
functionMap[HASH_ELEMENT_ENUM____GL_ENUMERATION_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_data__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_end__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_preBegin__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_preEnd__enum____gl_enumeration_type, &ColladaParserAutoGen15Private::_freeAttributes__enum____gl_enumeration_type);
functionMap[HASH_ELEMENT_FLOAT____FLOAT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float____float_type, &ColladaParserAutoGen15Private::_data__float____float_type, &ColladaParserAutoGen15Private::_end__float____float_type, &ColladaParserAutoGen15Private::_preBegin__float____float_type, &ColladaParserAutoGen15Private::_preEnd__float____float_type, &ColladaParserAutoGen15Private::_freeAttributes__float____float_type);
functionMap[HASH_ELEMENT_FLOAT2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2, &ColladaParserAutoGen15Private::_data__float2, &ColladaParserAutoGen15Private::_end__float2, &ColladaParserAutoGen15Private::_preBegin__float2, &ColladaParserAutoGen15Private::_preEnd__float2, &ColladaParserAutoGen15Private::_freeAttributes__float2);
functionMap[HASH_ELEMENT_FLOAT2X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2x2, &ColladaParserAutoGen15Private::_data__float2x2, &ColladaParserAutoGen15Private::_end__float2x2, &ColladaParserAutoGen15Private::_preBegin__float2x2, &ColladaParserAutoGen15Private::_preEnd__float2x2, &ColladaParserAutoGen15Private::_freeAttributes__float2x2);
functionMap[HASH_ELEMENT_FLOAT3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3, &ColladaParserAutoGen15Private::_data__float3, &ColladaParserAutoGen15Private::_end__float3, &ColladaParserAutoGen15Private::_preBegin__float3, &ColladaParserAutoGen15Private::_preEnd__float3, &ColladaParserAutoGen15Private::_freeAttributes__float3);
functionMap[HASH_ELEMENT_FLOAT3X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3x3, &ColladaParserAutoGen15Private::_data__float3x3, &ColladaParserAutoGen15Private::_end__float3x3, &ColladaParserAutoGen15Private::_preBegin__float3x3, &ColladaParserAutoGen15Private::_preEnd__float3x3, &ColladaParserAutoGen15Private::_freeAttributes__float3x3);
functionMap[HASH_ELEMENT_FLOAT4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4, &ColladaParserAutoGen15Private::_data__float4, &ColladaParserAutoGen15Private::_end__float4, &ColladaParserAutoGen15Private::_preBegin__float4, &ColladaParserAutoGen15Private::_preEnd__float4, &ColladaParserAutoGen15Private::_freeAttributes__float4);
functionMap[HASH_ELEMENT_FLOAT4X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4x4, &ColladaParserAutoGen15Private::_data__float4x4, &ColladaParserAutoGen15Private::_end__float4x4, &ColladaParserAutoGen15Private::_preBegin__float4x4, &ColladaParserAutoGen15Private::_preEnd__float4x4, &ColladaParserAutoGen15Private::_freeAttributes__float4x4);
functionMap[HASH_ELEMENT_INT____INT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int____int_type, &ColladaParserAutoGen15Private::_data__int____int_type, &ColladaParserAutoGen15Private::_end__int____int_type, &ColladaParserAutoGen15Private::_preBegin__int____int_type, &ColladaParserAutoGen15Private::_preEnd__int____int_type, &ColladaParserAutoGen15Private::_freeAttributes__int____int_type);
functionMap[HASH_ELEMENT_INT2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int2, &ColladaParserAutoGen15Private::_data__int2, &ColladaParserAutoGen15Private::_end__int2, &ColladaParserAutoGen15Private::_preBegin__int2, &ColladaParserAutoGen15Private::_preEnd__int2, &ColladaParserAutoGen15Private::_freeAttributes__int2);
functionMap[HASH_ELEMENT_INT3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int3, &ColladaParserAutoGen15Private::_data__int3, &ColladaParserAutoGen15Private::_end__int3, &ColladaParserAutoGen15Private::_preBegin__int3, &ColladaParserAutoGen15Private::_preEnd__int3, &ColladaParserAutoGen15Private::_freeAttributes__int3);
functionMap[HASH_ELEMENT_INT4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int4, &ColladaParserAutoGen15Private::_data__int4, &ColladaParserAutoGen15Private::_end__int4, &ColladaParserAutoGen15Private::_preBegin__int4, &ColladaParserAutoGen15Private::_preEnd__int4, &ColladaParserAutoGen15Private::_freeAttributes__int4);
functionMap[HASH_ELEMENT_SAMPLER1D] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler1D, &ColladaParserAutoGen15Private::_data__sampler1D, &ColladaParserAutoGen15Private::_end__sampler1D, &ColladaParserAutoGen15Private::_preBegin__sampler1D, &ColladaParserAutoGen15Private::_preEnd__sampler1D, &ColladaParserAutoGen15Private::_freeAttributes__sampler1D);
functionMap[HASH_ELEMENT_SAMPLER2D____FX_SAMPLER2D_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_data__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_end__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_preBegin__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_preEnd__sampler2D____fx_sampler2D_type, &ColladaParserAutoGen15Private::_freeAttributes__sampler2D____fx_sampler2D_type);
functionMap[HASH_ELEMENT_SAMPLER3D] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler3D, &ColladaParserAutoGen15Private::_data__sampler3D, &ColladaParserAutoGen15Private::_end__sampler3D, &ColladaParserAutoGen15Private::_preBegin__sampler3D, &ColladaParserAutoGen15Private::_preEnd__sampler3D, &ColladaParserAutoGen15Private::_freeAttributes__sampler3D);
functionMap[HASH_ELEMENT_SAMPLERCUBE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__samplerCUBE, &ColladaParserAutoGen15Private::_data__samplerCUBE, &ColladaParserAutoGen15Private::_end__samplerCUBE, &ColladaParserAutoGen15Private::_preBegin__samplerCUBE, &ColladaParserAutoGen15Private::_preEnd__samplerCUBE, &ColladaParserAutoGen15Private::_freeAttributes__samplerCUBE);
functionMap[HASH_ELEMENT_SAMPLERDEPTH] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__samplerDEPTH, &ColladaParserAutoGen15Private::_data__samplerDEPTH, &ColladaParserAutoGen15Private::_end__samplerDEPTH, &ColladaParserAutoGen15Private::_preBegin__samplerDEPTH, &ColladaParserAutoGen15Private::_preEnd__samplerDEPTH, &ColladaParserAutoGen15Private::_freeAttributes__samplerDEPTH);
functionMap[HASH_ELEMENT_SAMPLERRECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__samplerRECT, &ColladaParserAutoGen15Private::_data__samplerRECT, &ColladaParserAutoGen15Private::_end__samplerRECT, &ColladaParserAutoGen15Private::_preBegin__samplerRECT, &ColladaParserAutoGen15Private::_preEnd__samplerRECT, &ColladaParserAutoGen15Private::_freeAttributes__samplerRECT);
functionMap[HASH_ELEMENT_ANNOTATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__annotate, &ColladaParserAutoGen15Private::_data__annotate, &ColladaParserAutoGen15Private::_end__annotate, &ColladaParserAutoGen15Private::_preBegin__annotate, &ColladaParserAutoGen15Private::_preEnd__annotate, &ColladaParserAutoGen15Private::_freeAttributes__annotate);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass);
functionMap[HASH_ELEMENT_ANNOTATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__annotate, &ColladaParserAutoGen15Private::_data__annotate, &ColladaParserAutoGen15Private::_end__annotate, &ColladaParserAutoGen15Private::_preBegin__annotate, &ColladaParserAutoGen15Private::_preEnd__annotate, &ColladaParserAutoGen15Private::_freeAttributes__annotate);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__EVALUATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__evaluate, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__evaluate, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__evaluate, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__evaluate, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__evaluate, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__evaluate);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_PROGRAM____GLSL_PROGRAM_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__program____glsl_program_type, &ColladaParserAutoGen15Private::_data__program____glsl_program_type, &ColladaParserAutoGen15Private::_end__program____glsl_program_type, &ColladaParserAutoGen15Private::_preBegin__program____glsl_program_type, &ColladaParserAutoGen15Private::_preEnd__program____glsl_program_type, &ColladaParserAutoGen15Private::_freeAttributes__program____glsl_program_type);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__STATES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__states, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__states, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__states, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__states, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__states, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__states);
functionMap[HASH_ELEMENT_COLOR_CLEAR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__color_clear, &ColladaParserAutoGen15Private::_data__color_clear, &ColladaParserAutoGen15Private::_end__color_clear, &ColladaParserAutoGen15Private::_preBegin__color_clear, &ColladaParserAutoGen15Private::_preEnd__color_clear, &ColladaParserAutoGen15Private::_freeAttributes__color_clear);
functionMap[HASH_ELEMENT_COLOR_TARGET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__color_target, &ColladaParserAutoGen15Private::_data__color_target, &ColladaParserAutoGen15Private::_end__color_target, &ColladaParserAutoGen15Private::_preBegin__color_target, &ColladaParserAutoGen15Private::_preEnd__color_target, &ColladaParserAutoGen15Private::_freeAttributes__color_target);
functionMap[HASH_ELEMENT_DEPTH_CLEAR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__depth_clear, &ColladaParserAutoGen15Private::_data__depth_clear, &ColladaParserAutoGen15Private::_end__depth_clear, &ColladaParserAutoGen15Private::_preBegin__depth_clear, &ColladaParserAutoGen15Private::_preEnd__depth_clear, &ColladaParserAutoGen15Private::_freeAttributes__depth_clear);
functionMap[HASH_ELEMENT_DEPTH_TARGET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__depth_target, &ColladaParserAutoGen15Private::_data__depth_target, &ColladaParserAutoGen15Private::_end__depth_target, &ColladaParserAutoGen15Private::_preBegin__depth_target, &ColladaParserAutoGen15Private::_preEnd__depth_target, &ColladaParserAutoGen15Private::_freeAttributes__depth_target);
functionMap[HASH_ELEMENT_DRAW] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__draw, &ColladaParserAutoGen15Private::_data__draw, &ColladaParserAutoGen15Private::_end__draw, &ColladaParserAutoGen15Private::_preBegin__draw, &ColladaParserAutoGen15Private::_preEnd__draw, &ColladaParserAutoGen15Private::_freeAttributes__draw);
functionMap[HASH_ELEMENT_STENCIL_CLEAR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__stencil_clear, &ColladaParserAutoGen15Private::_data__stencil_clear, &ColladaParserAutoGen15Private::_end__stencil_clear, &ColladaParserAutoGen15Private::_preBegin__stencil_clear, &ColladaParserAutoGen15Private::_preEnd__stencil_clear, &ColladaParserAutoGen15Private::_freeAttributes__stencil_clear);
functionMap[HASH_ELEMENT_STENCIL_TARGET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__stencil_target, &ColladaParserAutoGen15Private::_data__stencil_target, &ColladaParserAutoGen15Private::_end__stencil_target, &ColladaParserAutoGen15Private::_preBegin__stencil_target, &ColladaParserAutoGen15Private::_preEnd__stencil_target, &ColladaParserAutoGen15Private::_freeAttributes__stencil_target);
functionMap[HASH_ELEMENT_GLSL_PROGRAM_TYPE____BIND_ATTRIBUTE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__glsl_program_type____bind_attribute, &ColladaParserAutoGen15Private::_data__glsl_program_type____bind_attribute, &ColladaParserAutoGen15Private::_end__glsl_program_type____bind_attribute, &ColladaParserAutoGen15Private::_preBegin__glsl_program_type____bind_attribute, &ColladaParserAutoGen15Private::_preEnd__glsl_program_type____bind_attribute, &ColladaParserAutoGen15Private::_freeAttributes__glsl_program_type____bind_attribute);
functionMap[HASH_ELEMENT_GLSL_PROGRAM_TYPE____BIND_UNIFORM] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__glsl_program_type____bind_uniform, &ColladaParserAutoGen15Private::_data__glsl_program_type____bind_uniform, &ColladaParserAutoGen15Private::_end__glsl_program_type____bind_uniform, &ColladaParserAutoGen15Private::_preBegin__glsl_program_type____bind_uniform, &ColladaParserAutoGen15Private::_preEnd__glsl_program_type____bind_uniform, &ColladaParserAutoGen15Private::_freeAttributes__glsl_program_type____bind_uniform);
functionMap[HASH_ELEMENT_SHADER____GLSL_SHADER_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__shader____glsl_shader_type, &ColladaParserAutoGen15Private::_data__shader____glsl_shader_type, &ColladaParserAutoGen15Private::_end__shader____glsl_shader_type, &ColladaParserAutoGen15Private::_preBegin__shader____glsl_shader_type, &ColladaParserAutoGen15Private::_preEnd__shader____glsl_shader_type, &ColladaParserAutoGen15Private::_freeAttributes__shader____glsl_shader_type);
functionMap[HASH_ELEMENT_SEMANTIC____TOKEN] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__semantic____token, &ColladaParserAutoGen15Private::_data__semantic____token, &ColladaParserAutoGen15Private::_end__semantic____token, &ColladaParserAutoGen15Private::_preBegin__semantic____token, &ColladaParserAutoGen15Private::_preEnd__semantic____token, &ColladaParserAutoGen15Private::_freeAttributes__semantic____token);
functionMap[HASH_ELEMENT_PROFILE_GLSL__TECHNIQUE__PASS__PROGRAM__BIND_UNIFORM__PARAM] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__profile_GLSL__technique__pass__program__bind_uniform__param, &ColladaParserAutoGen15Private::_data__profile_GLSL__technique__pass__program__bind_uniform__param, &ColladaParserAutoGen15Private::_end__profile_GLSL__technique__pass__program__bind_uniform__param, &ColladaParserAutoGen15Private::_preBegin__profile_GLSL__technique__pass__program__bind_uniform__param, &ColladaParserAutoGen15Private::_preEnd__profile_GLSL__technique__pass__program__bind_uniform__param, &ColladaParserAutoGen15Private::_freeAttributes__profile_GLSL__technique__pass__program__bind_uniform__param);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_SOURCES____FX_SOURCES_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sources____fx_sources_type, &ColladaParserAutoGen15Private::_data__sources____fx_sources_type, &ColladaParserAutoGen15Private::_end__sources____fx_sources_type, &ColladaParserAutoGen15Private::_preBegin__sources____fx_sources_type, &ColladaParserAutoGen15Private::_preEnd__sources____fx_sources_type, &ColladaParserAutoGen15Private::_freeAttributes__sources____fx_sources_type);
mapFilled = true;
}
return functionMap;
}

//---------------------------------------------------------------------
const ColladaParserAutoGen15Private::ElementFunctionMap& FunctionMapFactory::createFunctionMap__library_force_fields__allChildren() {
static ColladaParserAutoGen15Private::ElementFunctionMap functionMap;
static bool mapFilled = false;
if ( !mapFilled )
{
functionMap[HASH_ELEMENT_COLLADA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__COLLADA, &ColladaParserAutoGen15Private::_data__COLLADA, &ColladaParserAutoGen15Private::_end__COLLADA, &ColladaParserAutoGen15Private::_preBegin__COLLADA, &ColladaParserAutoGen15Private::_preEnd__COLLADA, &ColladaParserAutoGen15Private::_freeAttributes__COLLADA);
functionMap[HASH_ELEMENT_LIBRARY_FORCE_FIELDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_force_fields, &ColladaParserAutoGen15Private::_data__library_force_fields, &ColladaParserAutoGen15Private::_end__library_force_fields, &ColladaParserAutoGen15Private::_preBegin__library_force_fields, &ColladaParserAutoGen15Private::_preEnd__library_force_fields, &ColladaParserAutoGen15Private::_freeAttributes__library_force_fields);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_FORCE_FIELD] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__force_field, &ColladaParserAutoGen15Private::_data__force_field, &ColladaParserAutoGen15Private::_end__force_field, &ColladaParserAutoGen15Private::_preBegin__force_field, &ColladaParserAutoGen15Private::_preEnd__force_field, &ColladaParserAutoGen15Private::_freeAttributes__force_field);
functionMap[HASH_ELEMENT_CONTRIBUTOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__contributor, &ColladaParserAutoGen15Private::_data__contributor, &ColladaParserAutoGen15Private::_end__contributor, &ColladaParserAutoGen15Private::_preBegin__contributor, &ColladaParserAutoGen15Private::_preEnd__contributor, &ColladaParserAutoGen15Private::_freeAttributes__contributor);
functionMap[HASH_ELEMENT_COVERAGE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__coverage, &ColladaParserAutoGen15Private::_data__coverage, &ColladaParserAutoGen15Private::_end__coverage, &ColladaParserAutoGen15Private::_preBegin__coverage, &ColladaParserAutoGen15Private::_preEnd__coverage, &ColladaParserAutoGen15Private::_freeAttributes__coverage);
functionMap[HASH_ELEMENT_CREATED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__created, &ColladaParserAutoGen15Private::_data__created, &ColladaParserAutoGen15Private::_end__created, &ColladaParserAutoGen15Private::_preBegin__created, &ColladaParserAutoGen15Private::_preEnd__created, &ColladaParserAutoGen15Private::_freeAttributes__created);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_AUTHOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author, &ColladaParserAutoGen15Private::_data__author, &ColladaParserAutoGen15Private::_end__author, &ColladaParserAutoGen15Private::_preBegin__author, &ColladaParserAutoGen15Private::_preEnd__author, &ColladaParserAutoGen15Private::_freeAttributes__author);
functionMap[HASH_ELEMENT_AUTHOR_EMAIL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_email, &ColladaParserAutoGen15Private::_data__author_email, &ColladaParserAutoGen15Private::_end__author_email, &ColladaParserAutoGen15Private::_preBegin__author_email, &ColladaParserAutoGen15Private::_preEnd__author_email, &ColladaParserAutoGen15Private::_freeAttributes__author_email);
functionMap[HASH_ELEMENT_AUTHOR_WEBSITE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_website, &ColladaParserAutoGen15Private::_data__author_website, &ColladaParserAutoGen15Private::_end__author_website, &ColladaParserAutoGen15Private::_preBegin__author_website, &ColladaParserAutoGen15Private::_preEnd__author_website, &ColladaParserAutoGen15Private::_freeAttributes__author_website);
functionMap[HASH_ELEMENT_AUTHORING_TOOL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__authoring_tool, &ColladaParserAutoGen15Private::_data__authoring_tool, &ColladaParserAutoGen15Private::_end__authoring_tool, &ColladaParserAutoGen15Private::_preBegin__authoring_tool, &ColladaParserAutoGen15Private::_preEnd__authoring_tool, &ColladaParserAutoGen15Private::_freeAttributes__authoring_tool);
functionMap[HASH_ELEMENT_COMMENTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__comments, &ColladaParserAutoGen15Private::_data__comments, &ColladaParserAutoGen15Private::_end__comments, &ColladaParserAutoGen15Private::_preBegin__comments, &ColladaParserAutoGen15Private::_preEnd__comments, &ColladaParserAutoGen15Private::_freeAttributes__comments);
functionMap[HASH_ELEMENT_COPYRIGHT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__copyright, &ColladaParserAutoGen15Private::_data__copyright, &ColladaParserAutoGen15Private::_end__copyright, &ColladaParserAutoGen15Private::_preBegin__copyright, &ColladaParserAutoGen15Private::_preEnd__copyright, &ColladaParserAutoGen15Private::_freeAttributes__copyright);
functionMap[HASH_ELEMENT_SOURCE_DATA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source_data, &ColladaParserAutoGen15Private::_data__source_data, &ColladaParserAutoGen15Private::_end__source_data, &ColladaParserAutoGen15Private::_preBegin__source_data, &ColladaParserAutoGen15Private::_preEnd__source_data, &ColladaParserAutoGen15Private::_freeAttributes__source_data);
functionMap[HASH_ELEMENT_GEOGRAPHIC_LOCATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__geographic_location, &ColladaParserAutoGen15Private::_data__geographic_location, &ColladaParserAutoGen15Private::_end__geographic_location, &ColladaParserAutoGen15Private::_preBegin__geographic_location, &ColladaParserAutoGen15Private::_preEnd__geographic_location, &ColladaParserAutoGen15Private::_freeAttributes__geographic_location);
functionMap[HASH_ELEMENT_ALTITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__altitude, &ColladaParserAutoGen15Private::_data__altitude, &ColladaParserAutoGen15Private::_end__altitude, &ColladaParserAutoGen15Private::_preBegin__altitude, &ColladaParserAutoGen15Private::_preEnd__altitude, &ColladaParserAutoGen15Private::_freeAttributes__altitude);
functionMap[HASH_ELEMENT_LATITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__latitude, &ColladaParserAutoGen15Private::_data__latitude, &ColladaParserAutoGen15Private::_end__latitude, &ColladaParserAutoGen15Private::_preBegin__latitude, &ColladaParserAutoGen15Private::_preEnd__latitude, &ColladaParserAutoGen15Private::_freeAttributes__latitude);
functionMap[HASH_ELEMENT_LONGITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__longitude, &ColladaParserAutoGen15Private::_data__longitude, &ColladaParserAutoGen15Private::_end__longitude, &ColladaParserAutoGen15Private::_preBegin__longitude, &ColladaParserAutoGen15Private::_preEnd__longitude, &ColladaParserAutoGen15Private::_freeAttributes__longitude);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
mapFilled = true;
}
return functionMap;
}

//---------------------------------------------------------------------
const ColladaParserAutoGen15Private::ElementFunctionMap& FunctionMapFactory::createFunctionMap__library_geometries__allChildren() {
static ColladaParserAutoGen15Private::ElementFunctionMap functionMap;
static bool mapFilled = false;
if ( !mapFilled )
{
functionMap[HASH_ELEMENT_COLLADA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__COLLADA, &ColladaParserAutoGen15Private::_data__COLLADA, &ColladaParserAutoGen15Private::_end__COLLADA, &ColladaParserAutoGen15Private::_preBegin__COLLADA, &ColladaParserAutoGen15Private::_preEnd__COLLADA, &ColladaParserAutoGen15Private::_freeAttributes__COLLADA);
functionMap[HASH_ELEMENT_LIBRARY_GEOMETRIES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_geometries, &ColladaParserAutoGen15Private::_data__library_geometries, &ColladaParserAutoGen15Private::_end__library_geometries, &ColladaParserAutoGen15Private::_preBegin__library_geometries, &ColladaParserAutoGen15Private::_preEnd__library_geometries, &ColladaParserAutoGen15Private::_freeAttributes__library_geometries);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_GEOMETRY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__geometry, &ColladaParserAutoGen15Private::_data__geometry, &ColladaParserAutoGen15Private::_end__geometry, &ColladaParserAutoGen15Private::_preBegin__geometry, &ColladaParserAutoGen15Private::_preEnd__geometry, &ColladaParserAutoGen15Private::_freeAttributes__geometry);
functionMap[HASH_ELEMENT_CONTRIBUTOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__contributor, &ColladaParserAutoGen15Private::_data__contributor, &ColladaParserAutoGen15Private::_end__contributor, &ColladaParserAutoGen15Private::_preBegin__contributor, &ColladaParserAutoGen15Private::_preEnd__contributor, &ColladaParserAutoGen15Private::_freeAttributes__contributor);
functionMap[HASH_ELEMENT_COVERAGE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__coverage, &ColladaParserAutoGen15Private::_data__coverage, &ColladaParserAutoGen15Private::_end__coverage, &ColladaParserAutoGen15Private::_preBegin__coverage, &ColladaParserAutoGen15Private::_preEnd__coverage, &ColladaParserAutoGen15Private::_freeAttributes__coverage);
functionMap[HASH_ELEMENT_CREATED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__created, &ColladaParserAutoGen15Private::_data__created, &ColladaParserAutoGen15Private::_end__created, &ColladaParserAutoGen15Private::_preBegin__created, &ColladaParserAutoGen15Private::_preEnd__created, &ColladaParserAutoGen15Private::_freeAttributes__created);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_AUTHOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author, &ColladaParserAutoGen15Private::_data__author, &ColladaParserAutoGen15Private::_end__author, &ColladaParserAutoGen15Private::_preBegin__author, &ColladaParserAutoGen15Private::_preEnd__author, &ColladaParserAutoGen15Private::_freeAttributes__author);
functionMap[HASH_ELEMENT_AUTHOR_EMAIL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_email, &ColladaParserAutoGen15Private::_data__author_email, &ColladaParserAutoGen15Private::_end__author_email, &ColladaParserAutoGen15Private::_preBegin__author_email, &ColladaParserAutoGen15Private::_preEnd__author_email, &ColladaParserAutoGen15Private::_freeAttributes__author_email);
functionMap[HASH_ELEMENT_AUTHOR_WEBSITE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_website, &ColladaParserAutoGen15Private::_data__author_website, &ColladaParserAutoGen15Private::_end__author_website, &ColladaParserAutoGen15Private::_preBegin__author_website, &ColladaParserAutoGen15Private::_preEnd__author_website, &ColladaParserAutoGen15Private::_freeAttributes__author_website);
functionMap[HASH_ELEMENT_AUTHORING_TOOL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__authoring_tool, &ColladaParserAutoGen15Private::_data__authoring_tool, &ColladaParserAutoGen15Private::_end__authoring_tool, &ColladaParserAutoGen15Private::_preBegin__authoring_tool, &ColladaParserAutoGen15Private::_preEnd__authoring_tool, &ColladaParserAutoGen15Private::_freeAttributes__authoring_tool);
functionMap[HASH_ELEMENT_COMMENTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__comments, &ColladaParserAutoGen15Private::_data__comments, &ColladaParserAutoGen15Private::_end__comments, &ColladaParserAutoGen15Private::_preBegin__comments, &ColladaParserAutoGen15Private::_preEnd__comments, &ColladaParserAutoGen15Private::_freeAttributes__comments);
functionMap[HASH_ELEMENT_COPYRIGHT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__copyright, &ColladaParserAutoGen15Private::_data__copyright, &ColladaParserAutoGen15Private::_end__copyright, &ColladaParserAutoGen15Private::_preBegin__copyright, &ColladaParserAutoGen15Private::_preEnd__copyright, &ColladaParserAutoGen15Private::_freeAttributes__copyright);
functionMap[HASH_ELEMENT_SOURCE_DATA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source_data, &ColladaParserAutoGen15Private::_data__source_data, &ColladaParserAutoGen15Private::_end__source_data, &ColladaParserAutoGen15Private::_preBegin__source_data, &ColladaParserAutoGen15Private::_preEnd__source_data, &ColladaParserAutoGen15Private::_freeAttributes__source_data);
functionMap[HASH_ELEMENT_GEOGRAPHIC_LOCATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__geographic_location, &ColladaParserAutoGen15Private::_data__geographic_location, &ColladaParserAutoGen15Private::_end__geographic_location, &ColladaParserAutoGen15Private::_preBegin__geographic_location, &ColladaParserAutoGen15Private::_preEnd__geographic_location, &ColladaParserAutoGen15Private::_freeAttributes__geographic_location);
functionMap[HASH_ELEMENT_ALTITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__altitude, &ColladaParserAutoGen15Private::_data__altitude, &ColladaParserAutoGen15Private::_end__altitude, &ColladaParserAutoGen15Private::_preBegin__altitude, &ColladaParserAutoGen15Private::_preEnd__altitude, &ColladaParserAutoGen15Private::_freeAttributes__altitude);
functionMap[HASH_ELEMENT_LATITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__latitude, &ColladaParserAutoGen15Private::_data__latitude, &ColladaParserAutoGen15Private::_end__latitude, &ColladaParserAutoGen15Private::_preBegin__latitude, &ColladaParserAutoGen15Private::_preEnd__latitude, &ColladaParserAutoGen15Private::_freeAttributes__latitude);
functionMap[HASH_ELEMENT_LONGITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__longitude, &ColladaParserAutoGen15Private::_data__longitude, &ColladaParserAutoGen15Private::_end__longitude, &ColladaParserAutoGen15Private::_preBegin__longitude, &ColladaParserAutoGen15Private::_preEnd__longitude, &ColladaParserAutoGen15Private::_freeAttributes__longitude);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_BREP] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__brep, &ColladaParserAutoGen15Private::_data__brep, &ColladaParserAutoGen15Private::_end__brep, &ColladaParserAutoGen15Private::_preBegin__brep, &ColladaParserAutoGen15Private::_preEnd__brep, &ColladaParserAutoGen15Private::_freeAttributes__brep);
functionMap[HASH_ELEMENT_CONVEX_MESH] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__convex_mesh, &ColladaParserAutoGen15Private::_data__convex_mesh, &ColladaParserAutoGen15Private::_end__convex_mesh, &ColladaParserAutoGen15Private::_preBegin__convex_mesh, &ColladaParserAutoGen15Private::_preEnd__convex_mesh, &ColladaParserAutoGen15Private::_freeAttributes__convex_mesh);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_MESH] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__mesh, &ColladaParserAutoGen15Private::_data__mesh, &ColladaParserAutoGen15Private::_end__mesh, &ColladaParserAutoGen15Private::_preBegin__mesh, &ColladaParserAutoGen15Private::_preEnd__mesh, &ColladaParserAutoGen15Private::_freeAttributes__mesh);
functionMap[HASH_ELEMENT_SPLINE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__spline, &ColladaParserAutoGen15Private::_data__spline, &ColladaParserAutoGen15Private::_end__spline, &ColladaParserAutoGen15Private::_preBegin__spline, &ColladaParserAutoGen15Private::_preEnd__spline, &ColladaParserAutoGen15Private::_freeAttributes__spline);
functionMap[HASH_ELEMENT_CURVES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__curves, &ColladaParserAutoGen15Private::_data__curves, &ColladaParserAutoGen15Private::_end__curves, &ColladaParserAutoGen15Private::_preBegin__curves, &ColladaParserAutoGen15Private::_preEnd__curves, &ColladaParserAutoGen15Private::_freeAttributes__curves);
functionMap[HASH_ELEMENT_EDGES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__edges, &ColladaParserAutoGen15Private::_data__edges, &ColladaParserAutoGen15Private::_end__edges, &ColladaParserAutoGen15Private::_preBegin__edges, &ColladaParserAutoGen15Private::_preEnd__edges, &ColladaParserAutoGen15Private::_freeAttributes__edges);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_FACES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__faces, &ColladaParserAutoGen15Private::_data__faces, &ColladaParserAutoGen15Private::_end__faces, &ColladaParserAutoGen15Private::_preBegin__faces, &ColladaParserAutoGen15Private::_preEnd__faces, &ColladaParserAutoGen15Private::_freeAttributes__faces);
functionMap[HASH_ELEMENT_PCURVES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__pcurves, &ColladaParserAutoGen15Private::_data__pcurves, &ColladaParserAutoGen15Private::_end__pcurves, &ColladaParserAutoGen15Private::_preBegin__pcurves, &ColladaParserAutoGen15Private::_preEnd__pcurves, &ColladaParserAutoGen15Private::_freeAttributes__pcurves);
functionMap[HASH_ELEMENT_SHELLS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__shells, &ColladaParserAutoGen15Private::_data__shells, &ColladaParserAutoGen15Private::_end__shells, &ColladaParserAutoGen15Private::_preBegin__shells, &ColladaParserAutoGen15Private::_preEnd__shells, &ColladaParserAutoGen15Private::_freeAttributes__shells);
functionMap[HASH_ELEMENT_SOLIDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__solids, &ColladaParserAutoGen15Private::_data__solids, &ColladaParserAutoGen15Private::_end__solids, &ColladaParserAutoGen15Private::_preBegin__solids, &ColladaParserAutoGen15Private::_preEnd__solids, &ColladaParserAutoGen15Private::_freeAttributes__solids);
functionMap[HASH_ELEMENT_SOURCE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source, &ColladaParserAutoGen15Private::_data__source, &ColladaParserAutoGen15Private::_end__source, &ColladaParserAutoGen15Private::_preBegin__source, &ColladaParserAutoGen15Private::_preEnd__source, &ColladaParserAutoGen15Private::_freeAttributes__source);
functionMap[HASH_ELEMENT_SURFACE_CURVES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__surface_curves, &ColladaParserAutoGen15Private::_data__surface_curves, &ColladaParserAutoGen15Private::_end__surface_curves, &ColladaParserAutoGen15Private::_preBegin__surface_curves, &ColladaParserAutoGen15Private::_preEnd__surface_curves, &ColladaParserAutoGen15Private::_freeAttributes__surface_curves);
functionMap[HASH_ELEMENT_SURFACES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__surfaces, &ColladaParserAutoGen15Private::_data__surfaces, &ColladaParserAutoGen15Private::_end__surfaces, &ColladaParserAutoGen15Private::_preBegin__surfaces, &ColladaParserAutoGen15Private::_preEnd__surfaces, &ColladaParserAutoGen15Private::_freeAttributes__surfaces);
functionMap[HASH_ELEMENT_VERTICES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__vertices, &ColladaParserAutoGen15Private::_data__vertices, &ColladaParserAutoGen15Private::_end__vertices, &ColladaParserAutoGen15Private::_preBegin__vertices, &ColladaParserAutoGen15Private::_preEnd__vertices, &ColladaParserAutoGen15Private::_freeAttributes__vertices);
functionMap[HASH_ELEMENT_WIRES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__wires, &ColladaParserAutoGen15Private::_data__wires, &ColladaParserAutoGen15Private::_end__wires, &ColladaParserAutoGen15Private::_preBegin__wires, &ColladaParserAutoGen15Private::_preEnd__wires, &ColladaParserAutoGen15Private::_freeAttributes__wires);
functionMap[HASH_ELEMENT_CURVE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__curve, &ColladaParserAutoGen15Private::_data__curve, &ColladaParserAutoGen15Private::_end__curve, &ColladaParserAutoGen15Private::_preBegin__curve, &ColladaParserAutoGen15Private::_preEnd__curve, &ColladaParserAutoGen15Private::_freeAttributes__curve);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_CIRCLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__circle, &ColladaParserAutoGen15Private::_data__circle, &ColladaParserAutoGen15Private::_end__circle, &ColladaParserAutoGen15Private::_preBegin__circle, &ColladaParserAutoGen15Private::_preEnd__circle, &ColladaParserAutoGen15Private::_freeAttributes__circle);
functionMap[HASH_ELEMENT_ELLIPSE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__ellipse, &ColladaParserAutoGen15Private::_data__ellipse, &ColladaParserAutoGen15Private::_end__ellipse, &ColladaParserAutoGen15Private::_preBegin__ellipse, &ColladaParserAutoGen15Private::_preEnd__ellipse, &ColladaParserAutoGen15Private::_freeAttributes__ellipse);
functionMap[HASH_ELEMENT_HYPERBOLA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__hyperbola, &ColladaParserAutoGen15Private::_data__hyperbola, &ColladaParserAutoGen15Private::_end__hyperbola, &ColladaParserAutoGen15Private::_preBegin__hyperbola, &ColladaParserAutoGen15Private::_preEnd__hyperbola, &ColladaParserAutoGen15Private::_freeAttributes__hyperbola);
functionMap[HASH_ELEMENT_LINE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__line, &ColladaParserAutoGen15Private::_data__line, &ColladaParserAutoGen15Private::_end__line, &ColladaParserAutoGen15Private::_preBegin__line, &ColladaParserAutoGen15Private::_preEnd__line, &ColladaParserAutoGen15Private::_freeAttributes__line);
functionMap[HASH_ELEMENT_NURBS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__nurbs, &ColladaParserAutoGen15Private::_data__nurbs, &ColladaParserAutoGen15Private::_end__nurbs, &ColladaParserAutoGen15Private::_preBegin__nurbs, &ColladaParserAutoGen15Private::_preEnd__nurbs, &ColladaParserAutoGen15Private::_freeAttributes__nurbs);
functionMap[HASH_ELEMENT_ORIENT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__orient, &ColladaParserAutoGen15Private::_data__orient, &ColladaParserAutoGen15Private::_end__orient, &ColladaParserAutoGen15Private::_preBegin__orient, &ColladaParserAutoGen15Private::_preEnd__orient, &ColladaParserAutoGen15Private::_freeAttributes__orient);
functionMap[HASH_ELEMENT_ORIGIN____ORIGIN_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__origin____origin_type, &ColladaParserAutoGen15Private::_data__origin____origin_type, &ColladaParserAutoGen15Private::_end__origin____origin_type, &ColladaParserAutoGen15Private::_preBegin__origin____origin_type, &ColladaParserAutoGen15Private::_preEnd__origin____origin_type, &ColladaParserAutoGen15Private::_freeAttributes__origin____origin_type);
functionMap[HASH_ELEMENT_PARABOLA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__parabola, &ColladaParserAutoGen15Private::_data__parabola, &ColladaParserAutoGen15Private::_end__parabola, &ColladaParserAutoGen15Private::_preBegin__parabola, &ColladaParserAutoGen15Private::_preEnd__parabola, &ColladaParserAutoGen15Private::_freeAttributes__parabola);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_RADIUS____FLOAT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__radius____float_type, &ColladaParserAutoGen15Private::_data__radius____float_type, &ColladaParserAutoGen15Private::_end__radius____float_type, &ColladaParserAutoGen15Private::_preBegin__radius____float_type, &ColladaParserAutoGen15Private::_preEnd__radius____float_type, &ColladaParserAutoGen15Private::_freeAttributes__radius____float_type);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_RADIUS____FLOAT2_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__radius____float2_type, &ColladaParserAutoGen15Private::_data__radius____float2_type, &ColladaParserAutoGen15Private::_end__radius____float2_type, &ColladaParserAutoGen15Private::_preBegin__radius____float2_type, &ColladaParserAutoGen15Private::_preEnd__radius____float2_type, &ColladaParserAutoGen15Private::_freeAttributes__radius____float2_type);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_RADIUS____FLOAT2_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__radius____float2_type, &ColladaParserAutoGen15Private::_data__radius____float2_type, &ColladaParserAutoGen15Private::_end__radius____float2_type, &ColladaParserAutoGen15Private::_preBegin__radius____float2_type, &ColladaParserAutoGen15Private::_preEnd__radius____float2_type, &ColladaParserAutoGen15Private::_freeAttributes__radius____float2_type);
functionMap[HASH_ELEMENT_DIRECTION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__direction, &ColladaParserAutoGen15Private::_data__direction, &ColladaParserAutoGen15Private::_end__direction, &ColladaParserAutoGen15Private::_preBegin__direction, &ColladaParserAutoGen15Private::_preEnd__direction, &ColladaParserAutoGen15Private::_freeAttributes__direction);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_ORIGIN____FLOAT3_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__origin____float3_type, &ColladaParserAutoGen15Private::_data__origin____float3_type, &ColladaParserAutoGen15Private::_end__origin____float3_type, &ColladaParserAutoGen15Private::_preBegin__origin____float3_type, &ColladaParserAutoGen15Private::_preEnd__origin____float3_type, &ColladaParserAutoGen15Private::_freeAttributes__origin____float3_type);
functionMap[HASH_ELEMENT_NURBS_TYPE____CONTROL_VERTICES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__nurbs_type____control_vertices, &ColladaParserAutoGen15Private::_data__nurbs_type____control_vertices, &ColladaParserAutoGen15Private::_end__nurbs_type____control_vertices, &ColladaParserAutoGen15Private::_preBegin__nurbs_type____control_vertices, &ColladaParserAutoGen15Private::_preEnd__nurbs_type____control_vertices, &ColladaParserAutoGen15Private::_freeAttributes__nurbs_type____control_vertices);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_SOURCE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source, &ColladaParserAutoGen15Private::_data__source, &ColladaParserAutoGen15Private::_end__source, &ColladaParserAutoGen15Private::_preBegin__source, &ColladaParserAutoGen15Private::_preEnd__source, &ColladaParserAutoGen15Private::_freeAttributes__source);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INPUT____INPUT_LOCAL_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__input____input_local_type, &ColladaParserAutoGen15Private::_data__input____input_local_type, &ColladaParserAutoGen15Private::_end__input____input_local_type, &ColladaParserAutoGen15Private::_preBegin__input____input_local_type, &ColladaParserAutoGen15Private::_preEnd__input____input_local_type, &ColladaParserAutoGen15Private::_freeAttributes__input____input_local_type);
functionMap[HASH_ELEMENT_IDREF_ARRAY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__IDREF_array, &ColladaParserAutoGen15Private::_data__IDREF_array, &ColladaParserAutoGen15Private::_end__IDREF_array, &ColladaParserAutoGen15Private::_preBegin__IDREF_array, &ColladaParserAutoGen15Private::_preEnd__IDREF_array, &ColladaParserAutoGen15Private::_freeAttributes__IDREF_array);
functionMap[HASH_ELEMENT_NAME_ARRAY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__Name_array, &ColladaParserAutoGen15Private::_data__Name_array, &ColladaParserAutoGen15Private::_end__Name_array, &ColladaParserAutoGen15Private::_preBegin__Name_array, &ColladaParserAutoGen15Private::_preEnd__Name_array, &ColladaParserAutoGen15Private::_freeAttributes__Name_array);
functionMap[HASH_ELEMENT_SIDREF_ARRAY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__SIDREF_array, &ColladaParserAutoGen15Private::_data__SIDREF_array, &ColladaParserAutoGen15Private::_end__SIDREF_array, &ColladaParserAutoGen15Private::_preBegin__SIDREF_array, &ColladaParserAutoGen15Private::_preEnd__SIDREF_array, &ColladaParserAutoGen15Private::_freeAttributes__SIDREF_array);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_BOOL_ARRAY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool_array, &ColladaParserAutoGen15Private::_data__bool_array, &ColladaParserAutoGen15Private::_end__bool_array, &ColladaParserAutoGen15Private::_preBegin__bool_array, &ColladaParserAutoGen15Private::_preEnd__bool_array, &ColladaParserAutoGen15Private::_freeAttributes__bool_array);
functionMap[HASH_ELEMENT_FLOAT_ARRAY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float_array, &ColladaParserAutoGen15Private::_data__float_array, &ColladaParserAutoGen15Private::_end__float_array, &ColladaParserAutoGen15Private::_preBegin__float_array, &ColladaParserAutoGen15Private::_preEnd__float_array, &ColladaParserAutoGen15Private::_freeAttributes__float_array);
functionMap[HASH_ELEMENT_INT_ARRAY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int_array, &ColladaParserAutoGen15Private::_data__int_array, &ColladaParserAutoGen15Private::_end__int_array, &ColladaParserAutoGen15Private::_preBegin__int_array, &ColladaParserAutoGen15Private::_preEnd__int_array, &ColladaParserAutoGen15Private::_freeAttributes__int_array);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_SOURCE_TYPE____TECHNIQUE_COMMON] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source_type____technique_common, &ColladaParserAutoGen15Private::_data__source_type____technique_common, &ColladaParserAutoGen15Private::_end__source_type____technique_common, &ColladaParserAutoGen15Private::_preBegin__source_type____technique_common, &ColladaParserAutoGen15Private::_preEnd__source_type____technique_common, &ColladaParserAutoGen15Private::_freeAttributes__source_type____technique_common);
functionMap[HASH_ELEMENT_TOKEN_ARRAY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__token_array, &ColladaParserAutoGen15Private::_data__token_array, &ColladaParserAutoGen15Private::_end__token_array, &ColladaParserAutoGen15Private::_preBegin__token_array, &ColladaParserAutoGen15Private::_preEnd__token_array, &ColladaParserAutoGen15Private::_freeAttributes__token_array);
functionMap[HASH_ELEMENT_ACCESSOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__accessor, &ColladaParserAutoGen15Private::_data__accessor, &ColladaParserAutoGen15Private::_end__accessor, &ColladaParserAutoGen15Private::_preBegin__accessor, &ColladaParserAutoGen15Private::_preEnd__accessor, &ColladaParserAutoGen15Private::_freeAttributes__accessor);
functionMap[HASH_ELEMENT_PARAM____PARAM_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__param____param_type, &ColladaParserAutoGen15Private::_data__param____param_type, &ColladaParserAutoGen15Private::_end__param____param_type, &ColladaParserAutoGen15Private::_preBegin__param____param_type, &ColladaParserAutoGen15Private::_preEnd__param____param_type, &ColladaParserAutoGen15Private::_freeAttributes__param____param_type);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_FOCAL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__focal, &ColladaParserAutoGen15Private::_data__focal, &ColladaParserAutoGen15Private::_end__focal, &ColladaParserAutoGen15Private::_preBegin__focal, &ColladaParserAutoGen15Private::_preEnd__focal, &ColladaParserAutoGen15Private::_freeAttributes__focal);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INPUT____INPUT_LOCAL_OFFSET_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__input____input_local_offset_type, &ColladaParserAutoGen15Private::_data__input____input_local_offset_type, &ColladaParserAutoGen15Private::_end__input____input_local_offset_type, &ColladaParserAutoGen15Private::_preBegin__input____input_local_offset_type, &ColladaParserAutoGen15Private::_preEnd__input____input_local_offset_type, &ColladaParserAutoGen15Private::_freeAttributes__input____input_local_offset_type);
functionMap[HASH_ELEMENT_P] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__p, &ColladaParserAutoGen15Private::_data__p, &ColladaParserAutoGen15Private::_end__p, &ColladaParserAutoGen15Private::_preBegin__p, &ColladaParserAutoGen15Private::_preEnd__p, &ColladaParserAutoGen15Private::_freeAttributes__p);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INPUT____INPUT_LOCAL_OFFSET_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__input____input_local_offset_type, &ColladaParserAutoGen15Private::_data__input____input_local_offset_type, &ColladaParserAutoGen15Private::_end__input____input_local_offset_type, &ColladaParserAutoGen15Private::_preBegin__input____input_local_offset_type, &ColladaParserAutoGen15Private::_preEnd__input____input_local_offset_type, &ColladaParserAutoGen15Private::_freeAttributes__input____input_local_offset_type);
functionMap[HASH_ELEMENT_P] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__p, &ColladaParserAutoGen15Private::_data__p, &ColladaParserAutoGen15Private::_end__p, &ColladaParserAutoGen15Private::_preBegin__p, &ColladaParserAutoGen15Private::_preEnd__p, &ColladaParserAutoGen15Private::_freeAttributes__p);
functionMap[HASH_ELEMENT_VCOUNT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__vcount, &ColladaParserAutoGen15Private::_data__vcount, &ColladaParserAutoGen15Private::_end__vcount, &ColladaParserAutoGen15Private::_preBegin__vcount, &ColladaParserAutoGen15Private::_preEnd__vcount, &ColladaParserAutoGen15Private::_freeAttributes__vcount);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INPUT____INPUT_LOCAL_OFFSET_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__input____input_local_offset_type, &ColladaParserAutoGen15Private::_data__input____input_local_offset_type, &ColladaParserAutoGen15Private::_end__input____input_local_offset_type, &ColladaParserAutoGen15Private::_preBegin__input____input_local_offset_type, &ColladaParserAutoGen15Private::_preEnd__input____input_local_offset_type, &ColladaParserAutoGen15Private::_freeAttributes__input____input_local_offset_type);
functionMap[HASH_ELEMENT_P] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__p, &ColladaParserAutoGen15Private::_data__p, &ColladaParserAutoGen15Private::_end__p, &ColladaParserAutoGen15Private::_preBegin__p, &ColladaParserAutoGen15Private::_preEnd__p, &ColladaParserAutoGen15Private::_freeAttributes__p);
functionMap[HASH_ELEMENT_VCOUNT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__vcount, &ColladaParserAutoGen15Private::_data__vcount, &ColladaParserAutoGen15Private::_end__vcount, &ColladaParserAutoGen15Private::_preBegin__vcount, &ColladaParserAutoGen15Private::_preEnd__vcount, &ColladaParserAutoGen15Private::_freeAttributes__vcount);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INPUT____INPUT_LOCAL_OFFSET_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__input____input_local_offset_type, &ColladaParserAutoGen15Private::_data__input____input_local_offset_type, &ColladaParserAutoGen15Private::_end__input____input_local_offset_type, &ColladaParserAutoGen15Private::_preBegin__input____input_local_offset_type, &ColladaParserAutoGen15Private::_preEnd__input____input_local_offset_type, &ColladaParserAutoGen15Private::_freeAttributes__input____input_local_offset_type);
functionMap[HASH_ELEMENT_P] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__p, &ColladaParserAutoGen15Private::_data__p, &ColladaParserAutoGen15Private::_end__p, &ColladaParserAutoGen15Private::_preBegin__p, &ColladaParserAutoGen15Private::_preEnd__p, &ColladaParserAutoGen15Private::_freeAttributes__p);
functionMap[HASH_ELEMENT_VCOUNT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__vcount, &ColladaParserAutoGen15Private::_data__vcount, &ColladaParserAutoGen15Private::_end__vcount, &ColladaParserAutoGen15Private::_preBegin__vcount, &ColladaParserAutoGen15Private::_preEnd__vcount, &ColladaParserAutoGen15Private::_freeAttributes__vcount);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INPUT____INPUT_LOCAL_OFFSET_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__input____input_local_offset_type, &ColladaParserAutoGen15Private::_data__input____input_local_offset_type, &ColladaParserAutoGen15Private::_end__input____input_local_offset_type, &ColladaParserAutoGen15Private::_preBegin__input____input_local_offset_type, &ColladaParserAutoGen15Private::_preEnd__input____input_local_offset_type, &ColladaParserAutoGen15Private::_freeAttributes__input____input_local_offset_type);
functionMap[HASH_ELEMENT_P] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__p, &ColladaParserAutoGen15Private::_data__p, &ColladaParserAutoGen15Private::_end__p, &ColladaParserAutoGen15Private::_preBegin__p, &ColladaParserAutoGen15Private::_preEnd__p, &ColladaParserAutoGen15Private::_freeAttributes__p);
functionMap[HASH_ELEMENT_VCOUNT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__vcount, &ColladaParserAutoGen15Private::_data__vcount, &ColladaParserAutoGen15Private::_end__vcount, &ColladaParserAutoGen15Private::_preBegin__vcount, &ColladaParserAutoGen15Private::_preEnd__vcount, &ColladaParserAutoGen15Private::_freeAttributes__vcount);
functionMap[HASH_ELEMENT_CURVE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__curve, &ColladaParserAutoGen15Private::_data__curve, &ColladaParserAutoGen15Private::_end__curve, &ColladaParserAutoGen15Private::_preBegin__curve, &ColladaParserAutoGen15Private::_preEnd__curve, &ColladaParserAutoGen15Private::_freeAttributes__curve);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_SURFACE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__surface, &ColladaParserAutoGen15Private::_data__surface, &ColladaParserAutoGen15Private::_end__surface, &ColladaParserAutoGen15Private::_preBegin__surface, &ColladaParserAutoGen15Private::_preEnd__surface, &ColladaParserAutoGen15Private::_freeAttributes__surface);
functionMap[HASH_ELEMENT_CONE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__cone, &ColladaParserAutoGen15Private::_data__cone, &ColladaParserAutoGen15Private::_end__cone, &ColladaParserAutoGen15Private::_preBegin__cone, &ColladaParserAutoGen15Private::_preEnd__cone, &ColladaParserAutoGen15Private::_freeAttributes__cone);
functionMap[HASH_ELEMENT_SURFACE_TYPE____CYLINDER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__surface_type____cylinder, &ColladaParserAutoGen15Private::_data__surface_type____cylinder, &ColladaParserAutoGen15Private::_end__surface_type____cylinder, &ColladaParserAutoGen15Private::_preBegin__surface_type____cylinder, &ColladaParserAutoGen15Private::_preEnd__surface_type____cylinder, &ColladaParserAutoGen15Private::_freeAttributes__surface_type____cylinder);
functionMap[HASH_ELEMENT_NURBS_SURFACE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__nurbs_surface, &ColladaParserAutoGen15Private::_data__nurbs_surface, &ColladaParserAutoGen15Private::_end__nurbs_surface, &ColladaParserAutoGen15Private::_preBegin__nurbs_surface, &ColladaParserAutoGen15Private::_preEnd__nurbs_surface, &ColladaParserAutoGen15Private::_freeAttributes__nurbs_surface);
functionMap[HASH_ELEMENT_ORIENT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__orient, &ColladaParserAutoGen15Private::_data__orient, &ColladaParserAutoGen15Private::_end__orient, &ColladaParserAutoGen15Private::_preBegin__orient, &ColladaParserAutoGen15Private::_preEnd__orient, &ColladaParserAutoGen15Private::_freeAttributes__orient);
functionMap[HASH_ELEMENT_ORIGIN____ORIGIN_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__origin____origin_type, &ColladaParserAutoGen15Private::_data__origin____origin_type, &ColladaParserAutoGen15Private::_end__origin____origin_type, &ColladaParserAutoGen15Private::_preBegin__origin____origin_type, &ColladaParserAutoGen15Private::_preEnd__origin____origin_type, &ColladaParserAutoGen15Private::_freeAttributes__origin____origin_type);
functionMap[HASH_ELEMENT_PLANE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__plane, &ColladaParserAutoGen15Private::_data__plane, &ColladaParserAutoGen15Private::_end__plane, &ColladaParserAutoGen15Private::_preBegin__plane, &ColladaParserAutoGen15Private::_preEnd__plane, &ColladaParserAutoGen15Private::_freeAttributes__plane);
functionMap[HASH_ELEMENT_SPHERE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sphere, &ColladaParserAutoGen15Private::_data__sphere, &ColladaParserAutoGen15Private::_end__sphere, &ColladaParserAutoGen15Private::_preBegin__sphere, &ColladaParserAutoGen15Private::_preEnd__sphere, &ColladaParserAutoGen15Private::_freeAttributes__sphere);
functionMap[HASH_ELEMENT_SWEPT_SURFACE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__swept_surface, &ColladaParserAutoGen15Private::_data__swept_surface, &ColladaParserAutoGen15Private::_end__swept_surface, &ColladaParserAutoGen15Private::_preBegin__swept_surface, &ColladaParserAutoGen15Private::_preEnd__swept_surface, &ColladaParserAutoGen15Private::_freeAttributes__swept_surface);
functionMap[HASH_ELEMENT_TORUS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__torus, &ColladaParserAutoGen15Private::_data__torus, &ColladaParserAutoGen15Private::_end__torus, &ColladaParserAutoGen15Private::_preBegin__torus, &ColladaParserAutoGen15Private::_preEnd__torus, &ColladaParserAutoGen15Private::_freeAttributes__torus);
functionMap[HASH_ELEMENT_ANGLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__angle, &ColladaParserAutoGen15Private::_data__angle, &ColladaParserAutoGen15Private::_end__angle, &ColladaParserAutoGen15Private::_preBegin__angle, &ColladaParserAutoGen15Private::_preEnd__angle, &ColladaParserAutoGen15Private::_freeAttributes__angle);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_RADIUS____FLOAT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__radius____float_type, &ColladaParserAutoGen15Private::_data__radius____float_type, &ColladaParserAutoGen15Private::_end__radius____float_type, &ColladaParserAutoGen15Private::_preBegin__radius____float_type, &ColladaParserAutoGen15Private::_preEnd__radius____float_type, &ColladaParserAutoGen15Private::_freeAttributes__radius____float_type);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_RADIUS____FLOAT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__radius____float_type, &ColladaParserAutoGen15Private::_data__radius____float_type, &ColladaParserAutoGen15Private::_end__radius____float_type, &ColladaParserAutoGen15Private::_preBegin__radius____float_type, &ColladaParserAutoGen15Private::_preEnd__radius____float_type, &ColladaParserAutoGen15Private::_freeAttributes__radius____float_type);
functionMap[HASH_ELEMENT_NURBS_SURFACE_TYPE____CONTROL_VERTICES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__nurbs_surface_type____control_vertices, &ColladaParserAutoGen15Private::_data__nurbs_surface_type____control_vertices, &ColladaParserAutoGen15Private::_end__nurbs_surface_type____control_vertices, &ColladaParserAutoGen15Private::_preBegin__nurbs_surface_type____control_vertices, &ColladaParserAutoGen15Private::_preEnd__nurbs_surface_type____control_vertices, &ColladaParserAutoGen15Private::_freeAttributes__nurbs_surface_type____control_vertices);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_SOURCE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source, &ColladaParserAutoGen15Private::_data__source, &ColladaParserAutoGen15Private::_end__source, &ColladaParserAutoGen15Private::_preBegin__source, &ColladaParserAutoGen15Private::_preEnd__source, &ColladaParserAutoGen15Private::_freeAttributes__source);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INPUT____INPUT_LOCAL_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__input____input_local_type, &ColladaParserAutoGen15Private::_data__input____input_local_type, &ColladaParserAutoGen15Private::_end__input____input_local_type, &ColladaParserAutoGen15Private::_preBegin__input____input_local_type, &ColladaParserAutoGen15Private::_preEnd__input____input_local_type, &ColladaParserAutoGen15Private::_freeAttributes__input____input_local_type);
functionMap[HASH_ELEMENT_EQUATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__equation, &ColladaParserAutoGen15Private::_data__equation, &ColladaParserAutoGen15Private::_end__equation, &ColladaParserAutoGen15Private::_preBegin__equation, &ColladaParserAutoGen15Private::_preEnd__equation, &ColladaParserAutoGen15Private::_freeAttributes__equation);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_RADIUS____FLOAT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__radius____float_type, &ColladaParserAutoGen15Private::_data__radius____float_type, &ColladaParserAutoGen15Private::_end__radius____float_type, &ColladaParserAutoGen15Private::_preBegin__radius____float_type, &ColladaParserAutoGen15Private::_preEnd__radius____float_type, &ColladaParserAutoGen15Private::_freeAttributes__radius____float_type);
functionMap[HASH_ELEMENT_AXIS____FLOAT3_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__axis____float3_type, &ColladaParserAutoGen15Private::_data__axis____float3_type, &ColladaParserAutoGen15Private::_end__axis____float3_type, &ColladaParserAutoGen15Private::_preBegin__axis____float3_type, &ColladaParserAutoGen15Private::_preEnd__axis____float3_type, &ColladaParserAutoGen15Private::_freeAttributes__axis____float3_type);
functionMap[HASH_ELEMENT_CURVE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__curve, &ColladaParserAutoGen15Private::_data__curve, &ColladaParserAutoGen15Private::_end__curve, &ColladaParserAutoGen15Private::_preBegin__curve, &ColladaParserAutoGen15Private::_preEnd__curve, &ColladaParserAutoGen15Private::_freeAttributes__curve);
functionMap[HASH_ELEMENT_DIRECTION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__direction, &ColladaParserAutoGen15Private::_data__direction, &ColladaParserAutoGen15Private::_end__direction, &ColladaParserAutoGen15Private::_preBegin__direction, &ColladaParserAutoGen15Private::_preEnd__direction, &ColladaParserAutoGen15Private::_freeAttributes__direction);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_ORIGIN____FLOAT3_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__origin____float3_type, &ColladaParserAutoGen15Private::_data__origin____float3_type, &ColladaParserAutoGen15Private::_end__origin____float3_type, &ColladaParserAutoGen15Private::_preBegin__origin____float3_type, &ColladaParserAutoGen15Private::_preEnd__origin____float3_type, &ColladaParserAutoGen15Private::_freeAttributes__origin____float3_type);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_RADIUS____FLOAT2_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__radius____float2_type, &ColladaParserAutoGen15Private::_data__radius____float2_type, &ColladaParserAutoGen15Private::_end__radius____float2_type, &ColladaParserAutoGen15Private::_preBegin__radius____float2_type, &ColladaParserAutoGen15Private::_preEnd__radius____float2_type, &ColladaParserAutoGen15Private::_freeAttributes__radius____float2_type);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INPUT____INPUT_LOCAL_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__input____input_local_type, &ColladaParserAutoGen15Private::_data__input____input_local_type, &ColladaParserAutoGen15Private::_end__input____input_local_type, &ColladaParserAutoGen15Private::_preBegin__input____input_local_type, &ColladaParserAutoGen15Private::_preEnd__input____input_local_type, &ColladaParserAutoGen15Private::_freeAttributes__input____input_local_type);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INPUT____INPUT_LOCAL_OFFSET_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__input____input_local_offset_type, &ColladaParserAutoGen15Private::_data__input____input_local_offset_type, &ColladaParserAutoGen15Private::_end__input____input_local_offset_type, &ColladaParserAutoGen15Private::_preBegin__input____input_local_offset_type, &ColladaParserAutoGen15Private::_preEnd__input____input_local_offset_type, &ColladaParserAutoGen15Private::_freeAttributes__input____input_local_offset_type);
functionMap[HASH_ELEMENT_P] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__p, &ColladaParserAutoGen15Private::_data__p, &ColladaParserAutoGen15Private::_end__p, &ColladaParserAutoGen15Private::_preBegin__p, &ColladaParserAutoGen15Private::_preEnd__p, &ColladaParserAutoGen15Private::_freeAttributes__p);
functionMap[HASH_ELEMENT_VCOUNT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__vcount, &ColladaParserAutoGen15Private::_data__vcount, &ColladaParserAutoGen15Private::_end__vcount, &ColladaParserAutoGen15Private::_preBegin__vcount, &ColladaParserAutoGen15Private::_preEnd__vcount, &ColladaParserAutoGen15Private::_freeAttributes__vcount);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_LINES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__lines, &ColladaParserAutoGen15Private::_data__lines, &ColladaParserAutoGen15Private::_end__lines, &ColladaParserAutoGen15Private::_preBegin__lines, &ColladaParserAutoGen15Private::_preEnd__lines, &ColladaParserAutoGen15Private::_freeAttributes__lines);
functionMap[HASH_ELEMENT_LINESTRIPS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__linestrips, &ColladaParserAutoGen15Private::_data__linestrips, &ColladaParserAutoGen15Private::_end__linestrips, &ColladaParserAutoGen15Private::_preBegin__linestrips, &ColladaParserAutoGen15Private::_preEnd__linestrips, &ColladaParserAutoGen15Private::_freeAttributes__linestrips);
functionMap[HASH_ELEMENT_POLYGONS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__polygons, &ColladaParserAutoGen15Private::_data__polygons, &ColladaParserAutoGen15Private::_end__polygons, &ColladaParserAutoGen15Private::_preBegin__polygons, &ColladaParserAutoGen15Private::_preEnd__polygons, &ColladaParserAutoGen15Private::_freeAttributes__polygons);
functionMap[HASH_ELEMENT_POLYLIST] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__polylist, &ColladaParserAutoGen15Private::_data__polylist, &ColladaParserAutoGen15Private::_end__polylist, &ColladaParserAutoGen15Private::_preBegin__polylist, &ColladaParserAutoGen15Private::_preEnd__polylist, &ColladaParserAutoGen15Private::_freeAttributes__polylist);
functionMap[HASH_ELEMENT_SOURCE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source, &ColladaParserAutoGen15Private::_data__source, &ColladaParserAutoGen15Private::_end__source, &ColladaParserAutoGen15Private::_preBegin__source, &ColladaParserAutoGen15Private::_preEnd__source, &ColladaParserAutoGen15Private::_freeAttributes__source);
functionMap[HASH_ELEMENT_TRIANGLES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__triangles, &ColladaParserAutoGen15Private::_data__triangles, &ColladaParserAutoGen15Private::_end__triangles, &ColladaParserAutoGen15Private::_preBegin__triangles, &ColladaParserAutoGen15Private::_preEnd__triangles, &ColladaParserAutoGen15Private::_freeAttributes__triangles);
functionMap[HASH_ELEMENT_TRIFANS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__trifans, &ColladaParserAutoGen15Private::_data__trifans, &ColladaParserAutoGen15Private::_end__trifans, &ColladaParserAutoGen15Private::_preBegin__trifans, &ColladaParserAutoGen15Private::_preEnd__trifans, &ColladaParserAutoGen15Private::_freeAttributes__trifans);
functionMap[HASH_ELEMENT_TRISTRIPS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__tristrips, &ColladaParserAutoGen15Private::_data__tristrips, &ColladaParserAutoGen15Private::_end__tristrips, &ColladaParserAutoGen15Private::_preBegin__tristrips, &ColladaParserAutoGen15Private::_preEnd__tristrips, &ColladaParserAutoGen15Private::_freeAttributes__tristrips);
functionMap[HASH_ELEMENT_VERTICES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__vertices, &ColladaParserAutoGen15Private::_data__vertices, &ColladaParserAutoGen15Private::_end__vertices, &ColladaParserAutoGen15Private::_preBegin__vertices, &ColladaParserAutoGen15Private::_preEnd__vertices, &ColladaParserAutoGen15Private::_freeAttributes__vertices);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INPUT____INPUT_LOCAL_OFFSET_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__input____input_local_offset_type, &ColladaParserAutoGen15Private::_data__input____input_local_offset_type, &ColladaParserAutoGen15Private::_end__input____input_local_offset_type, &ColladaParserAutoGen15Private::_preBegin__input____input_local_offset_type, &ColladaParserAutoGen15Private::_preEnd__input____input_local_offset_type, &ColladaParserAutoGen15Private::_freeAttributes__input____input_local_offset_type);
functionMap[HASH_ELEMENT_P] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__p, &ColladaParserAutoGen15Private::_data__p, &ColladaParserAutoGen15Private::_end__p, &ColladaParserAutoGen15Private::_preBegin__p, &ColladaParserAutoGen15Private::_preEnd__p, &ColladaParserAutoGen15Private::_freeAttributes__p);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INPUT____INPUT_LOCAL_OFFSET_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__input____input_local_offset_type, &ColladaParserAutoGen15Private::_data__input____input_local_offset_type, &ColladaParserAutoGen15Private::_end__input____input_local_offset_type, &ColladaParserAutoGen15Private::_preBegin__input____input_local_offset_type, &ColladaParserAutoGen15Private::_preEnd__input____input_local_offset_type, &ColladaParserAutoGen15Private::_freeAttributes__input____input_local_offset_type);
functionMap[HASH_ELEMENT_P] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__p, &ColladaParserAutoGen15Private::_data__p, &ColladaParserAutoGen15Private::_end__p, &ColladaParserAutoGen15Private::_preBegin__p, &ColladaParserAutoGen15Private::_preEnd__p, &ColladaParserAutoGen15Private::_freeAttributes__p);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INPUT____INPUT_LOCAL_OFFSET_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__input____input_local_offset_type, &ColladaParserAutoGen15Private::_data__input____input_local_offset_type, &ColladaParserAutoGen15Private::_end__input____input_local_offset_type, &ColladaParserAutoGen15Private::_preBegin__input____input_local_offset_type, &ColladaParserAutoGen15Private::_preEnd__input____input_local_offset_type, &ColladaParserAutoGen15Private::_freeAttributes__input____input_local_offset_type);
functionMap[HASH_ELEMENT_P] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__p, &ColladaParserAutoGen15Private::_data__p, &ColladaParserAutoGen15Private::_end__p, &ColladaParserAutoGen15Private::_preBegin__p, &ColladaParserAutoGen15Private::_preEnd__p, &ColladaParserAutoGen15Private::_freeAttributes__p);
functionMap[HASH_ELEMENT_PH] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__ph, &ColladaParserAutoGen15Private::_data__ph, &ColladaParserAutoGen15Private::_end__ph, &ColladaParserAutoGen15Private::_preBegin__ph, &ColladaParserAutoGen15Private::_preEnd__ph, &ColladaParserAutoGen15Private::_freeAttributes__ph);
functionMap[HASH_ELEMENT_H] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__h, &ColladaParserAutoGen15Private::_data__h, &ColladaParserAutoGen15Private::_end__h, &ColladaParserAutoGen15Private::_preBegin__h, &ColladaParserAutoGen15Private::_preEnd__h, &ColladaParserAutoGen15Private::_freeAttributes__h);
functionMap[HASH_ELEMENT_P] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__p, &ColladaParserAutoGen15Private::_data__p, &ColladaParserAutoGen15Private::_end__p, &ColladaParserAutoGen15Private::_preBegin__p, &ColladaParserAutoGen15Private::_preEnd__p, &ColladaParserAutoGen15Private::_freeAttributes__p);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INPUT____INPUT_LOCAL_OFFSET_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__input____input_local_offset_type, &ColladaParserAutoGen15Private::_data__input____input_local_offset_type, &ColladaParserAutoGen15Private::_end__input____input_local_offset_type, &ColladaParserAutoGen15Private::_preBegin__input____input_local_offset_type, &ColladaParserAutoGen15Private::_preEnd__input____input_local_offset_type, &ColladaParserAutoGen15Private::_freeAttributes__input____input_local_offset_type);
functionMap[HASH_ELEMENT_P] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__p, &ColladaParserAutoGen15Private::_data__p, &ColladaParserAutoGen15Private::_end__p, &ColladaParserAutoGen15Private::_preBegin__p, &ColladaParserAutoGen15Private::_preEnd__p, &ColladaParserAutoGen15Private::_freeAttributes__p);
functionMap[HASH_ELEMENT_VCOUNT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__vcount, &ColladaParserAutoGen15Private::_data__vcount, &ColladaParserAutoGen15Private::_end__vcount, &ColladaParserAutoGen15Private::_preBegin__vcount, &ColladaParserAutoGen15Private::_preEnd__vcount, &ColladaParserAutoGen15Private::_freeAttributes__vcount);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INPUT____INPUT_LOCAL_OFFSET_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__input____input_local_offset_type, &ColladaParserAutoGen15Private::_data__input____input_local_offset_type, &ColladaParserAutoGen15Private::_end__input____input_local_offset_type, &ColladaParserAutoGen15Private::_preBegin__input____input_local_offset_type, &ColladaParserAutoGen15Private::_preEnd__input____input_local_offset_type, &ColladaParserAutoGen15Private::_freeAttributes__input____input_local_offset_type);
functionMap[HASH_ELEMENT_P] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__p, &ColladaParserAutoGen15Private::_data__p, &ColladaParserAutoGen15Private::_end__p, &ColladaParserAutoGen15Private::_preBegin__p, &ColladaParserAutoGen15Private::_preEnd__p, &ColladaParserAutoGen15Private::_freeAttributes__p);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INPUT____INPUT_LOCAL_OFFSET_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__input____input_local_offset_type, &ColladaParserAutoGen15Private::_data__input____input_local_offset_type, &ColladaParserAutoGen15Private::_end__input____input_local_offset_type, &ColladaParserAutoGen15Private::_preBegin__input____input_local_offset_type, &ColladaParserAutoGen15Private::_preEnd__input____input_local_offset_type, &ColladaParserAutoGen15Private::_freeAttributes__input____input_local_offset_type);
functionMap[HASH_ELEMENT_P] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__p, &ColladaParserAutoGen15Private::_data__p, &ColladaParserAutoGen15Private::_end__p, &ColladaParserAutoGen15Private::_preBegin__p, &ColladaParserAutoGen15Private::_preEnd__p, &ColladaParserAutoGen15Private::_freeAttributes__p);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INPUT____INPUT_LOCAL_OFFSET_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__input____input_local_offset_type, &ColladaParserAutoGen15Private::_data__input____input_local_offset_type, &ColladaParserAutoGen15Private::_end__input____input_local_offset_type, &ColladaParserAutoGen15Private::_preBegin__input____input_local_offset_type, &ColladaParserAutoGen15Private::_preEnd__input____input_local_offset_type, &ColladaParserAutoGen15Private::_freeAttributes__input____input_local_offset_type);
functionMap[HASH_ELEMENT_P] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__p, &ColladaParserAutoGen15Private::_data__p, &ColladaParserAutoGen15Private::_end__p, &ColladaParserAutoGen15Private::_preBegin__p, &ColladaParserAutoGen15Private::_preEnd__p, &ColladaParserAutoGen15Private::_freeAttributes__p);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_LINES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__lines, &ColladaParserAutoGen15Private::_data__lines, &ColladaParserAutoGen15Private::_end__lines, &ColladaParserAutoGen15Private::_preBegin__lines, &ColladaParserAutoGen15Private::_preEnd__lines, &ColladaParserAutoGen15Private::_freeAttributes__lines);
functionMap[HASH_ELEMENT_LINESTRIPS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__linestrips, &ColladaParserAutoGen15Private::_data__linestrips, &ColladaParserAutoGen15Private::_end__linestrips, &ColladaParserAutoGen15Private::_preBegin__linestrips, &ColladaParserAutoGen15Private::_preEnd__linestrips, &ColladaParserAutoGen15Private::_freeAttributes__linestrips);
functionMap[HASH_ELEMENT_POLYGONS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__polygons, &ColladaParserAutoGen15Private::_data__polygons, &ColladaParserAutoGen15Private::_end__polygons, &ColladaParserAutoGen15Private::_preBegin__polygons, &ColladaParserAutoGen15Private::_preEnd__polygons, &ColladaParserAutoGen15Private::_freeAttributes__polygons);
functionMap[HASH_ELEMENT_POLYLIST] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__polylist, &ColladaParserAutoGen15Private::_data__polylist, &ColladaParserAutoGen15Private::_end__polylist, &ColladaParserAutoGen15Private::_preBegin__polylist, &ColladaParserAutoGen15Private::_preEnd__polylist, &ColladaParserAutoGen15Private::_freeAttributes__polylist);
functionMap[HASH_ELEMENT_SOURCE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source, &ColladaParserAutoGen15Private::_data__source, &ColladaParserAutoGen15Private::_end__source, &ColladaParserAutoGen15Private::_preBegin__source, &ColladaParserAutoGen15Private::_preEnd__source, &ColladaParserAutoGen15Private::_freeAttributes__source);
functionMap[HASH_ELEMENT_TRIANGLES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__triangles, &ColladaParserAutoGen15Private::_data__triangles, &ColladaParserAutoGen15Private::_end__triangles, &ColladaParserAutoGen15Private::_preBegin__triangles, &ColladaParserAutoGen15Private::_preEnd__triangles, &ColladaParserAutoGen15Private::_freeAttributes__triangles);
functionMap[HASH_ELEMENT_TRIFANS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__trifans, &ColladaParserAutoGen15Private::_data__trifans, &ColladaParserAutoGen15Private::_end__trifans, &ColladaParserAutoGen15Private::_preBegin__trifans, &ColladaParserAutoGen15Private::_preEnd__trifans, &ColladaParserAutoGen15Private::_freeAttributes__trifans);
functionMap[HASH_ELEMENT_TRISTRIPS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__tristrips, &ColladaParserAutoGen15Private::_data__tristrips, &ColladaParserAutoGen15Private::_end__tristrips, &ColladaParserAutoGen15Private::_preBegin__tristrips, &ColladaParserAutoGen15Private::_preEnd__tristrips, &ColladaParserAutoGen15Private::_freeAttributes__tristrips);
functionMap[HASH_ELEMENT_VERTICES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__vertices, &ColladaParserAutoGen15Private::_data__vertices, &ColladaParserAutoGen15Private::_end__vertices, &ColladaParserAutoGen15Private::_preBegin__vertices, &ColladaParserAutoGen15Private::_preEnd__vertices, &ColladaParserAutoGen15Private::_freeAttributes__vertices);
functionMap[HASH_ELEMENT_SPLINE_TYPE____CONTROL_VERTICES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__spline_type____control_vertices, &ColladaParserAutoGen15Private::_data__spline_type____control_vertices, &ColladaParserAutoGen15Private::_end__spline_type____control_vertices, &ColladaParserAutoGen15Private::_preBegin__spline_type____control_vertices, &ColladaParserAutoGen15Private::_preEnd__spline_type____control_vertices, &ColladaParserAutoGen15Private::_freeAttributes__spline_type____control_vertices);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_SOURCE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source, &ColladaParserAutoGen15Private::_data__source, &ColladaParserAutoGen15Private::_end__source, &ColladaParserAutoGen15Private::_preBegin__source, &ColladaParserAutoGen15Private::_preEnd__source, &ColladaParserAutoGen15Private::_freeAttributes__source);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INPUT____INPUT_LOCAL_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__input____input_local_type, &ColladaParserAutoGen15Private::_data__input____input_local_type, &ColladaParserAutoGen15Private::_end__input____input_local_type, &ColladaParserAutoGen15Private::_preBegin__input____input_local_type, &ColladaParserAutoGen15Private::_preEnd__input____input_local_type, &ColladaParserAutoGen15Private::_freeAttributes__input____input_local_type);
mapFilled = true;
}
return functionMap;
}

//---------------------------------------------------------------------
const ColladaParserAutoGen15Private::ElementFunctionMap& FunctionMapFactory::createFunctionMap__library_images__allChildren() {
static ColladaParserAutoGen15Private::ElementFunctionMap functionMap;
static bool mapFilled = false;
if ( !mapFilled )
{
functionMap[HASH_ELEMENT_COLLADA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__COLLADA, &ColladaParserAutoGen15Private::_data__COLLADA, &ColladaParserAutoGen15Private::_end__COLLADA, &ColladaParserAutoGen15Private::_preBegin__COLLADA, &ColladaParserAutoGen15Private::_preEnd__COLLADA, &ColladaParserAutoGen15Private::_freeAttributes__COLLADA);
functionMap[HASH_ELEMENT_LIBRARY_IMAGES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_images, &ColladaParserAutoGen15Private::_data__library_images, &ColladaParserAutoGen15Private::_end__library_images, &ColladaParserAutoGen15Private::_preBegin__library_images, &ColladaParserAutoGen15Private::_preEnd__library_images, &ColladaParserAutoGen15Private::_freeAttributes__library_images);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_IMAGE____IMAGE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__image____image_type, &ColladaParserAutoGen15Private::_data__image____image_type, &ColladaParserAutoGen15Private::_end__image____image_type, &ColladaParserAutoGen15Private::_preBegin__image____image_type, &ColladaParserAutoGen15Private::_preEnd__image____image_type, &ColladaParserAutoGen15Private::_freeAttributes__image____image_type);
functionMap[HASH_ELEMENT_CONTRIBUTOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__contributor, &ColladaParserAutoGen15Private::_data__contributor, &ColladaParserAutoGen15Private::_end__contributor, &ColladaParserAutoGen15Private::_preBegin__contributor, &ColladaParserAutoGen15Private::_preEnd__contributor, &ColladaParserAutoGen15Private::_freeAttributes__contributor);
functionMap[HASH_ELEMENT_COVERAGE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__coverage, &ColladaParserAutoGen15Private::_data__coverage, &ColladaParserAutoGen15Private::_end__coverage, &ColladaParserAutoGen15Private::_preBegin__coverage, &ColladaParserAutoGen15Private::_preEnd__coverage, &ColladaParserAutoGen15Private::_freeAttributes__coverage);
functionMap[HASH_ELEMENT_CREATED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__created, &ColladaParserAutoGen15Private::_data__created, &ColladaParserAutoGen15Private::_end__created, &ColladaParserAutoGen15Private::_preBegin__created, &ColladaParserAutoGen15Private::_preEnd__created, &ColladaParserAutoGen15Private::_freeAttributes__created);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_AUTHOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author, &ColladaParserAutoGen15Private::_data__author, &ColladaParserAutoGen15Private::_end__author, &ColladaParserAutoGen15Private::_preBegin__author, &ColladaParserAutoGen15Private::_preEnd__author, &ColladaParserAutoGen15Private::_freeAttributes__author);
functionMap[HASH_ELEMENT_AUTHOR_EMAIL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_email, &ColladaParserAutoGen15Private::_data__author_email, &ColladaParserAutoGen15Private::_end__author_email, &ColladaParserAutoGen15Private::_preBegin__author_email, &ColladaParserAutoGen15Private::_preEnd__author_email, &ColladaParserAutoGen15Private::_freeAttributes__author_email);
functionMap[HASH_ELEMENT_AUTHOR_WEBSITE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_website, &ColladaParserAutoGen15Private::_data__author_website, &ColladaParserAutoGen15Private::_end__author_website, &ColladaParserAutoGen15Private::_preBegin__author_website, &ColladaParserAutoGen15Private::_preEnd__author_website, &ColladaParserAutoGen15Private::_freeAttributes__author_website);
functionMap[HASH_ELEMENT_AUTHORING_TOOL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__authoring_tool, &ColladaParserAutoGen15Private::_data__authoring_tool, &ColladaParserAutoGen15Private::_end__authoring_tool, &ColladaParserAutoGen15Private::_preBegin__authoring_tool, &ColladaParserAutoGen15Private::_preEnd__authoring_tool, &ColladaParserAutoGen15Private::_freeAttributes__authoring_tool);
functionMap[HASH_ELEMENT_COMMENTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__comments, &ColladaParserAutoGen15Private::_data__comments, &ColladaParserAutoGen15Private::_end__comments, &ColladaParserAutoGen15Private::_preBegin__comments, &ColladaParserAutoGen15Private::_preEnd__comments, &ColladaParserAutoGen15Private::_freeAttributes__comments);
functionMap[HASH_ELEMENT_COPYRIGHT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__copyright, &ColladaParserAutoGen15Private::_data__copyright, &ColladaParserAutoGen15Private::_end__copyright, &ColladaParserAutoGen15Private::_preBegin__copyright, &ColladaParserAutoGen15Private::_preEnd__copyright, &ColladaParserAutoGen15Private::_freeAttributes__copyright);
functionMap[HASH_ELEMENT_SOURCE_DATA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source_data, &ColladaParserAutoGen15Private::_data__source_data, &ColladaParserAutoGen15Private::_end__source_data, &ColladaParserAutoGen15Private::_preBegin__source_data, &ColladaParserAutoGen15Private::_preEnd__source_data, &ColladaParserAutoGen15Private::_freeAttributes__source_data);
functionMap[HASH_ELEMENT_GEOGRAPHIC_LOCATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__geographic_location, &ColladaParserAutoGen15Private::_data__geographic_location, &ColladaParserAutoGen15Private::_end__geographic_location, &ColladaParserAutoGen15Private::_preBegin__geographic_location, &ColladaParserAutoGen15Private::_preEnd__geographic_location, &ColladaParserAutoGen15Private::_freeAttributes__geographic_location);
functionMap[HASH_ELEMENT_ALTITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__altitude, &ColladaParserAutoGen15Private::_data__altitude, &ColladaParserAutoGen15Private::_end__altitude, &ColladaParserAutoGen15Private::_preBegin__altitude, &ColladaParserAutoGen15Private::_preEnd__altitude, &ColladaParserAutoGen15Private::_freeAttributes__altitude);
functionMap[HASH_ELEMENT_LATITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__latitude, &ColladaParserAutoGen15Private::_data__latitude, &ColladaParserAutoGen15Private::_end__latitude, &ColladaParserAutoGen15Private::_preBegin__latitude, &ColladaParserAutoGen15Private::_preEnd__latitude, &ColladaParserAutoGen15Private::_freeAttributes__latitude);
functionMap[HASH_ELEMENT_LONGITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__longitude, &ColladaParserAutoGen15Private::_data__longitude, &ColladaParserAutoGen15Private::_end__longitude, &ColladaParserAutoGen15Private::_preBegin__longitude, &ColladaParserAutoGen15Private::_preEnd__longitude, &ColladaParserAutoGen15Private::_freeAttributes__longitude);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_CREATE_2D] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_2d, &ColladaParserAutoGen15Private::_data__create_2d, &ColladaParserAutoGen15Private::_end__create_2d, &ColladaParserAutoGen15Private::_preBegin__create_2d, &ColladaParserAutoGen15Private::_preEnd__create_2d, &ColladaParserAutoGen15Private::_freeAttributes__create_2d);
functionMap[HASH_ELEMENT_CREATE_3D] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_3d, &ColladaParserAutoGen15Private::_data__create_3d, &ColladaParserAutoGen15Private::_end__create_3d, &ColladaParserAutoGen15Private::_preBegin__create_3d, &ColladaParserAutoGen15Private::_preEnd__create_3d, &ColladaParserAutoGen15Private::_freeAttributes__create_3d);
functionMap[HASH_ELEMENT_CREATE_CUBE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_cube, &ColladaParserAutoGen15Private::_data__create_cube, &ColladaParserAutoGen15Private::_end__create_cube, &ColladaParserAutoGen15Private::_preBegin__create_cube, &ColladaParserAutoGen15Private::_preEnd__create_cube, &ColladaParserAutoGen15Private::_freeAttributes__create_cube);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_IMAGE_TYPE____INIT_FROM] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__image_type____init_from, &ColladaParserAutoGen15Private::_data__image_type____init_from, &ColladaParserAutoGen15Private::_end__image_type____init_from, &ColladaParserAutoGen15Private::_preBegin__image_type____init_from, &ColladaParserAutoGen15Private::_preEnd__image_type____init_from, &ColladaParserAutoGen15Private::_freeAttributes__image_type____init_from);
functionMap[HASH_ELEMENT_RENDERABLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__renderable, &ColladaParserAutoGen15Private::_data__renderable, &ColladaParserAutoGen15Private::_end__renderable, &ColladaParserAutoGen15Private::_preBegin__renderable, &ColladaParserAutoGen15Private::_preEnd__renderable, &ColladaParserAutoGen15Private::_freeAttributes__renderable);
functionMap[HASH_ELEMENT_CREATE_2D__ARRAY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_2d__array, &ColladaParserAutoGen15Private::_data__create_2d__array, &ColladaParserAutoGen15Private::_end__create_2d__array, &ColladaParserAutoGen15Private::_preBegin__create_2d__array, &ColladaParserAutoGen15Private::_preEnd__create_2d__array, &ColladaParserAutoGen15Private::_freeAttributes__create_2d__array);
functionMap[HASH_ELEMENT_CREATE_2D__FORMAT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_2d__format, &ColladaParserAutoGen15Private::_data__create_2d__format, &ColladaParserAutoGen15Private::_end__create_2d__format, &ColladaParserAutoGen15Private::_preBegin__create_2d__format, &ColladaParserAutoGen15Private::_preEnd__create_2d__format, &ColladaParserAutoGen15Private::_freeAttributes__create_2d__format);
functionMap[HASH_ELEMENT_CREATE_2D__INIT_FROM] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_2d__init_from, &ColladaParserAutoGen15Private::_data__create_2d__init_from, &ColladaParserAutoGen15Private::_end__create_2d__init_from, &ColladaParserAutoGen15Private::_preBegin__create_2d__init_from, &ColladaParserAutoGen15Private::_preEnd__create_2d__init_from, &ColladaParserAutoGen15Private::_freeAttributes__create_2d__init_from);
functionMap[HASH_ELEMENT_MIPS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__mips, &ColladaParserAutoGen15Private::_data__mips, &ColladaParserAutoGen15Private::_end__mips, &ColladaParserAutoGen15Private::_preBegin__mips, &ColladaParserAutoGen15Private::_preEnd__mips, &ColladaParserAutoGen15Private::_freeAttributes__mips);
functionMap[HASH_ELEMENT_SIZE_EXACT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__size_exact, &ColladaParserAutoGen15Private::_data__size_exact, &ColladaParserAutoGen15Private::_end__size_exact, &ColladaParserAutoGen15Private::_preBegin__size_exact, &ColladaParserAutoGen15Private::_preEnd__size_exact, &ColladaParserAutoGen15Private::_freeAttributes__size_exact);
functionMap[HASH_ELEMENT_SIZE_RATIO] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__size_ratio, &ColladaParserAutoGen15Private::_data__size_ratio, &ColladaParserAutoGen15Private::_end__size_ratio, &ColladaParserAutoGen15Private::_preBegin__size_ratio, &ColladaParserAutoGen15Private::_preEnd__size_ratio, &ColladaParserAutoGen15Private::_freeAttributes__size_ratio);
functionMap[HASH_ELEMENT_UNNORMALIZED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unnormalized, &ColladaParserAutoGen15Private::_data__unnormalized, &ColladaParserAutoGen15Private::_end__unnormalized, &ColladaParserAutoGen15Private::_preBegin__unnormalized, &ColladaParserAutoGen15Private::_preEnd__unnormalized, &ColladaParserAutoGen15Private::_freeAttributes__unnormalized);
functionMap[HASH_ELEMENT_EXACT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__exact, &ColladaParserAutoGen15Private::_data__exact, &ColladaParserAutoGen15Private::_end__exact, &ColladaParserAutoGen15Private::_preBegin__exact, &ColladaParserAutoGen15Private::_preEnd__exact, &ColladaParserAutoGen15Private::_freeAttributes__exact);
functionMap[HASH_ELEMENT_CREATE_2D__FORMAT__HINT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_2d__format__hint, &ColladaParserAutoGen15Private::_data__create_2d__format__hint, &ColladaParserAutoGen15Private::_end__create_2d__format__hint, &ColladaParserAutoGen15Private::_preBegin__create_2d__format__hint, &ColladaParserAutoGen15Private::_preEnd__create_2d__format__hint, &ColladaParserAutoGen15Private::_freeAttributes__create_2d__format__hint);
functionMap[HASH_ELEMENT_LIBRARY_IMAGES__IMAGE__INIT_FROM__HEX] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_images__image__init_from__hex, &ColladaParserAutoGen15Private::_data__library_images__image__init_from__hex, &ColladaParserAutoGen15Private::_end__library_images__image__init_from__hex, &ColladaParserAutoGen15Private::_preBegin__library_images__image__init_from__hex, &ColladaParserAutoGen15Private::_preEnd__library_images__image__init_from__hex, &ColladaParserAutoGen15Private::_freeAttributes__library_images__image__init_from__hex);
functionMap[HASH_ELEMENT_REF____ANYURI] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__ref____anyURI, &ColladaParserAutoGen15Private::_data__ref____anyURI, &ColladaParserAutoGen15Private::_end__ref____anyURI, &ColladaParserAutoGen15Private::_preBegin__ref____anyURI, &ColladaParserAutoGen15Private::_preEnd__ref____anyURI, &ColladaParserAutoGen15Private::_freeAttributes__ref____anyURI);
functionMap[HASH_ELEMENT_CREATE_3D__ARRAY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_3d__array, &ColladaParserAutoGen15Private::_data__create_3d__array, &ColladaParserAutoGen15Private::_end__create_3d__array, &ColladaParserAutoGen15Private::_preBegin__create_3d__array, &ColladaParserAutoGen15Private::_preEnd__create_3d__array, &ColladaParserAutoGen15Private::_freeAttributes__create_3d__array);
functionMap[HASH_ELEMENT_CREATE_3D__FORMAT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_3d__format, &ColladaParserAutoGen15Private::_data__create_3d__format, &ColladaParserAutoGen15Private::_end__create_3d__format, &ColladaParserAutoGen15Private::_preBegin__create_3d__format, &ColladaParserAutoGen15Private::_preEnd__create_3d__format, &ColladaParserAutoGen15Private::_freeAttributes__create_3d__format);
functionMap[HASH_ELEMENT_CREATE_3D__INIT_FROM] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_3d__init_from, &ColladaParserAutoGen15Private::_data__create_3d__init_from, &ColladaParserAutoGen15Private::_end__create_3d__init_from, &ColladaParserAutoGen15Private::_preBegin__create_3d__init_from, &ColladaParserAutoGen15Private::_preEnd__create_3d__init_from, &ColladaParserAutoGen15Private::_freeAttributes__create_3d__init_from);
functionMap[HASH_ELEMENT_MIPS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__mips, &ColladaParserAutoGen15Private::_data__mips, &ColladaParserAutoGen15Private::_end__mips, &ColladaParserAutoGen15Private::_preBegin__mips, &ColladaParserAutoGen15Private::_preEnd__mips, &ColladaParserAutoGen15Private::_freeAttributes__mips);
functionMap[HASH_ELEMENT_CREATE_3D__SIZE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_3d__size, &ColladaParserAutoGen15Private::_data__create_3d__size, &ColladaParserAutoGen15Private::_end__create_3d__size, &ColladaParserAutoGen15Private::_preBegin__create_3d__size, &ColladaParserAutoGen15Private::_preEnd__create_3d__size, &ColladaParserAutoGen15Private::_freeAttributes__create_3d__size);
functionMap[HASH_ELEMENT_EXACT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__exact, &ColladaParserAutoGen15Private::_data__exact, &ColladaParserAutoGen15Private::_end__exact, &ColladaParserAutoGen15Private::_preBegin__exact, &ColladaParserAutoGen15Private::_preEnd__exact, &ColladaParserAutoGen15Private::_freeAttributes__exact);
functionMap[HASH_ELEMENT_CREATE_3D__FORMAT__HINT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_3d__format__hint, &ColladaParserAutoGen15Private::_data__create_3d__format__hint, &ColladaParserAutoGen15Private::_end__create_3d__format__hint, &ColladaParserAutoGen15Private::_preBegin__create_3d__format__hint, &ColladaParserAutoGen15Private::_preEnd__create_3d__format__hint, &ColladaParserAutoGen15Private::_freeAttributes__create_3d__format__hint);
functionMap[HASH_ELEMENT_CREATE_CUBE__ARRAY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_cube__array, &ColladaParserAutoGen15Private::_data__create_cube__array, &ColladaParserAutoGen15Private::_end__create_cube__array, &ColladaParserAutoGen15Private::_preBegin__create_cube__array, &ColladaParserAutoGen15Private::_preEnd__create_cube__array, &ColladaParserAutoGen15Private::_freeAttributes__create_cube__array);
functionMap[HASH_ELEMENT_CREATE_CUBE__FORMAT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_cube__format, &ColladaParserAutoGen15Private::_data__create_cube__format, &ColladaParserAutoGen15Private::_end__create_cube__format, &ColladaParserAutoGen15Private::_preBegin__create_cube__format, &ColladaParserAutoGen15Private::_preEnd__create_cube__format, &ColladaParserAutoGen15Private::_freeAttributes__create_cube__format);
functionMap[HASH_ELEMENT_CREATE_CUBE__INIT_FROM] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_cube__init_from, &ColladaParserAutoGen15Private::_data__create_cube__init_from, &ColladaParserAutoGen15Private::_end__create_cube__init_from, &ColladaParserAutoGen15Private::_preBegin__create_cube__init_from, &ColladaParserAutoGen15Private::_preEnd__create_cube__init_from, &ColladaParserAutoGen15Private::_freeAttributes__create_cube__init_from);
functionMap[HASH_ELEMENT_MIPS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__mips, &ColladaParserAutoGen15Private::_data__mips, &ColladaParserAutoGen15Private::_end__mips, &ColladaParserAutoGen15Private::_preBegin__mips, &ColladaParserAutoGen15Private::_preEnd__mips, &ColladaParserAutoGen15Private::_freeAttributes__mips);
functionMap[HASH_ELEMENT_CREATE_CUBE__SIZE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_cube__size, &ColladaParserAutoGen15Private::_data__create_cube__size, &ColladaParserAutoGen15Private::_end__create_cube__size, &ColladaParserAutoGen15Private::_preBegin__create_cube__size, &ColladaParserAutoGen15Private::_preEnd__create_cube__size, &ColladaParserAutoGen15Private::_freeAttributes__create_cube__size);
functionMap[HASH_ELEMENT_EXACT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__exact, &ColladaParserAutoGen15Private::_data__exact, &ColladaParserAutoGen15Private::_end__exact, &ColladaParserAutoGen15Private::_preBegin__exact, &ColladaParserAutoGen15Private::_preEnd__exact, &ColladaParserAutoGen15Private::_freeAttributes__exact);
functionMap[HASH_ELEMENT_CREATE_CUBE__FORMAT__HINT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__create_cube__format__hint, &ColladaParserAutoGen15Private::_data__create_cube__format__hint, &ColladaParserAutoGen15Private::_end__create_cube__format__hint, &ColladaParserAutoGen15Private::_preBegin__create_cube__format__hint, &ColladaParserAutoGen15Private::_preEnd__create_cube__format__hint, &ColladaParserAutoGen15Private::_freeAttributes__create_cube__format__hint);
mapFilled = true;
}
return functionMap;
}

//---------------------------------------------------------------------
const ColladaParserAutoGen15Private::ElementFunctionMap& FunctionMapFactory::createFunctionMap__library_lights__allChildren() {
static ColladaParserAutoGen15Private::ElementFunctionMap functionMap;
static bool mapFilled = false;
if ( !mapFilled )
{
functionMap[HASH_ELEMENT_COLLADA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__COLLADA, &ColladaParserAutoGen15Private::_data__COLLADA, &ColladaParserAutoGen15Private::_end__COLLADA, &ColladaParserAutoGen15Private::_preBegin__COLLADA, &ColladaParserAutoGen15Private::_preEnd__COLLADA, &ColladaParserAutoGen15Private::_freeAttributes__COLLADA);
functionMap[HASH_ELEMENT_LIBRARY_LIGHTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_lights, &ColladaParserAutoGen15Private::_data__library_lights, &ColladaParserAutoGen15Private::_end__library_lights, &ColladaParserAutoGen15Private::_preBegin__library_lights, &ColladaParserAutoGen15Private::_preEnd__library_lights, &ColladaParserAutoGen15Private::_freeAttributes__library_lights);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_LIGHT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__light, &ColladaParserAutoGen15Private::_data__light, &ColladaParserAutoGen15Private::_end__light, &ColladaParserAutoGen15Private::_preBegin__light, &ColladaParserAutoGen15Private::_preEnd__light, &ColladaParserAutoGen15Private::_freeAttributes__light);
functionMap[HASH_ELEMENT_CONTRIBUTOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__contributor, &ColladaParserAutoGen15Private::_data__contributor, &ColladaParserAutoGen15Private::_end__contributor, &ColladaParserAutoGen15Private::_preBegin__contributor, &ColladaParserAutoGen15Private::_preEnd__contributor, &ColladaParserAutoGen15Private::_freeAttributes__contributor);
functionMap[HASH_ELEMENT_COVERAGE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__coverage, &ColladaParserAutoGen15Private::_data__coverage, &ColladaParserAutoGen15Private::_end__coverage, &ColladaParserAutoGen15Private::_preBegin__coverage, &ColladaParserAutoGen15Private::_preEnd__coverage, &ColladaParserAutoGen15Private::_freeAttributes__coverage);
functionMap[HASH_ELEMENT_CREATED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__created, &ColladaParserAutoGen15Private::_data__created, &ColladaParserAutoGen15Private::_end__created, &ColladaParserAutoGen15Private::_preBegin__created, &ColladaParserAutoGen15Private::_preEnd__created, &ColladaParserAutoGen15Private::_freeAttributes__created);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_AUTHOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author, &ColladaParserAutoGen15Private::_data__author, &ColladaParserAutoGen15Private::_end__author, &ColladaParserAutoGen15Private::_preBegin__author, &ColladaParserAutoGen15Private::_preEnd__author, &ColladaParserAutoGen15Private::_freeAttributes__author);
functionMap[HASH_ELEMENT_AUTHOR_EMAIL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_email, &ColladaParserAutoGen15Private::_data__author_email, &ColladaParserAutoGen15Private::_end__author_email, &ColladaParserAutoGen15Private::_preBegin__author_email, &ColladaParserAutoGen15Private::_preEnd__author_email, &ColladaParserAutoGen15Private::_freeAttributes__author_email);
functionMap[HASH_ELEMENT_AUTHOR_WEBSITE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_website, &ColladaParserAutoGen15Private::_data__author_website, &ColladaParserAutoGen15Private::_end__author_website, &ColladaParserAutoGen15Private::_preBegin__author_website, &ColladaParserAutoGen15Private::_preEnd__author_website, &ColladaParserAutoGen15Private::_freeAttributes__author_website);
functionMap[HASH_ELEMENT_AUTHORING_TOOL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__authoring_tool, &ColladaParserAutoGen15Private::_data__authoring_tool, &ColladaParserAutoGen15Private::_end__authoring_tool, &ColladaParserAutoGen15Private::_preBegin__authoring_tool, &ColladaParserAutoGen15Private::_preEnd__authoring_tool, &ColladaParserAutoGen15Private::_freeAttributes__authoring_tool);
functionMap[HASH_ELEMENT_COMMENTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__comments, &ColladaParserAutoGen15Private::_data__comments, &ColladaParserAutoGen15Private::_end__comments, &ColladaParserAutoGen15Private::_preBegin__comments, &ColladaParserAutoGen15Private::_preEnd__comments, &ColladaParserAutoGen15Private::_freeAttributes__comments);
functionMap[HASH_ELEMENT_COPYRIGHT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__copyright, &ColladaParserAutoGen15Private::_data__copyright, &ColladaParserAutoGen15Private::_end__copyright, &ColladaParserAutoGen15Private::_preBegin__copyright, &ColladaParserAutoGen15Private::_preEnd__copyright, &ColladaParserAutoGen15Private::_freeAttributes__copyright);
functionMap[HASH_ELEMENT_SOURCE_DATA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source_data, &ColladaParserAutoGen15Private::_data__source_data, &ColladaParserAutoGen15Private::_end__source_data, &ColladaParserAutoGen15Private::_preBegin__source_data, &ColladaParserAutoGen15Private::_preEnd__source_data, &ColladaParserAutoGen15Private::_freeAttributes__source_data);
functionMap[HASH_ELEMENT_GEOGRAPHIC_LOCATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__geographic_location, &ColladaParserAutoGen15Private::_data__geographic_location, &ColladaParserAutoGen15Private::_end__geographic_location, &ColladaParserAutoGen15Private::_preBegin__geographic_location, &ColladaParserAutoGen15Private::_preEnd__geographic_location, &ColladaParserAutoGen15Private::_freeAttributes__geographic_location);
functionMap[HASH_ELEMENT_ALTITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__altitude, &ColladaParserAutoGen15Private::_data__altitude, &ColladaParserAutoGen15Private::_end__altitude, &ColladaParserAutoGen15Private::_preBegin__altitude, &ColladaParserAutoGen15Private::_preEnd__altitude, &ColladaParserAutoGen15Private::_freeAttributes__altitude);
functionMap[HASH_ELEMENT_LATITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__latitude, &ColladaParserAutoGen15Private::_data__latitude, &ColladaParserAutoGen15Private::_end__latitude, &ColladaParserAutoGen15Private::_preBegin__latitude, &ColladaParserAutoGen15Private::_preEnd__latitude, &ColladaParserAutoGen15Private::_freeAttributes__latitude);
functionMap[HASH_ELEMENT_LONGITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__longitude, &ColladaParserAutoGen15Private::_data__longitude, &ColladaParserAutoGen15Private::_end__longitude, &ColladaParserAutoGen15Private::_preBegin__longitude, &ColladaParserAutoGen15Private::_preEnd__longitude, &ColladaParserAutoGen15Private::_freeAttributes__longitude);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_LIGHT_TYPE____TECHNIQUE_COMMON] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__light_type____technique_common, &ColladaParserAutoGen15Private::_data__light_type____technique_common, &ColladaParserAutoGen15Private::_end__light_type____technique_common, &ColladaParserAutoGen15Private::_preBegin__light_type____technique_common, &ColladaParserAutoGen15Private::_preEnd__light_type____technique_common, &ColladaParserAutoGen15Private::_freeAttributes__light_type____technique_common);
functionMap[HASH_ELEMENT_LIGHT__TECHNIQUE_COMMON__AMBIENT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__light__technique_common__ambient, &ColladaParserAutoGen15Private::_data__light__technique_common__ambient, &ColladaParserAutoGen15Private::_end__light__technique_common__ambient, &ColladaParserAutoGen15Private::_preBegin__light__technique_common__ambient, &ColladaParserAutoGen15Private::_preEnd__light__technique_common__ambient, &ColladaParserAutoGen15Private::_freeAttributes__light__technique_common__ambient);
functionMap[HASH_ELEMENT_DIRECTIONAL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__directional, &ColladaParserAutoGen15Private::_data__directional, &ColladaParserAutoGen15Private::_end__directional, &ColladaParserAutoGen15Private::_preBegin__directional, &ColladaParserAutoGen15Private::_preEnd__directional, &ColladaParserAutoGen15Private::_freeAttributes__directional);
functionMap[HASH_ELEMENT_POINT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__point, &ColladaParserAutoGen15Private::_data__point, &ColladaParserAutoGen15Private::_end__point, &ColladaParserAutoGen15Private::_preBegin__point, &ColladaParserAutoGen15Private::_preEnd__point, &ColladaParserAutoGen15Private::_freeAttributes__point);
functionMap[HASH_ELEMENT_SPOT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__spot, &ColladaParserAutoGen15Private::_data__spot, &ColladaParserAutoGen15Private::_end__spot, &ColladaParserAutoGen15Private::_preBegin__spot, &ColladaParserAutoGen15Private::_preEnd__spot, &ColladaParserAutoGen15Private::_freeAttributes__spot);
functionMap[HASH_ELEMENT_COLOR____TARGETABLE_FLOAT3_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__color____targetable_float3_type, &ColladaParserAutoGen15Private::_data__color____targetable_float3_type, &ColladaParserAutoGen15Private::_end__color____targetable_float3_type, &ColladaParserAutoGen15Private::_preBegin__color____targetable_float3_type, &ColladaParserAutoGen15Private::_preEnd__color____targetable_float3_type, &ColladaParserAutoGen15Private::_freeAttributes__color____targetable_float3_type);
functionMap[HASH_ELEMENT_COLOR____TARGETABLE_FLOAT3_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__color____targetable_float3_type, &ColladaParserAutoGen15Private::_data__color____targetable_float3_type, &ColladaParserAutoGen15Private::_end__color____targetable_float3_type, &ColladaParserAutoGen15Private::_preBegin__color____targetable_float3_type, &ColladaParserAutoGen15Private::_preEnd__color____targetable_float3_type, &ColladaParserAutoGen15Private::_freeAttributes__color____targetable_float3_type);
functionMap[HASH_ELEMENT_COLOR____TARGETABLE_FLOAT3_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__color____targetable_float3_type, &ColladaParserAutoGen15Private::_data__color____targetable_float3_type, &ColladaParserAutoGen15Private::_end__color____targetable_float3_type, &ColladaParserAutoGen15Private::_preBegin__color____targetable_float3_type, &ColladaParserAutoGen15Private::_preEnd__color____targetable_float3_type, &ColladaParserAutoGen15Private::_freeAttributes__color____targetable_float3_type);
functionMap[HASH_ELEMENT_CONSTANT_ATTENUATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__constant_attenuation, &ColladaParserAutoGen15Private::_data__constant_attenuation, &ColladaParserAutoGen15Private::_end__constant_attenuation, &ColladaParserAutoGen15Private::_preBegin__constant_attenuation, &ColladaParserAutoGen15Private::_preEnd__constant_attenuation, &ColladaParserAutoGen15Private::_freeAttributes__constant_attenuation);
functionMap[HASH_ELEMENT_LINEAR_ATTENUATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__linear_attenuation, &ColladaParserAutoGen15Private::_data__linear_attenuation, &ColladaParserAutoGen15Private::_end__linear_attenuation, &ColladaParserAutoGen15Private::_preBegin__linear_attenuation, &ColladaParserAutoGen15Private::_preEnd__linear_attenuation, &ColladaParserAutoGen15Private::_freeAttributes__linear_attenuation);
functionMap[HASH_ELEMENT_QUADRATIC_ATTENUATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__quadratic_attenuation, &ColladaParserAutoGen15Private::_data__quadratic_attenuation, &ColladaParserAutoGen15Private::_end__quadratic_attenuation, &ColladaParserAutoGen15Private::_preBegin__quadratic_attenuation, &ColladaParserAutoGen15Private::_preEnd__quadratic_attenuation, &ColladaParserAutoGen15Private::_freeAttributes__quadratic_attenuation);
functionMap[HASH_ELEMENT_COLOR____TARGETABLE_FLOAT3_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__color____targetable_float3_type, &ColladaParserAutoGen15Private::_data__color____targetable_float3_type, &ColladaParserAutoGen15Private::_end__color____targetable_float3_type, &ColladaParserAutoGen15Private::_preBegin__color____targetable_float3_type, &ColladaParserAutoGen15Private::_preEnd__color____targetable_float3_type, &ColladaParserAutoGen15Private::_freeAttributes__color____targetable_float3_type);
functionMap[HASH_ELEMENT_CONSTANT_ATTENUATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__constant_attenuation, &ColladaParserAutoGen15Private::_data__constant_attenuation, &ColladaParserAutoGen15Private::_end__constant_attenuation, &ColladaParserAutoGen15Private::_preBegin__constant_attenuation, &ColladaParserAutoGen15Private::_preEnd__constant_attenuation, &ColladaParserAutoGen15Private::_freeAttributes__constant_attenuation);
functionMap[HASH_ELEMENT_FALLOFF_ANGLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__falloff_angle, &ColladaParserAutoGen15Private::_data__falloff_angle, &ColladaParserAutoGen15Private::_end__falloff_angle, &ColladaParserAutoGen15Private::_preBegin__falloff_angle, &ColladaParserAutoGen15Private::_preEnd__falloff_angle, &ColladaParserAutoGen15Private::_freeAttributes__falloff_angle);
functionMap[HASH_ELEMENT_FALLOFF_EXPONENT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__falloff_exponent, &ColladaParserAutoGen15Private::_data__falloff_exponent, &ColladaParserAutoGen15Private::_end__falloff_exponent, &ColladaParserAutoGen15Private::_preBegin__falloff_exponent, &ColladaParserAutoGen15Private::_preEnd__falloff_exponent, &ColladaParserAutoGen15Private::_freeAttributes__falloff_exponent);
functionMap[HASH_ELEMENT_LINEAR_ATTENUATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__linear_attenuation, &ColladaParserAutoGen15Private::_data__linear_attenuation, &ColladaParserAutoGen15Private::_end__linear_attenuation, &ColladaParserAutoGen15Private::_preBegin__linear_attenuation, &ColladaParserAutoGen15Private::_preEnd__linear_attenuation, &ColladaParserAutoGen15Private::_freeAttributes__linear_attenuation);
functionMap[HASH_ELEMENT_QUADRATIC_ATTENUATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__quadratic_attenuation, &ColladaParserAutoGen15Private::_data__quadratic_attenuation, &ColladaParserAutoGen15Private::_end__quadratic_attenuation, &ColladaParserAutoGen15Private::_preBegin__quadratic_attenuation, &ColladaParserAutoGen15Private::_preEnd__quadratic_attenuation, &ColladaParserAutoGen15Private::_freeAttributes__quadratic_attenuation);
mapFilled = true;
}
return functionMap;
}

//---------------------------------------------------------------------
const ColladaParserAutoGen15Private::ElementFunctionMap& FunctionMapFactory::createFunctionMap__library_materials__allChildren() {
static ColladaParserAutoGen15Private::ElementFunctionMap functionMap;
static bool mapFilled = false;
if ( !mapFilled )
{
functionMap[HASH_ELEMENT_COLLADA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__COLLADA, &ColladaParserAutoGen15Private::_data__COLLADA, &ColladaParserAutoGen15Private::_end__COLLADA, &ColladaParserAutoGen15Private::_preBegin__COLLADA, &ColladaParserAutoGen15Private::_preEnd__COLLADA, &ColladaParserAutoGen15Private::_freeAttributes__COLLADA);
functionMap[HASH_ELEMENT_LIBRARY_MATERIALS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_materials, &ColladaParserAutoGen15Private::_data__library_materials, &ColladaParserAutoGen15Private::_end__library_materials, &ColladaParserAutoGen15Private::_preBegin__library_materials, &ColladaParserAutoGen15Private::_preEnd__library_materials, &ColladaParserAutoGen15Private::_freeAttributes__library_materials);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_MATERIAL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__material, &ColladaParserAutoGen15Private::_data__material, &ColladaParserAutoGen15Private::_end__material, &ColladaParserAutoGen15Private::_preBegin__material, &ColladaParserAutoGen15Private::_preEnd__material, &ColladaParserAutoGen15Private::_freeAttributes__material);
functionMap[HASH_ELEMENT_CONTRIBUTOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__contributor, &ColladaParserAutoGen15Private::_data__contributor, &ColladaParserAutoGen15Private::_end__contributor, &ColladaParserAutoGen15Private::_preBegin__contributor, &ColladaParserAutoGen15Private::_preEnd__contributor, &ColladaParserAutoGen15Private::_freeAttributes__contributor);
functionMap[HASH_ELEMENT_COVERAGE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__coverage, &ColladaParserAutoGen15Private::_data__coverage, &ColladaParserAutoGen15Private::_end__coverage, &ColladaParserAutoGen15Private::_preBegin__coverage, &ColladaParserAutoGen15Private::_preEnd__coverage, &ColladaParserAutoGen15Private::_freeAttributes__coverage);
functionMap[HASH_ELEMENT_CREATED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__created, &ColladaParserAutoGen15Private::_data__created, &ColladaParserAutoGen15Private::_end__created, &ColladaParserAutoGen15Private::_preBegin__created, &ColladaParserAutoGen15Private::_preEnd__created, &ColladaParserAutoGen15Private::_freeAttributes__created);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_AUTHOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author, &ColladaParserAutoGen15Private::_data__author, &ColladaParserAutoGen15Private::_end__author, &ColladaParserAutoGen15Private::_preBegin__author, &ColladaParserAutoGen15Private::_preEnd__author, &ColladaParserAutoGen15Private::_freeAttributes__author);
functionMap[HASH_ELEMENT_AUTHOR_EMAIL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_email, &ColladaParserAutoGen15Private::_data__author_email, &ColladaParserAutoGen15Private::_end__author_email, &ColladaParserAutoGen15Private::_preBegin__author_email, &ColladaParserAutoGen15Private::_preEnd__author_email, &ColladaParserAutoGen15Private::_freeAttributes__author_email);
functionMap[HASH_ELEMENT_AUTHOR_WEBSITE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_website, &ColladaParserAutoGen15Private::_data__author_website, &ColladaParserAutoGen15Private::_end__author_website, &ColladaParserAutoGen15Private::_preBegin__author_website, &ColladaParserAutoGen15Private::_preEnd__author_website, &ColladaParserAutoGen15Private::_freeAttributes__author_website);
functionMap[HASH_ELEMENT_AUTHORING_TOOL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__authoring_tool, &ColladaParserAutoGen15Private::_data__authoring_tool, &ColladaParserAutoGen15Private::_end__authoring_tool, &ColladaParserAutoGen15Private::_preBegin__authoring_tool, &ColladaParserAutoGen15Private::_preEnd__authoring_tool, &ColladaParserAutoGen15Private::_freeAttributes__authoring_tool);
functionMap[HASH_ELEMENT_COMMENTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__comments, &ColladaParserAutoGen15Private::_data__comments, &ColladaParserAutoGen15Private::_end__comments, &ColladaParserAutoGen15Private::_preBegin__comments, &ColladaParserAutoGen15Private::_preEnd__comments, &ColladaParserAutoGen15Private::_freeAttributes__comments);
functionMap[HASH_ELEMENT_COPYRIGHT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__copyright, &ColladaParserAutoGen15Private::_data__copyright, &ColladaParserAutoGen15Private::_end__copyright, &ColladaParserAutoGen15Private::_preBegin__copyright, &ColladaParserAutoGen15Private::_preEnd__copyright, &ColladaParserAutoGen15Private::_freeAttributes__copyright);
functionMap[HASH_ELEMENT_SOURCE_DATA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source_data, &ColladaParserAutoGen15Private::_data__source_data, &ColladaParserAutoGen15Private::_end__source_data, &ColladaParserAutoGen15Private::_preBegin__source_data, &ColladaParserAutoGen15Private::_preEnd__source_data, &ColladaParserAutoGen15Private::_freeAttributes__source_data);
functionMap[HASH_ELEMENT_GEOGRAPHIC_LOCATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__geographic_location, &ColladaParserAutoGen15Private::_data__geographic_location, &ColladaParserAutoGen15Private::_end__geographic_location, &ColladaParserAutoGen15Private::_preBegin__geographic_location, &ColladaParserAutoGen15Private::_preEnd__geographic_location, &ColladaParserAutoGen15Private::_freeAttributes__geographic_location);
functionMap[HASH_ELEMENT_ALTITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__altitude, &ColladaParserAutoGen15Private::_data__altitude, &ColladaParserAutoGen15Private::_end__altitude, &ColladaParserAutoGen15Private::_preBegin__altitude, &ColladaParserAutoGen15Private::_preEnd__altitude, &ColladaParserAutoGen15Private::_freeAttributes__altitude);
functionMap[HASH_ELEMENT_LATITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__latitude, &ColladaParserAutoGen15Private::_data__latitude, &ColladaParserAutoGen15Private::_end__latitude, &ColladaParserAutoGen15Private::_preBegin__latitude, &ColladaParserAutoGen15Private::_preEnd__latitude, &ColladaParserAutoGen15Private::_freeAttributes__latitude);
functionMap[HASH_ELEMENT_LONGITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__longitude, &ColladaParserAutoGen15Private::_data__longitude, &ColladaParserAutoGen15Private::_end__longitude, &ColladaParserAutoGen15Private::_preBegin__longitude, &ColladaParserAutoGen15Private::_preEnd__longitude, &ColladaParserAutoGen15Private::_freeAttributes__longitude);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INSTANCE_EFFECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_effect, &ColladaParserAutoGen15Private::_data__instance_effect, &ColladaParserAutoGen15Private::_end__instance_effect, &ColladaParserAutoGen15Private::_preBegin__instance_effect, &ColladaParserAutoGen15Private::_preEnd__instance_effect, &ColladaParserAutoGen15Private::_freeAttributes__instance_effect);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INSTANCE_EFFECT_TYPE____SETPARAM] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_effect_type____setparam, &ColladaParserAutoGen15Private::_data__instance_effect_type____setparam, &ColladaParserAutoGen15Private::_end__instance_effect_type____setparam, &ColladaParserAutoGen15Private::_preBegin__instance_effect_type____setparam, &ColladaParserAutoGen15Private::_preEnd__instance_effect_type____setparam, &ColladaParserAutoGen15Private::_freeAttributes__instance_effect_type____setparam);
functionMap[HASH_ELEMENT_TECHNIQUE_HINT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique_hint, &ColladaParserAutoGen15Private::_data__technique_hint, &ColladaParserAutoGen15Private::_end__technique_hint, &ColladaParserAutoGen15Private::_preBegin__technique_hint, &ColladaParserAutoGen15Private::_preEnd__technique_hint, &ColladaParserAutoGen15Private::_freeAttributes__technique_hint);
functionMap[HASH_ELEMENT_BOOL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool, &ColladaParserAutoGen15Private::_data__bool, &ColladaParserAutoGen15Private::_end__bool, &ColladaParserAutoGen15Private::_preBegin__bool, &ColladaParserAutoGen15Private::_preEnd__bool, &ColladaParserAutoGen15Private::_freeAttributes__bool);
functionMap[HASH_ELEMENT_BOOL2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool2, &ColladaParserAutoGen15Private::_data__bool2, &ColladaParserAutoGen15Private::_end__bool2, &ColladaParserAutoGen15Private::_preBegin__bool2, &ColladaParserAutoGen15Private::_preEnd__bool2, &ColladaParserAutoGen15Private::_freeAttributes__bool2);
functionMap[HASH_ELEMENT_BOOL3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool3, &ColladaParserAutoGen15Private::_data__bool3, &ColladaParserAutoGen15Private::_end__bool3, &ColladaParserAutoGen15Private::_preBegin__bool3, &ColladaParserAutoGen15Private::_preEnd__bool3, &ColladaParserAutoGen15Private::_freeAttributes__bool3);
functionMap[HASH_ELEMENT_BOOL4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bool4, &ColladaParserAutoGen15Private::_data__bool4, &ColladaParserAutoGen15Private::_end__bool4, &ColladaParserAutoGen15Private::_preBegin__bool4, &ColladaParserAutoGen15Private::_preEnd__bool4, &ColladaParserAutoGen15Private::_freeAttributes__bool4);
functionMap[HASH_ELEMENT_ENUM____STRING] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__enum____string, &ColladaParserAutoGen15Private::_data__enum____string, &ColladaParserAutoGen15Private::_end__enum____string, &ColladaParserAutoGen15Private::_preBegin__enum____string, &ColladaParserAutoGen15Private::_preEnd__enum____string, &ColladaParserAutoGen15Private::_freeAttributes__enum____string);
functionMap[HASH_ELEMENT_FLOAT____FLOAT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float____float_type, &ColladaParserAutoGen15Private::_data__float____float_type, &ColladaParserAutoGen15Private::_end__float____float_type, &ColladaParserAutoGen15Private::_preBegin__float____float_type, &ColladaParserAutoGen15Private::_preEnd__float____float_type, &ColladaParserAutoGen15Private::_freeAttributes__float____float_type);
functionMap[HASH_ELEMENT_FLOAT2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2, &ColladaParserAutoGen15Private::_data__float2, &ColladaParserAutoGen15Private::_end__float2, &ColladaParserAutoGen15Private::_preBegin__float2, &ColladaParserAutoGen15Private::_preEnd__float2, &ColladaParserAutoGen15Private::_freeAttributes__float2);
functionMap[HASH_ELEMENT_FLOAT2X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2x1, &ColladaParserAutoGen15Private::_data__float2x1, &ColladaParserAutoGen15Private::_end__float2x1, &ColladaParserAutoGen15Private::_preBegin__float2x1, &ColladaParserAutoGen15Private::_preEnd__float2x1, &ColladaParserAutoGen15Private::_freeAttributes__float2x1);
functionMap[HASH_ELEMENT_FLOAT2X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2x2, &ColladaParserAutoGen15Private::_data__float2x2, &ColladaParserAutoGen15Private::_end__float2x2, &ColladaParserAutoGen15Private::_preBegin__float2x2, &ColladaParserAutoGen15Private::_preEnd__float2x2, &ColladaParserAutoGen15Private::_freeAttributes__float2x2);
functionMap[HASH_ELEMENT_FLOAT2X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2x3, &ColladaParserAutoGen15Private::_data__float2x3, &ColladaParserAutoGen15Private::_end__float2x3, &ColladaParserAutoGen15Private::_preBegin__float2x3, &ColladaParserAutoGen15Private::_preEnd__float2x3, &ColladaParserAutoGen15Private::_freeAttributes__float2x3);
functionMap[HASH_ELEMENT_FLOAT2X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float2x4, &ColladaParserAutoGen15Private::_data__float2x4, &ColladaParserAutoGen15Private::_end__float2x4, &ColladaParserAutoGen15Private::_preBegin__float2x4, &ColladaParserAutoGen15Private::_preEnd__float2x4, &ColladaParserAutoGen15Private::_freeAttributes__float2x4);
functionMap[HASH_ELEMENT_FLOAT3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3, &ColladaParserAutoGen15Private::_data__float3, &ColladaParserAutoGen15Private::_end__float3, &ColladaParserAutoGen15Private::_preBegin__float3, &ColladaParserAutoGen15Private::_preEnd__float3, &ColladaParserAutoGen15Private::_freeAttributes__float3);
functionMap[HASH_ELEMENT_FLOAT3X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3x1, &ColladaParserAutoGen15Private::_data__float3x1, &ColladaParserAutoGen15Private::_end__float3x1, &ColladaParserAutoGen15Private::_preBegin__float3x1, &ColladaParserAutoGen15Private::_preEnd__float3x1, &ColladaParserAutoGen15Private::_freeAttributes__float3x1);
functionMap[HASH_ELEMENT_FLOAT3X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3x2, &ColladaParserAutoGen15Private::_data__float3x2, &ColladaParserAutoGen15Private::_end__float3x2, &ColladaParserAutoGen15Private::_preBegin__float3x2, &ColladaParserAutoGen15Private::_preEnd__float3x2, &ColladaParserAutoGen15Private::_freeAttributes__float3x2);
functionMap[HASH_ELEMENT_FLOAT3X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3x3, &ColladaParserAutoGen15Private::_data__float3x3, &ColladaParserAutoGen15Private::_end__float3x3, &ColladaParserAutoGen15Private::_preBegin__float3x3, &ColladaParserAutoGen15Private::_preEnd__float3x3, &ColladaParserAutoGen15Private::_freeAttributes__float3x3);
functionMap[HASH_ELEMENT_FLOAT3X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float3x4, &ColladaParserAutoGen15Private::_data__float3x4, &ColladaParserAutoGen15Private::_end__float3x4, &ColladaParserAutoGen15Private::_preBegin__float3x4, &ColladaParserAutoGen15Private::_preEnd__float3x4, &ColladaParserAutoGen15Private::_freeAttributes__float3x4);
functionMap[HASH_ELEMENT_FLOAT4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4, &ColladaParserAutoGen15Private::_data__float4, &ColladaParserAutoGen15Private::_end__float4, &ColladaParserAutoGen15Private::_preBegin__float4, &ColladaParserAutoGen15Private::_preEnd__float4, &ColladaParserAutoGen15Private::_freeAttributes__float4);
functionMap[HASH_ELEMENT_FLOAT4X1] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4x1, &ColladaParserAutoGen15Private::_data__float4x1, &ColladaParserAutoGen15Private::_end__float4x1, &ColladaParserAutoGen15Private::_preBegin__float4x1, &ColladaParserAutoGen15Private::_preEnd__float4x1, &ColladaParserAutoGen15Private::_freeAttributes__float4x1);
functionMap[HASH_ELEMENT_FLOAT4X2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4x2, &ColladaParserAutoGen15Private::_data__float4x2, &ColladaParserAutoGen15Private::_end__float4x2, &ColladaParserAutoGen15Private::_preBegin__float4x2, &ColladaParserAutoGen15Private::_preEnd__float4x2, &ColladaParserAutoGen15Private::_freeAttributes__float4x2);
functionMap[HASH_ELEMENT_FLOAT4X3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4x3, &ColladaParserAutoGen15Private::_data__float4x3, &ColladaParserAutoGen15Private::_end__float4x3, &ColladaParserAutoGen15Private::_preBegin__float4x3, &ColladaParserAutoGen15Private::_preEnd__float4x3, &ColladaParserAutoGen15Private::_freeAttributes__float4x3);
functionMap[HASH_ELEMENT_FLOAT4X4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__float4x4, &ColladaParserAutoGen15Private::_data__float4x4, &ColladaParserAutoGen15Private::_end__float4x4, &ColladaParserAutoGen15Private::_preBegin__float4x4, &ColladaParserAutoGen15Private::_preEnd__float4x4, &ColladaParserAutoGen15Private::_freeAttributes__float4x4);
functionMap[HASH_ELEMENT_INT____INT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int____int_type, &ColladaParserAutoGen15Private::_data__int____int_type, &ColladaParserAutoGen15Private::_end__int____int_type, &ColladaParserAutoGen15Private::_preBegin__int____int_type, &ColladaParserAutoGen15Private::_preEnd__int____int_type, &ColladaParserAutoGen15Private::_freeAttributes__int____int_type);
functionMap[HASH_ELEMENT_INT2] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int2, &ColladaParserAutoGen15Private::_data__int2, &ColladaParserAutoGen15Private::_end__int2, &ColladaParserAutoGen15Private::_preBegin__int2, &ColladaParserAutoGen15Private::_preEnd__int2, &ColladaParserAutoGen15Private::_freeAttributes__int2);
functionMap[HASH_ELEMENT_INT3] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int3, &ColladaParserAutoGen15Private::_data__int3, &ColladaParserAutoGen15Private::_end__int3, &ColladaParserAutoGen15Private::_preBegin__int3, &ColladaParserAutoGen15Private::_preEnd__int3, &ColladaParserAutoGen15Private::_freeAttributes__int3);
functionMap[HASH_ELEMENT_INT4] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__int4, &ColladaParserAutoGen15Private::_data__int4, &ColladaParserAutoGen15Private::_end__int4, &ColladaParserAutoGen15Private::_preBegin__int4, &ColladaParserAutoGen15Private::_preEnd__int4, &ColladaParserAutoGen15Private::_freeAttributes__int4);
functionMap[HASH_ELEMENT_SAMPLER_IMAGE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler_image, &ColladaParserAutoGen15Private::_data__sampler_image, &ColladaParserAutoGen15Private::_end__sampler_image, &ColladaParserAutoGen15Private::_preBegin__sampler_image, &ColladaParserAutoGen15Private::_preEnd__sampler_image, &ColladaParserAutoGen15Private::_freeAttributes__sampler_image);
functionMap[HASH_ELEMENT_SAMPLER_STATES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sampler_states, &ColladaParserAutoGen15Private::_data__sampler_states, &ColladaParserAutoGen15Private::_end__sampler_states, &ColladaParserAutoGen15Private::_preBegin__sampler_states, &ColladaParserAutoGen15Private::_preEnd__sampler_states, &ColladaParserAutoGen15Private::_freeAttributes__sampler_states);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_BORDER_COLOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__border_color, &ColladaParserAutoGen15Private::_data__border_color, &ColladaParserAutoGen15Private::_end__border_color, &ColladaParserAutoGen15Private::_preBegin__border_color, &ColladaParserAutoGen15Private::_preEnd__border_color, &ColladaParserAutoGen15Private::_freeAttributes__border_color);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_MAGFILTER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__magfilter, &ColladaParserAutoGen15Private::_data__magfilter, &ColladaParserAutoGen15Private::_end__magfilter, &ColladaParserAutoGen15Private::_preBegin__magfilter, &ColladaParserAutoGen15Private::_preEnd__magfilter, &ColladaParserAutoGen15Private::_freeAttributes__magfilter);
functionMap[HASH_ELEMENT_MAX_ANISOTROPY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__max_anisotropy, &ColladaParserAutoGen15Private::_data__max_anisotropy, &ColladaParserAutoGen15Private::_end__max_anisotropy, &ColladaParserAutoGen15Private::_preBegin__max_anisotropy, &ColladaParserAutoGen15Private::_preEnd__max_anisotropy, &ColladaParserAutoGen15Private::_freeAttributes__max_anisotropy);
functionMap[HASH_ELEMENT_MINFILTER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__minfilter, &ColladaParserAutoGen15Private::_data__minfilter, &ColladaParserAutoGen15Private::_end__minfilter, &ColladaParserAutoGen15Private::_preBegin__minfilter, &ColladaParserAutoGen15Private::_preEnd__minfilter, &ColladaParserAutoGen15Private::_freeAttributes__minfilter);
functionMap[HASH_ELEMENT_MIP_BIAS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__mip_bias, &ColladaParserAutoGen15Private::_data__mip_bias, &ColladaParserAutoGen15Private::_end__mip_bias, &ColladaParserAutoGen15Private::_preBegin__mip_bias, &ColladaParserAutoGen15Private::_preEnd__mip_bias, &ColladaParserAutoGen15Private::_freeAttributes__mip_bias);
functionMap[HASH_ELEMENT_MIP_MAX_LEVEL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__mip_max_level, &ColladaParserAutoGen15Private::_data__mip_max_level, &ColladaParserAutoGen15Private::_end__mip_max_level, &ColladaParserAutoGen15Private::_preBegin__mip_max_level, &ColladaParserAutoGen15Private::_preEnd__mip_max_level, &ColladaParserAutoGen15Private::_freeAttributes__mip_max_level);
functionMap[HASH_ELEMENT_MIP_MIN_LEVEL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__mip_min_level, &ColladaParserAutoGen15Private::_data__mip_min_level, &ColladaParserAutoGen15Private::_end__mip_min_level, &ColladaParserAutoGen15Private::_preBegin__mip_min_level, &ColladaParserAutoGen15Private::_preEnd__mip_min_level, &ColladaParserAutoGen15Private::_freeAttributes__mip_min_level);
functionMap[HASH_ELEMENT_MIPFILTER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__mipfilter, &ColladaParserAutoGen15Private::_data__mipfilter, &ColladaParserAutoGen15Private::_end__mipfilter, &ColladaParserAutoGen15Private::_preBegin__mipfilter, &ColladaParserAutoGen15Private::_preEnd__mipfilter, &ColladaParserAutoGen15Private::_freeAttributes__mipfilter);
functionMap[HASH_ELEMENT_WRAP_P] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__wrap_p, &ColladaParserAutoGen15Private::_data__wrap_p, &ColladaParserAutoGen15Private::_end__wrap_p, &ColladaParserAutoGen15Private::_preBegin__wrap_p, &ColladaParserAutoGen15Private::_preEnd__wrap_p, &ColladaParserAutoGen15Private::_freeAttributes__wrap_p);
functionMap[HASH_ELEMENT_WRAP_S____FX_SAMPLER_WRAP_ENUM] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__wrap_s____fx_sampler_wrap_enum, &ColladaParserAutoGen15Private::_data__wrap_s____fx_sampler_wrap_enum, &ColladaParserAutoGen15Private::_end__wrap_s____fx_sampler_wrap_enum, &ColladaParserAutoGen15Private::_preBegin__wrap_s____fx_sampler_wrap_enum, &ColladaParserAutoGen15Private::_preEnd__wrap_s____fx_sampler_wrap_enum, &ColladaParserAutoGen15Private::_freeAttributes__wrap_s____fx_sampler_wrap_enum);
functionMap[HASH_ELEMENT_WRAP_T____FX_SAMPLER_WRAP_ENUM] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__wrap_t____fx_sampler_wrap_enum, &ColladaParserAutoGen15Private::_data__wrap_t____fx_sampler_wrap_enum, &ColladaParserAutoGen15Private::_end__wrap_t____fx_sampler_wrap_enum, &ColladaParserAutoGen15Private::_preBegin__wrap_t____fx_sampler_wrap_enum, &ColladaParserAutoGen15Private::_preEnd__wrap_t____fx_sampler_wrap_enum, &ColladaParserAutoGen15Private::_freeAttributes__wrap_t____fx_sampler_wrap_enum);
mapFilled = true;
}
return functionMap;
}

//---------------------------------------------------------------------
const ColladaParserAutoGen15Private::ElementFunctionMap& FunctionMapFactory::createFunctionMap__library_nodes__allChildren() {
static ColladaParserAutoGen15Private::ElementFunctionMap functionMap;
static bool mapFilled = false;
if ( !mapFilled )
{
functionMap[HASH_ELEMENT_COLLADA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__COLLADA, &ColladaParserAutoGen15Private::_data__COLLADA, &ColladaParserAutoGen15Private::_end__COLLADA, &ColladaParserAutoGen15Private::_preBegin__COLLADA, &ColladaParserAutoGen15Private::_preEnd__COLLADA, &ColladaParserAutoGen15Private::_freeAttributes__COLLADA);
functionMap[HASH_ELEMENT_LIBRARY_NODES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_nodes, &ColladaParserAutoGen15Private::_data__library_nodes, &ColladaParserAutoGen15Private::_end__library_nodes, &ColladaParserAutoGen15Private::_preBegin__library_nodes, &ColladaParserAutoGen15Private::_preEnd__library_nodes, &ColladaParserAutoGen15Private::_freeAttributes__library_nodes);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_NODE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__node, &ColladaParserAutoGen15Private::_data__node, &ColladaParserAutoGen15Private::_end__node, &ColladaParserAutoGen15Private::_preBegin__node, &ColladaParserAutoGen15Private::_preEnd__node, &ColladaParserAutoGen15Private::_freeAttributes__node);
functionMap[HASH_ELEMENT_CONTRIBUTOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__contributor, &ColladaParserAutoGen15Private::_data__contributor, &ColladaParserAutoGen15Private::_end__contributor, &ColladaParserAutoGen15Private::_preBegin__contributor, &ColladaParserAutoGen15Private::_preEnd__contributor, &ColladaParserAutoGen15Private::_freeAttributes__contributor);
functionMap[HASH_ELEMENT_COVERAGE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__coverage, &ColladaParserAutoGen15Private::_data__coverage, &ColladaParserAutoGen15Private::_end__coverage, &ColladaParserAutoGen15Private::_preBegin__coverage, &ColladaParserAutoGen15Private::_preEnd__coverage, &ColladaParserAutoGen15Private::_freeAttributes__coverage);
functionMap[HASH_ELEMENT_CREATED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__created, &ColladaParserAutoGen15Private::_data__created, &ColladaParserAutoGen15Private::_end__created, &ColladaParserAutoGen15Private::_preBegin__created, &ColladaParserAutoGen15Private::_preEnd__created, &ColladaParserAutoGen15Private::_freeAttributes__created);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_AUTHOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author, &ColladaParserAutoGen15Private::_data__author, &ColladaParserAutoGen15Private::_end__author, &ColladaParserAutoGen15Private::_preBegin__author, &ColladaParserAutoGen15Private::_preEnd__author, &ColladaParserAutoGen15Private::_freeAttributes__author);
functionMap[HASH_ELEMENT_AUTHOR_EMAIL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_email, &ColladaParserAutoGen15Private::_data__author_email, &ColladaParserAutoGen15Private::_end__author_email, &ColladaParserAutoGen15Private::_preBegin__author_email, &ColladaParserAutoGen15Private::_preEnd__author_email, &ColladaParserAutoGen15Private::_freeAttributes__author_email);
functionMap[HASH_ELEMENT_AUTHOR_WEBSITE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_website, &ColladaParserAutoGen15Private::_data__author_website, &ColladaParserAutoGen15Private::_end__author_website, &ColladaParserAutoGen15Private::_preBegin__author_website, &ColladaParserAutoGen15Private::_preEnd__author_website, &ColladaParserAutoGen15Private::_freeAttributes__author_website);
functionMap[HASH_ELEMENT_AUTHORING_TOOL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__authoring_tool, &ColladaParserAutoGen15Private::_data__authoring_tool, &ColladaParserAutoGen15Private::_end__authoring_tool, &ColladaParserAutoGen15Private::_preBegin__authoring_tool, &ColladaParserAutoGen15Private::_preEnd__authoring_tool, &ColladaParserAutoGen15Private::_freeAttributes__authoring_tool);
functionMap[HASH_ELEMENT_COMMENTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__comments, &ColladaParserAutoGen15Private::_data__comments, &ColladaParserAutoGen15Private::_end__comments, &ColladaParserAutoGen15Private::_preBegin__comments, &ColladaParserAutoGen15Private::_preEnd__comments, &ColladaParserAutoGen15Private::_freeAttributes__comments);
functionMap[HASH_ELEMENT_COPYRIGHT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__copyright, &ColladaParserAutoGen15Private::_data__copyright, &ColladaParserAutoGen15Private::_end__copyright, &ColladaParserAutoGen15Private::_preBegin__copyright, &ColladaParserAutoGen15Private::_preEnd__copyright, &ColladaParserAutoGen15Private::_freeAttributes__copyright);
functionMap[HASH_ELEMENT_SOURCE_DATA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source_data, &ColladaParserAutoGen15Private::_data__source_data, &ColladaParserAutoGen15Private::_end__source_data, &ColladaParserAutoGen15Private::_preBegin__source_data, &ColladaParserAutoGen15Private::_preEnd__source_data, &ColladaParserAutoGen15Private::_freeAttributes__source_data);
functionMap[HASH_ELEMENT_GEOGRAPHIC_LOCATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__geographic_location, &ColladaParserAutoGen15Private::_data__geographic_location, &ColladaParserAutoGen15Private::_end__geographic_location, &ColladaParserAutoGen15Private::_preBegin__geographic_location, &ColladaParserAutoGen15Private::_preEnd__geographic_location, &ColladaParserAutoGen15Private::_freeAttributes__geographic_location);
functionMap[HASH_ELEMENT_ALTITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__altitude, &ColladaParserAutoGen15Private::_data__altitude, &ColladaParserAutoGen15Private::_end__altitude, &ColladaParserAutoGen15Private::_preBegin__altitude, &ColladaParserAutoGen15Private::_preEnd__altitude, &ColladaParserAutoGen15Private::_freeAttributes__altitude);
functionMap[HASH_ELEMENT_LATITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__latitude, &ColladaParserAutoGen15Private::_data__latitude, &ColladaParserAutoGen15Private::_end__latitude, &ColladaParserAutoGen15Private::_preBegin__latitude, &ColladaParserAutoGen15Private::_preEnd__latitude, &ColladaParserAutoGen15Private::_freeAttributes__latitude);
functionMap[HASH_ELEMENT_LONGITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__longitude, &ColladaParserAutoGen15Private::_data__longitude, &ColladaParserAutoGen15Private::_end__longitude, &ColladaParserAutoGen15Private::_preBegin__longitude, &ColladaParserAutoGen15Private::_preEnd__longitude, &ColladaParserAutoGen15Private::_freeAttributes__longitude);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INSTANCE_CAMERA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_camera, &ColladaParserAutoGen15Private::_data__instance_camera, &ColladaParserAutoGen15Private::_end__instance_camera, &ColladaParserAutoGen15Private::_preBegin__instance_camera, &ColladaParserAutoGen15Private::_preEnd__instance_camera, &ColladaParserAutoGen15Private::_freeAttributes__instance_camera);
functionMap[HASH_ELEMENT_INSTANCE_CONTROLLER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_controller, &ColladaParserAutoGen15Private::_data__instance_controller, &ColladaParserAutoGen15Private::_end__instance_controller, &ColladaParserAutoGen15Private::_preBegin__instance_controller, &ColladaParserAutoGen15Private::_preEnd__instance_controller, &ColladaParserAutoGen15Private::_freeAttributes__instance_controller);
functionMap[HASH_ELEMENT_INSTANCE_GEOMETRY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_geometry, &ColladaParserAutoGen15Private::_data__instance_geometry, &ColladaParserAutoGen15Private::_end__instance_geometry, &ColladaParserAutoGen15Private::_preBegin__instance_geometry, &ColladaParserAutoGen15Private::_preEnd__instance_geometry, &ColladaParserAutoGen15Private::_freeAttributes__instance_geometry);
functionMap[HASH_ELEMENT_INSTANCE_LIGHT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_light, &ColladaParserAutoGen15Private::_data__instance_light, &ColladaParserAutoGen15Private::_end__instance_light, &ColladaParserAutoGen15Private::_preBegin__instance_light, &ColladaParserAutoGen15Private::_preEnd__instance_light, &ColladaParserAutoGen15Private::_freeAttributes__instance_light);
functionMap[HASH_ELEMENT_INSTANCE_NODE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_node, &ColladaParserAutoGen15Private::_data__instance_node, &ColladaParserAutoGen15Private::_end__instance_node, &ColladaParserAutoGen15Private::_preBegin__instance_node, &ColladaParserAutoGen15Private::_preEnd__instance_node, &ColladaParserAutoGen15Private::_freeAttributes__instance_node);
functionMap[HASH_ELEMENT_LOOKAT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__lookat, &ColladaParserAutoGen15Private::_data__lookat, &ColladaParserAutoGen15Private::_end__lookat, &ColladaParserAutoGen15Private::_preBegin__lookat, &ColladaParserAutoGen15Private::_preEnd__lookat, &ColladaParserAutoGen15Private::_freeAttributes__lookat);
functionMap[HASH_ELEMENT_MATRIX____MATRIX_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__matrix____matrix_type, &ColladaParserAutoGen15Private::_data__matrix____matrix_type, &ColladaParserAutoGen15Private::_end__matrix____matrix_type, &ColladaParserAutoGen15Private::_preBegin__matrix____matrix_type, &ColladaParserAutoGen15Private::_preEnd__matrix____matrix_type, &ColladaParserAutoGen15Private::_freeAttributes__matrix____matrix_type);
functionMap[HASH_ELEMENT_NODE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__node, &ColladaParserAutoGen15Private::_data__node, &ColladaParserAutoGen15Private::_end__node, &ColladaParserAutoGen15Private::_preBegin__node, &ColladaParserAutoGen15Private::_preEnd__node, &ColladaParserAutoGen15Private::_freeAttributes__node);
functionMap[HASH_ELEMENT_ROTATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__rotate, &ColladaParserAutoGen15Private::_data__rotate, &ColladaParserAutoGen15Private::_end__rotate, &ColladaParserAutoGen15Private::_preBegin__rotate, &ColladaParserAutoGen15Private::_preEnd__rotate, &ColladaParserAutoGen15Private::_freeAttributes__rotate);
functionMap[HASH_ELEMENT_SCALE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__scale, &ColladaParserAutoGen15Private::_data__scale, &ColladaParserAutoGen15Private::_end__scale, &ColladaParserAutoGen15Private::_preBegin__scale, &ColladaParserAutoGen15Private::_preEnd__scale, &ColladaParserAutoGen15Private::_freeAttributes__scale);
functionMap[HASH_ELEMENT_SKEW] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__skew, &ColladaParserAutoGen15Private::_data__skew, &ColladaParserAutoGen15Private::_end__skew, &ColladaParserAutoGen15Private::_preBegin__skew, &ColladaParserAutoGen15Private::_preEnd__skew, &ColladaParserAutoGen15Private::_freeAttributes__skew);
functionMap[HASH_ELEMENT_TRANSLATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__translate, &ColladaParserAutoGen15Private::_data__translate, &ColladaParserAutoGen15Private::_end__translate, &ColladaParserAutoGen15Private::_preBegin__translate, &ColladaParserAutoGen15Private::_preEnd__translate, &ColladaParserAutoGen15Private::_freeAttributes__translate);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_BIND_MATERIAL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bind_material, &ColladaParserAutoGen15Private::_data__bind_material, &ColladaParserAutoGen15Private::_end__bind_material, &ColladaParserAutoGen15Private::_preBegin__bind_material, &ColladaParserAutoGen15Private::_preEnd__bind_material, &ColladaParserAutoGen15Private::_freeAttributes__bind_material);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_SKELETON] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__skeleton, &ColladaParserAutoGen15Private::_data__skeleton, &ColladaParserAutoGen15Private::_end__skeleton, &ColladaParserAutoGen15Private::_preBegin__skeleton, &ColladaParserAutoGen15Private::_preEnd__skeleton, &ColladaParserAutoGen15Private::_freeAttributes__skeleton);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_PARAM____PARAM_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__param____param_type, &ColladaParserAutoGen15Private::_data__param____param_type, &ColladaParserAutoGen15Private::_end__param____param_type, &ColladaParserAutoGen15Private::_preBegin__param____param_type, &ColladaParserAutoGen15Private::_preEnd__param____param_type, &ColladaParserAutoGen15Private::_freeAttributes__param____param_type);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_BIND_MATERIAL_TYPE____TECHNIQUE_COMMON] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bind_material_type____technique_common, &ColladaParserAutoGen15Private::_data__bind_material_type____technique_common, &ColladaParserAutoGen15Private::_end__bind_material_type____technique_common, &ColladaParserAutoGen15Private::_preBegin__bind_material_type____technique_common, &ColladaParserAutoGen15Private::_preEnd__bind_material_type____technique_common, &ColladaParserAutoGen15Private::_freeAttributes__bind_material_type____technique_common);
functionMap[HASH_ELEMENT_INSTANCE_MATERIAL____INSTANCE_MATERIAL_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_material____instance_material_type, &ColladaParserAutoGen15Private::_data__instance_material____instance_material_type, &ColladaParserAutoGen15Private::_end__instance_material____instance_material_type, &ColladaParserAutoGen15Private::_preBegin__instance_material____instance_material_type, &ColladaParserAutoGen15Private::_preEnd__instance_material____instance_material_type, &ColladaParserAutoGen15Private::_freeAttributes__instance_material____instance_material_type);
functionMap[HASH_ELEMENT_INSTANCE_MATERIAL_TYPE____BIND] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_material_type____bind, &ColladaParserAutoGen15Private::_data__instance_material_type____bind, &ColladaParserAutoGen15Private::_end__instance_material_type____bind, &ColladaParserAutoGen15Private::_preBegin__instance_material_type____bind, &ColladaParserAutoGen15Private::_preEnd__instance_material_type____bind, &ColladaParserAutoGen15Private::_freeAttributes__instance_material_type____bind);
functionMap[HASH_ELEMENT_BIND_VERTEX_INPUT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bind_vertex_input, &ColladaParserAutoGen15Private::_data__bind_vertex_input, &ColladaParserAutoGen15Private::_end__bind_vertex_input, &ColladaParserAutoGen15Private::_preBegin__bind_vertex_input, &ColladaParserAutoGen15Private::_preEnd__bind_vertex_input, &ColladaParserAutoGen15Private::_freeAttributes__bind_vertex_input);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_BIND_MATERIAL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bind_material, &ColladaParserAutoGen15Private::_data__bind_material, &ColladaParserAutoGen15Private::_end__bind_material, &ColladaParserAutoGen15Private::_preBegin__bind_material, &ColladaParserAutoGen15Private::_preEnd__bind_material, &ColladaParserAutoGen15Private::_freeAttributes__bind_material);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_ROTATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__rotate, &ColladaParserAutoGen15Private::_data__rotate, &ColladaParserAutoGen15Private::_end__rotate, &ColladaParserAutoGen15Private::_preBegin__rotate, &ColladaParserAutoGen15Private::_preEnd__rotate, &ColladaParserAutoGen15Private::_freeAttributes__rotate);
functionMap[HASH_ELEMENT_SCALE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__scale, &ColladaParserAutoGen15Private::_data__scale, &ColladaParserAutoGen15Private::_end__scale, &ColladaParserAutoGen15Private::_preBegin__scale, &ColladaParserAutoGen15Private::_preEnd__scale, &ColladaParserAutoGen15Private::_freeAttributes__scale);
functionMap[HASH_ELEMENT_SKEW] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__skew, &ColladaParserAutoGen15Private::_data__skew, &ColladaParserAutoGen15Private::_end__skew, &ColladaParserAutoGen15Private::_preBegin__skew, &ColladaParserAutoGen15Private::_preEnd__skew, &ColladaParserAutoGen15Private::_freeAttributes__skew);
functionMap[HASH_ELEMENT_TRANSLATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__translate, &ColladaParserAutoGen15Private::_data__translate, &ColladaParserAutoGen15Private::_end__translate, &ColladaParserAutoGen15Private::_preBegin__translate, &ColladaParserAutoGen15Private::_preEnd__translate, &ColladaParserAutoGen15Private::_freeAttributes__translate);
mapFilled = true;
}
return functionMap;
}

//---------------------------------------------------------------------
const ColladaParserAutoGen15Private::ElementFunctionMap& FunctionMapFactory::createFunctionMap__library_physics_materials__allChildren() {
static ColladaParserAutoGen15Private::ElementFunctionMap functionMap;
static bool mapFilled = false;
if ( !mapFilled )
{
functionMap[HASH_ELEMENT_COLLADA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__COLLADA, &ColladaParserAutoGen15Private::_data__COLLADA, &ColladaParserAutoGen15Private::_end__COLLADA, &ColladaParserAutoGen15Private::_preBegin__COLLADA, &ColladaParserAutoGen15Private::_preEnd__COLLADA, &ColladaParserAutoGen15Private::_freeAttributes__COLLADA);
functionMap[HASH_ELEMENT_LIBRARY_PHYSICS_MATERIALS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_physics_materials, &ColladaParserAutoGen15Private::_data__library_physics_materials, &ColladaParserAutoGen15Private::_end__library_physics_materials, &ColladaParserAutoGen15Private::_preBegin__library_physics_materials, &ColladaParserAutoGen15Private::_preEnd__library_physics_materials, &ColladaParserAutoGen15Private::_freeAttributes__library_physics_materials);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_PHYSICS_MATERIAL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__physics_material, &ColladaParserAutoGen15Private::_data__physics_material, &ColladaParserAutoGen15Private::_end__physics_material, &ColladaParserAutoGen15Private::_preBegin__physics_material, &ColladaParserAutoGen15Private::_preEnd__physics_material, &ColladaParserAutoGen15Private::_freeAttributes__physics_material);
functionMap[HASH_ELEMENT_CONTRIBUTOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__contributor, &ColladaParserAutoGen15Private::_data__contributor, &ColladaParserAutoGen15Private::_end__contributor, &ColladaParserAutoGen15Private::_preBegin__contributor, &ColladaParserAutoGen15Private::_preEnd__contributor, &ColladaParserAutoGen15Private::_freeAttributes__contributor);
functionMap[HASH_ELEMENT_COVERAGE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__coverage, &ColladaParserAutoGen15Private::_data__coverage, &ColladaParserAutoGen15Private::_end__coverage, &ColladaParserAutoGen15Private::_preBegin__coverage, &ColladaParserAutoGen15Private::_preEnd__coverage, &ColladaParserAutoGen15Private::_freeAttributes__coverage);
functionMap[HASH_ELEMENT_CREATED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__created, &ColladaParserAutoGen15Private::_data__created, &ColladaParserAutoGen15Private::_end__created, &ColladaParserAutoGen15Private::_preBegin__created, &ColladaParserAutoGen15Private::_preEnd__created, &ColladaParserAutoGen15Private::_freeAttributes__created);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_AUTHOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author, &ColladaParserAutoGen15Private::_data__author, &ColladaParserAutoGen15Private::_end__author, &ColladaParserAutoGen15Private::_preBegin__author, &ColladaParserAutoGen15Private::_preEnd__author, &ColladaParserAutoGen15Private::_freeAttributes__author);
functionMap[HASH_ELEMENT_AUTHOR_EMAIL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_email, &ColladaParserAutoGen15Private::_data__author_email, &ColladaParserAutoGen15Private::_end__author_email, &ColladaParserAutoGen15Private::_preBegin__author_email, &ColladaParserAutoGen15Private::_preEnd__author_email, &ColladaParserAutoGen15Private::_freeAttributes__author_email);
functionMap[HASH_ELEMENT_AUTHOR_WEBSITE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_website, &ColladaParserAutoGen15Private::_data__author_website, &ColladaParserAutoGen15Private::_end__author_website, &ColladaParserAutoGen15Private::_preBegin__author_website, &ColladaParserAutoGen15Private::_preEnd__author_website, &ColladaParserAutoGen15Private::_freeAttributes__author_website);
functionMap[HASH_ELEMENT_AUTHORING_TOOL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__authoring_tool, &ColladaParserAutoGen15Private::_data__authoring_tool, &ColladaParserAutoGen15Private::_end__authoring_tool, &ColladaParserAutoGen15Private::_preBegin__authoring_tool, &ColladaParserAutoGen15Private::_preEnd__authoring_tool, &ColladaParserAutoGen15Private::_freeAttributes__authoring_tool);
functionMap[HASH_ELEMENT_COMMENTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__comments, &ColladaParserAutoGen15Private::_data__comments, &ColladaParserAutoGen15Private::_end__comments, &ColladaParserAutoGen15Private::_preBegin__comments, &ColladaParserAutoGen15Private::_preEnd__comments, &ColladaParserAutoGen15Private::_freeAttributes__comments);
functionMap[HASH_ELEMENT_COPYRIGHT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__copyright, &ColladaParserAutoGen15Private::_data__copyright, &ColladaParserAutoGen15Private::_end__copyright, &ColladaParserAutoGen15Private::_preBegin__copyright, &ColladaParserAutoGen15Private::_preEnd__copyright, &ColladaParserAutoGen15Private::_freeAttributes__copyright);
functionMap[HASH_ELEMENT_SOURCE_DATA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source_data, &ColladaParserAutoGen15Private::_data__source_data, &ColladaParserAutoGen15Private::_end__source_data, &ColladaParserAutoGen15Private::_preBegin__source_data, &ColladaParserAutoGen15Private::_preEnd__source_data, &ColladaParserAutoGen15Private::_freeAttributes__source_data);
functionMap[HASH_ELEMENT_GEOGRAPHIC_LOCATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__geographic_location, &ColladaParserAutoGen15Private::_data__geographic_location, &ColladaParserAutoGen15Private::_end__geographic_location, &ColladaParserAutoGen15Private::_preBegin__geographic_location, &ColladaParserAutoGen15Private::_preEnd__geographic_location, &ColladaParserAutoGen15Private::_freeAttributes__geographic_location);
functionMap[HASH_ELEMENT_ALTITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__altitude, &ColladaParserAutoGen15Private::_data__altitude, &ColladaParserAutoGen15Private::_end__altitude, &ColladaParserAutoGen15Private::_preBegin__altitude, &ColladaParserAutoGen15Private::_preEnd__altitude, &ColladaParserAutoGen15Private::_freeAttributes__altitude);
functionMap[HASH_ELEMENT_LATITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__latitude, &ColladaParserAutoGen15Private::_data__latitude, &ColladaParserAutoGen15Private::_end__latitude, &ColladaParserAutoGen15Private::_preBegin__latitude, &ColladaParserAutoGen15Private::_preEnd__latitude, &ColladaParserAutoGen15Private::_freeAttributes__latitude);
functionMap[HASH_ELEMENT_LONGITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__longitude, &ColladaParserAutoGen15Private::_data__longitude, &ColladaParserAutoGen15Private::_end__longitude, &ColladaParserAutoGen15Private::_preBegin__longitude, &ColladaParserAutoGen15Private::_preEnd__longitude, &ColladaParserAutoGen15Private::_freeAttributes__longitude);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_PHYSICS_MATERIAL_TYPE____TECHNIQUE_COMMON] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__physics_material_type____technique_common, &ColladaParserAutoGen15Private::_data__physics_material_type____technique_common, &ColladaParserAutoGen15Private::_end__physics_material_type____technique_common, &ColladaParserAutoGen15Private::_preBegin__physics_material_type____technique_common, &ColladaParserAutoGen15Private::_preEnd__physics_material_type____technique_common, &ColladaParserAutoGen15Private::_freeAttributes__physics_material_type____technique_common);
functionMap[HASH_ELEMENT_DYNAMIC_FRICTION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__dynamic_friction, &ColladaParserAutoGen15Private::_data__dynamic_friction, &ColladaParserAutoGen15Private::_end__dynamic_friction, &ColladaParserAutoGen15Private::_preBegin__dynamic_friction, &ColladaParserAutoGen15Private::_preEnd__dynamic_friction, &ColladaParserAutoGen15Private::_freeAttributes__dynamic_friction);
functionMap[HASH_ELEMENT_RESTITUTION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__restitution, &ColladaParserAutoGen15Private::_data__restitution, &ColladaParserAutoGen15Private::_end__restitution, &ColladaParserAutoGen15Private::_preBegin__restitution, &ColladaParserAutoGen15Private::_preEnd__restitution, &ColladaParserAutoGen15Private::_freeAttributes__restitution);
functionMap[HASH_ELEMENT_STATIC_FRICTION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__static_friction, &ColladaParserAutoGen15Private::_data__static_friction, &ColladaParserAutoGen15Private::_end__static_friction, &ColladaParserAutoGen15Private::_preBegin__static_friction, &ColladaParserAutoGen15Private::_preEnd__static_friction, &ColladaParserAutoGen15Private::_freeAttributes__static_friction);
mapFilled = true;
}
return functionMap;
}

//---------------------------------------------------------------------
const ColladaParserAutoGen15Private::ElementFunctionMap& FunctionMapFactory::createFunctionMap__library_physics_models__allChildren() {
static ColladaParserAutoGen15Private::ElementFunctionMap functionMap;
static bool mapFilled = false;
if ( !mapFilled )
{
functionMap[HASH_ELEMENT_COLLADA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__COLLADA, &ColladaParserAutoGen15Private::_data__COLLADA, &ColladaParserAutoGen15Private::_end__COLLADA, &ColladaParserAutoGen15Private::_preBegin__COLLADA, &ColladaParserAutoGen15Private::_preEnd__COLLADA, &ColladaParserAutoGen15Private::_freeAttributes__COLLADA);
functionMap[HASH_ELEMENT_LIBRARY_PHYSICS_MODELS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_physics_models, &ColladaParserAutoGen15Private::_data__library_physics_models, &ColladaParserAutoGen15Private::_end__library_physics_models, &ColladaParserAutoGen15Private::_preBegin__library_physics_models, &ColladaParserAutoGen15Private::_preEnd__library_physics_models, &ColladaParserAutoGen15Private::_freeAttributes__library_physics_models);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_PHYSICS_MODEL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__physics_model, &ColladaParserAutoGen15Private::_data__physics_model, &ColladaParserAutoGen15Private::_end__physics_model, &ColladaParserAutoGen15Private::_preBegin__physics_model, &ColladaParserAutoGen15Private::_preEnd__physics_model, &ColladaParserAutoGen15Private::_freeAttributes__physics_model);
functionMap[HASH_ELEMENT_CONTRIBUTOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__contributor, &ColladaParserAutoGen15Private::_data__contributor, &ColladaParserAutoGen15Private::_end__contributor, &ColladaParserAutoGen15Private::_preBegin__contributor, &ColladaParserAutoGen15Private::_preEnd__contributor, &ColladaParserAutoGen15Private::_freeAttributes__contributor);
functionMap[HASH_ELEMENT_COVERAGE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__coverage, &ColladaParserAutoGen15Private::_data__coverage, &ColladaParserAutoGen15Private::_end__coverage, &ColladaParserAutoGen15Private::_preBegin__coverage, &ColladaParserAutoGen15Private::_preEnd__coverage, &ColladaParserAutoGen15Private::_freeAttributes__coverage);
functionMap[HASH_ELEMENT_CREATED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__created, &ColladaParserAutoGen15Private::_data__created, &ColladaParserAutoGen15Private::_end__created, &ColladaParserAutoGen15Private::_preBegin__created, &ColladaParserAutoGen15Private::_preEnd__created, &ColladaParserAutoGen15Private::_freeAttributes__created);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_AUTHOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author, &ColladaParserAutoGen15Private::_data__author, &ColladaParserAutoGen15Private::_end__author, &ColladaParserAutoGen15Private::_preBegin__author, &ColladaParserAutoGen15Private::_preEnd__author, &ColladaParserAutoGen15Private::_freeAttributes__author);
functionMap[HASH_ELEMENT_AUTHOR_EMAIL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_email, &ColladaParserAutoGen15Private::_data__author_email, &ColladaParserAutoGen15Private::_end__author_email, &ColladaParserAutoGen15Private::_preBegin__author_email, &ColladaParserAutoGen15Private::_preEnd__author_email, &ColladaParserAutoGen15Private::_freeAttributes__author_email);
functionMap[HASH_ELEMENT_AUTHOR_WEBSITE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_website, &ColladaParserAutoGen15Private::_data__author_website, &ColladaParserAutoGen15Private::_end__author_website, &ColladaParserAutoGen15Private::_preBegin__author_website, &ColladaParserAutoGen15Private::_preEnd__author_website, &ColladaParserAutoGen15Private::_freeAttributes__author_website);
functionMap[HASH_ELEMENT_AUTHORING_TOOL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__authoring_tool, &ColladaParserAutoGen15Private::_data__authoring_tool, &ColladaParserAutoGen15Private::_end__authoring_tool, &ColladaParserAutoGen15Private::_preBegin__authoring_tool, &ColladaParserAutoGen15Private::_preEnd__authoring_tool, &ColladaParserAutoGen15Private::_freeAttributes__authoring_tool);
functionMap[HASH_ELEMENT_COMMENTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__comments, &ColladaParserAutoGen15Private::_data__comments, &ColladaParserAutoGen15Private::_end__comments, &ColladaParserAutoGen15Private::_preBegin__comments, &ColladaParserAutoGen15Private::_preEnd__comments, &ColladaParserAutoGen15Private::_freeAttributes__comments);
functionMap[HASH_ELEMENT_COPYRIGHT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__copyright, &ColladaParserAutoGen15Private::_data__copyright, &ColladaParserAutoGen15Private::_end__copyright, &ColladaParserAutoGen15Private::_preBegin__copyright, &ColladaParserAutoGen15Private::_preEnd__copyright, &ColladaParserAutoGen15Private::_freeAttributes__copyright);
functionMap[HASH_ELEMENT_SOURCE_DATA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source_data, &ColladaParserAutoGen15Private::_data__source_data, &ColladaParserAutoGen15Private::_end__source_data, &ColladaParserAutoGen15Private::_preBegin__source_data, &ColladaParserAutoGen15Private::_preEnd__source_data, &ColladaParserAutoGen15Private::_freeAttributes__source_data);
functionMap[HASH_ELEMENT_GEOGRAPHIC_LOCATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__geographic_location, &ColladaParserAutoGen15Private::_data__geographic_location, &ColladaParserAutoGen15Private::_end__geographic_location, &ColladaParserAutoGen15Private::_preBegin__geographic_location, &ColladaParserAutoGen15Private::_preEnd__geographic_location, &ColladaParserAutoGen15Private::_freeAttributes__geographic_location);
functionMap[HASH_ELEMENT_ALTITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__altitude, &ColladaParserAutoGen15Private::_data__altitude, &ColladaParserAutoGen15Private::_end__altitude, &ColladaParserAutoGen15Private::_preBegin__altitude, &ColladaParserAutoGen15Private::_preEnd__altitude, &ColladaParserAutoGen15Private::_freeAttributes__altitude);
functionMap[HASH_ELEMENT_LATITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__latitude, &ColladaParserAutoGen15Private::_data__latitude, &ColladaParserAutoGen15Private::_end__latitude, &ColladaParserAutoGen15Private::_preBegin__latitude, &ColladaParserAutoGen15Private::_preEnd__latitude, &ColladaParserAutoGen15Private::_freeAttributes__latitude);
functionMap[HASH_ELEMENT_LONGITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__longitude, &ColladaParserAutoGen15Private::_data__longitude, &ColladaParserAutoGen15Private::_end__longitude, &ColladaParserAutoGen15Private::_preBegin__longitude, &ColladaParserAutoGen15Private::_preEnd__longitude, &ColladaParserAutoGen15Private::_freeAttributes__longitude);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INSTANCE_PHYSICS_MODEL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_physics_model, &ColladaParserAutoGen15Private::_data__instance_physics_model, &ColladaParserAutoGen15Private::_end__instance_physics_model, &ColladaParserAutoGen15Private::_preBegin__instance_physics_model, &ColladaParserAutoGen15Private::_preEnd__instance_physics_model, &ColladaParserAutoGen15Private::_freeAttributes__instance_physics_model);
functionMap[HASH_ELEMENT_RIGID_BODY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__rigid_body, &ColladaParserAutoGen15Private::_data__rigid_body, &ColladaParserAutoGen15Private::_end__rigid_body, &ColladaParserAutoGen15Private::_preBegin__rigid_body, &ColladaParserAutoGen15Private::_preEnd__rigid_body, &ColladaParserAutoGen15Private::_freeAttributes__rigid_body);
functionMap[HASH_ELEMENT_RIGID_CONSTRAINT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__rigid_constraint, &ColladaParserAutoGen15Private::_data__rigid_constraint, &ColladaParserAutoGen15Private::_end__rigid_constraint, &ColladaParserAutoGen15Private::_preBegin__rigid_constraint, &ColladaParserAutoGen15Private::_preEnd__rigid_constraint, &ColladaParserAutoGen15Private::_freeAttributes__rigid_constraint);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INSTANCE_FORCE_FIELD] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_force_field, &ColladaParserAutoGen15Private::_data__instance_force_field, &ColladaParserAutoGen15Private::_end__instance_force_field, &ColladaParserAutoGen15Private::_preBegin__instance_force_field, &ColladaParserAutoGen15Private::_preEnd__instance_force_field, &ColladaParserAutoGen15Private::_freeAttributes__instance_force_field);
functionMap[HASH_ELEMENT_INSTANCE_RIGID_BODY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_rigid_body, &ColladaParserAutoGen15Private::_data__instance_rigid_body, &ColladaParserAutoGen15Private::_end__instance_rigid_body, &ColladaParserAutoGen15Private::_preBegin__instance_rigid_body, &ColladaParserAutoGen15Private::_preEnd__instance_rigid_body, &ColladaParserAutoGen15Private::_freeAttributes__instance_rigid_body);
functionMap[HASH_ELEMENT_INSTANCE_RIGID_CONSTRAINT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_rigid_constraint, &ColladaParserAutoGen15Private::_data__instance_rigid_constraint, &ColladaParserAutoGen15Private::_end__instance_rigid_constraint, &ColladaParserAutoGen15Private::_preBegin__instance_rigid_constraint, &ColladaParserAutoGen15Private::_preEnd__instance_rigid_constraint, &ColladaParserAutoGen15Private::_freeAttributes__instance_rigid_constraint);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_INSTANCE_RIGID_BODY_TYPE____TECHNIQUE_COMMON] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_rigid_body_type____technique_common, &ColladaParserAutoGen15Private::_data__instance_rigid_body_type____technique_common, &ColladaParserAutoGen15Private::_end__instance_rigid_body_type____technique_common, &ColladaParserAutoGen15Private::_preBegin__instance_rigid_body_type____technique_common, &ColladaParserAutoGen15Private::_preEnd__instance_rigid_body_type____technique_common, &ColladaParserAutoGen15Private::_freeAttributes__instance_rigid_body_type____technique_common);
functionMap[HASH_ELEMENT_ANGULAR_VELOCITY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__angular_velocity, &ColladaParserAutoGen15Private::_data__angular_velocity, &ColladaParserAutoGen15Private::_end__angular_velocity, &ColladaParserAutoGen15Private::_preBegin__angular_velocity, &ColladaParserAutoGen15Private::_preEnd__angular_velocity, &ColladaParserAutoGen15Private::_freeAttributes__angular_velocity);
functionMap[HASH_ELEMENT_INSTANCE_RIGID_BODY__TECHNIQUE_COMMON__DYNAMIC] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_rigid_body__technique_common__dynamic, &ColladaParserAutoGen15Private::_data__instance_rigid_body__technique_common__dynamic, &ColladaParserAutoGen15Private::_end__instance_rigid_body__technique_common__dynamic, &ColladaParserAutoGen15Private::_preBegin__instance_rigid_body__technique_common__dynamic, &ColladaParserAutoGen15Private::_preEnd__instance_rigid_body__technique_common__dynamic, &ColladaParserAutoGen15Private::_freeAttributes__instance_rigid_body__technique_common__dynamic);
functionMap[HASH_ELEMENT_INERTIA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__inertia, &ColladaParserAutoGen15Private::_data__inertia, &ColladaParserAutoGen15Private::_end__inertia, &ColladaParserAutoGen15Private::_preBegin__inertia, &ColladaParserAutoGen15Private::_preEnd__inertia, &ColladaParserAutoGen15Private::_freeAttributes__inertia);
functionMap[HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_physics_material, &ColladaParserAutoGen15Private::_data__instance_physics_material, &ColladaParserAutoGen15Private::_end__instance_physics_material, &ColladaParserAutoGen15Private::_preBegin__instance_physics_material, &ColladaParserAutoGen15Private::_preEnd__instance_physics_material, &ColladaParserAutoGen15Private::_freeAttributes__instance_physics_material);
functionMap[HASH_ELEMENT_MASS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__mass, &ColladaParserAutoGen15Private::_data__mass, &ColladaParserAutoGen15Private::_end__mass, &ColladaParserAutoGen15Private::_preBegin__mass, &ColladaParserAutoGen15Private::_preEnd__mass, &ColladaParserAutoGen15Private::_freeAttributes__mass);
functionMap[HASH_ELEMENT_INSTANCE_RIGID_BODY__TECHNIQUE_COMMON__MASS_FRAME] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_rigid_body__technique_common__mass_frame, &ColladaParserAutoGen15Private::_data__instance_rigid_body__technique_common__mass_frame, &ColladaParserAutoGen15Private::_end__instance_rigid_body__technique_common__mass_frame, &ColladaParserAutoGen15Private::_preBegin__instance_rigid_body__technique_common__mass_frame, &ColladaParserAutoGen15Private::_preEnd__instance_rigid_body__technique_common__mass_frame, &ColladaParserAutoGen15Private::_freeAttributes__instance_rigid_body__technique_common__mass_frame);
functionMap[HASH_ELEMENT_PHYSICS_MATERIAL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__physics_material, &ColladaParserAutoGen15Private::_data__physics_material, &ColladaParserAutoGen15Private::_end__physics_material, &ColladaParserAutoGen15Private::_preBegin__physics_material, &ColladaParserAutoGen15Private::_preEnd__physics_material, &ColladaParserAutoGen15Private::_freeAttributes__physics_material);
functionMap[HASH_ELEMENT_INSTANCE_RIGID_BODY__TECHNIQUE_COMMON__SHAPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_rigid_body__technique_common__shape, &ColladaParserAutoGen15Private::_data__instance_rigid_body__technique_common__shape, &ColladaParserAutoGen15Private::_end__instance_rigid_body__technique_common__shape, &ColladaParserAutoGen15Private::_preBegin__instance_rigid_body__technique_common__shape, &ColladaParserAutoGen15Private::_preEnd__instance_rigid_body__technique_common__shape, &ColladaParserAutoGen15Private::_freeAttributes__instance_rigid_body__technique_common__shape);
functionMap[HASH_ELEMENT_VELOCITY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__velocity, &ColladaParserAutoGen15Private::_data__velocity, &ColladaParserAutoGen15Private::_end__velocity, &ColladaParserAutoGen15Private::_preBegin__velocity, &ColladaParserAutoGen15Private::_preEnd__velocity, &ColladaParserAutoGen15Private::_freeAttributes__velocity);
functionMap[HASH_ELEMENT_ROTATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__rotate, &ColladaParserAutoGen15Private::_data__rotate, &ColladaParserAutoGen15Private::_end__rotate, &ColladaParserAutoGen15Private::_preBegin__rotate, &ColladaParserAutoGen15Private::_preEnd__rotate, &ColladaParserAutoGen15Private::_freeAttributes__rotate);
functionMap[HASH_ELEMENT_TRANSLATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__translate, &ColladaParserAutoGen15Private::_data__translate, &ColladaParserAutoGen15Private::_end__translate, &ColladaParserAutoGen15Private::_preBegin__translate, &ColladaParserAutoGen15Private::_preEnd__translate, &ColladaParserAutoGen15Private::_freeAttributes__translate);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_PHYSICS_MATERIAL_TYPE____TECHNIQUE_COMMON] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__physics_material_type____technique_common, &ColladaParserAutoGen15Private::_data__physics_material_type____technique_common, &ColladaParserAutoGen15Private::_end__physics_material_type____technique_common, &ColladaParserAutoGen15Private::_preBegin__physics_material_type____technique_common, &ColladaParserAutoGen15Private::_preEnd__physics_material_type____technique_common, &ColladaParserAutoGen15Private::_freeAttributes__physics_material_type____technique_common);
functionMap[HASH_ELEMENT_DYNAMIC_FRICTION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__dynamic_friction, &ColladaParserAutoGen15Private::_data__dynamic_friction, &ColladaParserAutoGen15Private::_end__dynamic_friction, &ColladaParserAutoGen15Private::_preBegin__dynamic_friction, &ColladaParserAutoGen15Private::_preEnd__dynamic_friction, &ColladaParserAutoGen15Private::_freeAttributes__dynamic_friction);
functionMap[HASH_ELEMENT_RESTITUTION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__restitution, &ColladaParserAutoGen15Private::_data__restitution, &ColladaParserAutoGen15Private::_end__restitution, &ColladaParserAutoGen15Private::_preBegin__restitution, &ColladaParserAutoGen15Private::_preEnd__restitution, &ColladaParserAutoGen15Private::_freeAttributes__restitution);
functionMap[HASH_ELEMENT_STATIC_FRICTION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__static_friction, &ColladaParserAutoGen15Private::_data__static_friction, &ColladaParserAutoGen15Private::_end__static_friction, &ColladaParserAutoGen15Private::_preBegin__static_friction, &ColladaParserAutoGen15Private::_preEnd__static_friction, &ColladaParserAutoGen15Private::_freeAttributes__static_friction);
functionMap[HASH_ELEMENT_BOX] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__box, &ColladaParserAutoGen15Private::_data__box, &ColladaParserAutoGen15Private::_end__box, &ColladaParserAutoGen15Private::_preBegin__box, &ColladaParserAutoGen15Private::_preEnd__box, &ColladaParserAutoGen15Private::_freeAttributes__box);
functionMap[HASH_ELEMENT_CAPSULE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__capsule, &ColladaParserAutoGen15Private::_data__capsule, &ColladaParserAutoGen15Private::_end__capsule, &ColladaParserAutoGen15Private::_preBegin__capsule, &ColladaParserAutoGen15Private::_preEnd__capsule, &ColladaParserAutoGen15Private::_freeAttributes__capsule);
functionMap[HASH_ELEMENT_CYLINDER____CYLINDER_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__cylinder____cylinder_type, &ColladaParserAutoGen15Private::_data__cylinder____cylinder_type, &ColladaParserAutoGen15Private::_end__cylinder____cylinder_type, &ColladaParserAutoGen15Private::_preBegin__cylinder____cylinder_type, &ColladaParserAutoGen15Private::_preEnd__cylinder____cylinder_type, &ColladaParserAutoGen15Private::_freeAttributes__cylinder____cylinder_type);
functionMap[HASH_ELEMENT_DENSITY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__density, &ColladaParserAutoGen15Private::_data__density, &ColladaParserAutoGen15Private::_end__density, &ColladaParserAutoGen15Private::_preBegin__density, &ColladaParserAutoGen15Private::_preEnd__density, &ColladaParserAutoGen15Private::_freeAttributes__density);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INSTANCE_RIGID_BODY__TECHNIQUE_COMMON__SHAPE__HOLLOW] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_rigid_body__technique_common__shape__hollow, &ColladaParserAutoGen15Private::_data__instance_rigid_body__technique_common__shape__hollow, &ColladaParserAutoGen15Private::_end__instance_rigid_body__technique_common__shape__hollow, &ColladaParserAutoGen15Private::_preBegin__instance_rigid_body__technique_common__shape__hollow, &ColladaParserAutoGen15Private::_preEnd__instance_rigid_body__technique_common__shape__hollow, &ColladaParserAutoGen15Private::_freeAttributes__instance_rigid_body__technique_common__shape__hollow);
functionMap[HASH_ELEMENT_INSTANCE_GEOMETRY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_geometry, &ColladaParserAutoGen15Private::_data__instance_geometry, &ColladaParserAutoGen15Private::_end__instance_geometry, &ColladaParserAutoGen15Private::_preBegin__instance_geometry, &ColladaParserAutoGen15Private::_preEnd__instance_geometry, &ColladaParserAutoGen15Private::_freeAttributes__instance_geometry);
functionMap[HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_physics_material, &ColladaParserAutoGen15Private::_data__instance_physics_material, &ColladaParserAutoGen15Private::_end__instance_physics_material, &ColladaParserAutoGen15Private::_preBegin__instance_physics_material, &ColladaParserAutoGen15Private::_preEnd__instance_physics_material, &ColladaParserAutoGen15Private::_freeAttributes__instance_physics_material);
functionMap[HASH_ELEMENT_MASS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__mass, &ColladaParserAutoGen15Private::_data__mass, &ColladaParserAutoGen15Private::_end__mass, &ColladaParserAutoGen15Private::_preBegin__mass, &ColladaParserAutoGen15Private::_preEnd__mass, &ColladaParserAutoGen15Private::_freeAttributes__mass);
functionMap[HASH_ELEMENT_PHYSICS_MATERIAL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__physics_material, &ColladaParserAutoGen15Private::_data__physics_material, &ColladaParserAutoGen15Private::_end__physics_material, &ColladaParserAutoGen15Private::_preBegin__physics_material, &ColladaParserAutoGen15Private::_preEnd__physics_material, &ColladaParserAutoGen15Private::_freeAttributes__physics_material);
functionMap[HASH_ELEMENT_PLANE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__plane, &ColladaParserAutoGen15Private::_data__plane, &ColladaParserAutoGen15Private::_end__plane, &ColladaParserAutoGen15Private::_preBegin__plane, &ColladaParserAutoGen15Private::_preEnd__plane, &ColladaParserAutoGen15Private::_freeAttributes__plane);
functionMap[HASH_ELEMENT_ROTATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__rotate, &ColladaParserAutoGen15Private::_data__rotate, &ColladaParserAutoGen15Private::_end__rotate, &ColladaParserAutoGen15Private::_preBegin__rotate, &ColladaParserAutoGen15Private::_preEnd__rotate, &ColladaParserAutoGen15Private::_freeAttributes__rotate);
functionMap[HASH_ELEMENT_SPHERE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sphere, &ColladaParserAutoGen15Private::_data__sphere, &ColladaParserAutoGen15Private::_end__sphere, &ColladaParserAutoGen15Private::_preBegin__sphere, &ColladaParserAutoGen15Private::_preEnd__sphere, &ColladaParserAutoGen15Private::_freeAttributes__sphere);
functionMap[HASH_ELEMENT_TRANSLATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__translate, &ColladaParserAutoGen15Private::_data__translate, &ColladaParserAutoGen15Private::_end__translate, &ColladaParserAutoGen15Private::_preBegin__translate, &ColladaParserAutoGen15Private::_preEnd__translate, &ColladaParserAutoGen15Private::_freeAttributes__translate);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_HALF_EXTENTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half_extents, &ColladaParserAutoGen15Private::_data__half_extents, &ColladaParserAutoGen15Private::_end__half_extents, &ColladaParserAutoGen15Private::_preBegin__half_extents, &ColladaParserAutoGen15Private::_preEnd__half_extents, &ColladaParserAutoGen15Private::_freeAttributes__half_extents);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_HEIGHT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__height, &ColladaParserAutoGen15Private::_data__height, &ColladaParserAutoGen15Private::_end__height, &ColladaParserAutoGen15Private::_preBegin__height, &ColladaParserAutoGen15Private::_preEnd__height, &ColladaParserAutoGen15Private::_freeAttributes__height);
functionMap[HASH_ELEMENT_RADIUS____FLOAT3_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__radius____float3_type, &ColladaParserAutoGen15Private::_data__radius____float3_type, &ColladaParserAutoGen15Private::_end__radius____float3_type, &ColladaParserAutoGen15Private::_preBegin__radius____float3_type, &ColladaParserAutoGen15Private::_preEnd__radius____float3_type, &ColladaParserAutoGen15Private::_freeAttributes__radius____float3_type);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_HEIGHT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__height, &ColladaParserAutoGen15Private::_data__height, &ColladaParserAutoGen15Private::_end__height, &ColladaParserAutoGen15Private::_preBegin__height, &ColladaParserAutoGen15Private::_preEnd__height, &ColladaParserAutoGen15Private::_freeAttributes__height);
functionMap[HASH_ELEMENT_RADIUS____FLOAT2_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__radius____float2_type, &ColladaParserAutoGen15Private::_data__radius____float2_type, &ColladaParserAutoGen15Private::_end__radius____float2_type, &ColladaParserAutoGen15Private::_preBegin__radius____float2_type, &ColladaParserAutoGen15Private::_preEnd__radius____float2_type, &ColladaParserAutoGen15Private::_freeAttributes__radius____float2_type);
functionMap[HASH_ELEMENT_BIND_MATERIAL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bind_material, &ColladaParserAutoGen15Private::_data__bind_material, &ColladaParserAutoGen15Private::_end__bind_material, &ColladaParserAutoGen15Private::_preBegin__bind_material, &ColladaParserAutoGen15Private::_preEnd__bind_material, &ColladaParserAutoGen15Private::_freeAttributes__bind_material);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_PARAM____PARAM_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__param____param_type, &ColladaParserAutoGen15Private::_data__param____param_type, &ColladaParserAutoGen15Private::_end__param____param_type, &ColladaParserAutoGen15Private::_preBegin__param____param_type, &ColladaParserAutoGen15Private::_preEnd__param____param_type, &ColladaParserAutoGen15Private::_freeAttributes__param____param_type);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_BIND_MATERIAL_TYPE____TECHNIQUE_COMMON] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bind_material_type____technique_common, &ColladaParserAutoGen15Private::_data__bind_material_type____technique_common, &ColladaParserAutoGen15Private::_end__bind_material_type____technique_common, &ColladaParserAutoGen15Private::_preBegin__bind_material_type____technique_common, &ColladaParserAutoGen15Private::_preEnd__bind_material_type____technique_common, &ColladaParserAutoGen15Private::_freeAttributes__bind_material_type____technique_common);
functionMap[HASH_ELEMENT_INSTANCE_MATERIAL____INSTANCE_MATERIAL_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_material____instance_material_type, &ColladaParserAutoGen15Private::_data__instance_material____instance_material_type, &ColladaParserAutoGen15Private::_end__instance_material____instance_material_type, &ColladaParserAutoGen15Private::_preBegin__instance_material____instance_material_type, &ColladaParserAutoGen15Private::_preEnd__instance_material____instance_material_type, &ColladaParserAutoGen15Private::_freeAttributes__instance_material____instance_material_type);
functionMap[HASH_ELEMENT_INSTANCE_MATERIAL_TYPE____BIND] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_material_type____bind, &ColladaParserAutoGen15Private::_data__instance_material_type____bind, &ColladaParserAutoGen15Private::_end__instance_material_type____bind, &ColladaParserAutoGen15Private::_preBegin__instance_material_type____bind, &ColladaParserAutoGen15Private::_preEnd__instance_material_type____bind, &ColladaParserAutoGen15Private::_freeAttributes__instance_material_type____bind);
functionMap[HASH_ELEMENT_BIND_VERTEX_INPUT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bind_vertex_input, &ColladaParserAutoGen15Private::_data__bind_vertex_input, &ColladaParserAutoGen15Private::_end__bind_vertex_input, &ColladaParserAutoGen15Private::_preBegin__bind_vertex_input, &ColladaParserAutoGen15Private::_preEnd__bind_vertex_input, &ColladaParserAutoGen15Private::_freeAttributes__bind_vertex_input);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_EQUATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__equation, &ColladaParserAutoGen15Private::_data__equation, &ColladaParserAutoGen15Private::_end__equation, &ColladaParserAutoGen15Private::_preBegin__equation, &ColladaParserAutoGen15Private::_preEnd__equation, &ColladaParserAutoGen15Private::_freeAttributes__equation);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_RADIUS____FLOAT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__radius____float_type, &ColladaParserAutoGen15Private::_data__radius____float_type, &ColladaParserAutoGen15Private::_end__radius____float_type, &ColladaParserAutoGen15Private::_preBegin__radius____float_type, &ColladaParserAutoGen15Private::_preEnd__radius____float_type, &ColladaParserAutoGen15Private::_freeAttributes__radius____float_type);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_RIGID_BODY_TYPE____TECHNIQUE_COMMON] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__rigid_body_type____technique_common, &ColladaParserAutoGen15Private::_data__rigid_body_type____technique_common, &ColladaParserAutoGen15Private::_end__rigid_body_type____technique_common, &ColladaParserAutoGen15Private::_preBegin__rigid_body_type____technique_common, &ColladaParserAutoGen15Private::_preEnd__rigid_body_type____technique_common, &ColladaParserAutoGen15Private::_freeAttributes__rigid_body_type____technique_common);
functionMap[HASH_ELEMENT_RIGID_BODY__TECHNIQUE_COMMON__DYNAMIC] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__rigid_body__technique_common__dynamic, &ColladaParserAutoGen15Private::_data__rigid_body__technique_common__dynamic, &ColladaParserAutoGen15Private::_end__rigid_body__technique_common__dynamic, &ColladaParserAutoGen15Private::_preBegin__rigid_body__technique_common__dynamic, &ColladaParserAutoGen15Private::_preEnd__rigid_body__technique_common__dynamic, &ColladaParserAutoGen15Private::_freeAttributes__rigid_body__technique_common__dynamic);
functionMap[HASH_ELEMENT_INERTIA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__inertia, &ColladaParserAutoGen15Private::_data__inertia, &ColladaParserAutoGen15Private::_end__inertia, &ColladaParserAutoGen15Private::_preBegin__inertia, &ColladaParserAutoGen15Private::_preEnd__inertia, &ColladaParserAutoGen15Private::_freeAttributes__inertia);
functionMap[HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_physics_material, &ColladaParserAutoGen15Private::_data__instance_physics_material, &ColladaParserAutoGen15Private::_end__instance_physics_material, &ColladaParserAutoGen15Private::_preBegin__instance_physics_material, &ColladaParserAutoGen15Private::_preEnd__instance_physics_material, &ColladaParserAutoGen15Private::_freeAttributes__instance_physics_material);
functionMap[HASH_ELEMENT_MASS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__mass, &ColladaParserAutoGen15Private::_data__mass, &ColladaParserAutoGen15Private::_end__mass, &ColladaParserAutoGen15Private::_preBegin__mass, &ColladaParserAutoGen15Private::_preEnd__mass, &ColladaParserAutoGen15Private::_freeAttributes__mass);
functionMap[HASH_ELEMENT_RIGID_BODY__TECHNIQUE_COMMON__MASS_FRAME] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__rigid_body__technique_common__mass_frame, &ColladaParserAutoGen15Private::_data__rigid_body__technique_common__mass_frame, &ColladaParserAutoGen15Private::_end__rigid_body__technique_common__mass_frame, &ColladaParserAutoGen15Private::_preBegin__rigid_body__technique_common__mass_frame, &ColladaParserAutoGen15Private::_preEnd__rigid_body__technique_common__mass_frame, &ColladaParserAutoGen15Private::_freeAttributes__rigid_body__technique_common__mass_frame);
functionMap[HASH_ELEMENT_PHYSICS_MATERIAL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__physics_material, &ColladaParserAutoGen15Private::_data__physics_material, &ColladaParserAutoGen15Private::_end__physics_material, &ColladaParserAutoGen15Private::_preBegin__physics_material, &ColladaParserAutoGen15Private::_preEnd__physics_material, &ColladaParserAutoGen15Private::_freeAttributes__physics_material);
functionMap[HASH_ELEMENT_RIGID_BODY__TECHNIQUE_COMMON__SHAPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__rigid_body__technique_common__shape, &ColladaParserAutoGen15Private::_data__rigid_body__technique_common__shape, &ColladaParserAutoGen15Private::_end__rigid_body__technique_common__shape, &ColladaParserAutoGen15Private::_preBegin__rigid_body__technique_common__shape, &ColladaParserAutoGen15Private::_preEnd__rigid_body__technique_common__shape, &ColladaParserAutoGen15Private::_freeAttributes__rigid_body__technique_common__shape);
functionMap[HASH_ELEMENT_ROTATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__rotate, &ColladaParserAutoGen15Private::_data__rotate, &ColladaParserAutoGen15Private::_end__rotate, &ColladaParserAutoGen15Private::_preBegin__rotate, &ColladaParserAutoGen15Private::_preEnd__rotate, &ColladaParserAutoGen15Private::_freeAttributes__rotate);
functionMap[HASH_ELEMENT_TRANSLATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__translate, &ColladaParserAutoGen15Private::_data__translate, &ColladaParserAutoGen15Private::_end__translate, &ColladaParserAutoGen15Private::_preBegin__translate, &ColladaParserAutoGen15Private::_preEnd__translate, &ColladaParserAutoGen15Private::_freeAttributes__translate);
functionMap[HASH_ELEMENT_BOX] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__box, &ColladaParserAutoGen15Private::_data__box, &ColladaParserAutoGen15Private::_end__box, &ColladaParserAutoGen15Private::_preBegin__box, &ColladaParserAutoGen15Private::_preEnd__box, &ColladaParserAutoGen15Private::_freeAttributes__box);
functionMap[HASH_ELEMENT_CAPSULE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__capsule, &ColladaParserAutoGen15Private::_data__capsule, &ColladaParserAutoGen15Private::_end__capsule, &ColladaParserAutoGen15Private::_preBegin__capsule, &ColladaParserAutoGen15Private::_preEnd__capsule, &ColladaParserAutoGen15Private::_freeAttributes__capsule);
functionMap[HASH_ELEMENT_CYLINDER____CYLINDER_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__cylinder____cylinder_type, &ColladaParserAutoGen15Private::_data__cylinder____cylinder_type, &ColladaParserAutoGen15Private::_end__cylinder____cylinder_type, &ColladaParserAutoGen15Private::_preBegin__cylinder____cylinder_type, &ColladaParserAutoGen15Private::_preEnd__cylinder____cylinder_type, &ColladaParserAutoGen15Private::_freeAttributes__cylinder____cylinder_type);
functionMap[HASH_ELEMENT_DENSITY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__density, &ColladaParserAutoGen15Private::_data__density, &ColladaParserAutoGen15Private::_end__density, &ColladaParserAutoGen15Private::_preBegin__density, &ColladaParserAutoGen15Private::_preEnd__density, &ColladaParserAutoGen15Private::_freeAttributes__density);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_RIGID_BODY__TECHNIQUE_COMMON__SHAPE__HOLLOW] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__rigid_body__technique_common__shape__hollow, &ColladaParserAutoGen15Private::_data__rigid_body__technique_common__shape__hollow, &ColladaParserAutoGen15Private::_end__rigid_body__technique_common__shape__hollow, &ColladaParserAutoGen15Private::_preBegin__rigid_body__technique_common__shape__hollow, &ColladaParserAutoGen15Private::_preEnd__rigid_body__technique_common__shape__hollow, &ColladaParserAutoGen15Private::_freeAttributes__rigid_body__technique_common__shape__hollow);
functionMap[HASH_ELEMENT_INSTANCE_GEOMETRY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_geometry, &ColladaParserAutoGen15Private::_data__instance_geometry, &ColladaParserAutoGen15Private::_end__instance_geometry, &ColladaParserAutoGen15Private::_preBegin__instance_geometry, &ColladaParserAutoGen15Private::_preEnd__instance_geometry, &ColladaParserAutoGen15Private::_freeAttributes__instance_geometry);
functionMap[HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_physics_material, &ColladaParserAutoGen15Private::_data__instance_physics_material, &ColladaParserAutoGen15Private::_end__instance_physics_material, &ColladaParserAutoGen15Private::_preBegin__instance_physics_material, &ColladaParserAutoGen15Private::_preEnd__instance_physics_material, &ColladaParserAutoGen15Private::_freeAttributes__instance_physics_material);
functionMap[HASH_ELEMENT_MASS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__mass, &ColladaParserAutoGen15Private::_data__mass, &ColladaParserAutoGen15Private::_end__mass, &ColladaParserAutoGen15Private::_preBegin__mass, &ColladaParserAutoGen15Private::_preEnd__mass, &ColladaParserAutoGen15Private::_freeAttributes__mass);
functionMap[HASH_ELEMENT_PHYSICS_MATERIAL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__physics_material, &ColladaParserAutoGen15Private::_data__physics_material, &ColladaParserAutoGen15Private::_end__physics_material, &ColladaParserAutoGen15Private::_preBegin__physics_material, &ColladaParserAutoGen15Private::_preEnd__physics_material, &ColladaParserAutoGen15Private::_freeAttributes__physics_material);
functionMap[HASH_ELEMENT_PLANE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__plane, &ColladaParserAutoGen15Private::_data__plane, &ColladaParserAutoGen15Private::_end__plane, &ColladaParserAutoGen15Private::_preBegin__plane, &ColladaParserAutoGen15Private::_preEnd__plane, &ColladaParserAutoGen15Private::_freeAttributes__plane);
functionMap[HASH_ELEMENT_ROTATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__rotate, &ColladaParserAutoGen15Private::_data__rotate, &ColladaParserAutoGen15Private::_end__rotate, &ColladaParserAutoGen15Private::_preBegin__rotate, &ColladaParserAutoGen15Private::_preEnd__rotate, &ColladaParserAutoGen15Private::_freeAttributes__rotate);
functionMap[HASH_ELEMENT_SPHERE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sphere, &ColladaParserAutoGen15Private::_data__sphere, &ColladaParserAutoGen15Private::_end__sphere, &ColladaParserAutoGen15Private::_preBegin__sphere, &ColladaParserAutoGen15Private::_preEnd__sphere, &ColladaParserAutoGen15Private::_freeAttributes__sphere);
functionMap[HASH_ELEMENT_TRANSLATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__translate, &ColladaParserAutoGen15Private::_data__translate, &ColladaParserAutoGen15Private::_end__translate, &ColladaParserAutoGen15Private::_preBegin__translate, &ColladaParserAutoGen15Private::_preEnd__translate, &ColladaParserAutoGen15Private::_freeAttributes__translate);
functionMap[HASH_ELEMENT_ATTACHMENT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__attachment, &ColladaParserAutoGen15Private::_data__attachment, &ColladaParserAutoGen15Private::_end__attachment, &ColladaParserAutoGen15Private::_preBegin__attachment, &ColladaParserAutoGen15Private::_preEnd__attachment, &ColladaParserAutoGen15Private::_freeAttributes__attachment);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_REF_ATTACHMENT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__ref_attachment, &ColladaParserAutoGen15Private::_data__ref_attachment, &ColladaParserAutoGen15Private::_end__ref_attachment, &ColladaParserAutoGen15Private::_preBegin__ref_attachment, &ColladaParserAutoGen15Private::_preEnd__ref_attachment, &ColladaParserAutoGen15Private::_freeAttributes__ref_attachment);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_RIGID_CONSTRAINT_TYPE____TECHNIQUE_COMMON] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__rigid_constraint_type____technique_common, &ColladaParserAutoGen15Private::_data__rigid_constraint_type____technique_common, &ColladaParserAutoGen15Private::_end__rigid_constraint_type____technique_common, &ColladaParserAutoGen15Private::_preBegin__rigid_constraint_type____technique_common, &ColladaParserAutoGen15Private::_preEnd__rigid_constraint_type____technique_common, &ColladaParserAutoGen15Private::_freeAttributes__rigid_constraint_type____technique_common);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_ROTATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__rotate, &ColladaParserAutoGen15Private::_data__rotate, &ColladaParserAutoGen15Private::_end__rotate, &ColladaParserAutoGen15Private::_preBegin__rotate, &ColladaParserAutoGen15Private::_preEnd__rotate, &ColladaParserAutoGen15Private::_freeAttributes__rotate);
functionMap[HASH_ELEMENT_TRANSLATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__translate, &ColladaParserAutoGen15Private::_data__translate, &ColladaParserAutoGen15Private::_end__translate, &ColladaParserAutoGen15Private::_preBegin__translate, &ColladaParserAutoGen15Private::_preEnd__translate, &ColladaParserAutoGen15Private::_freeAttributes__translate);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_ROTATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__rotate, &ColladaParserAutoGen15Private::_data__rotate, &ColladaParserAutoGen15Private::_end__rotate, &ColladaParserAutoGen15Private::_preBegin__rotate, &ColladaParserAutoGen15Private::_preEnd__rotate, &ColladaParserAutoGen15Private::_freeAttributes__rotate);
functionMap[HASH_ELEMENT_TRANSLATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__translate, &ColladaParserAutoGen15Private::_data__translate, &ColladaParserAutoGen15Private::_end__translate, &ColladaParserAutoGen15Private::_preBegin__translate, &ColladaParserAutoGen15Private::_preEnd__translate, &ColladaParserAutoGen15Private::_freeAttributes__translate);
functionMap[HASH_ELEMENT_ENABLED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__enabled, &ColladaParserAutoGen15Private::_data__enabled, &ColladaParserAutoGen15Private::_end__enabled, &ColladaParserAutoGen15Private::_preBegin__enabled, &ColladaParserAutoGen15Private::_preEnd__enabled, &ColladaParserAutoGen15Private::_freeAttributes__enabled);
functionMap[HASH_ELEMENT_INTERPENETRATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__interpenetrate, &ColladaParserAutoGen15Private::_data__interpenetrate, &ColladaParserAutoGen15Private::_end__interpenetrate, &ColladaParserAutoGen15Private::_preBegin__interpenetrate, &ColladaParserAutoGen15Private::_preEnd__interpenetrate, &ColladaParserAutoGen15Private::_freeAttributes__interpenetrate);
functionMap[HASH_ELEMENT_RIGID_CONSTRAINT__TECHNIQUE_COMMON__LIMITS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__rigid_constraint__technique_common__limits, &ColladaParserAutoGen15Private::_data__rigid_constraint__technique_common__limits, &ColladaParserAutoGen15Private::_end__rigid_constraint__technique_common__limits, &ColladaParserAutoGen15Private::_preBegin__rigid_constraint__technique_common__limits, &ColladaParserAutoGen15Private::_preEnd__rigid_constraint__technique_common__limits, &ColladaParserAutoGen15Private::_freeAttributes__rigid_constraint__technique_common__limits);
functionMap[HASH_ELEMENT_SPRING] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__spring, &ColladaParserAutoGen15Private::_data__spring, &ColladaParserAutoGen15Private::_end__spring, &ColladaParserAutoGen15Private::_preBegin__spring, &ColladaParserAutoGen15Private::_preEnd__spring, &ColladaParserAutoGen15Private::_freeAttributes__spring);
functionMap[HASH_ELEMENT_RIGID_CONSTRAINT__TECHNIQUE_COMMON__LIMITS__LINEAR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__rigid_constraint__technique_common__limits__linear, &ColladaParserAutoGen15Private::_data__rigid_constraint__technique_common__limits__linear, &ColladaParserAutoGen15Private::_end__rigid_constraint__technique_common__limits__linear, &ColladaParserAutoGen15Private::_preBegin__rigid_constraint__technique_common__limits__linear, &ColladaParserAutoGen15Private::_preEnd__rigid_constraint__technique_common__limits__linear, &ColladaParserAutoGen15Private::_freeAttributes__rigid_constraint__technique_common__limits__linear);
functionMap[HASH_ELEMENT_SWING_CONE_AND_TWIST] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__swing_cone_and_twist, &ColladaParserAutoGen15Private::_data__swing_cone_and_twist, &ColladaParserAutoGen15Private::_end__swing_cone_and_twist, &ColladaParserAutoGen15Private::_preBegin__swing_cone_and_twist, &ColladaParserAutoGen15Private::_preEnd__swing_cone_and_twist, &ColladaParserAutoGen15Private::_freeAttributes__swing_cone_and_twist);
functionMap[HASH_ELEMENT_MAX____TARGETABLE_FLOAT3_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__max____targetable_float3_type, &ColladaParserAutoGen15Private::_data__max____targetable_float3_type, &ColladaParserAutoGen15Private::_end__max____targetable_float3_type, &ColladaParserAutoGen15Private::_preBegin__max____targetable_float3_type, &ColladaParserAutoGen15Private::_preEnd__max____targetable_float3_type, &ColladaParserAutoGen15Private::_freeAttributes__max____targetable_float3_type);
functionMap[HASH_ELEMENT_MIN____TARGETABLE_FLOAT3_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__min____targetable_float3_type, &ColladaParserAutoGen15Private::_data__min____targetable_float3_type, &ColladaParserAutoGen15Private::_end__min____targetable_float3_type, &ColladaParserAutoGen15Private::_preBegin__min____targetable_float3_type, &ColladaParserAutoGen15Private::_preEnd__min____targetable_float3_type, &ColladaParserAutoGen15Private::_freeAttributes__min____targetable_float3_type);
functionMap[HASH_ELEMENT_MAX____TARGETABLE_FLOAT3_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__max____targetable_float3_type, &ColladaParserAutoGen15Private::_data__max____targetable_float3_type, &ColladaParserAutoGen15Private::_end__max____targetable_float3_type, &ColladaParserAutoGen15Private::_preBegin__max____targetable_float3_type, &ColladaParserAutoGen15Private::_preEnd__max____targetable_float3_type, &ColladaParserAutoGen15Private::_freeAttributes__max____targetable_float3_type);
functionMap[HASH_ELEMENT_MIN____TARGETABLE_FLOAT3_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__min____targetable_float3_type, &ColladaParserAutoGen15Private::_data__min____targetable_float3_type, &ColladaParserAutoGen15Private::_end__min____targetable_float3_type, &ColladaParserAutoGen15Private::_preBegin__min____targetable_float3_type, &ColladaParserAutoGen15Private::_preEnd__min____targetable_float3_type, &ColladaParserAutoGen15Private::_freeAttributes__min____targetable_float3_type);
functionMap[HASH_ELEMENT_ANGULAR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__angular, &ColladaParserAutoGen15Private::_data__angular, &ColladaParserAutoGen15Private::_end__angular, &ColladaParserAutoGen15Private::_preBegin__angular, &ColladaParserAutoGen15Private::_preEnd__angular, &ColladaParserAutoGen15Private::_freeAttributes__angular);
functionMap[HASH_ELEMENT_SPRING__LINEAR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__spring__linear, &ColladaParserAutoGen15Private::_data__spring__linear, &ColladaParserAutoGen15Private::_end__spring__linear, &ColladaParserAutoGen15Private::_preBegin__spring__linear, &ColladaParserAutoGen15Private::_preEnd__spring__linear, &ColladaParserAutoGen15Private::_freeAttributes__spring__linear);
functionMap[HASH_ELEMENT_DAMPING] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__damping, &ColladaParserAutoGen15Private::_data__damping, &ColladaParserAutoGen15Private::_end__damping, &ColladaParserAutoGen15Private::_preBegin__damping, &ColladaParserAutoGen15Private::_preEnd__damping, &ColladaParserAutoGen15Private::_freeAttributes__damping);
functionMap[HASH_ELEMENT_STIFFNESS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__stiffness, &ColladaParserAutoGen15Private::_data__stiffness, &ColladaParserAutoGen15Private::_end__stiffness, &ColladaParserAutoGen15Private::_preBegin__stiffness, &ColladaParserAutoGen15Private::_preEnd__stiffness, &ColladaParserAutoGen15Private::_freeAttributes__stiffness);
functionMap[HASH_ELEMENT_TARGET_VALUE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__target_value, &ColladaParserAutoGen15Private::_data__target_value, &ColladaParserAutoGen15Private::_end__target_value, &ColladaParserAutoGen15Private::_preBegin__target_value, &ColladaParserAutoGen15Private::_preEnd__target_value, &ColladaParserAutoGen15Private::_freeAttributes__target_value);
functionMap[HASH_ELEMENT_DAMPING] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__damping, &ColladaParserAutoGen15Private::_data__damping, &ColladaParserAutoGen15Private::_end__damping, &ColladaParserAutoGen15Private::_preBegin__damping, &ColladaParserAutoGen15Private::_preEnd__damping, &ColladaParserAutoGen15Private::_freeAttributes__damping);
functionMap[HASH_ELEMENT_STIFFNESS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__stiffness, &ColladaParserAutoGen15Private::_data__stiffness, &ColladaParserAutoGen15Private::_end__stiffness, &ColladaParserAutoGen15Private::_preBegin__stiffness, &ColladaParserAutoGen15Private::_preEnd__stiffness, &ColladaParserAutoGen15Private::_freeAttributes__stiffness);
functionMap[HASH_ELEMENT_TARGET_VALUE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__target_value, &ColladaParserAutoGen15Private::_data__target_value, &ColladaParserAutoGen15Private::_end__target_value, &ColladaParserAutoGen15Private::_preBegin__target_value, &ColladaParserAutoGen15Private::_preEnd__target_value, &ColladaParserAutoGen15Private::_freeAttributes__target_value);
mapFilled = true;
}
return functionMap;
}

//---------------------------------------------------------------------
const ColladaParserAutoGen15Private::ElementFunctionMap& FunctionMapFactory::createFunctionMap__library_physics_scenes__allChildren() {
static ColladaParserAutoGen15Private::ElementFunctionMap functionMap;
static bool mapFilled = false;
if ( !mapFilled )
{
functionMap[HASH_ELEMENT_COLLADA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__COLLADA, &ColladaParserAutoGen15Private::_data__COLLADA, &ColladaParserAutoGen15Private::_end__COLLADA, &ColladaParserAutoGen15Private::_preBegin__COLLADA, &ColladaParserAutoGen15Private::_preEnd__COLLADA, &ColladaParserAutoGen15Private::_freeAttributes__COLLADA);
functionMap[HASH_ELEMENT_LIBRARY_PHYSICS_SCENES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_physics_scenes, &ColladaParserAutoGen15Private::_data__library_physics_scenes, &ColladaParserAutoGen15Private::_end__library_physics_scenes, &ColladaParserAutoGen15Private::_preBegin__library_physics_scenes, &ColladaParserAutoGen15Private::_preEnd__library_physics_scenes, &ColladaParserAutoGen15Private::_freeAttributes__library_physics_scenes);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_PHYSICS_SCENE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__physics_scene, &ColladaParserAutoGen15Private::_data__physics_scene, &ColladaParserAutoGen15Private::_end__physics_scene, &ColladaParserAutoGen15Private::_preBegin__physics_scene, &ColladaParserAutoGen15Private::_preEnd__physics_scene, &ColladaParserAutoGen15Private::_freeAttributes__physics_scene);
functionMap[HASH_ELEMENT_CONTRIBUTOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__contributor, &ColladaParserAutoGen15Private::_data__contributor, &ColladaParserAutoGen15Private::_end__contributor, &ColladaParserAutoGen15Private::_preBegin__contributor, &ColladaParserAutoGen15Private::_preEnd__contributor, &ColladaParserAutoGen15Private::_freeAttributes__contributor);
functionMap[HASH_ELEMENT_COVERAGE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__coverage, &ColladaParserAutoGen15Private::_data__coverage, &ColladaParserAutoGen15Private::_end__coverage, &ColladaParserAutoGen15Private::_preBegin__coverage, &ColladaParserAutoGen15Private::_preEnd__coverage, &ColladaParserAutoGen15Private::_freeAttributes__coverage);
functionMap[HASH_ELEMENT_CREATED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__created, &ColladaParserAutoGen15Private::_data__created, &ColladaParserAutoGen15Private::_end__created, &ColladaParserAutoGen15Private::_preBegin__created, &ColladaParserAutoGen15Private::_preEnd__created, &ColladaParserAutoGen15Private::_freeAttributes__created);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_AUTHOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author, &ColladaParserAutoGen15Private::_data__author, &ColladaParserAutoGen15Private::_end__author, &ColladaParserAutoGen15Private::_preBegin__author, &ColladaParserAutoGen15Private::_preEnd__author, &ColladaParserAutoGen15Private::_freeAttributes__author);
functionMap[HASH_ELEMENT_AUTHOR_EMAIL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_email, &ColladaParserAutoGen15Private::_data__author_email, &ColladaParserAutoGen15Private::_end__author_email, &ColladaParserAutoGen15Private::_preBegin__author_email, &ColladaParserAutoGen15Private::_preEnd__author_email, &ColladaParserAutoGen15Private::_freeAttributes__author_email);
functionMap[HASH_ELEMENT_AUTHOR_WEBSITE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_website, &ColladaParserAutoGen15Private::_data__author_website, &ColladaParserAutoGen15Private::_end__author_website, &ColladaParserAutoGen15Private::_preBegin__author_website, &ColladaParserAutoGen15Private::_preEnd__author_website, &ColladaParserAutoGen15Private::_freeAttributes__author_website);
functionMap[HASH_ELEMENT_AUTHORING_TOOL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__authoring_tool, &ColladaParserAutoGen15Private::_data__authoring_tool, &ColladaParserAutoGen15Private::_end__authoring_tool, &ColladaParserAutoGen15Private::_preBegin__authoring_tool, &ColladaParserAutoGen15Private::_preEnd__authoring_tool, &ColladaParserAutoGen15Private::_freeAttributes__authoring_tool);
functionMap[HASH_ELEMENT_COMMENTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__comments, &ColladaParserAutoGen15Private::_data__comments, &ColladaParserAutoGen15Private::_end__comments, &ColladaParserAutoGen15Private::_preBegin__comments, &ColladaParserAutoGen15Private::_preEnd__comments, &ColladaParserAutoGen15Private::_freeAttributes__comments);
functionMap[HASH_ELEMENT_COPYRIGHT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__copyright, &ColladaParserAutoGen15Private::_data__copyright, &ColladaParserAutoGen15Private::_end__copyright, &ColladaParserAutoGen15Private::_preBegin__copyright, &ColladaParserAutoGen15Private::_preEnd__copyright, &ColladaParserAutoGen15Private::_freeAttributes__copyright);
functionMap[HASH_ELEMENT_SOURCE_DATA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source_data, &ColladaParserAutoGen15Private::_data__source_data, &ColladaParserAutoGen15Private::_end__source_data, &ColladaParserAutoGen15Private::_preBegin__source_data, &ColladaParserAutoGen15Private::_preEnd__source_data, &ColladaParserAutoGen15Private::_freeAttributes__source_data);
functionMap[HASH_ELEMENT_GEOGRAPHIC_LOCATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__geographic_location, &ColladaParserAutoGen15Private::_data__geographic_location, &ColladaParserAutoGen15Private::_end__geographic_location, &ColladaParserAutoGen15Private::_preBegin__geographic_location, &ColladaParserAutoGen15Private::_preEnd__geographic_location, &ColladaParserAutoGen15Private::_freeAttributes__geographic_location);
functionMap[HASH_ELEMENT_ALTITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__altitude, &ColladaParserAutoGen15Private::_data__altitude, &ColladaParserAutoGen15Private::_end__altitude, &ColladaParserAutoGen15Private::_preBegin__altitude, &ColladaParserAutoGen15Private::_preEnd__altitude, &ColladaParserAutoGen15Private::_freeAttributes__altitude);
functionMap[HASH_ELEMENT_LATITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__latitude, &ColladaParserAutoGen15Private::_data__latitude, &ColladaParserAutoGen15Private::_end__latitude, &ColladaParserAutoGen15Private::_preBegin__latitude, &ColladaParserAutoGen15Private::_preEnd__latitude, &ColladaParserAutoGen15Private::_freeAttributes__latitude);
functionMap[HASH_ELEMENT_LONGITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__longitude, &ColladaParserAutoGen15Private::_data__longitude, &ColladaParserAutoGen15Private::_end__longitude, &ColladaParserAutoGen15Private::_preBegin__longitude, &ColladaParserAutoGen15Private::_preEnd__longitude, &ColladaParserAutoGen15Private::_freeAttributes__longitude);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INSTANCE_FORCE_FIELD] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_force_field, &ColladaParserAutoGen15Private::_data__instance_force_field, &ColladaParserAutoGen15Private::_end__instance_force_field, &ColladaParserAutoGen15Private::_preBegin__instance_force_field, &ColladaParserAutoGen15Private::_preEnd__instance_force_field, &ColladaParserAutoGen15Private::_freeAttributes__instance_force_field);
functionMap[HASH_ELEMENT_INSTANCE_PHYSICS_MODEL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_physics_model, &ColladaParserAutoGen15Private::_data__instance_physics_model, &ColladaParserAutoGen15Private::_end__instance_physics_model, &ColladaParserAutoGen15Private::_preBegin__instance_physics_model, &ColladaParserAutoGen15Private::_preEnd__instance_physics_model, &ColladaParserAutoGen15Private::_freeAttributes__instance_physics_model);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_PHYSICS_SCENE_TYPE____TECHNIQUE_COMMON] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__physics_scene_type____technique_common, &ColladaParserAutoGen15Private::_data__physics_scene_type____technique_common, &ColladaParserAutoGen15Private::_end__physics_scene_type____technique_common, &ColladaParserAutoGen15Private::_preBegin__physics_scene_type____technique_common, &ColladaParserAutoGen15Private::_preEnd__physics_scene_type____technique_common, &ColladaParserAutoGen15Private::_freeAttributes__physics_scene_type____technique_common);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INSTANCE_FORCE_FIELD] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_force_field, &ColladaParserAutoGen15Private::_data__instance_force_field, &ColladaParserAutoGen15Private::_end__instance_force_field, &ColladaParserAutoGen15Private::_preBegin__instance_force_field, &ColladaParserAutoGen15Private::_preEnd__instance_force_field, &ColladaParserAutoGen15Private::_freeAttributes__instance_force_field);
functionMap[HASH_ELEMENT_INSTANCE_RIGID_BODY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_rigid_body, &ColladaParserAutoGen15Private::_data__instance_rigid_body, &ColladaParserAutoGen15Private::_end__instance_rigid_body, &ColladaParserAutoGen15Private::_preBegin__instance_rigid_body, &ColladaParserAutoGen15Private::_preEnd__instance_rigid_body, &ColladaParserAutoGen15Private::_freeAttributes__instance_rigid_body);
functionMap[HASH_ELEMENT_INSTANCE_RIGID_CONSTRAINT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_rigid_constraint, &ColladaParserAutoGen15Private::_data__instance_rigid_constraint, &ColladaParserAutoGen15Private::_end__instance_rigid_constraint, &ColladaParserAutoGen15Private::_preBegin__instance_rigid_constraint, &ColladaParserAutoGen15Private::_preEnd__instance_rigid_constraint, &ColladaParserAutoGen15Private::_freeAttributes__instance_rigid_constraint);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_INSTANCE_RIGID_BODY_TYPE____TECHNIQUE_COMMON] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_rigid_body_type____technique_common, &ColladaParserAutoGen15Private::_data__instance_rigid_body_type____technique_common, &ColladaParserAutoGen15Private::_end__instance_rigid_body_type____technique_common, &ColladaParserAutoGen15Private::_preBegin__instance_rigid_body_type____technique_common, &ColladaParserAutoGen15Private::_preEnd__instance_rigid_body_type____technique_common, &ColladaParserAutoGen15Private::_freeAttributes__instance_rigid_body_type____technique_common);
functionMap[HASH_ELEMENT_ANGULAR_VELOCITY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__angular_velocity, &ColladaParserAutoGen15Private::_data__angular_velocity, &ColladaParserAutoGen15Private::_end__angular_velocity, &ColladaParserAutoGen15Private::_preBegin__angular_velocity, &ColladaParserAutoGen15Private::_preEnd__angular_velocity, &ColladaParserAutoGen15Private::_freeAttributes__angular_velocity);
functionMap[HASH_ELEMENT_INSTANCE_RIGID_BODY__TECHNIQUE_COMMON__DYNAMIC] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_rigid_body__technique_common__dynamic, &ColladaParserAutoGen15Private::_data__instance_rigid_body__technique_common__dynamic, &ColladaParserAutoGen15Private::_end__instance_rigid_body__technique_common__dynamic, &ColladaParserAutoGen15Private::_preBegin__instance_rigid_body__technique_common__dynamic, &ColladaParserAutoGen15Private::_preEnd__instance_rigid_body__technique_common__dynamic, &ColladaParserAutoGen15Private::_freeAttributes__instance_rigid_body__technique_common__dynamic);
functionMap[HASH_ELEMENT_INERTIA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__inertia, &ColladaParserAutoGen15Private::_data__inertia, &ColladaParserAutoGen15Private::_end__inertia, &ColladaParserAutoGen15Private::_preBegin__inertia, &ColladaParserAutoGen15Private::_preEnd__inertia, &ColladaParserAutoGen15Private::_freeAttributes__inertia);
functionMap[HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_physics_material, &ColladaParserAutoGen15Private::_data__instance_physics_material, &ColladaParserAutoGen15Private::_end__instance_physics_material, &ColladaParserAutoGen15Private::_preBegin__instance_physics_material, &ColladaParserAutoGen15Private::_preEnd__instance_physics_material, &ColladaParserAutoGen15Private::_freeAttributes__instance_physics_material);
functionMap[HASH_ELEMENT_MASS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__mass, &ColladaParserAutoGen15Private::_data__mass, &ColladaParserAutoGen15Private::_end__mass, &ColladaParserAutoGen15Private::_preBegin__mass, &ColladaParserAutoGen15Private::_preEnd__mass, &ColladaParserAutoGen15Private::_freeAttributes__mass);
functionMap[HASH_ELEMENT_INSTANCE_RIGID_BODY__TECHNIQUE_COMMON__MASS_FRAME] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_rigid_body__technique_common__mass_frame, &ColladaParserAutoGen15Private::_data__instance_rigid_body__technique_common__mass_frame, &ColladaParserAutoGen15Private::_end__instance_rigid_body__technique_common__mass_frame, &ColladaParserAutoGen15Private::_preBegin__instance_rigid_body__technique_common__mass_frame, &ColladaParserAutoGen15Private::_preEnd__instance_rigid_body__technique_common__mass_frame, &ColladaParserAutoGen15Private::_freeAttributes__instance_rigid_body__technique_common__mass_frame);
functionMap[HASH_ELEMENT_PHYSICS_MATERIAL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__physics_material, &ColladaParserAutoGen15Private::_data__physics_material, &ColladaParserAutoGen15Private::_end__physics_material, &ColladaParserAutoGen15Private::_preBegin__physics_material, &ColladaParserAutoGen15Private::_preEnd__physics_material, &ColladaParserAutoGen15Private::_freeAttributes__physics_material);
functionMap[HASH_ELEMENT_INSTANCE_RIGID_BODY__TECHNIQUE_COMMON__SHAPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_rigid_body__technique_common__shape, &ColladaParserAutoGen15Private::_data__instance_rigid_body__technique_common__shape, &ColladaParserAutoGen15Private::_end__instance_rigid_body__technique_common__shape, &ColladaParserAutoGen15Private::_preBegin__instance_rigid_body__technique_common__shape, &ColladaParserAutoGen15Private::_preEnd__instance_rigid_body__technique_common__shape, &ColladaParserAutoGen15Private::_freeAttributes__instance_rigid_body__technique_common__shape);
functionMap[HASH_ELEMENT_VELOCITY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__velocity, &ColladaParserAutoGen15Private::_data__velocity, &ColladaParserAutoGen15Private::_end__velocity, &ColladaParserAutoGen15Private::_preBegin__velocity, &ColladaParserAutoGen15Private::_preEnd__velocity, &ColladaParserAutoGen15Private::_freeAttributes__velocity);
functionMap[HASH_ELEMENT_ROTATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__rotate, &ColladaParserAutoGen15Private::_data__rotate, &ColladaParserAutoGen15Private::_end__rotate, &ColladaParserAutoGen15Private::_preBegin__rotate, &ColladaParserAutoGen15Private::_preEnd__rotate, &ColladaParserAutoGen15Private::_freeAttributes__rotate);
functionMap[HASH_ELEMENT_TRANSLATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__translate, &ColladaParserAutoGen15Private::_data__translate, &ColladaParserAutoGen15Private::_end__translate, &ColladaParserAutoGen15Private::_preBegin__translate, &ColladaParserAutoGen15Private::_preEnd__translate, &ColladaParserAutoGen15Private::_freeAttributes__translate);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_PHYSICS_MATERIAL_TYPE____TECHNIQUE_COMMON] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__physics_material_type____technique_common, &ColladaParserAutoGen15Private::_data__physics_material_type____technique_common, &ColladaParserAutoGen15Private::_end__physics_material_type____technique_common, &ColladaParserAutoGen15Private::_preBegin__physics_material_type____technique_common, &ColladaParserAutoGen15Private::_preEnd__physics_material_type____technique_common, &ColladaParserAutoGen15Private::_freeAttributes__physics_material_type____technique_common);
functionMap[HASH_ELEMENT_DYNAMIC_FRICTION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__dynamic_friction, &ColladaParserAutoGen15Private::_data__dynamic_friction, &ColladaParserAutoGen15Private::_end__dynamic_friction, &ColladaParserAutoGen15Private::_preBegin__dynamic_friction, &ColladaParserAutoGen15Private::_preEnd__dynamic_friction, &ColladaParserAutoGen15Private::_freeAttributes__dynamic_friction);
functionMap[HASH_ELEMENT_RESTITUTION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__restitution, &ColladaParserAutoGen15Private::_data__restitution, &ColladaParserAutoGen15Private::_end__restitution, &ColladaParserAutoGen15Private::_preBegin__restitution, &ColladaParserAutoGen15Private::_preEnd__restitution, &ColladaParserAutoGen15Private::_freeAttributes__restitution);
functionMap[HASH_ELEMENT_STATIC_FRICTION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__static_friction, &ColladaParserAutoGen15Private::_data__static_friction, &ColladaParserAutoGen15Private::_end__static_friction, &ColladaParserAutoGen15Private::_preBegin__static_friction, &ColladaParserAutoGen15Private::_preEnd__static_friction, &ColladaParserAutoGen15Private::_freeAttributes__static_friction);
functionMap[HASH_ELEMENT_BOX] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__box, &ColladaParserAutoGen15Private::_data__box, &ColladaParserAutoGen15Private::_end__box, &ColladaParserAutoGen15Private::_preBegin__box, &ColladaParserAutoGen15Private::_preEnd__box, &ColladaParserAutoGen15Private::_freeAttributes__box);
functionMap[HASH_ELEMENT_CAPSULE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__capsule, &ColladaParserAutoGen15Private::_data__capsule, &ColladaParserAutoGen15Private::_end__capsule, &ColladaParserAutoGen15Private::_preBegin__capsule, &ColladaParserAutoGen15Private::_preEnd__capsule, &ColladaParserAutoGen15Private::_freeAttributes__capsule);
functionMap[HASH_ELEMENT_CYLINDER____CYLINDER_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__cylinder____cylinder_type, &ColladaParserAutoGen15Private::_data__cylinder____cylinder_type, &ColladaParserAutoGen15Private::_end__cylinder____cylinder_type, &ColladaParserAutoGen15Private::_preBegin__cylinder____cylinder_type, &ColladaParserAutoGen15Private::_preEnd__cylinder____cylinder_type, &ColladaParserAutoGen15Private::_freeAttributes__cylinder____cylinder_type);
functionMap[HASH_ELEMENT_DENSITY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__density, &ColladaParserAutoGen15Private::_data__density, &ColladaParserAutoGen15Private::_end__density, &ColladaParserAutoGen15Private::_preBegin__density, &ColladaParserAutoGen15Private::_preEnd__density, &ColladaParserAutoGen15Private::_freeAttributes__density);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INSTANCE_RIGID_BODY__TECHNIQUE_COMMON__SHAPE__HOLLOW] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_rigid_body__technique_common__shape__hollow, &ColladaParserAutoGen15Private::_data__instance_rigid_body__technique_common__shape__hollow, &ColladaParserAutoGen15Private::_end__instance_rigid_body__technique_common__shape__hollow, &ColladaParserAutoGen15Private::_preBegin__instance_rigid_body__technique_common__shape__hollow, &ColladaParserAutoGen15Private::_preEnd__instance_rigid_body__technique_common__shape__hollow, &ColladaParserAutoGen15Private::_freeAttributes__instance_rigid_body__technique_common__shape__hollow);
functionMap[HASH_ELEMENT_INSTANCE_GEOMETRY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_geometry, &ColladaParserAutoGen15Private::_data__instance_geometry, &ColladaParserAutoGen15Private::_end__instance_geometry, &ColladaParserAutoGen15Private::_preBegin__instance_geometry, &ColladaParserAutoGen15Private::_preEnd__instance_geometry, &ColladaParserAutoGen15Private::_freeAttributes__instance_geometry);
functionMap[HASH_ELEMENT_INSTANCE_PHYSICS_MATERIAL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_physics_material, &ColladaParserAutoGen15Private::_data__instance_physics_material, &ColladaParserAutoGen15Private::_end__instance_physics_material, &ColladaParserAutoGen15Private::_preBegin__instance_physics_material, &ColladaParserAutoGen15Private::_preEnd__instance_physics_material, &ColladaParserAutoGen15Private::_freeAttributes__instance_physics_material);
functionMap[HASH_ELEMENT_MASS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__mass, &ColladaParserAutoGen15Private::_data__mass, &ColladaParserAutoGen15Private::_end__mass, &ColladaParserAutoGen15Private::_preBegin__mass, &ColladaParserAutoGen15Private::_preEnd__mass, &ColladaParserAutoGen15Private::_freeAttributes__mass);
functionMap[HASH_ELEMENT_PHYSICS_MATERIAL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__physics_material, &ColladaParserAutoGen15Private::_data__physics_material, &ColladaParserAutoGen15Private::_end__physics_material, &ColladaParserAutoGen15Private::_preBegin__physics_material, &ColladaParserAutoGen15Private::_preEnd__physics_material, &ColladaParserAutoGen15Private::_freeAttributes__physics_material);
functionMap[HASH_ELEMENT_PLANE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__plane, &ColladaParserAutoGen15Private::_data__plane, &ColladaParserAutoGen15Private::_end__plane, &ColladaParserAutoGen15Private::_preBegin__plane, &ColladaParserAutoGen15Private::_preEnd__plane, &ColladaParserAutoGen15Private::_freeAttributes__plane);
functionMap[HASH_ELEMENT_ROTATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__rotate, &ColladaParserAutoGen15Private::_data__rotate, &ColladaParserAutoGen15Private::_end__rotate, &ColladaParserAutoGen15Private::_preBegin__rotate, &ColladaParserAutoGen15Private::_preEnd__rotate, &ColladaParserAutoGen15Private::_freeAttributes__rotate);
functionMap[HASH_ELEMENT_SPHERE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__sphere, &ColladaParserAutoGen15Private::_data__sphere, &ColladaParserAutoGen15Private::_end__sphere, &ColladaParserAutoGen15Private::_preBegin__sphere, &ColladaParserAutoGen15Private::_preEnd__sphere, &ColladaParserAutoGen15Private::_freeAttributes__sphere);
functionMap[HASH_ELEMENT_TRANSLATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__translate, &ColladaParserAutoGen15Private::_data__translate, &ColladaParserAutoGen15Private::_end__translate, &ColladaParserAutoGen15Private::_preBegin__translate, &ColladaParserAutoGen15Private::_preEnd__translate, &ColladaParserAutoGen15Private::_freeAttributes__translate);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_HALF_EXTENTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__half_extents, &ColladaParserAutoGen15Private::_data__half_extents, &ColladaParserAutoGen15Private::_end__half_extents, &ColladaParserAutoGen15Private::_preBegin__half_extents, &ColladaParserAutoGen15Private::_preEnd__half_extents, &ColladaParserAutoGen15Private::_freeAttributes__half_extents);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_HEIGHT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__height, &ColladaParserAutoGen15Private::_data__height, &ColladaParserAutoGen15Private::_end__height, &ColladaParserAutoGen15Private::_preBegin__height, &ColladaParserAutoGen15Private::_preEnd__height, &ColladaParserAutoGen15Private::_freeAttributes__height);
functionMap[HASH_ELEMENT_RADIUS____FLOAT3_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__radius____float3_type, &ColladaParserAutoGen15Private::_data__radius____float3_type, &ColladaParserAutoGen15Private::_end__radius____float3_type, &ColladaParserAutoGen15Private::_preBegin__radius____float3_type, &ColladaParserAutoGen15Private::_preEnd__radius____float3_type, &ColladaParserAutoGen15Private::_freeAttributes__radius____float3_type);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_HEIGHT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__height, &ColladaParserAutoGen15Private::_data__height, &ColladaParserAutoGen15Private::_end__height, &ColladaParserAutoGen15Private::_preBegin__height, &ColladaParserAutoGen15Private::_preEnd__height, &ColladaParserAutoGen15Private::_freeAttributes__height);
functionMap[HASH_ELEMENT_RADIUS____FLOAT2_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__radius____float2_type, &ColladaParserAutoGen15Private::_data__radius____float2_type, &ColladaParserAutoGen15Private::_end__radius____float2_type, &ColladaParserAutoGen15Private::_preBegin__radius____float2_type, &ColladaParserAutoGen15Private::_preEnd__radius____float2_type, &ColladaParserAutoGen15Private::_freeAttributes__radius____float2_type);
functionMap[HASH_ELEMENT_BIND_MATERIAL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bind_material, &ColladaParserAutoGen15Private::_data__bind_material, &ColladaParserAutoGen15Private::_end__bind_material, &ColladaParserAutoGen15Private::_preBegin__bind_material, &ColladaParserAutoGen15Private::_preEnd__bind_material, &ColladaParserAutoGen15Private::_freeAttributes__bind_material);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_PARAM____PARAM_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__param____param_type, &ColladaParserAutoGen15Private::_data__param____param_type, &ColladaParserAutoGen15Private::_end__param____param_type, &ColladaParserAutoGen15Private::_preBegin__param____param_type, &ColladaParserAutoGen15Private::_preEnd__param____param_type, &ColladaParserAutoGen15Private::_freeAttributes__param____param_type);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_BIND_MATERIAL_TYPE____TECHNIQUE_COMMON] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bind_material_type____technique_common, &ColladaParserAutoGen15Private::_data__bind_material_type____technique_common, &ColladaParserAutoGen15Private::_end__bind_material_type____technique_common, &ColladaParserAutoGen15Private::_preBegin__bind_material_type____technique_common, &ColladaParserAutoGen15Private::_preEnd__bind_material_type____technique_common, &ColladaParserAutoGen15Private::_freeAttributes__bind_material_type____technique_common);
functionMap[HASH_ELEMENT_INSTANCE_MATERIAL____INSTANCE_MATERIAL_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_material____instance_material_type, &ColladaParserAutoGen15Private::_data__instance_material____instance_material_type, &ColladaParserAutoGen15Private::_end__instance_material____instance_material_type, &ColladaParserAutoGen15Private::_preBegin__instance_material____instance_material_type, &ColladaParserAutoGen15Private::_preEnd__instance_material____instance_material_type, &ColladaParserAutoGen15Private::_freeAttributes__instance_material____instance_material_type);
functionMap[HASH_ELEMENT_INSTANCE_MATERIAL_TYPE____BIND] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_material_type____bind, &ColladaParserAutoGen15Private::_data__instance_material_type____bind, &ColladaParserAutoGen15Private::_end__instance_material_type____bind, &ColladaParserAutoGen15Private::_preBegin__instance_material_type____bind, &ColladaParserAutoGen15Private::_preEnd__instance_material_type____bind, &ColladaParserAutoGen15Private::_freeAttributes__instance_material_type____bind);
functionMap[HASH_ELEMENT_BIND_VERTEX_INPUT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bind_vertex_input, &ColladaParserAutoGen15Private::_data__bind_vertex_input, &ColladaParserAutoGen15Private::_end__bind_vertex_input, &ColladaParserAutoGen15Private::_preBegin__bind_vertex_input, &ColladaParserAutoGen15Private::_preEnd__bind_vertex_input, &ColladaParserAutoGen15Private::_freeAttributes__bind_vertex_input);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_EQUATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__equation, &ColladaParserAutoGen15Private::_data__equation, &ColladaParserAutoGen15Private::_end__equation, &ColladaParserAutoGen15Private::_preBegin__equation, &ColladaParserAutoGen15Private::_preEnd__equation, &ColladaParserAutoGen15Private::_freeAttributes__equation);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_RADIUS____FLOAT_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__radius____float_type, &ColladaParserAutoGen15Private::_data__radius____float_type, &ColladaParserAutoGen15Private::_end__radius____float_type, &ColladaParserAutoGen15Private::_preBegin__radius____float_type, &ColladaParserAutoGen15Private::_preEnd__radius____float_type, &ColladaParserAutoGen15Private::_freeAttributes__radius____float_type);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_GRAVITY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__gravity, &ColladaParserAutoGen15Private::_data__gravity, &ColladaParserAutoGen15Private::_end__gravity, &ColladaParserAutoGen15Private::_preBegin__gravity, &ColladaParserAutoGen15Private::_preEnd__gravity, &ColladaParserAutoGen15Private::_freeAttributes__gravity);
functionMap[HASH_ELEMENT_TIME_STEP] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__time_step, &ColladaParserAutoGen15Private::_data__time_step, &ColladaParserAutoGen15Private::_end__time_step, &ColladaParserAutoGen15Private::_preBegin__time_step, &ColladaParserAutoGen15Private::_preEnd__time_step, &ColladaParserAutoGen15Private::_freeAttributes__time_step);
mapFilled = true;
}
return functionMap;
}

//---------------------------------------------------------------------
const ColladaParserAutoGen15Private::ElementFunctionMap& FunctionMapFactory::createFunctionMap__library_visual_scenes__allChildren() {
static ColladaParserAutoGen15Private::ElementFunctionMap functionMap;
static bool mapFilled = false;
if ( !mapFilled )
{
functionMap[HASH_ELEMENT_COLLADA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__COLLADA, &ColladaParserAutoGen15Private::_data__COLLADA, &ColladaParserAutoGen15Private::_end__COLLADA, &ColladaParserAutoGen15Private::_preBegin__COLLADA, &ColladaParserAutoGen15Private::_preEnd__COLLADA, &ColladaParserAutoGen15Private::_freeAttributes__COLLADA);
functionMap[HASH_ELEMENT_LIBRARY_VISUAL_SCENES] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__library_visual_scenes, &ColladaParserAutoGen15Private::_data__library_visual_scenes, &ColladaParserAutoGen15Private::_end__library_visual_scenes, &ColladaParserAutoGen15Private::_preBegin__library_visual_scenes, &ColladaParserAutoGen15Private::_preEnd__library_visual_scenes, &ColladaParserAutoGen15Private::_freeAttributes__library_visual_scenes);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_VISUAL_SCENE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__visual_scene, &ColladaParserAutoGen15Private::_data__visual_scene, &ColladaParserAutoGen15Private::_end__visual_scene, &ColladaParserAutoGen15Private::_preBegin__visual_scene, &ColladaParserAutoGen15Private::_preEnd__visual_scene, &ColladaParserAutoGen15Private::_freeAttributes__visual_scene);
functionMap[HASH_ELEMENT_CONTRIBUTOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__contributor, &ColladaParserAutoGen15Private::_data__contributor, &ColladaParserAutoGen15Private::_end__contributor, &ColladaParserAutoGen15Private::_preBegin__contributor, &ColladaParserAutoGen15Private::_preEnd__contributor, &ColladaParserAutoGen15Private::_freeAttributes__contributor);
functionMap[HASH_ELEMENT_COVERAGE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__coverage, &ColladaParserAutoGen15Private::_data__coverage, &ColladaParserAutoGen15Private::_end__coverage, &ColladaParserAutoGen15Private::_preBegin__coverage, &ColladaParserAutoGen15Private::_preEnd__coverage, &ColladaParserAutoGen15Private::_freeAttributes__coverage);
functionMap[HASH_ELEMENT_CREATED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__created, &ColladaParserAutoGen15Private::_data__created, &ColladaParserAutoGen15Private::_end__created, &ColladaParserAutoGen15Private::_preBegin__created, &ColladaParserAutoGen15Private::_preEnd__created, &ColladaParserAutoGen15Private::_freeAttributes__created);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_AUTHOR] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author, &ColladaParserAutoGen15Private::_data__author, &ColladaParserAutoGen15Private::_end__author, &ColladaParserAutoGen15Private::_preBegin__author, &ColladaParserAutoGen15Private::_preEnd__author, &ColladaParserAutoGen15Private::_freeAttributes__author);
functionMap[HASH_ELEMENT_AUTHOR_EMAIL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_email, &ColladaParserAutoGen15Private::_data__author_email, &ColladaParserAutoGen15Private::_end__author_email, &ColladaParserAutoGen15Private::_preBegin__author_email, &ColladaParserAutoGen15Private::_preEnd__author_email, &ColladaParserAutoGen15Private::_freeAttributes__author_email);
functionMap[HASH_ELEMENT_AUTHOR_WEBSITE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__author_website, &ColladaParserAutoGen15Private::_data__author_website, &ColladaParserAutoGen15Private::_end__author_website, &ColladaParserAutoGen15Private::_preBegin__author_website, &ColladaParserAutoGen15Private::_preEnd__author_website, &ColladaParserAutoGen15Private::_freeAttributes__author_website);
functionMap[HASH_ELEMENT_AUTHORING_TOOL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__authoring_tool, &ColladaParserAutoGen15Private::_data__authoring_tool, &ColladaParserAutoGen15Private::_end__authoring_tool, &ColladaParserAutoGen15Private::_preBegin__authoring_tool, &ColladaParserAutoGen15Private::_preEnd__authoring_tool, &ColladaParserAutoGen15Private::_freeAttributes__authoring_tool);
functionMap[HASH_ELEMENT_COMMENTS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__comments, &ColladaParserAutoGen15Private::_data__comments, &ColladaParserAutoGen15Private::_end__comments, &ColladaParserAutoGen15Private::_preBegin__comments, &ColladaParserAutoGen15Private::_preEnd__comments, &ColladaParserAutoGen15Private::_freeAttributes__comments);
functionMap[HASH_ELEMENT_COPYRIGHT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__copyright, &ColladaParserAutoGen15Private::_data__copyright, &ColladaParserAutoGen15Private::_end__copyright, &ColladaParserAutoGen15Private::_preBegin__copyright, &ColladaParserAutoGen15Private::_preEnd__copyright, &ColladaParserAutoGen15Private::_freeAttributes__copyright);
functionMap[HASH_ELEMENT_SOURCE_DATA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__source_data, &ColladaParserAutoGen15Private::_data__source_data, &ColladaParserAutoGen15Private::_end__source_data, &ColladaParserAutoGen15Private::_preBegin__source_data, &ColladaParserAutoGen15Private::_preEnd__source_data, &ColladaParserAutoGen15Private::_freeAttributes__source_data);
functionMap[HASH_ELEMENT_GEOGRAPHIC_LOCATION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__geographic_location, &ColladaParserAutoGen15Private::_data__geographic_location, &ColladaParserAutoGen15Private::_end__geographic_location, &ColladaParserAutoGen15Private::_preBegin__geographic_location, &ColladaParserAutoGen15Private::_preEnd__geographic_location, &ColladaParserAutoGen15Private::_freeAttributes__geographic_location);
functionMap[HASH_ELEMENT_ALTITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__altitude, &ColladaParserAutoGen15Private::_data__altitude, &ColladaParserAutoGen15Private::_end__altitude, &ColladaParserAutoGen15Private::_preBegin__altitude, &ColladaParserAutoGen15Private::_preEnd__altitude, &ColladaParserAutoGen15Private::_freeAttributes__altitude);
functionMap[HASH_ELEMENT_LATITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__latitude, &ColladaParserAutoGen15Private::_data__latitude, &ColladaParserAutoGen15Private::_end__latitude, &ColladaParserAutoGen15Private::_preBegin__latitude, &ColladaParserAutoGen15Private::_preEnd__latitude, &ColladaParserAutoGen15Private::_freeAttributes__latitude);
functionMap[HASH_ELEMENT_LONGITUDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__longitude, &ColladaParserAutoGen15Private::_data__longitude, &ColladaParserAutoGen15Private::_end__longitude, &ColladaParserAutoGen15Private::_preBegin__longitude, &ColladaParserAutoGen15Private::_preEnd__longitude, &ColladaParserAutoGen15Private::_freeAttributes__longitude);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_KEYWORDS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__keywords, &ColladaParserAutoGen15Private::_data__keywords, &ColladaParserAutoGen15Private::_end__keywords, &ColladaParserAutoGen15Private::_preBegin__keywords, &ColladaParserAutoGen15Private::_preEnd__keywords, &ColladaParserAutoGen15Private::_freeAttributes__keywords);
functionMap[HASH_ELEMENT_MODIFIED] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__modified, &ColladaParserAutoGen15Private::_data__modified, &ColladaParserAutoGen15Private::_end__modified, &ColladaParserAutoGen15Private::_preBegin__modified, &ColladaParserAutoGen15Private::_preEnd__modified, &ColladaParserAutoGen15Private::_freeAttributes__modified);
functionMap[HASH_ELEMENT_REVISION] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__revision, &ColladaParserAutoGen15Private::_data__revision, &ColladaParserAutoGen15Private::_end__revision, &ColladaParserAutoGen15Private::_preBegin__revision, &ColladaParserAutoGen15Private::_preEnd__revision, &ColladaParserAutoGen15Private::_freeAttributes__revision);
functionMap[HASH_ELEMENT_SUBJECT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__subject, &ColladaParserAutoGen15Private::_data__subject, &ColladaParserAutoGen15Private::_end__subject, &ColladaParserAutoGen15Private::_preBegin__subject, &ColladaParserAutoGen15Private::_preEnd__subject, &ColladaParserAutoGen15Private::_freeAttributes__subject);
functionMap[HASH_ELEMENT_TITLE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__title, &ColladaParserAutoGen15Private::_data__title, &ColladaParserAutoGen15Private::_end__title, &ColladaParserAutoGen15Private::_preBegin__title, &ColladaParserAutoGen15Private::_preEnd__title, &ColladaParserAutoGen15Private::_freeAttributes__title);
functionMap[HASH_ELEMENT_UNIT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__unit, &ColladaParserAutoGen15Private::_data__unit, &ColladaParserAutoGen15Private::_end__unit, &ColladaParserAutoGen15Private::_preBegin__unit, &ColladaParserAutoGen15Private::_preEnd__unit, &ColladaParserAutoGen15Private::_freeAttributes__unit);
functionMap[HASH_ELEMENT_UP_AXIS] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__up_axis, &ColladaParserAutoGen15Private::_data__up_axis, &ColladaParserAutoGen15Private::_end__up_axis, &ColladaParserAutoGen15Private::_preBegin__up_axis, &ColladaParserAutoGen15Private::_preEnd__up_axis, &ColladaParserAutoGen15Private::_freeAttributes__up_axis);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EVALUATE_SCENE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__evaluate_scene, &ColladaParserAutoGen15Private::_data__evaluate_scene, &ColladaParserAutoGen15Private::_end__evaluate_scene, &ColladaParserAutoGen15Private::_preBegin__evaluate_scene, &ColladaParserAutoGen15Private::_preEnd__evaluate_scene, &ColladaParserAutoGen15Private::_freeAttributes__evaluate_scene);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_NODE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__node, &ColladaParserAutoGen15Private::_data__node, &ColladaParserAutoGen15Private::_end__node, &ColladaParserAutoGen15Private::_preBegin__node, &ColladaParserAutoGen15Private::_preEnd__node, &ColladaParserAutoGen15Private::_freeAttributes__node);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_RENDER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__render, &ColladaParserAutoGen15Private::_data__render, &ColladaParserAutoGen15Private::_end__render, &ColladaParserAutoGen15Private::_preBegin__render, &ColladaParserAutoGen15Private::_preEnd__render, &ColladaParserAutoGen15Private::_freeAttributes__render);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_RENDER__INSTANCE_MATERIAL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__render__instance_material, &ColladaParserAutoGen15Private::_data__render__instance_material, &ColladaParserAutoGen15Private::_end__render__instance_material, &ColladaParserAutoGen15Private::_preBegin__render__instance_material, &ColladaParserAutoGen15Private::_preEnd__render__instance_material, &ColladaParserAutoGen15Private::_freeAttributes__render__instance_material);
functionMap[HASH_ELEMENT_LAYER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__layer, &ColladaParserAutoGen15Private::_data__layer, &ColladaParserAutoGen15Private::_end__layer, &ColladaParserAutoGen15Private::_preBegin__layer, &ColladaParserAutoGen15Private::_preEnd__layer, &ColladaParserAutoGen15Private::_freeAttributes__layer);
functionMap[HASH_ELEMENT_RENDER__INSTANCE_MATERIAL__BIND] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__render__instance_material__bind, &ColladaParserAutoGen15Private::_data__render__instance_material__bind, &ColladaParserAutoGen15Private::_end__render__instance_material__bind, &ColladaParserAutoGen15Private::_preBegin__render__instance_material__bind, &ColladaParserAutoGen15Private::_preEnd__render__instance_material__bind, &ColladaParserAutoGen15Private::_freeAttributes__render__instance_material__bind);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_TECHNIQUE_OVERRIDE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique_override, &ColladaParserAutoGen15Private::_data__technique_override, &ColladaParserAutoGen15Private::_end__technique_override, &ColladaParserAutoGen15Private::_preBegin__technique_override, &ColladaParserAutoGen15Private::_preEnd__technique_override, &ColladaParserAutoGen15Private::_freeAttributes__technique_override);
functionMap[HASH_ELEMENT_ASSET] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__asset, &ColladaParserAutoGen15Private::_data__asset, &ColladaParserAutoGen15Private::_end__asset, &ColladaParserAutoGen15Private::_preBegin__asset, &ColladaParserAutoGen15Private::_preEnd__asset, &ColladaParserAutoGen15Private::_freeAttributes__asset);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_INSTANCE_CAMERA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_camera, &ColladaParserAutoGen15Private::_data__instance_camera, &ColladaParserAutoGen15Private::_end__instance_camera, &ColladaParserAutoGen15Private::_preBegin__instance_camera, &ColladaParserAutoGen15Private::_preEnd__instance_camera, &ColladaParserAutoGen15Private::_freeAttributes__instance_camera);
functionMap[HASH_ELEMENT_INSTANCE_CONTROLLER] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_controller, &ColladaParserAutoGen15Private::_data__instance_controller, &ColladaParserAutoGen15Private::_end__instance_controller, &ColladaParserAutoGen15Private::_preBegin__instance_controller, &ColladaParserAutoGen15Private::_preEnd__instance_controller, &ColladaParserAutoGen15Private::_freeAttributes__instance_controller);
functionMap[HASH_ELEMENT_INSTANCE_GEOMETRY] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_geometry, &ColladaParserAutoGen15Private::_data__instance_geometry, &ColladaParserAutoGen15Private::_end__instance_geometry, &ColladaParserAutoGen15Private::_preBegin__instance_geometry, &ColladaParserAutoGen15Private::_preEnd__instance_geometry, &ColladaParserAutoGen15Private::_freeAttributes__instance_geometry);
functionMap[HASH_ELEMENT_INSTANCE_LIGHT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_light, &ColladaParserAutoGen15Private::_data__instance_light, &ColladaParserAutoGen15Private::_end__instance_light, &ColladaParserAutoGen15Private::_preBegin__instance_light, &ColladaParserAutoGen15Private::_preEnd__instance_light, &ColladaParserAutoGen15Private::_freeAttributes__instance_light);
functionMap[HASH_ELEMENT_INSTANCE_NODE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_node, &ColladaParserAutoGen15Private::_data__instance_node, &ColladaParserAutoGen15Private::_end__instance_node, &ColladaParserAutoGen15Private::_preBegin__instance_node, &ColladaParserAutoGen15Private::_preEnd__instance_node, &ColladaParserAutoGen15Private::_freeAttributes__instance_node);
functionMap[HASH_ELEMENT_LOOKAT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__lookat, &ColladaParserAutoGen15Private::_data__lookat, &ColladaParserAutoGen15Private::_end__lookat, &ColladaParserAutoGen15Private::_preBegin__lookat, &ColladaParserAutoGen15Private::_preEnd__lookat, &ColladaParserAutoGen15Private::_freeAttributes__lookat);
functionMap[HASH_ELEMENT_MATRIX____MATRIX_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__matrix____matrix_type, &ColladaParserAutoGen15Private::_data__matrix____matrix_type, &ColladaParserAutoGen15Private::_end__matrix____matrix_type, &ColladaParserAutoGen15Private::_preBegin__matrix____matrix_type, &ColladaParserAutoGen15Private::_preEnd__matrix____matrix_type, &ColladaParserAutoGen15Private::_freeAttributes__matrix____matrix_type);
functionMap[HASH_ELEMENT_NODE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__node, &ColladaParserAutoGen15Private::_data__node, &ColladaParserAutoGen15Private::_end__node, &ColladaParserAutoGen15Private::_preBegin__node, &ColladaParserAutoGen15Private::_preEnd__node, &ColladaParserAutoGen15Private::_freeAttributes__node);
functionMap[HASH_ELEMENT_ROTATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__rotate, &ColladaParserAutoGen15Private::_data__rotate, &ColladaParserAutoGen15Private::_end__rotate, &ColladaParserAutoGen15Private::_preBegin__rotate, &ColladaParserAutoGen15Private::_preEnd__rotate, &ColladaParserAutoGen15Private::_freeAttributes__rotate);
functionMap[HASH_ELEMENT_SCALE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__scale, &ColladaParserAutoGen15Private::_data__scale, &ColladaParserAutoGen15Private::_end__scale, &ColladaParserAutoGen15Private::_preBegin__scale, &ColladaParserAutoGen15Private::_preEnd__scale, &ColladaParserAutoGen15Private::_freeAttributes__scale);
functionMap[HASH_ELEMENT_SKEW] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__skew, &ColladaParserAutoGen15Private::_data__skew, &ColladaParserAutoGen15Private::_end__skew, &ColladaParserAutoGen15Private::_preBegin__skew, &ColladaParserAutoGen15Private::_preEnd__skew, &ColladaParserAutoGen15Private::_freeAttributes__skew);
functionMap[HASH_ELEMENT_TRANSLATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__translate, &ColladaParserAutoGen15Private::_data__translate, &ColladaParserAutoGen15Private::_end__translate, &ColladaParserAutoGen15Private::_preBegin__translate, &ColladaParserAutoGen15Private::_preEnd__translate, &ColladaParserAutoGen15Private::_freeAttributes__translate);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_BIND_MATERIAL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bind_material, &ColladaParserAutoGen15Private::_data__bind_material, &ColladaParserAutoGen15Private::_end__bind_material, &ColladaParserAutoGen15Private::_preBegin__bind_material, &ColladaParserAutoGen15Private::_preEnd__bind_material, &ColladaParserAutoGen15Private::_freeAttributes__bind_material);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_SKELETON] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__skeleton, &ColladaParserAutoGen15Private::_data__skeleton, &ColladaParserAutoGen15Private::_end__skeleton, &ColladaParserAutoGen15Private::_preBegin__skeleton, &ColladaParserAutoGen15Private::_preEnd__skeleton, &ColladaParserAutoGen15Private::_freeAttributes__skeleton);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_PARAM____PARAM_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__param____param_type, &ColladaParserAutoGen15Private::_data__param____param_type, &ColladaParserAutoGen15Private::_end__param____param_type, &ColladaParserAutoGen15Private::_preBegin__param____param_type, &ColladaParserAutoGen15Private::_preEnd__param____param_type, &ColladaParserAutoGen15Private::_freeAttributes__param____param_type);
functionMap[HASH_ELEMENT_TECHNIQUE____TECHNIQUE_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__technique____technique_type, &ColladaParserAutoGen15Private::_data__technique____technique_type, &ColladaParserAutoGen15Private::_end__technique____technique_type, &ColladaParserAutoGen15Private::_preBegin__technique____technique_type, &ColladaParserAutoGen15Private::_preEnd__technique____technique_type, &ColladaParserAutoGen15Private::_freeAttributes__technique____technique_type);
functionMap[HASH_ELEMENT_BIND_MATERIAL_TYPE____TECHNIQUE_COMMON] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bind_material_type____technique_common, &ColladaParserAutoGen15Private::_data__bind_material_type____technique_common, &ColladaParserAutoGen15Private::_end__bind_material_type____technique_common, &ColladaParserAutoGen15Private::_preBegin__bind_material_type____technique_common, &ColladaParserAutoGen15Private::_preEnd__bind_material_type____technique_common, &ColladaParserAutoGen15Private::_freeAttributes__bind_material_type____technique_common);
functionMap[HASH_ELEMENT_INSTANCE_MATERIAL____INSTANCE_MATERIAL_TYPE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_material____instance_material_type, &ColladaParserAutoGen15Private::_data__instance_material____instance_material_type, &ColladaParserAutoGen15Private::_end__instance_material____instance_material_type, &ColladaParserAutoGen15Private::_preBegin__instance_material____instance_material_type, &ColladaParserAutoGen15Private::_preEnd__instance_material____instance_material_type, &ColladaParserAutoGen15Private::_freeAttributes__instance_material____instance_material_type);
functionMap[HASH_ELEMENT_INSTANCE_MATERIAL_TYPE____BIND] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__instance_material_type____bind, &ColladaParserAutoGen15Private::_data__instance_material_type____bind, &ColladaParserAutoGen15Private::_end__instance_material_type____bind, &ColladaParserAutoGen15Private::_preBegin__instance_material_type____bind, &ColladaParserAutoGen15Private::_preEnd__instance_material_type____bind, &ColladaParserAutoGen15Private::_freeAttributes__instance_material_type____bind);
functionMap[HASH_ELEMENT_BIND_VERTEX_INPUT] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bind_vertex_input, &ColladaParserAutoGen15Private::_data__bind_vertex_input, &ColladaParserAutoGen15Private::_end__bind_vertex_input, &ColladaParserAutoGen15Private::_preBegin__bind_vertex_input, &ColladaParserAutoGen15Private::_preEnd__bind_vertex_input, &ColladaParserAutoGen15Private::_freeAttributes__bind_vertex_input);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_BIND_MATERIAL] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__bind_material, &ColladaParserAutoGen15Private::_data__bind_material, &ColladaParserAutoGen15Private::_end__bind_material, &ColladaParserAutoGen15Private::_preBegin__bind_material, &ColladaParserAutoGen15Private::_preEnd__bind_material, &ColladaParserAutoGen15Private::_freeAttributes__bind_material);
functionMap[HASH_ELEMENT_EXTRA] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__extra, &ColladaParserAutoGen15Private::_data__extra, &ColladaParserAutoGen15Private::_end__extra, &ColladaParserAutoGen15Private::_preBegin__extra, &ColladaParserAutoGen15Private::_preEnd__extra, &ColladaParserAutoGen15Private::_freeAttributes__extra);
functionMap[HASH_ELEMENT_ROTATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__rotate, &ColladaParserAutoGen15Private::_data__rotate, &ColladaParserAutoGen15Private::_end__rotate, &ColladaParserAutoGen15Private::_preBegin__rotate, &ColladaParserAutoGen15Private::_preEnd__rotate, &ColladaParserAutoGen15Private::_freeAttributes__rotate);
functionMap[HASH_ELEMENT_SCALE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__scale, &ColladaParserAutoGen15Private::_data__scale, &ColladaParserAutoGen15Private::_end__scale, &ColladaParserAutoGen15Private::_preBegin__scale, &ColladaParserAutoGen15Private::_preEnd__scale, &ColladaParserAutoGen15Private::_freeAttributes__scale);
functionMap[HASH_ELEMENT_SKEW] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__skew, &ColladaParserAutoGen15Private::_data__skew, &ColladaParserAutoGen15Private::_end__skew, &ColladaParserAutoGen15Private::_preBegin__skew, &ColladaParserAutoGen15Private::_preEnd__skew, &ColladaParserAutoGen15Private::_freeAttributes__skew);
functionMap[HASH_ELEMENT_TRANSLATE] = ColladaParserAutoGen15Private::FunctionStruct(&ColladaParserAutoGen15Private::_begin__translate, &ColladaParserAutoGen15Private::_data__translate, &ColladaParserAutoGen15Private::_end__translate, &ColladaParserAutoGen15Private::_preBegin__translate, &ColladaParserAutoGen15Private::_preEnd__translate, &ColladaParserAutoGen15Private::_freeAttributes__translate);
mapFilled = true;
}
return functionMap;
}

} // namespace
